-- name=cheater
--------------------------------------------------------------------------------
function  public.Init ()
  
  ModLoad( "assets/cheater/mod_review" );
  review.Init();
  
  private.app_w = int.W();
  private.app_h = int.H();

  private.active              = false;
  private.active_stage_cheats = false;
  private.active_stages       = false;
  private.active_edit         = false;

  common.MessageSubscribe( "Event_Common_ScriptError", function ( name, params )
  
    -- Set newline symbol
    local line_ends =
    {
      pc      = "\r\n",
      ios     = "\r",
      android = "\n"
    };
    local nl = line_ends[ GetPlatform() ] or "\n";
    
    local text = "";
    local function put_string ( str, no_nl )
    
      text = text..str;
      
      if ( not no_nl ) then
        text = text..nl;
      end;
    
    end;
    
    -- Setup message text
    put_string( "--------------------" );
    put_string( "Game:" );
    put_string( "Version: "..VersionGetComponentVersion( "game" ) );
    put_string( "Platform: "..GetPlatform() );
    
    if ( IsAndroid() ) then
      put_string( "Android platform: "..wiz.Android.GetPlatform() );
    end;
    
    put_string( "User id: "..m_player.GetUserId() );
    put_string( "Server: "..ConfigGetInfo( "server.host" ) );
    
    put_string( "--------------------" );
    put_string( "Content:" );
    put_string( "Version: "..( parts.version.GetCurrent( "content" ) or "?" ) );
    put_string( "Packages:", true );
    put_string(
      common.TableToString( ContentGetPacketList(), 0, nl, "" ), true
    );
    
    put_string( "--------------------" );
    put_string( "Social Info:", true );
    put_string( common.TableToString( SocialGetInfo(), 0, nl, "" ), true );
    
    put_string( "--------------------" );
    put_string( "Call Stack:", true );
    put_string( common.TableToString( params.callstack, 0, nl, "    " ), true );
    put_string( "--------------------" );
  
    -- Try send report
    wiz.bfg.SendEmail(
      {
        email   = "support.sir-match-a-lot@elephant-games.com",
        subject = "[ DEV ] Script Error",
        body    = text
      }
    );

  end );

  public.Show();

end;
--------------------------------------------------------------------------------
function  public.Show ()
  
  ObjAttach( "cheater", InterfaceName );
  
  private.Defaults_Set();
  private.Subscribes_Add();
  
  -- Add hook after screen open
  local function func_opened_hook ( screen )
  
    if     ( screen == "scr_map" ) then
      
      mediator.Notify( "View_Cheater_MapOpened" );

    elseif ( screen == "scr_stage" ) then
      
      mediator.Notify( "View_Cheater_StageOpened" );
      
    end;
  
  end;

  mediator.Subscribe( screen.Notifications.ScreenOpened, func_opened_hook );

  private.ShowDrawCalls();
  private.ShowFPS();
  private.SetOnlineState( m_client.IsOnline() );
  private.SetDebugState( debug );
  
  mediator.Subscribe( "Model_Client_ConnectionRestored", function () 
    private.SetOnlineState( true );
  end );
  
  mediator.Subscribe( "Model_Client_ConnectionLost", function () 
    private.SetOnlineState( false );
  end );

end;
--------------------------------------------------------------------------------
function private.Defaults_Set ()

  local cols = 4;
  local step = 70;

  local groups = ObjGetRelations( "obj_cheater" ).childs;

  local row_current = 0;

  for group_id = 1, #groups do

    local group = groups[ group_id ];

    local group_x = - step * cols + 25;
    local group_y =   step * (group_id + row_current);
    
    ObjSet( group, { pos_x = group_x, pos_y = group_y } );

    local buttons = ObjGetRelations( group ).childs;
    local rows    = math.ceil( #buttons / cols );

    row_current = row_current + rows - 1;

    for row = 1, rows do
      for col = 1, cols do

        local button_id = ( ( row - 1 ) * cols ) + col;
        if ( button_id <= #buttons ) then

          local button = buttons[ button_id ];

          local btn_x = step * ( col - 1 );
          local btn_y = step * ( row - 1 );

          ObjSet( button, { pos_x = btn_x, pos_y = btn_y } );

        end;

      end;
    end;

  end;
  
  local version = VersionGetComponentVersion( "game" );
  local user_id = m_player.GetUserId();
  local server  = ConfigGetInfo( "server.host" );

  ObjMultiSet( {
    { "cheater",                   { pos_x = private.app_w - 60 } },
    { "btn_cheater",               { event_mdown = private.Cheater_Switch } },
    
    { "btn_cheater_win",           { event_mdown = private.Stage_Win  } },
    { "btn_cheater_fail",          { event_mdown = private.Stage_Fail  } },
    { "btn_cheater_autoplay",      { event_mdown = private.Stage_Autoplay } },
    
    { "btn_cheater_gold_add",       { event_mdown = private.Gold_Add  } },
    { "btn_cheater_gold_null",      { event_mdown = private.Gold_Null } },
    { "btn_cheater_res1_add",       { event_mdown = function() private.Resources_Add(  1 ) end } },
    { "btn_cheater_res1_null",      { event_mdown = function() private.Resources_Null( 1 ) end } },
    { "btn_cheater_res2_add",       { event_mdown = function() private.Resources_Add(  2 ) end } },
    { "btn_cheater_res2_null",      { event_mdown = function() private.Resources_Null( 2 ) end } },
    { "btn_cheater_res3_add",       { event_mdown = function() private.Resources_Add(  3 ) end } },
    { "btn_cheater_res3_null",      { event_mdown = function() private.Resources_Null( 3 ) end } },
    
    { "btn_cheater_stages",        { event_mdown = private.Map_EnableStagesHere } },
    
    { "obj_cheater_states",        { input = false, alp = 0.5 } },
    { "btn_cheater_winallstates",  { event_mdown = private.Map_WinAllStates } },

    { "btn_cheater_state1", { event_mdown = function() private.Stage_PlayState( 1 ); end } },
    { "btn_cheater_state2", { event_mdown = function() private.Stage_PlayState( 2 ); end } },
    { "btn_cheater_state3", { event_mdown = function() private.Stage_PlayState( 3 ); end } },

    { "btn_cheater_section_2",     { event_mdown = function() private.Unlock_Section( 2 ) end } },
    { "btn_cheater_section_3",     { event_mdown = function() private.Unlock_Section( 3 ) end } },
    { "btn_cheater_section_4",     { event_mdown = function() private.Unlock_Section( 4 ) end } },
    { "btn_cheater_section_5",     { event_mdown = function() private.Unlock_Section( 5 ) end } },      
    
    { "btn_cheater_hearts",        { event_mdown = private.Hearts_Add   } },
    { "btn_cheater_dailybonus",    { event_mdown = private.DailyBonus   } },
    { "btn_cheater_dailyquest",    { event_mdown = private.DailyQuest   } },
    { "btn_cheater_shop",          { event_mdown = private.RefreshShop  } },
    { "btn_cheater_skins",         { event_mdown = private.GetAllSkins  } },
    { "btn_cheater_boosts",        { event_mdown = private.GetAllBoosts } },
    { "btn_cheater_nodbmode",      { event_mdown = private.SetNoDBMode  } },
    --{ "btn_cheater_autotest",      { event_mdown = private.Autotest     } },
    { "btn_cheater_starterpack",   { event_mdown = private.Starterpack  } },
    
    { "obj_cheater_info",          { pos_x = - private.app_w / 2 + 60, pos_y = private.app_h - 30 } },
    { "obj_cheater_info_drawcalls",{ pos_x = 60 - private.app_w } },
    { "obj_cheater_info_fps",      { pos_x = 60 - private.app_w + 80 } },
    { "obj_cheater_info_online",   { pos_x = 60 - private.app_w + 140 } },
    { "obj_cheater_info_debug",    { pos_x = 60 - private.app_w + 200 } },
    { "txt_cheater_info",          { text = "User ID: "..user_id.."          "..server.."          ver. "..version } },
    
    --{ "btn_cheater_intlock",            { alp = 1, input = true, event_mdown = private.InterfaceLock_Show } } 
  } );

  if ( not private.active_stage_cheats ) then

    ObjSet( "obj_cheater_stage",      { alp = 0.5, input = false } );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Subscribes_Add ()

  mediator.Subscribe( "View_Cheater_MapOpened",   private.Map_OnOpened );
  mediator.Subscribe( "View_Cheater_StageOpened", private.Stage_OnOpened );

  mediator.Subscribe( "Interface_Window_WindowOpened", private.WindowOpened );
  mediator.Subscribe( "Interface_Window_WindowClosed", private.WindowClosed );

  mediator.Subscribe(
    "View_Stage_DialogStartActivated",
    private.DialogStartOpened
  );

  mediator.Subscribe(
    "View_Stage_DialogStartDeactivated",
    private.DialogStartClosed
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Map_OnOpened ()

  if not ( private.playtest ) then

    ObjMultiSet( {
      { "obj_cheater_map",       { alp = 1.0, input = true  } },    
      { "obj_cheater_stage",     { alp = 0.5, input = false } }
    } );

    private.active_stage_cheats = false;

  end;

end;
--------------------------------------------------------------------------------
function private.Stage_OnOpened ()

  if not ( private.playtest ) then

    ObjMultiSet( {
      { "obj_cheater_map",       { alp = 0.5, input = false } },    
      { "obj_cheater_stage",     { alp = 1.0, input = true  } }
    } );

    private.active_stage_cheats = true;

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.WindowOpened ( name )

  if ( name == "int_dialog_stage_start" ) then

    ObjSet( "obj_cheater_states", { alp = 1.0, input = true } );


  end;

end;
--------------------------------------------------------------------------------
function private.WindowClosed ( name )

  if ( name == "int_dialog_stage_start" ) then

    ObjSet( "obj_cheater_states", { alp = 0.5, input = false } );

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.DialogStartOpened ( params )

  private.dialog_start_stage_name = params.name;

end;
--------------------------------------------------------------------------------
function private.DialogStartClosed ()

  private.dialog_start_stage_name = nil;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Cheater_Switch ()

  if not ( private.active ) then

    ObjSet( "obj_cheater",          { active = true, visible = true, input = true, alp = 0 } );
    ObjSet( "obj_cheater_info",     { active = true, visible = true, alp = 0 } );
    ObjSet( "obj_cheater_info_top", { active = true, visible = true, alp = 0 } );
    
    common.Animation_Fade_In( 0.3, "obj_cheater" );
    common.Animation_Fade_In( 0.3, "obj_cheater_info" );
    common.Animation_Fade_In( 0.3, "obj_cheater_info_top" );

  else

    local func_end = function ()

      ObjSet( "obj_cheater",          { active = false, visible = false, input = false } );
      ObjSet( "obj_cheater_info",     { active = false, visible = false } );
      ObjSet( "obj_cheater_info_top", { active = false, visible = false } );

    end;

    common.Animation_Fade_Out( 0.3, "obj_cheater", func_end );
    common.Animation_Fade_Out( 0.3, "obj_cheater_info" );
    common.Animation_Fade_Out( 0.3, "obj_cheater_info_top" );

  end;

  private.active = not private.active;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Gold_Add ()

  m_player.TryChangeGoldCount( 1000 );

end;
--------------------------------------------------------------------------------
function private.Gold_Null ()

  m_player.TryChangeGoldCount( -m_player.GetGoldCount() );

end;
--------------------------------------------------------------------------------
function private.Resources_Add ( id )

  m_player.TryChangeResourceCount( "resource"..id, 1000 );

end;
--------------------------------------------------------------------------------
function private.Resources_Null ( id )

  local resource = "resource"..id;
  local count = m_player.GetResourceCount( resource );
  
  m_player.TryChangeResourceCount( resource, -count );

end;
--------------------------------------------------------------------------------
function private.Hearts_Add ()

  m_player.TryChangeHeartsCount( 1 );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Stage_Win ()

  m_stage.Cheater_SetWinState();

end;
--------------------------------------------------------------------------------
function private.Stage_Fail ()

  m_stage.Cheater_SetFailState();

end;
--------------------------------------------------------------------------------
function private.Stage_Autoplay ()
  
  ObjSet( "match", { ap = true } );

  int_panel_map_top.ShowNotification(
    "Human, let robot do it for you!"
  );

end;
--------------------------------------------------------------------------------
function private.Stage_PlayState ( stage_state )

  if ( not private.dialog_start_stage_name ) then

    return;

  end;

  m_storage.Database_SetRO();

  local func_end = function ()

    v_stage.TryStart(
      private.dialog_start_stage_name,
      nil,
      stage_state
    );

  end;

  v_stage.Cheater_DialogStart_Hide( func_end, true );

  mediator.SubscribeOnce(
    "View_Map_Update_Triggered",
    function ()

      m_map.DestroyChangesList();
      v_map.AppearClearAllQueue();

      m_storage.Database_SetRW();

    end
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.DailyBonus ()
  
  m_dailybonus.Refresh();
  v_dailybonus.Dialog_TryShow();

end;
--------------------------------------------------------------------------------
function private.DailyQuest ()
  
  m_dailyquest.Cheater_Refresh();

end;
--------------------------------------------------------------------------------
function private.RefreshShop ()
  
  m_market.Cheater_Refresh();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Map_WinAllStates ()

  local stage_name = int_dialog_stage_start.GetStageName();
  local stage_data = v_stage.Cheater_GetStartData( stage_name );

  v_stage.Cheater_DialogStart_Hide();

  local pick_quest_item = false; -- Pick only questitem reward

  if ( stage_data.noreward ) then 

    if ( private.HasQuestItemReward( stage_data.reward ) ) then
      pick_quest_item = true;
    else
      return false;
    end;

  end;

  v_map.Interface_Disable( false );

  -- Max state
  local state_max = math.min( stage_data.states, 3 );
  
  -- Collectable rewards range
  local state_from = stage_data.state;
  local state_to   = state_max;

  if ( stage_data.stateless ) then
    state_to = 1;
  end;

  if ( stage_data.type == "fight" ) then
    state_to = state_from;
  end;

  local fireflies_album_locked = not m_fireflies.Unlocked();

  -- Collect rewards
  local rewards   = stage_data.reward;
  local collected = {};
  local collected_gold = 0;

  for state = state_from, state_to do

    -- Notify about completed fight
    if ( stage_data.type == "fight" ) then

      mediator.Notify( "Model_Stage_EnemyDefeated", 
                       { name = stage_data.enemy.name } );

    end;

    -- Notify about completed stage
    local no_reward = nil;

    local notify_state_from = state;
    local notify_state_to   = state + 1;

    if ( notify_state_to > state_max ) then

      notify_state_to = state_max;
      no_reward = true;

    end;
    
    local enabled = true;
    local completed   = true;
    local enable_next = true;
    local saved_type  = nil

    if ( stage_data.realtype == "misc" ) and ( stage_data.type == "fight" ) then

      notify_state_from = 4;
      notify_state_to   = 1;
      no_reward = nil;

      enable_next = false;
      completed   = false;

      saved_type  = "ride";
      
    elseif ( stage_data.realtype == "fight" ) then

      notify_state_to = 1;
      enabled = false;

    end;

    -- Update save
    local query =
    {
      query      = "update",
      group      = "player",
      collection = "stages",
      indice     = "name",
      criteria   = { like = stage_name },
      set        = {
                     completed = completed,
                     score     = 99999,
                     state     = notify_state_to,
                     noreward  = no_reward,
                     type      = saved_type,
                     enabled   = enabled
                   }
    };

    m_storage.Execute( query );

    local notify_data = 
    { 
      name = stage_name,
      type = stage_data.type,
      state_old = notify_state_from,
      state_new = notify_state_to,
      noreward = no_reward,
      enable_next = enable_next
    }; 

    mediator.Notify( "Model_Stage_StageWon", notify_data );

    -- Obtain rewards
    local state_rewards = rewards[ state ];
    for i = 1, #state_rewards do

      local reward = state_rewards[ i ];

      if ( pick_quest_item ) then

        if ( reward.type == "quest" ) then
          mediator.Notify( "Model_Stage_QuestItemDropped", 
                           { name = reward.name, stage = stage_name } );

          table.insert( collected, reward );

        end;

      else
        if ( reward.type ) then

          if ( reward.type == "quest" ) then

            mediator.Notify( "Model_Stage_QuestItemDropped", 
                             { name = reward.name, stage = stage_name } );

          elseif ( reward.type == "worker" ) then

            -- Worker collected in visual changes

          elseif ( reward.type == "gold" ) then

            collected_gold = collected_gold + reward.count;

          else
            reward.stage_name = stage_name;
            reward.stage_state = notify_state_from;
            m_items.TryObtain( reward );
          end;

          table.insert( collected, reward );

        end;
      end;
      
    end;

    -- Show merged reawrds at last iteration
    if ( state == state_to ) then

      if ( fireflies_album_locked ) then

        local unlock_album_stage = m_storage.Settings( "album" ).unlock;

        if ( stage_name == unlock_album_stage ) then
          v_fireflies.Panel_TryShow();
        end;

      end;

      -- Append resources
      for i, chip_name in pairs( stage_data.resources ) do

        local res_id = tonumber( chip_name:match( "%d+" ) );
        if ( res_id ) then

          local res_reward = 
          {
            type       = "resource",
            name       = "resource"..res_id,
            count      = math.random( 1, 50 ),
            stage_name = stage_name,
            stage_state = notify_state_from
          };
  
          m_items.TryObtain( res_reward );
          table.insert( collected, res_reward );

        end;

      end;

      -- Append merged gold
      if ( collected_gold > 0 ) then

        local gold_reward =
        { 
          name       = "gold",
          type       = "gold",
          count      = collected_gold,
          stage_name = stage_name,
          stage_state = notify_state_from
        };

        m_items.TryObtain( gold_reward );

      end;

      local is_first = ( ( state_from == 1 ) or ( state_from == 4) );
      
      if ( is_first ) then
    
        local query =
        {
          query      = "update",
          group      = "player",
          collection = "info",
          set        = { stage_last = stage_name }
        };
        m_storage.Execute( query );
      
      end;
      
      m_map.Update( stage_name,
                    is_first,
                    private.MergeRewards( collected ),
                    no_reward,
                    enable_next );

      -- Tutorials hook
      mediator.Notify( "View_Tutorial_MapOpened" );

      v_map.Cheater_Update();
      v_quests.Questlist.Update( stage_data.section );

    end;

  end;

  return true;
  
end;
--------------------------------------------------------------------------------
function private.MergeRewards ( rewards, resources )

  local states_rewards = common.DeepCopy( rewards );

  local merged_rewards = {};
  local collected =
  {
    gold  = nil,
    items = {}
  };

  for s = 1, #states_rewards do

    local reward = states_rewards[ s ];

    if ( reward.type == "gold" ) then

      if not ( collected.gold ) then
        collected.gold = reward;
      else
        collected.gold.count = collected.gold.count + reward.count;
      end;

    elseif ( reward.type == "resource" ) then

      -- NOTE: Resources already merged
      table.insert( merged_rewards, reward );

    else

      table.insert( collected.items, reward );

    end;

  end;

  if ( collected.gold ) then
    table.insert( merged_rewards, collected.gold );
  end;

  if ( #collected.items > 0 ) then

    table.insert( merged_rewards, collected.items[ 1 ] );

    if ( #collected.items > 1 ) then

      DbgTrace( "[ ERROR ][ CHEATER ] More then one item in rewards!" );
      DbgTrace( collected.items );

    end;

  end;

  return merged_rewards;

end;
--------------------------------------------------------------------------------
function private.HasQuestItemReward ( states_rewards )

  for state = 1, #states_rewards do

    local state_rewards = states_rewards[ state ];
    for reward = 1, #state_rewards do

      if ( state_rewards[ reward ].type == "quest" ) then
        return true;
      end;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Unlock_Section ( section_number )

  for s = 1, section_number do

    local section_name = string.format( "section_%s", s );

    if ( s > 1 ) then
      m_map.Cheater_UnlockSection( section_name );
    end;

    if ( s < section_number ) then

      m_map.Section_SetIntro( section_name, true );
      m_map.Section_SetOutro( section_name, true );
      private.EnableStages( section_name );

    end;

    if ( s == section_number ) then

      local first_stage = m_map.Section_Stage_First( section_name );

      local query =
      {
        query      = "update",
        group      = "player",
        collection = "info",
        set        =
        {
          section    = s,
          stage_last = first_stage
        }
      };

      m_storage.Execute( query );

      m_stage.TryEnable( first_stage );

    end;

  end;

  m_client.Section_SetAllowed( "section_"..section_number );

  -- Complete some tutorials
  local tutorials = { "intro", "worldmap" };

  for i = 1, #tutorials do

    local tutorial = tutorials[ i ];

    if not ( m_tutorial.IsCompleted( tutorial ) ) then
      m_tutorial.Complete( tutorial );
    end;

  end;

  common.Return( true );

end;
--------------------------------------------------------------------------------
function private.Map_EnableStagesHere ()

  private.EnableStages( m_map.Section_Current_Name() );
  common.Return( true );

end;
--------------------------------------------------------------------------------
function private.EnableStages ( section_name )

  local stages_query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "section",
    criteria   = { like = section_name }
  };
 
  local stages = m_storage.Execute( stages_query ).result;

  if ( stages ) then

    for i = 1, #stages do
      
      local stage_name = stages[ i ].name;
      m_stage.TryEnable( stage_name );

    end;

    private.Unlock_Boosters( section_name );

  end;

end;
--------------------------------------------------------------------------------
function private.Unlock_Boosters ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "chests",
    indice     = "section",
    criteria   = { like = section_name }
  };

  local section_chests = m_storage.Execute( query ).result;

  if ( section_chests ) then

    for c = 1, #section_chests do

      local booster_name = section_chests[ c ].content.name;
      m_boosters.Unlock( booster_name );

    end;

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.GetAllSkins ()

  local skins = m_skins.GetAllSkinsInfo();

  for i, skin in pairs( skins ) do

    if not ( skin.wearable ) then

      local total     = skin.parts or 0;
      local collected = skin.collected_parts or 0;

      if ( collected < total ) then

        local name = skin.name .. "_item";
        local what = "skinpart";
        local count = total - collected;

        for item = 1, count do

          m_items.TryObtain( name, what, 1 );

        end;

      end;

    end;

  end;

  int_panel_map_top.ShowNotification(
    "All skins are unlocked."
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.GetAllBoosts()

  local boosters = m_boosters.GetList();
  
  for id, booster in pairs( boosters ) do
  
    m_boosters.Unlock( booster.name );
    m_items.TryObtain( booster.name, "booster", 999 );
  
  end;

  int_panel_map_top.ShowNotification(
    "All boosters are unlocked."
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.SetNoDBMode ()
  
  if ( m_storage.Database_Detach() ) then
  
    m_object.Set( "btn_cheater_nodbmode",  { alp = 0.5, input = false } );

    int_panel_map_top.ShowNotification(
      "Further progress won't be saved."
    );

  end;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Autotest ()
  
  private.Autotest_Start();
  
end;
--------------------------------------------------------------------------------
function private.Autotest_WinPoint ()
  
  mediator.SubscribeOnce(
    "View_Stage_DialogStartActivated",
    function()
      
      local function func_accept ()
        
        local window_name = int_window.Upper_Name();
        
        if ( window_name ) then
        
          local accept_func = int_window.Handler_Accept( window_name );
          
          if ( accept_func ) then
    
            accept_func();
            
          end;
          
        end;
        
      end;
      
      mediator.SubscribeOnce(
        "Interface_Window_WindowOpened",
        function ( window_name )
          
          --DbgTrace( "Autotest: window opened: "..window_name );
      
          common.Defer(
            "autotest_accept_reward",
            2,
            function ()
              
              func_accept();
              
            end
          );
          
        end
      );
      
      if ( private.Map_WinAllStates() ) then
      
        --
        
      else
        
        mediator.Unsubscribe(
          "Interface_Window_WindowOpened",
          func_accept
        );
        
        private.Autotest_Stop();
    
      end;
      
    end
  );
  
  local current_point = v_map.Marker_CurrentPoint();
  
  v_stage.DialogStart_Show( current_point );
  
end;
--------------------------------------------------------------------------------
function private.Autotest_Start ()

  mediator.Subscribe(
    "View_Map_Update_Finished",
    private.Autotest_Continue
  );
  
  v_tutorial.Disable();
  
  private.Autotest_Continue();

  int_panel_map_top.ShowNotification(
    "Autotest started."
  );
  
end;
--------------------------------------------------------------------------------
function private.Autotest_Continue ()
  
  common.Defer(
    "autotest_win_point",
    2,
    function ()

      private.Autotest_WinPoint();

    end
  );
  
end;
--------------------------------------------------------------------------------
function private.Autotest_Stop ()
  
  mediator.Unsubscribe(
    "View_Map_Update_Finished",
    private.Autotest_Continue
  );
  
  v_tutorial.Enable();

  int_panel_map_top.ShowNotification(
    "Autotest can't continue."
  );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Starterpack ()

  v_shop.Cheat_Starterpack();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.InterfaceLock_Show ()
  
  local color_object = "ng_interface_lock_spr";
  local text_object = "txt_cheater_intlock";
  
  local alp = ObjGet( color_object ).alp;
  local alp_new = 0.0;
  
  if ( alp == alp_new ) then
    
    alp_new = 0.5;  
    
  end;
  
  ObjMultiSet( {
    { color_object, { alp = alp_new } },
    { text_object, { alp = alp_new + 0.5 } }
  } );

end;
--------------------------------------------------------------------------------
-- function *** TOP PANEL *** () end;
--------------------------------------------------------------------------------
function private.SetOnlineState ( state )
  
  if ( state ) then
    
    ObjSet( "txt_cheater_info_online", { text = "online" } );
    
  else
    
    ObjSet( "txt_cheater_info_online", { text = "offline" } );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.SetDebugState ( debug )
  
  if ( debug ) then
    
    ObjSet( "txt_cheater_info_debug", { text = "debug" } );
    
  else
    
    ObjSet( "txt_cheater_info_debug", { text = "no debug" } );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.ShowDrawCalls ()

  local func_update;
  func_update = function ()

    local drawcalls = ne.GetNumDrawCalls();

    ObjSet( "txt_cheater_info_drawcalls", { text = "dc: "..drawcalls } );
    ObjSet( "tmr_cheater_info_drawcalls", { time = 1, playing = true, endtrig = func_update } );

  end;

  func_update();

end;
--------------------------------------------------------------------------------
function private.ShowFPS ()

  local func_update;
  func_update = function ()

  local fps = math.floor( ne.GetFPS() );

    ObjSet( "txt_cheater_info_fps", { text = "fps: "..fps } );
    ObjSet( "tmr_cheater_info_fps", { time = 1, playing = true, endtrig = func_update } );

  end;

  func_update();

end;
--------------------------------------------------------------------------------
-- function *** TEST *** () end;
--------------------------------------------------------------------------------
function private.TestMessages ( code )

  if ( not code ) then

    code = 97;

  end;

  local messages = 
  {
    [ 97 ]  = Command_ShowSupport,
    [ 98 ]  = Command_ShowPrivacyPolicy,
    [ 99 ]  = Command_ShowTermsOfService,
    [ 100 ] = Command_RateGame, 
    [ 101 ] = Command_OpenUrl
  };

  local params = nil;

  if ( code == 101 ) then

    params = "http://www.google.com/";

  end;

  --DbgTrace( "TestMessages: code = "..code );

  MsgSend( messages[ code ], { url = params } );  

end;