-- name=common
--------------------------------------------------------------------------------
function public.Trace ( text, force )

  local all = false;

  if ( all or force ) then

    DbgTrace( text );

  end;

end;
--------------------------------------------------------------------------------
function public.Start ()

  math.randomseed( os.time() );

  DbgTrace( "[ VERSION ] "..tostring( VersionGetComponentVersion( "game" ) ) );
  DbgTrace( "[ USER ] "..tostring( loader.GetUserId() ) );
  DbgTrace( "[ SERVER ] "..ConfigGetInfo( "server.host" ) );

  private.LoadStrings();

  ObjCreate( "ng_level",    "obj" );
  ObjSet(    "ng_level",    { pos_z = 13 } );
  ObjAttach( "ng_level",    "ng_application" );
  
  InterfaceName = "ng_interface";
  ObjCreate( InterfaceName, "obj" );
  ObjSet(    InterfaceName, { pos_z = 121 } );
  ObjAttach( InterfaceName, "ng_application" );

  ld  = {};
  cmn = {};
  
  ------------------------------------------------------------
  private.init_public();
  ------------------------------------------------------------
  private.init_audio();
  ------------------------------------------------------------
  private.TapInit();

  SetCursor( CURSOR_DEFAULT );
  ------------------------------------------------------------

  ld.current_sound = GetSoundVolume( "sfx" );

  if ( ld.current_sound > 0 ) then

    ld.current_sound = true;

  else

    ld.current_sound = false;

  end;

  ld.current_music = GetSoundVolume( "soundtrack" );

  if ( ld.current_music > 0 ) then

    ld.current_music = true;

  else

    ld.current_music = false;

  end;

  ld.player_name = "";

  ld.start_game = true;

  cmn.project = {};
  ------------------------------------------------------------
  ne.msg.Subscribe( "Event_notEngine_ScriptHub_Script_Error", function ( name, params )

    private.error_info = params;

    public.MessageSend( "Event_Common_ScriptError", params );

    DbgTrace( "[ ERROR ] LUA"..public.TableToString( params ) );

    --public.ERROR = private.error_info;

  end );

--  MsgSubscribe( "Command_EditorShowLogMesage", function ( name, params )
--
--    DbgTrace( "+++ [ "..os.time().." ] "..tostring( params.message ) );
--
--  end );

  ne.msg.Subscribe( Event_LocalNotifications_GetInfo, function ()

    private.LocalNotifications_AddData();

  end );

  private.MessageClearAll();

end;
--------------------------------------------------------------------------------
function public.Save ( to_server )

  ApplicationSave( to_server );

end;
--------------------------------------------------------------------------------
function public.Return ( with_save )

  private.MessageClearAll();
  wiz.bfg.ccs.HideButton();

  RestartApplication( with_save );

end;
--------------------------------------------------------------------------------
-- function ****** Local notifications ****** () end;
--------------------------------------------------------------------------------
function public.LocalNotifications_SetData ( notification_type, value )

  common.Trace( "LocalNotifications_SetData" );

  if ( "comeback_level" == notification_type ) then

    private.comeback_level = value;

  end;

end;
--------------------------------------------------------------------------------
function private.LocalNotifications_AddData ()

  common.Trace( "LocalNotifications_AddData" );

  local events = {};

  local hearts_restore_time = m_hearts.GetFullHeartsRestoreTime();

  if ( hearts_restore_time > 0 ) then

    table.insert( events, { type = "hearts_restored", 
                            time = hearts_restore_time } );

  end;

  local mine_ready = m_mines.GetRandomMineCooldown();

  if ( mine_ready ) then

    if ( mine_ready.time and mine_ready.time > 0 ) then

      table.insert( events, {
        type = "mine_ready", 
        time = mine_ready.time, 
        param = StringGet( "str_mine_name_"..mine_ready.name ) 
      } );

    end;

  end;

  local dailyquest_cooldown = m_dailyquest.GetRemainingTime( "dailyquest" );

  if ( dailyquest_cooldown ) and ( dailyquest_cooldown > 0 ) then

    table.insert( events, { type = "dailyquest_ready", 
                            time = dailyquest_cooldown } );

  end;
  
  if ( private.comeback_level ) then
    
    local stage_name = private.comeback_level;
    
    if ( stage_name:find( "_daily" ) ) then
      
      stage_name = "dailyquest";
      
    end;
    
    table.insert( events, { 
      type  = "comeback_level", 
      time  = 3600, 
      param = StringGet( "str_stage_name_"..stage_name ) 
    } );

  end;  

  table.insert( events, { type = "comeback_over12hours", time = 43200 } );

  wiz.localnotifications.Update( { events = events } );

end;
--------------------------------------------------------------------------------
-- function ****** Playhaven ****** () end;
--------------------------------------------------------------------------------
function public.Playhaven_LogPlacement ( string_id )

  common.Trace( "Playhaven_LogPlacement: "..string_id );

  wiz.bfg.playhaven.LogPlacement( string_id );

end;
--------------------------------------------------------------------------------
-- function ****** Kontagent ****** () end;
--------------------------------------------------------------------------------
function public.Kontagent_LogEvent ( event_type, event_data )

  local event_handlers = 
  {
    economy  = private.Kontagent_EconomyEvent,
    tutorial = private.Kontagent_TutorialEvent,
    stage    = private.Kontagent_StageEvent,
    progress = private.Kontagent_ProgressEvent,
    quest    = private.Kontagent_QuestEvent,
    skins    = private.Kontagent_SkinsEvent,
    social   = private.Kontagent_SocialEvent,
    arena    = private.Kontagent_ArenaEvent
  };  

  local events_table = event_handlers[ event_type ]( event_data );

  events_table.params = private.Kontagent_AdditionalDetails( events_table or {} );
 
  common.Trace( "Kontagent_LogEvent: "..events_table.default1..", "..events_table.default2..", "..tostring( events_table.default3 )..", name = "..events_table.name..", value = "..tostring( events_table.params.v )..", level = "..tostring( events_table.params.l ) ); 
  
  wiz.bfg.kontagent.LogEvent( events_table );

end;
--------------------------------------------------------------------------------
function private.Kontagent_EconomyEvent ( event_data )

  local default1 = "economy";
  local default2 = event_data.description;
  local default3 = event_data.category or "";
  
  local value = event_data.value;
  local level = event_data.level or 0;
  local name  = event_data.name;

  local stagename = function ( stage_name )

    if ( not stage_name ) then

      return nil;

    end;

    if ( stage_name:find( "_daily" ) ) then

      return "dailyquest";

    end;
    
    local prefix = "";

    local section_query =
    {
      query      = "find",
      group      = "static",
      collection = "stages",
      indice     = "name",
      criteria = { like = stage_name }
    };

    local section_name = m_storage.Execute( section_query ).result.section;

    local section_number = tonumber( string.match( section_name, "%d+" ) );

    local stage_number = tonumber( string.match( stage_name, "%d+" ) );

    if ( section_number < 2 ) then
      
      if ( stage_name:find( "_bonus" ) ) then

        prefix = "bonus_";

      end;

      return prefix.."level_"..math.ceil( section_number * 1000 ) + tonumber( stage_name:match( "%d+" ) ); --stage_name;

    end;

    local query = 
    {
      query      = "find_all",
      group      = "static",
      collection = "stages",
      indice     = "section",
      criteria = { like = section_name }
    }; 

    local stages_list = m_storage.Execute( query ).result;

    local bonus_stages_list = {};

    for i = #stages_list, 1, - 1 do

      if ( string.find( stages_list[ i ].name, "_bonus" ) ) then

        table.insert( bonus_stages_list, 1, table.remove( stages_list, i ) );

      end;

    end;

    for i = 1, #stages_list do

      if ( stages_list[ i ].name == stage_name ) then

        stage_number = i;

        break;

      end;

    end;

    for i = 1, #bonus_stages_list do

      if ( bonus_stages_list[ i ].name == stage_name ) then
        
        prefix = "bonus_";

        stage_number = i;

        break;

      end;

    end;

    return prefix.."level_"..math.ceil( section_number * 1000 ) + tonumber( stage_number );--"stage_"..stage_number;

  end;

  local stage = nil;
  
  if ( cmn.GetObjectPrefix( name ) == "stage" ) then

    stage = event_data.name;
    name = stagename( name );

  end;

  if ( event_data.type ) then

    local resources = 
    {
      resource1 = "yellowflies",
      resource2 = "blueflies",
      resource3 = "purpleflies"
    };

    default2 = string.gsub( default2, "currency", resources[ event_data.type ] );

    event_data.type = nil;

  end;
  
  --[[DbgTrace( { 
    name = name,
    default1 = default1,
    default2 = default2,
    default3 = default3,
    params = { 
      v = value, 
      l = level,
      transaction_id = event_data.transaction_id,
      source = event_data.source,
      stage = stage,
      state = event_data.state,
      purchase = event_data.purchase
    } } );]]

  return { 
    name = name,
    default1 = default1,
    default2 = default2,
    default3 = default3,
    params = { 
      v = value, 
      l = level,
      transaction_id = event_data.transaction_id,
      source = event_data.source,
      stage = stage,
      state = event_data.state,
      purchase = event_data.purchase
    } };

end;
--------------------------------------------------------------------------------
function private.Kontagent_TutorialEvent ( event_data )

  local default1 = "tutorial";
  local default2 = "";
  local default3 = "";
  
  local value = event_data.value;
  local level = event_data.level;
  local name  = event_data.name;

  return { 
    name     = name,
    default1 = default1,
    default2 = default2,
    default3 = default3,
    params   = { v = value, l = level }
  };

end;
--------------------------------------------------------------------------------
function private.Kontagent_StageEvent ( event_data )

  local getdefault2 = function ( stage_name )

    if ( string.find( stage_name, "_bonus" ) ) then

      return "bonus_level_complete"

    end

    if ( string.find( stage_name, "_daily" ) ) then

      return "daily_level_complete"

    end

    return "level_complete";

  end;

  local getlevel = function ( event_name, stage_state, noreward )

    if ( event_name == "success" ) then

      if ( stage_state == 4 ) or ( stage_state == 3 and noreward ) then
  
        return stage_state;

      end;

      local state = stage_state;
  
      if ( state == 0 ) then

        state = 1;

      end;

      return state;

    end;

    return stage_state;

  end;

  local getvalue = function ( section_name, stage_name, daily )

    if ( not section_name and daily ) then
      
      return daily;

    end;

    local section_number = tonumber( string.match( section_name, "%d+" ) );

    local stage_number = tonumber( string.match( stage_name, "%d+" ) );

    if ( section_number < 2 ) then

      return stage_number;

    end;

    local query = 
    {
      query      = "find_all",
      group      = "static",
      collection = "stages",
      indice     = "section",
      criteria   = { like = section_name }
    }; 

    local stages_list = m_storage.Execute( query ).result;

    local bonus_stages_list = {};

    for i = #stages_list, 1, - 1 do

      if ( string.find( stages_list[ i ].name, "_bonus" ) ) then

        table.insert( bonus_stages_list, 1, table.remove( stages_list, i ) );

      end;

    end;

    for i = 1, #stages_list do

      if ( stages_list[ i ].name == stage_name ) then

        stage_number = i;

        break;

      end;

    end;

    for i = 1, #bonus_stages_list do

      if ( bonus_stages_list[ i ].name == stage_name ) then

        stage_number = i;

        break;

      end;

    end;

    return stage_number;

  end;

  local tiles, butterflies, bubbles;

  local getcellsdestroyed = function ( cells_data )

    if ( cells_data ) then

      if ( cells_data.type == "Dirt" ) then

        tiles = cells_data.value;

      end;

      if ( cells_data.type == "Bug" ) then

        butterflies = cells_data.value;

      end;
      
      if ( cells_data.type == "Bubble" ) then

        bubbles = cells_data.value;

      end;

    end;

  end;

  getcellsdestroyed( event_data.cells );

  local default1 = "progression";
  local default2 = getdefault2( event_data.stage );
  local default3 = event_data.section;
  
  local value = getvalue( event_data.section, event_data.stage, event_data.daily ) or 0;
  local level = getlevel( event_data.name, event_data.state, event_data.noreward );
  local name  = event_data.name;

  local score   = event_data.score;
  local moves   = event_data.moves;
  local boost   = event_data.boost[ 1 ];
  local power   = event_data.boost[ 2 ];
  local items   = event_data.items;
  local flies   = event_data.flies;
  local bombs   = event_data.bombs;
  local is_boss = event_data.is_boss;

  event_data.state    = nil;
  event_data.noreward = nil;
  event_data.score    = nil;
  event_data.moves    = nil;
  event_data.boost    = nil;
  event_data.cells    = nil;
  event_data.items    = nil;
  event_data.flies    = nil;
  event_data.bombs    = nil;
  event_data.bubbles  = nil;
  event_data.is_boss  = nil;

  --[[DbgTrace( { 
    name = name, 
    default1 = default1, 
    default2 = default2, 
    default3 = default3, 
    params = { 
      v = value, 
      l = level, 
      score = score, 
      moves = moves, 
      boost = boost, 
      power = power, 
      tiles = tiles, 
      butterflies = butterflies, 
      items = items, 
      flies = flies, 
      bubbles = bubbles, 
      bombs = bombs, 
      is_boss = is_boss
    } 
  } );]]
  
  return { 
    name = name, 
    default1 = default1, 
    default2 = default2, 
    default3 = default3, 
    params = { 
      v = value, 
      l = level, 
      score = score, 
      moves = moves, 
      boost = boost, 
      power = power, 
      tiles = tiles, 
      butterflies = butterflies, 
      items = items, 
      flies = flies, 
      bubbles = bubbles, 
      bombs = bombs, 
      is_boss = is_boss
    } 
  };

end;
--------------------------------------------------------------------------------
function private.Kontagent_ProgressEvent ( event_data )

  -- Reset progress option / section complete event.
  local default1 = "progression";
  local default2 = tern( event_data.name:find( "section_" ) )( "world_complete", "" );
  local default3 = "";

  return {
    name     = event_data.name, 
    default1 = default1, 
    default2 = default2, 
    default3 = default3, 
    params   = { v = 0, l = 0 }
  };

end;
--------------------------------------------------------------------------------
function private.Kontagent_QuestEvent ( event_data )

  local default1 = "progression";
  local default2 = "quest_complete";
  local default3 = event_data.section;

  return {
    name     = event_data.name,
    default1 = default1,
    default2 = default2,
    default3 = default3,
    params   = { v = 0, l = 0 }
  };

end;
--------------------------------------------------------------------------------
function private.Kontagent_SkinsEvent ( event_data )

  local default1 = "skins";
  local default2 = event_data.description;
  local default3 = event_data.category;

  return { 
    name = event_data.name, 
    default1 = default1, 
    default2 = default2, 
    default3 = default3, 
    params = { v = event_data.value or 0, l = 0, tokens = event_data.tokens } 
  };

end;
--------------------------------------------------------------------------------
function private.Kontagent_SocialEvent ( event_data )

  local default1 = "social";
  local default2 = event_data.description;
  local default3 = event_data.category or "";

  return {
    name = event_data.name,
    default1 = default1,
    default2 = default2,
    default3 = default3,
    params = { v = 0, l = 1, 
      login_channel = ( wiz.social.GetChannel() or {} ).login_channel }
  };

end;
--------------------------------------------------------------------------------
function private.Kontagent_ArenaEvent ( event_data )

  local default1 = "arena";
  local default2 = event_data.description or "";
  local default3 = event_data.category or "";
  
  return {
    name = event_data.name,
    default1 = default1,
    default2 = default2,
    default3 = default3,
    params = { v = 0, l = event_data.level }
  };

end;
--------------------------------------------------------------------------------
function private.Kontagent_AdditionalDetails ( event_data )

  local query;
  local additional_data = event_data.params;
  
  local timeplayed = function ()

    local token = "wtc_ingame_time";

    if ( TimeIsWatchExist( token ) ) then

      local time = TimeGetWatchTime( token );

      return tostring( time );

    end;

    return tostring( 0 );

  end;

  local find_last_generic_stage = function ( stage_name )

    query =
    {
      query      = "find_all",
      group      = "player",
      collection = "stages"
    };
    
    local stages_list = m_storage.Execute( query ).result;

    if ( stages_list ) then

      for i = #stages_list, 1, -1 do

        if not (    stages_list[ i ].name:find( "_bonus" ) 
                 or stages_list[ i ].name:find( "_daily" ) ) then

          local stage_number = tonumber( stages_list[ i ].name:match( "%d+" ) );

          if ( stages_list[ i ].enabled ) then

            return "stage_" .. stage_number;

          end;

        end;

      end;

    end;

    DbgTrace( "[ ERROR ] Last generic stage not found. Returned 'stage_0'." );
    return "stage_0";

  end;

  local section_name;
  
  local maxlevel = function ()

    local stage_last = m_storage.Player.Info().stage_last;

    query =
    {
      query      = "find",
      group      = "static",
      collection = "stages",
      indice     = "name",
      criteria   = { like = stage_last }
    };

    section_name = m_storage.Execute( query ).result.section;

    local section_number = tonumber( section_name:match( "%d+" ) );

    if ( stage_last:find( "_bonus" ) or stage_last:find( "_daily" ) ) then

      stage_last = find_last_generic_stage( stage_last ); 

    end;

    query =
    {
      query      = "find",
      group      = "player",
      collection = "stages",
      indice     = "name",
      criteria   = { like = stage_last }
    };

    local latest = m_storage.Execute( query ).result;

    if ( latest and latest.completed ) then

      local stage_last_next = "stage_"..( tonumber( stage_last:match( "%d+" ) ) + 1 );

      query =
      {
        query      = "find",
        group      = "static",
        collection = "stages",
        indice     = "name",
        criteria   = { like = stage_last_next }
      };

      local enabled = m_storage.Execute( query ).result;

      if ( enabled ) then

        stage_last = stage_last_next;

      end;

    end;

    stage_last = math.ceil( section_number * 1000 ) + tonumber( stage_last:match( "%d+" ) );

    if ( section_number > 1 ) then

      stage_last = stage_last - 110 * ( section_number - 1 );

    end;

    return tonumber( stage_last );

  end;

  local userlvl = function ( stage_name )

    if ( not stage_name ) then

      return nil;

    end;
    
    local stage_number = tonumber( string.match( stage_name, "%d+" ) );
    if ( stage_name:find( "_daily" ) ) then
      return stage_number.."_d";
    end;
    
    query =
    {
      query      = "find",
      group      = "static",
      collection = "stages",
      indice     = "name",
      criteria   = { like = stage_name }
    };

    local section_name = m_storage.Execute( query ).result.section;
    local section_number = tonumber( string.match( section_name, "%d+" ) );
    
    if ( stage_name:find( "_bonus" ) ) then
      query = 
      {
        query      = "find_all",
        group      = "static",
        collection = "stages",
        indice     = "section",
        criteria   = { like = section_name }
      }; 

      local stages_list = m_storage.Execute( query ).result;

      local bonus_stages_list = {};

      for i = #stages_list, 1, - 1 do
        if ( string.find( stages_list[ i ].name, "_bonus" ) ) then
          table.insert( bonus_stages_list, 1, table.remove( stages_list, i ) );
        end;
      end;

      for i = 1, #bonus_stages_list do
        if ( bonus_stages_list[ i ].name == stage_name ) then
          stage_number = i;
          break;
        end;
      end;
      
      return ( math.ceil( section_number * 1000 ) + stage_number ).."_b";
    end;    
    
    if ( section_number > 1 ) then
      stage_number = stage_number - 110 * ( section_number - 1 );
    end;

    return math.ceil( section_number * 1000 ) + stage_number;

  end;

  local currency = function ( resource_name, section_name )
    
    local currency = m_storage.Player.Info()[ resource_name ] or 0;
    
    if not ( resource_name == "gold" ) then
      
      -- JK: fake -1 if the currency is nil by that index
      currency = currency[ tonumber( section_name:match( "%d+" ) ) ] or -1;
      
    end;
    
    if ( currency < 0 ) then
      
      currency = 0;
      
    end;
    
    return tostring( currency );

  end;

  local ispaying = function ()

    local result = m_storage.Player.Info().ispaying or 0;
    result = tern( result == 0 )( 0, 1 );

    return result;

  end;

  local isfb = function ()

    local result = SocialIsLogged().result;

    if ( result ) then

      return 1;

    end;

    return 0;

  end;

  local userlvltier = function ( stage_name )

    if ( stage_name ) then

      return m_stage.Kontagent_GetStageState( stage_name );

    end;

    return nil;

  end;

  local is_googleplay_connected = function ()

    if ( wiz.Android.GooglePlayIsConnected() ) then

      return 1;

    end;

    return 0;

  end;

  additional_data.appVersion = string.match( 
                        VersionGetComponentVersion( "game" ), "(%d+.%d+.%d+)" );
  additional_data.host       = private.Kontagent_GetHostAddress();
  additional_data.cntVersion = string.match( parts.version.GetCurrent( "content" ), 
                               "%d+.%d+.%d+.(%d+)" );
  additional_data.timeplayed = timeplayed();
  additional_data.maxlevel   = maxlevel();
  additional_data.userlvl    = tostring( 
                                 userlvl( m_stage.Kontagent_GetCurrentStage() ) 
                              or userlvl( additional_data.stage ) 
                              or 0 );
  additional_data.userlvltier = tostring( 
                             userlvltier( m_stage.Kontagent_GetCurrentStage() )
                             or additional_data.state or 0 );
  additional_data.softbal_yellowflies = currency( "resource1", section_name );
  additional_data.softbal_blueflies   = currency( "resource2", section_name );
  additional_data.softbal_purpleflies = currency( "resource3", section_name );
  additional_data.premiumbal  = currency( "gold" );
  additional_data.isfb        = isfb();
  additional_data.isgc        = is_googleplay_connected();
  additional_data.ispaying    = ispaying();
  additional_data.stickers    = tostring( m_fireflies.Kontagent_GetStickersTotal() );
  additional_data.currentskin = m_skins.GetSkin() or "default";
  
  if ( IsMobile() ) then
    
    additional_data.friends   = tostring( #m_social.GetFriends() or 0 );
    
  else
    
    additional_data.friends   = "0";
    
    local bfginfo = wiz.bfg.lot.GetBigFishInfo();
    
    additional_data.wrappingID = bfginfo.wrappingID;
    additional_data.gmguid     = bfginfo.gmguid;
    additional_data.bfgudID    = bfginfo.bfgudID;
    additional_data.userID     = bfginfo.userID;
    
  end;
  
  additional_data.stage = nil;
  additional_data.state = nil;
  
  --DbgTrace( additional_data );

  return additional_data;

end;
--------------------------------------------------------------------------------
function private.Kontagent_GetHostAddress ()

  local host = ConfigGetInfo( "server.host" ) or "UNKNOWN";

  local hosts_list = 
  {
    { host = "http://dev1.fairysaga.elephant-games.com/", type = "EG_DEV1" },
    { host = "http://dev2.fairysaga.elephant-games.com/", type = "EG_DEV2" },
    { host = "http://dev3.fairysaga.elephant-games.com/", type = "EG_DEV3" },
    { host = "http://f2p-qa.bigfishgames.com/FairySaga1/", type = "BFG_QA1" },
    { host = "http://f2p-qa.bigfishgames.com/FairySaga2/", type = "BFG_QA2" },
    { host = "http://f2p-st.bigfishgames.com/FairySaga/", type = "BFG_STAGE" },
    { host = "http://f2p.bigfishgames.com/FairySaga/", type = "BFG_PROD" }
  };

  for i = 1, #hosts_list do

    if ( hosts_list[ i ].host:find( host, 1, true ) ) then

      host = hosts_list[ i ].type;

    end;

  end;
  
  return host;

end;
--------------------------------------------------------------------------------
-- function ****** Message Subscribers ****** () end
--------------------------------------------------------------------------------
function public.MessageSend ( msg, param )

  return ne.msg.Send( msg, param );

end;
--------------------------------------------------------------------------------
function public.MessageSubscribe ( msg, func )

  private.message_list = private.message_list or {};

  table.insert( private.message_list, { msg = msg, func = func } );

  ne.msg.Subscribe( msg, func );

end;
--------------------------------------------------------------------------------
function public.MessageUnsubscribe ( msg, func )

  ne.msg.Unsubscribe( msg, func );

end;
--------------------------------------------------------------------------------
function public.MessageClearAll ()

  private.MessageClearAll();

end;
--------------------------------------------------------------------------------
function private.MessageClearAll ()

  if ( private.message_list ) then

    for m = 1, #private.message_list do

      local data = private.message_list[ m ];

      public.MessageUnsubscribe( data.msg, data.func );

    end;

    private.message_list = {};

  end;

end;
--******************************************************************************
-- function ****** Unresolved ****** () end
--******************************************************************************
--******************************************************************************
function private.init_public ()
  
  -- возвращает префикс объекта
  function cmn.GetObjectPrefix( object )

    local underscore = string.find( object, "_" );

    local prefix = nil;

    if ( underscore ) then

      prefix = string.sub( object, 1, ( underscore - 1 ) );

    end;

    return prefix;

  end;
  --------------------------------------------------------------------
  -- возвращает имя объекта без префикса
  function cmn.GetObjectName( object )

    local underscore = string.find( object, "_" );

    local name = nil;

    if ( underscore ) then

      name = string.sub( object, ( underscore + 1 ) );

    end;

    return name;

  end;
  --------------------------------------------------------------------
  -- возвращает постфикс объекта
  function cmn.GetObjectPostfix( object )

    object = string.reverse( object );

    local postfix = cmn.GetObjectPrefix( object );

    postfix = string.reverse( postfix );

    return postfix;

  end;
  --------------------------------------------------------------------
  -- возвращает имя объекта без постфикса
  --function cmn.GetObjectNameNoPostfix( object )
  --  
  --  local postfix = cmn.GetObjectPostfix( object );
  --  
  --  local name = string.sub( object, 1, ( string.len( object ) - string.len( postfix ) - 1 ) );
  --  
  --  return name;
  --  
  --end;
  --------------------------------------------------------------------
  function cmn.GetTimeFromSec ( seconds )

    seconds = math.floor( seconds + 0.5 );
    
    local day_sec    = 60 * 60 * 24;
    local hour_sec   = 60 * 60;
    local minute_sec = 60; 
    
    local d = math.floor( seconds / day_sec );
    seconds = seconds - ( d * day_sec );
    
    local h = math.floor( seconds / hour_sec );
    seconds = seconds - ( h * hour_sec );
    
    local m = math.floor( seconds / minute_sec );
    seconds = seconds - ( m * minute_sec );
    
    local s = seconds;
    
    local time_struct =
    {
      d = string.format( "%.2d", d ),
      h = string.format( "%.2d", h ),
      m = string.format( "%.2d", m ),
      s = string.format( "%.2d", s )
    };
    
    return time_struct;

  end;  
  --************************************************************************** 
  -- Rost -> неведомая функция, используется при клонировании объектов
  function cmn.GetObjParamsForSet( object )

    local params = ObjGet( object );

    if ( params ) then

      params.name = nil;

    end;

    return params;

  end;
  --*****************************************************************
  -- AL> Возвращает порядковый номер entity, если он есть в list.
  -- Возможно сравнение по entity.field == list[N].field, если указан field

  function cmn.IsEntityInList( list, entity, field, force_boolean )

    if ( list and #list > 0 ) then

      for l = 1, #list do

        if ( field and ( list[ l ][ field ] == entity[ field ] ) ) or
           ( list[ l ] == entity                                 ) then

          if ( force_boolean ) then

            return true;

          end;

          return l;

        end;

      end;

    end;

    return false;

  end;
  --*****************************************************************
  function cmn.ConvertToAssociated( indexed, field )
    
    if ( type( indexed ) ~= "table" ) then
      public.Trace( "[ ERROR ][ COMMON ] ConvertToAssociated -> Not table");
      return;
    end;
    
    if ( #indexed == 0 ) then
      public.Trace( "[ WARNING ][ COMMON ] ConvertToAssociated -> Already associated or empty");
      return;
    end;
    
    field = field or "name";
    local associated = {};
    
    for i = 1, #indexed do
      local key = indexed[ i ][ field ];
      associated[ key ] = indexed[ i ];
    end;
    
    return associated;
    
  end;
  
end;
--******************************************************************************
function private.init_audio ()

  local init_channel = function ( channel, volume ) 
    
    local current_volume = GetSoundVolume( channel );
    if ( current_volume > 0 ) then
      current_volume = volume or current_volume;
    end;
    SetSoundVolume( channel, current_volume );

  end; 

  init_channel( "sfx" );
  init_channel( "soundtrack", 0.5 );

  --------------------------------------------------------------------------------
  -- sound
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  function cmn.PlaySnd( name, loop, fade )

    local path = "assets/content/audio/sound/aud_"..name;

    loop = loop or 0;
    fade = fade or 0;

    -- detect absolute path
    if ( string.find( name, "/" ) ) then

      path = name;

    end;

    ne.snd.Play( path, "sfx", loop, "", fade );

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  function cmn.StopSnd( name, fade )

    local path = "assets/content/audio/sound/aud_"..name;

    fade = fade or 0;

    -- detect absolute path
    if ( string.find( name, "/" ) ) then

      path = name;

    end;

    ne.snd.Stop( path, fade );

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  function cmn.GetSound()

    return ld.current_sound;

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  function cmn.SetSound()

    local set_value;

    if ( ld.current_sound ) then

      ld.current_sound = false;
      set_value = 0.0;

    else

      ld.current_sound = true;
      set_value = 0.7;

    end;

    SetSoundVolume( "sfx", set_value );

    --cmn.LogTrace( "sound" );

    return ld.current_sound;

  end;
  --------------------------------------------------------------------------------
  -- music
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  function cmn.PlayMusic( name, loop, fade )

    local path = "assets/content/audio/music/aud_"..name.."_mus";
    
    -- detect absolute path
    if ( string.find( name, "/" ) ) then

      path = name;

    end;

    loop = loop or 1;
    fade = fade or 2;
    
    ne.snd.Play( path, "soundtrack", loop, "", fade );

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  function cmn.StopMusic( name, fade )

    local path = "assets/content/audio/music/aud_"..name.."_mus";

    -- detect absolute path
    if ( string.find( name, "/" ) ) then

      path = name;

    end;
    
    fade = fade or 0.5;

    ne.snd.Stop( path, fade );

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  function cmn.GetMusic()

    return ld.current_music;

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  function cmn.SetMusic()

    local set_value;

    if ( ld.current_music ) then

      ld.current_music = false;
      set_value = 0.0;

    else

      ld.current_music = true;
      set_value = 0.5;

    end

    SetSoundVolume( "soundtrack", set_value );

    --cmn.LogTrace( "music" );

    return ld.current_music;

  end;

end;
--********************************************************************************************************************
--********************************************************************************************************************
-- function ****** Taps ****** () end
--********************************************************************************************************************
--********************************************************************************************************************
function private.TapTrace ( text )

  --DbgTrace( "[ common ] [ TAP ] "..text );

end;
--********************************************************************************************************************
function private.TapInit ()

  private.tap = {};

  private.tap.list = {};

  private.tap.object_current = nil;

  private.tap.TIMER_LOCK_OBJECT = "tmr_common_tap_lock";
  private.tap.TIMER_LOCK_TIME   = 0.5;

  ObjCreate( private.tap.TIMER_LOCK_OBJECT, "timer" );
  ObjAttach( private.tap.TIMER_LOCK_OBJECT, InterfaceName );

  private.tap.TIMER_HOLD_OBJECT = "tmr_common_tap_hold";
  private.tap.TIMER_HOLD_TIME   = 1.0;

  ObjCreate( private.tap.TIMER_HOLD_OBJECT, "timer" );
  ObjAttach( private.tap.TIMER_HOLD_OBJECT, InterfaceName );

  private.tap.anim = {};

  private.TapInitAnims();

end;
--********************************************************************************************************************
function public.TapClear ( object )

  if ( private.tap.list[ object ] ) then

    private.tap.list[ object ].assigned = false;

  end;

end;
--********************************************************************************************************************
function public.TapBreak ( object )

  if ( not private.TapIsCurrent( ( object or private.TapGetCurrent() ) ) ) then

    return;

  end;

  private.TapSetCurrent();

  ObjSet( private.tap.TIMER_HOLD_OBJECT, { playing = false } );

end;
--********************************************************************************************************************
function public.TapSet ( object, params )

  private.tap.list[ object ] = private.tap.list[ object ] or {};

  private.tap.list[ object ].anim = params.anim or private.tap.list[ object ].anim;

  private.tap.list[ object ].func_true      = params.func_true      or private.tap.list[ object ].func_true;
  private.tap.list[ object ].func_false     = params.func_false     or private.tap.list[ object ].func_false;
  private.tap.list[ object ].func_menter    = params.func_menter    or private.tap.list[ object ].func_menter;
  private.tap.list[ object ].func_mleave    = params.func_mleave    or private.tap.list[ object ].func_mleave;
  private.tap.list[ object ].func_hold_down = params.func_hold_down or private.tap.list[ object ].func_hold_down;
  private.tap.list[ object ].func_hold_up   = params.func_hold_up   or private.tap.list[ object ].func_hold_up;

  -- DK> Введено для случаев, когда нужно дополнительно обработать mouse up "мимо" целевого объекта ( т.к. ObjWaitForMouseUp перетирает событие ).
  if ( params.func_false ) then

    ObjWaitForMouseUp( object, function ()

      private.TapCallFuncFalse( object );

    end );

  end;

  if not ( private.tap.list[ object ].assigned ) then

    ObjSet( object,
    {
      event_mdown  = function () private.TapMouseDown( object ); end,
      event_mup    = function () private.TapMouseUp( object, true ); end,
      event_menter = function () private.TapMouseEnter( object ); end,
      event_mleave = function () private.TapMouseLeave( object ); end
    } );

    private.tap.list[ object ].assigned = true;

  end;

end;
--********************************************************************************************************************
function public.TapGet ( object )

  return private.tap.list[ object ];

end;
--********************************************************************************************************************
function private.TapSetCurrent ( object )

  private.tap.object_current = object;

end;
--********************************************************************************************************************
function private.TapGetCurrent ()

  return private.tap.object_current;

end;
--********************************************************************************************************************
function private.TapIsCurrent ( object )

  return ( private.tap.object_current == object );

end;
--********************************************************************************************************************
function private.TapCallFuncFalse ( object )

  if ( private.tap.list[ object ].func_false ) then

    private.tap.list[ object ].func_false( object );

    private.tap.list[ object ].func_false = nil;

  end;

end;
--********************************************************************************************************************
function private.TapHoldDown ( object )

  if ( not private.TapIsCurrent( object ) ) then

    return;

  end;

  private.tap.was_holded = true;

  local func_hold_down = private.tap.list[ object ].func_hold_down;

  if ( func_hold_down ) then

    func_hold_down( object );

  end;

end;
--********************************************************************************************************************
function private.TapHoldUp ( object )

  if ( not private.tap.was_holded ) then

    return;

  end;

  private.tap.was_holded = false;

  local func_hold_up = private.tap.list[ object ].func_hold_up;

  if ( func_hold_up ) then

    func_hold_up( object );

  end;

end;
--********************************************************************************************************************
function private.TapMouseDown ( object )

  if ( private.TapGetCurrent() ) then

    return;

  end;

  private.TapTrace( "DOWN: "..object );

  if ( ObjGet( private.tap.TIMER_LOCK_OBJECT ).playing ) then

    return;

  end;

  private.TapSetCurrent( object );

  local func = function ()

    private.TapMouseUp( object );

    private.TapCallFuncFalse( object );

  end;

  ObjWaitForMouseUp( object, func );

  private.TapMouseDownAnim( object );

  ObjSet( private.tap.TIMER_HOLD_OBJECT,
  {
    time = private.tap.TIMER_HOLD_TIME,
    endtrig = function () private.TapHoldDown( object ); end,
    playing = true
  } );

end;
--********************************************************************************************************************
function private.TapMouseUp ( object, is_true )

  if ( not private.TapIsCurrent( object ) ) then

    return;

  end;

  private.TapSetCurrent();

  private.TapHoldUp( object );

  if ( is_true ) then

    private.TapTrace( "UP: ".. object );

    ObjSet( private.tap.TIMER_LOCK_OBJECT, { time = private.tap.TIMER_LOCK_TIME, endtrig = "", playing = true } );

    local func_true = public.TapGet( object ).func_true;

    if ( func_true ) then

      if ( "string" == type( func_true ) ) then

        common.CodeExecute( func_true );
        DbgTrace( string.format( "Warning! Using code string in TAP for [ %s ] ( code: '%s' )!", object, func_true ) );

      elseif ( "function" == type( func_true ) ) then

        func_true( object );

      end;

    else

      DbgTrace( string.format( "Warning! Trigger for < %s > doesn't exist!", object ) );

    end;

  end;

  private.TapMouseUpAnim( object );

end;
--********************************************************************************************************************
function private.TapMouseEnter ( object )

  if ( GetPlatform() == "ios" ) then

    return;

  end;
  
  SetCursor( CURSOR_HAND );

  private.TapTrace( "ENTER: ".. object );

  private.TapMouseEnterAnim( object );

  if ( private.tap.list[ object ].func_menter ) then

    private.tap.list[ object ].func_menter( object );

  end;
  
end;
--********************************************************************************************************************
function private.TapMouseLeave ( object )

  -- Снятие фокуса с объекта может быть после нажатия, и до отпускания.
  private.TapMouseUp( object );

  if ( GetPlatform() == "ios" ) then

    return;

  end;
  
  SetCursor( CURSOR_DEFAULT );

  private.TapTrace( "LEAVE: ".. object );

  private.TapMouseLeaveAnim( object );

  if ( private.tap.list[ object ].func_mleave ) then

    private.tap.list[ object ].func_mleave( object );

  end;

end;
--********************************************************************************************************************
--********************************************************************************************************************
function private.TapRegisterAnim ( anim_type, func_down, func_up, func_enter, func_leave )

  local func_empty = function () end;

  private.tap.anim[ anim_type ] =
  {
  MouseDown  = func_down  or func_empty,
  MouseUp    = func_up    or func_empty,
  MouseEnter = func_enter or func_empty,
  MouseLeave = func_leave or func_empty
  };

end;
--********************************************************************************************************************
function private.TapMouseDownAnim ( object )

  local anim = public.TapGet( object ).anim;

  if not ( anim ) then

    return;

  end;

  private.tap.anim[ anim.type ].MouseDown( object, anim.value );

end;
--********************************************************************************************************************
function private.TapMouseUpAnim ( object )

  local anim = public.TapGet( object ).anim;

  if not ( anim ) then

    return;

  end;

  private.tap.anim[ anim.type ].MouseUp( object, anim.value );

end;
--********************************************************************************************************************
function private.TapMouseEnterAnim ( object )

  local anim = public.TapGet( object ).anim;

  if not ( anim ) then

    return;

  end;

  private.tap.anim[ anim.type ].MouseEnter( object, anim.value );

end;
--********************************************************************************************************************
function private.TapMouseLeaveAnim ( object )

  local anim = public.TapGet( object ).anim;

  if not ( anim ) then

    return;

  end;

  private.tap.anim[ anim.type ].MouseLeave( object, anim.value );

end;
--********************************************************************************************************************
--********************************************************************************************************************
-- function ****** Tap:anims ****** () end
--********************************************************************************************************************
--********************************************************************************************************************
function private.TapInitAnims ()

  private.TapRegisterAnim( "focus_alpha",
    private.AnimFocusAlpShow,
    private.AnimFocusAlpHide,
    private.AnimFocusAlpShow,
    private.AnimFocusAlpHide
  );

  private.TapRegisterAnim( "object_scale",
    private.AnimObjectScaleDown,
    private.AnimObjectScaleUp,
    --[[TEMP]]private.AnimObjectScaleDown,--private.AnimObjectScaleEnter,
    --[[TEMP]]private.AnimObjectScaleUp--private.AnimObjectScaleLeave
  );

  private.TapRegisterAnim( "icon_scale",
    private.AnimIconScaleDown,
    private.AnimIconScaleUp,
    private.AnimIconScaleDown,
    private.AnimIconScaleLeave
  );
  
  private.TapRegisterAnim( "button",
    private.AnimButtonDown,
    private.AnimButtonLeave,
    private.AnimButtonDown,
    private.AnimButtonLeave
  );

  private.TapRegisterAnim( "button_color",
    private.AnimButtonColorDown,
    private.AnimButtonColorLeave,
    private.AnimButtonColorDown,
    private.AnimButtonColorLeave
  );

end;
--********************************************************************************************************************
function private.AnimFocusAlpShow ( sender, anim_value, trg )

  local sender_name = public.GetObjectName( sender );

  local obj = "fcs_"..sender_name;

  if ( ObjGet( obj ) ) then

    local alp = ObjGet( obj ).alp;
    local tme = 0.3 * ( 1 - alp );

    ObjAnimate( obj, "alp", 0, 0, trg,
    {
      0.0, 2, alp,
      tme, 2, 1.0
    } );

    common.PlayAudio( "all", "sfx", "assets/audio/sound/aud_focus_mark", 0 , 0 );

  end;

end;
-------------------------------------------------------------------------------------
function private.AnimFocusAlpHide ( sender, anim_value, trg )

  local sender_name = public.GetObjectName( sender );

  local obj = "fcs_"..sender_name;

  if ( ObjGet( obj ) ) then

    local alp = ObjGet( obj ).alp;
    local tme = 0.3 * alp;

    ObjAnimate( obj, "alp", 0, 0, trg,
    {
      0.0, 1, alp,
      tme, 1, 0.0
    } );

  end;

end;
-------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------
function private.AnimObjectScaleDown ( sender, anim_value, trg )

  local sender_name = cmn.GetObjectName( sender );

  local obj = "btn_"..sender_name;
  local param_obj = ObjGet( obj );

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.0, 2, param_obj.scale_x, param_obj.scale_y,
    0.3, 2, anim_value * 0.8, anim_value * 0.8
  } );

end;
-------------------------------------------------------------------------------------
function private.AnimObjectScaleUp ( sender, anim_value, trg )

  local sender_name = cmn.GetObjectName( sender );

  local obj = "btn_"..sender_name;

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.00, 3, anim_value * 0.80, anim_value * 0.80,
    0.30, 3, anim_value * 1.00, anim_value * 1.00
    --0.30, 3, anim_value * 1.20, anim_value * 1.20,
    --0.60, 3, anim_value * 0.90, anim_value * 0.90,
    --0.90, 3, anim_value * 1.00, anim_value * 1.00
  } );

end;
-------------------------------------------------------------------------------------
function private.AnimObjectScaleEnter ( sender, anim_value, trg )

  local sender_name = cmn.GetObjectName( sender );

  local obj = "btn_"..sender_name;

  local param_obj = ObjGet( obj );

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.0, 2, param_obj.scale_x, param_obj.scale_y,
    0.3, 2, anim_value * 1.15, anim_value * 1.15
  } );

end;
-------------------------------------------------------------------------------------
function private.AnimObjectScaleLeave ( sender, anim_value, trg )

  local sender_name = cmn.GetObjectName( sender );

  local obj = "btn_"..sender_name;

  local param_obj = ObjGet( obj );

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.0, 2, param_obj.scale_x, param_obj.scale_y,
    0.3, 2, anim_value * 1.0, anim_value * 1.0
  } );

end;
-------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------
function private.AnimIconScaleDown( sender, anim_value, trg )

  local sender_name = public.GetObjectName( sender );

  local obj = "spr_"..sender_name.."_icon";
  local param_obj = ObjGet( obj );

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.0, 2, param_obj.scale_x, param_obj.scale_y,
    0.3, 2, anim_value * 0.8, anim_value * 0.8
  } );

end;
-------------------------------------------------------------------------------------
function private.AnimIconScaleUp ( sender, anim_value, trg )

  local sender_name = public.GetObjectName( sender );

  local obj = "spr_"..sender_name.."_icon";

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.00, 3, anim_value * 0.80, anim_value * 0.80,
    0.30, 3, anim_value * 1.20, anim_value * 1.20,
    0.60, 3, anim_value * 0.90, anim_value * 0.90,
    0.90, 3, anim_value * 1.00, anim_value * 1.00
  } );

end;
-------------------------------------------------------------------------------------
function private.AnimIconScaleEnter ( sender, anim_value, trg )

  local sender_name = public.GetObjectName( sender );

  local obj = "btn_"..sender_name.."_icon";

  local param_obj = ObjGet( obj );

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.0, 2, param_obj.scale_x, param_obj.scale_y,
    0.3, 2, anim_value * 1.15, anim_value * 1.15
  } );

end;
-------------------------------------------------------------------------------------
function private.AnimIconScaleLeave( sender, anim_value, trg )

  local sender_name = public.GetObjectName( sender );

  local obj = "spr_"..sender_name.."_icon";

  local param_obj = ObjGet( obj );

  ObjAnimate( obj, "scale_xy", 0, 0, trg,
  {
    0.0, 2, param_obj.scale_x, param_obj.scale_y,
    0.3, 2, anim_value * 1.0, anim_value * 1.0
  } );

end;
--********************************************************************************************************************
--********************************************************************************************************************
function public.StdFlyAnimation( item_obj_name, task_obj_name, trigger_end, mult_factor, reduced_time )

  reduced_time = reduced_time or 0;

  SendMessage( Command_Transporter_MoveObject,
  {
    type        = Trans_SceneToTaskPanel,
    obj_name    = item_obj_name,
    toobj_name  = task_obj_name,
    end_trig    = trigger_end,
    mult_factor = mult_factor,
    red_time    = reduced_time
  } );

end;
--********************************************************************************************************************
--********************************************************************************************************************
function public.TimeForm ( totalsec, divider )

  local divider = divider or ":";

  local func_format_two_digit = function ( number )
    if ( number < 10 ) then return "0"..number; end;
    return number;
  end;

  local sec = totalsec % 60;
  local min = math.floor( totalsec / 60 );  

  sec = func_format_two_digit( sec );
  min = func_format_two_digit( min );

  return string.format( min..divider..sec );

end;
--------------------------------------------------------------------------------
function public.GetTimeFromSec ( seconds )

  seconds = math.floor( seconds + 0.5 );
  
  local day_sec    = 60 * 60 * 24;
  local hour_sec   = 60 * 60;
  local minute_sec = 60; 
  
  local d = math.floor( seconds / day_sec );
  seconds = seconds - ( d * day_sec );
  
  local h = math.floor( seconds / hour_sec );
  seconds = seconds - ( h * hour_sec );
  
  local m = math.floor( seconds / minute_sec );
  seconds = seconds - ( m * minute_sec );
  
  local s = seconds;
  
  local time_struct =
  {
    d = string.format( "%.2d", d ),
    h = string.format( "%.2d", h ),
    m = string.format( "%.2d", m ),
    s = string.format( "%.2d", s )
  };
  
  return time_struct;

end;  
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.LoadStrings ()
  
  ne.strings.Load( "assets/content/strings/strings_common" );
  
  --[[for s = 1, 5 do
    ne.strings.Load( "assets/content/section_"..s.."/strings_section_"..s );
  end;]]
  
end;
--------------------------------------------------------------------------------
function public.TableToString ( table_in, table_depth, symbol_lf, symbol_ct )

  if not ( table_in and "table" == type( table_in ) ) then

    return "";

  end;

  symbol_lf = symbol_lf or "\n";
  symbol_ct = symbol_ct or "\t";

  table_depth = table_depth or 0;

  local string_depth = "";

  for d = 1, table_depth do

    string_depth = string_depth..symbol_ct;

  end;
      
  local string_out = "";

  if ( 0 == table_depth ) then

    string_out = symbol_lf;

  end;

  for key, value in pairs( table_in ) do

    if ( "string" == type( value ) ) then

      value = "'"..value.."'";

    end;

    string_out = string_out..string_depth..tostring( key ).." = "..tostring( value )..symbol_lf;

    if ( "table" == type( value ) ) then

      string_out = string_out..public.TableToString( value, 1 + table_depth, symbol_lf, symbol_ct );

    end;

  end;

  return string_out;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Animation_Bounce ( object, func_end )

  local obj_params = ObjGet( object );

  local scale_x = obj_params.scale_x;
  local scale_y = obj_params.scale_y;

  ObjAnimate( object, "scale_xy", 0, 0, func_end,
  {
    0.00, 0, scale_x * 1.0, scale_y * 1.0,
    0.10, 0, scale_x * 1.5, scale_y * 1.0,
    0.20, 0, scale_x * 1.0, scale_y * 1.5,
    0.30, 0, scale_x * 0.8, scale_y * 0.8,
    0.35, 0, scale_x * 1.2, scale_y * 1.2,
    0.40, 0, scale_x * 1.0, scale_y * 1.0
  } );

end;
--------------------------------------------------------------------------------
function public.Animation_Fade_In ( time, object, reset, func_end )

  local alp = ObjGet( object ).alp;

  if ( alp > 0 ) and ( reset ) then

    ObjSet( object, { alp = 0.0 } );

  end;  

  ObjAnimate( object, "alp", 0, 0, func_end,
  {
    0.0,  0, alp,
    time, 0, 1.0
  } );

end;
--------------------------------------------------------------------------------
function public.Animation_Fade_Out ( time, object, func_end )

  local alp = ObjGet( object ).alp;

  ObjAnimate( object, "alp", 0, 0, func_end,
  {
    0.0,  0, alp,
    time, 0, 0.0
  } );

end;
--------------------------------------------------------------------------------
function public.Animation_Fake ( time, object, func_end )

  local alp = ObjGet( object ).alp;

  ObjAnimate( object, "alp", 0, 0, func_end,
  {
    0.0,  0, alp,
    time, 0, alp
  } );

end;
--------------------------------------------------------------------------------
function public.Animation_Pulse ( object, func_end, linear, time_factor, scale_factor )

  time_factor  = time_factor  or 1;
  scale_factor = scale_factor or 1;

  local base_scale = ObjGet( object ).scale_x;
  
  local upper_scale = base_scale + 0.05 * scale_factor * base_scale;
  
  if ( linear ) then

    local lower_scale = base_scale - 0.05 * scale_factor * base_scale;
  
    ObjAnimate( object, "scale_xy", 0, 0, func_end, 
    {
      0.00 * time_factor, 0, base_scale,  base_scale,
      0.07 * time_factor, 0, upper_scale, upper_scale,
      0.15 * time_factor, 0, base_scale,  base_scale,
      0.22 * time_factor, 0, lower_scale, lower_scale,
      0.30 * time_factor, 0, base_scale,  base_scale
    } );

  else

    local lower_scale = base_scale - 0.03 * scale_factor * base_scale;

    ObjAnimate( object, "scale_xy", 0, 0, func_end, 
    {
      0.00 * time_factor, 0, base_scale,  base_scale,
      0.15 * time_factor, 2, upper_scale, upper_scale,
      0.25 * time_factor, 2, lower_scale, lower_scale,
      0.28 * time_factor, 1, base_scale,  base_scale
    } );

  end;

end;
--------------------------------------------------------------------------------
function public.Animation_Spin ( object, func_end )

  local scale_x     = ObjGet( object ).scale_x;
  local inv_scale_x = -1 * scale_x;

  ObjAnimate( object, "scale_x", 0, 0, func_end, 
  {
    0.00, 0, scale_x,
    0.10, 0, inv_scale_x,
    0.20, 0, scale_x,
    0.30, 0, inv_scale_x,
    0.45, 0, scale_x,
    0.60, 0, inv_scale_x,
    0.85, 0, scale_x
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetItemResource ( reward_type, reward_name, small )

  local reward_content = m_content.GetContent( reward_type, reward_name );

  if     ( reward_type == "fragment" ) then

    local cut_name = reward_name:gsub( "_%d+$", "" );
    local res      = "assets/content/"..reward_content.."/"..cut_name;

    if ( small ) then

      res = res.."_small";

    end;

    return res;

  elseif ( reward_type == "stickerpack" ) and
         ( reward_name == "stickerpack" ) then

    return "assets/content/items/stickerpack";

  elseif ( reward_type == "gold" ) and
         ( reward_name == "gold" ) then

    return "assets/content/"..reward_content.."/gold_2";

  else

    return "assets/content/"..reward_content.."/"..reward_name;

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Timer_Create ( placeholder, timer_name, step, func_exec )
  
  local properties = {
    time    = step,
    playing = true,
    endtrig = func_exec
  };
  
  m_object.Create(
    timer_name,
    "timer",
    placeholder,
    properties
  );

end;
--------------------------------------------------------------------------------
function public.Timer_Delete ( timer_name )
  
  m_object.Delete( timer_name );
  
end;
--------------------------------------------------------------------------------
function public.Timer_Start ( timer_name, step, func_exec, multiset )

  local properties =
  {
    time    = step, 
    playing = true, 
    endtrig = func_exec
  };

  m_object.Set( timer_name, properties, multiset );

end;
--------------------------------------------------------------------------------
function public.Timer_Stop ( timer_name, multiset )

  local properties =
  {
    time    = 0,
    playing = false,
    endtrig = function () end
  };

  m_object.Set( timer_name, properties, multiset );

end;
--------------------------------------------------------------------------------
-- function ****** Callbacks ****** () end;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Run callback funcs sequence
--   callback_list : list, contain pointers to callback funcs, that accept ONE
--                   argument - pointer to callback func
-- Usage
--   common.RunCallbackList( { model.RequestData, view.Show, interface.Show } )
--------------------------------------------------------------------------------
function public.RunCallbackList ( callback_list )
  
  local callback_count = #callback_list;
  
  if     ( callback_count == 0 ) then
      
    public.Trace( "[ ERROR ] Call empty callback list." );
    return;
      
  elseif ( callback_count == 1 ) then
      
    local last_callback = table.remove( callback_list, 1 );
    last_callback();
      
  else
      
    local first_callback = table.remove( callback_list, 1 );
    
    local function run_other_callbacks ()
      public.RunCallbackList( callback_list );
    end;
    
    first_callback( run_other_callbacks );
      
  end;

end;

--------------------------------------------------------------------------------
-- Process array data without EventLoop locking
--   array           : processing data array
--   element_handler : array element handler, accept that as argument
--   func_after      : after processing callback
--   step            : time step
-- NOTE: async function, control will be passed to next instruction after call
--------------------------------------------------------------------------------
function public.ProcessArrayByTimer ( array, element_handler, func_after, step )
  
  if ( private.array_processing ) then
    
    DbgTrace( "[ ERROR ] common.ProcessArrayByTimer is busy now!" );
    return;
    
  end;
  
  local timer_name = "tmr_common_array_processor";
  local step       = step or 0.001;
  local total      = #array;
 
  local function func_iteration ( current )
  
    ObjSet( timer_name, { time = step, playing = true, endtrig = function ()
    
      ObjSet( timer_name, { playing = false } );
      
      local progress = common.Round( current / total, 2 ) ;
      element_handler( array[ current ], progress );

      local next_id = current + 1;

      if ( next_id <= total ) then
          
        func_iteration( next_id );
        
      else
        
        ObjDetach( timer_name );
        private.array_processing = false;
        
        if ( func_after ) then func_after(); end;

      end;
      
    end } );
    
  end;

  if ( total > 0 ) then

    ObjAttach( timer_name, "ng_application" );
    private.array_processing = true;
    func_iteration( 1 );

  else

    if ( func_after ) then func_after(); end;

  end;
  
end;

--------------------------------------------------------------------------------
-- Defer callback call to specified time
--   action_name : name of defer action, must be unique
--   defer_time  : defer time, seconds
--   callback    : deferred function callback
-- NOTE: async function, control will be passed to next instruction after call
--------------------------------------------------------------------------------
function public.Defer ( action_name, defer_time, callback )
  
  local timer_name = "tmr_common_defer_" .. action_name;
  
  ObjCreate( timer_name, "timer" );
  ObjAttach( timer_name, "ng_application" );
  
  local function func_deferred ()
  
    callback();
    ObjDelete( timer_name );
  
  end;
  
  ObjSet( timer_name,
  { 
    time    = defer_time,
    playing = true,
    endtrig = func_deferred
  } );
  
end;

--------------------------------------------------------------------------------
-- function ****** Math ****** () end;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Return value rounded to nearest number
--  value      : rounding numeric value
--  digit_count: count of decimal digits after comma (optional, default = 0)
--------------------------------------------------------------------------------
function public.Round( value, digit_count )
  
  digit_count = digit_count or 0;
  
  local precision = 10^digit_count;
  
  return math.floor( ( value * precision ) + 0.5 ) / precision;
  
end;

--------------------------------------------------------------------------------
-- function ****** Common ****** () end;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Return array of substrings splitted by separator
--------------------------------------------------------------------------------
function public.String_Split ( string, separator )
  
  local split = {};
  local start = 0;

  if ( #string == 1 ) then return { string }; end;
   
  while true do
    
    local pos = string.find( string, separator, start, true );
    
    if pos ~= nil then
    
      table.insert( split, string.sub( string, start, pos - 1 ) ); 
      
      start = pos + 1;
      
    else
      
      table.insert( split, string.sub( string, start ) );
      
      break;
      
    end;
    
  end
  
  return split;
  
end;

--------------------------------------------------------------------------------
-- Returns deep copy of data (recursive function)
--------------------------------------------------------------------------------
function public.DeepCopy ( data )

  if ( type( data ) ~= "table" ) then

    return data;

  end;

  local copy   = {};
  local backup = ( getmetatable( data ) or {} ).__watch_backup;

  for key, value in pairs( backup or data ) do
        
    copy[ key ] = public.DeepCopy( value );
      
  end;

  return copy;

end;

--------------------------------------------------------------------------------
-- function ****** Debug ****** () end;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Trace function and parent function info where DbgCallpoint called 
--------------------------------------------------------------------------------
function public.DbgCallpoint ( message, stack_shift )
  
  if ( debug ) then
  
    stack_shift = stack_shift or 0;
    
    local info    = debug.getinfo( 2 + stack_shift, "Sln" ) or {};
    local info_up = debug.getinfo( 3 + stack_shift, "Sln" ) or {};
    
    local trace = string.format(
      "%s:%s:%s -> %s:%s:%s",
      info_up.source, info_up.currentline, info_up.name,
      info.source,    info.currentline,    info.name
    );

    if ( message ) then
      trace = trace .. string.format( ": %s", message );
    end;
    
    DbgTrace( trace );
    
  else
    
    DbgTrace( "[ WARNING ] common.DbgCallpoint: debug module not loaded" );
    
    if ( message ) then
      DbgTrace( "common.DbgCallpoint: " .. tostring( message )  );
    end;
    
  end;
  
end;

--------------------------------------------------------------------------------
-- Return watching copy of table
--------------------------------------------------------------------------------
function public.DbgWatchTable( tab, name, field, writeonly )

  name = name or "table";

  if ( type(tab) == "table" ) then
  
    -- Backup table
    local backup = {};
    for k, v in pairs( tab ) do
      backup[ k ] = v;
      tab[ k ] = nil;
    end;
    
    local mt = {};
    mt.__watch_backup = backup;
    mt.__index = function ( t, k )
    
      local value = backup[ k ];
    
      if ( field == nil or field == k and not writeonly ) then
      
        local msg = ("INFO: READ %s.%s = %s"):format(name, k, value);
        public.DbgCallpoint( msg, 1 );
      
      end;
    
      return value;
    
    end;
    
    mt.__newindex = function ( t, k, v )
    
      if ( field == nil or field == k ) then
      
        local msg = ("WARNING: WRITE %s.%s = %s"):format(name, k, v);
        public.DbgCallpoint( msg, 1 );
      
      end;
      
      backup[ k ] = v;
    
    end;
    
    mt.__len = function ()
    
      return #backup;
    
    end;
    
    setmetatable( tab, mt );

  end;

end;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------