-- name=int_control
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function public.Init ()

  private.list = {};

  ObjAttach( "int_control", InterfaceName );

end;
--------------------------------------------------------------------------------
function private.Add ( control, parent, pos_z )

  local hub_obj = string.format( "hub_%s", control );

  ObjCreate( hub_obj, "obj" );

  table.insert( private.list, control );
  
  if ( pos_z ) then

    ObjSet( hub_obj, { pos_z = pos_z } );

  else

    local msg = "[ CONTROLS ][ ERROR ] Unknown control '%s', pos_z = %s";
    DbgTrace( msg:format( control, #private.list ) );
    
    ObjSet( hub_obj, { pos_z = #private.list } );

  end;
  
  ObjAttach( hub_obj, parent );

end;
--------------------------------------------------------------------------------
function public.Add ( control )

  local pos_z =
  {
    int_dialog_info           = 320, -- system messages
    int_dialog_notifications  = 310, -- enable push notifications window
    int_system_message        = 300, -- OSD effects layer
    
    tutorial                  = 290,
    int_subtitles             = 280,
    interface_lock            = 270,
    int_popup_cloud           = 260,
    
    int_panel_map_top         = 250,

    int_panel_social_score    = 240,
    int_window                = 230,
    
    int_panel_questlist       = 220,
    int_panel_darkhole        = 210, -- highlight quests
    int_panel_map_left        = 210,
    int_panel_map_right       = 210,
    
    int_panel_map_bottom      = 200,
    
    int_panel_stage_top       = 180,
    int_panel_stage_time      = 180, -- obsolete
    
    int_panel_stage_win       = 170, -- You Win!
    
    int_panel_stage_settings  = 165,
    
    int_dialog_stage_friends  = 160,
    int_dialog_stage_shuffle  = 160,
    int_popup_stage_combo     = 150,
    int_popup_stage_bonus     = 140,
    int_panel_booster_sorcery = 130, -- booster variant (v/h)
    int_popup_stage_score     = 120,
    int_panel_stage_boosters  = 120,
    int_panel_stage_resources = 120,
    int_panel_stage_score     = 120
  };
  
  private.Add( control, "int_control", pos_z[ control ] );
  
end;
--------------------------------------------------------------------------------
function public.AddToStage ( control )

  local pos_z =
  {
   int_panel_stage_progress_fight = 1,
   int_panel_stage_progress_ride  = 1,
  };
  
  private.Add( control, "scr_stage", pos_z[ control ] );
  
end;
--------------------------------------------------------------------------------
function public.SetPosZ ( control, pos_z )

  local hub_obj       = string.format( "hub_%s", control );
  local current_pos_z = ObjGet( hub_obj ).pos_z;
  
  if ( current_pos_z == pos_z ) then

    return;

  end;

  ObjSet( hub_obj, { pos_z = pos_z } );

end;
--------------------------------------------------------------------------------
function public.Remove ( control )

  local hub_obj = string.format( "hub_%s", control );

  ObjDetach( hub_obj );

  ObjDelete( hub_obj );

  _G[ control ] = nil;

  for control_idx = 1, #private.list do

    if ( control == private.list[ control_idx ] ) then

      table.remove( private.list, control_idx );

      break;

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.Attach ( control )

  local hub_obj = string.format( "hub_%s", control );

  ObjAttach( control, hub_obj );

end;
--------------------------------------------------------------------------------
function public.Detach ( control )

  ObjDetach( control );

end;
--------------------------------------------------------------------------------
function public.SetInput ( control_input )

  ObjSet( "int_control", { input = control_input } );

end;
--------------------------------------------------------------------------------
function public.SetVisible ( control_visible )

  ObjSet( "int_control", { visible = control_visible } );

end;
--------------------------------------------------------------------------------
function public.ControlSetInput ( control, control_input )

  local hub_obj = string.format( "hub_%s", control );

  ObjSet( hub_obj, { input = control_input } );

end;
--------------------------------------------------------------------------------
function public.ControlSetVisible ( control, control_visible )

  local hub_obj = string.format( "hub_%s", control );

  ObjSet( hub_obj, { visible = control_visible } );

end;
--------------------------------------------------------------------------------
function public.SetControlsActive ( state )

  for control_idx = 1, #private.list do

    local control = private.list[ control_idx ];

    ObjSet( "hub_"..control, { active = state, visible = state, input = state } );

  end;

end;
--------------------------------------------------------------------------------