-- name=int_dialog_dailyquest
--------------------------------------------------------------------------------
function public.__InitModule ()
  
  private.cooldown        = {};
  private.challenge_count = 4;
  
  private.window_area = {
    portrait  = { -320, -420, 640, 900 },
    landscape = { -250, -320, 500, 700 },
    none      = { 0,    0,    0,   0   }
  };
  
  private.int_o = int.Orientation();

  private.strings =
  {
    caption     = ne.strings.Get( "str_int_dialog_dailyquest_caption" ),
    description = {
      challenge  = ne.strings.Get( "str_int_dialog_dailyquest_challenge_description" ),
      dailyquest = ne.strings.Get( "str_int_dialog_dailyquest_dailyquest_description" ),
      countdown  = ne.strings.Get( "str_int_dialog_dailyquest_countdown_description" )
    },
    skip        = ne.strings.Get( "str_int_dialog_dailyquest_skip" ),
    hint        = ne.strings.Get( "str_int_dialog_dailyquest_hint" ),
    woodchest   = ne.strings.Get( "str_dialog_items_woodchest" ),
    silverchest = ne.strings.Get( "str_dialog_items_silverchest" ),
    goldchest   = ne.strings.Get( "str_dialog_items_goldchest" ),
    epicchest   = ne.strings.Get( "str_dialog_items_epicchest" )
  };
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.Adapt();
  private.CreateButtons();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( 
    states, 
    dailyquest, 
    func
  )

  private.challenge_states = states;
  private.reward_id        = dailyquest.reward_id;
  private.is_ready         = dailyquest.ready;

  if ( private.active ) then

    return;

  end;

  private.active = true;

  private.Reward_Tap       = func.Reward_Tap;
  private.Reward_Notify    = func.Reward_Notify;
  private.Cooldown_GetTime = func.Cooldown_GetTime;
  private.Challenge_Tap    = func.Challenge_Tap;
  private.Close            = func.DialogHide;
  private.Cooldown_Skip     = func.Cooldown_Skip;

  private.SetDefault( states, dailyquest );
  
  int.Dialog_Open( "dailyquest" );
    
  cmn.PlaySnd( "daily_quest" );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_OnHide )
  
  if ( private.active ) then
  
    private.active = false;
    
    int.Dialog_Close( "dailyquest", function ()

      if ( func_OnHide ) then
        
        func_OnHide();

      end;

    end );

  end;

end;
--------------------------------------------------------------------------------
function public.IsActive ()
  
  return private.active;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Challenge_Set (
    id,
    new_dailyquest,
    old_dailyquest
  )

  private.Challenge_Set( 
    id, 
    new_dailyquest[ "challenge"..id ], 
    old_dailyquest[ "challenge"..id ].state,
    function ()

      private.Timer_TryShow( "challenge" );

    end
  );

end;
--------------------------------------------------------------------------------
function public.Challenge_Update ( id, multiset )
 
  local state = ( not private.Reward_IsPending() );

  private.Challenge_Update( id, state, true, multiset );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Reward_Set ( reward_id, reward )

  private.reward_id = reward_id;

  private.Reward_Set( reward );
  
end;
--------------------------------------------------------------------------------
function public.Reward_Enable ()

  private.Reward_Enable();
  
end;
--------------------------------------------------------------------------------
function public.Reward_Disable ( hide_reward )

  private.Reward_Disable( hide_reward );
  
end;
--------------------------------------------------------------------------------
function public.Reward_ShowPopup ()

  private.Reward_ShowPopup();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Timer_TryShow ( timer_type, skip_price )

  private.Timer_TryShow( timer_type, skip_price );

end;
--------------------------------------------------------------------------------
function public.Timer_Hide ()

  private.Timer_Hide();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  local multiset = {};

  int.Center_Object( "obj_int_dialog_dailyquest", nil, nil, multiset );  

  for c = 1, private.challenge_count do

    m_object.Set(
      "anm_int_dialog_dailyquest_challenge_"..c,
      {
        inputrect_init = true,
        inputrect_x    = -100,
        inputrect_y    = -100,
        inputrect_w    = 200,
        inputrect_h    = 200
      },
      multiset
    );

    common.TapSet( "anm_int_dialog_dailyquest_challenge_"..c,
    {    
      func_true = function ()

        private.Challenge_Tap( c );

        cmn.PlaySnd( "simply_click" );

      end
    } );

  end;

  m_object.Set(
    "anm_int_dialog_dailyquest_window",
    {
      animfunc = private.int_o,
      title    = private.strings.caption
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  int.Button_Create(
    "btn_int_dialog_dailyquest_close",
    {
      design = "close",
      sound  = "simply_click",
    },
    function()
      
      private.Close();
      
    end
  );
  
  int.Button_Create(
    "btn_int_dialog_dailyquest_skip",
    {
      design = "price",
      sound  = "buy"
    },
    function()
      
      private.Cooldown_Skip( "dailyquest" );
      
    end
  );  

  common.TapSet( "obj_int_dialog_dailyquest_cloud",
  {    
    func_true = function ()

      private.Reward_Tap();

    end
  } );

end;
--------------------------------------------------------------------------------
function private.SetDefault ( states, dailyquest )

  local multiset = {};

  private.Cooldown_Update( "challenge" );
  private.Cooldown_Update( "countdown" );

  for id = 1, private.challenge_count do

    local challenge = dailyquest[ "challenge"..id ];
    
    private.Challenge_Set( id, challenge );
    
  end;

  private.Reward_Set( dailyquest.reward );

  if ( not private.cooldown.challenge ) then

    if ( private.Reward_IsPending () ) then

      private.Reward_Enable();

    else

      private.Reward_Disable();

    end;

  end;

  local timer_type = "challenge";
  local skipprice;

  if ( private.cooldown.countdown ) then

    timer_type = "countdown";
    
  elseif ( not dailyquest.ready           ) and
         ( not private.cooldown.challenge ) then

    timer_type = "dailyquest";
    skipprice = dailyquest.skipprice;

  end;

  private.Timer_TryShow( timer_type, skipprice );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Timer_TryShow ( timer_type, skip_price )
  
  local multiset = {};
    
  if ( private.Timer_TryStart( timer_type ) ) then

    local animfunc_appear = "appear";
    local animfunc_static = "static";
    
    int.Button_Disable( "btn_int_dialog_dailyquest_skip" );

    if ( timer_type == "dailyquest" ) then

      animfunc_appear = "appear_skip";
      animfunc_static = "static_skip";

      int.Button_Enable( "btn_int_dialog_dailyquest_skip" );
      int.Button_SetText( "btn_int_dialog_dailyquest_skip", skip_price );
      
    end;

    int.Input_Off( "obj_int_dialog_dailyquest_cloud", multiset );
    
    m_object.Set(
      "anm_int_dialog_dailyquest_cooldown",
      { 
        active      = true,
        input       = true, 
        visible     = true, 
        animfunc    = animfunc_appear,
        hint        = private.strings.hint,
        tt_skip     = private.strings.skip,
        description = private.strings.description[ timer_type ],
        chest_icon  = private.reward_res,
        playing     = true,
        frame       = 0,
        endtrig     = function ()

          AnimPlay(
            "anm_int_dialog_dailyquest_cooldown",
            animfunc_static,
            ""
          );

        end
      },
      multiset
    );
    
    int_window.SetArea(
      "obj_int_dialog_dailyquest",
      private.window_area.none,
      multiset
    );

    m_object.Multiset( multiset );

    return true;
    
  end;

  int.Input_On( "obj_int_dialog_dailyquest_cloud", multiset );

  m_object.Disable(
    "anm_int_dialog_dailyquest_cooldown",
    false,
    multiset
  );
    
  int_window.SetArea(
    "obj_int_dialog_dailyquest",
    private.window_area[ private.int_o ],
    multiset
  );

  m_object.Multiset( multiset );

  return false;

end;
--------------------------------------------------------------------------------
function private.Timer_TryStart ( timer_type )
  
  private.Cooldown_Update( timer_type );
  
  if ( private.Cooldown_IsActive( timer_type ) ) and
     ( not private.Reward_IsPending() ) then  

    private.active_timer = timer_type;
    private.Timer_Update( timer_type );

    return true;
    
  end;

  private.active_timer = nil;
  common.Timer_Stop( "tmr_int_dialog_dailyquest_timer" );

  return false;
  
end;
--------------------------------------------------------------------------------
function private.Timer_Update ( timer_type )

  private.Cooldown_Update( timer_type );
  
  if ( private.Cooldown_IsActive( timer_type ) ) then

    local string_time;
    local multiset = {};

    local time = common.GetTimeFromSec( private.cooldown[ timer_type ] );

    if ( private.cooldown[ timer_type ] > 3600 ) then

      string_time = int.Text_TimerFormat( time, "h:m" );

    else

      string_time = int.Text_TimerFormat( time, "m:s" );

    end;

    common.Timer_Start(
      "tmr_int_dialog_dailyquest_timer",
      1,
      function ()

        private.Timer_Update( timer_type );

      end,
      multiset
    );

    if ( private.active_timer ) then

      m_object.Set(
        "anm_int_dialog_dailyquest_cooldown",
        { time = string_time },
        multiset
      );
      
    end;
      
    m_object.Multiset( multiset );

  else

    private.Timer_Hide();

  end;

end;
--------------------------------------------------------------------------------
function private.Timer_Hide ()
  
  int_window.SetArea(
    "obj_int_dialog_dailyquest",
    private.window_area[ private.int_o ]
  );

  local multiset = {};

  common.Timer_Stop( "tmr_int_dialog_dailyquest_timer", multiset );

  if ( private.active_timer ) then

    local animfunc = "disappear";

    if ( private.active_timer == "dailyquest" ) then

      animfunc = "disappear_skip";
      
      int.Button_Disable( "btn_int_dialog_dailyquest_skip" );

    end;

    int.Input_On( "obj_int_dialog_dailyquest_cloud", multiset );
  
    m_object.Set(
      "anm_int_dialog_dailyquest_cooldown",
      { 
        animfunc = animfunc,
        playing  = true,
        frame    = 0,
        endtrig  = function ()

          m_object.Disable( "anm_int_dialog_dailyquest_cooldown" );

        end
      },
      multiset
     );
    
  end;

  for number = 1, private.challenge_count do

    private.Challenge_Update(
      number,
      ( private.reward_id == false ),
      true,
      multiset
    );

  end;

  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Challenge_Set (
    id,
    challenge,
    state_old,
    func_OnAnimEnd
  )
  
  local anm_slot     = "anm_int_dialog_dailyquest_challenge_"..id;
  local new_animfunc = private.GetAnimfuncByState( challenge.state );
  local old_animfunc = private.GetAnimfuncByState( state_old );
  local anm_playing  = true;

  local func_set_state = function ( anm_slot, animfunc, playing )   

    AnimStop( anm_slot );

    local anm_params = 
    {
      animfunc = animfunc,
      playing  = playing,
      frame    = 0,
      endtrig  = function() end
    };

    if ( func_OnAnimEnd ) then
        
      func_OnAnimEnd();
        
    end;

    m_object.Set( anm_slot, anm_params );

  end;

  if ( challenge.state == private.challenge_states[ 1 ] ) then

    anm_playing = false;

    private.Challenge_ShowEnemy(
      id,
      challenge.enemy.name,
      challenge.enemy.content
    );

  else 

    private.Challenge_HideEnemy( id );

  end;

  private.Challenge_Update(
    id,
    ( not private.Reward_IsPending() )
  );
  
  if ( old_animfunc ) then
    
    if ( new_animfunc == "state_3" ) then

      cmn.PlaySnd( "daily_quest_new" );

    end;
    
    AnimPlay(
      anm_slot,
      old_animfunc,
      function ()

        func_set_state( anm_slot, new_animfunc, anm_playing );

      end
    );

  else

    func_set_state( anm_slot, new_animfunc, anm_playing );

  end;

end;
--------------------------------------------------------------------------------
function private.Challenge_Update ( id, state, animated, multiset )
  
  local color = 1.0;

  if ( private.Cooldown_IsActive( "challenge" ) ) or
     ( private.Cooldown_IsActive( "countdown" ) ) or
     ( not private.is_ready                     ) then

    state = false;

  end;

  if ( not state ) then

    color = 0.5;

  end;
  
  local obj_name = "anm_int_dialog_dailyquest_challenge_"..id;

  if ( animated ) then

    local start = m_object.Get( obj_name );

    if ( start.color_r ~= color ) then

      m_object.Set( obj_name, { input = state }, multiset );

      ObjAnimate( obj_name, "color_rgb", 0, 0, "", 
      {
        0.0, 0, start.color_r, start.color_g, start.color_b,
        0.3, 0, color,         color,         color,
      } );

    end;

  else

    m_object.Set(
      obj_name,
      { 
        input   = state,
        color_r = color,
        color_g = color,
        color_b = color
      },
      multiset
    );

  end;

end;
--------------------------------------------------------------------------------
function private.Challenge_ShowEnemy ( num, enemy_name, enemy_content )

  --FF Костыль для гигантских монстров
  local scale = 1.0;

  if ( enemy_name == "octocrab" ) then

    scale = 0.65;

  elseif ( enemy_name == "bonedragon" ) then

    scale = 0.8;

  end;

  m_object.Set(
    "anm_int_dialog_dailyquest_challenge_"..num.."_enemy",
    {
      active   = true,
      visible  = true,
      res      = "assets/content/"..enemy_content.."/"..enemy_name.."/anm_map_"..enemy_name,
      animfunc = "static",
      playing  = true,
      scale_x  = scale,
      scale_y  = scale
    }
  );

end;
--------------------------------------------------------------------------------
function private.Challenge_HideEnemy ( num, enemy_name, enemy_content )

  m_object.Set(
    "anm_int_dialog_dailyquest_challenge_"..num.."_enemy",
    {
      active  = false,
      visible = false,
      playing = false
    }
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Reward_IsPending ()

  return ( type( private.reward_id ) == "number" );
  
end;
--------------------------------------------------------------------------------
function private.Reward_Enable ()

  m_object.Enable( "spr_int_dialog_dailyquest_challenge_reward", true );

  AnimPlay(
    "anm_int_dialog_dailyquest_cloud",
    "cloud_reward",
    ""
  );
  
end;
--------------------------------------------------------------------------------
function private.Reward_Disable ( hide_reward )

  AnimPlay(
    "anm_int_dialog_dailyquest_cloud",
    "cloud",
    ""
  );

  if ( hide_reward ) then

    m_object.Disable( "spr_int_dialog_dailyquest_challenge_reward", true );

  else

    m_object.Enable( "spr_int_dialog_dailyquest_challenge_reward", true );

  end;
  
end;
--------------------------------------------------------------------------------
function private.Reward_ShowPopup ()
  
  int.ShowPopupCloud( 
    "monolog", 
    "anm_int_dialog_dailyquest_cloud", 
    "str_int_dialog_dailyquest_reward_description",
    nil,
    { -20, -50 }
  );
  
end
--------------------------------------------------------------------------------
function private.Reward_Set ( reward )

  local multiset = {};

  if ( reward ) then

    m_object.Set(
      "anm_int_dialog_dailyquest_cloud",
      {
        chest = private.strings[ reward.name ]
      },
      multiset
    );

  else

    m_object.Set(
      "anm_int_dialog_dailyquest_cloud",
      {
        chest = ""
      },
      multiset
    );

  end;

  if ( not reward ) then

    m_object.Disable(
      "spr_int_dialog_dailyquest_challenge_reward",
      false,
      multiset
    );

    m_object.Multiset( multiset );

    return;

  end;

  local scale = 1.0;

  if ( reward.type == "booster" ) then

    scale = 0.7;  

  end;

  private.reward_res = common.GetItemResource( reward.type, reward.name );

  m_object.Set(
    "spr_int_dialog_dailyquest_challenge_reward",
    {
      res     = private.reward_res,
      scale_x = scale,
      scale_y = scale,
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.GetAnimfuncByState ( state )

  if ( state == private.challenge_states[ 1 ] ) then

    return "state_1";

  elseif ( state == private.challenge_states[ 2 ] ) then

    return "state_2";

  elseif ( state == private.challenge_states[ 3 ] ) then

    return "state_3";

  end;

  return nil;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Cooldown_IsActive ( timer_type )
  
  private.Cooldown_Update( timer_type );
  
  return ( private.cooldown[ timer_type ]     ) and
         ( private.cooldown[ timer_type ] > 0 );
  
end;
--------------------------------------------------------------------------------
function private.Cooldown_Update ( timer_type )

  private.cooldown[ timer_type ] = private.Cooldown_GetTime( timer_type );

end;
--------------------------------------------------------------------------------