-- name=int_dialog_fireflies
--------------------------------------------------------------------------------
function public.__InitModule ()
  
  private.app_w       = int.W();
  private.app_h       = int.H();
  private.orientation = int.Orientation();

  private.notification_count =
  {
    album       = 0,
    stickerpack = 0,
    duplicates  = 0
  };

  private.current_tab = 1;
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.this   = "obj_int_dialog_fireflies";
  private.window = "anm_int_dialog_fireflies_window";

  private.reward_label = ne.strings.Get( "str_dialog_fireflies_reward_label" );

  private.window_area = {
    portrait  = { -315, -444, 630, 924 },
    landscape = { -243, -345, 486, 710 }
  };
  
  int_window.SetArea( private.this, private.window_area[ private.orientation ] );

  private.is_pc = IsPC();

  private.Adapt();
  private.CreateButtons();
  private.CreateTabs();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Show ( 
    tab_number,
    func_page_data,
    func_album_data,
    func_stickerpack_data,
    func_duplicates_data,
    func_stickerpack_get_count,
    func_try_stick,
    func_stats,
    func_close,
    func_top_panel_show,
    func_top_panel_hide,
    func_reward_show,
    func_reward_pending,
    func_reward_params
  )

  if ( private.active ) then

    return;

  end;

  private.active = true;

  private.func_Page_Data        = func_page_data;
  private.func_Album_Data       = func_album_data;
  private.func_Stickerpack_Data = func_stickerpack_data;
  private.func_Duplicates_Data  = func_duplicates_data;

  private.func_StickerpackGetCount = func_stickerpack_get_count;
  private.func_TryStick            = func_try_stick;
  private.func_Stats               = func_stats;

  private.func_Close         = func_close;
  private.func_TopPanel_Show = func_top_panel_show;
  private.func_TopPanel_Hide = func_top_panel_hide;

  private.Reward_Show      = func_reward_show;
  private.Reward_IsPending = func_reward_pending;
  private.Reward_Params    = func_reward_params;

  private.stickerpack_count = private.func_StickerpackGetCount();

  if ( private.stickerpack_count > 0 ) then

    tab_number = 2;

  end;

  private.Tab_Show( tab_number );

  int.Dialog_Open(
    "fireflies",
    function()

      mediator.Notify( "Interface_DialogFireflies_Opened" );

    end
  );
  
end;
--------------------------------------------------------------------------------
function  public.Hide ( func_end )

  if ( not private.active ) then
    
    return;
    
  end;
    
  private.active = false;

  private.func_TopPanel_Hide();

  int.Dialog_Close( "fireflies", function ()

    private.Cleanup();
    
    if ( func_end ) then

      func_end();

    end;
    
  end );
  
end;
--------------------------------------------------------------------------------
function public.Tab_Show ( tab_index, animate, func_end )

  private.Tab_Show( tab_index, animate, func_end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Tutorial_SetButtonsInput ( value )
  
  ObjMultiSet( 
    {
      { "anm_int_dialog_fireflies_album_tabs",  { input = value } },
      { "btn_int_dialog_fireflies_album_prev",  { input = value } },
      { "btn_int_dialog_fireflies_album_next",  { input = value } },
      { "btn_int_dialog_fireflies_close",       { input = value } }
    }
  );

  if ( not private.is_pc ) then

    ObjSet(
      "obj_int_dialog_fireflies_swipe",
      {
        input = value
      }
    );

  end;
  
end;
--------------------------------------------------------------------------------
function public.Tutorial_GoToSticker ( sticker )
  
  local page_number = private.Album_FindPage( sticker.firefly );

  private.Album_ShowPage( page_number );
  
end;
--------------------------------------------------------------------------------
function public.Tutorial_StickerPosition ( sticker_index )
  
  return private.Album_StickerPosition( sticker_index );
  
end;
--------------------------------------------------------------------------------
function public.Tutorial_GetStickerPoolPos ( sticker_name )
  
  return int.Position( 
    "anm_int_dialog_fireflies_album_stickerpack_stickerpool_"..sticker_name,
    InterfaceName
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  int.Center_Object( private.this );

  if ( wiz.IsCheatsEnabled() ) then
    
    ObjSet( "btn_int_dialog_fireflies_album_cheat", { 
      active  = true,
      visible = true,
      input   = true
    } );
    
  end;

  -- JK: duplicates scroller settings
  private.duplicates_scc_x         = 5;
  private.duplicates_scc_min_pages = 5;
  private.duplicates_scc_cell_size = 290;
  private.duplicates_per_page      = 3;
  private.duplicates_item_scale    = 1;
  private.duplicates_item_shift_x  = 200;
  
  for p = 1, private.duplicates_per_page do
   
    ObjCreate(
      "obj_int_dialog_fireflies_duplicates_page_slot"..p,
      "obj"
    );

    ObjAttach(
      "obj_int_dialog_fireflies_duplicates_page_slot"..p,
      "obj_int_dialog_fireflies_duplicates_page"
    );
    
  end;

  private.Tab_HideNotification( "album" );
  private.Tab_HideNotification( "stickerpack" );
  private.Tab_HideNotification( "duplicates" );

  ObjMultiSet( {
    {
      "anm_int_dialog_fireflies_album_tabs",
      {
        animfunc = "tabs_"..private.orientation
      }
    }
  } );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  common.TapSet( "btn_int_dialog_fireflies_album_cheat",
  {
    anim = { type = "object_scale", value = 1.0 },
    func_true = function ()

      m_fireflies.Cheat_GetAllStickers();
      private.Tab_SetupAlbum();
      
    end
  } );
  
  if ( private.is_pc ) then

    m_object.Disable( "obj_int_dialog_fireflies_swipe" );

  else

    int.Swipe_Create(
      "obj_int_dialog_fireflies_swipe",
      {
        left  = function() private.Album_ShowNextPage(); end,
        right = function() private.Album_ShowPrevPage(); end
      }
    );

  end;

  int.Button_Create(
    "btn_int_dialog_fireflies_close",
    {
      design = "close"
    },
    function ()

      private.func_Close();
      
    end
  );

  int.Button_Create(
    "btn_int_dialog_fireflies_album_prev",
    {
      design = "arrow_left"
    },
    function ()
      
      cmn.PlaySnd( "simply_click" );
      
      private.Album_ShowPrevPage();
      
    end
  );

  int.Button_Create(
    "btn_int_dialog_fireflies_album_next",
    {
      design = "arrow_right"
    },
    function ()
      
      cmn.PlaySnd( "simply_click" );
      
      private.Album_ShowNextPage();
      
    end
  );
  
  -- JK: stickerpack buttons
  int.Button_Create(
    "btn_int_dialog_fireflies_stickerpack_buy",
    {
      title  = "str_dialog_fireflies_booster_buy",
      color  = "green",
      design = "medium",
      price  = true
    },
    function()
      
      private.Stickerpack_TryBuy();
      
    end
  ); 

  int.Button_Create(
    "btn_int_dialog_fireflies_stickerpack_get",
    {
      title  = "str_dialog_fireflies_booster_get",
      color  = "green",
      design = "medium"
    },
    function()

      private.Stickerpack_Claim();

    end
  );

  -- JK: duplicates buttons
  int.Button_Create(
    "btn_int_dialog_fireflies_duplicates_sell_all",
    {
      title  = "str_dialog_fireflies_stickers_sellall",
      design = "medium",
      color  = "blue"
    },
    function()
      
      private.Duplicates_TrySellAll();

    end
  );
  
end;
--------------------------------------------------------------------------------
function private.CreateTabs ()

  ObjSet( private.window, {
    tab_album     = "str_dialog_fireflies_album_tab_album",
    tab_stickers  = "str_dialog_fireflies_album_tab_stickerpack",
    tab_duplicate = "str_dialog_fireflies_album_tab_duplicates",
    tab_info      = "str_dialog_fireflies_album_tab_info"
  } );

  local function func_sound ()
    
    cmn.PlaySnd( "tab_switch" );
    
  end;
  
  int.Control_EventsSet( "tab_int_dialog_fireflies_album",
  {
    func_on_click = function ()

      if ( private.current_tab == 1 ) then

        return;

      end;

      func_sound();
      private.Tab_Show( 1 );

    end
  } );

  int.Control_EventsSet( "tab_int_dialog_fireflies_stickerpack",
  {
    func_on_click = function ()

      if ( private.current_tab == 2 ) then

        return;

      end;

      func_sound();
      private.Tab_Show( 2 );

    end
  } );
  
  int.Control_EventsSet( "tab_int_dialog_fireflies_duplicates",
  {
    func_on_click = function ()

      if ( private.current_tab == 3 ) then

        return;

      end;

      func_sound();
      private.Tab_Show( 3 );

    end
  } );

  int.Control_EventsSet( "tab_int_dialog_fireflies_info",
  {
    func_on_click = function ()

      if ( private.current_tab == 4 ) then

        return;

      end;

      func_sound();
      private.Tab_Show( 4 );

    end
  } );

end;
--------------------------------------------------------------------------------
function private.Cleanup ()

  -- JK: clear data
  private.album_data       = nil;
  private.stickerpack_data = nil;
  private.duplicates_data  = nil;

  private.Duplicates_DestroyScroller();
  private.Info_DestroyScroller();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Tab_Show ( tab_index, animate, func_end )

  private.current_tab = tab_index;

  local tabs =
  {
    {
      name       = "album",
      animfunc   = "tab_"..tab_index.."_"..private.orientation,
      title      = "str_dialog_fireflies_title_album",
      func_Setup = private.Tab_SetupAlbum
    },
    {
      name        = "stickerpack",
      animfunc    = "tab_"..tab_index.."_"..private.orientation,
      title       = "str_dialog_fireflies_title_stickerpack",
      func_Setup  = private.Tab_SetupStickerpack,
      description = "str_dialog_fireflies_booster_description"
    },
    {
      name       = "duplicates",
      animfunc   = "tab_"..tab_index.."_"..private.orientation,
      func_Setup = private.Tab_SetupDuplicates,
      title      = "str_dialog_fireflies_title_duplicates",
    },
    {
      name       = "info",
      animfunc   = "tab_"..tab_index.."_"..private.orientation,
      func_Setup = private.Tab_SetupInfo,
      title      = "str_dialog_fireflies_title_info",
    }
  };

  local animfunc = tabs[ tab_index ].animfunc;
  local endtrig  = nil;

  -- JK: setup may indicate that tab func should play or not
  local playing = tabs[ tab_index ].func_Setup();

  if ( animate ) then

    animfunc = animfunc.."_"..animate;
    playing  = true;
    endtrig  = function () 

      ObjSet( private.window,
        {
          animfunc = tabs[ tab_index ].animfunc,
          playing  = false,
          endtrig  = function () end;
        }
      );

      if ( func_end ) then

        func_end();

      end;

    end;

  end;

  local set =
  {
    frame                   = 0,
    animfunc                = animfunc,
    playing                 = playing,
    endtrig                 = endtrig,
    title                   = ne.strings.Get( tabs[ tab_index ].title ),
    stickerpack_description = tabs[ tab_index ].description or nil
  };  

  ObjSet( private.window, set );  

end;
--------------------------------------------------------------------------------
function private.Tab_SetupAlbum ()

  private.Tab_HideNotification( "album" );

  private.func_TopPanel_Hide();

  private.album_current_page = 1;

  private.Album_RefreshData();  
  private.Album_ShowPage( private.album_current_page );

  private.Album_UpdateStickerpool();

  -- JK: don't play tab animation
  return false;

end;
--------------------------------------------------------------------------------
function private.Tab_SetupStickerpack ()

  private.stickerpack_data = private.func_Stickerpack_Data();

  private.Tab_ShowNotification(
    "stickerpack",
    private.stickerpack_count,
    true,
    true
  );
  
  if ( private.stickerpack_count > 0 ) then

    private.func_TopPanel_Hide();

    int.Button_Disable( "btn_int_dialog_fireflies_stickerpack_buy" );
    int.Button_Enable( "btn_int_dialog_fireflies_stickerpack_get" );

  else

    private.func_TopPanel_Show();

    int.Button_Disable( "btn_int_dialog_fireflies_stickerpack_get" );
    int.Button_Enable( "btn_int_dialog_fireflies_stickerpack_buy" );

    int.Button_SetPrice(
      "btn_int_dialog_fireflies_stickerpack_buy",
      private.stickerpack_data.price
    );

    mediator.Notify( "Interface_DialogFireflies_BuyMode" );

  end;

  -- JK: play tab animation
  return true;

end;
--------------------------------------------------------------------------------
function private.Tab_SetupDuplicates ()

  private.Tab_HideNotification( "duplicates" );

  private.func_TopPanel_Show();

  private.duplicates_data = private.func_Duplicates_Data();

  if ( #private.duplicates_data.duplicates > 0 ) then

    ObjSet( "anm_int_dialog_fireflies_window",
    {
      noduplicates = ""
    } );
    
    ObjSet( "anm_int_dialog_fireflies_duplicate_empty",
    {
      active = false, visible = false
    } );

    int.Button_Enable( "btn_int_dialog_fireflies_duplicates_sell_all" );

    if ( not private.duplicates_scc_created ) then

      private.Duplicates_CreateScroller( private.duplicates_data.duplicates );

    end;

    -- JK: play tab animation
    return true;

  else

    ObjSet( "anm_int_dialog_fireflies_window",
    {
      noduplicates = "str_dialog_fireflies_stickers_empty"
    } );
    
    ObjSet( "anm_int_dialog_fireflies_duplicate_empty",
    {
      active = true, visible = true
    } );

    int.Button_Disable( "btn_int_dialog_fireflies_duplicates_sell_all" );

    -- JK: don't play tab animation
    return false;

  end;

end;
--------------------------------------------------------------------------------
function private.Tab_SetupInfo ()

  private.func_TopPanel_Hide();

  if ( not private.album_data ) then

    private.Album_RefreshData();

  end;

  if ( not private.info_scroller_created ) then

    private.Info_CreateScroller();

  end;

  local stats = private.func_Stats();

  private.Info_CreateProgressbar( stats );

  -- JK: don't play tab animation
  return false;

end;
--------------------------------------------------------------------------------
function private.Tab_ShowNotification ( tab_name, delta, clear_first, hide_zero )

  if ( delta < 1 ) and ( not clear_first ) then

    return;

  end;
  
  if ( clear_first ) then
    
    private.notification_count[ tab_name ] = 0;
    
  end;

  private.notification_count[ tab_name ] =
    private.notification_count[ tab_name ] + delta;
    
  if ( hide_zero ) and ( private.notification_count[ tab_name ] < 1 ) then
    
    private.Tab_HideNotification( tab_name );
    
    return;
    
  end;

  m_object.Set(
    "anm_int_dialog_fireflies_tab_"..tab_name.."_notification",
    {
      active  = true,
      visible = true,
      count   = tostring( private.notification_count[ tab_name ] )
    }
  );  

end;
--------------------------------------------------------------------------------
function private.Tab_HasNotification ( tab_name )

  return private.notification_count[ tab_name ] > 0;

end;
--------------------------------------------------------------------------------
function private.Tab_HideNotification ( tab_name )

  private.notification_count[ tab_name ] = 0;

  m_object.Disable(
    "anm_int_dialog_fireflies_tab_"..tab_name.."_notification",
    true
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Album_RefreshData ( page_number )

  if ( private.album_data ) and
     ( page_number        ) then

    local firefly_name                = private.album_data[ page_number ].name;
    private.album_data[ page_number ] = private.func_Page_Data( firefly_name );

    return;    

  end;

  private.album_data        = private.func_Album_Data();
  private.album_total_pages = #private.album_data;

end;
--------------------------------------------------------------------------------
function private.Album_ShowPage ( page_number )
  
  local stickers = private.album_data[ page_number ].stickers;
  local rewards  = private.album_data[ page_number ].rewards;
  
  local objects = {};

  if ( #stickers < 6 ) then

    for s = #stickers, 6 do
      
      table.insert( objects, { "anm_int_dialog_fireflies_album_sticker_"..s,
      {
        active  = false,
        visible = false,
        input   = false
      } } );
      
    end;

  end;
  
  for s, sticker in pairs( stickers ) do
    
    if ( sticker.collected ) then
    
      table.insert( objects, { "anm_int_dialog_fireflies_album_sticker_"..s,
      {
        animfunc = "sticked",
        active  = true,
        visible = true, 
        image   = m_content.Res( "sticker", sticker.name )
      } } );

      table.insert( objects, { "anm_int_dialog_fireflies_album_index_"..s,
      {
        active  = false,
        visible = false,
        alp     = 0
      } } );
    
    else

      table.insert( objects, { "anm_int_dialog_fireflies_album_sticker_"..s,
      {
        animfunc = "sticked",
        active   = true,
        visible  = true,
        image    = ""
      } } );

      table.insert( objects, { "anm_int_dialog_fireflies_album_index_"..s,
      {
        txt_index = tostring( sticker.index ),
        active    = true,
        visible   = true,
        alp       = 1
      } } );

    end;
    
  end;

  for r = 1, #rewards do

    if ( rewards[ r ]. complete ) then

      table.insert( objects,
      {
        "anm_int_dialog_fireflies_album_reward_"..r,
        {
          active  = false,
          visible = false
        }
      } );

    else

      local res = m_content.Res(
        rewards[ r ].type,
        rewards[ r ].name
      );

      table.insert( objects,
      {
        "anm_int_dialog_fireflies_album_reward_"..r,
        {
          active       = true,
          visible      = true,
          alp          = 1,
          reward_icon  = res,
          reward_label = private.reward_label
        }
      } );

    end;

  end;

  table.insert( objects, { "anm_int_dialog_fireflies_album_slot",
  {
    animfunc = "slot_"..private.album_data[ page_number ].name } } );
  
  table.insert( objects, { "anm_int_dialog_fireflies_window",
  {
    page_number = tostring( page_number ).."/"..private.album_total_pages } } );
  
  ObjMultiSet( objects );
  
  private.album_current_page = page_number;
  
end;
--------------------------------------------------------------------------------
function private.Album_ShowNextPage ()
  
  local next_page = private.album_current_page + 1;
  
  if ( next_page > private.album_total_pages ) then
    
    next_page = 1;
    
  end;
  
  AnimPlay( "anm_int_dialog_fireflies_album_slot", "page_turn_next", function()
    
    private.Album_ShowPage( next_page );
  
  end );
  
end;
--------------------------------------------------------------------------------
function private.Album_ShowPrevPage ()
  
  local prev_page = private.album_current_page - 1;
  
  if ( prev_page < 1 ) then
    
    prev_page = private.album_total_pages;
    
  end;
  
  AnimPlay( "anm_int_dialog_fireflies_album_slot", "page_turn_prev", function()
     
    private.Album_ShowPage( prev_page );
  
  end );
  
end;
--------------------------------------------------------------------------------
function private.Album_UpdatePage ( page_number )

  page_number = page_number or private.album_current_page;

  private.Album_RefreshData();
  private.Album_ShowPage( page_number );

end;
--------------------------------------------------------------------------------
function private.Album_FindPage ( firefly_name )
  
  return cmn.IsEntityInList( 
    private.album_data, 
    { name = firefly_name },
    "name"
  );
  
end;
--------------------------------------------------------------------------------
function private.Album_CreateStickerpool ( stickers )
  
  local pool_step_x     = 28;
  local pool_step_ang   = 0.10;
  local pool_step_scale = 0.08;
  local pool_step_color = 0.12;
  local pool_amount     = 6;
  local obj_pool        = "obj_int_dialog_fireflies_stickerpool_stickers";
  local multiset        = {};
  
  private.Album_DestroyStickerpool();
  
  ObjCreate( obj_pool, "obj" );
  ObjAttach( obj_pool, "obj_int_dialog_fireflies_stickerpool" );
  
  local pool_max = #stickers;

  if ( pool_max > pool_amount ) then

    pool_max = pool_amount;

  end;
    
  for s, sticker in ipairs( stickers ) do

    if ( s > pool_amount ) then

      break;

    end;
    
    local anm_sticker = "anm_int_dialog_fireflies_album_stickerpack_stickerpool_"..sticker.name;
    
    local s_adjusted   = s;
    --if ( s_adjusted > pool_amount ) then s_adjusted = pool_amount; end;
    local s_normalized = s_adjusted - 1;
    
    --AL> easy fine view
    local pos_x = - 2 * pool_step_x + s_normalized * pool_step_x;
    local ang   =     - s_normalized * pool_step_ang;
    local scale = 1.0 - s_normalized * pool_step_scale;
    local color = 1.0 - s_normalized * pool_step_color;
    
    ObjCreate( anm_sticker, "anim" );
    ObjAttach( anm_sticker, obj_pool );

    table.insert( multiset, { anm_sticker, {
      res      = "assets/content/interface/int_dialog_fireflies/anm_fireflies_sticker",
      animfunc = "sticked",
      pos_x    = pos_x,
      pos_z    = #stickers - s,
      scale_x  = scale,
      scale_y  = scale,
      color_r  = color,
      color_g  = color,
      color_b  = color,
      ang      = ang,
      image    = m_content.Res( "sticker", sticker.name )
    } } );
    
    if ( s == 1 ) then

      private.Album_DragSticker( sticker, anm_sticker );

    end;
    
  end;
  
  ObjMultiSet( multiset );
  
end;
--------------------------------------------------------------------------------
function private.Album_DestroyStickerpool ()
  
  local obj_pool = "obj_int_dialog_fireflies_stickerpool_stickers";
  if ObjGet( obj_pool ) then ObjDelete( obj_pool ); end;
  
end;
--------------------------------------------------------------------------------
function private.Album_UpdateStickerpool ( stickerpack )

  local multiset           = {};
  local stickable_stickers = {};
  local duplicate_stickers = {};

  local obj_stickerpool = "obj_int_dialog_fireflies_stickerpool";

  local enabled  = { active = true,  input = true,  visible = true };
  local disabled = { active = false, input = false, visible = false };
  
  for f, firefly in pairs( private.album_data ) do

    for s, sticker in ipairs( firefly.stickers ) do

      if ( sticker.stickable ) then

        table.insert( stickable_stickers, sticker );

      end;

    end;

  end;
    
  if ( stickerpack ) then

    for n, new_sticker in ipairs( stickerpack ) do
      
      local s = cmn.IsEntityInList( stickable_stickers, new_sticker, "name" );
      
      if ( s ) then
        
        local sticker = stickable_stickers[ s ];
        
        if ( not stickable_stickers[ s ].from_stickerpack ) then

          stickable_stickers[ s ].from_stickerpack = true;

        else

          table.insert( duplicate_stickers, stickable_stickers[ s ] );

        end;
        
      else
        
        table.insert( duplicate_stickers, new_sticker );
        
      end;
      
    end;
    
  end;
  
  if ( #stickable_stickers > 0 ) then
    
    private.Album_CreateStickerpool( stickable_stickers );
    
    table.insert( multiset, { obj_stickerpool, enabled  } );
    
    mediator.Notify(
      "Interface_DialogFireflies_HasStickable",
      stickable_stickers
    );
    
  else
      
    table.insert( multiset, { obj_stickerpool, disabled } );
    
  end;
  
  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
function private.Album_StickerPosition ( sticker_index )

  local pos_sticker =
    int.Position(
      "anm_int_dialog_fireflies_album_sticker_"..sticker_index,
      "anm_int_dialog_fireflies_album_slot"
    );

  local pos_slot =
    int.Position(
      "anm_int_dialog_fireflies_album_slot"
    );

  return { 
    pos_slot[1] + pos_sticker[1],
    pos_slot[2] + pos_sticker[2]
  };
    
end;
--------------------------------------------------------------------------------
function private.Album_DragSticker ( sticker, anm_sticker )

  local func_startdrag = function()

    private.Album_OnDragSticker( sticker, anm_sticker );

  end;
  
  local func_dragdrop = function()

    private.Album_OnDropSticker( sticker, anm_sticker );

  end;
  
  ObjSet( anm_sticker, {
    input           = true,
    drag            = true,
    realdrag        = true,
    event_startdrag = func_startdrag,
    event_dragdrop  = func_dragdrop,
    inputrect_init  = true,
    inputrect_x     = -150,
    inputrect_y     = -160,
    inputrect_w     = 280,
    inputrect_h     = 300
  } );
      
end;
--------------------------------------------------------------------------------
function private.Album_OnDragSticker ( sticker_data, anm_sticker )

  ObjSet( anm_sticker, {
    animfunc = "drag",
    playing  = true
  } );

  local sticker_index = 0;

  for f, firefly in pairs( private.album_data ) do
    
    if ( firefly.name == sticker_data.firefly ) then
      
      for s, sticker in pairs( firefly.stickers ) do

        if ( sticker.name == sticker_data.name ) then

          sticker_index = s;

          private.Album_IndexStartPop( s );

          break;

        end;

      end;

      private.Album_ShowPage( f );
      private.Sticker_ShowGrayscale( sticker_data.name, sticker_index );

      break;

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Album_OnDropSticker ( sticker, anm_sticker )

  local sticker_index = string.match( sticker.name, "%d+" );

  private.Sticker_HideGrayscale( sticker_index );

  private.Album_IndexStopPop( sticker_index );
  
  local pos_target = private.Album_StickerPosition( sticker_index );
  local pos_cursor = GetGameCursorPos();

  local diff       = {
    math.abs( pos_target[1] - pos_cursor[1] ), 
    math.abs( pos_target[2] - pos_cursor[2] )
  };
 
  if ( diff[ 1 ] < 70 and diff[ 2 ] < 100 ) then

    int.Lock( true );
    
    if ( private.func_TryStick( sticker.name ) ) then
    
      private.Info_TryActivateSticker( sticker.name );

      private.Album_RefreshData( private.album_current_page );

      --private.Album_DestroyStickerpool();
      private.Album_UpdateStickerpool();
      
      local anm_sticker =
        "anm_int_dialog_fireflies_album_sticker_"..sticker_index;
      local anm_index =
        "anm_int_dialog_fireflies_album_index_"..sticker_index;
      
      ObjMultiSet( {
        { anm_index,
          { 
            active   = false, 
            visible  = false 
          }
        },
        { anm_sticker,
          { 
            active   = true, 
            visible  = true, 
            animfunc = "stick", 
            image    = m_content.Res( "sticker", sticker.name ),
            frame    = 0,
            playing  = true,
            endtrig  = function ()
              
              private.Album_UpdatePage();

              mediator.Notify(
                "Interface_DialogFireflies_StickerSticked",
                sticker.name
              );

              if ( private.Reward_IsPending() ) then                

                local params   = private.Reward_Params();
                local anm_name =
                  "anm_int_dialog_fireflies_album_reward_"..params.index;

                ObjSet( anm_name,
                  {
                    active  = true,
                    visible = true
                  }
                );

                AnimPlay(
                  anm_name,
                  "light_"..params.layout,
                  function ()

                    ObjSet( anm_name,
                      {
                        active   = false,
                        visible  = false,
                        animfunc = "reward_static"
                      }
                    );

                    private.Reward_Show( 
                      function()

                        int.Lock( false );                        

                      end
                    );                    

                  end
                );
                
              else

                int.Lock( false );

              end;

              ObjSet( anm_sticker, { endtrig = function () end } );

            end 
          }
        }
      } );
      
      cmn.PlaySnd( "place_booster" );
      
    end;

  end;
  
end;
--------------------------------------------------------------------------------
function private.Album_IndexStartPop ( sticker_index )
  
  local anm_index = "anm_int_dialog_fireflies_album_index_"..sticker_index;
        
  ObjAnimate( anm_index, "scale_xy", 1, 0, "", 
  { 
    0.0, 3, 1.0, 1.0,
    0.4, 3, 1.4, 1.4,
    0.8, 3, 1.0, 1.0
  } );
  
end;
--------------------------------------------------------------------------------
function private.Album_IndexStopPop ( sticker_index )
  
  local anm_index = "anm_int_dialog_fireflies_album_index_"..sticker_index;
        
  ObjStopAnimate( anm_index, "scale_xy" );

  ObjSet( anm_index,
  {
    scale_x = 1.0,
    scale_y = 1.0
  } );
  
end;
--------------------------------------------------------------------------------
function private.Album_IndexAppear ( sticker )

  local anm_slot   = "anm_int_dialog_fireflies_album_slot_"..sticker.firefly;
  local anm_page   = ObjGetRelations( anm_slot ).parent;
  local p          = tonumber( anm_page:match( "%d+" ));
  local multiset   = {};
  local index_list = {};
  local func_set   = false;
  
  for f, firefly in ipairs( private.album_data ) do
  
    if ( firefly.name == sticker.firefly ) then
    
      if ( #firefly.stickers == 1 ) and ( not firefly.stickers[ 1 ].collected ) then
        return;
      end;
      
      for s, sticker in ipairs( firefly.stickers ) do
        
        if ( sticker.collected ) then
        
          local anm_index = "anm_int_dialog_fireflies_album_index_"..sticker.name;
          
          table.insert( index_list, anm_index )
          
          table.insert( multiset, { anm_index, { 
            active  = true,
            visible = true
          } } );
          
          ObjStopAnimate( anm_index, "alp" );
          
          local alp = ObjGet( anm_index ).alp;
          local anim_table = { 
            0.0, 0, alp,
            0.3, 3, 1.0,
            3.0, 0, 1.0
          };
          
          if ( not func_set ) then
          
            func_set = true;
            
            ObjAnimate( anm_index, "alp", 0, 0, function ()
              private.Album_IndexDisappear( index_list );
            end, anim_table );
            
          else
            
            ObjAnimate( anm_index, "alp", 0, 0, "", anim_table );
            
          end;
          
        end;
        
      end;
      
      break;
      
    end;
    
  end;
  
  if ( #multiset > 0 ) then
  
    ObjMultiSet( multiset );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Album_IndexDisappear ( index_list )
  
  for i, anm_index in ipairs( index_list ) do
    
    ObjAnimate( anm_index, "alp", 0, 0, function ()
    
      ObjSet( anm_index, {
        active  = false,
        visible = false
      } );
      
    end,
    { 
      0.0, 0, 1.0,
      0.3, 0, 0.0
    } );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Sticker_ShowGrayscale ( sticker_name, sticker_index )

  local anm_sticker = "anm_int_dialog_fireflies_album_sticker_"..sticker_index;

  ObjSet( anm_sticker, {
    active    = true,
    visible   = true,
    animfunc  = "sticked",
    image     = m_content.Res( "sticker", sticker_name ),
    grayscale = true,
    alp       = 0.4
  } );

end;
--------------------------------------------------------------------------------
function private.Sticker_HideGrayscale ( sticker_index )

  local anm_sticker = "anm_int_dialog_fireflies_album_sticker_"..sticker_index;

  ObjSet( anm_sticker, {
    image     = "",
    grayscale = false,
    alp       = 1.0
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Stickerpack_TryBuy ()

  local stickers = private.stickerpack_data.func_Buy();

  if ( not stickers ) then

    return;

  end;

  private.Stickerpack_Open( stickers );

end;
--------------------------------------------------------------------------------
function private.Stickerpack_Claim ()

  local stickers = private.stickerpack_data.func_Get();

  if ( not stickers ) then

    return;

  end;

  private.Stickerpack_Open( stickers );

end;
--------------------------------------------------------------------------------
function private.Stickerpack_Open ( stickers )

  int.Lock( true );

  -- JK: hide button
  private.Stickerpack_HideButton();

  private.stickerpack_count = private.func_StickerpackGetCount();
  
  private.Tab_ShowNotification(
    "stickerpack",
    private.stickerpack_count,
    true,
    true
  );
  
  private.stickers_in_pack  = stickers;
  private.stickers_open     = 0;
  private.stickers_tapped   = {};

  private.Stickerpack_SetStickers( stickers );

  cmn.PlaySnd( "open_booster" );

  AnimPlay(
    "anm_int_dialog_fireflies_stickerpack",
    "booster_open",
    function()

      private.func_TopPanel_Hide();

      private.Stickerpack_OpenSticker( stickers[ 1 ], 1, function ()

        private.Stickerpack_OpenSticker( stickers[ 2 ], 2, function ()

          private.Stickerpack_OpenSticker( stickers[ 3 ], 3, function ()            

            AnimPlay(
              "anm_int_dialog_fireflies_stickerpack",
              "booster_close",
              function()

                private.Tab_SetupStickerpack();

                cmn.PlaySnd( "booster_candy_fall" );

                AnimPlay(
                  "anm_int_dialog_fireflies_stickerpack",
                  "booster_appear",
                  function()                    

                    AnimPlay(
                      "anm_int_dialog_fireflies_stickerpack",
                      "booster_static",
                      ""
                    );

                    if ( private.Tab_HasNotification( "album" ) ) then

                      AnimPlay(
                        "anm_int_dialog_fireflies_window",
                        "tab_2_"..private.orientation.."_action",
                        function ()

                          private.Tab_Show(
                            1,
                            "action",
                            function()

                              int.Lock( false );

                            end
                          );

                        end
                      );

                    else

                      int.Lock( false );
                      private.Stickerpack_ShowButton();

                    end;

                  end
                );

              end
            );

          end );

        end );

      end );

    end
  );

end;
--------------------------------------------------------------------------------
function private.Stickerpack_OpenSticker (
    sticker_data,
    sticker_index,
    func_end
  )

  cmn.PlaySnd( "open_booster_"..sticker_index );

  AnimPlay(
    "anm_int_dialog_fireflies_stickerpack_sticker_"..sticker_index,
    "open",
    function()

      if ( sticker_data.dupes > 0 ) then

        private.Tab_ShowNotification( "duplicates", 1 );
        private.Duplicates_DestroyScroller();

      else

        private.Tab_ShowNotification( "album", 1 );

      end;

      if ( func_end ) then

        func_end();

      end;

    end
  );

end;
--------------------------------------------------------------------------------
function private.Stickerpack_SetStickers ( stickers )

  local objects       = {};
  local stickers_safe = {};

  for s, sticker in ipairs( stickers ) do

    table.insert( stickers_safe, sticker );

  end;
  
  for s = #stickers, 1, -1 do
    
    local sticker = stickers_safe[ s ];
    table.remove( stickers_safe, s ); 
    
    table.insert( objects,
    { "anm_int_dialog_fireflies_stickerpack_sticker_"..s,
      {
        animfunc = "back",
        playing  = true,
        frame    = 0,
        image    = m_content.Res( "sticker", sticker.name )
      }
    } );
    
    table.insert( objects,
    { "anm_int_dialog_fireflies_stickerpack_sticker_"..s.."_new",
      {
        active   = true,
        visible  = true,
        animfunc = "sticked"
      }
    } );
    
  end;
  
  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------
function private.Stickerpack_TapSticker ( sticker_index )

  if ( cmn.IsEntityInList( private.stickers_tapped, sticker_index ) ) then
    
    return;
    
  end;
  
  table.insert( private.stickers_tapped, sticker_index );
  
  local sticker  = private.stickers_in_pack[ sticker_index ];
  local anm_name = "anm_int_dialog_fireflies_stickerpack_sticker_"..sticker_index;
    
  ObjSet( anm_name, {
    input    = false,
    animfunc = "open",
    playing  = true,
    frame    = 0,
    endtrig  = function ()
    
      AnimPlay( anm_name, "drag", "" );
    
    end;
  } );
    
  private.stickers_open = private.stickers_open + 1;

  cmn.PlaySnd( "open_booster_"..sticker_index );

end;
--------------------------------------------------------------------------------
function private.Stickerpack_HideButton ()

  --DbgTrace( "Stickerpack_HideButton" );

  AnimPlay(
    private.window,
    "tab_2_"..private.orientation.."_buyloop",
    ""
  );

end;
--------------------------------------------------------------------------------
function private.Stickerpack_ShowButton ()

  --DbgTrace( "Stickerpack_ShowButton" );

  AnimPlay(
    private.window,
    "tab_2_"..private.orientation,
    ""
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Duplicates_TrySell ( sticker_name )

  return private.duplicates_data.func_TrySell( sticker_name, 1 );

end;
--------------------------------------------------------------------------------
function private.Duplicates_TrySellAll ()

  private.duplicates_data.duplicates =
    private.duplicates_data.func_TrySellAll(
      private.duplicates_data.duplicates
    );

  private.Duplicates_DestroyScroller();
  private.Tab_Show( 3 );

end;
--------------------------------------------------------------------------------
function private.Duplicates_CreateScroller ( duplicates )
  
  local objects  = {};
  local slot_num = 1;
  local w, h     = 604, 700 - private.duplicates_scc_x;
  local x, y     = -w / 2, -h / 2 + private.duplicates_scc_x / 2;
  local pages    = math.ceil( #duplicates / private.duplicates_per_page );
  
  ObjClone(
    "scc_int_dialog_fireflies_duplicates_default",
    "_rename",
    false,
    "anm_int_dialog_fireflies_window"
  );
  
  ObjSet( "scc_int_dialog_fireflies_duplicates_default_rename",
    {
      pos_y          = private.duplicates_scc_x,
      name           = "scc_int_dialog_fireflies_duplicates",
      cellsize       = private.duplicates_scc_cell_size,
      inputrect_init = true,
      inputrect_x    = x,
      inputrect_y    = y,
      inputrect_w    = w,
      inputrect_h    = h,
      croprect_init  = true,
      croprect_x     = x,
      croprect_y     = y,
      croprect_w     = w,
      croprect_h     = h
    }
  );
  
  private.duplicates_scc_created = true;
  
  for p = 1, pages do
  
    ObjClone( 
      "obj_int_dialog_fireflies_duplicates_page", 
      "_"..p, 
      true,
      "scc_int_dialog_fireflies_duplicates"
    );
    
  end;
  
  local btn_names = {};
  
  for s, sticker in ipairs( duplicates ) do
    
    local page_num = math.floor( ( s - 1 ) / private.duplicates_per_page ) + 1;
    local slot_x   = ( slot_num - 2 ) * private.duplicates_item_shift_x;
    local anm_x    = 0;
    local btn_name = "btn_int_dialog_fireflies_duplicates_sell_"..s;
    
    ObjClone( 
      "anm_int_dialog_fireflies_duplicates_cell", 
      "_"..s, 
      1,
      "obj_int_dialog_fireflies_duplicates_page_slot"..slot_num.."_"..page_num
    );
    
    table.insert( btn_names, btn_name );
    
    int.Button_Create(
      btn_name,
      {
        title  = "str_dialog_fireflies_stickers_sell",
        design = "medium",
        color  = "green",
        price  = true,
        sound  = "simply_click"
      },
      function()
        
        if ( private.Duplicates_TrySell( private.duplicates_data.duplicates[ s ].name ) ) then
        
          if ( private.duplicates_data.duplicates[ s ].count == 1 ) then
            
            private.Duplicates_DestroyScroller();
            
            if ( #private.duplicates_data.duplicates == 1 ) then
              
              private.Tab_Show( 3 );
              
            else
              
              private.Tab_SetupDuplicates();
              
            end;
            
          else
          
            private.duplicates_data.duplicates[ s ].count =
              private.duplicates_data.duplicates[ s ].count - 1;
            
            ObjSet( 
              "anm_int_dialog_fireflies_duplicates_cell_"..s,
              { count = tostring( "#"..sticker.count ) } 
            );
            
          end;

        end;
        
      end
    );

    table.insert( objects, { 
      "obj_int_dialog_fireflies_duplicates_page_slot"..slot_num.."_"..page_num,
      {
        pos_x   = slot_x,
        pos_y   = 35,
        scale_x = private.duplicates_item_scale,
        scale_y = private.duplicates_item_scale
      } 
    } );

    table.insert( objects, { 
      "anm_int_dialog_fireflies_duplicates_cell_"..s,
      {
        pos_x = anm_x,
        count = tostring( "#"..sticker.count ),
        image = m_content.Res( "sticker", sticker.name )
      } 
    } );
    
    slot_num = slot_num + 1;
    
    if ( slot_num == private.duplicates_per_page + 1 ) then slot_num = 1; end;
    
  end;
  
  -- JK: fill scc with dummies
  if ( pages < private.duplicates_scc_min_pages ) then
    
    for d = ( pages + 1 ), private.duplicates_scc_min_pages do 
    
      ObjClone( 
        "obj_int_dialog_fireflies_duplicates_page", 
        "_"..d, 
        false,
        "scc_int_dialog_fireflies_duplicates"
      );
    
    end;
    
  end;
  
  private.duplicates_scc_pages_number = pages;
  
  ObjMultiSet( objects );
  
  if ( #btn_names > 0 ) then
    
    mediator.Notify(
      "Interface_DialogFireflies_ButtonSellShown",
      btn_names[ 1 ]
    );
  
    private.btn_names = btn_names;
    
    for i, btn_name in ipairs( private.btn_names ) do
    
      int.Button_SetPrice( btn_name, duplicates[ i ].price );
    
    end;
    
  end;

end;
--------------------------------------------------------------------------------
function private.Duplicates_DestroyScroller ()

  if ( private.duplicates_scc_created ) then
    
    ObjDelete( "scc_int_dialog_fireflies_duplicates" );
    
    if ( private.btn_names and #private.btn_names > 0 ) then
  
      for i, btn_name in ipairs( private.btn_names ) do
        
        int.Button_Delete( btn_name );
        
      end;
    
      private.btn_names = nil;
      
    end;
    
    private.duplicates_scc_created = false;
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Duplicates_ScrollerNext ()
  
  local scrollpos = int.Scroller_Next(
    "scc_int_dialog_fireflies_duplicates",
    private.scc_pages_number,
    false
  );
  
end;
--------------------------------------------------------------------------------
function private.Duplicates_ScrollerPrev ()
  
  local scrollpos = int.Scroller_Prev(
    "scc_int_dialog_fireflies_duplicates",
    private.scc_pages_number,
    false
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Info_CreateScroller ()

  local objects        = {};
  local page_number    = 1;

  ObjClone(
    "scc_int_dialog_fireflies_info_default",
    "_rename",
    0,
    "hub_int_dialog_fireflies"
  );

  table.insert( objects, 
    {
      "scc_int_dialog_fireflies_info_default_rename",
      {
        name = "scc_int_dialog_fireflies_info"
      }
    }
  );

  for i = 1, 8 do

    ObjClone(
      "anm_int_dialog_fireflies_info_slot",
      "_"..i,
      1,
      "scc_int_dialog_fireflies_info_default_rename"
    );

    table.insert( objects, 
      {
        "anm_int_dialog_fireflies_info_slot_"..i,
        {
          animfunc = "info_slot_"..i
        }
      }
    );    

    for s = 1, 4 do

      if ( private.album_data[ page_number ] ) then

        local tap_number = page_number;

        common.TapSet(
          "obj_int_dialog_fireflies_info_slot_input_"..s.."_"..i,
          {
            func_true = function() private.Info_TapPage( tap_number ); end
          }
        );

        local stickers     = private.album_data[ page_number ].stickers;
        local firefly_name = private.album_data[ page_number ].name;

        for s, sticker in pairs( stickers ) do

          ObjClone(
            "anm_int_dialog_fireflies_info_sticker",
            "_"..sticker.name,
            0,
            "anm_int_dialog_fireflies_info_slot_"..i
          );

          local animfunc = "sticker_noactive";

          if ( sticker.collected ) then

            animfunc = "sticker_active";

          end;

          table.insert( objects,
          {
            "anm_int_dialog_fireflies_info_sticker_"..sticker.name,
            {
              animfunc = animfunc,
              anim_tag = sticker.name
            }
          } );

          table.insert( objects,
          {
            "anm_int_dialog_fireflies_info_slot_"..i,
            {
              [ firefly_name.."_number" ] = tostring( page_number )
            }
          } );

        end;

      else

        table.insert( objects,
        {
          "obj_int_dialog_fireflies_info_slot_input_"..s.."_"..i,
          {
            active  = false,
            visible = false,
            input   = false
          }
        } );

      end;

      page_number = page_number + 1;

    end;

  end;

  ObjMultiSet( objects );

  ObjAttach(
    "scc_int_dialog_fireflies_info",
    "anm_int_dialog_fireflies_window"
  );

  private.info_scroller_created = true;

end;
--------------------------------------------------------------------------------
function private.Info_CreateProgressbar ( stats )

  ObjSet(
    "anm_int_dialog_fireflies_info_progressbar",
    {
      frame = stats.stickers.collected,
      info_progress_title = "str_int_panel_fireflies_statistics",
      info_progress_count =
        tostring( stats.stickers.collected.."/"..stats.stickers.total )
    }
  );

end;
--------------------------------------------------------------------------------
function private.Info_DestroyScroller ()

  if ( private.duplicates_scc_created ) then
      
    ObjDelete( "scc_int_dialog_fireflies_info" );

    private.duplicates_scc_created = false;

  end;

end;
--------------------------------------------------------------------------------
function private.Info_TryActivateSticker ( sticker_name )

  if ( private.info_scroller_created ) then

    ObjSet(
      "anm_int_dialog_fireflies_info_sticker_"..sticker_name,
      {
        animfunc = "sticker_active"
      }
    );

  end;

end;
--------------------------------------------------------------------------------
function private.Info_TapPage ( page_number )

  cmn.PlaySnd( "tab_switch" );

  private.Tab_Show( 1 );
  private.Album_ShowPage( page_number );

end;
--------------------------------------------------------------------------------
function private.Info_TryCompleteAchievement ()

  --

end;
--------------------------------------------------------------------------------