-- name=int_dialog_game_pause
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.active = false;
  private.module = "int_dialog_game_pause";
  private.object = "obj_int_dialog_game_pause";
  private.splash = "anm_int_dialog_game_pause";
  
  private.pause_allowed = true;

  private.animfunc =
  {
    static = "static",
    hide   = "disappear"
  };

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  local app_params = ne.obj.Get( "ng_application" ) or {};
  
  local scale_x = app_params.scale_x or 1;
  local scale_y = app_params.scale_x or 1;
  
  local pos_z = app_params.pos_z or 0;

  local w = GetAppWidth()  * scale_x;
  local h = GetAppHeight() * scale_y;
  
  ne.obj.Set(
    private.object,
    {
      pos_x   = w / 2,
      pos_y   = h / 2,
      pos_z   = pos_z + 1,
      scale_x = scale_x,
      scale_y = scale_y,
      
      inputrect_init = true,
      inputrect_w =  w,
      inputrect_x = -w / 2,
      inputrect_h =  h,
      inputrect_y = -h / 2,
      event_mup   =  public.Hide
    }
  );
  ne.obj.Set(
    private.splash,
    {
      animfunc = private.animfunc.static,
      frame    = 0,
      playing  = true,
      tt_pause = "str_systemmessage_pause"
    }
  );
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.CreateHub ( obj_hub, obj_app )

  ne.obj.Create( obj_hub, "obj"   );
  ne.obj.Attach( obj_hub, obj_app );
  ne.obj.Set( obj_hub, { pos_z = 10 } );
  
  private.parent = obj_hub;

end;
--------------------------------------------------------------------------------
function public.SetHandlers ( func_pause, func_resume )

  private.func_pause  = func_pause;
  private.func_resume = func_resume;

end;
--------------------------------------------------------------------------------
function public.Subscribe ()

  mediator.Subscribe(
    "View_Shop_PurchaseStart",
    private.Disallow
  );

  mediator.Subscribe(
    "View_Social_LoginStart",
    private.Disallow
  );

  mediator.Subscribe(
    "View_Game_GooglePlayLoginStart",
    private.Disallow
  );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Show ()

  if ( private.active ) then return; end;
  if ( not private.pause_allowed ) then return; end;

  private.active = true;
  
  private.func_pause();
  private.Show();

end;
--------------------------------------------------------------------------------
function public.Hide ()

  if ( not private.active ) then return; end;
  private.active = false;
  
  private.Hide( private.func_resume );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Show ()

  ne.obj.Attach( private.module, private.parent );
  AnimPlay( private.splash, private.animfunc.static, function () end );

end;
--------------------------------------------------------------------------------
function private.Hide ( func_after )

  AnimPlay(
    private.splash,
    private.animfunc.hide,
    function ()
      if ( func_after ) then func_after() end;
      ne.obj.Detach( private.module );
    end
  );

end;
--------------------------------------------------------------------------------
function private.Allow ()

  private.pause_allowed = true;

  ne.msg.Unsubscribe(
    "Event_Application_GainFocus",
    private.Allow
  );

end;
--------------------------------------------------------------------------------
function private.Disallow ()

  private.pause_allowed = false;

  ne.msg.Subscribe(
    "Event_Application_GainFocus",
    private.Allow
  );

end;
--------------------------------------------------------------------------------