-- name=int_dialog_hearts
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.int_w       = int.W();
  private.int_h       = int.H();
  private.orientation = int.Orientation(); 

  private.this = {
    dialog = "obj_int_dialog_hearts",
    window = "anm_int_dialog_hearts_window",
    button = {
      close    = "btn_int_dialog_hearts_close",
      accept   = "btn_int_dialog_hearts_accept",
      increase = "btn_int_dialog_hearts_increase",
      ask      = "btn_int_dialog_hearts_ask"
    },
    timer = {
      main   = "tmr_int_dialog_hearts_main",
      social = "tmr_int_dialog_hearts_social"
    }
  };

  private.strings = {
    title      = ne.strings.Get( "str_int_dialog_hearts_caption" ),
    title_full = ne.strings.Get( "str_int_dialog_hearts_caption_full" ),
    full       = ne.strings.Get( "str_int_dialog_hearts_full" ),
    donate     = ne.strings.Get( "str_int_dialog_hearts_donate" )
  };

  if ( IsPC() ) then

    private.window_area = {
      default  = { -313, -250, 626, 600 },
      full     = { -313, -220, 626, 540 },
    };

  else

    private.window_area = {
      default  = { -313, -360, 626, 715 },
      full     = { -313, -360, 626, 715 },
      facebook = { -313, -360, 626, 715 }
    }; 

  end;

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.Adapt();
  private.CreateButtons();

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Show (
    count,
    max_count,
    price,
    count_donate,
    donate,
    facebook,
    facebook_timer_token, 
    func_donate,
    func_facebook,
    func_accept,
    func_close,
    func_top_panel_show,
    func_top_panel_hide,
    func_skin_attach,
    func_skin_detach
  )

  -- JK: debug facebook
  --facebook = true;

  private.active = true;

  private.FuncDonate    = func_donate;
  private.FuncFacebook  = func_facebook;
  private.FuncAccept    = func_accept;
  private.FuncClose     = func_close;
  private.TopPanel_Hide = func_top_panel_hide;

  private.donate   = donate;
  private.facebook = facebook;

  private.facebook_timer_token = facebook_timer_token;

  private.Skin_Attach = func_skin_attach;
  private.Skin_Detach = func_skin_detach;

  -- JK: Setup hero
  private.hero = private.Skin_Attach( private.this.window );

  ObjSet( private.hero,
  {
    animfunc = "dialog",
    playing  = true,
    anim_tag = "hero" } );

  private.SetDefault(
    count,
    max_count,
    price,
    count_donate,
    donate,
    facebook
  );

  public.Timer_Main_Show();

  if ( facebook and TimeIsEventExist( private.facebook_timer_token ) ) then

    public.Timer_Social_Show();
    
  end;

  int.Dialog_Open( "hearts", function ()

    func_top_panel_show();

  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_OnHide )

  if ( not private.active ) then

    return;

  end;

  private.active = false;

  private.TopPanel_Hide();

  int.Dialog_Close(
    "hearts",
    function ()

      private.Skin_Detach( private.hero );

      if ( func_OnHide ) then

        func_OnHide();

      end;

    end
  );

end;
--------------------------------------------------------------------------------
function public.Update ( count, max_count, price, reason )

  if ( not  private.active ) then

    return;

  end;

  if ( reason ) and ( reason == "donate" ) then
    
    private.accept = true;
    private.donate = false;
    
  end;
  
  if ( reason ) and ( reason == "facebook" ) then
    
    private.accept   = true;
    private.facebook = false;
    
  end;

  local multiset = {};

  m_object.Set(
    private.this.window,
    {
      hearts_count = tostring( count )
    },
    multiset
  );

  int.Button_SetTextTag(
    private.this.button.accept,
    "count",
    "+"..int.NumberForm( max_count - count ),
    multiset
  );

  if ( price ) then

    int.Button_SetPrice(
      private.this.button.accept,
      price,
      false,
      multiset
    );

  end;

  private.SetButtons( count, max_count, multiset );
  private.SetAnimfunc( count, max_count, multiset );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Timer_Main_Show ()

  if ( not private.active ) then

    return;

  end;

  local multiset     = {};
  local total_time   = TimeGetEventRemainingTime( "tmr_hearts_recover" );
  local current_time = cmn.GetTimeFromSec( math.ceil( total_time ) );
  local time_string  = private.strings.full;
  local window_title = private.strings.title;
  
  if ( total_time <= 0 ) then

    common.Timer_Stop(
      private.this.timer.main,
      multiset
    );

    window_title = private.strings.title_full;

  else

    common.Timer_Start(
      private.this.timer.main,
      1,
      function()

        public.Timer_Main_Show();

      end,
      multiset
    );

    time_string = int.Text_TimerFormat( current_time, "m:s" );
    
  end;

  m_object.Set(
    private.this.window,
    {
      time  = time_string,
      title = window_title
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function public.Timer_Social_Show ()

  if ( not private.active   ) or
     ( not private.facebook ) then

    return;

  end;

  local multiset   = {};
  local total_time = TimeGetEventRemainingTime( private.facebook_timer_token );
  local wait_time  = cmn.GetTimeFromSec( math.ceil( total_time ) );
  
  if ( total_time <= 0 ) then

    int.Input_On( private.this.button.ask, multiset );

    int.Button_SetText(
      private.this.button.ask,
      "str_int_dialog_hearts_ask",
      multiset
    );

    common.Timer_Stop(
      private.this.timer.social,
      multiset
    );

  else

    int.Input_Off( private.this.button.ask, multiset );

    int.Button_SetText(
      private.this.button.ask,
      wait_time.h..":"..wait_time.m..":"..wait_time.s,
      multiset
    );

    common.Timer_Start(
      private.this.timer.social,
      1,
      function()

        public.Timer_Social_Show();

      end,
      multiset
    );

  end;

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Adapt ()
  
  local multiset = {};

  int.Center_Object( private.this.dialog, 0, 0, multiset );
  int_window.SetArea(
    private.this.dialog,
    private.window_area.default,
    multiset
  );

  m_object.Set(
    private.this.window,
    {
      title = private.strings.title
    },
    multiset
  );

  m_object.Set(
    "anm_int_dialog_hearts_increase_panel",
    {
      title = private.strings.donate
    },
    multiset
  );

  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------
function private.CreateButtons ()
  
  int.Button_Create(
    private.this.button.close,
    {
      design = "close",
      sound  = "simply_click",
    },
    function()
      
      private.FuncClose( private.accept );
      private.accept = nil;
      
    end
  );

  if ( IsMobile() ) then

    int.Button_Create(
      private.this.button.increase,
      {
        design = "price"
      },
      function()
        
        if ( private.FuncDonate() ) then

          cmn.PlaySnd( "buy" );
          
          private.donate = false;
          private.FuncClose( true );
        
        end;
        
      end
    );

  else

    int.Button_Create(
      private.this.button.increase,
      {
        color  = "green",
        design = "medium"
      },
      function()
        
        if ( private.FuncDonate() ) then

          private.donate = false;
          private.FuncClose( true );
        
        end;
        
      end
    );

  end;

  int.Button_Create(
    private.this.button.accept,
    {
      title  = "str_int_dialog_hearts_accept",
      design = "heart",
      price  = true,
      pos_y  = 70 -- JK: workaround for buggy animation
    },
    function()
      
      private.accept = private.FuncAccept();

      if ( private.accept ) then

        int.Button_Disable( private.this.button.accept );

        if not ( private.donate ) then

          private.FuncClose( private.accept );
          private.accept = nil;

        end;

      end;
      
    end
  );

  int.Button_Create(
    private.this.button.ask,
    {
      title  = "str_int_dialog_hearts_ask",
      design = "facebook_large",
      sound  = "simply_click"
    },
    function()
      
      private.FuncFacebook();
      
    end
  );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.SetDefault (
    count,
    max_count,
    price,
    count_donate,
    donate,
    facebook
  )

  local multiset = {};

  int.Button_SetPrice( private.this.button.accept, price, multiset );

  int.Button_SetTextTag(
    private.this.button.accept,
    "count",
    "+"..int.NumberForm( max_count - count ),
    multiset
  );

  -- NOTE: 'donate' is donation price (number or string) or false (bool)
  local donate_price = donate or "0";

  if not ( IsPC() ) then
    
    donate_price = int.NumberForm( donate or 0 ); -- donate is number on IOS
    
  end;
  
  donate_price = tern( donate_price == "" )( 
    ne.strings.Get( "str_dialog_shop_gold_buy" ), 
    donate_price
  );

  m_object.Set(
    private.this.window,
    {
      hearts_count = int.NumberForm( count )
    },
    multiset
  );
  
  m_object.Set(
    "anm_int_dialog_hearts_increase_panel",
    {
      description =
        ne.strings.Get(
          "str_int_dialog_hearts_donate_price_description",
          int.NumberForm( max_count + count_donate )
        )
    },
    multiset
  );

  int.Button_SetText( private.this.button.increase, donate_price, multiset );

  m_object.Multiset( multiset );  

  private.SetButtons( count, max_count );
  private.SetAnimfunc( count, max_count );
 
end;
--------------------------------------------------------------------------------
function private.SetAnimfunc ( count, max_count, multiset )

  local restore  = count < max_count;
  local animfunc = "static_full_"..private.orientation;

  if ( restore ) then

    int_window.SetArea(
      private.this.dialog,
      private.window_area.default,
      multiset
    );

    animfunc = "static_"..private.orientation;

  else

    int_window.SetArea(
      private.this.dialog,
      private.window_area.full,
      multiset
    );

  end;

  if ( private.facebook ) and
     ( restore          ) then

    int_window.SetArea(
      private.this.dialog,
      private.window_area.facebook,
      multiset
    );

    animfunc = "static_fb_"..private.orientation;

  end;

  m_object.Set(
    private.this.window,
    {
      animfunc = animfunc,
      playing  = true
    },
    multiset
  );

end;
--------------------------------------------------------------------------------
function private.SetButtons ( count, max_count, multiset )

  int.Buttons_Disable(
    {
      private.this.button.accept,
      private.this.button.ask
    },
    multiset
  );

  if ( count < max_count ) then

    int.Button_Enable( private.this.button.accept, multiset );

  end;

  if ( private.donate ) then

    m_object.Enable( "anm_int_dialog_hearts_increase_panel", multiset );

  else

    m_object.Disable( "anm_int_dialog_hearts_increase_panel", multiset );

  end;

  if ( private.facebook ) then

    int.Button_Enable( private.this.button.ask, multiset );

  end;

end;
--------------------------------------------------------------------------------