-- name=int_dialog_hero
function  public.__InitModule ()

  private.app_w = int.W();
  private.app_h = int.H();
  private.app_o = int.Orientation();
  
  private.skins = 
  {
    list = {}, -- skins list
    map  =     -- skins info by name
    {
      data = {},
      id   = {}
    },

    notified = nil, -- notified skin list
    markers  = nil, -- notification markers

    focus =
    {
      skin = nil, -- skin name
      obj  = nil  -- attached object
    },

    scc =
    {
      width      = { portrait = 600, landscape = 600 },
      scales     = { focus = 1.2, near = 0.9, far = 0.75 },
      scale_time = 0.3
    },

    tab_created = false,
    func_create = private.SkinsTab_Create,
    func_update = private.SkinsTab_Update
  }

  private.achievements =
  {
    list = {},
    id   = {},

    scc  =
    {
      size =
      { 
        portrait  = { w = 600, h = 700 }, 
        landscape = { w = 600, h = 700 } 
      }
    },

    tab_created = false,
    func_create = private.AchievementsTab_Create,
    func_update = private.AchievementsTab_Update
  }

  private.info =
  {
    statistic   = {},

    skin     = nil, -- Skin & attached object
    skin_obj = nil,

    tab_created = false,
    func_create = private.InfoTab_Create,
    func_update = private.InfoTab_Update
  }
  
  private.window_area = {
    [ "portrait"  ] = { -320, -420, 640, 900 },
    [ "landscape" ] = { -250, -320, 500, 700 }
  };
  
end;
--------------------------------------------------------------------------------
function  public.__InitObjects ()

  int_window.SetArea( "obj_int_dialog_hero", 
                    private.window_area[ private.app_o ] );
  
  local multiset = {};

  -- Setup static text tags
  local str_prefix = "str_int_dialog_hero";

  local window = "anm_int_dialog_hero_window";
  local prefix = "assets/content/interface/int_dialog_hero/";

  local res =
  {
    [ "portrait"  ] = prefix.."anm_int_dialog_hero",
    [ "landscape" ] = prefix.."anm_int_dialog_hero_landscape"
  }

  local window_params = {
    res              = res[ private.app_o ],
    tab_skins        = StringGet( str_prefix.."_tab_skins" ),
    tab_achievements = StringGet( str_prefix.."_tab_achievements" ),
    tab_info         = StringGet( str_prefix.."_tab_info" ),
    
  };

  table.insert( multiset, { window, window_params } );

  -- Setup tab buttons and indicators positions
  table.insert( multiset, { "anm_int_dialog_hero_tabs", 
  {
    res      = res[ private.app_o ],
    animfunc = "tabs_"..private.app_o,
    playing  = true
  } } );

  -- Centering dialog
  table.insert( multiset, { "obj_int_dialog_hero", {
    pos_x       = private.app_w / 2,
    pos_y       = private.app_h / 2
  } } );

  -- Skin scroller width
  local skins_scc_w = private.skins.scc.width[ private.app_o ];

  table.insert( multiset, { "scc_int_dialog_hero_skin_scroller",
  {
    inputrect_x   = - skins_scc_w / 2,
    inputrect_w   =   skins_scc_w,
    croprect_x    = - skins_scc_w / 2,
    croprect_w    =   skins_scc_w,

    begindragtrig = private.SkinsTab_DragStart,
    enddragtrig   = private.SkinsTab_DragEnd
  } } );

  -- Achievments scroller
  local ach_scc = private.achievements.scc.size[ private.app_o ];

  table.insert( multiset, { "scc_int_dialog_hero_achievements_scroller",
  {
    inputrect_init = true,
    inputrect_x   = - ach_scc.w / 2,
    inputrect_y   = - ach_scc.h / 2,
    inputrect_w   =   ach_scc.w,
    inputrect_h   =   ach_scc.h,

    croprect_init = true,
    croprect_x    = - ach_scc.w / 2,
    croprect_y    = - ach_scc.h / 2,
    croprect_w    =   ach_scc.w,
    croprect_h    =   ach_scc.h
  } } );
  
  ObjMultiSet( multiset );
  
  int.Button_Create(
    "btn_int_dialog_hero_close",
    {
      design = "close"
    },
    function () private.Hide( private.FuncClose ); end 
  );
  
  int.Button_Create(
    "btn_int_dialog_hero_skin_select",
    {
      title  = str_prefix.."_skin_wear",
      design = "medium",
      color  = "green",
      sound  = "wear"
    },
    function () private.SkinsTab_SkinWear(); end 
  );

  int.Button_Create(
    "btn_int_dialog_hero_info_login",
    {
      title  = str_prefix.."_info_login",
      design = "facebook"
    },
    function () private.InfoTab_Login() end 
  );
  
  if ( IsAndroid() and ( wiz.Android.GetPlatform() == "google" ) ) then
    int.Button_Create(
      "btn_int_dialog_hero_googleachievements",
      {
        design = "googleach"
      },
      function () private.OpenGoogleAchievements(); end
    );
  end;

  int.Control_EventsSet( "btn_int_dialog_hero_tab_skins", 
  { 
    func_on_click = function () private.ShowTab( "skins" ); end 
  } );

  int.Control_EventsSet( "btn_int_dialog_hero_tab_achievements", 
  { 
    func_on_click = function () private.ShowTab( "achievements" ); end 
  } );

  int.Control_EventsSet( "btn_int_dialog_hero_tab_info", 
  { 
    func_on_click = function () private.ShowTab( "info" ); end 
  } );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.SetSkinHandlers (
    func_skin_attach,
    func_skin_detach, 
    func_change_skin
  )

  private.FuncSkinAttach  = func_skin_attach;
  private.FuncSkinDetach  = func_skin_detach;
  private.FuncChangeSkin  = func_change_skin;

end;
--------------------------------------------------------------------------------
function public.SetAchievHandlers (
    func_get_achievement_reward,
    func_get_achievement_reward_data,
    func_ach_notif_count,
    func_open_gpgs_achievements
  )

  private.FuncAchGetReward       = func_get_achievement_reward;
  private.FuncGetAchRewardData   = func_get_achievement_reward_data;
  private.Ach_Notif_Count        = func_ach_notif_count;
  private.OpenGoogleAchievements = func_open_gpgs_achievements;

end;
--------------------------------------------------------------------------------
function public.SetInfoHandlers ( func_is_need_login, func_login )

  private.IsNeedLogin = func_is_need_login;
  private.Login       = func_login;

end;
--------------------------------------------------------------------------------
function public.SetGenericHandlers ( func_top_panel_toggle, func_close )

  private.TopPanel_Toggle = func_top_panel_toggle;
  private.FuncClose       = func_close;

end;
--------------------------------------------------------------------------------
function public.HasSelectButton ()

  local animfunc = m_object.Get( "anm_int_dialog_hero_window" ).animfunc;

  return ( animfunc:find( "select" ) );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.AchievementsTab_DisableScrolling ()

  local scroller    = "scc_int_dialog_hero_achievements_scroller";
  private.scrollpos = m_object.Get( scroller ).scrollpos;
  
  m_object.Set(
    scroller,
    {
      dragenabled = false
    }
  );
  
end;
--------------------------------------------------------------------------------
function public.AchievementsTab_EnableScrolling ()

  m_object.Set(
    "scc_int_dialog_hero_achievements_scroller",
    {
      dragenabled = true,
      scrollpos   = private.scrollpos
    }
  );
  
  private.scrollpos = nil;
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function  public.Show ( 
    tab_name,
    skins,
    achievements,
    statistic,
    focused_skin,
    skins_notified,
    skinparts_markers
  )

  private.skins.list        = skins;
  private.achievements.list = achievements;
  private.info.statistic    = statistic;

  private.skins.notified = skins_notified;
  private.skins.markers  = skinparts_markers;

  private.SkinsTab_MakeIndex();
  
  private.weared_skin  = private.GetWearedSkin( skins );
  private.focused_skin = focused_skin or private.weared_skin;

  private.ShowTab( tab_name );
  private.active = true;

  private.Tab_UpdateMarkers( "skins", #skins_notified );
  private.Tab_UpdateMarkers( "achievements", private.Ach_Notif_Count() );

  int_window.Show( "int_dialog_hero", function ()

    int.Dialog_Open( "hero", function ()

      -- Achievements notify
      if ( private.notify ) then
        
        mediator.Notify( private.notify.message, private.notify.data );
        private.notify = nil;
        
      else

        mediator.Notify( "Interface_DialogHero_Opened", tab_name );

      end;

    end );
  
  end );
  
end;
--------------------------------------------------------------------------------
function private.Hide ( func_execute )

  private.active = false;
  
  int.Dialog_Close( "hero", function ()

    int_window.Hide( "int_dialog_hero", func_execute );

  end );

end;
--------------------------------------------------------------------------------
function private.ShowTab ( tab )

  if ( tab == "skins" or tab == "achievements" or tab == "info" ) then
    
    local tab_data = private[ tab ];
    
    if ( private.tab_current == tab ) then
    
      tab_data.func_update();

      return;

    end;
    
    if ( private.active ) then cmn.PlaySnd( "tab_switch" ); end;

    private.TopPanel_Toggle( tab );
    private.ToggleTabsInput( tab );

    if not ( tab_data.tab_created ) then

      tab_data.func_create();
      tab_data.tab_created = true;

    end;

    tab_data.func_update();

  else

    DbgTrace( "[ INTERFACE ][ DIALOG HERO ] Unknown tab: " .. tostring( tab ) );

  end;

end;
--------------------------------------------------------------------------------
function private.ToggleTabsInput ( tab )

  local tab_prewious  = private.tab_current;
  private.tab_current = tab;

  if ( tab_prewious ~= tab ) then

    local btn = "btn_int_dialog_hero_tab_";

    if ( tab_prewious ) then
      ObjSet( btn..tab_prewious, { input = true  } );
    end;

    ObjSet( btn..tab,  { input = false } );

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.GetWearedSkin ( skins )

  for s = 1, #skins do
    if ( skins[ s ].weared ) then return skins[ s ].name; end;
  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Tab_UpdateMarkers ( tab, count )

  count = count or 0;

  local objects = {};

  local obj_ind = "obj_int_dialog_hero_tab_"..tab.."_notification";
  local txt_ind = "anm_int_dialog_hero_tab_"..tab.."_notification_bg";
  
  if ( count > 0 ) then

    table.insert( objects, { obj_ind, { active = true, visible = true } } );
    table.insert( objects, { txt_ind, { count  = tostring( count )    } } );

  else

    table.insert( objects, { obj_ind, { active = false, visible = false } } );

  end;

  ObjMultiSet( objects );
  
end;
--------------------------------------------------------------------------------
function private.Skin_UpdateMarker ( skin )

  local obj = "obj_int_dialog_skin_slot_notifications_"..skin;
  local cnt = "anm_int_dialog_skin_slot_notifications_bg_"..skin;

  local text;

  if ( cmn.IsEntityInList( private.skins.notified, skin ) ) then
    text = "!";
  end;

  if ( private.skins.markers[ skin ] ) then
    text = private.skins.markers[ skin ];
  end;

  local objects = {};

  if ( text ) then

    table.insert( objects, { obj, { active = true, visible = true } } );
    table.insert( objects, { cnt, { count  = tostring( text )     } } );

  else

    table.insert( objects, { obj, { active = false, visible = false } } );

  end;

  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.SkinsTab_MakeIndex ()

  local list = private.skins.list;
  local map  = private.skins.map;

  for id = 1, #list do

    local name = list[ id ].name;

    map.data[ name ] = list[ id ];
    map.id[ name ]   = id;

  end;

end;
--------------------------------------------------------------------------------
function private.SkinsTab_Create ()

  local skin_list = private.skins.list;
  local skin_map  = private.skins.map;
  local scc       = private.skins.scc;

  local scroller   = "scc_int_dialog_hero_skin_scroller";
  local item_dummy = "anm_int_dialog_hero_skin_slot";
  local spacer     = "obj_int_dialog_hero_skin_spacer";

  ObjClone( spacer, "_pre_1", true, scroller );
  ObjClone( spacer, "_pre_2", true, scroller );

  for s = 1, #skin_list do

    local skin = skin_list[ s ];
    local name = skin.name;

    ObjClone( item_dummy, "_"..name, true, scroller );

    int.Control_EventsSet( item_dummy.."_"..name,
    { 
      func_on_click = function () private.SkinsTab_SkinTap( name ) end
    } );

  end;

  ObjClone( spacer, "_post_1", true, scroller );
  ObjClone( spacer, "_post_2", true, scroller );

  -- Async skins list attach (start from focused skin)
  local id = skin_map.id[ private.focused_skin ];

  local ranged_list = {};
  table.insert( ranged_list, skin_list[ id ] );

  for i = 1, #skin_list do

    if ( skin_list[ id - i ] ) then
      table.insert( ranged_list, skin_list[ id - i ] );
    end;

    if  ( skin_list[ id + i ] ) then
      table.insert( ranged_list, skin_list[ id + i ] );
    end;

  end;

  local function atatch_list_item ( skin, progress )

    local skin_tag = "obj_int_dialog_skin_slot_hero_" .. skin.name;
    local skin_obj = private.FuncSkinAttach( skin_tag, "ride", skin.name );

    ObjSet( skin_obj, 
    {
      animfunc = "dialog",
      playing  = true
    } );

  end;

  common.ProcessArrayByTimer( ranged_list, atatch_list_item, nil, 0.05 );

  private.SkinsTab_AttachFocusedSkin();

end;
--------------------------------------------------------------------------------
function private.SkinsTab_Update ()

  private.SkinsTab_UpdateContainer();
  private.SkinsTab_Focus();

end;
--------------------------------------------------------------------------------
function private.SkinsTab_UpdateContainer ()

  local item_dummy = "anm_int_dialog_hero_skin_slot";

  local skin_list = private.skins.list;

  for s = 1, #skin_list do

    local skin     = skin_list[ s ];
    local name     = skin.name;
    local active   = skin.wearable;
    local selected = skin.weared;

    local animfunc = "skin_inactive";

    if     ( selected ) then animfunc = "skin_selected";
    elseif ( active   ) then animfunc = "skin_active";
    end;

    local item = item_dummy .. "_" .. name;
    AnimPlay( item, animfunc, "" );

    private.Skin_UpdateMarker( name );
  
  end;

end;
--------------------------------------------------------------------------------
function private.SkinsTab_Focus ()

  local skin_map  = private.skins.map;
  local focused   = private.skins.focus;

  local skin = private.focused_skin;

  private.SkinsTab_UpdateInfo();

  if ( focused.skin ~= skin ) then

    private.FuncSkinDetach( focused.obj );
    private.SkinsTab_AttachFocusedSkin();

  end;

  local skin_data  = skin_map.data[ skin ];
  local selectable = skin_data.wearable and not skin_data.weared;

  local af = "tab_1_"..private.app_o;
  if ( selectable ) then af = af .. "_select"; end;

  local id    = skin_map.id[ private.focused_skin ];
  local total = #private.skins.list;

  ObjSet( "anm_int_dialog_hero_window",
  {
    animfunc      = af,
    skins_counter = id .. "/" .. total,
    title         = StringGet( "str_int_dialog_hero_tab_skins" )
  } );

  local scrollpos = id - 1;
  ObjSet( "scc_int_dialog_hero_skin_scroller", { scrollpos = scrollpos } );

  private.SkinsTab_ScalesReset();
  private.SkinsTab_ScalesSet();

  AnimPlay( focused.obj, "skinview_fun1",
    function ()
      AnimPlay( focused.obj, "dialog", "" );
    end );

end;
--------------------------------------------------------------------------------
function private.SkinsTab_AttachFocusedSkin ()
  
  local parent    = "obj_int_dialog_hero_skin_hero";
  local skin_name = private.focused_skin;

  local focused   = private.skins.focus;

  focused.skin = skin_name;
  focused.obj  = private.FuncSkinAttach( parent, "ride", focused.skin );

  local grayscale = false;

  if not ( private.skins.map.data[ focused.skin ].wearable ) then
    grayscale = true;
  end;

  ObjSet( parent, { grayscale = grayscale } );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.SkinsTab_UpdateInfo ()

  local skin = private.focused_skin;

  if ( private.skins.markers[ skin ] ) then

    local to_step = private.skins.map.data[ skin ].collected_parts;
    local steps   = private.skins.map.data[ skin ].parts;

    local delta     = private.skins.markers[ skin ];
    local from_step = to_step - delta;

    private.SkinsTab_SetInfo();
    private.SkinsTab_AnimateInfo( from_step, to_step, steps );

  else

    private.SkinsTab_SetInfo();

  end;

end;
--------------------------------------------------------------------------------
function private.SkinsTab_SetInfo ()

  local info           = "anm_int_dialog_hero_skin_info";
  local str_prefix     = "str_skins_";
  local bonus_title_id = "str_int_dialog_hero_skins_bonus_ribbon";

  local skin      = private.focused_skin;
  local skin_data = private.skins.map.data[ skin ];

  local animfunc;
  if ( skin_data.wearable ) then
    animfunc = "skin_info_complete";
  else
    animfunc = "skin_info_progress";
  end;

  local progress_text = "";

  if ( skin_data.parts and skin_data.collected_parts and skin_data.parts > 0 ) then

    local parts = skin_data.collected_parts;
    local total = skin_data.parts;

    progress_text = parts .. "/" .. total;

    ObjSet( "spr_int_dialog_hero_skin_info_icon", 
    { 
      res = "assets/content/"..skin_data.content.."/"..skin.."_item"
    } );

    ObjSet( "anm_int_dialog_hero_skin_info_progress", 
    { 
      frame = parts / total * 120
    } );

  else

    animfunc = "skin_info_complete";

  end;

  ObjSet( info,
  {
    skin_name        = StringGet( str_prefix..skin ),
    skin_description = StringGet( str_prefix..skin.."_description" ),
    bonus_title      = StringGet( bonus_title_id ),
    bonus_text       = StringGet( str_prefix..skin.."_bonus" ),
    brief_text       = StringGet( str_prefix..skin.."_brief" ),
    progress_text    = progress_text,
    animfunc         = animfunc,
    playing          = true
  } );

  local id = cmn.IsEntityInList( private.skins.notified, skin );
  if ( id ) then

    table.remove( private.skins.notified, id );

  end;

  private.Tab_UpdateMarkers( "skins", #private.skins.notified );
  private.Skin_UpdateMarker( skin );

end;
--------------------------------------------------------------------------------
function private.SkinsTab_AnimateInfo ( from_step, to_step, steps )

  local inf = "anm_int_dialog_hero_skin_info";
  local bar = "anm_int_dialog_hero_skin_info_progress";

  AnimStop( inf );
  AnimPlay( inf, "skin_info_progress", "" );

  local anim_list = {};

  local delta    = to_step - from_step;
  local anm_time = 0.6 / delta;

  local function func_anim_step ( from, to, func_after )

    local pb_from = from / steps * 120;
    local pb_to   = to   / steps * 120;

    ObjSet( inf, { progress_text = from .. "/" .. steps } );

    ObjStopAnimate( bar, "frame" );
    ObjAnimate( bar, "frame", 0, 0, 
    function ()
      ObjSet( inf, { progress_text = to .. "/" .. steps } );
      if ( func_after ) then func_after(); end;
    end, 
    {
      0.0,      0, pb_from,
      anm_time, 0, pb_to
    } );

  end;

  for i = 1, delta do

    local from     =   from_step + i - 1;
    local to       =   from_step + i;

    local function func_step ( func_after )
      func_anim_step( from, to, func_after )
    end;

    table.insert( anim_list, func_step );

  end;
  
  common.RunCallbackList( anim_list );
  
  local af = "skin_info_action";
  if ( to_step == steps ) then   
  
    af = "skin_info_action_final"

  end;
  
  if ( private.focused_skin ~= "justknight" ) then
  
    private.skins.markers[ private.focused_skin ] = nil;
    private.Skin_UpdateMarker( private.focused_skin );
    
    AnimPlay( inf, af, private.SkinsTab_SetInfo );
    
  else
    
    local func_end = function()
      
      private.skins.markers[ private.focused_skin ] = nil;
      private.Skin_UpdateMarker( private.focused_skin );
      
      AnimPlay(
        "anm_int_dialog_hero_skin_info",
        "skin_info_action_final",
        private.SkinsTab_SetInfo
      );
      
    end;
    
    mediator.SubscribeOnce( "View_Tutorial_SkinProgressComplete", func_end );
    
  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.SkinsTab_DragStart ()

  private.SkinsTab_ScalesReset();

end;
--------------------------------------------------------------------------------
function private.SkinsTab_DragEnd ()

  cmn.PlaySnd( "button_click" );
  
  local id = ObjGet( "scc_int_dialog_hero_skin_scroller" ).scrollpos + 1;

  local skin_name = private.skins.list[ id ].name;

  if ( private.focused_skin ~= skin_name ) then

    private.focused_skin = skin_name;
    private.SkinsTab_Focus();

  else

    private.SkinsTab_ScalesSet();

  end;

end;
--------------------------------------------------------------------------------
function private.SkinsTab_SkinTap ( skin_name )

  if ( private.focused_skin == skin_name ) then

    return;

  end;

  private.focused_skin = skin_name;

  private.SkinsTab_Focus();

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.SkinsTab_ScalesReset ( func_after )

  local function func_after_safe ()

    if ( func_after ) then func_after() end;

  end;

  local scale_new = private.skins.scc.scales.far;
  local tme       = private.skins.scc.scale_time;

  local list = private.skins.list;

  for id, skin in pairs( list ) do

    local obj = "obj_int_dialog_skin_slot_hero_"..skin.name;

    ObjStopAnimate( obj, "scale_xy" );

    local scale = ObjGet( obj ).scale_x;

    ObjAnimate( obj, "scale_xy",  0, 0, func_after_safe,
    {
      0.0, 0, scale,     scale,
      tme, 0, scale_new, scale_new
    } );

  end;

end;
--------------------------------------------------------------------------------
function private.SkinsTab_ScalesSet ()

  local skin_list = private.skins.list;
  local skin_map  = private.skins.map;
  local scc       = private.skins.scc;
  local tme       = scc.scale_time;

  local scrollpos = ObjGet( "scc_int_dialog_hero_skin_scroller" ).scrollpos;

  local scales =
  {
    scc.scales.far, 
    scc.scales.near,
    scc.scales.focus,
    scc.scales.near,
    scc.scales.far
  };

  for i = 1, #scales do

    local skin_id = scrollpos + i - 2;

    if ( skin_id > 0 and skin_id <= #skin_list ) then

      local name = skin_list[ skin_id ].name;
      local obj  = "obj_int_dialog_skin_slot_hero_"..name;

      ObjStopAnimate( obj, "scale_xy" );

      local scale = ObjGet( obj ).scale_x;

      ObjAnimate( obj, "scale_xy",  0, 0, "",
      {
        0.0, 0, scale,       scale,
        tme, 0, scales[ i ], scales[ i ]
      } );

    end;

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.SkinsTab_SkinWear ()

  private.FuncChangeSkin( private.focused_skin );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Skins_Update ( skins )

  private.skins.list  = skins;
  private.weared_skin = private.GetWearedSkin( skins );

  private.SkinsTab_MakeIndex();
  private.SkinsTab_Update();

end;
--------------------------------------------------------------------------------
function public.Skins_Focus ( skin_name, func_end )

  if ( func_end ) then

    func_end();

  end;

  if ( private.focused_skin == skin_name ) then

    return;

  end;

  private.focused_skin = skin_name;

  private.ShowTab( "skins" );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.AchievementsTab_Create ()

  local scc    = "scc_int_dialog_hero_achievements_scroller";
  local slot   = "anm_int_dialog_hero_achievements_slot";
  local reward = "anm_int_dialog_hero_achievements_slot_reward";
  local button = "btn_int_dialog_hero_achievements_slot_button";

  local str_ach = "str_achievement_";
  local str_dlg = "str_int_dialog_hero_achievement_";

  local multiset = {};

  for i, achievement in pairs( private.achievements.list ) do

    local name   = achievement.name;
    local suffix = "_"..name;

    private.achievements.id[ name ] = i;

    ObjClone( slot, suffix, true, scc );

    local item        =   slot..suffix;
    local item_reward = reward..suffix;
    local item_button = button..suffix;

    int.Button_Create( item_button,
      {
        title  = str_dlg.."get_reward",
        design = "medium",
        color  = "green"
      },
      function () end 
    );

    table.insert( multiset, 
    { item, 
      { achievement_name = StringGet( str_ach..name ) }
    } );

    table.insert( multiset, 
    { item_reward, 
      { 
        reward_label = StringGet( str_dlg.."reward" ),
        tt_complete  = StringGet( str_dlg.."complete" )
      } 
    } );

  end;

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
function private.AchievementsTab_Update ()

  for _, achievement in pairs( private.achievements.list ) do

    private.AchievementsTab_UpdateItem( achievement )

  end;

  ObjSet( "anm_int_dialog_hero_window",
  {
    animfunc = "tab_2_"..private.app_o,
    title    = StringGet( "str_int_dialog_hero_tab_achievements" )
  } );

end;
--------------------------------------------------------------------------------
function private.AchievementsTab_UpdateItem ( achievement )

  local scc    = "scc_int_dialog_hero_achievements_scroller";

  local slot   = "anm_int_dialog_hero_achievements_slot";
  local reward = "anm_int_dialog_hero_achievements_slot_reward";
  local icon   = "anm_int_dialog_hero_achievements_slot_icon";
  local button = "btn_int_dialog_hero_achievements_slot_button";

  local str_ach = "str_achievement_";
  local str_dlg = "str_int_dialog_hero_achievement_";

  local name   = achievement.name;
  local suffix = "_"..name;

  local item        =   slot..suffix;
  local item_reward = reward..suffix;
  local item_icon   =   icon..suffix;
  local item_button = button..suffix;

  local level     = achievement.level;
  local level_max = 3; 

  local reward_level;
  for i, reward in pairs( achievement.reward ) do

    if ( reward.get and not reward.done ) then
      reward_level = i;
      break;
    end;

  end;

  local item_animfunc, description_id, progress, progress_text;
  local icon_animfunc, reward_animfunc;

  if ( reward_level ) then
   
    int.Button_Enable( item_button );

    level          = reward_level;
    item_animfunc  = "achievement_slot_button";
    description_id = str_ach..name.."_description_"..level;

    int.Button_EventSet( item_button,
      function ()
        
        int.Button_Disable( item_button );

        local count       = achievement.levels[ level ].reward_count;
        local reward_type = achievement.levels[ level ].reward_type;

        private.FuncAchGetReward( name, reward_type, count, level );
        achievement.reward = private.FuncGetAchRewardData( name );
        private.AchievementsTab_AnimateItem( achievement, 
                                             item, item_reward, item_icon,
                                             level );

        private.Tab_UpdateMarkers( "achievements", private.Ach_Notif_Count() );
        
        cmn.PlaySnd( "achievement_reward_click" );

      end );

    private.notify =
    { 
      message = "Interface_Hero_AchievementUpdated", 
      data    = name 
    };

  elseif ( level > level_max ) then

    item_animfunc  = "achievement_slot_complete";
    description_id = "str_achievement_complete_description";

    icon_animfunc   = "icon_state"   .. level;
    reward_animfunc = "reward_state" .. level;

  else

    item_animfunc  = "achievement_slot_progress";
    description_id = str_ach..name.."_description_"..level;

    local step  = achievement.count;
    local steps = achievement.levels[ level ].value;

    progress = step / steps * 120;
    progress_text = int.NumberForm( step ).."/"..int.NumberForm( steps );

  end;

  local icon_animfunc   = "icon_state"   .. level;
  local reward_animfunc = "reward_state" .. level;

  local levels       = achievement.levels;
  local reward_count = levels[ math.min( level, level_max ) ].reward_count;
  local value        = levels[ math.min( level, level_max ) ].value;

  local multiset = {};

  table.insert( multiset, 
  { item, 
    { 
      animfunc         = item_animfunc,
      frame            = progress,
      progress_text    = progress_text,
      achievement_desc = StringGet( description_id, value )
    }
  } );

  table.insert( multiset, 
  { item_icon, 
    { 
      animfunc = icon_animfunc,
      playing  = true
    }
  } );

  table.insert( multiset, 
  { item_reward, 
    { 
      animfunc     = reward_animfunc,
      reward_count = int.NumberForm( reward_count )
    }
  } );

  -- Focus at achievement with reward
  if ( reward_level ) then

    local list = private.achievements.list;
    local id   = private.achievements.id[ name ] or 0;

    local pos  = math.max( id - 3, 0 );

    table.insert( multiset, { scc, { scrollpos = pos } } );

  end;

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
function private.AchievementsTab_AnimateItem ( achievement, 
                                               item, item_reward, item_icon,
                                               level )

  local suffix = level.."_"..( level + 1 );

  local icon_func   = "icon_state"..suffix
  local reward_func = "reward_state"..suffix;

  local icon_func_next = "icon_state"..( level + 1 );
  
  AnimPlay( item_icon,   icon_func,
    function ()

        AnimPlay( item_icon, icon_func_next, "" );
        AnimPlay( item, "achievement_slot_action",
        function ()

          AnimPlay( item_reward, reward_func,
            function ()
              private.AchievementsTab_UpdateItem( achievement );
            end );

        end );

    end );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.InfoTab_Create ()

  private.InfoTab_AttachWearedSkin();

end;
--------------------------------------------------------------------------------
function private.InfoTab_Update ()

  local stat = private.info.statistic;

  local ach     = stat.achievements.completed_count;
  local ach_tot = stat.achievements.total_count;
  local quests  = stat.quests;
  local stg     = stat.stages.completed_count;
  local stg_tot = stat.stages.total_count - stat.stages.bonus_total;

  local id  = stat.id;
  local str = "str_int_dialog_hero_info_";
  
  local id_label = tern( id == "0" )(
    function() return ne.strings.Get( str.."id_offline" ); end,
    function() return ne.strings.Get( str.."id", id ); end
  )();

  m_object.Set(
    "anm_int_dialog_hero_info_data",
    {
      achievements_label   = StringGet( str.."achievements" ),
      achievements_counter = StringGet( str.."achievements_value", ach, ach_tot ),
      quests_label         = StringGet( str.."quests" ),
      quests_counter       = StringGet( str.."quests_value", quests ),
      levels_label         = StringGet( str.."levels" ),
      levels_counter       = StringGet( str.."levels_value", stg, stg_tot ),
      player_id            = id_label;
    }
  );

  if ( private.info.skin ~= private.weared_skin ) then

    private.FuncSkinDetach( private.info.skin_obj );
    private.InfoTab_AttachWearedSkin();

  end;

  local animfunc = "tab_3_"..private.app_o

  if ( private.IsNeedLogin() ) then

    animfunc = animfunc .. "_fb";

  end;

  ObjSet( "anm_int_dialog_hero_window",
  {
    animfunc      = animfunc,
    title         = StringGet( "str_int_dialog_hero_tab_info" )
  } );

  AnimPlay( private.info.skin_obj, "skinview_fun1",
    function ()
      AnimPlay( private.info.skin_obj, "dialog", "" );
    end );

end;
--------------------------------------------------------------------------------
function private.InfoTab_AttachWearedSkin ()

  local skin_name = private.weared_skin;
  local parent = "obj_int_dialog_hero_info_skin";

  private.info.skin     = skin_name;
  private.info.skin_obj = private.FuncSkinAttach( parent, "ride", skin_name );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.InfoTab_Login ()

  private.Login();
  mediator.SubscribeOnce( "Model_Social_LoginComplete", 
                          private.InfoTab_Update );

end;
--------------------------------------------------------------------------------