-- name=int_dialog_info
--------------------------------------------------------------------------------
function public.Init ()

  int_control.Add( "int_dialog_info" );

  private.Adapt();
  private.CreateButtons();  

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  local app_w = int.W();
  local app_h = int.H();

  private.this = "obj_int_dialog_info";

  private.window_area = {
    default  = { -272, -300, 544, 600 }
  };

  int_window.SetArea( private.this, private.window_area.default );

  ObjMultiSet( {
    { private.this, {
      pos_x = 0.5 * app_w,
      pos_y = 0.5 * app_h
    } },
    { "spr_int_dialog_info_bg_black", {
      scale_x = app_w / 64,
      scale_y = app_h / 64,
    } }
  } );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  int.Button_Create(
    "btn_int_dialog_info_close",
    {
      design = "close"
    },
    function() private.FuncClose(); end
  );
  
  int.Button_Create(
    "btn_int_dialog_info_accept",
    {
      color  = "green",
      design = "medium",
      timer  = "down",
      info   = true
    },
    function () private.FuncOk(); end
  );
  
  int.Button_Create(
    "btn_int_dialog_info_accept_alt",
    {
      color  = "green",
      design = "medium",
      timer  = "down",
      info   = true
    },
    function () private.FuncAltOk(); end
  );    
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( message_type, func_accept, func_reject, params )

  int_control.Attach( "int_dialog_info" );
  
  private.active = true;

  private.FuncAccept = func_accept;
  private.FuncReject = func_reject;
  
  private.FuncClose = public.Hide;
  private.FuncOk = function ()
    private.FuncAccept(); 
    private.FuncClose();
  end;
  private.FuncAltOk = func_reject;

  local button_text = "str_dialog_info_button_"..message_type;
  local message     = "str_dialog_info_message_"..message_type;
  local animfunc    = "portrait";
  
  ObjMultiSet(
    {
      {
        "anm_int_dialog_info",
        {
          attention = message,
          title     = "str_dialog_info_caption"
        }
      },
      {
        "btn_int_dialog_info_accept",
        {
          active = true,
          input  = true,
          visible = true,
          anim_tag = "btn_accept"
        }
      },
      {
        "btn_int_dialog_info_accept_alt",
        {
          active = false,
          input = false,
          visible = false
        }
      }
    }
  );

  cmn.PlaySnd( "app_info" );

  int.Button_SetText( "btn_int_dialog_info_accept", button_text );
  
  local multiset_table = {};

  int.Button_SetTime( "btn_int_dialog_info_accept", false, multiset_table );
  int.Button_SetTime( "btn_int_dialog_info_accept_alt", false, multiset_table );
  int.Button_SetInfo( "btn_int_dialog_info_accept", false, multiset_table );
  int.Button_SetInfo( "btn_int_dialog_info_accept_alt", false, multiset_table );
  
  if ( message_type == "connectionlost" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = false,
      visible = false,
      input   = false 
    } } );

  elseif ( message_type == "newserver" ) then

    button_text = "str_dialog_info_message_"..message_type.."_"..( params.state or "old" );
    
    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = false,
      visible = false,
      input   = false } } );

    private.FuncOk = function () private.FuncReject(); end;

  elseif ( message_type == "savedelete" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true } } );

    private.FuncOk = function () private.FuncAccept(); end;

  elseif ( message_type == "contentobsolete" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = false,
      visible = false,
      input   = false } } );

    private.FuncOk = function () private.FuncAccept(); end;

  elseif ( message_type == "playhaven_notify" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true } } );

    private.FuncOk = function () private.FuncAccept(); private.FuncClose(); end;
      
  elseif ( message_type == "exitconfirm" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true } } );

    private.FuncOk = function () private.FuncAccept(); end;
    
    if ( func_reject ) then
    
      private.FuncClose = func_reject;
    
    end;
      
    elseif ( message_type == "languageconfirm" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true } } );

    private.FuncOk = function () private.FuncAccept(); end;      
      
  elseif ( message_type == "widescreen_map" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true } } );

    private.FuncOk = function () private.FuncAccept(); end;
      
  elseif ( message_type == "widescreen_stage" ) then

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true } } );

    private.FuncOk = function () private.FuncAccept(); end;
 
  elseif ( message_type == "prioritysession" ) then
      
    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = false,
      visible = false,
      input   = false 
    } } );    

  elseif ( message_type == "conflictstorage" ) then
    
    animfunc = "portrait_two_btn";
    
    table.insert( multiset_table, { "btn_int_dialog_info_accept",
    {
      anim_tag = "at_btn_accept_1"
    } } );   
    
    table.insert( multiset_table, { "anm_int_dialog_info",
    {
      title  = message.."_title"
    } } );       
      
    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = false,
      visible = false,
      input   = false 
    } } );
      
    table.insert( multiset_table, { "btn_int_dialog_info_accept_alt",
    {
      active  = true,
      visible = true,
      input   = true 
    } } );      

    private.FuncOk = private.FuncAccept;
    private.FuncAltOk = private.FuncReject;
    
    int.Button_SetTime( "btn_int_dialog_info_accept", params.btn1[ 1 ], multiset_table );
    int.Button_SetTime( "btn_int_dialog_info_accept_alt", params.btn2[ 1 ], multiset_table );
    int.Button_SetText( "btn_int_dialog_info_accept", button_text.."1", multiset_table );
    int.Button_SetText( "btn_int_dialog_info_accept_alt", button_text.."2", multiset_table );
    
  elseif ( message_type == "conflictstorage_local" ) or 
         ( message_type == "conflictstorage_server" ) then
         
    table.insert( multiset_table, { "anm_int_dialog_info",
    {
      title  = message.."_title"
    } } );             
      
    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true 
    } } );

    private.FuncOk = function ()
      private.FuncAccept(); 
      public.Hide();
    end;    
    private.FuncClose = private.FuncReject;
      
  else

    table.insert( multiset_table, { "btn_int_dialog_info_close",
    {
      active  = true,
      visible = true,
      input   = true } } );

    private.FuncOk = function () private.FuncClose(); end;

  end;

  ObjMultiSet( multiset_table );
  
  AnimPlay( "anm_int_dialog_info", animfunc, "" );
  
end;
--------------------------------------------------------------------------------
function public.Hide ( func_end )

  ObjDelete( "tmr_dialog_info" );
  ObjDelete( "tmr_dialog_info_send" );
    
  int_control.Detach( "int_dialog_info" );
  
  private.active = false;

  cmn.PlaySnd( "ok_info" );

  if ( func_end ) then

    func_end();

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.IsActive ()

  return private.active;

end;
--------------------------------------------------------------------------------
function public.GetCloseFunc ()

  local btn_close = m_object.Get( "btn_int_dialog_info_close" ) or {};

  if ( btn_close.input ) then
  
    return private.FuncClose;
  
  end;
  
  return nil;

end;
--------------------------------------------------------------------------------
function public.GetOkFunc ()

  local btn_altok = m_object.Get( "btn_int_dialog_info_accept_alt" ) or {};
  
  if ( btn_altok.input ) then
    return nil;
  end;
  
  if ( private.FuncOk ) then
    return private.FuncOk;
  end;
  
  return nil;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.ShowLikeSysMsg ( params )

  int_control.Attach( "int_dialog_info" );
  
  private.active = true;
  
  local title     = params.title;
  local text      = params.body;
  local btn_left  = params.buttons[ 1 ];
  local btn_right = params.buttons[ 2 ];
  
  local animfunc = "portrait";
  local at_left  = "btn_accept";
  
  local set = {};
  
  local obj =
  {
    btn_close = "btn_int_dialog_info_close",
    btn_left  = "btn_int_dialog_info_accept",
    btn_right = "btn_int_dialog_info_accept_alt",
    dialog    = "anm_int_dialog_info"
  };
  
  -- Setup buttons
  int.Button_Disable( obj.btn_close, set );
  int.Button_Disable( obj.btn_right, set );
  
  int.Button_Enable( obj.btn_left, set );
  int.Button_SetText( obj.btn_left, btn_left.title, set );
  int.Button_SetTime( obj.btn_left, btn_left.params[ 1 ] or false, set );
  int.Button_SetInfo( obj.btn_left, btn_left.params[ 2 ] or false, set );
  
  private.FuncOk = function () public.Hide( btn_left.action ); end;
  
  if ( btn_right ) then
  
    int.Button_Enable( obj.btn_right, set );
    int.Button_SetText( obj.btn_right, btn_right.title, set );
    int.Button_SetTime( obj.btn_right, btn_right.params[ 1 ] or false, set );
    int.Button_SetInfo( obj.btn_right, btn_right.params[ 2 ] or false, set );
    
    private.FuncAltOk = function () public.Hide( btn_right.action ); end;
    
    animfunc = "portrait_two_btn";
    at_left  = "at_btn_accept_1";
  
  end;
  
  m_object.Set( obj.btn_left, { anim_tag = at_left }, set );
  
  -- Setup dialog
  m_object.Set(
    obj.dialog,
    {
      title     = title,
      attention = text,
      animfunc  = animfunc,
      playing   = true
    },
    set
  );
  
  m_object.Multiset( set );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.CheckConnection ()

  local timer = "tmr_dialog_info_send";

  ObjCreate( timer, "timer" );
  ObjAttach( timer, "obj_int_dialog_info" );

  ObjSet( timer,
  {
    time = 1.0,
    playing = true,
    endtrig = function()

      ObjDelete( timer );

      private.FuncAccept();
      private.CheckConnection();
      
    end;
  } );

end;
--------------------------------------------------------------------------------
function private.ShowMessage ( message_type )

  ObjSet( "anm_int_dialog_info", { attention = "str_dialog_info_message_wait" } );
  ObjSet( "btn_int_dialog_info_accept", { active = false, input = false, visible = false } );
  
  local timer = "tmr_dialog_info";

  ObjCreate( timer, "timer" );
  ObjAttach( timer, "int_dialog_info" );

  ObjSet( timer,
  {
    time = 10.0,
    playing = true,
    endtrig = function()

      ObjDelete( timer );
      ObjDelete( "tmr_dialog_info_send" );
      ObjSet( "anm_int_dialog_info", { attention = "str_dialog_info_message_"..message_type } );
      ObjSet( "btn_int_dialog_info_accept", { active = true, input = true, visible = true } );

    end;
  } );

end;
--------------------------------------------------------------------------------