-- name=int_dialog_ingame_friends
function  public.__InitModule ()

  private.app_w = int.W();
  private.app_h = int.H();
  private.app_o = int.Orientation();
  
  private.friends = 
  {
    list = {},
    
    scc  =
    {
      size =
      { 
        portrait  = { w = 600, h = 620 }, 
        landscape = { w = 600, h = 620 } 
      }
    },   
    
    tab_created = false,
    func_create = private.FriendsTab_Create,
    func_update = private.FriendsTab_Update
  };
  
  private.players = 
  {
    
    list = {},
    
    scc  =
    {
      size =
      { 
        portrait  = { w = 600, h = 620 }, 
        landscape = { w = 600, h = 620 } 
      }
    },    
    
    tab_created = false,
    func_create = private.PlayersTab_Create,
    func_update = private.PlayersTab_Update
  };  
  
  private.window_area = {
    [ "portrait"  ] = { -320, -420, 640, 900 },
    [ "landscape" ] = { -250, -320, 500, 700 }
  };
  
end;
--------------------------------------------------------------------------------
function  public.__InitObjects ()

  int_window.SetArea( "obj_int_dialog_ingame_friends", 
                    private.window_area[ private.app_o ] );
  
  local multiset = {};

  -- Setup static text tags
  local str_prefix = "str_int_dialog_ingame_friends";

  local window = "anm_int_dialog_ingame_friends_window";
  local prefix = "assets/content/interface/int_dialog_ingame_friends/";

  local res =
  {
    [ "portrait"  ] = prefix.."anm_int_dialog_ingame_friends",
    [ "landscape" ] = prefix.."anm_int_dialog_ingame_friends"
  }

  local window_params = {
    res         = res[ private.app_o ],
    friends = str_prefix.."_tab_friends",
    players = str_prefix.."_tab_players"    
  };

  table.insert( multiset, { window, window_params } );

  -- Setup tab buttons and indicators positions
  table.insert( multiset, { "anm_int_dialog_ingame_friends_tabs", 
  {
    res      = res[ private.app_o ],
    animfunc = "tabs_"..private.app_o,    
    playing  = true
  } } );

  -- Centering dialog
  table.insert( multiset, { "obj_int_dialog_ingame_friends", {
    pos_x       = private.app_w / 2,
    pos_y       = private.app_h / 2
  } } );

  local friends_scc = private.friends.scc.size[ private.app_o ];

  table.insert( multiset, { "scc_int_dialog_ingame_friends_friends_scroller",
  {
    input          = false,
    inputrect_init = true,
    inputrect_x   = - friends_scc.w / 2,
    inputrect_y   = - friends_scc.h / 2,
    inputrect_w   =   friends_scc.w,
    inputrect_h   =   friends_scc.h,

    croprect_init = true,
    croprect_x    = - friends_scc.w / 2,
    croprect_y    = - friends_scc.h / 2,
    croprect_w    =   friends_scc.w,
    croprect_h    =   friends_scc.h
  } } );
  
  local players_scc = private.players.scc.size[ private.app_o ];
  
  table.insert( multiset, { "scc_int_dialog_ingame_friends_players_scroller",
  {
    input          = false,
    inputrect_init = true,
    inputrect_x   = - players_scc.w / 2,
    inputrect_y   = - players_scc.h / 2,
    inputrect_w   =   players_scc.w,
    inputrect_h   =   players_scc.h,

    croprect_init = true,
    croprect_x    = - players_scc.w / 2,
    croprect_y    = - players_scc.h / 2,
    croprect_w    =   players_scc.w,
    croprect_h    =   players_scc.h
  } } );  

  ObjMultiSet( multiset );
  
  int.Button_Create(
    "btn_int_dialog_ingame_friends_close",
    {
      design = "close"
    },
    function () private.Hide( private.FuncClose ); end 
  );
  
  int.Button_Create(
    "btn_int_dialog_ingame_friends_login",
    {
      title  = str_prefix.."_tab_friends_login",
      design = "facebook"
    },
    function () private.FuncLogin() end 
  );

  int.Control_EventsSet( "btn_int_dialog_ingame_friends_tab_friends", 
  { 
    func_on_click = function () private.ShowTab( "friends" ); end 
  } );

  int.Control_EventsSet( "btn_int_dialog_ingame_friends_tab_players", 
  { 
    func_on_click = function () private.ShowTab( "players" ); end 
  } );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function  public.Show ( 
    tab_name,
    friends,
    players,
    func_login,
    func_addfriend
  )
  
  private.friends.list  = friends;
  private.players.list  = players;
  private.FuncLogin     = func_login;
  private.FuncAddFriend = func_addfriend;
  
  private.active = true;

  int_window.Show( "int_dialog_ingame_friends", function ()
    
    private.ShowTab( tab_name );

    int.Dialog_Open( "ingame_friends", function () end );
  
  end );
  
end;
--------------------------------------------------------------------------------
function private.Hide ( func_execute )

  private.active = false;
  
  int.Dialog_Close( "ingame_friends", function ()

    int_window.Hide( "int_dialog_ingame_friends", func_execute );

  end );

end;
--------------------------------------------------------------------------------
function private.ShowTab ( tab )

  if ( tab == "friends" or tab == "players" ) then
    
    local tab_data = private[ tab ];
    
    if ( private.tab_current == tab ) then
    
      tab_data.func_update();

      return;

    end;
    
    if ( private.active ) then cmn.PlaySnd( "tab_switch" ); end;

    private.ToggleTabsInput( tab );

    if not ( tab_data.tab_created ) then

      tab_data.func_create();
      tab_data.tab_created = true;

    end;

    tab_data.func_update();

  else

    DbgTrace( "[ INTERFACE ][ DIALOG INGAME FRIENDS ] Unknown tab: " .. tostring( tab ) );

  end;

end;
--------------------------------------------------------------------------------
function private.ToggleTabsInput ( tab )

  local tab_previous  = private.tab_current;
  private.tab_current = tab;

  if ( tab_previous ~= tab ) then

    local btn = "btn_int_dialog_ingame_friends_tab_";
    local scc = "scc_int_dialog_ingame_friends_";
    
    if ( tab_previous ) then
      ObjSet( btn..tab_previous,              { input = true  } );
      ObjSet( scc..tab_previous.."_scroller", { input = false } );
    end;

    ObjSet( btn..tab,              { input = false } );
    ObjSet( scc..tab.."_scroller", { input = true  } );

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.FriendsTab_Create ()

  local scc    = "scc_int_dialog_ingame_friends_friends_scroller";
  local slot   = "int_dialog_ingame_friends_friends_slot";

  local multiset = {};

  local hub_index = 1;
  
  for i, friend in pairs( private.friends.list ) do

    local id   = friend.userId;
    local suffix = "_"..id;

    hub_index = tern( i > 2 and ( i % 2 ) == 1 )( hub_index + 1, hub_index ); 
    local hub_name = "obj_int_dialog_ingame_friends_friends_hub_"..hub_index;
    
    if ( not ObjGet( hub_name ) ) then
      
      ObjCreate( hub_name, "obj" );
      ObjAttach( hub_name, scc );
      
    end;   
    
    ObjClone( "anm_"..slot, suffix, true, hub_name );

    local friend_slot   = "anm_"..slot..suffix;
    local friend_avatar = "spr_"..slot.."_avatar"..suffix;

    table.insert( multiset, 
    { friend_slot, 
      { pos_x = ( - 1 ) ^ ( i % 2 ) * 150,
        player  = friend.name,
        section = StringGet( "str_int_dialog_worldmap_"..( friend.section or "section_1" ) ) }
    } );

    table.insert( multiset, 
    { friend_avatar, 
      { url         = friend.url,
        res         = friend.res,
        default_res = friend.res,
        trg_resloaded = function ()

          int.ImageAdjust( friend_avatar, 74 );

        end }
    } );

  end;

  if ( #multiset > 0 ) then
    
    ObjMultiSet( multiset ); 
    
  end; 

end;
--------------------------------------------------------------------------------
function private.FriendsTab_Update ()
  
  local scc  = "scc_int_dialog_ingame_friends_friends_scroller";
  local slot = "int_dialog_ingame_friends_friends_slot";

  for _, friend in pairs( private.friends.list ) do

    local friend_avatar = "spr_"..slot.."_avatar_"..friend.userId;
    int.ImageAdjust( friend_avatar, 74 );

  end;
  
  local animfunc = "tab_friends_"..private.app_o;
  animfunc = tern( #private.friends.list == 0 )( "tab_nofriends_"..private.app_o, animfunc );
  animfunc = tern( private.FuncLogin )( "tab_friends_fb_"..private.app_o, animfunc );
 
  ObjSet( scc,
  {
    input = true
  } ); 
  
  ObjSet( "anm_int_dialog_ingame_friends_window",
  {
    animfunc = animfunc,
    count_header = "str_int_dialog_ingame_friends_tab_friends_counter",
    counter = #private.friends.list,
    title    = "str_int_dialog_ingame_friends_tab_friends"
  } );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.PlayersTab_Create ()

  local scc    = "scc_int_dialog_ingame_friends_players_scroller";
  local slot   = "int_dialog_ingame_friends_players_slot";

  local multiset = {};
  
  for i, player in pairs( private.players.list ) do
    
    local id   = player.userId;
    local suffix = "_"..id;
    
    ObjClone( "anm_"..slot, suffix, true, scc );

    local player_slot   = "anm_"..slot..suffix;
    local player_avatar = "spr_"..slot.."_avatar"..suffix;

    table.insert( multiset, 
    { player_slot, 
      { pos_y = ( i - 1 ) * 120,
        player  = player.name,
        section = StringGet( "str_int_dialog_worldmap_"..( player.section or "section_1" ) ) }
    } );

    table.insert( multiset, 
    { player_avatar, 
      { url         = player.url,
        res         = player.res,
        default_res = player.res,
        trg_resloaded = function ()

          int.ImageAdjust( player_avatar, 74 );

        end } 
    } ); 

  end;

  if ( #multiset > 0 ) then
    
    ObjMultiSet( multiset );     
    
  end; 

  for _, player in pairs( private.players.list ) do

    int.Button_Create(
    "btn_int_dialog_ingame_friends_players_slot_add_"..player.userId,
      {
        title  = "str_int_dialog_ingame_friends_tab_players_add",
        design = "medium",
        color  = "green"
      },
      function () private.FuncAddFriend( player.userId ) end 
    );     

  end;

  
end;
--------------------------------------------------------------------------------
function private.PlayersTab_Update ()
  
  local scc  = "scc_int_dialog_ingame_friends_players_scroller";
  local slot = "int_dialog_ingame_friends_players_slot";

  for _, player in pairs( private.players.list ) do

    local player_avatar = "spr_"..slot.."_avatar_"..player.userId;
    int.ImageAdjust( player_avatar, 74 );

  end;  

  local animfunc = "tab_players_"..private.app_o;

  ObjSet( scc,
  {
    input = true
  } );   

  ObjSet( "anm_int_dialog_ingame_friends_window",
  {
    animfunc = animfunc,
    tab_description = "str_int_dialog_ingame_friends_tab_players_description",
    title    = "str_int_dialog_ingame_friends_tab_players"
  } );  
  
end;