-- name=int_dialog_items_chest
--------------------------------------------------------------------------------
function  public.__InitModule ()
  
  private.window = "anm_int_dialog_items_chest_window";
  private.chest  = "anm_int_dialog_items_chest";

  private.chest_types =
  {
    [ "woodchest"   ] = "wooden",
    [ "silverchest" ] = "silver",
    [ "goldchest"   ] = "gold",
    [ "epicchest"   ] = "epic"
  };
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.Adapt();
  private.CreateButtons();
    
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Show (
    chest_name,
    func_GetRewards,
    func_CountRewards,
    func_OnHide,
    func_OnShow
  )

  private.type = private.chest_types[ chest_name ];
  
  private.GetRewards   = func_GetRewards;
  private.CountRewards = func_CountRewards;
  private.OnHide       = func_OnHide;
  
  private.SetInput( "none" );
  
  m_object.Set( 
    private.chest,
    {
      reward = ne.strings.Get( "str_dialog_items_"..chest_name )
    }
  );
  
  AnimPlay( private.window, "open_medium", function ()
   
    AnimPlay( private.window, "static_medium", "" );
    
    cmn.PlaySnd( "items_chest_appear" );

    AnimPlay( private.chest, private.type.."_appear", function ()
    
      AnimPlay( private.chest, private.type.."_static_closed", "" );

      private.SetInput( "chest" );
      
      cmn.PlaySnd( "items_chest_waiting", 1 );

      if ( func_OnShow ) then

        func_OnShow();

      end;
      
    end );
    
  end );
  
  private.active = true;
  
end;
--------------------------------------------------------------------------------
function  public.Hide ( func_OnHide )

  if not ( private.active ) then

    return;

  end;
  
  private.active = false;
 
  private.Cleanup();
  
  if ( func_OnHide ) then func_OnHide(); end;
 
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  local w, h = int.W(), int.H();

  private.this = "obj_int_dialog_items_chest";

  local multiset = {};
  
  int.Center_Object( private.this, nil, nil, multiset );

  local biggest = int.Dimension_Max();

  private.window_area = {
    default  = { -0.5 * biggest, -0.5 * biggest, biggest, biggest }
  };
  
  int_window.SetArea(
    private.this,
    private.window_area.default,
    multiset
  );
  
  m_object.Set(
    "obj_int_dialog_items_chest_input",
    {
      inputrect_init = true,
      --inputrect_x    = 0,
      --inputrect_y    = 0,
      inputrect_w    = w,
      inputrect_h    = h
    },
    multiset
  );

  m_object.Set( 
    private.chest,
    {
      click  = ne.strings.Get( "str_dialog_items_chest_tap" )
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  int.Button_Create( "btn_int_dialog_items_chest_claim", 
  {
    title       = "str_dialog_items_chest_claim",
    color       = "green",
    design      = "medium"
  },
  function()
    
    private.Tap_Button();
    
  end );
  
  common.TapSet( "obj_int_dialog_items_chest_input",
  {
    func_true = function()

      private.Tap_Chest();

    end
  } );

  for i = 1, 3 do

    common.TapSet( "obj_int_dialog_items_chest_"..i,
    {
      func_true = function()

        private.Tap_Reward( i );

      end
    } );

  end;

end;
--------------------------------------------------------------------------------
function private.SetRewards ()

  private.reward_count = 3;
  
  local rewards    = private.GetRewards();
  local objects    = {};
  local txt_params = {};
  
  for r, reward in ipairs( rewards ) do
    
    local icon_type = "spr";
    local scale     = 1.0;
    
    if ( reward.name ~= "gold"     ) then scale     = 0.7;   end;
    if ( reward.type == "resource" ) then icon_type = "anm"; end;
      
    ObjClone(
      icon_type.."_int_dialog_items_chest_icon",
      "_"..r, 0,
      "obj_int_dialog_items_chest_"..r
    );
    
    if ( icon_type == "spr" ) then
    
      table.insert( objects,
      { "spr_int_dialog_items_chest_icon_"..r,
        {
          res     = common.GetItemResource( reward.type, reward.name ),
          scale_x = scale,
          scale_y = scale } } );
      
    else
      
      table.insert( objects,
      { "anm_int_dialog_items_chest_icon_"..r,
        {
          name      = "spr_int_dialog_items_chest_icon_"..r,
          animfunc  = "static_"..reward.name,
          playing   = true } } );
      
    end;
    
    if ( reward.count > 1 ) then

      txt_params[ "txt_reward_"..r ] = "#"..tostring( reward.count );

    else

      txt_params[ "txt_reward_"..r ] = "";

    end;
    
  end;
  
  table.insert( objects, { "anm_int_dialog_items_chest", txt_params } );
  
  ObjMultiSet( objects, "int_dialog_items_chest.SetRewards" );
  
end;
--------------------------------------------------------------------------------
function private.SetInput ( target )
  
  local multiset = {};
  
  local window = "anm_int_dialog_items_chest_window";
  local input  = "obj_int_dialog_items_chest_input";
  local button = "btn_int_dialog_items_chest_claim";
  
  if     ( target == "none" ) then
    
    int.Input_Off( window, multiset );
    int.Input_Off( input, multiset );
      
  elseif ( target == "chest" ) then
  
    int.Input_On( window, multiset );
    int.Input_On( input, multiset );
    int.Input_Off( button, multiset );
    
  elseif ( target == "button" ) then
  
    int.Input_On( window, multiset );
    int.Input_Off( input, multiset );
    int.Input_On( button, multiset );

  end;
  
  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------
function private.Cleanup ()
  
  for r = 1, 3 do
    
    m_object.Delete( "spr_int_dialog_items_chest_icon_"..r );
    m_object.Enable( "obj_int_dialog_items_chest_"..r );
    
  end;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Tap_Chest ()

  private.SetInput( "none" );
  
  cmn.StopSnd( "items_chest_waiting", 1 );
  cmn.PlaySnd( "items_chest_click" );

  private.SetRewards();
  
  cmn.PlaySnd( "open_chest_derty" );
  AnimPlay( private.chest, private.type.."_open", function ()
  
    AnimPlay( private.chest, private.type.."_static_open", "" );
    
    private.SetInput( "button" );
    
  end );
    
end;
--------------------------------------------------------------------------------
function private.Tap_Button ()
  
  private.SetInput( "none" );
  
  private.CountRewards();
  
  AnimPlay( private.chest, private.type.."_disappear", function ()
    
    AnimPlay( private.window, "close_medium", function ()
     
      private.OnHide();
    
    end );
  
  end );
  
end;
--------------------------------------------------------------------------------
function private.Tap_Reward ( index )

  local multiset = {};

  m_object.Set(
    "anm_int_dialog_items_chest",
    {
      [ "txt_reward_"..index ] = ""
    },
    multiset
  );

  m_object.Set(
    "obj_int_dialog_items_chest_"..index,
    {
      input = false
    },
    multiset
  );

  m_object.Multiset( multiset );

  int.Receive_Gold(
    "obj_int_dialog_items_chest_"..index,
    function ()

      m_object.Disable( "obj_int_dialog_items_chest_"..index );

      private.reward_count = private.reward_count - 1;

      if ( private.reward_count < 1 ) then

        private.Tap_Button();

      end;

    end
  );  

end;
--------------------------------------------------------------------------------