-- name=int_dialog_map_build_construct
function public.__InitModule ()
  
  private.app_h = int.H();
  private.app_o = int.Orientation();
  
  private.strings = {
    res_desc = ne.strings.Get( "str_dialog_map_build_construct_resources" ),
    reward   = ne.strings.Get( "str_dialog_map_build_construct_reward" ),
    for_str  = ne.strings.Get( "str_dialog_map_build_construct_for" )
  };
  
  private.this = {
    dialog = "obj_int_dialog_map_build_construct",
    window = "anm_int_dialog_map_build_construct_window",
    button = {
      close  = "btn_int_dialog_map_build_construct_close",
      buy    = "btn_int_dialog_map_build_construct_buy",
      find   = "btn_int_dialog_map_build_construct_find",
      accept = "btn_int_dialog_map_build_construct_accept"
    },
    item = "spr_int_dialog_map_build_construct_item",
    resources = "anm_int_dialog_map_build_construct_resources"
  };
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.Adapt();
  private.CreateButtons();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( 
    build_name,
    build_content,
    price_data,
    deficient_data,
    fragments_data,
    rewards_data,
    func_accept,
    func_reject,
    func_find,
    func_resolve_deficient,
    func_top_panel_show,
    func_top_panel_hide,
    func_top_panel_lock
  )

  private.Accept           = func_accept;
  private.Reject           = func_reject;
  private.Find             = func_find;
  private.ResolveDeficient = func_resolve_deficient;
  private.TopPanel_Hide    = func_top_panel_hide;
  private.TopPanel_Lock    = func_top_panel_lock;

  private.active = true;
  
  private.price_count = #price_data;
  
  private.SetDefaults(
    build_name,
    price_data,
    deficient_data,
    fragments_data,
    rewards_data
  );
  
  int.Dialog_Open(
    "map_build_construct",
    function ()

      AnimPlay(
        "anm_int_dialog_map_build_construct_caption",
        "appear",
        function ()

          AnimPlay(
            "anm_int_dialog_map_build_construct_caption",
            "static",
            ""
          );

        end
      );

      func_top_panel_show();

    end
  );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_execute )

  if ( not private.active ) then 

    return; 

  end;

  private.active = false;

  private.TopPanel_Hide();
  
  int.Dialog_Close(
    "map_build_construct",
    function ()
      int_window.Hide( "int_dialog_map_build_construct", func_execute );
    end
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  int.Center_Object( private.this.dialog );
  
  private.window_area = {
    default  = { -270, -190, 538, 640 },
    wide     = { -370, -210, 740, 645 },
  };

  -- AL> Random phase for header's fireflies animations
  local func_play;
  func_play = function ( anm_name )
    ObjSet( anm_name, { 
      playing = true, 
      endtrig = function () 
        func_play( anm_name ); 
      end } );
  end;
  
  for c, color in ipairs( { "yellow", "blue", "magneta" } ) do
    ObjSet( "anm_int_dialog_map_build_construct_"..color, { 
      frame = math.random( 1, 1000 ), 
      endtrig = function ()
        func_play( "anm_int_dialog_map_build_construct_"..color );
      end } );
  end;

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()
  int.Button_Create(
    private.this.button.accept,
    {
      color  = "green",
      design = "large",
      sound  = "simply_click"
    },
    function()
      
      private.ApplyBuild();
      
    end
  );
  
  int.Button_Create(
    private.this.button.close,
    {
      design = "close",
      sound  = "simply_click"
    },
    function()
      
      private.Reject();
      
    end
  );
  
  int.Button_Create(
    private.this.button.find,
    {
      title  = "str_dialog_map_build_construct_find",
      color  = "green",
      design = "medium",
      sound  = "simply_click"
    },
    function()
      
      private.FindFragment();
      
    end
  );
  
  int.Button_Create(
    private.this.button.buy,
    {
      title  = "str_dialog_map_build_construct_buy",
      design = "buy_resource",
      tt_for = private.strings.for_str,
      sound  = "simply_click"
    },
    function()
      
      if ( private.ResolveDeficient() ) then
        
        private.SetNormal();
        private.ApplyBuild();
        
      end;
      
    end
  );

end;
--------------------------------------------------------------------------------
function private.SetDefaults (
    build_name,
    price_data,
    deficient_data,
    fragments_data,
    reward_data
  )
  
  local multiset = {};
  
  local price_count = private.price_count;
  local postfix     = tern( not deficient_data )( "normal", "deficient" );
  local is_fragment = ( fragments_data and ( #fragments_data > 0 ) );
  local is_reward   = ( reward_data and ( #reward_data > 0 ) );
  
  local win_anim, item_count;
  
  win_anim = tern( is_fragment               )( "static_1", "static_2" );
  win_anim = tern( is_reward                 )( "static_3", win_anim );
  win_anim = tern( is_fragment and is_reward )( "static_4_"..private.app_o, win_anim );
  
  local win_area = tern( is_fragment and is_reward )( "wide", "default" );

  int_window.SetArea( private.this.dialog, private.window_area[ win_area ] );
  
  if ( is_fragment ) then

    local fragment        = fragments_data[ 1 ].name;
    local collected_count = 0;

    for f = 1, #fragments_data do
      
      collected_count = tern( fragments_data[ f ].collected )(
        collected_count + 1,
        collected_count
      );
      
    end;

    item_count = tostring( collected_count.." / "..#fragments_data );

    local func_SetFindButton = tern( collected_count < #fragments_data )(
      function () int.Button_Enable(  private.this.button.find, multiset ); end,
      function () int.Button_Disable( private.this.button.find, multiset ); end
    );
    
    func_SetFindButton();
    
    m_object.Set(
      private.this.item,
      { 
        res = common.GetItemResource( "fragment", fragment ),
        alp = tern( collected_count < #fragments_data )( 0.5, 1 )
      },
      multiset
    );

    private.FindFragment = function () private.Find( fragment ); end;

  end;
  
  if ( is_reward ) then
    
    m_object.Set(
      "spr_int_dialog_map_build_construct_reward",
      { 
        res = common.GetItemResource( reward_data[ 1 ].type, reward_data[ 1 ].name )
      },
      multiset
    );

  end;
  
  for r = 1, price_count do
    
    local condition = false;
    
    if ( deficient_data ) then
    
      condition = cmn.IsEntityInList(
        deficient_data.list,
        price_data[ r ].resource
      );
      
    end;
    
    local res_animfunc = tern( condition )(
      "res_"..r.."_deficient",
      "res_"..r.."_normal"
    );
    
    m_object.Set(
      "anm_int_dialog_map_build_construct_resource"..r,
      {
        animfunc = res_animfunc,
        tt_count = tostring( price_data[ r ].count )
      },
      multiset
    );
    
  end;
  
  m_object.Set(
    "anm_int_dialog_map_build_construct_resources",
    {
      animfunc   = "res_area_"..postfix.."_"..price_count,
      frame      = 0,
      playing    = true
    },
    multiset
  );
  
  local strings = {
    title       = "str_dialog_map_build_construct_caption_"..build_name,
    description = "str_dialog_map_build_construct_description_"..build_name,
    accept      = "str_dialog_map_build_construct_accept_"..build_name,
  };
  
  m_object.Set(
    private.this.window,
    {
      animfunc        = win_anim.."_"..postfix,
      frame           = 0,
      playing         = true,
      item_count      = item_count,
      title           = ne.strings.Get( strings.title ),
      tt_description  = ne.strings.Get( strings.description ),
      res_description = private.strings.res_desc,
      reward          = private.strings.reward
    },
    multiset
  );
  
  m_object.Set(
    "anm_int_dialog_map_build_construct_caption",
    {
      animfunc = "appear",
      playing  = false,
      frame    = 0
    },
    multiset
  );
  
  if ( deficient_data ) then
  
    int.Button_Enable( private.this.button.buy, multiset );
    
    int.Button_SetTextTag(
      private.this.button.buy,
      "tt_count",
      tostring( deficient_data.gold ),
      multiset
    );
    
  else
    
    int.Button_Disable( private.this.button.buy, multiset );
    
  end;
  
  int.Button_SetText( private.this.button.accept, strings.accept, multiset );
  
  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------
function private.ApplyBuild ()
  
  if ( private.Accept ) then
  
    private.TopPanel_Lock();
    private.Accept();
  
  end;
  
end;
--------------------------------------------------------------------------------
function private.SetNormal ()
  
  local multiset = {};
  
  for r = 1, private.price_count do
    
    m_object.Set(
      "anm_int_dialog_map_build_construct_resource"..r,
      {
        animfunc = "res_"..r.."_normal"
      },
      multiset
    );
    
  end;
  
  m_object.Set(
    private.this.resources,
    {
      animfunc   = "res_area_normal_"..private.price_count
    },
    multiset
  );
  
  local win_anim = m_object.Get( private.this.window ).animfunc;
  
  win_anim = string.gsub( win_anim, "deficient", "normal" );
  
  m_object.Set(
    private.this.window,
    {
      animfunc = win_anim
    },
    multiset
  );
  
  int.Button_Disable( private.this.button.buy, multiset );
  
  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------