-- name=int_dialog_map_mine
--------------------------------------------------------------------------------
function public.__InitModule()

  private.strings = {
    hours       = ne.strings.Get( "str_int_dialog_map_mine_select_hours" ),
    minutes     = ne.strings.Get( "str_int_dialog_map_mine_select_minutes" ),
    seconds     = ne.strings.Get( "str_int_dialog_map_mine_select_seconds" ),
    description = ne.strings.Get( "str_int_dialog_map_mine_select_message_resource" )
  };

end;
--------------------------------------------------------------------------------
function public.__InitObjects()

  private.Adapt();
  private.CreateButtons();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Select_Show (
    mine_type,
    resource_types,
    mine_price,
    produce_data,
    func_accept,
    func_close
  )

  private.active = true;

  private.selected_resource = nil;

  private.Accept = func_accept;
  private.Close  = func_close;

  private.Select_SetDefault(
    mine_type,
    mine_price,
    resource_types,
    produce_data
  );

  int_window.SetArea( private.this, private.window_area.default );

  int.Dialog_Open( "map_mine" );

end;
--------------------------------------------------------------------------------
function public.Select_Hide ( func_execute )

  if ( not private.active ) then

    return;

  end;

  private.active = false;

  private.Price_Hide( true );  

  int.Dialog_Close( "map_mine", func_execute );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Upgrade_Show (
    resource,
    upgrade_data,
    func_accept,
    func_close,
    func_top_panel_show,
    func_top_panel_hide
  )

  private.active = true;

  private.Accept        = func_accept;
  private.Close         = func_close;
  private.TopPanel_Hide = func_top_panel_hide;

  private.Upgrade_SetDefault( resource, upgrade_data );

  int_window.SetArea( private.this, private.window_area.price );

  int.Dialog_Open( "map_mine", function ()

    func_top_panel_show();

  end );

end;
--------------------------------------------------------------------------------
function public.Upgrade_Hide ( func_execute )

  if ( not private.active ) then

    return;

  end;

  private.active = false;

  private.TopPanel_Hide();

  int.Dialog_Close( "map_mine", func_execute );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  private.this = "obj_int_dialog_map_mine";

  private.window_area = {
    default  = { -260, -220, 523, 430 },
    price    = { -260, -220, 523, 512 }
  };

  int.Center_Object( private.this );  

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  common.TapSet( "anm_int_dialog_map_mine_resource1",
  {      
    func_true = function ()

      private.Set( "resource1" );

    end
  } );

  common.TapSet( "anm_int_dialog_map_mine_resource2",
  {      
    func_true = function ()

      private.Set( "resource2" );

    end
  } );

  common.TapSet( "anm_int_dialog_map_mine_resource3",
  {      
    func_true = function ()

      private.Set( "resource3" );

    end
  } );

  int.Button_Create(
    "btn_int_dialog_map_mine_upgrade",
    {
      title  = "str_int_dialog_map_mine_upgrade_accept",
      color  = "green",
      design = "medium",
      price  = true
    },
    function()
      
      private.Accept();
      private.Close();
      
    end
  );
  
  int.Button_Create(
    "btn_int_dialog_map_mine_select",
    {
      title  = "str_int_dialog_map_mine_select_accept",
      color  = "green",
      design = "medium",
      price  = true
    },
    function()
      
      private.Accept( private.selected_resource );
      private.Close();
      
    end
  );
  
  int.Button_Create(
    "btn_int_dialog_map_mine_close",
    {
      design = "close"
    },
    function()
      
      private.Close();
      
    end
  );

end;
--------------------------------------------------------------------------------
function private.Set ( resource )

  local multiset = {};

  if ( resource                              ) and
     ( private.selected_resource == resource ) then

    m_object.Set(
      "spr_int_dialog_map_mine_"..resource,
      {
        animfunc = "sleep_"..resource,
        playing  = true
      },
      multiset
    );

    m_object.Set(
      "anm_int_dialog_map_mine_"..resource,
      {
        animfunc = "firefly_stat",
        playing  = true,
        frame    = 0
      },
      multiset
    );

    m_object.Multiset( multiset );

    private.Price_Hide();

    private.selected_resource = nil;

    return;

  end;

  if ( resource ) then

    private.Price_Show();

  end;

  private.selected_resource = resource;

  for i = 1, 3 do

    if ( resource == "resource"..i ) then

        m_object.Set(
          "spr_int_dialog_map_mine_resource"..i,
          {
            animfunc = "static_resource"..i,
            playing  = true
          },
          multiset
        );

        m_object.Set(
          "anm_int_dialog_map_mine_resource"..i,
          {
            active   = true,
            visible  = true,
            animfunc = "firefly_action",
            playing  = true,
            endtrig  = function()

              m_object.Set(
                "anm_int_dialog_map_mine_resource"..i,
                {
                  animfunc = "firefly_select"
                }
              );

            end
          },
          multiset
        );

    else

      m_object.Set(
        "spr_int_dialog_map_mine_resource"..i,
        {
          animfunc = "sleep_resource"..i,
          playing  = true
        },
        multiset
      );

      m_object.Set(
        "anm_int_dialog_map_mine_resource"..i,
        {
          active   = true,
          visible  = true,
          animfunc = "firefly_stat",
          playing  = true,
          frame    = 0
        },
        multiset
      );

    end;

  end;

  m_object.Multiset( multiset );

  if ( resource ) then

    private.Select_Restart( resource );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Select_Restart ( resource )

  AnimPlay(
    "spr_int_dialog_map_mine_"..resource,
    "static_"..resource,
    function ()

      private.Select_Restart( resource );

    end
  );

end;
--------------------------------------------------------------------------------
function private.Upgrade_Restart ( resource )

  AnimPlay(
    "spr_int_dialog_map_mine_upgrade_from",
    "static_"..resource,
    function ()
      
      private.Upgrade_Restart( resource );
      
    end
  );

  AnimPlay(
    "spr_int_dialog_map_mine_upgrade_to",
    "static_"..resource,
    function ()
      
      private.Upgrade_Restart( resource );
      
    end
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Price_Show ()

  if ( private.price_active ) then

    return;

  end;

  private.price_active = true;

  AnimPlay(
    "anm_int_dialog_map_mine_window",
    "action",
    function()

      int_window.SetArea( private.this, private.window_area.price );

    end
  );  

end;
--------------------------------------------------------------------------------
function private.Price_Hide ( instant )

  if not ( private.price_active ) then

    return;

  end;

  private.price_active = false;

  AnimPlay(
    "anm_int_dialog_map_mine_window",
    "hide",
    function()

      int_window.SetArea( private.this, private.window_area.default );

    end
  ); 

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Select_SetDefault (
    mine_type,
    mine_price,
    resource_types,
    produce_data
  )

  local res_number  = #resource_types;
  local time_string = "";
  local time_table  = common.GetTimeFromSec( produce_data.time );

  local skip = {
    h = ( tonumber( time_table.h ) == 0 ),
    m = ( tonumber( time_table.m ) == 0 ),
    s = ( tonumber( time_table.s ) == 0 )
  };

  if not ( skip.h ) then

    time_string = tonumber( time_table.h ).." "..private.strings.hours;

  end;

  if not ( skip.m ) then

    if ( time_string ~= "" ) then

      time_string = time_string.." "..
        tonumber( time_table.m ).." "..private.strings.minutes;
    
    else

      time_string = tonumber( time_table.m ).." "..private.strings.minutes;

    end;

  end;

  if not ( skip.s ) then

    if ( time_string ~= "" ) then

      time_string = time_string.." "..
        tonumber( time_table.s ).." "..private.strings.seconds;
    
    else

      time_string = tonumber( time_table.s ).." "..private.strings.seconds;

    end;

  end;

  local title =
    ne.strings.Get( "str_int_dialog_map_mine_select_caption_"..mine_type );

  local condition =
    ne.strings.Get(
      "str_int_dialog_map_mine_select_message_info_"..mine_type,
      produce_data.resources
    )
    .. " " .. time_string;

  local multiset = {};

  m_object.Set(
    "anm_int_dialog_map_mine_window", 
    {
      animfunc    = "static",
      title       = title,
      description = private.strings.description,
      condition   = condition
    },
    multiset
  );

  m_object.Set(
    "anm_int_dialog_map_mine_resources",
    {
      animfunc = "fireflies_"..res_number
    },
    multiset
  );

  private.Set();

  int.Button_Disable( "btn_int_dialog_map_mine_upgrade", multiset );
  int.Button_Enable( "btn_int_dialog_map_mine_select", multiset );
  
  m_object.Disable( "spr_int_dialog_map_mine_upgrade_from", false, multiset );
  m_object.Disable( "spr_int_dialog_map_mine_upgrade_to", false, multiset );

  int.Button_SetPrice(
    "btn_int_dialog_map_mine_select",
    mine_price,
    true,
    multiset
  );
    
  int.Button_SetText(
    "btn_int_dialog_map_mine_select",
    "str_int_dialog_map_mine_select_accept_"..mine_type,
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.Upgrade_SetDefault ( resource, upgrade_data )

  local objects = {};

  table.insert( objects, { "anm_int_dialog_map_mine_window", 
  {
    animfunc    = "upgrade",
    title       = ne.strings.Get( "str_int_dialog_map_mine_upgrade_caption" ),
    description = ne.strings.Get( "str_int_dialog_map_mine_upgrade_message" ),
    count_from  = tostring( upgrade_data.count ),
    count_to    = tostring( upgrade_data.count_next )
  } } );

  table.insert( objects, { "spr_int_dialog_map_mine_upgrade_from",
  {
    active   = true,
    visible  = true,
    animfunc = "static_"..resource,
    playing  = true
  } } );

  table.insert( objects, { "spr_int_dialog_map_mine_upgrade_to",
  {
    active   = true,
    visible  = true,
    animfunc = "static_"..resource,
    playing  = true
  } } );

  ObjMultiSet( objects );
  
  int.Button_Disable( "btn_int_dialog_map_mine_select" );
  int.Button_Enable( "btn_int_dialog_map_mine_upgrade" );

  m_object.Disable( "anm_int_dialog_map_mine_resource1", true );
  m_object.Disable( "anm_int_dialog_map_mine_resource2", true );
  m_object.Disable( "anm_int_dialog_map_mine_resource3", true );

  int.Button_SetPrice(
    "btn_int_dialog_map_mine_upgrade",
    upgrade_data.price
  );

  private.Upgrade_Restart( resource );

end;
--------------------------------------------------------------------------------