-- name=int_dialog_map_quest_reward
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.Adapt();
  private.CreateButtons();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show (
    character_name,
    character_content,
    reward_data,
    func_accept
  )

  private.nogold = true;
  private.active = true;
  private.Accept = func_accept;

  private.SetDefault( character_name, character_content, reward_data );

  local animfunc_postfix = tern( private.nogold )( "nogold_"..private.orientation, 
                                                    private.orientation );
  
  AnimPlay(
    "anm_int_dialog_map_quest_reward_window",
    "appear_"..animfunc_postfix,
    function ()

      AnimPlay(
        "anm_int_dialog_map_quest_reward_window",
        "static_"..animfunc_postfix,
        ""
      );

    end
  );
  
  int.Dialog_Open( "map_quest_reward" );

  cmn.PlaySnd( "quest_complete" );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_preclose, func_close )

  if ( not private.active ) then return; end;

  private.active = false;
  
  local animfunc_postfix = tern( private.nogold )( "nogold_"..private.orientation, 
                                                    private.orientation );  
  
  AnimPlay(
    "anm_int_dialog_map_quest_reward_window",
    "disappear_"..animfunc_postfix,
    function ()
      
      if ( func_preclose ) then

        -- JK: this should be first to prevent sprite duplication
        func_preclose();

        ObjSet( "spr_int_dialog_map_quest_reward_item_icon",
                { visible = false } );

      end;
      
      int.Dialog_Close( "map_quest_reward", func_close );
      
    end 
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  private.orientation = int.Orientation();
  
  int.Center_Object( "obj_int_dialog_map_quest_reward" );
  
  local pos_y = {
    portrait  = -20,
    landscape = -40
  };

  ObjSet( "anm_int_dialog_map_quest_reward_window", 
  { title = StringGet( "str_int_dialog_map_reward_caption" ) } );
  
  ObjSet( "obj_int_dialog_map_quest_reward_window",
  {
    pos_y = pos_y[ private.orientation ] } );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()
  
  int.Button_Create(
    "btn_int_dialog_map_quest_reward_accept",
    {
      title  = "str_int_dialog_map_quest_reward_accept",
      design = "medium",
      color  = "green",
      sound  = "bottom_button"
    },
    function()
      
      private.Accept();
      
    end
  );

  common.TapSet( "spr_int_dialog_map_quest_reward_item_icon",
  {    
    func_true = function ()

      private.Accept();

      cmn.PlaySnd( "bottom_button" );

    end
  } );
  
end;
--------------------------------------------------------------------------------
function private.SetDefault ( character_name, character_content, reward_data )

  local objects = {};

  local content_dir = "assets/content/"..character_content;
      
  table.insert( objects, { "anm_int_dialog_map_quest_reward_character",
  { 
    res      = content_dir.."/anm_character_"..character_name,
    animfunc = "static",
    playing  = true 
  } } );
  
  table.insert( objects, { "anm_int_dialog_map_quest_reward_window",
  {
    tt_message = ne.strings.Get( "str_int_popup_cloud_characters_completed_"..character_name )
  } } );

  for r = 1, #reward_data do

    if ( reward_data[ r ].type == "gold" ) then

      table.insert( objects, { "anm_int_dialog_map_quest_reward_window",
      {
        gold_count = reward_data[ r ].count
      } } );
      
      private.nogold = false;

    else

      local rew_type = reward_data[ r ].type;
      local rew_name = reward_data[ r ].name;

      table.insert( objects, { "spr_int_dialog_map_quest_reward_item_icon",
      {
        visible = true,
        res     = common.GetItemResource( rew_type, rew_name )
      } } );

    end;

  end;
  
  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------