-- name=int_dialog_map_reward
--------------------------------------------------------------------------------
function public.__InitObjects ()

  int.Button_Create(
    "btn_int_dialog_map_reward_accept",
    {
      title  = "str_int_dialog_map_reward_accept",
      design = "medium",
      color  = "green",
      sound  = "simply_click"
    },
    function()
      
      private.Accept();
      
    end
  );

  ObjSet( "anm_int_dialog_map_reward_window", 
  { title = StringGet( "str_int_dialog_map_reward_caption" ) } );

  int.Center_Object( "obj_int_dialog_map_reward" );

  private.orientation = int.Orientation();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( reward_data, func_accept )

  private.Accept = func_accept;
  private.active = true;

  private.SetDefault( reward_data );
  
  int.Dialog_Open( "map_reward", function ()

    AnimPlay(
      "anm_int_dialog_map_reward_window",
      "appear_" .. private.orientation,
      function ()

        AnimPlay(
          "anm_int_dialog_map_reward_window",
          "static_" .. private.orientation,
          ""
        );

      end
    );

    cmn.PlaySnd( "level_win_on_map" );
    
  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_preclose, func_close )

  if ( not private.active ) then return; end;

  private.active = false;

  if ( func_preclose ) then

    -- JK: this should be first to prevent sprite duplication
    func_preclose();
    
    if ( private.reward.item ) then
      
      ObjSet( "spr_int_dialog_map_reward_item_icon", { visible = false } );
      
    end;

  end;

  AnimPlay(
    "anm_int_dialog_map_reward_window",
    "disappear_" .. private.orientation,
    function ()

      int.Dialog_Close( "map_reward", func_close );

    end
  );

  cmn.PlaySnd( "ok_info" );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SetDefault ( reward_data )

  private.reward = 
  {
    gold = false,
    res  = false,
    item = false
  }

  local multiset = {};

  local wnd_obj = "anm_int_dialog_map_reward_content";
  local res_obj = "anm_int_dialog_map_reward_resources"

  local wnd_params = {};
  local res_params = {};

  local res_count = 0;

  for id, reward in pairs( reward_data ) do

    local name  = reward.name;
    local count = reward.count;

    if ( name == "gold" ) then

      private.reward.gold   = true;
      wnd_params.gold_count = int.NumberForm( count );
      

    elseif ( string.find( name, "resource" ) ) then

      private.reward.res = true;
      res_count = res_count + 1;
      res_params["res"..res_count.."_count"] = int.NumberForm( count );

      local icon   = "anm_int_dialog_map_reward_resource"..res_count.."_icon";

      local color_id = name:match( "%d+" );
      local params   =
      { 
        animfunc = "static_resource"..color_id,
        playing  = true
      };

      table.insert( multiset, { icon, params } );

    else

      private.reward.item = true;

      if ( reward.type == "fragment" ) then
        name = name:gsub( "_%d+$", "" );
      end;

      local icon   = "spr_int_dialog_map_reward_item_icon";
      local params = 
      {
        visible = true,
        res = "assets/content/"..reward.content.."/"..name
      };

      table.insert( multiset, { icon, params } );

    end;

  end;

  local wnd_anim = "static";

  if ( private.reward.gold ) then wnd_anim = wnd_anim .. "_gold"; end;
  if ( private.reward.res  ) then wnd_anim = wnd_anim .. "_res";  end;
  if ( private.reward.item ) then wnd_anim = wnd_anim .. "_item"; end;

  wnd_params.animfunc = wnd_anim;
  wnd_params.playing  = true;

  table.insert( multiset, { wnd_obj, wnd_params } );

  res_params.animfunc = "resources"..res_count;
  res_params.playing  = true;

  table.insert( multiset, { res_obj, res_params } );

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------