-- name=int_dialog_minibank
--------------------------------------------------------------------------------
function public.Init ()

  private.this = "obj_int_dialog_minibank";

  private.window_area = {
    default = { -270, -300, 540, 580 }
  };
  
  int_window.SetArea( private.this, private.window_area.default );

  private.Func_Buy_1 = function () end;
  private.Func_Buy_2 = function () end;
  
  int.Button_Create(
    "btn_int_dialog_minibank_close",
    {
      design = "close"
    },
    function()
      
      public.Hide();
      private.Reject();
      
    end
  );

  int.Button_Create(
    "btn_int_dialog_minibank_buy_1",
    {
      color  = "green",
      design = "medium"
    },
    function() private.Func_Buy_1(); end
  );
  
  int.Button_Create(
    "btn_int_dialog_minibank_buy_2",
    {
      color  = "green",
      design = "medium"
    },
    function() private.Func_Buy_2(); end
  );
  
  int.Center_Object( "obj_int_dialog_minibank" );

  ObjSet( "anm_int_dialog_minibank_window",
  {
    title           = StringGet( "str_dialog_minibank_caption" ),
    defecient_text  = StringGet( "str_dialog_minibank_text" )
  } );

end;
--------------------------------------------------------------------------------
function public.Show ( deficient, func_accept, func_reject )

  private.active = true;

  private.Accept = func_accept;
  private.Reject = func_reject;
  
  private.SetLots( deficient );

  local window = "anm_int_dialog_minibank_window";

  int_window.Show( "int_dialog_minibank", function ()

    int.Dialog_Open( "minibank" );

    AnimPlay( window, "open",
    function ()
      AnimPlay( window, "static", "" );
    end );

  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_end )

  private.active = false;

  int.Dialog_Close( "minibank", function ()

    int_window.Hide( "int_dialog_minibank", func_end );

  end );

end;
--------------------------------------------------------------------------------
function private.SetLots ( deficient, objects )
  
  local window = "anm_int_dialog_minibank_window";
  local btn_1  = "btn_int_dialog_minibank_buy_1";
  local btn_2  = "btn_int_dialog_minibank_buy_2";
  local icon_1 = "anm_int_dialog_minibank_lot_1";
  local icon_2 = "anm_int_dialog_minibank_lot_2";

  local lots = deficient.lots;

  local objects = {};
  
  if ( lots and #lots > 0 ) then
    
    local lot_1 = lots[ 1 ];
    local lot_2 = lots[ 2 ];
    
    private.lot_1_id = lot_1.name_ios;
    private.lot_2_id = lot_2.name_ios;
    
    local header_1 = "str_dialog_minibank_gold_1";
    local header_2 = "str_dialog_minibank_gold_2";
    
    if ( lot_1.currency_count > lot_2.currency_count ) then
      
      header_2 = "str_dialog_minibank_gold_less";
      
    end;
    
    table.insert( objects, 
    { window,
      {
        header_1        = StringGet( header_1 ),
        header_2        = StringGet( header_2 ),
        count_1         = "+"..tostring( lot_1.currency_count ),
        count_2         = "+"..tostring( lot_2.currency_count ),
        defecient_count = tostring( deficient.count ),
      }
    } );

    local lot_1_price = tern( lot_1.price == "" )( ne.strings.Get( "str_dialog_shop_gold_buy" ), lot_1.price );
    local lot_2_price = tern( lot_2.price == "" )( ne.strings.Get( "str_dialog_shop_gold_buy" ), lot_2.price );

    int.Button_SetText( btn_1, lot_1_price );
    int.Button_SetText( btn_2, lot_2_price );
    
    table.insert( objects, { icon_1, {
      animfunc = "icon_pack_gold_"..lot_1.number,
      playing = true
    } } );
    
    table.insert( objects, { icon_2, {
      animfunc = "icon_pack_gold_"..lot_2.number,
      playing = true
    } } );
    
    private.Func_Buy_1 = function () 
      private.Accept( lot_1.name );
    end;
  
    private.Func_Buy_2 = function () 
      private.Accept( lot_2.name );
    end;
    
    ObjMultiSet( objects );

  end;

end;
--------------------------------------------------------------------------------
function public.UpdateLots ( gold_list )

  if ( private.active ) then
  
    local price_1, price_2 = "", "";
    for _, lot in pairs( gold_list ) do
    
      if ( lot.name_ios == private.lot_1_id ) then price_1 = lot.price; 
      end;
      if ( lot.name_ios == private.lot_2_id ) then price_2 = lot.price; 
      end;
    
    end;
    
    local str_buy = ne.strings.Get( "str_dialog_shop_gold_buy" );
    local txt_price_1 = tern( price_1 == "" )( str_buy, price_1 );
    local txt_price_2 = tern( price_2 == "" )( str_buy, price_2 );
    
    int.Button_SetText( "btn_int_dialog_minibank_buy_1", txt_price_1 );
    int.Button_SetText( "btn_int_dialog_minibank_buy_2", txt_price_2 );
  
  end;

end;
--------------------------------------------------------------------------------