-- name=int_dialog_options
--------------------------------------------------------------------------------
function  public.__InitModule ()

  private.orientation = int.Orientation();
  private.is_pc       = IsPC();

  private.this = {
    window = "anm_int_dialog_options_window",
    button = {
      googleplay = "btn_int_dialog_options_googleplay"
    }
  };
  
  private.titles =
  {
    [ "options" ] = ne.strings.Get( "str_int_dialog_options_tab_options" ),
    [ "help" ]    = ne.strings.Get( "str_int_dialog_options_tab_help" ),
    [ "credits" ] = ne.strings.Get( "str_int_dialog_options_tab_credits" )
  };

  private.animfuncs =
  {
    [ "options" ] = "tab_1_" .. private.orientation,
    [ "help" ]    = "tab_2_" .. private.orientation,
    [ "credits" ] = "tab_3_" .. private.orientation
  };

  private.func_open =
  {
    [ "options" ] = private.Options_Open,
    [ "help" ]    = private.Help_Open,
    [ "credits" ] = private.Credits_Open
  };
  
  private.func_playhaven = common.Playhaven_LogPlacement;

  private.area = {
    [ "portrait"  ] = { -320, -420, 640, 900 },
    [ "landscape" ] = { -250, -320, 500, 700 }
  };

  private.switches =
  {
    music      = { func_state = nil, func_toggle = nil },
    sound      = { func_state = nil, func_toggle = nil },
    fullscreen = { func_state = nil, func_toggle = nil },
    widescreen = { func_state = nil, func_toggle = nil },
    cursor     = { func_state = nil, func_toggle = nil }
  };

  private.help =
  {
    current = 1,
    total   = 9
  };

  private.app_h = int.H();

  private.credits = { bfg = 0, eg = 0 };

  private.credits = 
  {
    count  = { bfg = 0, eg = 0 },
    text_h = 0,
    size   =
    { 
      [ "portrait"  ] = { w = 600, h = 700 }, 
      [ "landscape" ] = { w = 600, h = 700 } 
    }
  };
  
  private.strings = {
    googleplay_login  = ne.strings.Get( "str_int_dialog_options_googleplay_login" ),
    googleplay_logout = ne.strings.Get( "str_int_dialog_options_googleplay_logout" )
  };

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  local dialog = "obj_int_dialog_options";
  int_window.SetArea( dialog, private.area[ private.orientation ] );

  int.Center_Object( dialog );

  int.Button_Create(
    "btn_int_dialog_options_close",
    {
      design = "close"
    },
    function() private.Close(); end
  );

  int.Control_EventsSet( "btn_int_dialog_options_tab_options",
  {
    func_on_click = function () private.ShowTab( "options" ); end
  } );

  int.Control_EventsSet( "btn_int_dialog_options_tab_help",
  {
    func_on_click = function () private.ShowTab( "help" ); end
  } );
  
  int.Control_EventsSet( "btn_int_dialog_options_tab_credits",
  {
    func_on_click = function () private.ShowTab( "credits" ); end
  } );

  local multiset = {};

  local window = "anm_int_dialog_options_window";
  local text_tags = {
    options        = private.titles[ "options" ],
    help           = private.titles[ "help"    ],
    credits        = private.titles[ "credits" ],
    tt_url_terms   = "str_int_dialog_options_terms",
    tt_url_support = "str_int_dialog_options_support",
    tt_url_policy  = "str_int_dialog_options_policy",
    tt_google_play = "str_int_dialog_options_googleplay"
  };

  table.insert( multiset, { window, text_tags } );

  private.Options_Init( multiset );
  private.Help_Init();
  private.Credits_Init( multiset );

  local tabs        = "anm_int_dialog_options_tabs";
  local tabs_layout = { animfunc = "tabs_" .. private.orientation };

  table.insert( multiset, { tabs, tabs_layout } );

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
function public.Show ( funcs_state, funcs_toggle, funcs_social, funcs_other )

  private.active = true;
  private.current_tab = nil;

  local switches = private.switches;

  switches.music.func_state   = funcs_state.music;
  switches.music.func_toggle  = funcs_toggle.music;

  switches.sound.func_state   = funcs_state.sound;
  switches.sound.func_toggle  = funcs_toggle.sound;

  switches.fullscreen.func_state  = funcs_state.fullscreen;
  switches.fullscreen.func_toggle = funcs_toggle.fullscreen;
  
  switches.widescreen.func_state  = funcs_state.widescreen;
  switches.widescreen.func_toggle = funcs_toggle.widescreen;

  switches.cursor.func_state  = funcs_state.cursor;
  switches.cursor.func_toggle = funcs_toggle.cursor;
  
  private.Language = funcs_state.language;

  private.IsLogged      = funcs_social.is_logged;
  private.Login         = funcs_social.login;
  private.Logout        = funcs_social.logout;
  private.GetSocialInfo = funcs_social.get_info;

  private.Close    = funcs_other.close;
  private.Reset    = funcs_other.reset;
  private.Quit     = funcs_other.quit;
  private.IsOnline = funcs_other.is_online;

  private.ShowTab( "options" );

  int_window.Show( "int_dialog_options", function ()

    int.Dialog_Open( "options" );

  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_end )

  if ( not private.active ) then return; end;

  private.active = false;
  
  if ( private.current_tab ) then
    private.func_playhaven( private.current_tab.."_exit" );
    private.current_tab = nil;
  end; 

  int.Dialog_Close( "options", function ()

    int_window.Hide( "int_dialog_options", func_end );

  end );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.ShowTab ( tab )

  local func_open = private.func_open[ tab ];
  local title     = private.titles[ tab ];

  local window = "anm_int_dialog_options_window";
  
  if ( func_open ) then
    
    if ( private.current_tab ) then
      private.func_playhaven( private.current_tab.."_exit" );
    end;
    private.func_playhaven( tab.."_enter" );
    private.current_tab = tab;
    
    cmn.PlaySnd( "tab_switch" );
    private.ToggleTabs( tab );

    ObjSet( window, { title = title } );
    func_open();

  else

    DbgTrace( "[ INTERFACE ][ OPTIONS ] Unknown tab: " .. tostring( tab ) );

  end;

end;
--------------------------------------------------------------------------------
function private.ToggleTabs ( tab )

  local tab_prewious  = private.tab_current;
  private.tab_current = tab;

  if ( tab_prewious ~= tab ) then

    local btn = "btn_int_dialog_options_tab_";

    if ( tab_prewious ) then

      ObjSet( btn..tab_prewious, { input = true  } );

    end;

    ObjSet( btn..tab,  { input = false } );

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Options_Init ( multiset )

  local btn_disabled = { active = false, visible = false, input = false };

  -- Setup switches
  local sw_button   = "btn_int_dialog_options_sw_";
  local sw_prefix   = "anm_int_dialog_options_sw_";

  local sw_inactive = { active = false, visible = false };
  local sw_active   =
  {
    animfunc = "switch_off",
    playing  = false,
    active   = true,
    on       = ne.strings.Get( "str_int_dialog_options_switch_on"  ),
    off      = ne.strings.Get( "str_int_dialog_options_switch_off" )
  };
  
  table.insert( multiset, { sw_prefix .. "music", sw_active } );
  table.insert( multiset, { sw_prefix .. "sound", sw_active } );
  
  local button_language = "btn_int_dialog_options_language";
  
  int.Button_Create( button_language,
    {
      title  = "str_int_dialog_options_language",
      color  = "green",
      design = "large"
    },
    function () private.Language(); end
  );  

  int.Control_EventsSet( sw_button .. "music",
  {
    func_on_click = function () private.SwitchToggle( "music" ) end
  } );

  int.Control_EventsSet( sw_button .. "sound",
  {
    func_on_click = function () private.SwitchToggle( "sound" ) end
  } );

  if ( IsPC() ) then

    table.insert( multiset, { sw_prefix .. "fullscreen", sw_active } );
    table.insert( multiset, { sw_prefix .. "widescreen", sw_active } );
    table.insert( multiset, { sw_prefix .. "cursor",     sw_active } );

    int.Control_EventsSet( sw_button .. "fullscreen",
    {
      func_on_click = function () private.SwitchToggle( "fullscreen" ) end
    } );

    int.Control_EventsSet( sw_button .. "widescreen",
    {
      func_on_click = function () private.SwitchToggle( "widescreen" ) end
    } );
      
    int.Control_EventsSet( sw_button .. "cursor",
    {
      func_on_click = function () private.SwitchToggle( "cursor" ) end
    } );

  else

    table.insert( multiset, { sw_prefix .. "fullscreen", sw_inactive } );
    table.insert( multiset, { sw_prefix .. "widescreen", sw_inactive } );
    table.insert( multiset, { sw_prefix .. "cursor",     sw_inactive } );

    table.insert( multiset, { sw_button .. "fullscreen", btn_disabled } );
    table.insert( multiset, { sw_button .. "widescreen", btn_disabled } );
    table.insert( multiset, { sw_button .. "cursor",     btn_disabled } );

  end;

  -- Setup social
  local soc_inactive = { active = false, visible = false };
  local soc_active   = { active = true,  visible = true  };

  local avatar    = "spr_int_dialog_options_avatar";
  local button_fb = "btn_int_dialog_options_facebook";

  local obj_url = "obj_int_dialog_options_";
  local url_inactive = { active = false, visible = false };

  if ( IsMobile() ) then

    table.insert( multiset, { avatar, soc_active } );

    int.Button_Create( button_fb,
      {
        title  = "str_int_dialog_options_login",
        design = "facebook"
      },
      function () private.FacebookAction(); end
    );

    if ( IsAndroid() ) then
      
      int.Button_Create(
        private.this.button.googleplay,
        {
          title  = tern( private.googleplay_connected )( private.strings.googleplay_logout, private.strings.googleplay_login ),
          design = "medium",
          color  = "green"
        },
        function () private.GooglePlay_Tap(); end
      );
      
    end;
    
    int.Control_EventsSet( obj_url.."terms",
    {
      func_on_click = function ()
      
        if ( private.IsOnline() ) then ShowTermsOfUse() end;
      
      end
    } );

    int.Control_EventsSet( obj_url.."support",
    {
      func_on_click = function ()
      
        if ( private.IsOnline() ) then ShowSupport() end;
      
      end
    } );

    int.Control_EventsSet( obj_url.."policy",
    {
      func_on_click = function ()
      
        if ( private.IsOnline() ) then ShowPrivacyPolicy() end;
      
      end
    } );

  else

    table.insert( multiset, { avatar,    soc_inactive } );
    table.insert( multiset, { button_fb, btn_disabled } );

    table.insert( multiset, { obj_url.."terms",   url_inactive } );
    table.insert( multiset, { obj_url.."support", url_inactive } );
    table.insert( multiset, { obj_url.."policy",  url_inactive } );

  end;

  -- Setup down panel
  local button_reset = "btn_int_dialog_options_reset";
  local button_quit  = "btn_int_dialog_options_quit";

  int.Button_Create( button_reset,
    {
      title  = "str_int_dialog_options_reset",
      design = "medium",
      color  = "green"
    },
    function () private.Reset(); end
  );

  if ( IsPC() ) then

    int.Button_Create( button_quit,
      {
        title  = "str_int_dialog_options_exit",
        design = "medium",
        color  = "blue"
      },
      function () private.Quit(); end
    );

  else

    table.insert( multiset, { button_quit, btn_disabled } );

  end;

  -- Setup text_tags
  local window = "anm_int_dialog_options_window";
  local tags =
  {
    music   = ne.strings.Get( "str_int_dialog_options_music" ),
    sounds  = ne.strings.Get( "str_int_dialog_options_sound" )
  };

  tags.fb_description = "";
  
  if ( IsPC() ) then

    tags.cursor     = ne.strings.Get( "str_int_dialog_options_cursor" );
    tags.fullscreen = ne.strings.Get( "str_int_dialog_options_fullscreen" );
    tags.widescreen = ne.strings.Get( "str_int_dialog_options_widescreen" );

  elseif ( IsMobile() ) then

    tags.fb_description = ne.strings.Get( "str_int_dialog_options_fb_description" );

  end;

  tags.description = ne.strings.Get( "str_int_dialog_options_description" );

  table.insert( multiset, { window, tags } );

end;
--------------------------------------------------------------------------------
function private.Options_Open ()

  local animfunc = private.animfuncs.options;
  local window   = "anm_int_dialog_options_window";

  if ( IsMobile() and private.IsLogged() ) then
    animfunc = animfunc .. "_logged";
  end;

  if ( private.googleplay_mode ) then
    
    animfunc = animfunc.."_google";
    
    int.Button_SetText(
      private.this.button.googleplay,
      tern( private.googleplay_connected )( private.strings.googleplay_logout, private.strings.googleplay_login )
    );
    
  end;
  
  AnimPlay( window, animfunc, "" );

  private.SwitchSet( "music" );
  private.SwitchSet( "sound" );

  if ( IsPC() ) then

    private.SwitchSet( "fullscreen" );
    private.SwitchSet( "widescreen" );
    private.SwitchSet( "cursor" );

  end;

  if ( IsMobile() ) then

    local btn_fb = "btn_int_dialog_options_facebook";

    if ( private.IsLogged() ) then

      int.Button_SetText( btn_fb, "str_int_dialog_options_logout" );

      local info = private.GetSocialInfo();

      ObjSet( window, { username = info.name } );

      if ( info.url == '' ) then
        info.url = nil;
      end;
      local avatar = "spr_int_dialog_options_avatar";
      ObjSet( avatar,
      {
        url = info.url,
        res = "assets/content/interface/int_dialog_stage_friends/default_0",
        default_res = "assets/content/interface/int_dialog_stage_friends/default_0",
        trg_resloaded = function () int.ImageAdjust( avatar, 150 ); end;
      } );

      int.ImageAdjust( avatar, 150 );

    else

      int.Button_SetText( btn_fb, "str_int_dialog_options_login" );

    end;

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.SwitchSet ( name, forced_state )

  local object   = "anm_int_dialog_options_sw_"..name;
  local state    = forced_state or private.switches[ name ].func_state();
  local animfunc = tern( state )( "switch_on", "switch_off" );
  
  ObjSet( object,
  {
    animfunc = animfunc,
    playing  = false,
    frame    = 12 -- NOTE: last frame
  } );

end;
--------------------------------------------------------------------------------
function private.SwitchToggle ( name )

  local object = "anm_int_dialog_options_sw_"..name;
  local state = private.switches[ name ].func_toggle();

  if ( state ~= nil ) then
    local animfunc = tern( state )( "switch_on", "switch_off" );
    AnimPlay( object, animfunc, function () end );
  end;
  cmn.PlaySnd( "simply_click" );

end;
--------------------------------------------------------------------------------
function public.OnFullScreenToggle ( params )

  if ( private.active ) then
    private.SwitchSet( "fullscreen" );
    private.SwitchSet( "widescreen" );
  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.FacebookAction ()

  if ( private.IsLogged() ) then

    private.Logout();

  else

    private.Login();

  end;

end;
--------------------------------------------------------------------------------
function public.OnLogin ( state )

  if ( private.active and private.tab_current == "options" ) then
    
    private.Options_Open();

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.GooglePlay_Setup ( is_connected, func_action )
  
  private.googleplay_mode      = true;
  private.googleplay_connected = is_connected;
  private.GooglePlayAction     = func_action;
  
end;
--------------------------------------------------------------------------------
function private.GooglePlay_Tap ()
  
  if ( private.googleplay_connected ) then
    
    mediator.SubscribeOnce(
      "View_Game_GooglePlayLoggedOut",
      private.GooglePlay_OnLoggedOut
    );
    
  else

    int.Input_Off( private.this.button.googleplay );

    mediator.SubscribeOnce(
      "View_Game_GooglePlayLoggedIn",
      private.GooglePlay_OnLoggedIn
    );

    mediator.SubscribeOnce(
      "Model_Client_ApplicationGainFocus",
      private.GooglePlay_OnApplicationGainFocus
    );
    
  end;
  
  if ( private.GooglePlayAction ) then
    
    private.GooglePlayAction();
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.GooglePlay_OnApplicationGainFocus ()

  int.Input_On( private.this.button.googleplay );

end;
--------------------------------------------------------------------------------
function private.GooglePlay_OnLoggedIn ()
  
  int.Button_SetText(
    private.this.button.googleplay,
    private.strings.googleplay_logout
  );
  
  --int.Input_On( private.this.button.googleplay );
  
end;
--------------------------------------------------------------------------------
function private.GooglePlay_OnLoggedOut ()
  
  int.Button_SetText(
    private.this.button.googleplay,
    private.strings.googleplay_login
  );
  
  --int.Input_On( private.this.button.googleplay );
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Help_Init ()

  int.Button_Create( "btn_int_dialog_options_help_prev",
    {
      design      = "arrow_left",
      input_rects = { 60, 750 },
      input_shift = {  0, 50  }
    },
    function () private.Help_TurnPage( -1 ); end
  );

  int.Button_Create( "btn_int_dialog_options_help_next",
    {
      design      = "arrow_right",
      input_rects = { 60, 750 },
      input_shift = {  0, 50  }
    },
    function () private.Help_TurnPage( 1 ); end
  );

  if ( private.is_pc ) then

    m_object.Disable( "obj_int_dialog_options_swipe" );

  else

    int.Swipe_Create(
      "obj_int_dialog_options_swipe",
      {
        left  = function() private.Help_TurnPage(  1 ); end,
        right = function() private.Help_TurnPage( -1 ); end
      }
    );

  end;

end;
--------------------------------------------------------------------------------
function private.Help_Open ()

  local animfunc = private.animfuncs.help;
  AnimPlay( "anm_int_dialog_options_window", animfunc, "" );

  local page = private.help.current;

  private.Help_SetText( page );

  local animfunc = "help_page"..page;
  AnimPlay( "anm_int_dialog_options_help_content", animfunc, "" );

end;
--------------------------------------------------------------------------------
function private.Help_SetText ( page );

  ObjSet( "anm_int_dialog_options_window",
  {
    page_number = page.."/"..private.help.total
  } );

  ObjSet( "anm_int_dialog_options_help_content",
  {
    help_text1  = ne.strings.Get( "str_int_dialog_help_page"..page.."_top" ),
    help_text2  = ne.strings.Get( "str_int_dialog_help_page"..page.."_btm" )
  } );

end;
--------------------------------------------------------------------------------
function private.Help_AnimateTurn ( page, animfunc )

  private.help.current = page;

  local object    = "anm_int_dialog_options_help_content"
  local page_anim = "help_page"..page;  

  if ( page == 9         ) and
     ( not private.is_pc ) then

    page_anim = page_anim.."_fb";

  end;

  AnimPlay(
    object,
    animfunc,
    function () 

      private.Help_SetText( page );
      AnimPlay( object, page_anim, "" );

    end );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Help_TurnPage ( delta )

  local page = private.help.current + delta;
  local anim = "page_turn";

  if ( delta < 0 ) then

    anim = "page_turn_back";

  end;

  if ( page < 1 ) then

    page = private.help.total;

  elseif ( page > private.help.total ) then

    page = 1;

  end;

  private.Help_AnimateTurn( page, anim );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Credits_Init ( multiset )

  private.Credits_LoadStrings( "bfg" );
  private.Credits_LoadStrings( "eg"  );

  local size = private.credits.size[ private.orientation ];

  table.insert( multiset, { "obj_int_dialog_options_credits_text_container",
  {
    croprect_init =   true,
    croprect_x    = - size.w / 2,
    croprect_y    = - size.h / 2,
    croprect_w    =   size.w,
    croprect_h    =   size.h
  } } );

  table.insert( multiset, { "obj_int_dialog_options_credits_text_dragger",
  {
    inputrect_init =   true,
    inputrect_x    = - size.w / 2,
    inputrect_y    = - size.h / 2,
    inputrect_w    =   size.w,
    inputrect_h    =   size.h,

    drag            = true,
    realdrag        = true,
    inputrect_init  = true,
    
    event_startdrag = private.Credits_ScrollStart,
    event_dragdrop  = private.Credits_ScrollEnd,
    event_drag      = private.Credits_Scroll,
  } } );

end;
--------------------------------------------------------------------------------
function private.Credits_Open ()

  local animfunc = private.animfuncs.credits;
  AnimPlay( "anm_int_dialog_options_window", animfunc, "" );

  local pos_bfg_y = 300;
  local pos_eg_y  = 270;

  local obj_bfg = "obj_int_dialog_options_credits_group_bfg_";
  local txt_bfg = "txt_int_dialog_options_credits_names_bfg_";

  for i = 1, private.credits.count.bfg do

    ObjSet( obj_bfg..i, { pos_y = pos_bfg_y } );
    pos_bfg_y = pos_bfg_y + ObjGet( txt_bfg..i ).draw_height + 40;

  end;

  local obj_eg = "obj_int_dialog_options_credits_group_eg_";
  local txt_eg = "txt_int_dialog_options_credits_names_eg_";

  for i = 1, private.credits.count.eg do

    ObjSet( obj_eg..i, { pos_y = pos_eg_y } );
    pos_eg_y = pos_eg_y + ObjGet( txt_eg..i ).draw_height + 40;

  end;

  private.credits.text_h = pos_bfg_y + pos_eg_y;
    
  ObjSet( "obj_int_dialog_options_credits_text_eg",
  {
    pos_y = pos_bfg_y
  } );

  local start_h = -private.credits.size[ private.orientation ].h / 2;
  ObjSet( "obj_int_dialog_options_credits_text", { pos_y = start_h } );

  private.Credits_Slide();

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Credits_LoadStrings ( team )

  local str = "str_int_dialog_credits_text_";

  local txt_position = "txt_int_dialog_options_credits_position_";
  local txt_names    = "txt_int_dialog_options_credits_names_";

  local i        = 1;
  local position = ne.strings.Get( str..team.."_p"..i );

  while ( position ~= str..team.."_p"..i ) do

    local names = ne.strings.Get( str..team.."_p"..i.."_names" );

    ObjClone(
      "obj_int_dialog_options_credits_group", "_"..team.."_"..i, true, 
      "obj_int_dialog_options_credits_text_"..team
    );

    ObjSet( txt_position..team.."_"..i,
      {
        text     = position,
        maxwidth = private.credits.size[ private.orientation ].w
      }
    );
    ObjSet( txt_names..team.."_"..i,
      {
        text     = names,
        maxwidth = private.credits.size[ private.orientation ].w
      }
    );

    i = i + 1;  
    position = ne.strings.Get( str..team.."_p"..i );

  end;

  private.credits.count[ team ] = i - 1;

end;
--------------------------------------------------------------------------------
function private.Credits_Slide ()

  local pos_y = ObjGet( "obj_int_dialog_options_credits_text" ).pos_y;
  
  local total_t = 50;
  
  local shift = pos_y + private.credits.text_h;
  if ( shift < 0 ) then shift = 0; end;
  
  local time  = total_t * shift / private.credits.text_h;

  ObjAnimate( "obj_int_dialog_options_credits_text", "pos_y", 0, 0, 
  private.Credits_StopSlide,
  {
    0.0,  0, pos_y,
    0.5,  0, pos_y,
    time, 0, pos_y - shift
  } );

end;
--------------------------------------------------------------------------------
function private.Credits_StopSlide ()

  ObjStopAnimate( "obj_int_dialog_options_credits_text", "pos_y" );
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Credits_ScrollStart ()

  local dragger   = "obj_int_dialog_options_credits_text_dragger";
  local container = "obj_int_dialog_options_credits_text_container";
  
  private.Credits_StopSlide();
  
  local shift_pos = GetObjPosByObj( dragger, container );
  private.drag_begin_pos = shift_pos[2];

end;
--------------------------------------------------------------------------------
function private.Credits_ScrollEnd ()

  local dragger = "obj_int_dialog_options_credits_text_dragger";
  ObjSet( dragger, { pos_x = 0, pos_y = 0 } );
  
  private.Credits_Slide();
  
end;
--------------------------------------------------------------------------------
function private.Credits_Scroll ()
  
  local dragger   = "obj_int_dialog_options_credits_text_dragger";
  local container = "obj_int_dialog_options_credits_text_container";
  local text      = "obj_int_dialog_options_credits_text";

  -- Get scroll delta
  local current_pos = GetObjPosByObj( dragger, container );
  local delta = current_pos[2] - private.drag_begin_pos;
  private.drag_begin_pos = current_pos[2];
  
  -- Scroll text to delta but not over margins
  local posy = ObjGet( text ).pos_y;
  local newpos = posy + delta;
  
  local top_margin = -private.credits.size[ private.orientation ].h / 2 * 0.85;
  local bot_margin = -private.credits.text_h;
  
  if ( newpos >= top_margin ) then
    
    ObjSet( text, { pos_y = top_margin } );
    
  elseif ( newpos <= bot_margin ) then
    
    ObjSet( text, { pos_y = bot_margin } );
    
  else
    
    ObjSet( text, { pos_y = newpos } );
    
  end;
  
end;
--------------------------------------------------------------------------------