-- name=int_dialog_shop
function  public.__InitModule ()

  private.app_w = int.W();
  private.app_h = int.H();
  private.app_o = int.Orientation();

  private.this = {
    window = "anm_int_dialog_shop_window"
  };

  private.gold =
  {
    tab_created = false,
    func_create = private.GoldTab_Create,
    func_update = private.GoldTab_Update,
    funcs       = {}
  };

  private.boosters =
  {
    tab_created = false,
    func_create = private.BoostersTab_Create,
    func_update = private.BoostersTab_Update,
    slot_id     = {},
    scc_size    =
    { 
      portrait  = { w = 600, h = 720 }, 
      landscape = { w = 600, h = 720 } 
    },
    funcs       = {}
  };

  private.market =
  {
    tab_created = false,
    func_create = private.MarketTab_Create,
    func_update = private.MarketTab_Update,
    funcs       = {},
    buttons     = { plus = {}, minus = {} },
    purchase    = { 0, 0, 0 }
  };

  private.funcs = {};

  private.tab_current = nil;

  private.window_area = {
    [ "portrait"  ] = { -320, -420, 640, 900 },
    [ "landscape" ] = { -250, -320, 500, 700 }
  };
  
end;
--------------------------------------------------------------------------------
function  public.__InitObjects ()

  local area = private.window_area[ private.app_o ];
  int_window.SetArea( "obj_int_dialog_shop", area );
    
  int.Center_Object( "obj_int_dialog_shop" );

  local multiset = {};

  local str_prefix = "str_dialog_shop";
  local text_tags = {
    [ "tab_gold"   ] = StringGet( str_prefix.."_tab_gold" ),
    [ "tab_boosts" ] = StringGet( str_prefix.."_tab_boosters" ),
    [ "tab_shop"   ] = StringGet( str_prefix.."_tab_market" ),
    [ "sell_text"  ] = StringGet( str_prefix.."_market_sell_caption" ),
    [ "buy_text"   ] = StringGet( str_prefix.."_market_buy_caption" )
  };

  table.insert( multiset, { private.this.window, text_tags } );

  local tabs_params = { animfunc = "tabs_"..private.app_o };
  table.insert( multiset, { "anm_int_dialog_shop_tabs", tabs_params } );

  local scc_size = private.boosters.scc_size[ private.app_o ];

  table.insert( multiset, { "scc_int_dialog_shop",
  {
    inputrect_init = true,
    inputrect_x   = - scc_size.w / 2,
    inputrect_y   = - scc_size.h / 2,
    inputrect_w   =   scc_size.w,
    inputrect_h   =   scc_size.h,

    croprect_init = true,
    croprect_x    = - scc_size.w / 2,
    croprect_y    = - scc_size.h / 2,
    croprect_w    =   scc_size.w,
    croprect_h    =   scc_size.h
  } } );

  table.insert( multiset, { "anm_int_dialog_shop_market_sell_wait",
  {
    timer_description = StringGet( str_prefix.."_market_sell_wait" )
  } } );

  table.insert( multiset, { "anm_int_dialog_shop_tab_market_notification",
  {
    count   = "!"
  } } );

  int.Button_Create(
    "btn_int_dialog_shop_close",
    {
      design = "close"
    },
    function ()

      public.Hide();

    end 
  );

  int.Control_EventsSet(
    "btn_int_dialog_shop_tab_gold",
    {
      func_on_click = function () private.ShowTab( "gold" ); end
    },
    multiset
  );

  int.Control_EventsSet(
    "btn_int_dialog_shop_tab_boosters",
    {
      func_on_click = function () private.ShowTab( "boosters" ); end
    },
    multiset
  );
  
  int.Control_EventsSet(
    "btn_int_dialog_shop_tab_market",
    {
      func_on_click = function () private.ShowTab( "market" ); end
    },
    multiset
   );

  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function  public.SetGoldHandlers ( func_buy )

  private.gold.funcs.Buy = func_buy;
  
end;
--------------------------------------------------------------------------------
function  public.SetBoostersHandlers ( func_buy, 
                                       func_info )

  private.boosters.funcs.Buy  = func_buy;
  private.boosters.funcs.Info = func_info;

end;
--------------------------------------------------------------------------------
function  public.SetMarketHandlers ( func_share_value,
                                     func_share_price,
                                     func_share_limit,
                                     func_get_shares_sell,
                                     func_try_buy,
                                     func_try_sell,
                                     func_refresh_time )

  private.market.funcs.ShareValue = func_share_value;
  private.market.funcs.SharePrice = func_share_price;
  private.market.funcs.ShareLimit = func_share_limit;

  private.market.funcs.GetSharesSell = func_get_shares_sell;

  private.market.funcs.TryBuy      = func_try_buy;
  private.market.funcs.TrySell     = func_try_sell;
  private.market.funcs.RefreshTime = func_refresh_time;
  
end;
--------------------------------------------------------------------------------
function  public.SetGenericHandlers ( func_top_panel_lock,
                                      func_top_panel_unlock,
                                      func_on_tab_open,
                                      func_on_tab_close,
                                      func_is_market_available,
                                      func_show_popup,
                                      func_hide_popup )

  private.funcs.Panel_Lock    = func_top_panel_lock;
  private.funcs.Panel_Unlock  = func_top_panel_unlock;

  private.funcs.OnTabOpen  = func_on_tab_open;
  private.funcs.OnTabClose = func_on_tab_close;

  private.funcs.IsMarketAvailable = func_is_market_available;

  private.funcs.ShowPopup = func_show_popup;
  private.funcs.HidePopup = func_hide_popup;
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function  public.Show ( tab, gold_list, boosters_list )

  private.gold.list     = gold_list;
  private.boosters.list = boosters_list;

  private.funcs.Panel_Lock();
  private.active = true;

  int_window.Show(
    "int_dialog_shop",
    function ()

      private.ShowTab( tab, true );
      
      int.Dialog_Open(
        "shop",
        function ()
       
          private.funcs.OnTabOpen( tab );
       
        end
      );
  
    end
  );

end;
--------------------------------------------------------------------------------
function  public.Hide ( func_end )

  private.funcs.HidePopup( true );
  private.funcs.Panel_Unlock();

  private.funcs.OnTabClose( private.tab_current );
  private.active = false;
  
  int.Dialog_Close( "shop", function ()

    int_window.Hide( "int_dialog_shop", func_end );

  end );

end;
--------------------------------------------------------------------------------
function  public.IsActive ()
  
  return private.active;
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.UpdateLots ( gold_list )

  if ( private.active ) then
  
    private.gold.list = gold_list;
    
    if ( private.gold.tab_created and private.tab_current == "gold" ) then
    
      private.GoldTab_Update();
    
    end;
  
  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.ShowTab ( tab, skip_notification )

  if ( private.active ) then cmn.PlaySnd( "tab_switch" ); end;

  if ( tab == "market" and not private.funcs.IsMarketAvailable() ) then

      private.funcs.ShowPopup(
        "str_dialog_shop_tab_market_unlock_caption",
        "str_dialog_shop_tab_market_unlock",
        "btn_int_dialog_shop_tab_"..tab,
        { 0, -130 }
      );
    return;

  end;
  
  if ( private.funcs.IsMarketAvailable() ) then

    local sells = private.market.funcs.GetSharesSell();
    local has_offers = false;

    for res_name, res_count in pairs( sells ) do

      if ( res_count > 0 ) then
        has_offers = true;
        break;
      end; 
      
    end;
 
    public.Market_SetNotification( has_offers );
    
  end;

  if ( tab == "gold" or tab == "boosters" or tab == "market" ) then

    local tab_data = private[ tab ];

    private.ToggleTabs( tab );

    if not ( tab_data.tab_created ) then

      tab_data.func_create();
      tab_data.tab_created = true;

    end;

      tab_data.func_update();
      
      if ( not skip_notification ) then
      
        private.funcs.OnTabOpen( tab );
      
      end;

  else

    DbgTrace( "[ INTERFACE ][ DIALOG SHOP ] Unknown tab: " .. tostring( tab ) );

  end;

end;
--------------------------------------------------------------------------------
function private.ToggleTabs ( tab )

  local tab_prewious  = private.tab_current;
  private.tab_current = tab;

  if ( tab_prewious ~= tab ) then

    local btn = "btn_int_dialog_shop_tab_";

    if ( tab_prewious ) then

      private.funcs.OnTabClose( tab_prewious );
      ObjSet( btn..tab_prewious, { input = true  } );

    end;

    ObjSet( btn..tab,  { input = false } );

  end;

end;
--------------------------------------------------------------------------------
function  public.CurrentTab ()
  
  return private.tab_current;
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.GoldTab_Create ()

  local lot    = "anm_int_dialog_shop_lot";
  local parent = "obj_int_dialog_shop_lot"
  local button = "btn_int_dialog_shop_lot_buy";
  local icon   = "anm_int_dialog_shop_lot_icon";

  local str = "str_dialog_shop_gold_";

  local multiset = {};

  for i, lot_data in pairs( private.gold.list ) do

    local suff = "_"..i;

    ObjClone( lot, suff, true, parent..suff );

    int.Button_Create( button..suff,
      {
        title  = tern( lot_data.price == "" )( str.."buy", lot_data.price ),
        design = "medium",
        color  = "green"
      },
      function () private.gold.funcs.Buy( lot_data.name ) end
    );

    local lot_params =
    {
      animfunc   = "tab_1_frame",
      playing    = true,
      title      = StringGet( str..lot_data.text_id ),
      gold_count = int.NumberForm( lot_data.currency_count )
    };

    local af_suffix =
    {
      [ "best_value"   ] = "_blue",
      [ "most_popular" ] = "_red"
    };

    local label = lot_data.labels[ 1 ];
    if ( label and af_suffix[ label ] ) then

      lot_params.animfunc = lot_params.animfunc..af_suffix[ label ];
      lot_params.label    = StringGet( str..label );

    end;

    table.insert( multiset, { lot..suff, lot_params } );

    local icon_params =
    {
      animfunc = "icon_pack_gold_"..i, 
      playing = true
    };

    table.insert( multiset, { icon..suff, icon_params } );

  end;

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
function private.GoldTab_Update ()

  local multiset = {};
  local button   = "btn_int_dialog_shop_lot_buy";
  local str      = "str_dialog_shop_gold_";

  for i, lot_data in pairs( private.gold.list ) do

    local suff = "_"..i;

    int.Button_SetText(
      button..suff,
      tern( lot_data.price == "" )( str.."buy", lot_data.price ),
      multiset
    );

  end;

  m_object.Set(
    private.this.window,
    {
      animfunc = "tab_1_"..private.app_o,
      title    = StringGet( "str_dialog_shop_tab_gold" )
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.BoostersTab_Create ()

  m_object.Clone(
    "scc_int_dialog_shop",
    private.this.window,
    "_boosters"
  );

  local scc    = "scc_int_dialog_shop_boosters";
  local slot   = "anm_int_dialog_shop_booster";
  local button = "btn_int_dialog_shop_booster_buy";
  local icon   = "spr_int_dialog_shop_booster_icon";

  local str      = "str_dialog_shop_gold_";
  local str_item = "str_item_";

  local multiset = {};

  for i, booster in pairs( private.boosters.list ) do

    local suff    = "_"..i;
    local pack_id = 1;
    local pack    = booster.packs[ pack_id ];
    local name    = booster.name;

    private.boosters.slot_id[ name ] = i;

    ObjClone( slot, suff, true, scc );

    int.Button_Create( button..suff,
      {
        title  = pack.price,
        design = "price"
      },
      function () private.BoostersTab_Buy( name, pack_id ) end
    );

    local slot_params =
    {
      animfunc    = "tab_2_frame",
      playing     = true,
      title       = StringGet( str_item..name.."_caption" ),
      description = StringGet( str_item..name.."_description" ),
      count       = "x"..pack.count;
    }

    if not ( booster.unlocked ) then

      slot_params.animfunc     = slot_params.animfunc .. "_locked";
      slot_params.unlock_level = StringGet( str_item..name.."_unlock_level" );
      slot_params.unlock_world = StringGet( str_item..name.."_unlock_world" );

    end;

    table.insert( multiset, { slot..suff, slot_params } );

    local icon_params = { res = "assets/content/boosters/"..name };

    table.insert( multiset, { icon..suff, icon_params } );

  end;

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
function private.BoostersTab_Update ()

  ObjSet( private.this.window,
  {
    animfunc = "tab_2_"..private.app_o,
    title    = StringGet( "str_dialog_shop_tab_boosters" )
  } );

  for i, booster in pairs( private.boosters.list ) do

    if ( booster.unlocked ) then

      AnimPlay( "anm_int_dialog_shop_booster_" .. i, "tab_2_frame", "" );

    end;

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.BoostersTab_Buy ( name, pack_id )

  local id     = private.boosters.slot_id[ name ];
  local slot   = "anm_int_dialog_shop_booster_" .. id;
  local button = "btn_int_dialog_shop_booster_buy_" .. id;

  local action = "tab_2_frame_action";
  local static = "tab_2_frame";

  if ( private.boosters.funcs.Buy( name, pack_id ) ) then

    AnimStop( slot );
    AnimPlay( slot, action, function () AnimPlay( slot, static, ""); end );

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.MarketTab_Create ()

  local sell_parent = "obj_int_dialog_shop_market_sell_slot";
  local sell_slot   = "anm_int_dialog_shop_sell_slot";
  local sell_button = "btn_int_dialog_shop_sell_slot_sell";
  local sell_face   = "anm_int_dialog_shop_sell_slot_resource";

  local buy_parent  = "obj_int_dialog_shop_market_buy_slot";
  local buy_slot    = "anm_int_dialog_shop_buy_slot";
  local buy_fill    = "anm_int_dialog_shop_buy_slot_fill";
  local buy_face    = "anm_int_dialog_shop_buy_slot_resource";

  local btn_plus    = "btn_int_dialog_shop_buy_slot_plus";
  local icon_plus   = "anm_int_dialog_shop_buy_slot_plus";
  local btn_minus   = "btn_int_dialog_shop_buy_slot_minus";
  local icon_minus  = "anm_int_dialog_shop_buy_slot_minus";

  local accept = "btn_int_dialog_shop_market_buy_accept";

  local multiset = {};

  -- Create sell objects
  for i = 1, 3 do

    local suff = "_"..i;
    ObjClone( sell_slot, suff, true, sell_parent..suff );

    int.Button_Create( sell_button..suff,
      {
        title  = "-",
        design = "price"
      },
      function () private.MarketTab_Sell ( i ); end
    );

    local faces =
    { 
      "054_icon_resource1",
      "055_icon_resource2", 
      "056_icon_resource3"
    };

    local face_params =
    {
      animfunc = faces[ i ],
      playing  = true
    };

    table.insert( multiset, { sell_face..suff, face_params } );

    local slot_params =
    {
      animfunc   = "tab_3_frame_sell_static",
      playing    = true,
      sell_text  = StringGet( "str_dialog_shop_market_sell_accept" ),
      sell_count = ""
    };

    table.insert( multiset, { sell_slot..suff, slot_params } );

  end;

  -- Crete buy objects
  local buttons = private.market.buttons;

  for i = 1, 3 do

    local suff = "_"..i;
    ObjClone( buy_slot, suff, true, buy_parent..suff );

    -- Plus button
    local plus = { pressed = false, active = true };
    table.insert( buttons.plus, plus );

    local plus_icon_params = { animfunc = "plus", playing = true };
    local plus_btn_params  = 
    { 
      input = true,
      event_mdown   = function ()

        plus.pressed = true;
        private.MarketTab_TapPlus( i )

      end,
      event_mup     = function () plus.pressed = false; end,
      event_menter  = function ()

       plus.pressed = false;
       if ( IsPC() ) then SetCursor( CURSOR_HAND ); end; 

      end,
      event_mleave  = function ()

        plus.pressed = false;
        if ( IsPC() ) then SetCursor( CURSOR_DEFAULT ); end;

      end,
    };

    table.insert( multiset, { icon_plus..suff, plus_icon_params } );
    table.insert( multiset, { btn_plus..suff,  plus_btn_params  } );

    -- Minus button
    local minus = { pressed = false, active = false };
    table.insert( buttons.minus, minus );

    local minus_icon_params = { animfunc = "btn_minus_off", playing = true };
    local minus_btn_params  = 
    { 
      input = false,
      event_mdown   = function ()

        minus.pressed = true;
        private.MarketTab_TapMinus( i );

      end,
      event_mup     = function () minus.pressed = false; end,
      event_menter  = function ()

       minus.pressed = false;
       if ( IsPC() ) then SetCursor( CURSOR_HAND ); end; 

      end,
      event_mleave  = function ()

        minus.pressed = false;
        if ( IsPC() ) then SetCursor( CURSOR_DEFAULT ); end;

      end,
    };

    table.insert( multiset, { icon_minus..suff, minus_icon_params } );
    table.insert( multiset, { btn_minus..suff,  minus_btn_params  } );

    -- Set resousce face
    local face_params = { animfunc = "sleep_resource"..i, playing = true };
    table.insert( multiset, { buy_face..suff, face_params  } );

    -- Set fill
    local fill_params = { animfunc = "circle_resource_"..i, playing = true };
    table.insert( multiset, { buy_fill..suff, fill_params  } );

    -- Set count
    local slot_params = 
    {
      animfinc = "tab_3_frame_buy_static",
      playing = true,
      count_resource  = ""
    };
    table.insert( multiset, { buy_slot..suff, slot_params } );

  end;
  
  ObjMultiSet( multiset );

  int.Button_Create(
    accept,
    {
      title  = "str_dialog_shop_market_buy_accept",
      color  = "green",
      design = "medium",
      price  = true
    },
    function() private.MarketTab_Excahnge() end
  );

  int.Button_SetPrice( accept, private.MarketTab_GetBuyPrice(), true );
  
end;
--------------------------------------------------------------------------------
function private.MarketTab_Update ()

  private.MarketTab_UpdateSells();

  ObjSet( private.this.window, 
  {
    animfunc = "tab_3_"..private.app_o,
    title    = StringGet( "str_dialog_shop_tab_market" )
  } );

end;
--------------------------------------------------------------------------------
function private.MarketTab_UpdateSells ()

  local market = private.market.funcs;
  local sells = market.GetSharesSell();

  local has_offers = false;

  local sell_slot   = "anm_int_dialog_shop_sell_slot";
  local sell_button = "btn_int_dialog_shop_sell_slot_sell";
  local timer       = "anm_int_dialog_shop_market_sell_wait";

  -- Do not update during action animation
  for i = 1, 3 do

    local af = ObjGet( sell_slot.."_"..i ).animfunc;
    if (  af == "tab_3_frame_sell_action_"..i ) then return; end;

  end;

  local multiset = {};

  table.insert( multiset, { timer, { visible = false } } );

  for res_name, res_count in pairs( sells ) do

    local id = tonumber( string.match( res_name, "%d+" ) );
    local suff = "_"..id;

    local share_value = market.ShareValue( "sell", id );
    local share_price = market.SharePrice( "sell", id );

    local price = "-";
    local count = "";

    local animfunc = "tab_3_frame_sold_static";

    if ( res_count > 0 ) then

      animfunc = "tab_3_frame_sell_static";
      has_offers = true;

      price = math.floor( res_count / share_value * share_price );
      count = int.NumberForm( res_count );

    end;

    table.insert( multiset, { sell_slot..suff,
    {
      animfunc   = animfunc,
      sell_count = count,
      input      = false
    } } );

    int.Button_SetText( sell_button..suff, price );
 
  end;

  if not ( has_offers ) then

    table.insert( multiset, { timer, { visible = true } } );
    private.MarketTab_Tick();

  else

    for res_name, res_count in pairs( sells ) do

      local id = tonumber( string.match( res_name, "%d+" ) );
      local suff = "_"..id;

      if ( res_count > 0 ) then

        table.insert( multiset, { sell_slot..suff,
        {
          input = true
        } } );

      end;
 
    end;

  end;
  
  public.Market_SetNotification( has_offers );

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
function private.MarketTab_Tick ()

  local time = private.market.funcs.RefreshTime();
  
  if not ( time ) then return; end;

  local objects = {};

  table.insert( objects, { "anm_int_dialog_shop_market_sell_wait",
  {
    timer = time
  } } );
  
  table.insert( objects, { "tmr_int_dialog_shop_market_sell_wait",
  {
    playing = true,
    time    = 1.0,
    endtrig = private.MarketTab_Tick
  } } );

  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.MarketTab_Sell ( id )

  if ( private.market.funcs.TrySell( id ) ) then

    local slot     = "anm_int_dialog_shop_sell_slot_"..id;
    local action   = "tab_3_frame_sell_action_"..id;
    local fade_out = "tab_3_frame_sold_action";
    local static   = "tab_3_frame_sold_static";

    local button = "btn_int_dialog_shop_sell_slot_sell_"..id;

    ObjSet( slot, { input = false } );

    AnimPlay( slot, action,
    function ()

      AnimPlay(
        slot,
        fade_out,
        function ()

          AnimPlay( slot, static, "" );
          ObjSet( slot, { sell_count = "" } );

          int.Button_SetText( button, "-" );

          private.MarketTab_UpdateSells();

        end
      );

    end );

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.MarketTab_Excahnge ()

  local purchase = private.market.purchase;
  local price    = private.MarketTab_GetBuyPrice();

  if ( private.market.funcs.TryBuy( purchase, price ) ) then

    cmn.PlaySnd( "exchange" );

    for id, count in pairs( purchase ) do

      if ( count > 0 ) then

        local slot = "anm_int_dialog_shop_buy_slot_"..id;
        local action = "tab_3_frame_buy_buy";
        local static = "tab_3_frame_buy_static";

        AnimPlay( slot, action, function () AnimPlay( slot, static, "" ); end );

      end;

    end;

    for id = 1, #purchase do

      if ( purchase[ id ] > 0 ) then

        purchase[ id ] = 0;
        private.MarketTab_ChangeBuyCount( id, 0 );

      end;

    end;

  elseif ( price <= 0 ) then

    private.funcs.ShowPopup(
        "str_dialog_shop_market_buy_empty_caption",
        "str_dialog_shop_market_buy_empty_text",
        "btn_int_dialog_shop_market_buy_accept",
        { 0, -150 }
      );

  end;

end;
--------------------------------------------------------------------------------
function private.MarketTab_ChangeBuyCount ( id, inc )

  local market  = private.market.funcs;
  local current = private.market.purchase;
  local buttons = private.market.buttons;

  local plus_button = "btn_int_dialog_shop_buy_slot_plus_"..id;
  local plus_sprite = "anm_int_dialog_shop_buy_slot_plus_"..id;

  local minus_button = "btn_int_dialog_shop_buy_slot_minus_"..id;
  local minus_sprite = "anm_int_dialog_shop_buy_slot_minus_"..id;

  local slot = "anm_int_dialog_shop_buy_slot_"..id;
  local face = "anm_int_dialog_shop_buy_slot_resource_"..id;
  local fill = "anm_int_dialog_shop_buy_slot_fill_"..id;

  local accept = "btn_int_dialog_shop_market_buy_accept";

  local mutiset = {};

  local value = market.ShareValue( "buy", id );
  local limit = market.ShareLimit( "buy", id );

  local prew    = current[ id ];
  current[ id ] = current[ id ] + inc * value;

  -- Update buttons state
  if ( current[ id ] >= limit ) then

    current[ id ] = limit;
    
    buttons.plus[ id ].active = false;
    table.insert( multiset, { plus_button, { input = false } } );
    AnimPlay( plus_sprite, "btn_plus_off", "" );

  end;

  if ( current[ id ] <= 0 ) then

    current[ id ] = 0;

    buttons.minus[ id ].active = false;
    table.insert( multiset, { minus_button, { input = false } } );
    AnimPlay( minus_sprite, "btn_minus_off", "" );

  end;

  if ( current[ id ] < limit ) and not ( buttons.plus[ id ].active ) then

    buttons.plus[ id ].active = true;
    table.insert( multiset, { plus_button, { input = true } } );
    AnimPlay( plus_sprite, "plus", "" );

  end;

  if ( current[ id ] > 0 ) and not ( buttons.minus[ id ].active ) then

    buttons.minus[ id ].active = true;
    table.insert( multiset, { minus_button, { input = true } } );
    AnimPlay( minus_sprite, "minus", "" );

  end;

  -- Set count
  if ( current[ id ] > 0 ) then

    local slot_params =  { count_resource  = int.NumberForm( current[ id ] ) };
    table.insert( multiset, { slot, slot_params } );

  else

    local slot_params = { count_resource  = ""; };
    table.insert( multiset, { slot, slot_params } );

  end;

  ObjMultiSet( multiset );

  local action = "tab_3_frame_buy_press";
  local static = "tab_3_frame_buy_static";

  if ( ObjGet( slot ).animfunc == static ) then

    AnimPlay( slot, action,  function () AnimPlay( slot, static, "" ); end );

  end;

  -- Fill animation
  local fill_params = ObjGet( fill );
  local fill_size    = fill_params.croprect_h;
  local fill_current = fill_params.croprect_y;

  local fill_next = current[ id ] / limit;

  ObjStopAnimate( fill, "croprect_y" );
  ObjAnimate( fill, "croprect_y", 0, 0, "",
  {
    0.0, 0,     fill_current,
    0.3, 0, - ( fill_next - 0.5 ) * fill_size
  } );

  -- Face animation
  local sleep = "sleep_resource"..id;
  local awake = "static_resource"..id;

  local face_func = ObjGet( face ).animfunc

  if ( current[ id ] > 0 and face_func == sleep ) then

    AnimPlay( face, awake,  function () AnimPlay( face, sleep, "" ); end );

  elseif ( current[ id ] <= 0 and face_func == awake ) then

    AnimPlay( face, sleep, "" );

  end;

  -- Update price
  int.Button_SetPrice( accept, private.MarketTab_GetBuyPrice(), true );

end;
--------------------------------------------------------------------------------
function private.MarketTab_TapPlus ( id )

  local button = "btn_int_dialog_shop_buy_slot_plus_"..id;
  local sprite = "anm_int_dialog_shop_buy_slot_plus_"..id;

  local state = private.market.buttons.plus[ id ];

  -- Stop monkey-klicking
  if ( state.busy ) then return; end;

  if ( state.active and state.pressed ) then

    cmn.PlaySnd( "increase_flies" );
    private.MarketTab_ChangeBuyCount( id, 1 );

    -- Check again before animation
    if ( state.active and state.pressed ) then

      state.busy = true;
      AnimPlay( sprite, "plus",
      function ()

        state.busy = false;
        private.MarketTab_TapPlus( id );
      
      end );

    end;
    
  end;

end;
--------------------------------------------------------------------------------
function private.MarketTab_TapMinus ( id )

  local button = "btn_int_dialog_shop_buy_slot_minus_"..id;
  local sprite = "anm_int_dialog_shop_buy_slot_minus_"..id;

  local state = private.market.buttons.minus[ id ];

  -- Stop monkey-klicking
  if ( state.busy ) then return; end;

  if ( state.active and state.pressed ) then

    cmn.PlaySnd( "decrease_flies" );
    private.MarketTab_ChangeBuyCount( id, -1 );

    -- Check again before animation
    if ( state.active and state.pressed ) then

      state.busy = true;
      AnimPlay( sprite, "minus",
      function ()

        state.busy = false;
        private.MarketTab_TapMinus( id );
      
      end );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.MarketTab_GetBuyPrice ()

  local price = 0;

  for i, count in pairs( private.market.purchase ) do

    local share_value = private.market.funcs.ShareValue( "buy", i );
    local share_price = private.market.funcs.SharePrice( "buy", i );

    price = price + ( count / share_value ) * share_price;

  end;

  return price;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Market_SetNotification ( flag )
  
  ObjSet( "anm_int_dialog_shop_tab_market_notification",
  {
    active  = flag,
    visible = flag
  } );
  
end;
--------------------------------------------------------------------------------
function  public.Market_Sell_SetDefaults ()
  
  if ( private.market.tab_created ) then

    private.MarketTab_UpdateSells();

  end;
  
end;
--------------------------------------------------------------------------------