-- name=int_dialog_social_friends_choose
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.int_w = int.H();
  private.int_h = int.W();
  private.int_o = int.Orientation();
  
  private.CreateButtons();
  private.Adapt();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( friend_list, choose_max, func_send, func_close )

  if ( private.active ) then

    return;

  end;

  private.active = true;

  private.friend_list = friend_list;
  private.choose_max  = choose_max;
  private.FuncSend    = func_send;
  private.FuncClose   = func_close;

  private.friends_chosen = {};
  
  private.SetDefault( friend_list );

  int.Button_Disable( "btn_int_dialog_social_friends_choose_button_send" );

  ObjSet( "anm_int_dialog_social_friends_choose_window",
    {
      animfunc = "open_"..private.int_o,
      playing  = false,
      frame    = 0,
      tt_desc  = ne.strings.Get( "str_int_dialog_social_friends_choose_description", choose_max )
    }
  );

  int.Dialog_Open( "social_friends_choose", function ()

    AnimPlay(
      private.window,
      "open_"..private.int_o,
      function()

        AnimPlay(
          private.window,
          "static_no_btn_"..private.int_o,
          ""
        );

      end
    );

  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_OnHide )

  if ( not private.active ) then

    return;

  end;

  private.active = false;

  local func_hide = function ()

    AnimPlay(
      private.window,
      "close_"..private.int_o,
      function()

        int.Dialog_Close( "social_friends_choose", function ()

          private.friends_chosen = nil;
          private.btn_shown      = nil;

          ObjDelete( "scc_int_dialog_social_friends_choose" );

          if ( func_OnHide ) then

            func_OnHide();

          end;

        end );

      end
    );

  end;

  if ( private.btn_shown ) then

    AnimPlay(
      private.window,
      "static_btn_hide_"..private.int_o,
      func_hide
    );

    return;

  end;

  func_hide();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.CreateButtons ()
  
  int.Button_Create(
    "btn_int_dialog_social_friends_choose_close",
    {
      design = "close"
    },
    function()
      
      private.FuncClose();
      
    end
  );

  int.Button_Create(
    "btn_int_dialog_social_friends_choose_button_send",
    {
      title  = "str_int_dialog_social_friends_choose_button_send",
      color  = "blue",
      design = "medium"
    },
    function()
      
      private.FuncClose();
      private.FuncSend( private.friends_chosen );
      
    end
  );

end;
--------------------------------------------------------------------------------
function private.Adapt ()
  
  private.this   = "obj_int_dialog_social_friends_choose";
  private.window = "anm_int_dialog_social_friends_choose_window";

  private.window_area = {
    portrait  = { -230, -365, 460, 825 },
    landscape = { -230, -365, 460, 825 }
  };

  int.Center_Object( private.this );
  int_window.SetArea( private.this, private.window_area[ private.int_o ] );

  ObjSet( private.window,
    {
      tt_title = ne.strings.Get( "str_int_dialog_social_friends_choose_title" )
    }
  );

end;
--------------------------------------------------------------------------------
function private.SetDefault ( friend_list )

  -- JK: for debug
  --[[friend_list = {
    { userId = "1", name = "John", url = "" },
    { userId = "2", name = "Mark", url = "" }
  };]]
  
  ObjClone(
    "scc_int_dialog_social_friends_choose_default",
    "_rename",
    0,
    "anm_int_dialog_social_friends_choose_window"
  );

  ObjSet(
    "scc_int_dialog_social_friends_choose_default_rename",
    {
      name = "scc_int_dialog_social_friends_choose"
    }
  );

  local multiset = {};

  for f, friend in ipairs( friend_list ) do

    ObjClone( "anm_int_dialog_social_friends_choose_friend", "_"..friend.userId, true, "scc_int_dialog_social_friends_choose" );

    common.TapClear( "anm_int_dialog_social_friends_choose_friend_"..friend.userId );
    common.TapSet( "anm_int_dialog_social_friends_choose_friend_"..friend.userId, 
    {
      func_true = function ()
        private.Check( friend.userId, friend_list );
      end
    } );

    if ( friend.url == '' ) then
      friend.url = nil;
    end;

    table.insert( multiset, { "anm_int_dialog_social_friends_choose_friend_"..friend.userId, { tt_name = friend.name } } );
    table.insert( multiset, { "spr_int_dialog_social_friends_choose_friend_"..friend.userId, { 
      url = friend.url,
      res = "assets/content/interface/int_dialog_stage_friends/default_0",
      default_res = "assets/content/interface/int_dialog_stage_friends/default_0",
      trg_resloaded = function ()
        int.ImageAdjust( "spr_int_dialog_social_friends_choose_friend_"..friend.userId, 90 ); 
      end } } );

  end;

  ObjMultiSet( multiset );

  for f, friend in ipairs( friend_list ) do
    int.ImageAdjust( "spr_int_dialog_social_friends_choose_friend_"..friend.userId, 90 );
  end;

  -- -- Dummy -- -- --
  local friends_visible = 6;
  if ( #friend_list < friends_visible ) then
    for f = ( #friend_list + 1 ), friends_visible do
      ObjCreate( "obj_int_dialog_social_friends_choose_friend_"..f, "obj" );
      ObjAttach( "obj_int_dialog_social_friends_choose_friend_"..f, "scc_int_dialog_social_friends_choose" );
    end;
  end;

end;
--------------------------------------------------------------------------------
function private.Check ( userId, friend_list )

  local target_animfunc;
  local multiset         = {};
  local set_all_disabled = nil;
  local friend_num       = cmn.IsEntityInList( private.friends_chosen, userId );

  if ( #private.friends_chosen >= private.choose_max ) and ( not friend_num ) then
    return;
  end;

  if ( not friend_num ) then

    table.insert( private.friends_chosen, userId );
    target_animfunc = "checked";

    if ( #private.friends_chosen == private.choose_max ) then
      set_all_disabled = true;
    end;

  else

    table.remove( private.friends_chosen, friend_num );
    target_animfunc = "unchecked";

    if ( #private.friends_chosen == private.choose_max - 1 ) then
      set_all_disabled = false;
    end;

  end;

  table.insert( multiset, { 
    "anm_int_dialog_social_friends_choose_friend_"..userId, { 
      animfunc = target_animfunc, playing = true, frame = 0 } } );

  if ( set_all_disabled ~= nil ) then

    local other_animfunc;

    if ( set_all_disabled ) then
      other_animfunc = "inactive";
    else
      other_animfunc = "unchecked";
    end;

    for f, friend in ipairs( friend_list ) do

      if ( not cmn.IsEntityInList( private.friends_chosen, friend.userId )) then
        table.insert( multiset, { 
          "anm_int_dialog_social_friends_choose_friend_"..friend.userId, { 
            animfunc = other_animfunc, playing = true, frame = 0 } } );
      end;

    end;

  end;

  if ( #private.friends_chosen > 0 ) then

    if ( not private.btn_shown ) then

      private.btn_shown = true;

      int.Button_Enable( "btn_int_dialog_social_friends_choose_button_send" );

      AnimPlay(
        private.window,
        "static_btn_show_"..private.int_o,
        ""
      );

    end;       

  else

    private.btn_shown = nil;
    
    AnimPlay(
      private.window,
      "static_btn_hide_"..private.int_o,
      function ()

        int.Button_Disable(
          "btn_int_dialog_social_friends_choose_button_send"
        );

      end
    ); 

  end;

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------