-- name=int_dialog_social_mail
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.app_w = int.W();
  private.app_h = int.H();
  private.app_o = int.Orientation();
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.CreateButtons();
  private.Adapt();

  private.strings =
  {
    letter_title_help   = ne.strings.Get( "str_int_dialog_social_mail_title_help" ),
    letter_title_gift   = ne.strings.Get( "str_int_dialog_social_mail_title_gift" ),
    letter_title_thanks = ne.strings.Get( "str_int_dialog_social_mail_title_thanks" ),
    letter_title_custom = ne.strings.Get( "str_int_dialog_social_mail_title_custom" ),
    notavailable        = ne.strings.Get( "str_int_dialog_social_mail_heart_unavailable" ),
    removed             = ne.strings.Get( "str_int_dialog_social_mail_letter_removed" ),
    sent                = ne.strings.Get( "str_int_dialog_social_mail_letter_sent" ),
    received            = ne.strings.Get( "str_int_dialog_social_mail_letter_received" ),
    thank               = ne.strings.Get( "str_int_dialog_social_mail_thanks" )
  };

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()
  
  private.this_obj = "obj_int_dialog_social_mail";

  int.Center_Object( private.this_obj );

  private.window_area = {
    portrait  = { -279, -388, 540, 788 },
    landscape = { -223, -304, 433, 631 }
  };
  
  int_window.SetArea( private.this_obj, private.window_area[ private.app_o ] );
  
  ObjSet( "anm_int_dialog_social_mail_window",
  {
    animfunc = private.app_o,
    title    = ne.strings.Get( "str_int_dialog_social_mail_caption" )
  } );
  
end;
--------------------------------------------------------------------------------
function private.CreateButtons ()
  
  int.Button_Create(
    "btn_int_dialog_social_mail_close",
    {
      design = "close"
    },
    function()
      
      private.FuncHide();
      
    end
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( 
    mail, 
    func_read_message, 
    func_send_heart, 
    func_get_heart,
    func_obtain_attachment,
    func_send_thanks, 
    func_hide,
    func_toppanel_show
  )

  private.FuncReadMessage = func_read_message;
  private.FuncSendHeart   = func_send_heart;
  private.FuncGetHeart    = func_get_heart;
  private.FuncObtain      = func_obtain_attachment;
  private.FuncSendThanks  = func_send_thanks;
  private.FuncHide        = func_hide;

  -- JK: debug
  --local dummy = function () end;
  --private.FuncReadMessage = dummy;
  --private.FuncSendHeart   = dummy;
  --private.FuncGetHeart    = function () return true; end;
  --private.FuncObtain      = function () return true; end;
  --private.FuncSendThanks  = dummy;
  --private.FuncHide        = dummy;

  private.swiped = {};

  private.SetDefault( mail );

  private.active = true;

  int.Dialog_Open( "social_mail", func_toppanel_show );
  
  cmn.PlaySnd( "daily_quest" );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_end )

  if ( not private.active ) then
    
    return;
    
  end;

  private.active = false;
  
  if ( int.Popup_Flat_AttachedTo() ) then
    
    int.HidePopupFlat();
    
  end;

  int.Dialog_Close( "social_mail", function()

    private.Cleanup();

    if ( func_end ) then

      func_end();

    end;

  end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SetDefault ( mail )

  local objects = {};
  local types   = 
  {
    custom = "red",   -- custom letter    
    gift   = "green", -- letter with attached heart    
    help   = "blue",  -- letter with heart request
    thanks = "blue"   -- letter with 'thanks' message
  };
 
  ObjClone( 
    "scc_int_dialog_social_mail_slots_default", 
    "_rename", 
    0, 
    "anm_int_dialog_social_mail_window" 
  );

  ObjSet( "scc_int_dialog_social_mail_slots_default_rename",
  { 
    name = "scc_int_dialog_social_mail_slots"
  } );
  
  local empty_mail = ( #mail == 0 );
    
  m_object.Set( "anm_int_dialog_social_empty", {
    visible = empty_mail,
    active  = empty_mail
  }, objects );
  
  m_object.Set( "anm_int_dialog_social_mail_window", {
    tt_desc = tern( empty_mail )( "str_int_dialog_social_mail_empty", "" )
  }, objects );
  
  for i = 1, #mail do

    local send_thanks = true;
    local message_type = "custom";
    
    for key, value in pairs( types ) do
      if ( mail[ i ].text.string:find( key ) ) then
        message_type = key;
        break;
      end;
    end;

    ObjClone( 
      "obj_int_dialog_social_mail_slot", 
      "_"..i, 
      true, 
      "scc_int_dialog_social_mail_slots" 
    );
    
    -- Set message and title
    local title, message = "dummy", "dummy";
    local msgtype, str = message_type, mail[ i ].text.string;
    local params = mail[ i ].text.params;
    
    if ( msgtype ~= "custom" or str == "" ) then
    
      title = private.strings[ "letter_title_"..message_type ];
      
      local attach = ne.strings.Get(
        "str_int_dialog_social_mail_attach_heart", 
        mail[ i ].attach.name
      );

      message = ne.strings.Get(
        "str_int_dialog_social_mail_message_"..message_type,
        mail[ i ].from.name,
        attach
      );
    
    else
    
      title   = StringGet( str.."_title", table.unpack( params ) );
      message = StringGet( str, table.unpack( params ) );
    
    end;
      
    local expdate = tern( mail[ i ].expDate.month == 0 and 
                          mail[ i ].expDate.day == 0 and
                          mail[ i ].expDate.year == 0 )( "", 
                          ne.strings.Get(
                            "str_int_dialog_social_mail_expdate",
                            mail[ i ].expDate.month,
                            mail[ i ].expDate.day,
                            mail[ i ].expDate.year
                         ) );

    table.insert( objects,
      { "anm_int_dialog_social_mail_slot_"..i,
        {
          animfunc = "mail_static"
        }
      }
    );
    
    local reward_count = 0;
    if ( mail[ i ].attach and mail[ i ].attach.count ) then
      reward_count = mail[ i ].attach.count;
    end;
    
    reward_count = tern( reward_count > 1 )( reward_count, "" );

    table.insert( objects,
      { "anm_int_dialog_social_mail_letter_"..i,
        {
          animfunc     = types[ message_type ],
          letter_title = title,
          deleted      = expdate,
          message      = message,
          tt_count     = reward_count
        }
      }
    );

    if ( mail[ i ].from.avatar == '' ) then
      mail[ i ].from.avatar = nil;
    end;

    table.insert( objects,
      { "spr_int_dialog_social_mail_letter_"..i,
        {
          url = mail[ i ].from.avatar,
          res = "assets/content/interface/social/default_avatar",
          default_res = "assets/content/interface/social/default_avatar",
          trg_resloaded = function ()

            int.ImageAdjust(
              "spr_int_dialog_social_mail_letter_"..i,
              100
            );

          end
        }
      }
    );

    local btn_action_name = "btn_int_dialog_social_mail_letter_action_"..i;
    
    -- Default button is "DELETE".
    int.Button_Create(
      btn_action_name,
      {
        color  = "green",
        design = "medium"
      },
      function ()

        private.Message_MarkRemoved( i, mail[ i ].messageId );

      end
    );
    
    local btn_delete_name = "btn_int_dialog_social_mail_letter_delete_"..i;
    
    -- Default button is "DELETE".
    int.Button_Create(
      btn_delete_name,
      {
        design = "delete"
      },
      function ()

        private.Message_MarkRemoved( i, mail[ i ].messageId );

      end
    );    

    int.Button_SetText( btn_action_name, "str_int_dialog_social_mail_btn_delete" );
    
    table.insert( objects,
    { "spr_int_dialog_social_mail_letter_attach_"..i, {
        active = false, visible = false, input = false
    } } );

    if ( mail[ i ].attach ) then
      
      local func_obtain_heart = function ()
        if ( private.FuncGetHeart( mail[ i ].attach.count ) ) then

          private.Message_MarkReceived( i, mail[ i ].messageId );
          if ( send_thanks ) then
            private.FuncSendThanks( mail[ i ].from.userId );
          end;

        else

          m_object.Set( "anm_int_dialog_social_mail_slot_"..i, {
            notavailable = private.strings.notavailable
          } );

          AnimPlay( "anm_int_dialog_social_mail_slot_"..i,
            "mail_notavailable",
            function ()
              AnimPlay( "anm_int_dialog_social_mail_slot_"..i,
                "mail_static",
                "" );
            end
          );

        end;
      end;
      
      local func_obtain_other = function ( button_name )
        
        local is_obtained, reason = private.FuncObtain( mail[ i ].attach );
        
        if ( is_obtained ) then
        
          private.Message_MarkReceived( i, mail[ i ].messageId );
        
        elseif ( reason == "wait" ) then
          
          int.ShowPopupFlat(
            "str_dialog_social_mail_locked_title",
            "str_dialog_social_mail_locked_message",
            button_name,
            { 0, -130 }
          );
          
        end;
        
      end;

      int.Button_SetText( btn_action_name, "str_int_dialog_social_mail_btn_claim" );
      
      int.Button_EventSet( btn_action_name,
        function ()
      
          tern( message_type == "gift" )(
            func_obtain_heart,
            function() func_obtain_other( btn_action_name ); end
          )();
      
        end
      );
    
      local reward_name  = mail[ i ].attach.name;
      local reward_type  = mail[ i ].attach.type;
      local scales =
      {
        booster = 0.6,
        chest   = 0.7
      };
      local reward_scale = scales[ reward_type ] or 1.0;
      
      if ( reward_type == "gold" ) then
      
        reward_name = "gold_1";
      
      elseif ( reward_type == "chest" and reward_name:find( "event" ) ) then
      
        -- TODO: Fix it in 1.12
        reward_name = "woodchest";
      
      end;
      
      local res = m_content.Res( reward_type, reward_name );
      if ( message_type == "help" or message_type == "thanks" ) then
        res = "assets/content/interface/social/icon_"..message_type;
      end;
      
      table.insert( objects,
      { "spr_int_dialog_social_mail_letter_attach_"..i,
        {
          active  = true, visible = true, input = false,
          res     = res,
          scale_x = reward_scale,
          scale_y = reward_scale
        }
      } );
      
    end;
    
    -- Gift message (heart). Button "CLAIM".
    if ( message_type == "gift" ) then
      --
    end;

    -- Help message. Button "SEND".
    if ( message_type == "help" ) then
      
      int.Button_SetText( btn_action_name, "str_int_dialog_social_mail_btn_help" );
      int.Button_EventSet( btn_action_name,
        function ()
          private.Message_MarkSent( i, mail[ i ].messageId );
          private.FuncSendHeart( mail[ i ].from.userId );
        end );

    end;

    -- Thanks message. Button "DELETE".
    if ( message_type == "thanks" ) then

      int.Button_SetText( btn_action_name, "str_int_dialog_social_mail_btn_delete" );
      int.Button_EventSet( btn_action_name,
        function ()
          private.Message_MarkRemoved( i, mail[ i ].messageId );
        end );
          
    end;
    
    -- Custom message (event, admin, etc).
    if ( message_type == "custom" ) then
      
      table.insert( objects, { "spr_int_dialog_social_mail_letter_"..i,
      {
        url = mail[ i ].from.avatar,
        res = "assets/content/interface/social/admin_avatar",
        default_res = "assets/content/interface/social/admin_avatar",
        trg_resloaded = function ()
          int.ImageAdjust( "spr_int_dialog_social_mail_letter_"..i, 100 );
        end
      } } );

    end;

  end;
    
  tern( #objects > 0 )( function() ObjMultiSet( objects ); end, 
                        function() end )(); 

  for i = 1, #mail do
    int.ImageAdjust( "spr_int_dialog_social_mail_letter_"..i, 100 );
  end;

  --JK: dummies
  if ( #mail < 4 ) then

    for i = ( #mail + 1 ), 4 do

      ObjClone( 
        "anm_int_dialog_social_mail_slot", 
        "_"..i, 
        0, 
        "scc_int_dialog_social_mail_slots" 
      );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Cleanup ()

  ObjDelete( "scc_int_dialog_social_mail_slots" );

  int.Buttons_Delete(
    "int_dialog_social_mail",
    {
      "btn_int_dialog_social_mail_close"
    }
  );

  private.swiped = {};

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Message_MarkRemoved ( index, messageId )

  int.Input_Off( "anm_int_dialog_social_mail_slot_"..index );

  private.FuncReadMessage( messageId );

  ObjSet( "anm_int_dialog_social_mail_slot_"..index,
    {
      removed = private.strings.removed,
    }
  );

  AnimPlay(
    "anm_int_dialog_social_mail_slot_"..index,
    "mail_delete",
    ""
  );

end;
--------------------------------------------------------------------------------
function private.Message_MarkReceived ( index, messageId )

  int.Input_Off( "anm_int_dialog_social_mail_slot_"..index );

  private.FuncReadMessage( messageId );

  ObjSet( "anm_int_dialog_social_mail_slot_"..index,
    {
      received = private.strings.received
    }
  );

  AnimPlay(
    "anm_int_dialog_social_mail_slot_"..index,
    "mail_received",
    ""
  );

end;
--------------------------------------------------------------------------------
function private.Message_MarkSent ( index, messageId )

  int.Input_Off( "anm_int_dialog_social_mail_slot_"..index );

  private.FuncReadMessage( messageId );

  ObjSet( "anm_int_dialog_social_mail_slot_"..index,
    {
      sent = private.strings.sent
    }
  );

  AnimPlay(
    "anm_int_dialog_social_mail_slot_"..index,
    "mail_sent",
    ""
  );

end;
--------------------------------------------------------------------------------