-- name=int_dialog_stage_continue
--------------------------------------------------------------------------------
function public.__InitObjects ()
  
  private.app_w = int.W();
  private.app_h = int.H();
  private.app_o = int.Orientation();

  private.hint_time_popup    = 3;
  private.anim_time          = 0.3;
  private.cell_delta_x       = 90;
  private.cell_delta_scale_x = 0.2;
  private.cell_delta_scale_y = 0.05;
  private.cell_delta_color   = 0.2;
  private.wheel_open_y       = 0;
  private.wheel_close_y      = ( int.H() / 2 ) - 95;
  private.wheel_open_scale   = 1.0;
  private.wheel_close_scale  = 0.9;

  private.this = {
    dialog = "obj_int_dialog_stage_continue",
    obj = {      
      wheel = "obj_int_dialog_stage_continue_wheel"
    },
    anm = {
      wheel = "anm_int_dialog_stage_continue_wheel"
    },
    button = {
      accept = "btn_int_dialog_stage_continue_window_accept",
      close = "btn_int_dialog_stage_continue_close",
      wheel = {
        open  = "btn_int_dialog_stage_continue_window_open_wheel",
        close = "btn_int_dialog_stage_continue_wheel_close",
        run   = "btn_int_dialog_stage_continue_wheel_run",
        skip  = "btn_int_dialog_stage_continue_wheel_skip"
      }
    }    
  };

  local max = int.Dimension_Max();

  private.window_area = {
    default  = { -0.5 * max, -0.5 * max, max, max }
  };
  
  int_window.SetArea( private.this.dialog, private.window_area.default );

  private.Adapt();
  private.CreateButtons();
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  local multiset = {};

  int.Center_Object( private.this.dialog, 0, 0, multiset );

  local properties = {
    portrait = {
      pos_y       = ( private.app_h / 2 ) - 150,
      inputrect_y = -150,
      inputrect_h = 300
    },
    landscape = {
      pos_y       = ( private.app_h / 2 ) - 100,
      inputrect_y = -100,
      inputrect_h = 200
    }
  };

  m_object.Set(
    private.this.button.wheel.open,
    properties[ private.app_o ],
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  local close_handler = function ()

    private.FuncReject();

  end

  local wheel_close_handler = function()

    int_window.Handler_Close_Reset( "int_dialog_stage_continue", close_handler );

    local func_end = function ()

      local multiset = {};

      int.Input_On( private.this.button.accept, multiset );
      int.Input_On( private.this.button.close, multiset );
      int.Input_On( private.this.button.wheel.open, multiset );

      m_object.Multiset( multiset );

      -- Simulate window closed.
      mediator.Notify( "Interface_Window_WindowClosedEnd", "int_dialog_stage_continue" );

    end;

    if ( private.wheel_visual_state == "inactive" ) then

      private.WheelAnimDecreaseInactive( func_end );

    else

      private.WheelAnimDecreaseActive( func_end );

    end;

  end;

  common.TapSet( private.this.button.wheel.run,
  {
    func_true = function ()

      int_window.Handler_Close_Reset( "int_dialog_stage_continue", nil );

      local func_end = function ()
        int_window.Handler_Close_Reset( "int_dialog_stage_continue", close_handler );
      end;

      private.WheelLaunch( func_end );

    end
  } );

  int.Button_Create(
    private.this.button.accept,
    {
      title  = "str_int_dialog_stage_continue_use",
      color  = "green",
      design = "medium",
      price  = true
    },
    function()
      
      private.FuncBuyBooster();
      
    end
  );

  int.Button_Create(
    private.this.button.close,
    {
      design = "close"
    },
    close_handler
  );

  int.Button_Create(
    private.this.button.wheel.close,
    {
      design = "close",
      pos_x  = 274,
      pos_y  = -186,
      pos_z  = 10
    },
    wheel_close_handler
  );

  common.TapSet( private.this.button.wheel.open,
  {
    func_true = function ()
      int_window.Handler_Close_Reset( "int_dialog_stage_continue", nil );

      local multiset = {};

      int.Input_Off( private.this.button.accept, multiset );
      int.Input_Off( private.this.button.close, multiset );
      int.Input_Off( private.this.button.wheel.open, multiset );

      m_object.Multiset( multiset );

      local func_end = function ()
        int_window.Handler_Close_Reset( "int_dialog_stage_continue", wheel_close_handler );
      end;

      private.WheelAnimIncrease( private.wheel_logic_state, func_end );

    end
  } );

  int.Button_Create(
    private.this.button.wheel.skip,
    {
      design = "wheel",
      title  = "str_int_dialog_stage_continue_skip",
      timer  = "down",
      pos_y  = -12,
      pos_z  = 10
    },
    function()
      
      int.Input_Off( private.this.button.wheel.skip );

      if ( not private.FuncResetCooldown() ) then
        int.Input_On( private.this.button.wheel.skip );
      end;

      int_window.Handler_Close_Reset( "int_dialog_stage_continue", nil );

      local func_end = function ()
        if ( private.wheel_visual_state == "inactive" ) then
          int_window.Handler_Close_Reset( "int_dialog_stage_continue", wheel_close_handler );
          return;
        end;
        int_window.Handler_Close_Reset( "int_dialog_stage_continue", close_handler );
      end;

      private.WheelLaunch( func_end );
      
    end
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Show ( 
    info,                    continue_name,
    continue_cost,           continue_count,
    func_buy_booster,        func_turn,
    func_reset_cooldown,     func_item_get,
    func_attach_res_panel,   func_detach_res_panel, 
    func_get_flight_target,  func_popup_show,
    func_reject,             func_hero_fun,
    func_notify,             func_show_panel_stage_top,
    func_hide_panel_stage_top
  )

  private.wheel_visual_state = "hidden";

  local labels = {
    [ "extramoves" ] = "moves",
    [ "extralives" ] = "lives"
  };

  local continue_label = labels[ continue_name ];

  private.continue_name  = continue_name;
  private.continue_count = continue_count;

  private.FuncItemGet         = func_item_get;
  private.FuncBuyBooster      = func_buy_booster;
  private.FuncTurn            = func_turn;
  private.FuncResetCooldown   = func_reset_cooldown;
  private.TopPanel_Show       = func_attach_res_panel;
  private.TopPanel_Hide       = func_detach_res_panel;
  private.FuncGetFlightTarget = func_get_flight_target;
  private.FuncPopupShow       = func_popup_show;
  private.FuncReject          = func_reject;
  private.FuncHeroFun         = func_hero_fun;
  private.FuncNotifyReady     = info.notify_ready;
  
  private.Show_Stage_TopPanel = func_show_panel_stage_top;
  private.Hide_Stage_TopPanel = func_hide_panel_stage_top;
  
  private.wheel_logic_state = info.state;
  private.prizes            = info.prizes;
  private.spin_cost         = info.spin_cost;
  private.tmr_token         = info.tmr_token;

  private.WheelReset( private.prizes );

  ObjStopAnimate( private.this.obj.wheel, "scale_xy" );
  ObjStopAnimate( private.this.obj.wheel, "pos_y" );

  local description = "str_int_dialog_stage_continue_description_"..continue_name;

  if ( continue_count > 0 ) then

    description   = "str_int_dialog_stage_continue_description_"..continue_name.."_exist";
    continue_cost = 0;

  else

    private.TopPanel_Show();

  end;

  local multiset = {
    { private.this.dialog,             { alp = 1.0 } },
    { "obj_int_dialog_stage_continue_window",      { alp = 1.0 } },
    { "anm_int_dialog_stage_continue_wheel_win",   { visible = false, input = false, endtrig = "" } },
    { private.this.button.wheel.run,   { input = false } },
    { private.this.anm.wheel,       { animfunc = "static_close_arrow", playing = true, endtrig = "" } },
    { "anm_int_dialog_stage_continue_wheel_teeth", { animfunc = "tooth_open", playing = false, frame = 0 } },
    { private.this.button.wheel.close, { visible = false, input = false } },
    { private.this.obj.wheel, { 
      scale_x = private.wheel_close_scale, 
      scale_y = private.wheel_close_scale, 
      pos_y   = private.wheel_close_y 
    } }
  };

  if ( continue_count > 0 ) then

    table.insert( multiset, { "anm_int_dialog_stage_continue_window", {
      title       = ne.strings.Get( "str_int_dialog_stage_continue_caption_"..continue_name ),
      description = ne.strings.Get( description ),
      animfunc    = private.app_o.."_"..continue_label.."_"..tern( continue_count > 3 )( 3, continue_count ),
      label       = ne.strings.Get( "str_int_dialog_stage_continue_"..continue_name.."_label" ), 
      count       = "x"..tostring( continue_count ),
      playing     = true
    } } );
    
    table.insert( multiset, { "anm_price_int_dialog_stage_continue_window_accept", {
      active  = false,
      visible = false
    } } );
  
  else
      
    table.insert( multiset, { "anm_int_dialog_stage_continue_window", {
      title       = ne.strings.Get( "str_int_dialog_stage_continue_caption_"..continue_name ),
      description = ne.strings.Get( description ),
      animfunc    = private.app_o.."_"..continue_label.."_3",
      label       = ne.strings.Get( "str_int_dialog_stage_continue_"..continue_name.."_label" ), 
      count       = "x3",
      playing     = true
    } } );
    
    table.insert( multiset, { "anm_price_int_dialog_stage_continue_window_accept", {
      active   = true,
      visible  = true,
      tt_count = tostring( continue_cost )
    } } );
  
  end;

  int.Input_On( private.this.button.accept, multiset );
  int.Input_On( private.this.button.close, multiset );
  int.Input_On( private.this.button.wheel.open, multiset );
 
  int.Button_SetTextTag(
    private.this.button.wheel.skip,
    "price",
    info.spin_cost,
    multiset
  );

  ObjMultiSet( multiset );

  if ( private.wheel_logic_state == "recover" ) then
   
    private.WheelTimerCreate();
  
  end;

  cmn.PlaySnd( "click_energy" );

  int.Dialog_Open( "stage_continue", function ()

    -- Big Fish Games wish.
    -- private.FuncPopupShow( "tapme", { 140, 70 }, 0.75 );
    
    --JK: notify dialog is opened
    if ( func_notify ) then
      
      func_notify();
      
    end;

  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_end )

  private.WheelTimerDelete();
  private.WheelStopTeethHint();

  int.Dialog_Close( "stage_continue", func_end );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.CooldownStarted ( state )

  private.wheel_logic_state = state;

  private.WheelTimerCreate();

end;
--------------------------------------------------------------------------------
function public.CooldownFinished ( state )

  private.wheel_logic_state = state;

  if ( private.wheel_visual_state == "inactive" ) then

    private.WheelAnimInactiveToReady();

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.WheelStartTeethHint ()

  ObjSet( "tmr_int_dialog_stage_continue_wheel_waiting", {
    time    = private.hint_time_popup,
    playing = true,
    endtrig = function ()
      private.FuncPopupShow( "dragme", { 60, -110 } );
    end
  } );

end;
--------------------------------------------------------------------------------
function private.WheelStopTeethHint ()

  ObjSet( "tmr_int_dialog_stage_continue_wheel_waiting", {
    time    = 0.0,
    playing = false,
    endtrig = ""
  } );

end;
--------------------------------------------------------------------------------
function private.WheelReset ( content )

  private.current_pos = math.floor( #content / 2 );

  local multiset = {};
  
  for i = 1, 3 do

    table.insert( multiset, { "spr_int_dialog_stage_continue_wheel_teeth_2_"..i, { 
      res = "assets/content/boosters/"..private.continue_name } } );
    
    local props = private.WheelItemProps( private.WheelGetRandomRewards( content ) );

    table.insert( multiset, { "spr_int_dialog_stage_continue_wheel_teeth_main_"..i, { 
      res = props.res, scale_x = props.scale, scale_y = props.scale } } );
        
  end;
  
  ObjMultiSet( multiset );
  
  if ( not private.wheel_created ) then
   
    private.wheel_created = true;
 
  end;

end;
--------------------------------------------------------------------------------
function private.WheelLaunch ( func_end )

  if ( private.wheel_logic_state == "ready" ) then

    private.WheelStopTeethHint();

    local finish_pos = private.FuncTurn();
    private.WheelRun( finish_pos, func_end );

    private.WheelExitButtonHide();
        
    cmn.PlaySnd( "wheel_rotate" );

  end;

end;
--------------------------------------------------------------------------------
function private.WheelGetRandomRewards ( content )
  
  local reward_names = {};
  reward_names[ 1 ] = content[ math.random( 1, #content - 1 ) ].name;
  reward_names[ 2 ] = content[ math.random( 1, #content - 1 ) ].name;
  reward_names[ 3 ] = content[ math.random( 1, #content - 1 ) ].name;

  while 
  ( reward_names[ 3 ] == reward_names[ 2 ] ) or 
  ( reward_names[ 3 ] == reward_names[ 1 ] ) do

    reward_names[ 3 ] = content[ math.random( 1, #content - 1 ) ].name;

  end;
  
  return reward_names;
    
end;
--------------------------------------------------------------------------------
function private.WheelRun ( finish_pos, func_end )

  local content = private.prizes;

  local reward_name = content[ finish_pos ].name;

  local multiset = {};
  
  if ( reward_name == "empty" ) then

    local random = private.WheelGetRandomRewards( content );

    for i = 1, 3 do

      local props = private.WheelItemProps( random[ i ] );

      table.insert( multiset, { "spr_int_dialog_stage_continue_wheel_teeth_main_"..i, { 
        res = props.res, scale_x = props.scale, scale_y = props.scale } } );
      table.insert( multiset, { "spr_int_dialog_stage_continue_wheel_win_"..i, { 
        res = props.res, scale_x = props.scale, scale_y = props.scale } } );
      
    end;

  else

    local props = private.WheelItemProps( reward_name );

    for i = 1, 3 do

      table.insert( multiset, { "spr_int_dialog_stage_continue_wheel_teeth_main_"..i, { 
        res = props.res, scale_x = props.scale, scale_y = props.scale } } );
      table.insert( multiset, { "spr_int_dialog_stage_continue_wheel_win_"..i, { 
        res = props.res, scale_x = props.scale, scale_y = props.scale } } );

    end;

    table.insert( multiset, { "spr_int_dialog_stage_continue_win_icon", { 
      res = props.res } } );

  end;
  
  ObjMultiSet( multiset );
      
  cmn.PlaySnd( "wheel_working" );

  AnimPlay( private.this.anm.wheel, "spin", "" );
  AnimPlay( "anm_int_dialog_stage_continue_wheel_teeth", "tooth_spin", function ()

    private.WheelAnimEnd( finish_pos, func_end );

  end );
  
  ObjSet( "tmr_int_dialog_stage_continue_wheel_waiting", {
    time    = 1,
    playing = true,
    endtrig = function ()
      cmn.PlaySnd( "wheel_stop" );
       ObjSet( "tmr_int_dialog_stage_continue_wheel_waiting", {
    time    = 0.0,
    playing = false,
    endtrig = ""
  } );
    end
  } );

end;
--------------------------------------------------------------------------------
function private.WheelAnimEnd ( finish_pos, func_end )

  private.current_pos = finish_pos;

  AnimPlay(
    private.this.anm.wheel,
    "static_open",
    ""
  );

  local reward_name  = private.prizes[ finish_pos ].name;
  local reward_count = private.prizes[ finish_pos ].count;

  common.TapClear( "anm_int_dialog_stage_continue_wheel_win" );

  if ( reward_name == "empty" ) then

    private.WheelAnimActiveToInactive( func_end );

  else

    -- Win!
    private.FuncHeroFun();

    local func_reward_get;
    local multiset = {};
    
    m_object.Disable(
      "txt_int_dialog_stage_continue_win_icon_count",
      true,
      multiset
    );
    
    m_object.Disable(
      "txt_int_dialog_stage_continue_wheel_win_count",
      true,
      multiset
    );

    if ( reward_name == "gold"  ) or
       ( reward_name == "heart" ) then
      
      local target_name = private.FuncGetFlightTarget( reward_name );

      local count_pos = {
        gold  = { 40, 45 },
        heart = { 0, 0 }
      };

      m_object.Set(
        "txt_int_dialog_stage_continue_wheel_win_count",
        { 
          active  = true, 
          visible = true, 
          text    = reward_count,
          pos_x   = count_pos[ reward_name ][1],
          pos_y   = count_pos[ reward_name ][2]
        },
        multiset
      );

      func_reward_get = function ()

        int.Lock( true );
        
        local target_coords = int.Position(
          target_name,
          "anm_int_dialog_stage_continue_wheel_win"
        );

        local func_anim_end = function ()

          m_object.Set(
            "spr_int_dialog_stage_continue_win_icon",
            {
              pos_x = 0,
              pos_y = 0,
              alp   = 1.0
            }
          );
          
          ObjStopAnimate( "anm_int_dialog_stage_continue_wheel_win", "alp" );
          ObjStopAnimate( "spr_int_dialog_stage_continue_win_icon",  "alp" );
          ObjDetach( "spr_int_dialog_stage_continue_win_icon" );          

          private.FuncItemGet(
            private.prizes[ finish_pos ],
            function ()
            
              local func_on_end = function ()

                local multiset = {};

                int.Input_On( private.this.button.accept, multiset );
                int.Input_On( private.this.button.close, multiset );

                m_object.Multiset( multiset );

                func_end();

              end;
              
              private.WheelAnimDecreaseActive( func_on_end );
              int.Lock( false );
            
            end
          );

        end;
        
        local multiset = {};

        m_object.Set(
          "anm_int_dialog_stage_continue_wheel_win",
          { 
            visible = false, 
            input   = false, 
            endtrig = "", 
            playing = false 
          },
          multiset
        );

        m_object.Attach(
          "spr_int_dialog_stage_continue_win_icon",
          private.this.obj.wheel
        );

        m_object.Set(
          "txt_int_dialog_stage_continue_win_icon_count",
          {
            active  = true,
            visible = true,
            text    = reward_count
          },
          multiset
        );
        
        m_object.Multiset( multiset );

        ObjAnimate(
          "spr_int_dialog_stage_continue_win_icon", 
          "pos_xy", 0, 0,
          func_anim_end,
          { 
            0.0, 3, 0, 0, 
            0.6, 3, target_coords[1], target_coords[2]
          }
        );

        ObjAnimate(
          "spr_int_dialog_stage_continue_win_icon",
          "alp", 0, 0, "",
          {
            0.0, 0, 1.0,
            0.3, 0, 1.0, 
            0.6, 3, 0.0
          }
        );

        cmn.PlaySnd( "rotate_win" );

      end;

    elseif ( reward_name == "extramoves" ) or 
           ( reward_name == "extralives" ) or 
           ( reward_name == "extratime"  ) then
           
      if ( reward_name == "extramoves" ) then
        
        private.Show_Stage_TopPanel();
        
      end;

      func_reward_get = function ()

        local target_name   = private.FuncGetFlightTarget( reward_name );
        local target_coords = int.Position(
          target_name,
          "anm_int_dialog_stage_continue_wheel_win"
        );

        local func_anim_end = function ()

          m_object.Set(
            "spr_int_dialog_stage_continue_win_icon",
            {
              pos_x = 0,
              pos_y = 0,
              alp   = 1.0
            }
          );
          
          ObjStopAnimate( "anm_int_dialog_stage_continue_wheel_win", "alp" );
          ObjStopAnimate( "spr_int_dialog_stage_continue_win_icon",  "alp" );
          ObjDetach( "spr_int_dialog_stage_continue_win_icon" );

          private.FuncItemGet( private.prizes[ finish_pos ] );
          private.FuncReject( true );

        end;

        m_object.Set(
          "anm_int_dialog_stage_continue_wheel_win",
          { 
            visible = false, 
            input   = false, 
            endtrig = "", 
            playing = false 
          }
        );

        m_object.Attach(
          "spr_int_dialog_stage_continue_win_icon",
          private.this.obj.wheel
        );

        ObjAnimate(
          "spr_int_dialog_stage_continue_win_icon",
          "pos_xy", 0, 0, func_anim_end,
          { 
            0.0, 3, 0, 0, 
            0.6, 3, target_coords[1], target_coords[2]
          }
        );

        ObjAnimate(
          "spr_int_dialog_stage_continue_win_icon",
          "alp", 0, 0, "",
          {
            0.0, 0, 1.0,
            0.3, 0, 1.0, 
            0.6, 3, 0.0
          }
        );

        ObjAnimate(
          private.this.dialog,
          "alp", 0, 0, "",
          {
            0.0, 0, 1.0,
            0.3, 0, 1.0, 
            0.6, 3, 0.0
          }
        );

        cmn.PlaySnd( "rotate_win" );

      end;

    else

      func_reward_get = function ()

        ObjStopAnimate( "anm_int_dialog_stage_continue_wheel_win", "alp" );
        ObjStopAnimate( "spr_int_dialog_stage_continue_win_icon",  "alp" );

        m_object.Set(
          "anm_int_dialog_stage_continue_wheel_win",
          {
            input   = false,
            endtrig = ""
          }
        );
        
        ObjAnimate(
          "anm_int_dialog_stage_continue_wheel_win",
          "alp", 0, 0,
          function ()
            
            m_object.Hide( "anm_int_dialog_stage_continue_wheel_win" );
            
          end,
          {
            0.0, 3, 1.0, 
            0.3, 3, 0.0
          }
        );

        if ( reward_name ~= "empty" ) then

          private.FuncItemGet( private.prizes[ finish_pos ] );

        end;
        
        local func_on_end = function ()

          local multiset = {};

          int.Input_On( private.this.button.accept, multiset );
          int.Input_On( private.this.button.close, multiset );

          m_object.Multiset( multiset );

          func_end();

        end;

        private.WheelAnimDecreaseActive( func_on_end );

        cmn.PlaySnd( "rotate_win" );

      end;

    end;

    m_object.Set(
      "anm_int_dialog_stage_continue_wheel_teeth",
      { 
        animfunc = "tooth_close", 
        frame    = 0,
        playing  = false, 
        endtrig  = "" 
      },
      multiset
    );

    m_object.Set(
      "anm_int_dialog_stage_continue_wheel_win",
      { 
        alp      = 1.0, 
        input    = true, 
        visible  = true, 
        animfunc = "win",
        playing  = true,
        endtrig  = func_reward_get
      },
      multiset
    );

    m_object.Multiset( multiset );
    
    common.TapSet( "anm_int_dialog_stage_continue_wheel_win",
    {
      func_true = func_reward_get
    } );

  end;

end;
--------------------------------------------------------------------------------
function private.WheelAnimIncrease ( logic_state, func_end )

  local tme      = private.anim_time;
  local start_y  = private.wheel_close_y;
  local finish_y = private.wheel_open_y;
  
  private.WheelExitButtonShow();
  
  private.Hide_Stage_TopPanel();

  if ( logic_state == "ready" ) then

    private.wheel_visual_state = "ready";

    private.TopPanel_Hide();

    cmn.PlaySnd( "wheel_open_mouth" );

    ObjMultiSet( {
      { private.this.anm.wheel,         { animfunc = "open", playing = true, endtrig = function ()
        AnimPlay( private.this.anm.wheel, "static_open", "" );
        private.WheelStartTeethHint();
        ObjSet( private.this.button.wheel.run, { input = true } );
        private.FuncNotifyReady();
        ( func_end or function () end )();
      end } },
      { "anm_int_dialog_stage_continue_wheel_teeth",   { animfunc = "tooth_open", playing = true, endtrig = function ()
        AnimPlay( "anm_int_dialog_stage_continue_wheel_teeth", "tooth_static_open", "" );
      end } },
      { "obj_int_dialog_stage_continue_wheel_skip",    { input = false, visible = false } }
    } );

  else

    private.wheel_visual_state = "inactive";

    private.TopPanel_Show();

    ObjMultiSet( {
      { private.this.anm.wheel,       { animfunc = "static_close", playing = true, endtrig = "" } },
      { "anm_int_dialog_stage_continue_wheel_teeth", { animfunc = "tooth_open",   playing = false, frame = 0, endtrig = "" } },
      { "obj_int_dialog_stage_continue_wheel_skip",  { input = true, visible = true } },
      { private.this.button.wheel.run,   { input = false } },
    } );
    ObjAnimate( "obj_int_dialog_stage_continue_wheel_skip", "alp", 0, 0, function ()

      int.Input_On( private.this.button.wheel.skip );
      ( func_end or function () end )();

    end, 
    {
      0.0,   3, 0.0,
      tme,   3, 0.0,
      tme*2, 3, 1.0
    } );

  end;

  ObjAnimate( "obj_int_dialog_stage_continue_window", "alp", 0, 0, "", 
  {
    0.0, 3, 1.0,
    tme, 3, 0.0
  } );

  ObjAnimate( private.this.obj.wheel, "pos_y", 0, 0, "", 
  {
    0.0,   3, start_y,
    tme*2, 3, finish_y
  } );

  ObjAnimate( private.this.obj.wheel, "scale_xy", 0, 0, "", 
  {
    0.0,   3, private.wheel_close_scale, private.wheel_close_scale,
    tme*2, 3, private.wheel_open_scale, private.wheel_open_scale
  } );

end;
--------------------------------------------------------------------------------
function private.WheelAnimDecreaseActive ( func_end )

  private.wheel_visual_state = "hidden";

  if ( private.continue_count == 0 ) then
    private.TopPanel_Show();
  end;
  
  private.Show_Stage_TopPanel();

  local tme      = private.anim_time;
  local start_y  = private.wheel_open_y;
  local finish_y = private.wheel_close_y;

  ObjMultiSet( {
    { private.this.anm.wheel,       { animfunc = "close", playing = true, endtrig = function ()
      AnimPlay( private.this.anm.wheel, "static_close", "" );
    end } },
    { "anm_int_dialog_stage_continue_wheel_teeth", { animfunc = "tooth_close", playing = true, endtrig = function ()
      ObjSet( "anm_int_dialog_stage_continue_wheel_teeth", { animfunc = "tooth_open", playing = false, frame = 0 } )
    end } },
    { private.this.button.wheel.run,   { input = false } },
    { "anm_int_dialog_stage_continue_wheel_win",   { visible = false, input = false } }
  } );

  private.WheelExitButtonHide();

  ObjAnimate( "obj_int_dialog_stage_continue_window", "alp", 0, 0, "", 
  {
    0.0, 3, 0.0,
    tme, 3, 1.0
  } );

  ObjAnimate(
    private.this.obj.wheel,
    "pos_y", 0, 0,
    function ()

      int.Input_On( private.this.button.wheel.open );

      if ( func_end ) then

        func_end();

      end;

    end, 
    {
      0.0,   3, start_y,
      tme*2, 3, finish_y
    }
  );

  ObjAnimate( private.this.obj.wheel, "scale_xy", 0, 0, "", 
  {
    0.0,   3, private.wheel_open_scale, private.wheel_open_scale,
    tme*2, 3, private.wheel_close_scale, private.wheel_close_scale
  } );

  private.WheelStopTeethHint();  

end;
--------------------------------------------------------------------------------
function private.WheelAnimDecreaseInactive ( func_end )

  private.wheel_visual_state = "hidden";

  if ( private.continue_count > 0 ) then
    private.TopPanel_Hide();
  end;
  
  private.Show_Stage_TopPanel();

  local tme = private.anim_time;

  ObjMultiSet( {
    { "obj_int_dialog_stage_continue_wheel_skip", { input = false } },
    { private.this.button.wheel.run,  { input = false } },
    { "anm_int_dialog_stage_continue_wheel_win",  { visible = false, input = false } }
  } );
  
  private.WheelExitButtonHide();

  ObjAnimate( "obj_int_dialog_stage_continue_wheel_skip", "alp", 0, 0, function ()
    ObjSet( "obj_int_dialog_stage_continue_wheel_skip",  { visible = false } );
  end, 
  {
    0.0, 3, 1.0,
    tme, 3, 0.0
  } );

  ObjAnimate( "obj_int_dialog_stage_continue_window", "alp", 0, 0, "", 
  {
    0.0,   3, 0.0,
    tme,   3, 0.0,
    tme*2, 3, 1.0
  } );

  local start_y  = private.wheel_open_y;
  local finish_y = private.wheel_close_y;

  ObjAnimate(
    private.this.obj.wheel,
    "pos_y", 0, 0,
    function ()

      ObjSet( private.this.obj.wheel, { pos_y = private.wheel_close_y } );
      int.Input_On( private.this.button.wheel.open );
      if ( func_end ) then func_end(); end;

    end, 
    {
      0.0,   3, start_y,
      tme*2, 3, finish_y
    }
  );

  ObjAnimate( private.this.obj.wheel, "scale_xy", 0, 0, "", 
  {
    0.0,   3, private.wheel_open_scale, private.wheel_open_scale,
    tme*2, 3, private.wheel_close_scale, private.wheel_close_scale
  } );

  private.WheelStopTeethHint();

end;
--------------------------------------------------------------------------------
function private.WheelAnimInactiveToReady ()

  private.wheel_visual_state = "ready";

  private.TopPanel_Hide();

  local tme = private.anim_time;

  cmn.PlaySnd( "wheel_open_mouth" );

  ObjMultiSet( {
    { private.this.anm.wheel,       { animfunc = "open", playing = true, endtrig = function ()
      ObjSet( private.this.anm.wheel, { animfunc = "static_open", playing = true, endtrig = "" } );
    end } },
    { "anm_int_dialog_stage_continue_wheel_teeth", { animfunc = "tooth_open", playing = true, endtrig = "" } },
    { "obj_int_dialog_stage_continue_wheel_skip",  { visible = true } }
  } );

  ObjAnimate( "obj_int_dialog_stage_continue_wheel_skip", "alp", 0, 0, function ()
    ObjSet( "obj_int_dialog_stage_continue_wheel_skip",  { input = false } );
  end, 
  {
    0.0, 3, 1.0,
    tme, 3, 0.0
  } );

  ObjSet( private.this.button.wheel.run, { input = true } );

  private.WheelStartTeethHint();

end;
--------------------------------------------------------------------------------
function private.WheelAnimActiveToInactive ( func_end )

  private.wheel_visual_state = "inactive";

  private.TopPanel_Show();

  private.WheelExitButtonShow();

  local tme = private.anim_time;

  ObjMultiSet( {
    { private.this.anm.wheel,       { animfunc = "close", playing = true, endtrig = function ()
      ObjSet( private.this.anm.wheel, { animfunc = "static_close", playing = true, endtrig = "" } );
    end } },
    { "anm_int_dialog_stage_continue_wheel_teeth", { animfunc = "tooth_close", playing = true, endtrig = "" } },
    { "obj_int_dialog_stage_continue_wheel_skip",  { visible = true, input = true } }
  } );

  ObjAnimate( "obj_int_dialog_stage_continue_wheel_skip", "alp", 0, 0, function ()

    private.FuncPopupShow( "tryagain", { 50, -110 } );

    int.Input_On( private.this.button.wheel.skip );

    func_end();

  end, 
  {
    0.0, 3, 0.0,
    tme, 3, 1.0
  } );

  int.Input_Off( private.this.button.wheel.run );

end;
--------------------------------------------------------------------------------
function private.WheelTimerCreate ()

  if ( not ObjGet( "tmr_int_dialog_stage_continue_skip" ) ) then

    ObjCreate( "tmr_int_dialog_stage_continue_skip", "timer" );

  end;

  ObjAttach(
    "tmr_int_dialog_stage_continue_skip",
    private.this.dialog
  );

  private.WheelTimerUpdate();

end;
--------------------------------------------------------------------------------
function private.WheelTimerUpdate ()

  if ( TimeIsEventExist( private.tmr_token ) ) then

    local time = common.GetTimeFromSec(
      TimeGetEventRemainingTime( private.tmr_token )
    );

    time = int.Text_TimerFormat( time, "m:s" );

    int.Button_SetTime(
      private.this.button.wheel.skip,
      time
    );

    ObjSet( "tmr_int_dialog_stage_continue_skip",
      {
        time    = 1,
        playing = true,
        endtrig = private.WheelTimerUpdate
      }
    );

  else

    private.WheelTimerDelete();

  end;

end;
--------------------------------------------------------------------------------
function private.WheelTimerDelete ()

  if ( ObjGet( "tmr_int_dialog_stage_continue_skip" )) then
    ObjDelete( "tmr_int_dialog_stage_continue_skip" );
  end;

end;
--------------------------------------------------------------------------------
function private.WheelItemProps ( item_name )

  local props = {};

  if ( item_name == "extramoves" ) then

    item_name = private.continue_name;

    if ( item_name == "extramoves" ) then

      props.res   = "assets/content/boosters/extramoves";
      props.scale = 0.6;

    elseif ( item_name == "extralives" ) then

      props.res   = "assets/content/boosters/extralives";
      props.scale = 0.6;

    elseif ( item_name == "extratime" ) then

      props.res   = "assets/content/boosters/extramoves";
      props.scale = 0.6;

    end;

  elseif ( item_name == "heart" ) then 
    
    props.res   = "assets/content/items/heart";
    props.scale = 1.0;

  elseif ( item_name == "gold" ) then 

    props.res   = "assets/content/items/gold_1";
    props.scale = 1;

  end;

  return props;

end;
--------------------------------------------------------------------------------
function private.WheelExitButtonShow ( instant )

  local btn_name = private.this.button.wheel.close;

  ObjStopAnimate( btn_name, "alp" );

  if ( instant ) then

    ObjSet( btn_name, { alp = 1.0, input = true, visible = true, active = true } );

  else

    local tme = private.anim_time;

    ObjSet( btn_name, { alp = 0.0, active = true, input = true, visible = true } );
    ObjAnimate( btn_name, "alp", 0, 0, "", 
    {
      0.0,   3, 0.0,
      tme,   3, 0.0,
      tme*2, 3, 1.0
    } );

  end;

end;
--------------------------------------------------------------------------------
function private.WheelExitButtonHide ( instant )

  local btn_name = private.this.button.wheel.close;

  ObjStopAnimate( btn_name, "alp" );

  if ( instant ) then

    ObjSet( btn_name, { alp = 1.0, input = false, visible = false, active = false } );

  else

    local tme = private.anim_time;

    ObjSet( btn_name, { alp = 1.0, input = false } );
    ObjAnimate( btn_name, "alp", 0, 0, function ()
      ObjSet( btn_name, { alp = 1.0, input = false, visible = false, active = false } );
    end, 
    {
      0.0, 3, 1.0,
      tme, 3, 0.0
    } );

  end;

end;
--------------------------------------------------------------------------------
--*********************************************************************************************************************