-- name=int_dialog_stage_exit
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.this   = "obj_int_dialog_stage_exit";
  private.window = "anm_int_dialog_stage_exit_window";

  private.window_area = {
    default = { -272, -220, 544, 440 },
    heart   = { -272, -220, 544, 540 }
  };

  int.Center_Object( private.this );
  int_window.SetArea( private.this, private.window_area.default );

  int.Button_Create(
    "btn_int_dialog_stage_exit_exit",
    {
      title  = "str_dialog_stage_exit_exit",
      design = "medium",
      clear  = "window_outer_tap",
      color  = "blue",
      sound  = "ok_info"
    },
    function()
      
      private.Accept();
      
    end
  );

  int.Button_Create(
    "btn_int_dialog_stage_exit_close",
    {
      design = "close"
    },
    function()
      
      private.Reject();
      
    end
  );  
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show (
    stage_name,
    moves_done,
    heart_count,
    hearts_max,
    func_accept,
    func_reject,
    func_skin_attach,
    func_skin_detach
  )
  
  private.active = true;

  private.Accept      = func_accept;
  private.Reject      = func_reject;
  private.moves_done  = moves_done;
  private.heart_count = heart_count;
  private.Skin_Attach = func_skin_attach;
  private.Skin_Detach = func_skin_detach;

  private.hero = private.Skin_Attach( private.window );

  ObjSet( private.hero,
  {
    animfunc = "dialog",
    playing  = true,
    anim_tag = "hero"
  } );

  private.SetDefault( stage_name, heart_count, hearts_max );

  cmn.PlaySnd( "app_info" );

  int.Dialog_Open( "stage_exit" );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_execute, no_animation )

  private.active = false;
  
  if ( private.moves_done > 0   ) and
     ( not no_animation         ) then

    local func_end = function ()

     private.Close( func_execute );

    end;
  
    ObjSet( private.hero,
    {
      animfunc = "fail_dialog",
      playing  = true,
      anim_tag = "hero"
    } );
   
    AnimPlay( "anm_int_dialog_stage_exit_heart", "heart", func_end );

    AnimPlay( "anm_int_dialog_stage_exit_heart_dummy", "heart_dummy", function()

      ObjSet( "anm_int_dialog_stage_exit_heart",
      {
        heart_count = tostring( private.heart_count )
      } );

    end );

    local objects = {};

    table.insert( objects, { "btn_int_dialog_stage_exit_close",
    {
      input = false } } );
    
    table.insert( objects, { "btn_int_dialog_stage_exit_exit",
    {
      input = false } } );

    ObjMultiSet( objects );

    common.Animation_Fade_Out( 0.3, "btn_int_dialog_stage_exit_close" );

    cmn.PlaySnd( "heart_break" );

  else

    private.Close( func_execute );

  end;

end;
--------------------------------------------------------------------------------
function public.IsActive ()

  return private.active;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SetDefault ( stage_name, heart_count, hearts_max )

  local objects          = {};
  local animfunc         = "quit_nomoves_portrait";
  local caption_string   = "str_stage_name_"..stage_name;
  local attention_string = "str_dialog_stage_exit_message_default";

  if ( stage_name:find( "_daily" ) ) then

    caption_string = "str_stage_name_dailyquest";

  end;

  table.insert( objects, { "btn_int_dialog_stage_exit_close",
  {
    input = true,
    alp   = 1.0 } } );
  
    table.insert( objects, { "btn_int_dialog_stage_exit_exit",
  {
    input = true } } );

  if ( private.moves_done > 0 ) then

    animfunc = "quit_portrait";

    int_window.SetArea( private.this, private.window_area.heart );

    table.insert( objects, { "anm_int_dialog_stage_exit_heart",
    {
      heart_count = tostring( math.min( heart_count + 1, hearts_max ) ),
      active      = true,
      visible     = true } } );
    
    table.insert( objects, { "anm_int_dialog_stage_exit_character",
    {
      active  = false,
      visible = false } } );

  else

    table.insert( objects, { "anm_int_dialog_stage_exit_heart",
    {
      active  = false,
      visible = false } } );

    table.insert( objects, { "anm_int_dialog_stage_exit_character",
    {
      active  = true,
      visible = true } } );
    
    attention_string = "str_dialog_stage_exit_message_no_moves";

  end;
  
  table.insert( objects, { "anm_int_dialog_stage_exit_window",
  {
    animfunc  = animfunc,
    title     = ne.strings.Get( caption_string ),
    attention = ne.strings.Get( attention_string )
  } } );

  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------
function private.Close ( func_execute )

  int.Dialog_Close( "stage_exit", function ()

    private.Skin_Detach( private.hero );

    if ( func_execute ) then

      func_execute();

    end;

  end );

end;
--------------------------------------------------------------------------------