-- name=int_dialog_stage_fail
function public.__InitModule ()

  private.heart_count = 0;
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.this   = "obj_int_dialog_stage_fail";
  private.window = "anm_int_dialog_stage_fail_window"; 

  private.window_area = {
    default = { -272, -255, 544, 510 }
  };
  
  int_window.SetArea( private.this, private.window_area.default );
  
  private.Adapt();
  private.CreateButtons();
  private.CreateGoals();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show (
    stage_data,
    targets,
    heart_count,
    heart_max,
    func_close,
    func_retry,
    func_restore,
    func_skin_attach,
    func_skin_detach
  )

  private.active = true;

  private.heart_count = heart_count;
  private.Close       = func_close;
  private.Retry       = func_retry;
  private.Restore     = func_restore;
  private.Skin_Attach = func_skin_attach;
  private.Skin_Detach = func_skin_detach;

  private.hero = private.Skin_Attach( private.window );

  ObjSet( private.hero,
  {
    animfunc = "fail_dialog",
    playing  = true,
    anim_tag = "hero"
  } );

  local score   = 0;
  local objects = {};

  for g = 1, 3 do
    
    table.insert( objects, { "anm_int_dialog_stage_fail_goal_"..g, {
      visible  = false,
      active   = false,
      input    = false
    } } );
    
  end;

  if ( stage_data.type == "ride"  ) or
     ( stage_data.type == "bonus" ) then

    score = private.SetRide( targets, stage_data, objects );

  elseif( stage_data.type == "fight" ) then
    
    score = private.SetFight( targets, stage_data, objects );
    
  end;
  
  table.insert( objects, { "anm_int_dialog_stage_fail_heart", {
    heart_count = tostring( math.min( ( heart_count + 1 ), heart_max ) )
  } } );
  
  local background_res =
  "assets/content/"..stage_data.content.."/bg_"..stage_data.zone.."_"..stage_data.type;
  
  local title_string = "str_stage_name_"..stage_data.name;
  if ( stage_data.realtype == "dailyquest" ) then
    title_string = "str_stage_name_dailyquest";
  end;

  table.insert( objects, { "anm_int_dialog_stage_fail_window", {
    title       = title_string,
    animfunc    = "portrait",
    frame       = 0,
    playing     = false,
    bg          = background_res,
    goals       = "str_dialog_stage_fail_targets",
    description = "str_dialog_stage_fail_fail",
    score       = StringGet( "str_dialog_stage_fail_current_score", score )
  } } );

  ObjMultiSet( objects );

  int.Dialog_Open( "stage_fail", function()

    AnimPlay( "anm_int_dialog_stage_fail_heart", "heart", "" );
    
    if( heart_count==0 ) then
      
      cmn.PlaySnd("need_live");
      
    end;

    AnimPlay( "anm_int_dialog_stage_fail_heart_dummy", "heart_dummy", function()

      ObjSet( "anm_int_dialog_stage_fail_heart",
      {
        heart_count = tostring( heart_count )
      } );

    end );

  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_end )

  if ( not private.active ) then

    return;

  end;

  private.active = false;
  
  int.Dialog_Close(
    "stage_fail",
    function()

      private.Skin_Detach( private.hero );

      if ( func_end ) then

        func_end();

      end;

    end
 );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.CreateGoals ()
  
  for g = 1, 3 do
    
    ObjClone( 
      "anm_int_dialog_stage_fail_goal", 
      "_"..g, 
      1, 
      "anm_int_dialog_stage_fail_window" 
    );
    
    ObjSet( "anm_int_dialog_stage_fail_goal_"..g, {
      anim_tag = "goal_"..g
    } );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.CreateButtons ()
  
  int.Button_Create(
    "btn_int_dialog_stage_fail_close",
    {
      design = "close",
      sound  = "click_wood"
    },
    function()
      
      private.Close();
      
    end
  );
  
  int.Button_Create(
    "btn_int_dialog_stage_fail_retry",
    {
      title  = "str_dialog_stage_fail_retry",
      color  = "green",
      design = "medium"
    },
    function()
      
      if ( private.heart_count > 0 ) then

        private.Retry();

      else

        private.Restore();

      end;
      
    end
  );
  
end;
--------------------------------------------------------------------------------
function private.Adapt ()

  private.app_w       = int.W();
  private.app_h       = int.H();
  private.orientation = int.Orientation();
  
  int.Center_Object( "obj_int_dialog_stage_fail" );

end;
--------------------------------------------------------------------------------
function private.SetRide ( targets, stage_data, objects )

  local score;
  
  local resource = {
    default    = "items/icon_%s",
    DropItem   = stage_data.section.."/icon_%s_%s",
    DestroyGem = "items/icon_%s"
  };
  
  for t, target in ipairs( targets ) do

    local icon_postfix = "";
    local done_prefix  = "un";
    local text;
    
    if not ( target.type == "GainScore" ) then
    
      icon_postfix = "_icon";
      
      table.insert( objects, { "obj_int_dialog_stage_fail_goal_icon_"..t, { 
        visible = true
      } } );
      
      text = int.NumberForm( target.count, true );
      
      local res_part = string.format( resource[ target.type ] or resource.default, 
                                      string.lower( target.name ), 
                                      stage_data.section:match( "%d+" ) );
      
      table.insert( objects, { "spr_int_dialog_stage_fail_goal_icon_"..t, {
        res     = "assets/content/"..res_part,
        visible = true
      } } );
      
      table.insert( objects, { "anm_int_dialog_stage_fail_goal_icon_"..t, {
        visible = false
      } } );
      
    else
      
      score = int.NumberForm( target.progress or 0, true );
      text  = int.NumberForm( target.count or 0, true ); 
      
      table.insert( objects, { "obj_int_dialog_stage_fail_goal_icon_"..t, { 
        visible  = false
      } } );
      
    end;
    
    if ( target.done ) then
      
      done_prefix = "";
      
    end;
    
    table.insert( objects, { "anm_int_dialog_stage_fail_goal_"..t, {
      active   = true,
      visible  = true,
      animfunc = done_prefix.."performed"..icon_postfix,
      text     = text
    } } );

  end;
  
  return score;
  
end;
--------------------------------------------------------------------------------
function private.SetFight ( targets, stage_data, objects )
    
  table.insert( objects, { "anm_int_dialog_stage_fail_goal_2", {
    active   = true,
    visible  = true,
    animfunc = "unperformed",
    text     = ne.strings.Get(
        "str_dialog_stage_fail_battle",
        ne.strings.Get( "str_enemy_name_"..stage_data.enemy.name )
      )
  } } );
  
  table.insert( objects, { "obj_int_dialog_stage_fail_goal_icon_2", {
    visible = false
  } } );
  
end;
--------------------------------------------------------------------------------