-- name=int_dialog_starterpack
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.orientation = int.Orientation();

  private.area = {
    portrait  = { -260, -475, 520, 950 },
    landscape = { -207, -380, 416, 760 }
  };

  private.this = {
    dialog = "obj_int_dialog_starterpack",
    button = {
      close = "btn_int_dialog_starterpack_close",
      buy   = "btn_int_dialog_starterpack_buy"
    }
  };

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()
  
  int_window.SetArea( private.this.dialog, private.area[ private.orientation ] );
  
  int.Center_Object( private.this.dialog );
  
  int.Button_Create(
    private.this.button.close,
    {
      design = "close"
    },
    function () private.Reject(); end
  );
  
  int.Button_Create(
    private.this.button.buy,
    {
      design = "buy",
      timer  = ""
    },
    function () private.Accept(); end
  );

  -- JK: increase buy button area
  m_object.Set(
    private.this.button.buy,
    {
      inputrect_x = -200,
      inputrect_y = -450,
      inputrect_w =  400,
      inputrect_h =  500
    }
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( starterpack, func_accept, func_reject )
  
  private.active = true;
  
  private.Accept = func_accept;
  private.Reject = func_reject;
  
  private.SetDefaults( starterpack );
 
  private.Timer();
  
  AnimPlay(
    "anm_int_dialog_starterpack_window",
    "static_"..private.orientation,
    ""
  );  
  
  int.Dialog_Open( "starterpack" );
  
  cmn.StopMusic( "assets/content/audio/music/aud_map_mus" );
  cmn.PlaySnd( "starter_pack" );
  
  common.Timer_Create( "int_dialog_starterpack", "tmr_int_dialog_starterpack_music", 25.0, function ()
    common.Timer_Delete( "tmr_int_dialog_starterpack_music" );
    cmn.PlayMusic( "assets/content/audio/music/aud_map_mus", nil, nil, true );
  end );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_after )
  
  private.active = false;
  
  if ( m_object.Get( "tmr_int_dialog_starterpack_music" ) ) then
    common.Timer_Delete( "tmr_int_dialog_starterpack_music" );
    cmn.StopSnd( "starter_pack" );  
    cmn.PlayMusic( "assets/content/audio/music/aud_map_mus", nil, nil, true );
  end;
  
  int.Dialog_Close( "starterpack", func_after );

end;
--------------------------------------------------------------------------------
function private.Timer ()

  if ( not private.active ) then

    return;

  end;

  local multiset     = {};
  local total_time   = TimeGetEventRemainingTime( "mte_starterpack_cooldown" );
  local current_time = cmn.GetTimeFromSec( math.ceil( total_time ) );
  local time_string;
  
  if ( total_time <= 0 ) then

    common.Timer_Stop(
     "tmr_int_dialog_starterpack",
      multiset
    );

  else

    common.Timer_Start(
      "tmr_int_dialog_starterpack",
      1,
      function()

        private.Timer();

      end,
      multiset
    );
    
    local timer_format = tern( current_time.d == "00" )( "h:m", "d" );
    timer_format = tern( current_time.h == "00" )( "m:s", timer_format );

    time_string = int.Text_TimerFormat( current_time, timer_format );
    
  end;
  
  int.Button_SetTime( private.this.button.buy, time_string, multiset );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SetDefaults ( starterpack )
  
  local objects = {};  

  local multiset_params = {};  
  
  local boosters = { hammer = 1, magicgift = 2 };

  multiset_params.tt_title   = "str_int_dialog_starterpack_title";
  multiset_params.tt_percent = "str_int_dialog_starterpack_percent";
  multiset_params.tt_header  = "str_int_dialog_starterpack_header";
  
  for i = 1, #starterpack.currency_count do
    
    local starterpack_content = starterpack.currency_count[ i ];
    
    if ( starterpack_content.type == "gold" ) then
      multiset_params.tt_gold = int.NumberForm( starterpack_content.count );
    else
      multiset_params[ "tt_booster"..boosters[ starterpack_content.name ] ] = 
      ne.strings.Get( "str_int_dialog_starterpack_countx", int.NumberForm( starterpack_content.count ) ); 
    end;        
    
  end;
  
  local param0, param1, param2 = multiset_params.tt_gold, multiset_params.tt_booster1, multiset_params.tt_booster2;
  
  multiset_params.tt_description = ne.strings.Get( "str_int_dialog_starterpack_desc", param0, param1, param2 );
  
  m_object.Set( "anm_int_dialog_starterpack_window", multiset_params, objects );

  local price = tern( starterpack.price == "" )( ne.strings.Get( "str_dialog_shop_gold_buy" ),
                                                starterpack.price );

  int.Button_SetText( private.this.button.buy, price, objects );
  
  ObjMultiSet( objects );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.UpdatePrice ( price )

  if ( private.active and price and price ~= "" ) then
  
    int.Button_SetText( private.this.button.buy, price );
  
  end;

end;
--------------------------------------------------------------------------------