-- name=int_dialog_worldmap
function  public.__InitModule ()

  private.orientation = int.Orientation();

  private.scc_size =
  {
    portrait  = { w = 600, h = 800 }, 
    landscape = { w = 600, h = 600 } 
  };

  private.window_area =
  {
    portrait  = { -320, -420, 640, 900 },
    landscape = { -320, -320, 640, 700 }
  };

  private.section_list = {};
  private.section_current = "";
  private.skin = {};
  private.section_map  = { data = {}, id = {} };
  private.list_created = false;
  
end;
--------------------------------------------------------------------------------
function  public.__InitObjects ()

  int.Center_Object( "obj_int_dialog_worldmap" );

  int_window.SetArea( "obj_int_dialog_worldmap",
                      private.window_area[ private.orientation ] );

  int.Button_Create( "btn_int_dialog_worldmap_close",
    {
      design = "close"
    },
    function () private.Hide(); end
  );

  local size = private.scc_size[ private.orientation ];

  ObjSet( "scc_int_dialog_worldmap_sections",
  {
    inputrect_init = true,
    inputrect_x   = - size.w / 2,
    inputrect_y   = - size.h / 2,
    inputrect_w   =   size.w,
    inputrect_h   =   size.h,

    croprect_init = true,
    croprect_x    = - size.w / 2,
    croprect_y    = - size.h / 2,
    croprect_w    =   size.w,
    croprect_h    =   size.h
  } );

  ObjSet( "anm_int_dialog_worldmap_window",
  {
    animfunc = private.orientation,
    title    = StringGet( "str_int_dialog_worldmap_title" )
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Show ( section, list, skin, func_content, func_goto, unlock )

  private.Func_IsContentObsolete = func_content;
  private.Func_GoTo              = func_goto;

  private.section_current = section;
  private.section_list = list;
  private.skin = skin;
  private.IndexList();

  if not ( private.list_created ) then

    private.CreateListItems();

  end;

  private.UpdateListItems();
  private.Focus( section );

  int_window.Show( "int_dialog_worldmap",
  function ()

    if ( unlock ) then

      local id   = private.section_map.id[ section ] + 1;
      local name = private.section_list[ id ].name;
      local item = "anm_int_dialog_worldmap_slot_"..name;

      ObjSet( item, { animfunc = "island"..id.."_lock" } );
      private.Focus( name );

      int.Dialog_Open( "worldmap", 
      function()

        cmn.PlaySnd( "new_land" );
        AnimPlay( item, "island"..id.."_unlock", "" );

      end );

    else

      int.Dialog_Open( "worldmap" );

    end;

  end ); 

end;
--------------------------------------------------------------------------------
function private.Hide ( func_after )

  func_after = func_after or function () end;

  int.Dialog_Close( "worldmap", function ()

    int_window.Hide( "int_dialog_worldmap", func_after );

  end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Focus ( section_name )

  local scc = "scc_int_dialog_worldmap_sections";
  local id  = private.section_map.id[ section_name ];

  local scc_h        = private.scc_size[ private.orientation ].h;
  local visble_count = math.ceil( scc_h / 200 );
  local focus_at     = math.ceil( visble_count / 2 );

  local min_pos = 0;
  local max_pos = #private.section_list - visble_count;

  local scc_pos = id - focus_at;

  if     ( scc_pos < min_pos ) then scc_pos = min_pos;
  elseif ( scc_pos > max_pos ) then scc_pos = max_pos;
  end;

  ObjSet( scc, { scrollpos = scc_pos } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.CreateListItems ()

  local scc    = "scc_int_dialog_worldmap_sections";
  local slot   = "anm_int_dialog_worldmap_slot";
  local button = "btn_int_dialog_worldmap_slot_play";

  for id, section in pairs( private.section_list ) do

    local name = section.name;
    ObjClone( slot, "_"..name, true, scc );

    local item_button = button .. "_" .. name;

    int.Button_Create( item_button,
      {
        title  = "str_int_dialog_worldmap_play",
        design = "medium",
        color  = "green",
        sound  = "simply_click"
      },
      function () 

        if not ( private.Func_IsContentObsolete( name ) ) then

          private.Func_GoTo( id, private.Hide );

        end;
        
      end
    );

  end;

  private.list_created = true;

end;
--------------------------------------------------------------------------------
function private.UpdateListItems ()

  local slot   = "anm_int_dialog_worldmap_slot";
  local avatar = "anm_int_dialog_worldmap_marker"
  local str    = "str_int_dialog_worldmap_";

  local multiset = {};

  for id, section in pairs( private.section_list ) do

    local name = section.name;
    local item = slot .. "_" .. name;

    local params =
    { 
      animfunc = nil,
      frame    = 0, 
      playing  = true,
      island   = StringGet( str.."section_"..id )
    };

    if ( private.IsSectionCurrent( name ) ) then
    
      params.animfunc = "island"..id.."_current";
      params.stages   = private.GetSectionStages( name );
      
      local anm_avatar = avatar .. "_" .. name;
      local res = "assets/content/%s/anm_skin_%s"
      local ava_params =
      {
        res = res:format( private.skin.content, private.skin.name ),
        animfunc = "marker",
        playing  = true
      };
      table.insert( multiset, { anm_avatar, ava_params } );
    
    elseif ( private.IsSectionCompleted( name ) ) then

      params.animfunc = "island"..id.."_complete";
      params.stages   = private.GetSectionStages( name );

    elseif ( private.IsSectionUnlocked( name ) ) then

      params.animfunc = "island"..id.."_unlock";
      params.frame    = 54;
      playing         = false; -- Static unlock state (last frame)
      
      params.stages   = private.GetSectionStages( name );

    else

      local prew = StringGet( str.."section_"..( id - 1 ) );
      local text = StringGet( str.."description_locked", prew );

      params.animfunc  = "island"..id.."_lock";
      params.lock_text = text;

    end;

    table.insert( multiset, { item, params } );

  end;

  ObjMultiSet( multiset );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.IndexList ()

    local map = private.section_map;

    for id, section in pairs( private.section_list ) do

      map.data[ section.name ] = section;
      map.id  [ section.name ] = id;

    end;

end;
--------------------------------------------------------------------------------
function private.IsSectionCurrent ( name )

  return ( private.section_current == name );

end;
--------------------------------------------------------------------------------
function private.IsSectionUnlocked ( name )

  local section = private.section_map.data[ name ]

  return ( section and section.enabled );

end;
--------------------------------------------------------------------------------
function private.IsSectionCompleted ( name )

  local id  = private.section_map.id[ name ];
  local nxt = id + 1;

  local section_next = private.section_list[ nxt ];
  return ( section_next and section_next.enabled );

end;
--------------------------------------------------------------------------------
function private.GetSectionStages ( name )

  local pattern = "%s/%s";
  
  local id  = private.section_map.id[ name ];
  local section = private.section_list[ id ];
  
  return pattern:format( section.stages_done, section.stages_count );

end;
--------------------------------------------------------------------------------