-- name=int_panel_map_bottom
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.app_w = int.W();
  private.app_h = int.H();
  private.app_o = int.Orientation();
  
  private.btn = "btn4";
  
  private.panel_animfunc = {
    btn4 = {
      appear    = private.app_o.."_4btn_appear",
      disappear = private.app_o.."_4btn_disappear",
      static    = private.app_o.."_4btn"
    },
    btn5 = {
      appear    = private.app_o.."_5btn_appear",
      disappear = private.app_o.."_5btn_disappear",
      static    = private.app_o.."_5btn"
    }
  };

  private.this_obj = {
    panel    = "obj_int_panel_map_bottom",
    options  = "btn_int_panel_map_settings",
    hero     = "btn_int_panel_map_hero",
    shop     = "btn_int_panel_map_shop",
    stickers = "btn_int_panel_map_fireflies",
    worlds   = "btn_int_panel_map_worldmap"
  };

  private.this_anm = {
    panel    = "anm_int_panel_map_bottom",
    options  = "anm_int_panel_map_settings",
    hero     = "anm_int_panel_map_hero",
    shop     = "anm_int_panel_map_shop",
    stickers = "anm_int_panel_map_fireflies",
    worlds   = "anm_int_panel_map_worldmap"
  };

  private.this_animfunc_prefix = {
    options  = "btn_options",
    hero     = "btn_player",
    shop     = "btn_shop",
    stickers = "btn_stickers",
    worlds   = "btn_world"
  };

  private.strings = {
    options  = ne.strings.Get( "str_int_panel_map_settings" ),
    hero     = ne.strings.Get( "str_int_panel_map_hero" ),
    shop     = ne.strings.Get( "str_int_panel_map_shop" ),
    stickers = ne.strings.Get( "str_int_panel_map_fireflies" ),
    worlds   = ne.strings.Get( "str_int_panel_map_worldmap" )
  };

  private.active   = {};
  private.func_tap = {};
  private.items    = {};
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  int_control.Add( "int_panel_map_bottom" );

  private.Adapt();
  private.CreateButtons();

  int_control.Attach( "int_panel_map_bottom" );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( func_end )

  if ( private.active.panel ) then

    return;

  end;

  private.active.panel = true;
  private.off          = false;

  AnimPlay(
    private.this_anm.panel,
    private.panel_animfunc[ private.btn ].appear,
    function ()

      AnimPlay(
        private.this_anm.panel,
        private.panel_animfunc[ private.btn ].static,
        ""
      );
      
      int.Input_On( private.this_obj.panel );
      
      if ( func_end ) then
      
        func_end();
        
      end;
      
    end
  );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_OnHide )

  if ( not private.active.panel ) then

    return;

  end;

  private.active.panel = false;

  int.Input_Off( private.this_obj.panel );
  
  AnimPlay(
    private.this_anm.panel,
    private.panel_animfunc[ private.btn ].disappear,
    function ()

      if ( func_OnHide ) then
      
        func_OnHide();
        
      end;

      private.off = true;
      
    end
  );

end;
--------------------------------------------------------------------------------
function public.Off ()

  if ( private.off ) then

    return;

  end;

  local animfunc = m_object.Get( private.this_anm.panel ).animfunc;

  AnimStop( private.this_anm.panel, animfunc );

  m_object.Set(
    private.this_anm.panel,
    {
      animfunc = private.panel_animfunc[ private.btn ].disappear,
      playing  = false,
      frame    = 30,
      endtrig  = function () end
    }
  );

  int.Input_Off( private.this_obj.panel );

  private.active.panel = false;
  private.off          = true;

end;
--------------------------------------------------------------------------------
function public.Input_On ()
  
  int.Input_On( private.this_obj.panel );
  
end;
--------------------------------------------------------------------------------
function public.Input_Off ()
  
  int.Input_Off( private.this_obj.panel );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Options_Show ( func_tap )

  private.func_tap.options = func_tap;
  private.Button_Show( "options" );

end;
--------------------------------------------------------------------------------
function public.Hero_Show ( func_tap )

  private.func_tap.hero = func_tap;
  private.Button_Show( "hero" );

end;
--------------------------------------------------------------------------------
function public.Hero_SetNotification ( count )

  private.Button_SetNotification( "hero", count );

end;
--------------------------------------------------------------------------------
function public.Hero_SetPlatformNotification ( platform )

  private.hero_platform = platform;

end;
--------------------------------------------------------------------------------
function public.Shop_Show ( func_tap )

  private.func_tap.shop = func_tap;
  private.Button_Show( "shop" );

end;
--------------------------------------------------------------------------------
function public.Shop_SetNotification ( count )

  private.Button_SetNotification( "shop", count );

end;
--------------------------------------------------------------------------------
function public.Stickers_Show ( func_tap )

  private.func_tap.stickers = func_tap;
  private.Button_Show( "stickers" );

end;
--------------------------------------------------------------------------------
function public.Stickers_SetNotification ( count )

  private.Button_SetNotification( "stickers", count );

end;
--------------------------------------------------------------------------------
function public.Worlds_Show ( func_tap )

  private.func_tap.worlds = func_tap;
  private.Button_Show( "worlds" );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()
  
  local multiset = {};
  
  m_object.Set(
    private.this_obj.panel,
    {
      pos_x = private.app_w /2,
      pos_y = private.app_h
    },
    multiset
  );

  m_object.Set(
    private.this_anm.panel,
    {
      animfunc = private.panel_animfunc[ private.btn ].disappear,
      frame    = 30
    },
    multiset
  );
  
  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  for button_type, button_obj in pairs( private.this_obj ) do

    common.TapSet( button_obj,
    {
      func_true = function( sender, arg ) -- NOTE: see achievements tutorial

        private.Button_Tap( button_type, arg );

      end
    } );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Button_Show ( button_type )

  if ( private.active[ button_type ] ) then

    return;

  end;
  
  local multiset = {};

  m_object.Set(
    private.this_anm[ button_type ],
    {
      animfunc = private.this_animfunc_prefix[ button_type ].."_normal_static",
      btn_name = private.strings[ button_type ]
    },
    multiset
  );

  int.Button_Enable( private.this_obj[ button_type ], false, multiset );

  table.insert( private.items, private.this_obj[ button_type ] );
    
  if ( #private.items > 4 ) then
    
    private.btn = "btn5";

    if ( private.active.panel ) then
    
      m_object.Set(
        private.this_anm.panel,
        {
          animfunc = private.panel_animfunc[ private.btn ].static
        },
        multiset
      );

    end;
    
  end;

  m_object.Multiset( multiset );
  
  private.active[ button_type ] = true;

end;
--------------------------------------------------------------------------------
function private.Button_Hide ( button_type )

  int.Button_Disable( private.this_obj[ button_type ] );

end;
--------------------------------------------------------------------------------
function private.Button_Tap ( button_type, arg )

  private.func_tap[ button_type ]( arg );

  AnimPlay(
    private.this_anm[ button_type ],
    private.this_animfunc_prefix[ button_type ].."_normal_click",
    function()

      AnimPlay(
        private.this_anm[ button_type ],
        private.this_animfunc_prefix[ button_type ].."_normal_static",
        ""
      );

    end
  );
  
end;
--------------------------------------------------------------------------------
function private.Button_SetNotification ( button_type, count )

  if ( count > 0                         ) and
     ( not private.active[ button_type ] ) then

    private.Button_Show( button_type );

  end;

  local prefix_static = private.this_animfunc_prefix[ button_type ];
  local prefix_notify = private.this_animfunc_prefix[ button_type ];
  
  if ( button_type == "hero" and private.hero_platform ) then
    
    if ( count > 0 ) then
      prefix_notify = prefix_notify.."_"..private.hero_platform;
    else
      private.hero_platform = nil;
    end;
    
  end;

  if ( count > 0 ) then
  
    m_object.Set(
      private.this_anm[ button_type ],
      {
        animfunc = prefix_notify.."_notification_static",
        count    = tostring( count )
      }
    );

  else

    m_object.Set(
      private.this_anm[ button_type ],
      {
        animfunc = prefix_static.."_normal_static"
      }
    );

  end;

end;
--------------------------------------------------------------------------------