-- name=int_panel_map_left
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.this_int = "int_panel_map_left";
  private.this_obj = "obj_"..private.this_int;
  private.this_anm = "anm_"..private.this_int;

  private.labels =
  {
    hearts_full      = ne.strings.Get( "str_int_panel_map_hearts_full" ),
    hearts_label     = ne.strings.Get( "str_int_panel_map_hearts_label" ),
    social_label     = ne.strings.Get( "str_int_panel_map_social_label" ),
    mail_label       = ne.strings.Get( "str_int_panel_map_mail_label" ),
    dailyquest_label = ne.strings.Get( "str_int_panel_map_dailyquest_label" ),
  };

  private.dailyquest_notification = false;

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  int_control.Add( private.this_int );

  private.Adapt();
  private.CreateButtons();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Show ( anim_time, full_time )

  if ( private.active ) then

    return;

  end;

  private.active = true;
  private.off    = false;

  private.SetDefault();

  int_control.Attach( private.this_int );
  
  local hold_time = full_time - anim_time;
  local pos_x     = m_object.Get( private.this_obj ).pos_x;

  common.Animation_Fade_In( anim_time, private.this_obj );

  ObjAnimate(
    private.this_obj,
    "pos_x", 0, 0,
    function()

      public.Hearts_UpdateTimer(
        function()
         
          if ( private.Dailyquest_UpdateTimer ) then

            private.Dailyquest_UpdateTimer();

          end;
          
        end
      );
      
      int.Input_On( private.this_obj );

    end, 
    {
      0.0,       0, pos_x,
      hold_time, 0, pos_x,
      full_time, 0, 0,
    }
  );

end;
--------------------------------------------------------------------------------
function public.Hide ( anim_time, func_end )

  if ( not private.active ) then

    return;

  end;

  private.active = false;
  
  int.Input_Off( private.this_obj );

  private.Timer_Disable( "hearts",     true );
  private.Timer_Disable( "dailyquest", true );
  
  common.Animation_Fade_Out( anim_time, private.this_obj );

  ObjAnimate(
    private.this_obj,
    "pos_x", 0, 0,
    function ()
      
      private.Restore();
      if ( func_end ) then func_end(); end;
      int_control.Detach( private.this_int );

      private.off = true;

    end, 
    {
      0.0,       0, m_object.Get( private.this_obj ).pos_x,
      anim_time, 0, -100,
    }
  );

end;
--------------------------------------------------------------------------------
function public.Off ()

  if ( private.off ) then

    return;

  end;

  private.active = false;

  int.Input_Off( private.this_obj );

  ObjStopAnimate( private.this_obj, "pos_x" );

  m_object.Set(
    private.this_obj,
    {
      pos_x = -100
    }
  );

  private.Timer_Disable( "hearts",     true );
  private.Timer_Disable( "dailyquest", true );
  
  private.Restore();

  int_control.Detach( private.this_int );

  private.off = true;

end;
--------------------------------------------------------------------------------
function public.Input_On ()
  
  int.Input_On( private.this_obj );
  
end;
--------------------------------------------------------------------------------
function public.Input_Off ()
  
  int.Input_Off( private.this_obj );
  
end;
--------------------------------------------------------------------------------
function public.IsActive ()

  return private.active;

end;
--------------------------------------------------------------------------------
function private.Restore ()

  private.hearts_highlighted = false;
  
  m_object.Set(
    private.this_anm,
    {
      animfunc = "static_3"
    }
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Hearts_Show (
    count,
    func_tap,
    func_seconds,
    highlight_hearts
  )

  if ( highlight_hearts ) then

    private.Hearts_Highlight();

  end;
  
  if ( private.hearts_active ) then

    return;

  end;

  private.hearts_active = true;

  private.Hearts_Tap = function ()

--    AnimPlay(
--      "anm_int_panel_map_hearts",
--      "btn_hearts_click",
--      function ()
--
--        AnimPlay(
--          "anm_int_panel_map_hearts",
--          "btn_hearts_static",
--          ""
--        );

        func_tap();

    --  end
    --);

  end;

  private.Hearts_GetSeconds = func_seconds;

  local multiset = {};
  
  m_object.Enable( "btn_int_panel_map_hearts", nil, multiset );

  m_object.Set(
    "anm_int_panel_map_hearts",
    {
      animfunc = "btn_hearts_static",
      btn_name = private.labels.hearts_label
    },
    multiset
  );  

  public.Hearts_UpdateCount( count, multiset );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function public.Hearts_UpdateTimer ( func_end )
  
  -- JK: check if timer is initialized
  if ( not private.Hearts_GetSeconds ) or
     ( private.hearts_highlighted    ) then
    
    return;
    
  end;
  
  private.Timer_Update(
    "hearts",
    public.Hearts_UpdateTimer,
    private.Hearts_GetSeconds(),
    nil,
    nil,
    func_end
  );

end;
--------------------------------------------------------------------------------
function public.Hearts_UpdateCount ( count, multiset )

  m_object.Set(
    "anm_int_panel_map_hearts",
    {
      count = int.NumberForm( count )
    },
    multiset
  );
  
  public.Hearts_UpdateTimer();

end;
--------------------------------------------------------------------------------
function private.Hearts_Highlight ()

  private.hearts_highlighted = true;
  
  m_object.Set(
    private.this_anm,
    {
      animfunc = "static_1"
    }
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Social_Show ( func_tap )

  local multiset = {};

  private.Social_Tap = function ()

--    AnimPlay(
--      "anm_int_panel_map_social",
--      "btn_social_normal_click",
--      function ()
--
--        AnimPlay(
--          "anm_int_panel_map_social",
--          "btn_social_normal_static",
--          ""
--        );

        func_tap();

    --  end
    --);

  end;

  m_object.Enable( "btn_int_panel_map_social", nil, multiset );

  m_object.Set(
    "anm_int_panel_map_social",
    {
      animfunc = "btn_social_normal_static",
      btn_name = private.labels.social_label
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function public.Social_Hide ( func_end, animated )

  int.Input_Off( "btn_int_panel_map_social" );

  local func_hide = function ()

    m_object.Disable( "btn_int_panel_map_social" );

    if ( func_end ) then

      func_end();

    end;

  end;

  if ( animated ) then

    common.Animation_Fade_Out(
      0.3, "anm_int_panel_map_social",
      func_hide
    );

  else

    func_hide();    

  end;

end;
--------------------------------------------------------------------------------
function public.Social_Enable ()

  int.Input_On( "btn_int_panel_map_social" );

end;
--------------------------------------------------------------------------------
function public.Social_Disable ()

  int.Input_Off( "btn_int_panel_map_social" );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Mail_Show ( mail_count, func_tap, animated )
  
  private.mail_active = true;

  local multiset = {};
  
  local animfunc_static = tern( mail_count > 0 )( "btn_mail_notification_static",
                                                  "btn_mail_static" );

  local animfunc_click = tern( mail_count > 0 )( "btn_mail_notification_click",
                                                 "btn_mail_click" );
                                                  
  private.Mail_Tap = function ()

--    AnimPlay(
--      "anm_int_panel_map_mail",
--      animfunc_click,
--      function ()
--
--        AnimPlay(
--          "anm_int_panel_map_mail",
--          animfunc_static,
--          ""
--        );

        func_tap();

    --  end
    --);

  end;

  m_object.Enable( "btn_int_panel_map_mail", nil, multiset );

  m_object.Set(
    "anm_int_panel_map_mail",
    {
      animfunc = animfunc_static,
      btn_name = private.labels.mail_label,
      count    = tostring( mail_count )
    },
    multiset
  );

  if ( m_object.Get( "anm_int_panel_map_mail" ).alp < 1 ) then

    common.Animation_Fade_In( 0.3, "anm_int_panel_map_mail" );

  end;

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Dailyquest_Show (
    func_tap,
    func_update
  )

  if ( private.dailyquest_active ) then

    return;

  end;

  private.dailyquest_active = true;

  local multiset = {};
  local anm_name = "anm_int_panel_map_dailyquest";

  private.Dailyquest_Tap = function ()

--    local animfunc_static = m_object.Get( anm_name ).animfunc;
--    local animfunc_click  = "btn_daily_quest_normal_click";
--
--    if ( animfunc_static == "btn_daily_quest_notification_static" ) then
--
--      animfunc_click  = "btn_daily_quest_notification_click";
--
--    end;
--
--    AnimPlay(
--      anm_name,
--      animfunc_click,
--      function ()
--
--        AnimPlay(
--          anm_name,
--          animfunc_static,
--          ""
--        );

        func_tap();

    --  end
    --);

  end;

  m_object.Enable( "btn_int_panel_map_dailyquest", nil, multiset );

  m_object.Set(
    anm_name,
    {
      animfunc = "btn_daily_quest_normal_static",
      btn_name = private.labels.dailyquest_label
    },
    multiset
  );

  m_object.Multiset( multiset );

  private.Dailyquest_UpdateTimer = func_update;
  
  private.Dailyquest_UpdateTimer();

end;
--------------------------------------------------------------------------------
function public.Dailyquest_IsActive ()

  return private.dailyquest_active;

end;
--------------------------------------------------------------------------------
function public.Dailyquest_UpdateTimer (
    time,
    step,
    func_exec,
    func_notification
  )

  private.Timer_Update(
    "dailyquest",
    func_exec,
    time.sec,
    step,
    time.mode
  );

  func_notification();

end;
--------------------------------------------------------------------------------
function public.Dailyquest_UpdateNotification ( count )

  if ( count > 0 ) then

    m_object.Set(
      "anm_int_panel_map_dailyquest",
      {
        count = tostring( count )
      }
    );
      
    if ( private.dailyquest_notification ) then

      return;

    end;
    
    private.dailyquest_notification = true;

    m_object.Set(
      "anm_int_panel_map_dailyquest",
      {
        animfunc = "btn_daily_quest_notification_static",
        playing  = true,
        count    = tostring( count )
      }
    );

  else

    if ( not private.dailyquest_notification ) then

      return;

    end;

    private.dailyquest_notification = false;

    m_object.Set(
      "anm_int_panel_map_dailyquest",
      {
        animfunc = "btn_daily_quest_normal_static"
      }
    );

  end;

end;
--------------------------------------------------------------------------------
function public.Dailyquest_Hide ()

  if ( not private.dailyquest_active ) then

    return;

  end;

  private.dailyquest_active = false;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()

  local multiset = {};

  m_object.Set(
    private.this_obj,
    {
      input = false,
      alp   = 0
    },
    multiset
  );

  m_object.Disable( "btn_int_panel_map_hearts",     nil, multiset );
  m_object.Disable( "btn_int_panel_map_social",     nil, multiset );
  m_object.Disable( "btn_int_panel_map_dailyquest", nil, multiset );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()

  local function func_sound ()
    
    cmn.PlaySnd( "click" );
    
  end;
  
  common.TapSet( "btn_int_panel_map_hearts",
  {    
    func_true = function ()

      func_sound();
      private.Hearts_Tap();

    end
  } );

  common.TapSet( "btn_int_panel_map_social",
  {    
    func_true = function ()

      func_sound();
      private.Social_Tap();

    end
  } );

  common.TapSet( "btn_int_panel_map_dailyquest",
  {    
    func_true = function ()

      func_sound();
      private.Dailyquest_Tap();

    end
  } );
  
  common.TapSet( "btn_int_panel_map_mail",
  {    
    func_true = function ()

      func_sound();
      private.Mail_Tap();

    end
  } );

end;
--------------------------------------------------------------------------------
function private.SetDefault ()
  
  local animfunc = tern( private.dailyquest_active )( "static_3", "static_4" );
  
  animfunc = tern( private.mail_active )( "static_5", animfunc );
  animfunc = tern( private.mail_active and 
                   private.dailyquest_active )( "static_6", animfunc );
  animfunc = tern( private.hearts_highlighted )( "static_1", animfunc );

  m_object.Set(
    private.this_anm,
    {
      animfunc = animfunc
    }
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Timer_Update (
    subject,
    func_exec,
    seconds,
    step,
    mode,
    func_end
  )
  
  if ( not private.active ) then
    
    return;
    
  end;

  step = step or 1;
  mode = mode or "m:s";

  local anm_timer = "anm_int_panel_map_"..subject.."_timer";
  local tmr_timer = "tmr_int_panel_map_"..subject;
  local status    = m_object.Get( anm_timer ).animfunc;

  if ( seconds <= 0 ) then
    
    private.Timer_Disable( subject, true );

    if ( func_end ) then

      func_end();

    end;

  else

    local time = common.GetTimeFromSec( seconds );

    m_object.Multiset( {
      { tmr_timer,
        {
          playing = true,
          time    = step,
          endtrig = function()

            func_exec();

          end
        }
      },  
      { anm_timer,
        {
          timer = int.Text_TimerFormat( time, mode )
        }
      }
    } );

    if ( status == "timer_static" ) then

      return;

    end;

    AnimPlay(
      anm_timer,
      "timer_show",
      function ()

        AnimPlay(
          anm_timer,
          "timer_static",
          ""
        );

        if ( func_end ) then

          func_end();

        end; 

      end
    );

  end;

end;
--------------------------------------------------------------------------------
function private.Timer_Disable ( subject, animated )

  local multiset  = {};
  local tmr_timer = "tmr_int_panel_map_"..subject;
  local anm_timer = "anm_int_panel_map_"..subject.."_timer";
  local status    = m_object.Get( anm_timer ).animfunc;
  
  if ( status == "timer_hide" ) then

    return;

  end;
  
  if ( animated ) then
    
    AnimPlay( anm_timer, "timer_hide", "" );
    
  else
  
    m_object.Set(
      anm_timer,
      {
        playing  = false,
        animfunc = "timer_hide",
        frame    = 24
      },
      multiset
    );
  
  end;
  
  m_object.Set(
    tmr_timer,
    {
      playing = false
    },
    multiset
  );
  
  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------