-- name=int_panel_map_right
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.app_w = int.W();

  private.this_int = "int_panel_map_right";
  private.this_obj = "obj_"..private.this_int;
  private.this_anm = "anm_"..private.this_int;

  private.active  = false;
  private.visible = false;
  
  private.buttons =
  {
    quests = 
    {
      enabled       = false,
      notifications = 0,
      btn           = "btn_int_panel_map_quests",
      anm           = "anm_int_panel_map_quests",
      label         = ne.strings.Get( "str_int_panel_map_quests_label" ),
      animfunc      =
      {
        normal       = "btn_quest_normal_static",
        normal_click = "btn_quest_normal_click",
        notify       = "btn_quest_notification_static",
        notify_click = "btn_quest_notification_click"
      }
    },
    friends = 
    {
      enabled       = false,
      notifications = 0,
      btn           = "btn_int_panel_map_friends",
      anm           = "anm_int_panel_map_friends",
      label         = ne.strings.Get( "str_int_panel_map_friends_label" ),
      animfunc      =
      {
        normal       = "btn_friends_normal_static",
        normal_click = "btn_friends_normal_click",
        notify       = "btn_friends_notification_static",
        notify_click = "btn_friends_notification_click"
      }
    },
    sale = 
    {
      enabled       = false,
      notifications = 0,
      btn           = "btn_int_panel_map_sale",
      anm           = "anm_int_panel_map_sale",
      label         = ne.strings.Get( "str_int_panel_map_sale_label" ),
      animfunc      =
      {
        normal       = "btn_sale_normal_static",
        normal_click = "btn_sale_normal_click"
      }
    },
    google =
    {
      enabled       = false,
      notifications = 0, -- HACK: 1 if logged to GPGS
      btn           = "btn_int_panel_map_google",
      anm           = "anm_int_panel_map_google",
      label         = ne.strings.Get( "str_int_panel_map_google_label" ),
      animfunc      =
      {
        normal       = "btn_google_normal_static",
        normal_click = "btn_google_normal_click",
      }
    }
  };
  
  private.buttons_enabled = 0;

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  int_control.Add( private.this_int );
  
  ObjMultiSet( 
  {
    { private.this_obj, { input = false, alp = 0, pos_x = int.W() } },
    { private.this_anm, { animfunc = "static_1", playing = true   } }
  } );
  
  for i, button in pairs( private.buttons ) do
  
    m_object.Disable( button.btn );
  
  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Show ( anim_time, full_time )

  if ( private.active ) then
  
    return;
  
  end;
  
  private.active = true;
  private.off    = false;

  int_control.Attach( private.this_int );
  
  local hold_time = full_time - anim_time;
  local pos_x     = m_object.Get( private.this_obj ).pos_x;

  common.Animation_Fade_In( anim_time, private.this_obj );

  ObjAnimate(
    private.this_obj,
    "pos_x", 0, 0,
    function()

      int.Input_On( private.this_obj );
      
      private.Sale_ShowTimer();
      
      private.visible = true;
      
      mediator.Notify( "Interface_RightPanel_Visible" );

    end, 
    {
      0.0,       0, pos_x,
      hold_time, 0, pos_x,
      full_time, 0, int.W(),
    }
  );

end;
--------------------------------------------------------------------------------
function public.Hide ( tme )

  if ( not private.active ) then
  
    return;
  
  end;
  
  private.active = false;

  int.Input_Off( private.this_obj );
  
  private.Sale_HideTimer( true );
  
  common.Animation_Fade_Out( tme, private.this_obj );

  ObjAnimate( private.this_obj, "pos_x", 0, 0,
    function ()

      int_control.Detach( private.this_int );

      private.off     = true;
      private.visible = false;

    end, 
    {
      0.0, 0, ObjGet( private.this_obj ).pos_x,
      tme, 0, private.app_w + 100,
    }
  );
  
  mediator.Notify( "Interface_RightPanel_Invisible" );

end;
--------------------------------------------------------------------------------
function public.Off ()

  if ( private.off ) then

    return;

  end;

  private.active = false;

  int.Input_Off( private.this_obj );

  ObjStopAnimate( private.this_obj, "pos_x" );

  m_object.Set(
    private.this_obj,
    {
      pos_x = private.app_w + 100
    }
  );

  int_control.Detach( private.this_int );

  private.off = true;

end;
--------------------------------------------------------------------------------
function public.Input_On ()
  
  int.Input_On( private.this_obj );
  
end;
--------------------------------------------------------------------------------
function public.Input_Off ()
  
  int.Input_Off( private.this_obj );
  
end;
--------------------------------------------------------------------------------
function public.IsActive ()

  return private.active;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Button_EnabledCount ()

  return private.buttons_enabled;

end;
--------------------------------------------------------------------------------
function public.Button_IsEnabled ( button_name )

  return private.buttons[ button_name ].enabled;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Quests_Enable ( func_action )

  private.Button_Enable( "quests", func_action );

end;
--------------------------------------------------------------------------------
function public.Quests_Warn ( count )

  private.Button_Warn( "quests", count );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Friends_Enable ( func_action )

  private.Button_Enable( "friends", func_action );

end;
--------------------------------------------------------------------------------
function public.Friends_Warn ( count )

  private.Button_Warn( "friends", count );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Sale_Enable ( func_action, func_seconds )

  private.Button_Enable( "sale", func_action );
  
  private.Sale_GetSeconds = func_seconds;
  
  if ( private.active ) then
    
    private.Sale_ShowTimer();
    
  end;

end;
--------------------------------------------------------------------------------
function public.Sale_Warn ( count )

  private.Button_Warn( "sale", count );

end;
--------------------------------------------------------------------------------
function public.Sale_Disable ()

  private.Sale_HideTimer();
  
  private.Button_Disable( "sale" );

end;
--------------------------------------------------------------------------------
function private.Sale_ShowTimer ()
  
  if ( not private.Sale_GetSeconds ) then return; end;
  
  int.Timer_Show(
    "int_panel_map_sale",
    private.Sale_GetSeconds(),
    private.Sale_ShowTimer
  );

end;
--------------------------------------------------------------------------------
function private.Sale_HideTimer ( animated )
  
  int.Timer_Hide( "int_panel_map_sale", animated );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Google_Enable ( func_action )

  private.Button_Enable( "google", func_action );
  
  ObjSet(
    private.this_anm,
    {
      animfunc = "static_google"
    }
  );

end;
--------------------------------------------------------------------------------
function public.Google_Disable ()

  private.Button_Disable( "google" );
  
  ObjSet(
    private.this_anm,
    {
      animfunc = "static_1"
    }
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Button_Enable ( button_name, func_action )

  local button = private.buttons[ button_name ];
  
  ObjSet(
    button.anm,
    {
      btn_name = button.label,
      animfunc = button.animfunc.normal
    }
  );

  if ( button.enabled ) then

    return;
    
  end;
  
  button.enabled = true;
  
  int.Control_EventsSet( button.btn, { func_on_click = func_action } );
  
  m_object.Enable( button.btn );

  private.buttons_enabled = private.buttons_enabled + 1;
  
  mediator.Notify(
    "Interface_RightPanel_Updated",
    {
      name    = button_name,
      action  = "enabled",
      active  = private.active,
      visible = private.visible,
      count   = private.buttons_enabled
    }
  );
  
end;
--------------------------------------------------------------------------------
function private.Button_Disable ( button_name )

  local button = private.buttons[ button_name ];

  if not ( button.enabled ) then

    return;
    
  end;
  
  button.enabled = false;
  
  int.Control_EventsClear( button.btn );
  
  m_object.Disable( button.btn );
  
  private.buttons_enabled = private.buttons_enabled - 1;
  
  mediator.Notify(
    "Interface_RightPanel_Updated",
    {
      name    = button_name,
      action  = "disabled",
      active  = private.active,
      visible = private.visible,
      count   = private.buttons_enabled
    }
  );

end;
--------------------------------------------------------------------------------
function private.Button_Warn ( button_name, count )

  local button = private.buttons[ button_name ];
  
  button.notifications = count;

  if ( count > 0 ) then

    ObjSet( button.anm,
    { 
      animfunc = button.animfunc.notify,
      count    = tostring( count ),
      playing  = true
    } );

  else
  
    AnimPlay( button.anm, button.animfunc.normal, "" );

  end;

end;
--------------------------------------------------------------------------------