-- name=int_panel_map_top
function public.__InitModule ()
  
  private.int_w = int.W();
  private.int_h = int.H();
  private.int_o = int.Orientation();

  private.this_obj = {
    panel     = "obj_int_panel_map_top",
    gold      = "obj_int_panel_map_gold",
    resource1 = "obj_int_panel_map_resource1",
    resource2 = "obj_int_panel_map_resource2",
    resource3 = "obj_int_panel_map_resource3"
  };

  private.this_anm = {
    panel        = "anm_int_panel_map_top",
    notification = {
      panel = "anm_int_panel_map_top_notification",
      body  = "anm_int_panel_map_top_notification_body",
      icon  = "anm_int_panel_map_top_notification_icon"
    },
    gold      = "anm_int_panel_map_top_gold",
    resource1 = "anm_int_panel_map_top_resource1",
    resource2 = "anm_int_panel_map_top_resource2",
    resource3 = "anm_int_panel_map_top_resource3"
  };

  private.animfunc = {
    panel = {
      show   = private.int_o.."_appear",
      static = private.int_o.."_static",
      hide   = private.int_o.."_disappear"
    },
    highlight = {
      show   = "gold_appear",
      static = "gold_static",
      hide   = "gold_disappear"
    }
  };

  private.animfunc_static = {
    panel = {
      gold      = "res_gold",
      resource1 = "res_1",
      resource2 = "res_2",
      resource3 = "res_3"
    },
    highlight = {
      gold      = "res_gold_highlight",
      resource1 = "res_1_highlight",
      resource2 = "res_2_highlight",
      resource3 = "res_3_highlight"
    }
  };

  private.animfunc_count_increase = {
    panel = {
      gold      = "res_gold_count_i",
      resource1 = "res_1_count_i",
      resource2 = "res_2_count_i",
      resource3 = "res_3_count_i"
    },
    highlight = {
      gold      = "res_gold_count_i_highlight",
      resource1 = "res_1_count_i_highlight",
      resource2 = "res_2_count_i_highlight",
      resource3 = "res_3_count_i_highlight"
    }
  };
  
  private.animfunc_count_decrease = {
    panel = {
      gold      = "res_gold_count_d",
      resource1 = "res_1_count_d",
      resource2 = "res_2_count_d",
      resource3 = "res_3_count_d"
    },
    highlight = {
      gold      = "res_gold_count_d_highlight",
      resource1 = "res_1_count_d_highlight",
      resource2 = "res_2_count_d_highlight",
      resource3 = "res_3_count_d_highlight"
    }
  };

  private.target_queue = {
    gold      = {},
    resource1 = {},
    resource2 = {},
    resource3 = {}
  };

  private.dimmable = {
    resource1 = true,
    resource2 = true,
    resource3 = true
  };

  private.is_counting = {
    gold      = false,
    resource1 = false,
    resource2 = false,
    resource3 = false
  };

  private.active             = {};
  private.counter_stop       = {}
  private.func_show_popup    = {};
  private.func_hide_popup    = {};
  private.func_check_popup   = {};
  private.notification_queue = {};

  private.visible = false;
  private.locked  = false;
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  int_control.Add( "int_panel_map_top" );

  local multiset = {};
  
  m_object.Set(
    private.this_obj.panel,
    {
      pos_x = 0.5 * private.int_w
    },
    multiset
  );

  int.Control_EventsSet(
    private.this_obj.gold,
    {
      func_on_click = function()

        private.Counter_ShowPopup( "gold" );

      end
    },
    multiset
  );

  for r = 1, 3 do

    int.Control_EventsSet(
      private.this_obj[ "resource"..r ],
      {
        func_on_click = function()

          private.Counter_ShowPopup( "resource"..r );

        end
      },
      multiset
    );

  end;
  
  m_object.Multiset( multiset );

  int_control.Attach( "int_panel_map_top" );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.IsVisible ()

  local params   = m_object.Get( private.this_anm.panel ) or {};
  local animfunc = params.animfunc or "";

  return ( ( private.visible ) and ( not animfunc:find( "disappear" ) ) );

end;
--------------------------------------------------------------------------------
function public.IsLocked ()

  return private.locked;

end;
--------------------------------------------------------------------------------
function public.IsHighlighted ()

  return ( private.highlighted ~= nil );

end;
--------------------------------------------------------------------------------
function public.IsActive ( resource_type )

  return private.active[ resource_type ] or false;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ()
  
  if ( private.visible ) then
    
    return;
    
  end;

  private.visible = tern( private.highlighted )( "highlight", "panel" );
  private.off     = false;
  
  if ( private.highlighted ) then
  
    m_object.Set(
      private.this_anm[ private.highlighted ],
      {
        animfunc = private.animfunc_static[ private.visible ][ private.highlighted ]
      }
    );
  
  end;

  AnimPlay(
    private.this_anm.panel,
    private.animfunc[ private.visible ].show,
    function ()

      AnimPlay(
        private.this_anm.panel,
        private.animfunc[ private.visible ].static,
        ""
      );

      int.Input_On( private.this_obj.panel );

    end
  );

end;
--------------------------------------------------------------------------------
function public.Hide ( func_OnHide )

  if ( not private.visible ) then
    
    return;
    
  end;

  int.Input_Off( private.this_obj.panel );

  AnimPlay(
    private.this_anm.panel,
    private.animfunc[ private.visible ].hide,
    function ()

      private.Restore();

      if ( func_OnHide ) then

        func_OnHide();

      end;

      private.off = true;

    end
  );
  
  for resource_type, func_check_popup in pairs( private.func_check_popup ) do

    local is_popup_open = func_check_popup();
    
    if ( is_popup_open ) then
      
      private.Counter_HidePopup( resource_type, true );
      
    end;
    
  end;
  
  private.visible = false;

end;
--------------------------------------------------------------------------------
function public.Off ()

  if ( private.off ) then

    return;

  end;

  local animfunc = m_object.Get( private.this_anm.panel ).animfunc;

  AnimStop( private.this_anm.panel, animfunc );

  m_object.Set(
    private.this_anm.panel,
    {
      animfunc = private.animfunc.panel.hide,
      playing  = false,
      frame    = 30,
      endtrig  = function () end
    }
  );

  int.Input_Off( private.this_obj.panel );
  private.Restore();

  private.visible = false;
  private.off     = true;

end;
--------------------------------------------------------------------------------
function public.Input_On ()
  
  int.Input_On( private.this_obj.panel );
  
end;
--------------------------------------------------------------------------------
function public.Input_Off ()
  
  int.Input_Off( private.this_obj.panel );
  
end;
--------------------------------------------------------------------------------
function public.Lock ( state )

  private.locked = state;

end;
--------------------------------------------------------------------------------
function public.Highlight ( res_name )

  if ( private.highlighted ) then

    return;

  end;

  private.highlighted = res_name;

end;
--------------------------------------------------------------------------------
function public.Restore ()

  private.Restore();

end;
--------------------------------------------------------------------------------
function public.ShowPopup ( resource_type, text_string )

  private.Counter_ShowPopup( resource_type, text_string );

end;
--------------------------------------------------------------------------------
function public.ShowNotification ( text, icon, style )

  -- JK: style can be:
  -- - success
  -- - fail
  -- - info
  
  -- JK: icon can be:
  -- - gold / resource1, 2, 3
  -- - online / offline
  
  -- Defaults
  icon  = icon  or "default";
  style = style or "info";
  
  table.insert(
    private.notification_queue,
    {
      text  = text,
      icon  = icon,
      style = style
    }
  );

  private.Notification_TryShowFromQueue();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Gold_Show (
    gold_count,
    func_show_popup,
    func_hide_popup,
    func_check_popup
  )

  private.Counter_Show(
    "gold",
    gold_count,
    func_show_popup,
    func_hide_popup,
    func_check_popup
  );

end;
--------------------------------------------------------------------------------
function public.Gold_Hide ()

  private.Counter_Hide( "gold" );

end;
--------------------------------------------------------------------------------
function public.Gold_Update ( start_count, stop_count )

  if ( not private.active.gold  ) then

    return;

  end;

  private.counter_stop.gold = stop_count;

  if ( private.is_counting.gold ) then

    return;

  end;
  
  if ( not private.visible                                   ) or
     ( private.highlighted and private.highlighted ~= "gold" ) then
    
    ObjSet( private.this_anm.gold,
      {
        count = int.NumberForm( stop_count )
      }
    );
    
    return;
    
  end;

  private.is_counting.gold = true;

  private.Counter_Count( "gold", start_count, 1 );

  -- JK: play sound on gold increased  
  if ( start_count < stop_count ) then

    cmn.PlaySnd( "gold" );

  end;

  -- JK: play sound on gold decreased  
  if ( start_count > stop_count ) then

    --cmn.PlaySnd( "gold" );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Resource_Show (
    resource_type,
    resource_count,
    func_show_popup,
    func_hide_popup,
    func_check_popup
  )

  private.Counter_Show(
    resource_type,
    resource_count,
    func_show_popup,
    func_hide_popup,
    func_check_popup
  );

end;
--------------------------------------------------------------------------------
function public.Resource_Hide ( resource_type )

  private.Counter_Hide( resource_type );

end;
--------------------------------------------------------------------------------
function public.Resource_Update (
    resource_type,
    start_count,
    stop_count,
    func_end
  )

  if ( not private.active[ resource_type ] ) then

    return;

  end;
  
  if ( not private.visible                                          ) or
     ( private.highlighted and private.highlighted ~= resource_type ) then
    
    ObjSet( private.this_anm[ resource_type ],
      {
        count = int.NumberForm( stop_count )
      }
    );
    
    return;
    
  end;

  private.counter_stop[ resource_type ] = stop_count;

  if ( private.is_counting[ resource_type ] ) then

    return;

  end;

  private.is_counting[ resource_type ] = true;

  public.Resource_GrayOut( resource_type );
  
  local func_OnCountEnd = function ()      

    private.is_counting[ resource_type ] = false;

    if ( func_end ) then

      func_end();

    end;      

  end;

  private.Counter_Count( resource_type, start_count, 1, func_OnCountEnd );

end;
--------------------------------------------------------------------------------
function public.Resource_GrayIn ( resource_type )

  if ( not private.is_counting[ resource_type ] ) then

    m_object.Set(
      private.this_anm[ resource_type ],
      {
        color_r = 0.7,
        color_g = 0.7,
        color_b = 0.7
      }
    );

  end;

end;
--------------------------------------------------------------------------------
function public.Resource_GrayOut ( resource_type )

  m_object.Set(
    private.this_anm[ resource_type ],
    {
      color_r = 1.0,
      color_g = 1.0,
      color_b = 1.0
    }
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Restore ()

  if ( not private.highlighted ) then

    return;

  end;

  m_object.Set(
    private.this_anm[ private.highlighted ],
    {
      animfunc = private.animfunc_static.panel[ private.highlighted ]
    }
  );
  
  private.highlighted = nil;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Notification_TryShowFromQueue ()

  if ( #private.notification_queue < 1 ) or
     (  private.active.notification    ) then

    return;

  end;

  private.active.notification = true;

  local multiset     = {};
  local notification = table.remove( private.notification_queue, 1 );

  if ( notification.text:find( "str_" ) ) then

    notification.text = ne.strings.Get( notification.text );

  end;

  m_object.Set(
    private.this_anm.notification.body,
    {
      animfunc = notification.style.."_notification",
      tt_title = notification.text
    },
    multiset
  );

  m_object.Set(
    private.this_anm.notification.icon,
    {
      animfunc = notification.icon.."_icon"
    },
    multiset
  ); 

  m_object.Multiset( multiset );

  AnimPlay(
    private.this_anm.notification.panel,
    "notification_show",
    function ()

      private.Notification_Hide(
        function()

          private.Notification_TryShowFromQueue();

        end
      );

    end
  );

end;
--------------------------------------------------------------------------------
function private.Notification_Hide ( func_end )

  private.active.notification = nil;

  AnimPlay(
  private.this_anm.notification.panel,
    "notification_hide",
    func_end
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Counter_Show (
    resource_type,
    resource_count,
    func_show_popup,
    func_hide_popup,
    func_check_popup
  )

  local multiset = {};
  local visible  = ( resource_count >= 0 );
  
  m_object.Set(
    private.this_obj[ resource_type ],
    {
      input = visible
    },
    multiset
  );
    
  m_object.Set(
    private.this_anm[ resource_type ],
    {
      visible = visible,
      count   = int.NumberForm( resource_count ),
    },
    multiset
  );
  
  m_object.Multiset( multiset );

  if ( not visible ) then

    return;

  end;

  private.active[ resource_type ] = true;
  
  private.func_show_popup[ resource_type ]  = func_show_popup;
  private.func_hide_popup[ resource_type ]  = func_hide_popup;
  private.func_check_popup[ resource_type ] = func_check_popup;

end;
--------------------------------------------------------------------------------
function private.Counter_Hide ( resource_type )

  private.active[ resource_type ] = false;
  
  local multiset = {};
  
  int.Input_Off( private.this_obj[ resource_type ], multiset );
  m_object.Hide( private.this_anm[ resource_type ], multiset );
  
  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.Counter_Count ( resource_type, start_count, step, func_end )
  
  if ( not private.visible ) then
    
    private.is_counting[ resource_type ] = false;
    
    ObjSet( private.this_anm[ resource_type ],
      {
        count = int.NumberForm( private.counter_stop[ resource_type ] )
      }
    );
    
    if ( func_end ) then

      func_end();

    end;
    
    return;
    
  end;
  
  local anim_type  = private.visible;
  local stop_count = private.counter_stop[ resource_type ];
  local new_step   = step;

  local diff = stop_count - start_count;
  local mod  = 1;

  if ( start_count > stop_count ) then

    diff = start_count - stop_count;
    mod  = - 1;

  end;

  if ( diff > 5 ) then

    new_step = math.floor( diff / 2 );

  else

    new_step = 1;

  end;
  
  if ( start_count == stop_count ) then

    private.is_counting[ resource_type ] = false;

    AnimPlay(
      private.this_anm[ resource_type ],
      private.animfunc_static[ anim_type ][ resource_type ],
      ""
    );

    if ( func_end ) then

      func_end();

    end;

    return;

  end;
  
  local func_OnAnimationEnd = function ()

    local new_count = start_count + mod * new_step;

    ObjSet( private.this_anm[ resource_type ],
      {
        count = int.NumberForm( new_count )
      }
    );

    AnimPlay(
      private.this_anm[ resource_type ],
      private.animfunc_count_decrease[ anim_type ][ resource_type ],
      function ()

        private.Counter_Count(
          resource_type,
          new_count,
          new_step,
          func_end
        );

      end
    );

  end;
  
  AnimPlay(
    private.this_anm[ resource_type ],
    private.animfunc_count_increase[ anim_type ][ resource_type ],
    func_OnAnimationEnd
  );     

end;
--------------------------------------------------------------------------------
function private.Counter_ShowPopup ( resource_type, text_string )

  private.func_show_popup[ resource_type ]( text_string );

end;
--------------------------------------------------------------------------------
function private.Counter_HidePopup ( resource_type, instant )

  private.func_hide_popup[ resource_type ]( instant );

end;
--------------------------------------------------------------------------------