-- name=int_panel_questlist
--------------------------------------------------------------------------------
private.Quest = {};
--------------------------------------------------------------------------------
function public.__InitModule ()
  
  private.FuncOpen         = function () end;
  private.FuncClose        = function () end;
  private.FuncHighlightOn  = function () end;
  private.FuncHighlightOff = function () end;

  private.panel_w = 320;

  private.scc_name  = "scc_int_panel_questlist";
  private.offset    = 45; -- 45 + 290 -- TODO: 1.12.0 two titles + events frame
  private.cellsize  = 116;
  private.scc_h     = int.H() - private.offset;
  private.cells_max = math.floor( private.scc_h / private.cellsize );
  
  private.anim_time = 0.3;
  private.state     = "closed";
  private.quests    = {};
  private.dummies   = {};

  private.strings =
  {
    quests   = ne.strings.Get( "str_int_panel_questlist_title"             ),
    noquests = ne.strings.Get( "str_int_panel_questlist_no_quests"         )
  };
  
  private.event_obj =
  {
    anm = "anm_int_panel_questlist_event",
    btn = "btn_int_panel_questlist_event_action",
    spr = "spr_int_panel_questlist_event_image",
    tmr = "tmr_int_panel_questlist_event_remain"
  };
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()
  
  local screen_w = int.W();
  local screen_h = int.H();
  
  int_control.Add( "int_panel_questlist" );
  
  ObjMultiSet( { 
    { private.scc_name, {
      cellsize    = private.cellsize,
      inputrect_y = private.offset,
      inputrect_h = private.scc_h,
      croprect_y  = private.offset,
      croprect_h  = private.scc_h
    } },
    { "int_panel_questlist", { 
      pos_x = screen_w, 
      pos_y = 0 
    } },
    { "obj_int_panel_questlist_transparent", {
      pos_x          = -screen_w,
      inputrect_init = true,
      inputrect_w    = screen_w,
      inputrect_h    = screen_h,
      event_mup    = function ()
        private.Tap();
      end
    } },
    { "anm_int_panel_questlist", { 
      tt_quests = private.strings.quests
    } }
  } );

  common.TapSet( "btn_int_panel_questlist",
  {
    func_true = function ()
    
      private.Tap();

    end
  } );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Update ( 
    section_name, 
    quests_created,
    quests_updated,
    quests_deleted,
    skip_new_effect
  )
  
  local quests_prev = #private.quests;
  local multiset    = {};
  
  private.section_name = section_name;
  
  if ( #quests_created > 0 ) then
    for q, quest in ipairs( quests_created ) do
      table.insert( private.quests, quest );
      private.Quest.Create( quest, skip_new_effect );
    end;
  end;
  
  if ( #quests_deleted > 0 ) then
    for q, quest in ipairs( quests_deleted ) do
      local q_delete = private.Quest.GetNum( quest );
      if ( q_delete ) then
        table.remove( private.quests, q_delete );
        private.Quest.Delete( quest );
      end;
    end;
  end;
  
  --AL> Обновление обновлённых (кроме удалённых). Сделано не во VIEW т.к. 
  --    такие квесты могут удаляться в будущем с клёвой анимашкой
  
  if ( #quests_updated > 0 ) then
    for q, quest in ipairs( quests_updated ) do
      if ( not cmn.IsEntityInList( quests_deleted, quest, "name" )) then
        private.quests[ private.Quest.GetNum( quest ) ] = quest;
        private.Quest.Update( quest );
      end;
    end;
  end;
  
  if ( #quests_deleted > 0 ) or ( #quests_created > 0 ) then
    
    ----------------------------------------------------------------------------
    --AL> Сортировка внутри scc для корректной работы сдвига (private.Quest.Shift)
  
    table.sort( private.quests, function ( a, b ) 
      return a.priority < b.priority;
    end );
    
    for q, quest in ipairs( private.quests ) do
    
      table.insert( multiset, { "obj_int_panel_questlist_cell_"..quest.name, { 
        pos_z = q } } );
    
    end;
    
    ----------------------------------------------------------------------------
    --AL> Коррекция scrollpos после удаления квестов
    
    if ( #quests_deleted > 0 ) then
    
      local scc_params = ObjGet( private.scc_name );
      local pos_diff   = #private.quests - private.cells_max;
      if ( pos_diff < 0 ) then
        pos_diff       = 0;
      end;
      
      if ( scc_params.scrollpos > pos_diff ) then
        table.insert( multiset, { private.scc_name, { 
          scrollpos = pos_diff } } );
      end;
      
    end;
  
  end;
    
  ------------------------------------------------------------------------------
  --AL> Создание заглушек на пустые ячейки для отрисовки квестов сверху вниз
  
  if ( #quests_created ~= #quests_deleted ) then
  
    local dummies_need = 
      ( private.cells_max - quests_prev ) + 
      ( #quests_deleted - #quests_created );
    
    local dummies_add = dummies_need - #private.dummies;
    
    if     ( dummies_add > 0 ) then
        
      local current_count = #private.dummies;
      
      for d = current_count + 1, current_count + dummies_add do
        
        local obj_dummy = "obj_int_panel_questlist_dummy_"..d;
        
        ObjCreate( obj_dummy, "obj" );
        ObjAttach( obj_dummy, private.scc_name );
        table.insert( multiset, { obj_dummy, { pos_z = 100 } } );
        
        table.insert( private.dummies, obj_dummy );
        
      end;
      
    elseif ( dummies_add < 0 ) then
      
      local current_count = #private.dummies;
      local finish_count  = current_count + dummies_add + 1;
      if ( finish_count < 1 ) then
        finish_count = 1;
      end;
      
      for d = current_count, finish_count, -1 do
        
        local obj_dummy = "obj_int_panel_questlist_dummy_"..d;
        
        ObjDelete( private.dummies[ d ] );
        
        table.remove( private.dummies, d );
        
      end;
      
    end;
    
  end;

  if ( #private.quests < 1 ) then

    m_object.Set(
      "anm_int_panel_questlist",
      {
        tt_quests   = "",
        tt_noquests = private.strings.noquests
      },
      multiset
    );

  else

    m_object.Set(
      "anm_int_panel_questlist",
      {
        tt_quests   = private.strings.quests,
        tt_noquests = ""
      },
      multiset
    );

  end;

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function public.SetHandlers ( 
  func_open, 
  func_close, 
  func_highlight_on, 
  func_highlight_off 
)
  
  private.FuncOpen         = func_open;
  private.FuncClose        = func_close;
  private.FuncHighlightOn  = func_highlight_on;
  private.FuncHighlightOff = func_highlight_off;
  
end;
--------------------------------------------------------------------------------
function public.Tap ( show_event )
  
  private.Tap( show_event );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Show ()
  
  -- JK: nil is transition
  private.state = nil;

  local anm_panel = "anm_int_panel_questlist";
  
  int_control.Attach( "int_panel_questlist" );
  
  private.FuncOpen( private.ShowQuests );
  
  AnimPlay(
    anm_panel,
    tern( private.show_event )( "open_event", "open" ),
    function ()
  
      private.state = "opened";

      AnimPlay(
        anm_panel,
        tern( private.show_event )( "static_event", "static" ),
        ""
      );
      
      mediator.Notify(
        "View_Questlist_Opened",
        ( private.quests or {} )
      );
      
    end
  );
  
  int.Input_Off( "scr_map" );
  
  local screen_w = m_object.Get( "zmv_map" ).screen_w;
  
  ObjAnimate(
    "zmv_map",
    "screen_w", 0, 0, "", 
    {
      0.0, 0, screen_w, 
      0.4, 0, int.W() - private.panel_w
    }
  );
  
  cmn.PlaySnd( "quest_out" );
  
end;
--------------------------------------------------------------------------------
function public.IsActive ()

  return ( private.state and private.state == "opened" );

end;
--------------------------------------------------------------------------------
function private.OnHide ()
  
  if ( private.Quest.highlighted ) then
    
    private.Quest.Highlight( 
      private.Quest.highlighted, 
      "contract", 
      true
    );
    private.Quest.highlighted = nil;
    private.Quest.ShiftsReset();
    
  end;
  
  int_control.Detach( "int_panel_questlist" );
  
end;
--------------------------------------------------------------------------------
function public.Hide ( func_end )
  
  private.Hide( func_end );
  
end;
--------------------------------------------------------------------------------
function private.Hide ( func_end )

  -- JK: nil is transition
  private.state = nil;
  
  local obj_panel = "int_panel_questlist";
  local anm_panel = "anm_int_panel_questlist";

  ObjSet( obj_panel, { input = false } );
  
  private.FuncClose( private.HideQuests );
  
  local animfunc = tern( private.show_event )( "close_event", "close" );
  AnimPlay( anm_panel, animfunc, function () 
  
    private.state = "closed";

    ObjSet( obj_panel, { input = true } );
    private.OnHide();
    
  end );
  
  if ( private.Quest.highlighted ) then
    
    private.Quest.Tap( private.Quest.highlighted, true );
    
  end;
  
  local function func_OnShift ()

    int.Input_On( "scr_map" );
    
    if ( func_end ) then
      
      func_end();
      
    end;
    
  end;
  
  local screen_w = m_object.Get( "zmv_map" ).screen_w;
  
  ObjAnimate(
    "zmv_map",
    "screen_w", 0, 0,
    func_OnShift, 
    {
      0.0, 0, screen_w, 
      0.4, 0, int.W()
    }
  );
  
  cmn.PlaySnd( "quest_in" );
  
end;
--------------------------------------------------------------------------------
function private.Tap ( show_event )
  
  if ( not private.state ) then

    return;

  end;
  
  private.show_event = show_event;
  
  local func = {
    closed = private.Show,
    opened = private.Hide
  };
  
  func[ private.state ]();
      
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Quest.Create ( quest, skip_new_effect )
  
  local obj_name = "obj_int_panel_questlist_cell_"..quest.name;
  local new_name = "anm_int_panel_questlist_cell_new_"..quest.name;
  
  ObjClone( 
    "obj_int_panel_questlist_cell", 
    "_"..quest.name, 
    1, 
    private.scc_name
  );
  
  if ( not skip_new_effect ) then
  
    ObjClone( 
      "anm_int_panel_questlist_cell_new", 
      "_"..quest.name, 
      1, 
      "obj_int_panel_questlist_cell_"..quest.name
    );
    
    AnimPlay( new_name, "new", function ()
    
      ObjDelete( new_name );
    
    end );
    
  end;

  private.Quest.Update( quest );
  
end;
--------------------------------------------------------------------------------
function private.Quest.Update ( quest )
  
  local multiset = {};
  
  private.Quest.SetProgress( quest, "contract", true, multiset );
  private.Quest.SetTarget( quest, multiset );
  private.Quest.SetDescription( quest, multiset );
  private.Quest.SetReward( quest, multiset );
  
  ObjMultiSet( multiset );
  
  common.TapClear( "anm_int_panel_questlist_cell_"..quest.name );
  common.TapSet( "anm_int_panel_questlist_cell_"..quest.name,
  {
    func_true = function ()
    
      private.Quest.Tap( quest );

    end
  } );
  
end;
--------------------------------------------------------------------------------
function private.Quest.Delete ( quest )
  
  if ( ObjGet( "obj_int_panel_questlist_cell_"..quest.name )) then
    ObjDelete( "obj_int_panel_questlist_cell_"..quest.name );
  end;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Quest.SetTarget ( quest, multiset )
  
  local enabled  = { active = true,  visible = true };
  local disabled = { active = false, visible = false };
  local spr_params, anm_params, txt_num = private.Quest.GetTarget( quest );
  
  local spr_icon = "spr_int_panel_questlist_quest_icon_"..quest.name;
  local anm_icon = "anm_int_panel_questlist_quest_icon_"..quest.name;
  local txt_icon = "txt_int_panel_questlist_quest_num_"..quest.name;
  
  if ( spr_params ) then
    
    table.insert( multiset, { spr_icon, enabled } );
    table.insert( multiset, { anm_icon, disabled } );
    table.insert( multiset, { txt_icon, disabled } );
    table.insert( multiset, { spr_icon, spr_params } );
    
  else
    
    table.insert( multiset, { spr_icon, disabled } );
    table.insert( multiset, { anm_icon, enabled } );
    table.insert( multiset, { anm_icon, anm_params } );
    
    if ( txt_num ) then
    
      table.insert( multiset, { txt_icon, enabled } );
      table.insert( multiset, { txt_icon, { text = txt_num } } );
    
    else
    
      table.insert( multiset, { txt_icon, disabled } );
    
    end;
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Quest.SetReward ( quest, multiset )
  
  local enabled    = { active = true,  visible = true };
  local disabled   = { active = false, visible = false };
  local anm_reward = "anm_int_panel_questlist_reward_"..quest.name;
  local spr_icon   = "spr_int_panel_questlist_reward_icon_"..quest.name;
  
  if ( quest.reward ) then
  
    table.insert( multiset, { anm_reward, enabled } );
    table.insert( multiset, { anm_reward, {
      reward = "str_int_panel_questlist_reward" } } );
    table.insert( multiset, { spr_icon, { 
      res = private.Quest.GetRewardRes( quest ) } } );
    
  else
  
    table.insert( multiset, { anm_reward, disabled } );
    
  end;
    
end;
--------------------------------------------------------------------------------
function private.Quest.SetDescription ( quest, multiset )

  local text;
  local anm_name = "anm_int_panel_questlist_quest_"..quest.name;
  
  if ( quest.target.type == "QT_StageCompleted" ) then

    local stage_num          = private.GetAbsNumber( quest.target.name );
    local unique_id          = "str_qst_complete_stage_"..stage_num;
    local unique_string      = StringGet( unique_id, stage_num );
    local relative_stage_num = private.GetRelNumber( quest.target.name );

    if ( unique_id ~= unique_string ) then

      text = StringGet( unique_string, relative_stage_num );
      
    else

      text = StringGet( "str_qst_complete_stage", relative_stage_num );

    end;

  else

    text = StringGet( "str_"..quest.name );
    
  end;
  
  table.insert( multiset, { anm_name, { tt_description = text } } );
  
end;
--------------------------------------------------------------------------------
function private.Quest.SetProgress ( quest, mode, instant, ext_multiset )
  
  multiset = ext_multiset or {};
  
  local progress_current, progress_target = 0, 0;
  local cap_expand   = 120;
  local cap_contract = 100;
  local anm_progress = "anm_int_panel_questlist_quest_"..quest.name;

  ObjStopAnimate( anm_progress, "frame" );
  
  if not (
  ( quest.target.type == "QT_BuildConstructed" ) or 
  ( quest.target.type == "QT_MineConstructed" ) or 
  ( quest.target.type == "QT_MarketConstructed" ) or 
  ( quest.target.type == "QT_StageCompleted" ) or 
  ( quest.target.type == "QT_SpellCompleted" ) or 
  ( quest.target.type == "QT_MineResourceCollected" )) then

    progress_current = quest.count        or #quest.list        or 0;
    progress_target  = quest.target.count or #quest.target.list or 0;

  end;

  local progress_percent = progress_current / progress_target;
  
  if ( progress_percent > 0 ) then
    if ( mode == "expand" ) then progress_percent = progress_percent * cap_expand;
    else                         progress_percent = progress_percent * cap_contract;
    end;
  else
    progress_percent = 0;
  end;

  if ( not instant ) then
  
    local current_frame = ObjGet( anm_progress ).frame;
    if ( current_frame < 0 ) then current_frame = 0; end;
    
    if ( current_frame ~= progress_percent ) then
    
      local tme = private.anim_time;
      
      ObjAnimate( anm_progress, "frame", 0, 0, "", 
      {
        0.0, 0, current_frame, 
        tme, 2, progress_percent
      } );
      
    end;
    
  else
    
    table.insert( multiset, { anm_progress, { 
      playing = false, 
      frame   = progress_percent 
    } } );
    
  end;

  if ( not ext_multiset ) and ( instant ) then
  
    ObjMultiSet( multiset );

  end;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Quest.GetTarget ( quest )
  
  local spr_params, anm_params, txt_num;
  local icons_tex = 
    "assets/content/interface/int_panel_map_quests/anm_int_panel_quests_icons";
  local int_tex = "assets/content/interface/tex_interface_a";
  
  if ( quest.target.type == "QT_StageItemsCollected" ) then
  
    spr_params = { res = private.Quest.GetItemRes( quest ),
      scale_x = 0.5,
      scale_y = 0.5
    };
    
  elseif ( quest.target.type == "QT_FragmentCollected" ) then
    
    spr_params = { res = private.Quest.GetFragmentRes( quest ),
      scale_x = 0.5,
      scale_y = 0.5
    };
    
  elseif ( quest.target.type == "QT_StageCompleted" ) then
    
    if ( quest.enemy ) then
      
      spr_params = 
      { 
        res = private.Quest.GetEnemyRes( quest ),
        scale_x = 1.0,
        scale_y = 1.0
      };

    else -- "ride"
      
      anm_params = { 
        res      = int_tex,
        animfunc = "point_1"
      };
      txt_num = private.GetRelNumber( quest.target.name );
      
    end;
    
  elseif ( quest.target.type == "QT_StageListCompleted" ) or
         ( quest.target.type == "QT_ReachMaxLevelStages" ) then
      
    anm_params = { 
      res      = int_tex,
      animfunc = "point_1"
    };

    local target_stage = quest.current_stage;

    -- FIXME: hack for broken quests by cheater
    if not ( target_stage ) then

      target_stage = quest.target.list[ 1 ];
      DbgTrace( "[ ERROR ] Broken questlist: win states cheat used" );

    end;

    txt_num = private.GetRelNumber( target_stage );
    
  elseif ( quest.target.type == "QT_MineResourceCollected" ) then
    
    anm_params = { 
      res      = int_tex,
      animfunc = int.Icon_Animfunc( quest.target.resource )
    };
    
  elseif ( quest.target.type == "QT_MarketConstructed" ) or
         ( quest.target.type == "QT_MineConstructed" ) then
    
    anm_params = { 
      res      = icons_tex,
      animfunc = "icon_construct"
    };
    
  elseif ( quest.target.type == "QT_BuildConstructed" ) then
   
    anm_params = { 
      res      = icons_tex,
      animfunc = "icon_disenchant"
    };    
    
  elseif ( quest.target.type == "QT_SpellCompleted" ) then
    
    anm_params = { 
      res      = icons_tex,
      animfunc = "icon_spell"
    };
    
  end;
  
  if ( anm_params ) then
    anm_params.playing = true;
  end;
  
  return spr_params, anm_params, txt_num;

end;
--------------------------------------------------------------------------------
function private.Quest.GetItemRes ( quest )
  
  return common.GetItemResource( 
    "quest", 
    quest.target.name
  );
  
end;
--------------------------------------------------------------------------------
function private.Quest.GetFragmentRes ( quest )
  
  return common.GetItemResource( 
    "fragment", 
    quest.target.list[1]
  );
  
end;
--------------------------------------------------------------------------------
function private.Quest.GetRewardRes ( quest )
  
  return common.GetItemResource( 
    quest.reward[ 1 ].type, 
    quest.reward[ 1 ].name 
  );
  
end;
--------------------------------------------------------------------------------
function private.Quest.GetEnemyRes ( quest )
  
  return "assets/content/"..quest.enemy_content.."/"..quest.enemy.."/icon_"..quest.enemy;
  
end;
--------------------------------------------------------------------------------
function private.Quest.GetNum ( quest )
  
  return cmn.IsEntityInList( private.quests, quest, "name" );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
-- Выделение ячейки и раздвижение прочих, если у текущей есть ревард

function private.Quest.Tap ( quest, skip_highlight )
  
  if ( not private.Quest.highlighted ) then
    
    cmn.PlaySnd("horizontal_slide_in_true");
    
    private.FuncHighlightOn( quest.name );
    
    private.Quest.highlighted = quest;
    private.Quest.Highlight( quest, "expand", false );
    
    if ( quest.reward ) then
      private.Quest.Shift( "down", private.Quest.GetNum( quest ));
    end;
    
  else
    
    if ( private.Quest.highlighted == quest ) then
      
      if ( quest.target.type == "QT_StageListCompleted" ) or
         ( quest.target.type == "QT_ReachMaxLevelStages" ) or
         ( quest.target.type == "QT_StageItemsCollected" ) then
        
        --AL> Цикличное пролистывание по невыполненным задачам квеста
        if ( not skip_highlight ) then
          private.FuncHighlightOn( quest.name );
        end;
        
      else
      
        if ( not skip_highlight ) then
          private.FuncHighlightOff();
        end;
        
        private.Quest.highlighted = nil;
        private.Quest.Highlight( quest, "contract", false );
      
        if ( quest.reward ) then
          private.Quest.Shift( "up", private.Quest.GetNum( quest ));
        end;
        
        cmn.PlaySnd( "horizontal_slide_in_out" );
        
      end;
      
    else
    
      cmn.PlaySnd( "horizontal_slide_in_true" );
      
      private.FuncHighlightOn( quest.name );
      private.Quest.Highlight( private.Quest.highlighted, "contract", false );
      private.Quest.Highlight( quest, "expand", false );
      
      if     ( quest.reward ) and ( private.Quest.highlighted.reward ) then
        
        local new, old =
          private.Quest.GetNum( quest ),
          private.Quest.GetNum( private.Quest.highlighted );
          
        if     ( new < old ) then      private.Quest.Shift( "down", new, old + 1 );
        elseif ( new - old == 1 ) then private.Quest.Shift( "up", old - 1, new );
        else                           private.Quest.Shift( "up", old, new );
        end;
        
      elseif ( quest.reward ) and ( not private.Quest.highlighted.reward ) then
        
        private.Quest.Shift( "down", private.Quest.GetNum( quest ));
        
      elseif ( not quest.reward ) and ( private.Quest.highlighted.reward ) then
        
        private.Quest.Shift( "up", private.Quest.GetNum( private.Quest.highlighted ));
        
      --elseif (no reward for both) : do nothing
        
      end;
    
      private.Quest.highlighted = quest;
    
    end;
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Quest.Highlight ( quest, animfunc, instant )
  
  local anm_cell = "anm_int_panel_questlist_cell_"..quest.name;
        
  local end_animfunc = "contract";
  if ( animfunc == "contract" ) then
    end_animfunc = "expand";
  end;
  
  local func_static = function () 
        
    ObjSet( anm_cell, { 
      animfunc = end_animfunc, 
      frame    = 0, 
      playing  = false, 
      input    = true 
    } );
    
  end;
  
  if ( instant ) then
    
    func_static();
    
  else
  
    ObjSet( anm_cell, { input = false } );
    AnimStop( anm_cell );
    AnimPlay( 
      anm_cell,
      animfunc,
      function ()
        
        func_static();
        
      end
    );
    
  end;
  
  private.Quest.SetProgress( quest, animfunc, instant );
  
end;
--------------------------------------------------------------------------------
function private.Quest.Shift ( direction, start_index, stop_index )
  
  start_index = start_index + 1;
  stop_index = stop_index or #private.quests;
  
  local tme = private.anim_time;
  
  for q = start_index, stop_index do
    
    local obj_cell  = "anm_int_panel_questlist_cell_"..private.quests[ q ].name;
    local start_pos = ObjGet( obj_cell ).pos_y;
    local stop_pos  = 0;
    
    if ( direction == "up" ) then stop_pos = 0;
    else                          stop_pos = private.cellsize;
    end;
    
    ObjAnimate( obj_cell, "pos_y", 0, 0, "", 
    {
      0.0, 0, start_pos, 
      tme, 2, stop_pos
    } );
    
  end;
  
  --TODO: Пододвигаем контейнер вверх, если раскрываем ревард ниже границы экрана
  
end;
--------------------------------------------------------------------------------
function private.Quest.ShiftsReset ()

  if ( #private.quests >= 2 ) then
  
    local multiset = {};
    
    for q = 2, #private.quests do
      
      local obj_cell = "anm_int_panel_questlist_cell_"..private.quests[ q ].name;
      ObjStopAnimate( obj_cell, "pos_y" );
      table.insert( multiset, { obj_cell, { pos_y = 0 } } );

    end;
    
    ObjMultiSet( multiset );

  end;  
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.GetAbsNumber ( stage_name )

  return stage_name:match( "%d+" );

end;
--------------------------------------------------------------------------------
function private.GetRelNumber ( stage_name )

  local section_number  = 
  tonumber( private.GetAbsNumber( private.section_name ) );
    
  local relative_number = 
  tonumber( private.GetAbsNumber( stage_name ) ) - 110 * ( section_number - 1 );

  return relative_number;

end;
--------------------------------------------------------------------------------