-- name=int_panel_social_score
--------------------------------------------------------------------------------
function public.__InitModule ()
  
  private.app_h = int.H();
  private.app_w = int.W();
  private.app_o = int.Orientation();
  
  private.this = {
    obj    = "obj_int_panel_social_score",
    button = {
      top = "btn_int_panel_social_score_top",
      me  = "btn_int_panel_social_score_me"
    }
  };
  
  private.slot_animfunc = {
    no_heart   = "static_"..private.app_o.."_no_heart",
    with_heart = "static_"..private.app_o.."_with_heart",
    me_top     = "static_"..private.app_o.."_me_top",
    me         = "static_"..private.app_o.."_me",
    opponent   = "static_"..private.app_o.."_opponent"
  };
  
  private.scroller = "scc_int_panel_social_score_slots";

  private.transition = {
    portrait  = "pos_y",
    landscape = "pos_x"
  };
  
  private.hide_pos = {
    portrait  = private.app_h + 180,
    landscape = private.app_w + 300
  };
  
  private.show_pos = {
    portrait  = private.app_h,
    landscape = private.app_w
  };

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.Adapt();
  private.CreateButtons();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()
  
  int_control.Add( "int_panel_social_score" );

  local multiset = {};
  
  m_object.Clone(
    "scc_int_panel_social_score_default_slots",
    "anm_int_panel_social_score",
    "_rename"
  );
  
  local scroller_layout = {
    portrait = {
      cellsize =  148,
      rect_x   = -272,
      rect_y   = -85,
      rect_w   =  484,
      rect_h   =  170
    },
    landscape = {
      cellsize =  102,
      rect_x   = -150,
      rect_y   = -210,
      rect_w   =  300,
      rect_h   =  630
    }
  };
  
  m_object.Set(
    "scc_int_panel_social_score_default_slots_rename",
    {
      name        = private.scroller,
      scrollpos   = 0,
      dragenabled = true,
      horizontal  = ( private.app_o == "portrait" ),
      cellsize    = scroller_layout[ private.app_o ].cellsize,
      inputrect_x = scroller_layout[ private.app_o ].rect_x,
      inputrect_y = scroller_layout[ private.app_o ].rect_y,
      inputrect_w = scroller_layout[ private.app_o ].rect_w,
      inputrect_h = scroller_layout[ private.app_o ].rect_h,
      croprect_x  = scroller_layout[ private.app_o ].rect_x,
      croprect_y  = scroller_layout[ private.app_o ].rect_y,
      croprect_w  = scroller_layout[ private.app_o ].rect_w,
      croprect_h  = scroller_layout[ private.app_o ].rect_h,
    },
    multiset
  );
  
  local position = {
    portrait  = { private.app_w / 2, private.app_h },
    landscape = { private.app_w,     private.app_h / 2 }
  };
  
  m_object.Set(
    "obj_int_panel_social_score",
    {
      pos_x = position[ private.app_o ][ 1 ],
      pos_y = position[ private.app_o ][ 2 ]
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.CreateButtons ()
  
  int.Button_Create(
    private.this.button.top,
    {
      title  = "str_int_panel_social_score_top",
      color  = "green",
      design = "medium",
    },
    function()
      
      private.ScrollTo( "top" );
      
    end
  );
  
  int.Button_Create(
    private.this.button.me,
    {
      title  = "str_int_panel_social_score_me",
      color  = "blue",
      design = "medium",
    },
    function()
      
      private.ScrollTo( "me" );
      
    end
  );
  
end;
--------------------------------------------------------------------------------
function private.Clean ()
  
  private.friends = nil;
  
  local childs = ObjGetRelations( "scc_int_panel_social_score_slots" ).childs;

  for c = 1, #childs do

    ObjDetach( childs[ c ] );

  end;
  
  m_object.Delete( "anm_int_panel_social_score_slot_player" );
  m_object.Delete( "anm_int_panel_social_score_slot_opponent" );

  int_control.Detach( "int_panel_social_score" );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show (
    point, -- Point can be "start" or "win"
    friends,
    my_id,
    func_send_heart
  )

  if ( private.active ) then
    
    return;
    
  end;
  
  private.active = true;
  
  private.friends = friends;
  private.my_id   = tonumber( my_id );
  private.my_pos  = private.GetMyPos();

  private.FuncSendHeart = func_send_heart;
  
  int_control.Attach( "int_panel_social_score" );

  private.SetDefault( point, friends );

  public.VisualShow();

end;
--------------------------------------------------------------------------------
function public.Hide ()

  if ( not private.active ) then
    
    return;
    
  end;
  
  public.VisualHide( private.Clean );
  
  private.active = false;

end;
--------------------------------------------------------------------------------
function public.Friends_Create ( friends )
  
  -- Hack for ___InitObjects.
  m_object.Set( "int_panel_social_score", { active = true } );

  for i = 1, #friends do

    m_object.Clone(
      "anm_int_panel_social_score_slot",
      "ne_storage",
      "_"..friends[ i ].userId,
      1
    );

  end;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SetDefault ( point, friends )
  
  local multiset  = {};
  local view_type = tern( point:find( "win" ) )( "win", "start" );
  local cellsize  = m_object.Get( private.scroller ).cellsize;
  
  for i = 1, #friends do

    local friend    = friends[ i ];
    local id        = friend.userId;
    local animfunc  = private.slot_animfunc.no_heart;
    local anm_slot  = "anm_int_panel_social_score_slot_"..id;
    local spr_photo = "spr_int_panel_social_score_slot_photo_"..id;
    
    if ( not m_object.Exists( anm_slot ) ) then
      
      m_object.Clone(
        "anm_int_panel_social_score_slot",
        private.scroller,
        "_"..id,
        1
      );
      
    end;
    
    m_object.Attach( anm_slot, private.scroller );

    if ( friend.url == "" ) then friend.url = nil; end;
    
    friend.res = friend.res or
      "assets/content/interface/int_dialog_stage_friends/default_0";

    if ( friend.heart         ) and
       ( view_type == "start" ) then

      animfunc = private.slot_animfunc.with_heart;
       
      int.Button_Create(
        "btn_int_panel_social_score_slot_heart_"..id,
        {
          design = "heart_send"
        },
        function()
          
          private.FuncSendHeart( id );
          
          ObjSet(
            anm_slot,
            {
              animfunc = private.slot_animfunc.no_heart
            }
          );
          
        end
      );

    end;
    
    if ( id == private.my_id ) then

      friend.res  = friend.res or "assets/content/interface/int_dialog_stage_friends/default_0";
      friend.name = "str_int_panel_social_score_me";
      animfunc    = private.slot_animfunc.me_top;
      
    end;

    m_object.Set(
      spr_photo,
      {
        url          = friend.url,
        res          = friend.res,
        default_res  = friend.res,
        trg_resloaded = function ()

          int.ImageAdjust( spr_photo, 74 );

        end
      },
      multiset
    );


    m_object.Set(
      anm_slot,
      {
        animfunc = animfunc,
        tt_name  = friend.name,
        score    = int.NumberForm( friend.score, true ),
        place    = tostring( i )
      },
      multiset
    );
    
  end;

  private.SccAdapt( view_type );
  private.ScrollTo( "top" );
  
  if ( view_type == "win" ) then
    
    private.ScrollTo( "me", - 1 );
    
    local player_data = friends[ private.my_pos ];
    
    private.Win_SetupRating(
      "player",
      player_data,
      private.my_id,
      private.slot_animfunc.me,
      multiset
    );
    
    local opponent_pos = private.my_pos + 1;
    
    player_data = friends[ opponent_pos ];

    if ( player_data ) then
    
      private.Win_SetupRating(
        "opponent",
        player_data,
        opponent_pos,
        private.slot_animfunc.opponent,
        multiset
      );
      
    end;
    
  end;
 
  local func_create_dummy = function ( i )

    local obj_dummy = "obj_int_panel_social_score_slot_dummy_"..i;
    
    if ( not m_object.Exists( obj_dummy ) ) then
      
      m_object.Create( obj_dummy, "obj" );
      
    end;
    
    m_object.Attach( obj_dummy, private.scroller );

  end;

  local cells_visible = math.floor( private.app_w / cellsize ) - 1;
  
  -- func_create_dummy( 0 );
  
  if ( #friends < cells_visible + 1 ) then
    
    for i = 1, ( cells_visible ) - #friends do
      
      func_create_dummy( i );
      
    end;
    
  end;
  
  m_object.Set(
    "anm_int_panel_social_score",
    {
      animfunc = view_type.."_"..private.app_o
    },
    multiset
  );
  
  m_object.Set(
    "obj_int_panel_social_score",
    {
      [ private.transition[ private.app_o ] ] = private.hide_pos[ private.app_o ],
      alp = 0
    },
    multiset
  );

  m_object.Multiset( multiset );

  for i = 1, #friends do
    
    int.ImageAdjust(
      "spr_int_panel_social_score_slot_photo_"..friends[ i ].userId,
      74
    );
    
  end;
  
  if ( view_type == "win" ) then
    
    int.ImageAdjust(
    "spr_int_panel_social_score_slot_photo_player",
      74
    );
    
    int.ImageAdjust(
    "spr_int_panel_social_score_slot_photo_opponent",
      74
    );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.ScrollTo ( target, offset )
  
  offset = offset or 0;
  
  if ( target == "top" ) then

    m_object.Set(
      private.scroller,
      {
        scrollpos = 0
      }
    );

  elseif ( target == "me" ) then
    
    local friends_count = #private.friends;
    local cellsize      = m_object.Get( private.scroller ).cellsize;
    local dimension     = math.min( private.app_w, private.app_h );
    local cells_visible = 3;
    local target_pos    = private.my_pos - 1;

    if ( private.my_pos > friends_count - cells_visible ) then
      
      target_pos = friends_count - cells_visible;
      
    else
      
      target_pos = tern( target_pos < 1 )( 0, target_pos + offset );
      
    end;
    
    m_object.Set(
      private.scroller,
      {
        scrollpos = target_pos
      }
    );

  end;

end;
--------------------------------------------------------------------------------
function private.GetMyPos ()

  for f = 1, #private.friends do
    
    if ( tonumber( private.friends[ f ].userId ) == private.my_id ) then
      
      return f;
      
    end;
    
  end;

end;
--------------------------------------------------------------------------------
function private.Win_SetupRating ( player, data, place, animfunc, multiset )
  
  local anm_slot  = "anm_int_panel_social_score_slot_"..player;
  local spr_photo = "spr_int_panel_social_score_slot_photo_"..player;
  
  m_object.Clone(
    "anm_int_panel_social_score_slot",
    "obj_int_panel_social_score_"..player,
    "_"..player,
    1
  );
  
  m_object.Set(
    anm_slot,
    {
      animfunc = animfunc,
      tt_name  = data.name,
      score    = int.NumberForm( data.score, true ),
      place    = tostring( place ),
    },
    multiset
  );
  
  data.res = data.res or
    "assets/content/interface/int_dialog_stage_friends/default_0";
  
  m_object.Set(
    spr_photo,
    {
      url           = data.url,
      res           = data.res,
      default_res   = data.res,
      trg_resloaded = function ()
        
        int.ImageAdjust( spr_photo, 74 );
        
      end
    },
    multiset
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SccAdapt ( point )
  
  local scroller_layout = {
    start = {
      portrait = {
        pos_x    = - 215,
        rect_x   =   0,
        rect_w   =   492
      },
      landscape = {
        rect_x   =   0,
        rect_w   =   300
      }    
    },
    win = {
      portrait = {
        pos_x    = - 215,
        rect_x   =   0,
        rect_w   =   436
      },
      landscape = {
        rect_x   =   0,
        rect_w   =   300
      }    
    }
  };
  
  m_object.Set( private.scroller,
    {
      pos_x       = scroller_layout[ point ][ private.app_o ].pos_x,
      inputrect_x = scroller_layout[ point ][ private.app_o ].rect_x,
      inputrect_w = scroller_layout[ point ][ private.app_o ].rect_w,
      croprect_x  = scroller_layout[ point ][ private.app_o ].rect_x,
      croprect_w  = scroller_layout[ point ][ private.app_o ].rect_w
    }
  );
  
end;  
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.VisualShow ()
  
  if ( private.active ) then
    
    int.Input_Off( private.this.obj );
    
    local transition = private.transition[ private.app_o ];
    local start_pos  = m_object.Get( "obj_int_panel_social_score" )[ transition ];
    local stop_pos   = private.show_pos[ private.app_o ];
    
    ObjAnimate(
      "obj_int_panel_social_score",
      transition, 0, 0,
      function ()
        
        int.Input_On( private.this.obj );
        
      end, 
      {
        0.0, 0, start_pos,
        0.3, 0, stop_pos
      }
    );

    common.Animation_Fade_In( 0.3, "obj_int_panel_social_score" );
     
  end;
  
end;
--------------------------------------------------------------------------------
function public.VisualHide ( func_end )
  
  if ( private.active ) then
    
    int.Input_Off( private.this.obj );
    
    local transition = private.transition[ private.app_o ];
    local start_pos  = m_object.Get( "obj_int_panel_social_score" )[ transition ];
    local stop_pos   = private.hide_pos[ private.app_o ];
    
    ObjAnimate(
      "obj_int_panel_social_score",
      transition, 0, 0, func_end, 
      {
        0.0, 0, start_pos,
        0.3, 0, stop_pos
      }
    );

    common.Animation_Fade_Out( 0.3, "obj_int_panel_social_score" );
    
  end;
  
end;