-- name=int_panel_stage_boosters
function public.__InitModule ()

  private.anim_time  = 0.3;
  
  private.hide_x = 0;
  private.hide_y = 0;
  
  private.show_x = 0;
  private.show_y = 0;
  
  private.scale = 1.0;
  
  private.is_vertical = false;
  
  private.active   = false;
  private.hideable = true;
  
  private.desc_show = false;
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  int_control.Add( "int_panel_stage_boosters" );

end;
--------------------------------------------------------------------------------
function public.SetScale ( s )
  
  private.scale = s;
    
end;
--------------------------------------------------------------------------------
function public.SetPosVisible ( x, y )
  
  private.show_x = x;
  private.show_y = y;
  
end;
--------------------------------------------------------------------------------
function public.SetPosHidden ( x, y )
  
  private.hide_x = x;
  private.hide_y = y;
  
end;
--------------------------------------------------------------------------------
function public.SetVertical ( is_vertical )
  
  private.is_vertical = is_vertical;
  
end;
--------------------------------------------------------------------------------
function public.SetHideable ( hideable )
  
  private.hideable = hideable;
  
end;
--------------------------------------------------------------------------------
function public.IsHideable ()
  
  return private.hideable;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( boosters_data, func_tap, func_popup_show, func_popup_hide )
  
  if ( private.active ) then return end;
  
  private.active = true;

  private.booster_index  = {};
  private.emergency_mode = false;
  private.booster_count  = #boosters_data;

  private.Tap    = func_tap;
  private.Popup_Show = func_popup_show;
  private.Popup_Hide = func_popup_hide;

  int_control.Attach( "int_panel_stage_boosters" );
  
  ObjSet( "int_panel_stage_boosters",
  {
    input   = true,
    active  = true,
    visible = true
  } );
  
  private.SetDefault( boosters_data );
  
  local animfunc = "booster"..private.booster_count.."_static";
  
  AnimPlay( "anm_int_panel_stage_boosters", animfunc, "" );
  
  ObjSet( "anm_int_panel_stage_boosters",
  {
    input   = true,
    active  = true,
    visible = true
  } );

  local tme = private.anim_time;

  ObjAnimate( "obj_int_panel_stage_boosters", "pos_xy", 0, 0, "", 
  {
    0.0, 2, private.hide_x, private.hide_y,
    tme, 2, private.show_x, private.show_y
  } );

end;
--------------------------------------------------------------------------------
function public.Hide ()

  if ( not private.active ) then

    return;

  end;

  private.active = false;
  private.booster_count = nil;

  ObjSet( "int_panel_stage_boosters", { input = false } );

  local func_end = function ()

    ObjSet( "int_panel_stage_boosters", { active = false, visible = false } );

    int_control.Detach( "int_panel_stage_boosters" );

  end;

  local tme = private.anim_time;

  ObjAnimate( "obj_int_panel_stage_boosters", "pos_xy", 0, 0, func_end, 
  {
    0.0, 2, private.show_x, private.show_y,
    tme, 2, private.hide_x, private.hide_y
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Update ( boosters_list )

  if ( not private.active ) then

    return;

  end; 

  for b = 1, #boosters_list do

    local booster = boosters_list[ b ];
    local index   = private.booster_index[ booster.name ];

    local obj = "anm_int_panel_stage_booster_"..index.."_icon";

    local params =
    {
      count = int.NumberForm( booster.count ),
      playing = true
    };

    if ( booster.count > 0 ) then

      params.animfunc = "booster";

    else

      params.animfunc = "booster_add";

    end;

    ObjSet( obj, params );

  end;

end;
--------------------------------------------------------------------------------
function public.SetInput ( flag )
  
  ObjSet( "obj_int_panel_stage_boosters", { input = flag } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.OnBoosterTap ( booster_name )

  private.StartAnim( booster_name );
  
  cmn.PlaySnd("select_boost");

  if ( not private.Tap( booster_name ) ) then
    
    AnimPlay( "anm_int_panel_stage_boosters", 
              "booster"..private.booster_count.."_static", 
              "" );
    
    private.StopAnim( booster_name );

    return;

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.EmergencyMode_TryStart ()
  
  private.EmergencyMode_TryStart();
  
end;
--------------------------------------------------------------------------------
function public.EmergencyMode_Stop ()
  
  private.EmergencyMode_Stop();
  
end;
--------------------------------------------------------------------------------
function private.EmergencyMode_TryStart ()
  
  if      ( private.booster_count     )
  and     ( private.booster_count > 0 )
  and not ( private.emergency_mode    ) then
  
    private.emergency_mode = true;
    
    private.EmergencyMode_Cycle();
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.EmergencyMode_Cycle ()

  local function func_bounce ( booster_id, func_after_bounce )

    local sprite = "anm_int_panel_stage_booster_"..booster_id.."_icon";
    common.Animation_Bounce( sprite, func_after_bounce );

  end;

  local callbacks = {};

  for i = 1, private.booster_count do
    table.insert( callbacks, function ( after ) func_bounce( i, after ) end );
  end;

  table.insert( callbacks, private.EmergencyMode_Stop );

  common.RunCallbackList( callbacks );
  
end;
--------------------------------------------------------------------------------
function private.EmergencyMode_Stop ()
  
  private.emergency_mode = false;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.StartAnim ( booster_name )

  private.StartAnim( booster_name );

end;
--------------------------------------------------------------------------------
function public.StopAnim ( booster_name )
 
  if ( private.booster_index[ booster_name ] ) then
    
    if ( private.desc_show ) then 
      
      local anm_name;
      
      if ( private.booster_count == 1 ) then 
      
        anm_name = "booster1_desc_disappear";
      
      else
        
        anm_name = "booster"..private.booster_count..private.booster_index[ booster_name ].."_desc_disappear";
      
      end;
    
      ObjSet( "anm_int_panel_stage_boosters", { 
        description = "str_item_"..booster_name.."_description",
        animfunc = anm_name,
        playing = true;
      } );
      
      private.desc_show = false;
      
    end;  

    private.StopAnim( booster_name );
    
    return true;

  end;
  
  return false;

end;
--------------------------------------------------------------------------------
function private.StartAnim ( booster_name )
  
  local anm_name;
    
  if ( private.booster_count == 1 ) then 
    
    anm_name = "booster1_desc_appear";
    
  else
    
    anm_name = "booster"..private.booster_count..private.booster_index[ booster_name ].."_desc_appear";
  
  end;
  
  ObjSet( "anm_int_panel_stage_boosters", { 
    description = "str_item_"..booster_name.."_description",
    animfunc = anm_name,
    playing = true;
  } );
    
  private.desc_show = true;   

  local index = private.booster_index[ booster_name ];

  ObjAttach( "anm_int_panel_stage_booster_selection", "obj_int_panel_stage_booster_"..index );
  ObjSet( "anm_int_panel_stage_booster_selection", { active = true, visible = true, scale_x = 1, scale_y = 1 } );
  
  AnimPlay( "anm_int_panel_stage_booster_selection", "select", "" );
    
  local spr_params = ObjGet( "anm_int_panel_stage_booster_"..index.."_icon" );

  ObjAnimate( "anm_int_panel_stage_booster_"..index.."_icon", "scale_xy", 1, 0, "", 
  {
    0.0, 0, spr_params.scale_x, spr_params.scale_y,
    0.5, 0, spr_params.scale_x + 0.1, spr_params.scale_y + 0.1,
    1.0, 0, spr_params.scale_x, spr_params.scale_y
  } );

end;
--------------------------------------------------------------------------------
function private.StopAnim ( booster_name )

  AnimStop( "anm_int_panel_stage_booster_selection" );
  ObjSet( "anm_int_panel_stage_booster_selection", { active = false, visible = false } );

  local index = private.booster_index[ booster_name ];

  for i = 1, 3 do

    if not ( i == index ) then

      ObjStopAnimate( "anm_int_panel_stage_booster_"..i.."_icon", "scale_xy" );
      ObjSet( "anm_int_panel_stage_booster_"..i.."_icon", { scale_x = 1, scale_y = 1 } );

    end;

  end;

  ObjStopAnimate( "anm_int_panel_stage_booster_"..index.."_icon", "scale_xy" );
  ObjSet( "anm_int_panel_stage_booster_"..index.."_icon", { scale_x = 1, scale_y = 1 } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SetDefault ( boosters_data )

  ObjSet( "obj_int_panel_stage_boosters", { input = true, visible = true, active = true } );
  
  -- Booster icons step
  local step = 75;
  
  local pos_x = { { 0 }, { -step, step }, { -2 * step, 0, 2 * step } };
  local pos_y = { { 0 }, {     0,    0 }, {         0, 0,        0 } };  
  
  if ( private.is_vertical ) then
    
    -- Switch panel orientation
    pos_x, pos_y = pos_y, pos_x;
    
  end;

  -- Set booster scales and colors
  local multiset_table = {};
  
  for i = 1, 3 do

    table.insert( multiset_table, 
    {
      "obj_int_panel_stage_booster_"..i, { visible = false, input = false, active = false }
    } );

  end;

  ObjMultiSet( multiset_table );

  -- Set booster contents
  for i = 1, #boosters_data do

    local booster = boosters_data[ i ];
    private.booster_index[ booster.name ] = i;

    private.StopAnim( booster.name );

    local animfunc = "booster_add";
    if ( booster.count > 0 ) then

      animfunc = "booster";

    end;
    
    if ( booster.unlocked ) then
      
      ObjMultiSet( {
        { "obj_int_panel_stage_booster_"..i,
          { 
            active  = true,
            visible = true,
            input   = true,
           }
        },
        { "anm_int_panel_stage_booster_"..i.."_icon",
          { 
            count = int.NumberForm( booster.count ),
            booster_icon = "assets/content/boosters/"..booster.name,
            scale_x = 1,
            scale_y = 1,
            animfunc = animfunc,
            playing = true
          } 
        }
      } );

    else

      private.booster_count = private.booster_count - 1;
      
    end;

    common.TapSet( "obj_int_panel_stage_booster_"..i,
    {
      func_true = function ()

        if ( not private.popup_active ) then

          private.OnBoosterTap( booster.name );

          cmn.PlaySnd( "app_info" );

        end;

        private.popup_active = false;

        private.Popup_Hide( true );

      end,

      func_menter = function ()

        --

      end,

      func_mleave = function ()

        if ( private.popup_active ) then
          
          private.Popup_Hide( true ); 
          
        end;        
        
      end,

      func_hold_down = function ()

        --private.Popup_Show( 
        --"str_item_"..booster.name.."_caption",
        --"str_item_"..booster.name.."_description",
        --"obj_int_panel_stage_booster_"..i,
        --{ 0, -160 }
        --);

        --private.popup_active = true;

      end,

      func_hold_up = function ( ) 

        private.Popup_Hide( true );

      end;

    } );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Popup_Show ()
  
  if ( private.booster_count and private.booster_count > 0 ) then
    
    private.Popup_Show( 
      "str_int_panel_boosters_use_booster_caption", 
      "str_int_panel_boosters_use_booster",
      "obj_int_panel_stage_booster_2",
      { 0, - 160 }
    );
          
  end;
  
end;