-- name=int_panel_stage_progress_fight
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ INTERFACE ] [ PANEL_PROGRESS ] "..text );

end;
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.active = false;
  
  private.screen_w = int.W();
  private.screen_h = int.H();

  private.attack_queue = {};
  
  private.hero_audio  = "assets/content/hero/aud_hero_";
  private.healthbar_w = 200;

  private.anim_time = 0.3;
  private.fg_scale  = 1.0;
  
  -- Hero and enemy distance
  private.dst_near = 350;
  private.dst_far  = 350;
  private.dst_off  = 350;
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()
 
  int_control.AddToStage( "int_panel_stage_progress_fight" );

  private.Adapt();

  mediator.Subscribe( "Model_Stage_HeroHealthRecovered", private.Hero_Respawn );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show (
    stage_data,
    hero_data,
    enemy_data,
    func_skin_attach,
    func_skin_detach
  )
  
  if ( private.active ) then

    private.Reset();

  end;

  private.Hero_SkinAttach = func_skin_attach;
  private.Hero_SkinDetach = func_skin_detach;

  ObjSet( "obj_int_panel_stage_progress_fight_touch_enemy",
  {
    input = true } );

  int_control.Attach( "int_panel_stage_progress_fight" );

  ObjSet( "obj_int_panel_stage_progress_fight",
  {
    visible = false,
    active  = false } );

  private.SetDefaults( stage_data, hero_data, enemy_data );
  
  ObjMultiSet( {
    { "int_panel_stage_progress_fight",
      {
        input   = true,
        active  = true,
        visible = true,
        pos_x   = 0.5 * private.screen_w
      }
    },
    { "anm_int_panel_stage_progress_fight_animations_enemy",
      {
        pos_x = 0.5 * ( private.dst_far - private.dst_off )
      }
    },
    { "anm_int_panel_stage_progress_fight_animations_enemy_hero",
      {
        pos_x = -private.dst_far + private.dst_off
      }
    },
    { "anm_int_panel_stage_progress_fight_healthbars_hero",
      {
        pos_x = private.healthbar_hero_pos_start
      }
    },
    { "anm_int_panel_stage_progress_fight_healthbars_enemy",
      {
        pos_x = private.healthbar_enemy_pos_start
      }
    },
  } );
  
  public.ShowDots( 0, 0 );

  ObjSet( "int_panel_stage_progress_fight", { pos_y = private.pos_y } );

  ObjAnimate(
  "anm_int_panel_stage_progress_fight_healthbars_hero",
  "pos_x", 0, 0, "", 
  {
    0.0,               2, private.healthbar_hero_pos_start,
    private.anim_time, 2, private.healthbar_hero_pos_stop
  } );

  common.Animation_Fade_In(
    private.anim_time,
    "anm_int_panel_stage_progress_fight_healthbars_hero",
    true
  );

  ObjAnimate(
  "anm_int_panel_stage_progress_fight_healthbars_enemy",
  "pos_x", 0, 0, "", 
  {
    0.0,               2, private.healthbar_enemy_pos_start,
    private.anim_time, 2, private.healthbar_enemy_pos_stop
  } );

  common.Animation_Fade_In(
    private.anim_time,
    "anm_int_panel_stage_progress_fight_healthbars_enemy",
    true
  );
  
  private.active = true;

end;
--------------------------------------------------------------------------------
function public.Hide ()

  private.active = false;

  ObjSet(
    "int_panel_stage_progress_fight", 
    { input = false, active  = false, visible = false }
  );
  
  ObjSet(
    "anm_int_panel_stage_progress_fight_animations_hero_attack_time", 
    { playing = false, endtrig = "" }
  );

  int_control.Detach( "int_panel_stage_progress_fight" );
  private.Reset();

end;
--------------------------------------------------------------------------------
function public.IsWinAnimEnded ()

  return private.WinAnimEnd;

end;
--------------------------------------------------------------------------------
function private.Adapt ()

  local percent_w = private.screen_w / 100;

  private.healthbar_hero_pos_start  = - ( private.screen_w / 2 ) - private.healthbar_w;
  private.healthbar_hero_pos_stop   = - ( 50 * percent_w );
  private.healthbar_enemy_pos_start =     private.screen_w;
  private.healthbar_enemy_pos_stop  =   ( 50 * percent_w );
  
  local objects = {};

  table.insert( objects,
  { "anm_int_panel_stage_progress_fight_healthbars_enemy",
    {
      pos_x = private.healthbar_enemy_pos_start } } );
      
  table.insert( objects,
  { "anm_int_panel_stage_progress_fight_healthbars_hero",
    {
      pos_x = private.healthbar_hero_pos_start } } );

  local parts_count = private.screen_w / 128;

  for i = 1, parts_count do

    ObjClone(
      "spr_int_panel_stage_progress_fight_fg_decoration",
      "_"..i, 1,
      "obj_int_panel_stage_progress_fight_fg"
    );

    table.insert( objects,
    { "spr_int_panel_stage_progress_fight_fg_decoration_"..i,
      {
        pos_x = 128 * ( i - 0.5 )
      }
    } );

  end;

  ObjMultiSet( objects );
  
  -- Resize decoartion grid ( horizontal layout )
  local grd_size = {
    left   = private.screen_w / 2,
    right  = private.screen_w / 2,
    bottom = 0,
    top    = 40
  }
  
  local grd_border = {
    left = 0,
    right = 0,
    bottom = 0,
    top = 40
  }
  
  int.Grid_Resize(
    "grd_int_panel_stage_progress_fight_fg",
    grd_size,
    grd_border
  );

  common.TapSet( "obj_int_panel_stage_progress_fight_touch_hero",
  {
    func_true = function ()

      private.Hero_OnTap();

    end
  } );

  common.TapSet( "obj_int_panel_stage_progress_fight_touch_enemy",
  {
    func_true = function ()

      private.Enemy_OnTap();

    end
  } );

end;
--------------------------------------------------------------------------------
function public.SetOffset ( offset )
  
  private.pos_y = offset;
  
  local bg_w, bg_h = 768, 340; -- BG texture size
  
  local bg_scale_h = offset / bg_h;
  local bg_scale_w = private.screen_w / bg_w;
  
  local bg_scale = common.Round( math.max( bg_scale_h, bg_scale_w ), 2 );
  
  local objects = {};
  
  table.insert( objects,
  { "obj_int_panel_stage_progress_fight",
    {
      croprect_h =  offset, 
      croprect_y = -offset,
      croprect_w =  private.screen_w,
      croprect_x = -private.screen_w / 2 } } );
      
  table.insert( objects,
  { "spr_int_panel_stage_progress_fight_bg",
    {
      drawoff_y = -0.5 * bg_h,
      scale_x   =  bg_scale, 
      scale_y   =  bg_scale
    }
  } );
   
  table.insert( objects,
  { "obj_int_panel_stage_progress_fight_healthbars",
    {
      pos_y = -offset + 5
    }
  } );
  
  table.insert( objects, 
  { "obj_int_panel_stage_progress_fight_fg",
    {
      pos_x = -private.screen_w / 2
    }
  } );
  
  table.insert( objects, 
  { "anm_int_panel_stage_progress_fight_fg_decoration",
    {
      pos_x     = private.screen_w / 2,
      scale_x   = bg_scale,
      scale_y   = bg_scale
    }
  } );
  
  table.insert( objects, 
  { "obj_int_panel_stage_progress_fight_touch_hero",
    {
      pos_x        = -private.screen_w / 2,
      pos_y        = -offset,
      inputrect_w  =  private.screen_w / 2,
      inputrect_h  =  offset
    }
  } );
  
  table.insert( objects, 
  { "obj_int_panel_stage_progress_fight_touch_enemy",
    {
      pos_x        =  0,
      pos_y        = -offset,
      inputrect_w  =  private.screen_w / 2,
      inputrect_h  =  offset
    }
  } );

  ObjMultiSet( objects );
  
end;
--------------------------------------------------------------------------------
function public.SetSmoothBorder ( is_smooth_border )
  
  local is_waves_visible = true;
  local is_grad_visible  = false;
  
  if ( is_smooth_border ) then
    
    is_waves_visible, is_grad_visible = is_grad_visible, is_waves_visible;
    
  end;
  
  ObjMultiSet( {
    { "obj_int_panel_stage_progress_fight_fg", 
      { visible = is_waves_visible } },
    { "grd_int_panel_stage_progress_fight_fg", 
      { visible = is_grad_visible  } },
  } );
  
end;
--------------------------------------------------------------------------------
function public.SetDistances ( near, far )
  
  private.dst_near = near;
  private.dst_far  = far;
  
end;
--------------------------------------------------------------------------------
function public.SetScale ( scale )
  
  private.fg_scale = scale;
  
end;
--------------------------------------------------------------------------------
function private.Reset ()

  private.WinAnimEnd = false;

  private.Hero_SkinDetach( private.player_skin );

end;
--------------------------------------------------------------------------------
function private.SetColor ( color )

  local objects     = {};
  local parts_count = private.screen_w / 128;

  for i = 1, parts_count do

    table.insert( objects,
    { "spr_int_panel_stage_progress_fight_fg_decoration_"..i,
      {
        color_r = color.r,
        color_g = color.g,
        color_b = color.b } } );

  end;
  
  table.insert( objects,
  { "grd_int_panel_stage_progress_fight_fg",
    {
      color_r = color.r,
      color_g = color.g,
      color_b = color.b } } );
  
  ObjMultiSet( objects );

end;

--------------------------------------------------------------------------------
function private.SetDefaults ( stage_data, hero_data, enemy_data )

  private.WinAnimEnd = false;

  private.SetColor( stage_data.color );

  private.enemy_name = enemy_data.name;

  -- player name and skin
  local player_name   = m_player.GetName();
  private.player_skin = private.Hero_SkinAttach(
                          "anm_int_panel_stage_progress_fight_animations_enemy"
                        );

  -- health buffers
  private.enemy_health = enemy_data.health;
  private.hero_health  = hero_data.health;

  -- detect zone name
  local zone = stage_data.zone;
  local content = stage_data.content;

  -- enemy resources
  local enemy_content = enemy_data.content;
  local enemy_res     = "assets/content/"..enemy_content.."/"..enemy_data.name.."/anm_"..enemy_data.name;
  private.enemy_audio = "assets/content/"..enemy_content.."/"..enemy_data.name.."/aud_";

  -- setting objects
  local objects = {};

  local backgrounds = 
  {
    default   = "bg_"..zone.."_fight",
    reddragon = "bg_"..enemy_data.name.."_fight",
    banshee   = "bg_"..enemy_data.name.."_fight",
    trollking = "bg_"..enemy_data.name.."_fight"
  };

  local bg_name = ( backgrounds[ enemy_data.name ] or backgrounds.default );
  local decoration_res = "assets/content/"..content.."/anm_"..zone.."_fight_objects";

  if ( backgrounds[ enemy_data.name ] ) then

    decoration_res = "assets/content/"..content.."/anm_"..enemy_data.name.."_fight_objects";

  end;

  if ( stage_data.realtype ) and ( stage_data.realtype == "misc" ) then

    bg_name = backgrounds.default;
    decoration_res = "assets/content/"..content.."/anm_"..zone.."_fight_objects";

  end;
  
  table.insert( objects,
  { "obj_int_panel_stage_progress_fight_animations",
    {
      scale_x = private.fg_scale,
      scale_y = private.fg_scale,
      pos_y   = -50 * private.fg_scale
    }
  } );
  
  if ( int.Landscape() ) then
    
    private.static_suffix = "_static_landscape";
    
  else
    
    private.static_suffix = "_static";
    
  end;

  table.insert( objects,
  { "anm_int_panel_stage_progress_fight_healthbars_enemy",
    {
      animfunc = "enemy_healthbar" .. private.static_suffix
    }
  } );
  
  table.insert( objects,
  { "anm_int_panel_stage_progress_fight_healthbars_hero",
    {
      animfunc = "hero_healthbar" .. private.static_suffix
    }
  } );
  
  table.insert( objects,
  { "anm_int_panel_stage_progress_fight_healthbars_enemy_healthbar",
    {
      animfunc = "full",
      count    = tostring( math.floor( private.enemy_health ) )
    }
  } );
  
  table.insert( objects,
  { "anm_int_panel_stage_progress_fight_healthbars_hero_healthbar",
    {
      animfunc = "full",
      count    = tostring( math.floor( private.hero_health ) )
    }
  } );
  
  table.insert( objects, { "anm_int_panel_stage_progress_fight_animations_enemy",             { res = enemy_res } } );
  table.insert( objects, { "anm_int_panel_stage_progress_fight_animations_enemy_attack_time", { res = enemy_res } } );
  table.insert( objects, { "spr_int_panel_stage_progress_fight_bg",                           { res = "assets/content/"..content.."/"..bg_name } } );

  table.insert( objects, { "anm_int_panel_stage_progress_fight_fg_decoration",
  {
    res      = decoration_res,
    animfunc = "decoration_fg",
    playing  = true
  } } );

  table.insert( objects, { "obj_int_panel_stage_progress_fight",                  { visible = true, active = true } } );
  table.insert( objects, { "anm_int_panel_stage_progress_fight_healthbars_hero",  { tt_name = player_name } } );
  table.insert( objects, { "anm_int_panel_stage_progress_fight_healthbars_enemy", { tt_name = "str_enemy_name_"..enemy_data.name } } );
  
  table.insert( objects, { "anm_int_panel_stage_progress_fight_healthbars_hero_ability",  { visible = false } } );
  table.insert( objects, { "anm_int_panel_stage_progress_fight_healthbars_enemy_ability", { visible = false } } );
  
  table.insert( objects, { "anm_int_panel_stage_progress_fight_healthbars_hero_icon",  { visible = true } } );
  table.insert( objects, { "spr_int_panel_stage_progress_fight_healthbars_enemy_icon", { visible = true } } );
  
  table.insert( objects, { "anm_int_panel_stage_progress_fight_healthbars_hero_icon", 
  {
    res      = "assets/content/" .. hero_data.skin_res
                 .. "/anm_skin_" .. hero_data.skin_name,
    animfunc = "marker",
    playing  = true
  } } );
                            
  table.insert( objects, { "spr_int_panel_stage_progress_fight_healthbars_enemy_icon", 
  {
    res = "assets/content/" .. enemy_data.content .. "/" .. enemy_data.name
                            .. "/icon_" .. enemy_data.name
  } } );
  
  ObjMultiSet( objects );

  -- play static at the beginning
  private.Hero_Static();
  private.Enemy_Static();

  private.Enemy_Abilities_Hide();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Update ( action_data, func_TryContinue )

  private.Trace( "Update" );

  if not private.active then

    return;

  end;

  if ( not action_data.delta  ) or
     ( action_data.delta == 0 ) then

    return;

  end;

  private.Attack_InsertToQueue( action_data, func_TryContinue );
  private.Attack_TryProcessFromQueue();

end;
--------------------------------------------------------------------------------
function public.Update_Ability ()

  private.Hero_SetAbility( nil );
  private.Enemy_SetAbility( nil );

end;
--------------------------------------------------------------------------------
function public.Update_ComboHit ( power )
  
  -- Ignore combo hit on already started attack
  if ( private.attack_pending ) then
    
    return;
    
  end;
  
  -- JK: prepare for a combo hit only if enemy is still alive
  if ( private.enemy_health > 0 ) then
  
    if ( private.combo_hit         ) and
       ( private.combo_hit > power ) then
    
      return;
  
    end;
    
    private.combo_hit = power;
    
    AnimPlay(
      private.player_skin,
      "start_attack_combo"..private.combo_hit,
      private.Hero_Prepare
    );
    
    cmn.PlaySnd( private.hero_audio.."start_attack_combo"..private.combo_hit );
    
  end;

end;
--------------------------------------------------------------------------------
function public.Update_AttackPrepare ()
  
  private.Hero_Prepare();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Attack_IsQueueEmpty ()
  
  return ( #private.attack_queue < 1 );

end;
--------------------------------------------------------------------------------
function private.Attack_InsertToQueue ( action_data, func_TryContinue )
  
  table.insert(
    private.attack_queue,
    {
      action_data      = action_data,
      func_TryContinue = func_TryContinue
    }
  );
  
end;
--------------------------------------------------------------------------------
function private.Attack_TryProcessFromQueue ()
  
  if ( #private.attack_queue < 1 ) or
     ( private.attack_pending    ) then
    
    return;
    
  end;
  
  private.attack_pending = true;
  
  local action_data      = private.attack_queue[ 1 ].action_data;
  local func_TryContinue = private.attack_queue[ 1 ].func_TryContinue;
  
  local func_prefinish = function ()
    
    table.remove( private.attack_queue, 1 );
    
  end;
  
  local func_finish = function ()
    
    private.attack_pending = nil;
    private.Attack_TryProcessFromQueue();
    
  end;
  
  if ( action_data.type == "EnemyAttackActivated" ) then

    local func_OnAnimationEnd = function()

      func_prefinish();
      
      if ( private.hero_health < 1 ) then

        private.Enemy_Win( func_TryContinue );

      else

        func_TryContinue();

      end;
      
      func_finish();

    end;
    
    private.Enemy_Attack(
      action_data,
      private.Enemy_AttackStyle(),
      func_OnAnimationEnd
    );

  end;

  if ( action_data.type == "HeroAttackActivated" ) then

    local func_OnAnimationEnd = function()

      func_prefinish();
      
      if ( private.enemy_health < 1 ) then

        private.Hero_Win( func_TryContinue );

      else

        func_TryContinue();

      end;
      
      func_finish();

    end;
    
    private.Hero_Attack(
      action_data,
      func_OnAnimationEnd
    );    
    
  end;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.ShowDots ( left, right )
  
  ObjMultiSet( {
    { "anm_int_panel_stage_progress_fight_healthbars_hero_ability",  
      { animfunc = "state_" .. left  } },
    { "anm_int_panel_stage_progress_fight_healthbars_enemy_ability",
      { animfunc = "state_" .. right } }
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Hero_Fun_Wheel ()
  
  cmn.PlaySnd( private.hero_audio.."win_bonus" );
  
  private.Hero_OnTap();
  
end;
--------------------------------------------------------------------------------
function private.Hero_Prepare ()
  
  local animfunc = "prepare_attack1";
  
  if ( private.combo_hit ) then
  
    animfunc = "prepare_attack_combo"..private.combo_hit;
    
  end;
  
  AnimPlay(
    private.player_skin,
    animfunc,
    private.Hero_Prepare
  );
  
end;
--------------------------------------------------------------------------------
function private.Hero_Attack ( action_data, func_end )

  local attack_id = private.Hero_AttackStyle( action_data );
  
  local func_OnDefenceEnd = function()

    private.Enemy_Static();

    if ( action_data.ability ) then

      private.Enemy_SetAbility( action_data.ability );
      
      if ( action_data.ability == "stun" or action_data.ability == "superhit" ) then
        
        if ( action_data.ability == "superhit" ) then
          
          private.Hero_SetAbility( nil );
          
        end;
        
      end;

    end;

    func_end();

  end;

  local func_OnAttackEnd = function()

    --DbgTrace( "Enemy defends" );
    
    AnimPlay(
      "anm_int_panel_stage_progress_fight_animations_enemy",
      "defense",
      func_OnDefenceEnd
    );
    
    private.Enemy_SetHealth( action_data );

  end;

  if ( action_data.ability ) then

    private.Hero_SetAbility( action_data.ability );

  end;

  cmn.PlaySnd( private.hero_audio.."attack"..attack_id );

  --DbgTrace( "Hero attacks" );
  
  AnimPlay(
    private.player_skin,
    "attack"..attack_id,
    private.Hero_Static
  );
  
  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_hero_attack_time",
    "attack_time"..attack_id,
    func_OnAttackEnd
  );  

end;
--------------------------------------------------------------------------------
function private.Hero_AttackStyle ( action_data )

  private.Trace( "AttackStyle_Hero: "..action_data.delta );
  
  if ( private.combo_hit ) then
    
    local combo_id    = private.combo_hit;
    private.combo_hit = nil;
    
    return "_combo"..combo_id;
    
  end;

  if ( action_data.ability                                    ) and
     ( private.Hero_AvailableAbilities( action_data.ability ) ) then

    return 4;

  end;

  if ( math.abs( action_data.delta ) > 0.5 * private.enemy_health ) then

    return 3;

  end;

  if ( math.abs( action_data.delta ) > 0.25 * private.enemy_health ) then

    return 2;

  end;

  return 1;

end;
--------------------------------------------------------------------------------
function private.Hero_Static()

  AnimPlay(
    private.player_skin,
    "static",
    private.Hero_Static
  );

end;
--------------------------------------------------------------------------------
function private.Hero_Win ( func_end )

  ObjMultiSet( {
    { "obj_int_panel_stage_progress_fight_touch_enemy",
      {
        input = false } },
  } );
  
  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_hero_attack_time",
    "end_time",
    func_end
  );
  
  AnimPlay(
    private.player_skin,
    "win",
    ""
  );
  
  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_enemy",
    "fail",
    function ()

      mediator.Notify(
        "Interface_PanelStageProgress_WinAnimEnd",
        "fight"
      );

      private.WinAnimEnd = true;

    end
  );

  cmn.PlaySnd( private.enemy_audio..private.enemy_name.."_die" );

end;
--------------------------------------------------------------------------------
function private.Hero_Respawn ( action_data )

  private.Hero_Static();
  private.Hero_SetHealth( action_data );
  private.Enemy_Static();

  cmn.PlaySnd( private.hero_audio.."respawn" );

end;
--------------------------------------------------------------------------------
function private.Hero_SetHealth ( action_data )

  local before = private.hero_health;
  private.hero_health = action_data.health_current;
  
  local total   = action_data.health_total;
  local current = action_data.health_current;
  
  local before_frame = common.Round( before / total * 100 );
  local frame        = common.Round( current / total * 100 );
  
  ObjSet( "anm_int_panel_stage_progress_fight_healthbars_hero_healthbar",
  { 
    count    = tostring( math.floor( current ) ),
    animfunc = "fill"
  } );
  
  local function func_notify ()
  
    local hero_pos = int.Position( "anm_int_panel_stage_progress_fight_healthbars_hero" );
    local notify_data = 
    {
      time      = 1.5,
      offset    = 50,
      direction = "up", 
      posx      = ( hero_pos[ 1 ] + private.healthbar_w / 2 ), 
      posy      = hero_pos[ 2 ] + 100, 
      delta     = action_data.delta 
    };
  
    mediator.Notify( "Interface_PanelStageProgress_HealthChanged", notify_data );
  
  end;
  
  ObjAnimate( "anm_int_panel_stage_progress_fight_healthbars_hero_healthbar",
              "frame", 0, 0, func_notify, 
              {
                0.0, 0, before_frame,
                0.3, 0, frame
              } );

end;
--------------------------------------------------------------------------------
function private.Hero_SetAbility ( ability )

  local anm_char = "anm_int_panel_stage_progress_fight_animations_enemy_hero_ability";
  local anm_abil = "anm_int_panel_stage_progress_fight_healthbars_hero_ability"
  local anm_icon = "anm_int_panel_stage_progress_fight_healthbars_hero_ability_icon";
  local avatar   = "anm_int_panel_stage_progress_fight_healthbars_hero_icon";

  if ( not ability ) then

    if ( not private.hero_ability ) then

      return;

    end;

    private.hero_ability = false;

    ObjMultiSet( {
      { anm_char, { visible = false } },
      { anm_abil, { visible = false } },
      { anm_icon, { visible = false } },
      { avatar,   { visible = true  } }
    } );

    ObjAttach( anm_char, "hub_int_panel_stage_progress_fight_default" );

    if not ( int.Landscape() ) then
      
      local func_end = function ()

        AnimPlay(
          "anm_int_panel_stage_progress_fight_healthbars_hero",
          "hero_healthbar_static",
          ""
        );

      end;
      
      AnimPlay(
        "anm_int_panel_stage_progress_fight_healthbars_hero",
        "hero_ability_hide",
        func_end
      );
      
    end;

    return;

  end;

  if ( private.hero_ability ) then

    return;

  end;

  if ( private.Hero_AvailableAbilities( ability ) ) then

    private.hero_ability = true;

    ObjAttach( anm_char, private.player_skin );

    ObjMultiSet( {
      { anm_char,
        {
          visible  = true,
          animfunc = ability,
          playing  = true,
          anim_tag = ability } },
          
      { anm_abil,
        { 
          visible = true } },
          
      { anm_icon,
        {
          visible  = true,
          animfunc = "icon_"..ability,
          playing  = true } },
          
      { avatar,   { visible = false  } }
    } );
    
    if not ( int.Landscape() ) then
      
      local func_end = function ()

        AnimPlay(
          "anm_int_panel_stage_progress_fight_healthbars_hero",
          "hero_ability_static",
          ""
        );

      end;

      AnimPlay(
        "anm_int_panel_stage_progress_fight_healthbars_hero",
        "hero_ability_show",
        func_end
      );
        
    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Hero_AvailableAbilities ( ability )

  local abilities = {
    damage = true,
    superhit = true
  };

  return abilities[ ability ];

end;
--------------------------------------------------------------------------------
function private.Hero_OnTap ()

  cmn.PlaySnd( "hero_touch" );
  
  ObjSet( "obj_int_panel_stage_progress_fight_touch_hero", { input = false } );

  local func_OnAnimationEnd = function ()

    private.Hero_Static();

    ObjSet( "obj_int_panel_stage_progress_fight_touch_hero", { input = true } );

  end;

  AnimPlay( private.player_skin, "fun1", func_OnAnimationEnd );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Enemy_Attack ( action_data, attack_index, func_end )

  local func_OnAnimationEnd = function()

    --DbgTrace( "Enemy attack ended" );
    
    private.Enemy_Static();

    func_end();

  end;

  local func_OnDefenseEnd = function()

    private.Hero_Static();

    if ( private.hero_health < 1 ) then

      func_end();

    end;

  end;

  local func_OnAttackEnd = function()

    --DbgTrace( "Hero defends" );
    
    AnimPlay(
      private.player_skin,
      "defense",
      func_OnDefenseEnd
    );
    
    private.Hero_SetHealth( action_data );

  end;

  --DbgTrace( "Enemy attacks" );
  
  cmn.PlaySnd(
    private.enemy_audio..private.enemy_name.."_attack"..attack_index
  );

  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_enemy",
    "attack"..attack_index,
    func_OnAnimationEnd
  );
  
  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_enemy_attack_time",
    "attack_time"..attack_index,
    func_OnAttackEnd
  );  

end;
--------------------------------------------------------------------------------
function private.Enemy_AttackStyle ()

  return math.random( 1, 3 );

end;
--------------------------------------------------------------------------------
function private.Enemy_Static ()

  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_enemy",
    "static",
    private.Enemy_Static
  );

end;
--------------------------------------------------------------------------------
function private.Enemy_Win ( func_end )

  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_hero_attack_time",
    "end_time",
    func_end
  );
  
  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_enemy",
    "win",
    ""
  );
  
  AnimPlay(
    private.player_skin,
    "fail",
    ""
  );

end;
--------------------------------------------------------------------------------
function private.Enemy_SetHealth ( action_data )

  local before = private.enemy_health;
  private.enemy_health = action_data.health_current;
  
  local total   = action_data.health_total;
  local current = action_data.health_current;
  
  local before_frame = common.Round( before / total * 100 );
  local frame        = common.Round( current / total * 100 );
  
  ObjSet( "anm_int_panel_stage_progress_fight_healthbars_enemy_healthbar",
  { 
    count    = tostring( math.floor( current ) ),
    animfunc = "fill"
  } );
  
  local function func_notify ()
  
    local enemy_pos   = int.Position( "anm_int_panel_stage_progress_fight_healthbars_enemy" );
    local notify_data = 
    { 
      time      = 1.5,
      offset    = 50,
      direction = "up", 
      posx      = ( enemy_pos[ 1 ] - private.healthbar_w / 2 ),
      posy      = enemy_pos[ 2 ] + 100, 
      delta     = ( action_data.delta - ( action_data.ability_delta or 0 ) )
    };

    local notify_data_ability = nil;  

    if ( action_data.ability_delta ) then

      notify_data_ability = 
      { 
        time        = 1.5,
        offset      = 50,
        direction   = "up",
        posx        = ( enemy_pos[ 1 ] - private.healthbar_w / 2 ), 
        posy        = enemy_pos[ 2 ] + 100, 
        delta       = action_data.ability_delta, 
        fontcolor_r = 0.5, 
        fontcolor_g = 1, 
        fontcolor_b = 0.5 
      };
    
    end;
    
    mediator.Notify( "Interface_PanelStageProgress_HealthChanged", notify_data );

    if ( notify_data_ability ) then

      mediator.Notify( "Interface_PanelStageProgress_HealthChanged",
                        notify_data_ability );

    end;
  
  end;

  ObjAnimate( "anm_int_panel_stage_progress_fight_healthbars_enemy_healthbar",
              "frame", 0, 0, func_notify, 
              {
                0.0, 0, before_frame,
                0.3, 0, frame
              } );
  
end;
--------------------------------------------------------------------------------
function private.Enemy_SetAbility ( ability )

  local anm_char = "anm_int_panel_stage_progress_fight_animations_enemy_ability_"..( ability or private.enemy_ability or "" );
  local anm_abil = "anm_int_panel_stage_progress_fight_healthbars_enemy_ability"
  local anm_icon = "anm_int_panel_stage_progress_fight_healthbars_enemy_ability_icon";
  
  local avatar   = "spr_int_panel_stage_progress_fight_healthbars_enemy_icon";

  if ( not ability ) then

    if ( not private.enemy_ability ) then

      return;

    end;

    private.enemy_ability = false;

    ObjAnimate( anm_char, "alp", 0, 0, function ()

      ObjMultiSet( {
        { anm_char, { visible = false, alp = 1.0 } }
      } );

    end, 
    { 
      0.0, 0, 1.0, 
      0.3, 0, 0.0 
    } );

    ObjMultiSet( {
      { anm_abil, { visible = false } },
      { anm_icon, { visible = false } },
      { avatar,   { visible = true  } }
    } );

    if not ( private.enemy_health < 1 ) then

      private.Enemy_Static();

    end;
    
    if not ( int.Landscape() ) then
      
      local func_end = function ()

        AnimPlay(
          "anm_int_panel_stage_progress_fight_healthbars_enemy",
          "enemy_healthbar_static",
          ""
        );

      end;
      
      AnimPlay(
        "anm_int_panel_stage_progress_fight_healthbars_enemy",
        "enemy_ability_hide",
        func_end
      );
      
    end;

    return;

  end;

  if ( private.Enemy_AvailableAbilities( ability ) ) then

    AnimPlay(
      "anm_int_panel_stage_progress_fight_animations_enemy",
      ability,
      ""
    );

    if ( private.enemy_ability ) then return; end;

    private.enemy_ability = ability;

    ObjMultiSet( {
      { anm_char,
        {
          visible  = true,
          animfunc = ability,
          playing  = true } },
          
      { anm_abil,
        { visible  = true } },
          
      { anm_icon,
        {
          visible  = true,
          animfunc = "icon_"..ability,
          playing  = true } },
          
      { avatar,   { visible = false  } }
    } );
    
    if not ( int.Landscape() ) then
      
      local func_end = function ()

        AnimPlay(
          "anm_int_panel_stage_progress_fight_healthbars_enemy",
          "enemy_ability_static",
          ""
        );

      end;

      AnimPlay(
        "anm_int_panel_stage_progress_fight_healthbars_enemy",
        "enemy_ability_show",
        func_end
      );
      
    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Enemy_AvailableAbilities ( ability )

  local abilities = {
    stun = true,
    dot  = true
  };

  return abilities[ ability ];

end;
--------------------------------------------------------------------------------
function private.Enemy_Abilities_Hide ()

  local abilities = {
    stun = true,
    dot  = true
  };
  
  local objects = {};

  for key, value in pairs( abilities ) do

    table.insert( objects,
    { "anm_int_panel_stage_progress_fight_animations_enemy_ability_"..key,
      {
        visible = false } } );

  end;

  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------
function private.Enemy_OnTap ()

  cmn.PlaySnd( "enemy_click" );
  
  ObjSet( "obj_int_panel_stage_progress_fight_touch_enemy",
  {
    input = false } );

  local func_OnAnimationEnd = function ()

    private.Enemy_Static();

    ObjSet( "obj_int_panel_stage_progress_fight_touch_enemy",
    {
      input = true } );

  end;

  AnimPlay(
    "anm_int_panel_stage_progress_fight_animations_enemy",
    "defense",
    func_OnAnimationEnd
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Move_Near ( time )
  
  int.Input_Off( "obj_int_panel_stage_progress_fight_touch" );
  
  local near, far = private.dst_near, private.dst_far;
  local offset    = private.dst_off; -- internal animation objects offset
  
  ObjAnimate( "anm_int_panel_stage_progress_fight_animations_enemy", 
              "pos_x", 0, 0, "", 
              { 
                0.0,  0, 0.5 * ( far  - offset ), 
                time, 0, 0.5 * ( near - offset )
              } );
  
  ObjAnimate( "anm_int_panel_stage_progress_fight_animations_enemy_hero", 
              "pos_x", 0, 0, "", 
              { 
                0.0,  0, -far  + offset, 
                time, 0, -near + offset
              } );
              
end;
--------------------------------------------------------------------------------
function public.Move_Far ( time )
  
  local near, far = private.dst_near, private.dst_far;
  local offset    = private.dst_off; -- internal animation objects offset
  
  local function func_after_move ()
  
    int.Input_On( "obj_int_panel_stage_progress_fight_touch" );
  
  end;
  
  ObjAnimate( "anm_int_panel_stage_progress_fight_animations_enemy", 
              "pos_x", 0, 0, "", 
              { 
                0.0,               0, 0.5 * ( near - offset ), 
                time, 0, 0.5 * ( far  - offset )
              } );
  
  ObjAnimate( "anm_int_panel_stage_progress_fight_animations_enemy_hero", 
              "pos_x", 0, 0, func_after_move, 
              { 
                0.0,  0, -near + offset,
                time, 0, -far  + offset
              } );
  
end;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------