-- name=int_panel_stage_resources
--------------------------------------------------------------------------------
function public.__InitModule ()

  local app_width = GetAppWidth();

  private.anim_time  = 0.3;
  
  private.show_x = 0;
  private.show_y = 135;
  
  private.hide_x = app_width + 42;
  private.hide_y = 135;
  
  private.landscape = false;

end;
--------------------------------------------------------------------------------
function public.__InitObjects ()
  
  int_control.Add( "int_panel_stage_resources" );
  
end;
--------------------------------------------------------------------------------
function public.SetPosVisible ( x, y )

  private.show_x = x;
  private.show_y = y;

end;
--------------------------------------------------------------------------------
function public.GetPosVisible ()

  return { private.show_x, private.show_y };

end;
--------------------------------------------------------------------------------
function public.SetPosHidden ( x, y )

  private.hide_x = x;
  private.hide_y = y;

end;
--------------------------------------------------------------------------------
function public.SetLandscape ( is_landscape )

  private.landscape = is_landscape;
  
end;
--------------------------------------------------------------------------------
function public.Attach ( resources )
  
  if ( private.active ) then  

    return;

  end;
  
  private.active = true;

  int_control.Attach( "int_panel_stage_resources" );

  private.SetDefault( resources );

  ObjSet( "int_panel_stage_resources", { input = true, active = true, visible = true } );
  
  ObjSet( "obj_int_panel_stage_resources",
  {
    pos_x = private.hide_x,
    pos_y = private.hide_y
  } );

end;
--------------------------------------------------------------------------------
-- NOTE: Just animation, display function is public.Attach
function public.Show ()
  
  if ( not private.active ) then  

    return;

  end;

  local tme = private.anim_time;

  ObjStopAnimate( "obj_int_panel_stage_resources", "pos_xy" );
  
  local params = ObjGet( "obj_int_panel_stage_resources" );

  ObjAnimate( "obj_int_panel_stage_resources", "pos_xy", 0, 0, "", 
  {
    0.0, 2, params.pos_x, params.pos_y,
    tme, 2, private.show_x, private.show_y
  } );

end;
--------------------------------------------------------------------------------
function public.Hide ()

  if ( not private.active ) then  

    return;

  end;

  private.active = false;

  ObjSet( "int_panel_stage_resources", { input = false } );

  local func_end = function ()

    int_control.Detach( "int_panel_stage_resources" );

    ObjSet( "int_panel_stage_resources", { active = false, visible = false } );

  end;

  ObjStopAnimate( "obj_int_panel_stage_resources", "pos_xy" );
  
  local tme = private.anim_time;

  ObjAnimate( "obj_int_panel_stage_resources", "pos_xy", 0, 0, func_end, 
  {
    0.0, 2, private.show_x, private.show_y,
    tme, 2, private.hide_x, private.hide_y
  } );

end;
--------------------------------------------------------------------------------
function public.Update ( resource_data )

  if ( not private.active ) then  

    return;

  end;

  local resource_name  = resource_data.name;
  local resource_count = resource_data.count;

  local index = private.resources[ resource_name ];

  local tme = private.anim_time;

  ObjStopAnimate( "obj_int_panel_stage_resources", "pos_xy" );
  
  local params = ObjGet( "obj_int_panel_stage_resources" );

  ObjAnimate( "obj_int_panel_stage_resources", "pos_xy", 0, 0, "", 
  {
    0.0, 2, params.pos_x, params.pos_y,
    tme, 2, private.show_x, private.show_y,
    5 * tme, 2, private.show_x, private.show_y,
    7 * tme, 2, private.hide_x, private.hide_y
  } );

  ObjSet( "txt_int_panel_stage_resources_resource_"..index.."_count", { text = int.NumberForm( resource_count ) }  );

end;
--------------------------------------------------------------------------------
function private.SetDefault ( resources )
  
  private.resources = {};
  
  local multiset_table = {};
  
  for i = 1, 3 do

    table.insert( multiset_table, 
    { 
      "obj_int_panel_stage_resources_resource_"..i, { visible = false } 
    } );

  end;
  
  ObjMultiSet( multiset_table );
  
  if ( private.landscape ) then
    
    ObjMultiSet( {
      { "grd_int_panel_stage_resources_bg",     { visible = false } },
      { "grd_int_panel_stage_resources_bg_alt", { visible = true  } },
    } );
    
    private.SetHorisontal( resources );
    
  else
    
    ObjMultiSet( {
      { "grd_int_panel_stage_resources_bg",     { visible = true  } },
      { "grd_int_panel_stage_resources_bg_alt", { visible = false } },
    } );
    
    private.SetVertical( resources );
    
  end;

end;
--------------------------------------------------------------------------------
function private.SetVertical ( resources )
  
  local multiset_table = {};
  
  table.insert( multiset_table, 
  { 
    "grd_int_panel_stage_resources_bg", { frame = ( #resources - 1 ) } 
  } );
  
  for r = 1, #resources do

    local resource_name  = resources[ r ].name;
    local resource_count = resources[ r ].count;

    local index = string.match( resource_name, "%d+" );

    private.resources[ resource_name ] = index;

    table.insert( multiset_table,
    { 
      "obj_int_panel_stage_resources_resource_"..index,
        {
          pos_x   = -15,
          pos_y   = 30 + 35 * ( r - 1 ),
          visible = true
        }
    } );
    
    table.insert( multiset_table,
    {
      "spr_int_panel_stage_resources_resource_"..index.."_icon",
        {
          pos_x    = 36,
          pos_y    = 0,
          scale_x  = 0.7,
          scale_y  = 0.7,
          animfunc = "icon_"..resource_name,
          playing  = true
        }
    } );
    
    table.insert( multiset_table,
    { 
      "txt_int_panel_stage_resources_resource_"..index.."_count",
        {
          text = int.NumberForm( resource_count ),
          pos_x = 22,
          pos_y = 5,
          fontsize = 22, 
          align = 2
        }
    } );

  end;

  ObjMultiSet( multiset_table );
  
end;
--------------------------------------------------------------------------------
function private.SetHorisontal ( resources )
  
  local step = 80;
  
  local size = {
    top    = 30,
    left   = step * #resources / 2 + 30,
    bottom = 30,
    right  = step * #resources / 2 + 30,
  }
  
  local border = {
    top    = 30,
    left   = 30,
    bottom = 30,
    right  = 30
  }
  
  int.Grid_Resize( "grd_int_panel_stage_resources_bg_alt", size, border );
  
  local pos_x = {
    { 0 },
    { -0.5 * step, 0.5 * step },
    { -step, 0, step }
  };
  
  local multiset_table = {};
  
  for r = 1, #resources do

    local resource_name  = resources[ r ].name;
    local resource_count = resources[ r ].count;

    local index = string.match( resource_name, "%d+" );

    private.resources[ resource_name ] = index;

    table.insert( multiset_table,
    { 
      "obj_int_panel_stage_resources_resource_"..index,
        {
          pos_x   = pos_x[#resources][r],
          pos_y   = 0,
          visible = true
        }
    } );
    
    table.insert( multiset_table,
    {
      "spr_int_panel_stage_resources_resource_"..index.."_icon",
        {
          pos_x    = -20,
          pos_y    = 0,
          scale_x  = 0.7,
          scale_y  = 0.7,
          animfunc = "icon_"..resource_name,
          playing  = true
        }
    } );
    
    table.insert( multiset_table,
    { 
      "txt_int_panel_stage_resources_resource_"..index.."_count",
        {
          text = int.NumberForm( resource_count ),
          pos_x = 15,
          pos_y = 6,
          fontsize = 22,
          align = 1
        }
    } );

  end;
  
  ObjMultiSet( multiset_table );
  
end;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------