-- name=int_panel_stage_top
--------------------------------------------------------------------------------
function public.__InitModule ()
  
  private.app_h = int.H();
  private.app_w = int.W();
  private.app_o = int.Orientation();
  
  private.this = {
    obj   = "obj_int_panel_stage_top",
    hub   = "hub_int_panel_stage_top_level",
    panel = "anm_int_panel_stage_top",
    level = "anm_int_panel_stage_top_level",
    state = "anm_int_panel_stage_top_level_state",
    moves = "anm_int_panel_stage_top_moves",
    goals = "anm_int_panel_stage_top_goals"
  };
  
  private.strings = {
    moves = ne.strings.Get( "str_int_panel_stage_moves" ),
    goals = ne.strings.Get( "str_int_panel_stage_targets" )
  };

  private.goal_queue         = {};
  private.receiver_animating = {};
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  int_control.Add( "int_panel_stage_top" );

  private.Adapt();
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Show ()

  if ( private.active ) then
    
    return;
    
  end;
  
  private.active  = true;
  private.visible = true;
  
  int_control.Attach( "int_panel_stage_top" );
  
  AnimPlay(
    private.this.panel,
    "appear_"..private.app_o,
    function ()
  
      int.Input_On( private.this.obj );
      
      AnimPlay(
        private.this.panel,
        "static_"..private.app_o,
        ""
      );    
      
    end
  );

end;
--------------------------------------------------------------------------------
function public.Hide ()

  private.Level_Hide();
  
  if ( not private.active ) then
    
    return;
    
  end;
  
  private.active  = false;
  private.visible = false;
  
  int.Input_Off( private.this.obj );
  
  AnimPlay(
    private.this.panel,
    "disappear_"..private.app_o,
    function ()
  
      private.Level_Clean();
      private.Goals_Clean();
      
      int_control.Detach( "int_panel_stage_top" );
      
    end
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.SetVisible ()
  
  if ( private.visible ) then
    
    return;
    
  end;
  
  private.visible = true;
  
  AnimPlay(
    private.this.panel,
    "appear_"..private.app_o,
    function ()
  
      int.Input_On( private.this.obj );

    end
  );

end;
--------------------------------------------------------------------------------
function public.SetInvisible ()
  
  if ( not private.visible ) then
    
    return;
    
  end;
  
  private.visible = false;
  
  int.Input_Off( private.this.obj );
  
  AnimPlay(
    private.this.panel,
    "disappear_"..private.app_o,
    ""
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.IsActive ()
  
  return private.active;
  
end;
--------------------------------------------------------------------------------
function public.IsVisible ()
  
  return private.visible;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Level_Setup (
    stage_number,
    stage_state,
    stage_type,
    is_stateless
  )
  
  local multiset       = {};
  local stage_animfunc = stage_type.."_stage";
  
  m_object.Enable( private.this.level, false, multiset );
  
  if ( is_stateless ) then
    
    if ( stage_type ~= "fight" ) then
    
      stage_animfunc = "stateless";
    
    end;
  
    m_object.Disable( private.this.state, false, multiset );
    
  else
    
    m_object.Enable( private.this.state, false, multiset );
    
    m_object.Set(
      private.this.state,
      {
        animfunc = "state_"..stage_state
      },
      multiset
    );
  
  end;
  
  m_object.Set(
    private.this.level,
    {
      animfunc  = stage_animfunc,
      tt_number = tostring( stage_number )
    },
    multiset
  );
    
  if ( stage_type == "fight"       ) or
     ( private.app_o == "portrait" ) then
     
    m_object.Enable( private.this.hub, false, multiset );
     
    m_object.Attach(
      private.this.level,
      private.this.hub
    );
     
  else
   
    m_object.Disable( private.this.hub, false, multiset );
    
    m_object.Attach(
      private.this.level,
      private.this.panel
    );
     
  end;
  
  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function public.Level_Disable ()
  
  m_object.Disable( private.this.level );
  
end;
--------------------------------------------------------------------------------
function public.Level_Show ()
  
  private.level_standalone = true;
  
  m_object.Set(
    private.this.hub,
    {
      pos_z = 2
    }
  );
  
  m_object.Attach(
    private.this.hub,
    "int_control"
  );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Moves_Update ( moves, is_win )
  
  local animfunc = m_object.Get( private.this.moves ).animfunc;
  
  if ( moves < 5  ) and
     ( not is_win ) then

    if ( string.find( animfunc, "static" ) ) then   
     
      AnimPlay(
        private.this.moves,
        "action_"..private.app_o.."_moves",
        ""
      );
    
    end;
     
  else
    
    AnimPlay(
      private.this.moves,
      "static_"..private.app_o.."_moves",
      ""
    );
    
  end;

  m_object.Set(
    private.this.moves,
    {
      tt_moves_count = tostring( moves )
    }
  );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Goals_GetReceiver ( goal_type )

  local goal_number = private.goals.index[ goal_type ];

  return "anm_int_panel_stage_top_goals_goal"..goal_number;

end;
--------------------------------------------------------------------------------
function public.Goals_Setup ( goals, section )

  local multiset = {};  
  local number   = #goals;
  
  private.goal_queue = {};

  private.goals = {
    number   = number,
    data     = common.DeepCopy( goals ),
    index    = {},
    complete = 0
  };
  
  if ( private.app_o == "portrait" ) then
    
    m_object.Set(
      "anm_int_panel_stage_top_goals",
      {
        animfunc = "static_"..private.app_o.."_goal_"..number
      },
      multiset
    );
    
    m_object.Attach(
      "anm_int_panel_stage_top_goals",
      "anm_int_panel_stage_top"
    );
    
  end;
  
  for g = 1, #goals do
    
    private.goals.index[ goals[ g ].type ] = g;
    
    m_object.Set(
      "anm_int_panel_stage_top_goals_goal"..g,
      {
        animfunc = "static_goal",
        tt_count = private.Goals_Progress( goals[ g ] ),
        rt_icon  = private.Goals_Res( section, goals, g )
      },
      multiset
    );
    
    if ( private.app_o == "portrait" ) then
    
      m_object.Attach(
        "anm_int_panel_stage_top_goals_goal"..g,
        "anm_int_panel_stage_top_goals"
      );
    
    else
      
      m_object.Attach(
        "anm_int_panel_stage_top_goals_goal"..g,
        "anm_int_panel_stage_top"
      );
      
    end;
    
  end;
  
  m_object.Multiset( multiset );

  mediator.SubscribeOnce(
    "View_Stage_StageWon",
    private.Goals_OnStageWon
  );
  
end;
--------------------------------------------------------------------------------
function public.Goals_AddToQueue ( goal_data )

  goal_data = common.DeepCopy( goal_data );

  if ( not private.goal_queue[ goal_data.type ] ) then

    private.goal_queue[ goal_data.type ] = {};

  end;

  local queue_size = #private.goal_queue[ goal_data.type ];

  if ( queue_size > 0 ) then

    local prev_data = private.goal_queue[ goal_data.type ][ queue_size ];

    if ( prev_data.progress < goal_data.progress ) then

      table.insert( private.goal_queue[ goal_data.type ], goal_data );

    end;

  else

    table.insert( private.goal_queue[ goal_data.type ], goal_data );

  end;

end;
--------------------------------------------------------------------------------
function public.Goals_Update ( goal_data, goal_type, func_end )

  if ( not private.goals ) then
    
    -- JK: not initialized
    return;
    
  end;

  local goal_number = private.goals.index[ goal_type ];

  if ( private.goals.data[ goal_number ].done ) then

    return;

  end;

  goal_data = goal_data or table.remove( private.goal_queue[ goal_type ], 1 );

  if ( not goal_data ) then

    return;

  end;

  m_object.Set(
    "anm_int_panel_stage_top_goals_goal"..goal_number,
    {
      tt_count = private.Goals_Progress( goal_data ),
    }
  );
  
  if ( goal_data.progress >= goal_data.count ) then
    
    private.goals.data[ goal_number ].done = true;
    private.goals.complete = private.goals.complete + 1;

    if ( not private.target_animating                  ) and
       ( private.goals.complete < private.goals.number ) then
    
      cmn.PlaySnd( "half_target" );
    
    end;
    
    private.target_animating = true;
    
    AnimPlay(
      "anm_int_panel_stage_top_goals_goal"..goal_number,
      "action_goal",
      function ()
        
        AnimPlay(
          "anm_int_panel_stage_top_goals_goal"..goal_number,
          "complete_goal",
          ""
        );

        private.target_animating = nil;
        
        if ( func_end ) then

          func_end();

        end;
        
      end
    );
        
    private.Goals_Remind();

  else

    if ( func_end ) then

      func_end();

    end;
    
  end;

end;
--------------------------------------------------------------------------------
function public.Goals_Receive ( goal_type, func_end )

  local goal_number = private.goals.index[ goal_type ];
  local receiver    = "anm_int_panel_stage_top_goals_goal"..goal_number;
  
  local function func_OnReceived ()

    mediator.Notify(
      "Interface_PanelStageTop_GoalReceived",
      goal_number
    );

    if ( func_end ) then

      func_end();

    end;

  end;

  -- JK: receiver is not busy
  if ( not private.receiver_animating[ goal_type ] ) then

    private.receiver_animating[ goal_type ] = true;

    AnimPlay(
      receiver,
      "receive_goal",
      function ()
    
        AnimPlay(
          receiver,
          "static_goal",
          ""
        );

        private.receiver_animating[ goal_type ] = nil;

        func_OnReceived();
        
      end
    );

  else

    func_OnReceived();

  end;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Goals_Remind ()
  
  if ( private.goals.number < 2 ) then
    
    -- JK: no need if goal is one
    return;
    
  end;
  
  for g = 1, private.goals.number do
    
    local goal = private.goals.data[ g ];
    
    if ( not goal.done ) then
      
      local number   = private.goals.index[ goal.type ];
      local anm_goal = "anm_int_panel_stage_top_goals_goal"..number;
      local animfunc = m_object.Get( anm_goal ).animfunc;
        
      AnimPlay(
        anm_goal,
        "remind_goal",
        function ()
       
          AnimPlay(
            anm_goal,
            animfunc,
            ""
          );   
          
        end
      );
      
    end;
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Goals_Res ( section, goals, index )
  
  local goal = goals[ index ];
  
  if ( not goal                 ) or
     ( goal.type == "GainScore" ) then
    
    return nil;
    
  end;
  
  local res = {
    default    = "assets/content/items/icon_%s",
    DropItem   = "assets/content/"..section.."/icon_%s_%s",
    DestroyGem = "assets/content/items/icon_%s"
  };
  
  return string.format(
    res[ goal.type ] or res.default,
    string.lower( goal.name ),
    section:match( "%d+" )
  );

end;
--------------------------------------------------------------------------------
function private.Goals_Progress ( goal_data )
  
  if ( goal_data.type == "GainScore" ) then
    
    return int.NumberForm( goal_data.count, true );
    
  end;
  
  return int.NumberForm( goal_data.progress ).."/"..int.NumberForm( goal_data.count );
  
end;
--------------------------------------------------------------------------------
function private.Goals_OnStageWon ()

  for goal_type, queue in pairs( private.goal_queue ) do

    local goal_number = private.goals.index[ goal_type ];

    if ( not private.goals.data[ goal_number ].done ) and
       ( #queue > 0                                 ) then

      public.Goals_Update( nil, goal_type );

      DbgTrace( "[ WARNING ] Goal queue for "..goal_type.." is not empty" );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Goals_Clean ()
  
  m_object.Attach(
    "anm_int_panel_stage_top_goals",
    "hub_panel_stage_top"
  );
  
  for g = 1, private.goals.number do
    
    m_object.Attach(
      "anm_int_panel_stage_top_goals_goal"..g,
      "hub_panel_stage_top"
    );
    
  end;

  private.goals = nil;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Level_Clean ()

  m_object.Attach(
    private.this.level,
    "hub_panel_stage_top"
  );
  
end;
--------------------------------------------------------------------------------
function private.Level_Hide ()
  
  if ( not private.level_standalone ) then
    
    return;
    
  end;
  
  m_object.Set(
    private.this.hub,
    {
      pos_z = 0
    }
  );
  
  m_object.Attach(
    private.this.hub,
    "int_panel_stage_top"
  );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Adapt ()
  
  local multiset = {};
  
  m_object.Set(
    private.this.obj,
    {
      pos_x = private.app_w / 2
    },
    multiset
  );
  
  m_object.Set(
    private.this.panel,
    {
      tt_goals = private.strings.goals,
      tt_moves = private.strings.moves
    },
    multiset
  );  
  
  m_object.Set(
    private.this.hub,
    {
      pos_x = private.app_w - 35,
      pos_y = private.app_h - 40,
    },
    multiset
  );
  
  m_object.Set(
    private.this.goals,
    {
      tt_goals = private.strings.goals
    }
  );
  
  m_object.Set(
    private.this.moves,
    {
      animfunc = "static_"..private.app_o.."_moves",
      tt_moves = private.strings.moves
    },
    multiset
  );
  
  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------