-- name=int_popup_cloud
--[[
Universal animated interface pop-up (own interface hub)

Modes: 
- monolog - just text               - obj's input switching
- hint    - just text               - obj's input stay same
- counter - counter                 - obj's input switching
- complex - text, scroll to counter - obj's input switching
]]
--------------------------------------------------------------------------------
function public.Init ()

  int_control.Add( "int_popup_cloud" );

  private.queue = {};

  ObjSet( "obj_int_popup_cloud_transparent", 
    { 
      event_mup = function() 
      
        public.Hide();
      
      end
    }
  );

  local app_w = int.W();
  local app_h = int.H();

  ObjSet( "obj_int_popup_cloud_transparent", { 
    inputrect_x = -app_w, 
    inputrect_y = -app_h,
    inputrect_w =  app_w * 2, 
    inputrect_h =  app_h * 2
  } );

  private.block_shift_size = 400;

  private.state = "close";

  private.timer = "tmr_int_popup_cloud_autoclose";
  private.close_time =
  {
    [ "monolog" ] = 2.0,
    [ "hint"    ] = 3.0,
    [ "counter" ] = 1.5,
    [ "complex" ] = 3.5
  }

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function public.Show (
    mode,
    full_obj_name,
    text,
    data,
    offset,
    scale,
    functions,
    disable_sound,
    permanent
  )

  private.disable_sound = disable_sound;
  private.permanent     = permanent or false;

  if ( private.state == "open"    ) or
     ( private.state == "opening" ) then 

    --public.Hide();

    -- JK: Add to queue
    local params = {
      mode          = mode,
      full_obj_name = full_obj_name,
      text          = text,
      data          = data,
      offset        = offset,
      scale         = scale,
      functions     = functions,
      disable_sound = disable_sound,
      permanent     = permanent or false
    };

    private.AddToQueue( params );    

  else

    private.state = "opening";

    local anm_hub   = "anm_int_popup_cloud_bg";
    local obj_desc  = "obj_int_popup_cloud_info_description";
    local txt_desc  = "txt_int_popup_cloud_info_description";
    local obj_prog  = "obj_int_popup_cloud_info_progress";
    local txt_prog  = "txt_int_popup_cloud_info_progress";
    local spr_prog  = "spr_int_popup_cloud_info_progress";
    local obj_trans = "obj_int_popup_cloud_transparent";

    functions = functions or {
      func_preopen  = nil,
      func_open     = nil,
      func_close    = nil,
      func_preclose = nil,
      func_scale    = nil
    };

    private.mode = mode;

    private.FuncPreopen  = functions.func_preopen;
    private.FuncOpen     = functions.func_open;
    private.FuncPreclose = functions.func_preclose;
    private.FuncClose    = functions.func_close;
    private.FuncScale    = functions.func_scale;

    if ( private.FuncScale ) then

      functions.func_scale = nil;

      private.FuncScale(
        function()

          private.state = "focused";
          
          public.Show (
            mode,
            full_obj_name,
            text,
            data,
            offset,
            scale,
            functions,
            disable_sound,
            permanent
          );

        end
      );

      return;

    end;

    if ( private.FuncPreopen ) then

      private.FuncPreopen();

    end;
    
    if ( not private.disable_sound ) then
      
      cmn.PlaySnd( "create_cloud" );
      
    end;

    private.StopAnims();

    local func_on_open_end = function () end;
    local multiset = {
      { obj_trans, { input = true } }
    };

    offset = offset or { 0, 0 };
    scale  = scale or 1;
    local reverse_scale = 1 / scale;

    table.insert( multiset, { anm_hub,
    {
      scale_x = scale,
      scale_y = scale } } );
    
    table.insert( multiset, { txt_desc,
    {
      fontsize = 34,
      scale_x  = reverse_scale,
      scale_y  = reverse_scale } } );
    
    table.insert( multiset, { txt_prog,
    {
      scale_x = reverse_scale,
      scale_y = reverse_scale } } );
    
    table.insert( multiset, { spr_prog,
    {
      scale_x = reverse_scale,
      scale_y = reverse_scale } } );

    local obj_coords = int.Position( full_obj_name );

    obj_coords[ 1 ] = obj_coords[ 1 ] + offset[ 1 ];
    obj_coords[ 2 ] = obj_coords[ 2 ] + offset[ 2 ];

    if ( ( "counter" == mode ) or ( "complex" == mode ) ) and ( data ) then

      if not (
        ( data.target.type == "QT_BuildConstructed"      ) or 
        ( data.target.type == "QT_MineConstructed"       ) or 
        ( data.target.type == "QT_StageCompleted"        ) or 
        ( data.target.type == "QT_SpellCompleted"        ) or 
        ( data.target.type == "QT_MineResourceCollected" )
      ) then

        local data_list = data.list or {};

        local progress_current = data.count        or #data_list;
        local progress_target  = data.target.count or #data.target.list or 0;

        table.insert( multiset, { txt_prog,
        {
          param0 = progress_current,
          param1 = progress_target } } );
        
        table.insert( multiset, { spr_prog,
        {
          res = "assets/content/"..data.content.."/"..data.target.name } } );

      end;

      if ( "counter" == mode ) then

        table.insert( multiset, { obj_desc, { pos_x = private.block_shift_size } } );
        table.insert( multiset, { obj_prog, { pos_x = 0 } } );

      else
 
        if ( data.name ) then
          
          table.insert( multiset, { txt_desc, { text = "str_"..data.name } } );
          
        else
          
          table.insert( multiset, { txt_desc, { text = text } } );
          
        end;
        
        table.insert( multiset, { obj_desc, { pos_x = 0 } } );
        table.insert( multiset, { obj_prog, { pos_x = private.block_shift_size } } );

        func_on_open_end = function ()

          ObjAnimate( obj_desc, "pos_x", 0, 0, "", 
          {
            0.0, 3, 0, 
            1.5, 0, 0, 
            1.7, 3, -private.block_shift_size
          } );

          ObjAnimate( obj_prog, "pos_x", 0, 0, "", 
          {
            0.0, 3, private.block_shift_size, 
            1.5, 0, private.block_shift_size, 
            1.7, 3, 0 
          } );

        end;

      end;

    else

      table.insert( multiset, { txt_desc, { text  = text } } );
      table.insert( multiset, { obj_desc, { pos_x = 0 } } );
      table.insert( multiset, { obj_prog, { pos_x = private.block_shift_size } } );

    end;

    table.insert( multiset, { "int_popup_cloud", { 
      pos_x = math.floor( obj_coords[1] ), 
      pos_y = math.floor( obj_coords[2] ), 
      input = false 
    } } );

    ObjMultiSet( multiset );

    ObjAttach( obj_trans, "ng_interface" );

    int_control.Attach( "int_popup_cloud" );

    AnimPlay( "anm_int_popup_cloud_bg", "appear", function ()

      if not ( private.permanent ) then

        private.Autoclose_Start( mode );

      end;
      
      private.state = "open";

      ObjMultiSet( {
        { "int_popup_cloud",        { input = true } },
        { "anm_int_popup_cloud_bg", { animfunc = "static", playing = true } }
      } );

      func_on_open_end();

      if ( private.FuncOpen ) then

        private.FuncOpen();

      end;

    end );

    if ( mode == "complex" ) or
       ( mode == "hint"    ) or
       ( mode == "monolog" ) then

      int.Text_FontsizeByHeight( txt_desc, 130, 34 );

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.Hide ( func_close )

  if ( private.state == "open" ) then

    private.Autoclose_Stop();

    private.state = "closing";

    if ( private.FuncPreclose ) then

      private.FuncPreclose();

    end;

    local mode = private.mode;

    if ( not private.disable_sound ) then
      
      cmn.PlaySnd( "crash_cloud" );
      
    end;

    if ( #private.queue < 1 ) then

      ObjMultiSet( {
        { "obj_int_popup_cloud_transparent", { input = false } },
        { "int_popup_cloud",                 { input = false } }
      } );

    end;

    private.StopAnims();

    AnimPlay( "anm_int_popup_cloud_bg", "close", function ()

      private.state = "close";

      ObjDetach( "obj_int_popup_cloud_transparent" );

      int_control.Detach( "int_popup_cloud" );

      if ( private.FuncClose ) then

        private.FuncClose();

      end;

      private.TryShowFromQueue();

    end );

  end;

end;
--------------------------------------------------------------------------------
function public.AddToQueue (
    mode,
    full_obj_name,
    text,
    data,
    offset,
    scale,
    functions,
    disable_sound
  )

  local params = {
    mode          = mode,
    full_obj_name = full_obj_name,
    text          = text,
    data          = data,
    offset        = offset,
    scale         = scale,
    functions     = functions,
    disable_sound = disable_sound 
  };

  private.AddToQueue( params );

end;
--------------------------------------------------------------------------------
function public.TryShowFromQueue ()

  private.TryShowFromQueue();

end;
--------------------------------------------------------------------------------
function public.IsOnScreen ()

  return ( ( private.state == "open" ) or ( private.state == "opening" ) );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.StopAnims ()

  ObjStopAnimate( "obj_int_popup_cloud_info_description", "pos_x" );
  ObjStopAnimate( "obj_int_popup_cloud_info_progress",    "pos_x" );

end;
--------------------------------------------------------------------------------
function private.AddToQueue ( params, force_first )

  if ( force_first ) then

    table.insert( private.queue, 1, params );

  else

    table.insert( private.queue, params );

  end;

end;
--------------------------------------------------------------------------------
function private.TryShowFromQueue ()

  if ( #private.queue > 0 ) then

    local params = table.remove( private.queue, 1 );

    public.Show(
      params.mode,
      params.full_obj_name,
      params.text,
      params.data,
      params.offset,
      params.scale,
      params.functions,
      params.disable_sound,
      params.permanent
    );

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.Autoclose_Start ( mode )

  local time = private.close_time[ mode ];
  ObjSet( private.timer, 
  {
    playing = true,
    time    = time,
    endtrig = public.Hide
  } );

end;
--------------------------------------------------------------------------------
function private.Autoclose_Stop ()

  ObjSet( private.timer, { playing = false } );

end;
--------------------------------------------------------------------------------