-- name=int_system_message
--********************************************************************************************************************
--********************************************************************************************************************
function public.Init ()

  int_control.Add( "int_system_message" );

  int_control.Attach( "int_system_message" );

  private.count = 0;
  private.queue = {};
  private.process = false;

  -- DK> NEW.
  mediator.Subscribe( "View_Map_MineGathered", private.MineGathered );

  mediator.Subscribe( "Interface_PanelStageProgress_HealthChanged",  private.HealthChanged );

end;
--********************************************************************************************************************
--********************************************************************************************************************
function private.MineGathered ( params )

  if ( params.delta ~= 0 ) then

    local show_params =
    {
      text = params.delta,
      posx = params.posx, 
      posy = params.posy,
      fontcolor_r = params.fontcolor_r, 
      fontcolor_g = params.fontcolor_g, 
      fontcolor_b = params.fontcolor_b
    };

    private.Show( show_params );  

  end;

end;
--********************************************************************************************************************
function private.HealthChanged ( params )

  if ( params.delta ~= 0 ) then

    local show_params =
    {
      time = params.time,
      offset = params.offset,
      direction = params.direction,
      text = params.delta,
      posx = params.posx, 
      posy = params.posy,
      animfunc = "icon_heart",
      fontcolor_r = params.fontcolor_r, 
      fontcolor_g = params.fontcolor_g, 
      fontcolor_b = params.fontcolor_b
    };

    private.Show( show_params );  

  end;

end;
--********************************************************************************************************************
--********************************************************************************************************************
function private.Show ( params )

  local tPos = { iPosX = 512, iPosY = 384 };

  local default_fontcolor = { 1, 0.8, 0 };

  table.insert( private.queue, 
  { 
    time = params.time or 3.0,
    offset = params.offset or 150,
    direction = params.direction or nil,
    text = params.text, 
    posx = params.posx or tPos.iPosX, 
    posy = params.posy or tPos.iPosY,
    fontcolor_r = params.fontcolor_r or default_fontcolor[ 1 ], 
    fontcolor_g = params.fontcolor_g or default_fontcolor[ 2 ], 
    fontcolor_b = params.fontcolor_b or default_fontcolor[ 3 ], 
    res = params.res or nil,
    animfunc = params.animfunc or nil 
  } );

  if not ( private.process ) then

    private.ShowNext();

  end;

end;
--********************************************************************************************************************
--********************************************************************************************************************
function private.ShowNext ()

  if ( #private.queue == 0 ) then

    private.process = false;

    return;

  else

    private.process = true;

  end;

  local params = private.queue[ 1 ];

  table.remove( private.queue, 1 );

  private.count = private.count + 1;

  local obj_name  = "int_system_message_default";

  ObjClone( obj_name, "_"..private.count, 1, "int_system_message" );

  obj_name = "int_system_message_default_"..private.count;

  ObjMultiSet( {
    { obj_name, { pos_x = params.posx, pos_y = params.posy } },
    { "txt_int_system_message_default_"..private.count, 
      { 
        text = params.text, 
        fontcolor_r = params.fontcolor_r, 
        fontcolor_g = params.fontcolor_g, 
        fontcolor_b = params.fontcolor_b
      } }
  } );

  if ( params.res ) then

    ObjSet( "spr_int_system_message_default_"..private.count, { active = true, visible = true, res = "assets/content/"..params.res } );

  elseif ( params.animfunc ) then

    ObjSet( "spr_int_system_message_default_"..private.count, { active = true, visible = true, animfunc = params.animfunc, playing = true } );

  else

    ObjSet( "spr_int_system_message_default_"..private.count, { active = false, visible = false } );
    ObjSet( "txt_int_system_message_default_"..private.count, { pos_x = - 50 } );

  end;

  local func = function ()

    ObjDelete( obj_name );

  end;
  
  local tme = params.time;

  local factors = { up = - 1, down = 1 };

  if ( params.direction ) then

    local direction = factors[ params.direction ];

    ObjAnimate( obj_name, "pos_y", 0, 0, func,
    {
      ( 0.0 * tme ), 3, params.posy,
      ( 1.0 * tme ), 3, params.posy + direction * params.offset
    } );

  else

    local direction = - 1;

    if ( type( params.text ) == "number" ) then
      
      if ( params.text < 0 ) then

        direction = 1;

      else

        ObjSet( "txt_int_system_message_default_"..private.count, { text = "+"..params.text } );

      end;

    end;

    ObjAnimate( obj_name, "pos_y", 0, 0, func,
    {
      ( 0.0 * tme ), 3, params.posy,
      ( 1.0 * tme ), 3, params.posy + direction * params.offset
    } );

  end;

  ObjAnimate( obj_name, "alp", 0, 0, "",
  {
    ( 0.0 * tme ), 3, 0.0,
    ( 0.2 * tme ), 3, 1.0,
    ( 0.5 * tme ), 3, 1.0,
    ( 1.0 * tme ), 3, 0.0
  } );

  ObjAnimate( obj_name, "scale_xy", 0, 0, "",
  {
    ( 0.0 * tme ), 3, 0.6, 0.6,
    ( 0.2 * tme ), 3, 1.0, 1.0,
    ( 0.4 * tme ), 3, 1.0, 1.0,
    ( 1.0 * tme ), 3, 0.8, 0.8
  } );

  ObjSet( "tmr_int_system_message", { time = 0.35, endtrig = private.ShowNext, playing = true } );

end;
--********************************************************************************************************************
--********************************************************************************************************************