-- name=int_window
--------------------------------------------------------------------------------
function public.Init ()

  private.app_w = int.W();
  private.app_h = int.H();
  
  int_control.Add( "int_window" );

  private.window_data  = {};
  private.window_index = {};
  private.fullscreen   = {};

  private.anmtime = 0.30;
  private.bgalpha = 0.60;
  private.grayrgb = 0.40;
  private.outer   = "obj_int_window_outer";

  int.Center_Object( "spr_int_window_black" );
  int.Center_Object( "grd_int_window_vignette" );

  private.InitQueue();

  mediator.Subscribe( "View_Stage_Pause", private.FreezeWindowQueue );

  mediator.Subscribe(
    "Interface_ButtonPressed",
    public.OuterTap_Disable
  );

  mediator.Subscribe(
    "Interface_ButtonAction",
    public.OuterTap_Enable
  );

  mediator.Subscribe(
    "Interface_ButtonReleased",
    public.OuterTap_Enable
  );
  
  mediator.Subscribe(
    "View_Map_Section_Switched",
    private.OnSectionSwitched
  );

  private.Trace( "Init" );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.RegisterAsFullscreen ( window_name )

  private.fullscreen[ window_name ] = { name = window_name };

end;
--------------------------------------------------------------------------------
function public.Show ( window_name, func_end, params )

  local vignette   = false;
  local map_active = false;
  local notify     = true;
  local map_input  = false;

  if ( params ) then

    --vignette   = params.vignette or false;
    map_active = params.map_active or false;
    map_input  = params.map_input  or false;

    notify = tern( params.notify ~= nil )( params.notify, notify );

  end;

  private.Show( window_name, func_end, vignette );

  ObjSet( "zmv_map",
  {
    active = map_active,
    input  = map_input } );

  if ( notify ) then

    mediator.Notify( "Interface_Window_WindowOpened", window_name );

  end;

end;
--------------------------------------------------------------------------------
function public.Hide ( window_name, func_end, skip_map_enable )
  
  mediator.Notify( "Interface_Window_WindowClosed", window_name );

  private.Hide( window_name, func_end );

  if ( #private.window_index < 1 ) then
    
    if ( not skip_map_enable ) then
    
      ObjSet( "zmv_map", { active = true, input = true } );
    
    end;
    
  end;

end;
--------------------------------------------------------------------------------
function public.HideAll ( func_end, skip_map_activation )

  if ( #private.window_index > 0 ) then
    
    local func_hide = _G[ private.window_index[ #private.window_index ] ].Hide;
    
    if ( func_hide ) then
      
      func_hide( function ()
        public.HideAll( func_end );
      end );
      
    else
      
      private.Hide(
        private.window_index[ #private.window_index ],
        function () public.HideAll( func_end ); end
      );
      
    end;

  else

    if ( func_end ) then
    
      func_end();
    
    end;

    if ( not skip_map_activation ) then
    
      ObjSet( "zmv_map", { active = true, input = true } );
    
    end;

  end;

end;
--------------------------------------------------------------------------------
function public.Upper_Name ()
  
  if ( public.IsAnyWindowOnScreen() ) then
  
    return private.window_index[ #private.window_index ];
  
  end;
  
  return false;
  
end;
--------------------------------------------------------------------------------
function public.TryCloseUpperWindow ()

  if ( public.IsAnyWindowOnScreen() ) then
    
    local window_name = private.window_index[ #private.window_index ];
    local func_close  = private.window_data[ window_name ].close_handler;
    
    if ( func_close ) then

      func_close();

      return true, window_name;

    end;
    
    return false, window_name;

  end;

  return false, nil;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.SetArea ( window_name, area, multiset )

  m_object.Set(
    window_name,
    {
      inputrect_init = true,
      inputrect_x    = area[ 1 ],
      inputrect_y    = area[ 2 ],
      inputrect_w    = area[ 3 ],
      inputrect_h    = area[ 4 ]
    },
    multiset
  );

end;
--------------------------------------------------------------------------------
function public.SetAreaByButtons ( obj_window, btn_action, btn_close )
  
  if not ( btn_close ) then
  
    local base_name = cmn.GetObjectName( obj_window );
    btn_close  = "btn_"..base_name.."_close";
  
  end;
  
  -- Get positions
  local pos_action = int.Position( btn_action, obj_window );
  local pos_close  = int.Position( btn_close,  obj_window );
  
  local left   = -pos_close[1];
  local right  =  pos_close[1];
  local top    =  pos_close[2];
  local bottom =  pos_action[2];
  
  -- Shift from buttons centers
  local params_action = m_object.Get( btn_action );
  local params_close  = m_object.Get( btn_close  );
  
  local shift_up   = -params_close.inputrect_h * 0.6;
  local shift_down = params_action.inputrect_h * 0.6;
  
  top    = top    + shift_up;
  bottom = bottom + shift_down;
  
  public.SetArea( obj_window, 
    {
      left,
      top,
      right  - left,
      bottom - top
    }
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.OuterTap_Set ( window_name )

  private.OuterTap_Set( window_name );

end;
--------------------------------------------------------------------------------
function public.OuterTap_Clear ()

  private.OuterTap_Clear();

end;
--------------------------------------------------------------------------------
function public.OuterTap_Disable ()

  if ( not private.outertap_active ) then
    
    return;
    
  end;
  
  int.Input_Off( private.outer );

end;
--------------------------------------------------------------------------------
function public.OuterTap_Enable ()

  if ( not private.outertap_active ) then
    
    return;
    
  end;
  
  int.Input_On( private.outer );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Handler_Close ( window_name )
  
  if ( int.Button_Exists( "btn_"..window_name.."_close" ) ) then

    return int.Button_EventGet( "btn_"..window_name.."_close" );

  end;

  return false;
  
end;
--------------------------------------------------------------------------------
function public.Handler_Accept ( window_name )
  
  if ( int.Button_Exists( "btn_"..window_name.."_accept" ) ) then

    return int.Button_EventGet( "btn_"..window_name.."_accept" );

  end;

  return false;
  
end;
--------------------------------------------------------------------------------
function public.Handler_Close_Reset ( window_name, close_handler )

  if ( private.window_data[ window_name ] ) then

    private.window_data[ window_name ].close_handler = close_handler;

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.IsFullscreen ( window_name )

  return private.IsFullscreen( window_name );

end;
--------------------------------------------------------------------------------
function public.IsOnScreen ( window_name )

  return private.window_data[ window_name ] ~= nil;

end;
--------------------------------------------------------------------------------
function public.IsAnyWindowOnScreen ()
  
  return ( #private.window_index > 0 );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.IsFullscreen ( window_name )

  return private.fullscreen[ window_name ] ~= nil;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Show ( window_name, window_func, vignette )

  local pos_z = 1;

  if ( #private.window_index < 1 ) then
  
    int_control.Attach( "int_window" );

  else

    pos_z = private.window_data[ private.window_index[ #private.window_index ] ].pos_z + 2;

  end;

  if ( not private.window_data[ window_name ] ) then

    table.insert( private.window_index, window_name );

    private.window_data[ window_name ] =
    {
      index = #private.window_index;
      name  = window_name,
      pos_z = pos_z
    };

    ObjSet( window_name, { pos_z = pos_z } );
    ObjAttach( window_name, "int_window" );

  else

    DbgTrace( "[ WARNING ] Dialog window < "..window_name.." > already on the screen!" );
    return;

  end;

  private.window_data[ window_name ].close_handler = public.Handler_Close( window_name );

  if not ( private.IsFullscreen( window_name )  ) then
    
    if ( #private.window_index > 1 ) then

      -- JK: shade previous window
      private.Gray_Out( private.window_index[ #private.window_index - 1 ] );

    end;
    
    private.Background_Show( vignette );

    -- JK: now this is set in int.Dialog_Open via public method
    --private.OuterTap_Set( window_name );

  end;

  if ( window_func ) then

    window_func();

  end;

end;
--------------------------------------------------------------------------------
function private.Hide ( window_name, func_end )

  cmn.PlaySnd( "window_close" );
  
  local prev_window;

  if ( not private.IsFullscreen( window_name ) ) then

    private.OuterTap_Clear();

  end;

  ObjDetach( window_name );

  table.remove( private.window_index, private.window_data[ window_name ].index );

  -- JK: shift down if not last
  -- if ( private.window_data[ window_name ].index ~= #private.window_index ) then
  
    for i, n in pairs( private.window_index ) do

      private.window_data[ n ].index = i;

    end;

  -- end;

  private.window_data[ window_name ] = nil;

  if ( #private.window_index > 0 ) then

    prev_window = private.window_index[ #private.window_index ];

  end;

  local event_func = function ()

    mediator.Notify( "Interface_Window_WindowClosedEnd", window_name );
    private.Gray_Reset( window_name );
    
    ( func_end or function () end )();

    if ( #private.window_index < 1 ) then

      int_control.Detach( "int_window" );

      mediator.Notify( "Interface_Window_NoWindows" );

    end;

  end;

  if not ( private.IsFullscreen( window_name )  ) then

    private.Background_Hide( event_func );

    if ( prev_window ) then

      private.Gray_In( prev_window );

      if not ( private.IsFullscreen( prev_window ) ) then

        private.OuterTap_Set( prev_window );

      end;

    end;

  else

    event_func();

  end;

  private.ShowWindowFromQueue( window_name );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Background_Show ( vignette )

  if ( private.bg_active ) then

    return;

  end;

  private.bg_active  = true;
  private.background = "spr_int_window_black";
  
  if ( vignette ) then

    private.background = "grd_int_window_vignette";

    local aspect_ratio = int.AspectRatio();

    local hole_size = 
    {
      default    = 260,
      [ "3:2"  ] = 260,
      [ "4:3"  ] = 314,
      [ "16:9" ] = 340
    };
    
    local pos_y = hole_size[ aspect_ratio ] or hole_size.default;

    GridSet( private.background, 0, 
    {
       0,  { pos_x = 0 },
       1,  { pos_x = 2 },
       2,  { pos_x = private.app_w - 1 }, 
       3,  { pos_x = private.app_w }, 

       4,  { pos_x = 0,                 pos_y = pos_y },
       5,  { pos_x = 2,                 pos_y = pos_y },
       6,  { pos_x = private.app_w - 1, pos_y = pos_y }, 
       7,  { pos_x = private.app_w,     pos_y = pos_y },

       8,  { pos_x = 0,                 pos_y = private.app_h - 1 },
       9,  { pos_x = 2,                 pos_y = private.app_h - 1 },
       10, { pos_x = private.app_w - 1, pos_y = private.app_h - 1 }, 
       11, { pos_x = private.app_w,     pos_y = private.app_h - 1 },

       12, { pos_x = 0,                 pos_y = private.app_h },
       13, { pos_x = 2,                 pos_y = private.app_h },
       14, { pos_x = private.app_w - 1, pos_y = private.app_h }, 
       15, { pos_x = private.app_w,     pos_y = private.app_h }
    } );

  end;

  m_object.Enable( private.background );

  local current = m_object.Get( private.background ).alp;

  ObjStopAnimate( private.background, "alp" ); 

  ObjAnimate( private.background, "alp", 0, 0, "",
  {
    ( 0 * private.anmtime ), 0, current,
    ( 1 * private.anmtime ), 0, private.bgalpha
  } );

end;
--------------------------------------------------------------------------------
function private.Background_Hide ( event_func )

  if not ( private.bg_active ) then

    event_func();
    
    return;

  end;

  if ( #private.window_index > 0 ) then

    event_func();

    return;

  end;

  private.bg_active = false;

  ObjStopAnimate( private.background, "alp" );

  local current = m_object.Get( private.background ).alp;

  local OnAnimationEnd = function()

    m_object.Disable( private.background );
    event_func();

  end;

  if ( current > 0 ) then

    ObjAnimate( private.background, "alp", 0, 0, OnAnimationEnd,
    {
      ( 0 * private.anmtime ), 0, current,
      ( 1 * private.anmtime ), 0, 0.0
    } );
  
  else
    
    OnAnimationEnd();
  
  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Gray_Out ( window_name )

  -- JK: grayed out window has disabled input
  private.OuterTap_Clear();
  int.Input_Off( window_name );

  local current = m_object.Get( window_name );

  ObjAnimate( window_name, "color_rgb", 0, 0, "",
  {
    ( 0 * private.anmtime ), 3, current.color_r, current.color_g, current.color_b,
    ( 1 * private.anmtime ), 3, private.grayrgb, private.grayrgb, private.grayrgb
  } );

end;
--------------------------------------------------------------------------------
function private.Gray_In ( window_name )

  local current = m_object.Get( window_name );

  ObjAnimate(
    window_name,
    "color_rgb", 0, 0,
    function () 
  
      int.Input_On( window_name );

    end,
    {
      ( 0 * private.anmtime ), 3, current.color_r, current.color_g, current.color_b,
      ( 1 * private.anmtime ), 3, 1,               1,               1
    }
  );

end;
--------------------------------------------------------------------------------
function private.Gray_Reset ( window_name )

  int.Input_On( window_name );

  ObjSet( window_name,
  {
    color_r = 1.0,
    color_g = 1.0,
    color_b = 1.0
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.OuterTap_Set ( window_name )

  private.OuterTap_Clear();

  if ( private.window_data[ window_name ]               ) and
     ( private.window_data[ window_name ].close_handler ) then

    int.Control_EventsSet(
      private.outer,
      {
        func_on_click = function ()
          
          private.window_data[ window_name ].close_handler();
          private.OuterTap_Clear();

        end;
      }
    );

    local pos_z = private.window_data[ window_name ].pos_z - 1;

    m_object.Set(
      private.outer,
      {
        active  = true,
        visible = true,
        input   = true,
        pos_z   = pos_z
      }
    );
    
    private.outertap_active = true;

  end;

end;
--------------------------------------------------------------------------------
function private.OuterTap_Clear ()

  int.Control_EventsClear( private.outer );

  m_object.Set(
    private.outer,
    {
      active  = false,
      visible = false,
      input   = false,
      pos_z   = 0
    }
  );
  
  private.outertap_active = nil;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.InitQueue ()
  
  private.window_queue = {};
  private.window_queue.list = {};
  private.window_queue.last_high = 0;
  private.window_queue.last_medium = 0;
  private.window_queue.show = nil;
  private.window_backup = {};
  
  function public.AddWindowToQueue ( window_name, func_start, params )

    local position = #private.window_queue.list + 1;

    private.Trace( "public.AddWindowToQueue: "..window_name..", pos = "..position );

    table.insert(
      private.window_queue.list,
      position,
      {
        name   = window_name,
        func   = func_start,
        params = params
      }
    );

     -- DK> Трейс окон. Не удалять.
    --[[for k, v in pairs( private.window_queue.list ) do

      private.Trace( k.." = "..v.name );

    end;]]

    if ( #private.window_index < 1       ) and
       ( #private.window_queue.list == 1 )
    then

      private.ShowWindowFromQueue();

    end;

  end;
  --------------------------------------------------------------------------------------------------------------------
  function public.IsWindowQueueReleased ()

    if ( #private.window_queue.list > 0 ) or ( private.window_queue.show ) then

      return false;

    end;

    return true;

  end;
  --------------------------------------------------------------------------------------------------------------------
  function public.WindowBackup ( window_name )

    for w = 1, #private.window_queue.list do
   
      if ( private.window_queue.list[ w ].name == window_name ) then

        private.window_backup = table.remove( private.window_queue.list, w );

        break;

      end;

    end;

  end;
  --------------------------------------------------------------------------------------------------------------------
  function public.WindowRestore ()

    if ( private.window_backup.name ) then

      public.AddWindowToQueue( private.window_backup.name, private.window_backup.func )

      for w = 1, #private.window_queue.list do
     
        if ( private.window_queue.list[ w ].name == window_name ) then

          private.window_backup = table.remove( private.window_queue.list, w );

          break;

        end;

      end;

      private.window_backup = nil;

    end;

  end;
  --------------------------------------------------------------------------------------------------------------------
  function private.ShowWindowFromQueue ( last_closed_window )

    if ( private.window_queue.show ) then

      private.Trace( "ShowWindowFromQueue: "..tostring( private.window_queue.show.name ) );

    end;

    local prefix = cmn.GetObjectPrefix( screen.GetCurrentName() );

    if ( "int" == prefix ) or ( private.window_queue.freezed ) then

      --DK> Очищаем текущий, если был выполнен переход.
      if ( private.window_queue.show                            ) and
         ( last_closed_window == private.window_queue.show.name )
      then

        private.window_queue.show = nil;

      end;

      return;

    end;

    if ( private.window_queue.show                                    ) and
       ( not ( last_closed_window == private.window_queue.show.name ) )
    then

      return;

    end;

    private.Trace( "private.ShowWindowFromQueue: "..#private.window_queue.list );

    if ( #private.window_index < 1 ) and ( #private.window_queue.list > 0 ) then

      private.window_queue.show = table.remove( private.window_queue.list, 1 );

      local window_obj = private.window_queue.show.name;

      public.Show( window_obj, private.window_queue.show.func, private.window_queue.show.params );

    end;

    private.window_queue.show = nil;

    mediator.Notify( "Interface_Window_QueueReleased" );

  end;
  --------------------------------------------------------------------------------------------------------------------
  function private.FreezeWindowQueue ( param )

    private.window_queue.freezed = param;

    if ( not private.window_queue.freezed ) then
    
      private.ShowWindowFromQueue();
    
    end;

  end;
  -------------------------------------------------------------------------------------------------------------
  function private.GetCurrentWindowName ()

    if ( private.window_queue.show ) then

      return private.window_queue.show.name;

    end;

  end;
  --------------------------------------------------------------------------------------------------------------------
end;
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ WINDOW ] "..text );

end;
--------------------------------------------------------------------------------
function private.OnSectionSwitched ()
  
  public.HideAll( nil, true );
  
end;
--------------------------------------------------------------------------------