-- name=interface
int = {};
--------------------------------------------------------------------------------
function public.Init ()

  local int_hub_name = "ng_interface_hub";

  private.waiting_name = "anm_waiting";
  private.int_pos_z    = 1000;
  
  private.button = {
    any = {
      pressed = false
    }
  };
  
  private.swipe = {};
  
  private.popup_cloud_enabled = true;
  
  private.w = GetAppWidth();
  private.h = GetAppHeight();

  private.aspect_ratio = private.AspectRatio_Detect();
  
  DbgTrace(
    "[ SCREEN ] resolution: "..private.w.."x"..private.h..
    ", aspect ratio: "..private.aspect_ratio
  );

  m_object.Create( int_hub_name, "obj" );
  m_object.Set( int_hub_name, { pos_z = private.interface_pos_z } );
  m_object.Attach( int_hub_name, InterfaceName );

  loader.Modules_Load_Init({
    "int_popup_cloud",
    "int_popup_flat",
    "int_subtitles"
  });

  private.language = wiz.settings.GetLanguage();

  private.strings = {
    number_separator = ne.strings.Get( "str_number_separator" )
  };

  private.WaitingInit();
  private.LockInit();

  private.control_event_data = {}; -- for int.Control_EventsSet, etc

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function int.Lock( state )

  -- common.DbgCallpoint( tostring( state ) );
  
  ObjMultiSet( {
    { "ng_interface_lock",     { input   = state } },
    { "ng_interface_lock_spr", { visible = state } }
  } );

  private.screen_locked = state;

end;
--------------------------------------------------------------------------------
function int.IsLocked ()

  return private.screen_locked;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function int.WaitingShow ()

  --ShowLoadingWheel( true );
  int.Lock( true );

  ObjAttach( private.waiting_name, InterfaceName );

  ObjStopAnimate( private.waiting_name, "alp" );
  ObjAnimate(  private.waiting_name, "alp", 0, 0, "",
  {
    0.0, 0, 0.0,
    0.5, 0, 0.0,
    0.8, 0, 1.0
  } );

end;
--------------------------------------------------------------------------------
function int.WaitingHide ()
  
  --ShowLoadingWheel( false );
  int.Lock( false );

  local alp = ObjGet( private.waiting_name ).alp;

  ObjStopAnimate( private.waiting_name, "alp" );
  ObjAnimate(  private.waiting_name, "alp", 0, 0,
  function ()

    ObjDetach( private.waiting_name );

  end,
  {
    0.0, 0, alp,
    0.3, 0, 0.0
  } );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function int.ShowPopupCloud ( ... )
  
  int.Popup_Cloud_Show( ... );

end;
--------------------------------------------------------------------------------
function int.HidePopupCloud ()
  
  int.Popup_Cloud_Hide();

end;
--------------------------------------------------------------------------------
function int.Popup_Cloud_Show ( ... )
  --( mode, full_obj_name, text, data, offset, scale,
  -- functions, disable_sound, permanent )
    
  int_popup_cloud.Show( ... );

end;
--------------------------------------------------------------------------------
function int.Popup_Cloud_Hide ()
  
  int_popup_cloud.Hide(); 

end;
--------------------------------------------------------------------------------
function int.Popup_Cloud_AddToQueue ( ... )

  int_popup_cloud.AddToQueue( ... );

end;
--------------------------------------------------------------------------------
function int.Popup_Cloud_TryShowFromQueue ()

  int_popup_cloud.TryShowFromQueue();

end;
--------------------------------------------------------------------------------
function int.Popup_Cloud_IsOnScreen ()

  return int_popup_cloud.IsOnScreen();

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function int.ShowPopupFlat ( ... ) --caption, text, pos, parent, inverse

  int_popup_flat.Show( ... ); 

end;
--------------------------------------------------------------------------------
function int.HidePopupFlat ( instant, func_end )
  
  int_popup_flat.Hide( instant, func_end ); 

end;
--------------------------------------------------------------------------------
function int.Popup_Flat_AttachedTo ()
  
  return int_popup_flat.AttachedTo();
  
end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function int.Subtitles_Show ( subtitle_name, section_name, func_skip, func_after )

  local subtitle_table = 
  {
    section_1 = { intro = { 0.6, 3.2, 7.8, 12.4, 15.0 }, outro = { 0.6 } },
    section_2 = { intro = { 0.6, 4.2, 7.8, 10.4, 12.6, 16.2 }, outro = { 0.6 } },
    section_3 = { intro = { 0.6, 4.2, 7.8, 12.4 }, outro = { 0.1 } },
    section_4 = { intro = { 0.6, 2.2, 4.5, 6.7, 11.0 }, outro = { 0.6 } },
    section_5 = { intro = { 0.6, 2.2, 4.5, 6.7, 11.0 }, outro = { 0.6 } }
  };

  if ( subtitle_table[ section_name ] ) and ( subtitle_table[ section_name ][ subtitle_name ] ) then

    int_subtitles.Show( 
      section_name.."_"..subtitle_name, 
      subtitle_table[ section_name ][ subtitle_name ], 
      func_skip,
      func_after
    );

  end;

end;
--******************************************************************************
-- function ... () end
--******************************************************************************
function int.ImageAdjust ( spr_name, size, spr_params )

  spr_params = spr_params or ObjGet( spr_name );

  if not ( spr_params and spr_params.draw_width and spr_params.draw_width > 0 ) then 
    return;
  end;

  local smaller_side = spr_params.draw_width;
  if ( spr_params.draw_height < spr_params.draw_width ) then
    smaller_side = spr_params.draw_height;
  end;

  ObjSet( spr_name, {
    scale_x = size / spr_params.draw_width,
    scale_y = size / spr_params.draw_height
  } );

end;
--******************************************************************************
-- function ... () end
--******************************************************************************
function int.NumberForm ( number, unconverted )

  local postfix   = "";
  local separator = private.strings.number_separator;
  
  -- JK: russians don't separate thousands less than 10
  if ( private.language == "ru" ) and
     ( number < 10000           ) then

    separator = "";

  end;

  if ( not unconverted ) then
    
    if ( number > 999999 ) then

      number = math.floor( number / 1000000 );

      postfix = "M";

    elseif ( number > 99999 ) then

      number = math.floor( number / 1000 );

      postfix = "K";

    end;
    
  end;

  local text_number  = tostring( math.floor( number ) );
  local first_symbol = string.sub( text_number, 1, 1 );
  local sign         = "";

  if ( first_symbol == "-" ) then

    sign        = "-";
    text_number = string.sub( text_number, 2 );

  end;

  local count_add_symbol  = math.floor( #text_number / 3 );
  local count_digit_begin = math.fmod( #text_number, 3 );
  local new_text          = "";

  if ( count_digit_begin > 0 ) then

    new_text    = string.sub( text_number, 1, count_digit_begin );
    text_number = string.sub( text_number, count_digit_begin + 1 );

  end;

  for i = 1, count_add_symbol do
    
    local part_text = string.sub( text_number, 1, 3 );
    text_number     = string.sub( text_number, 4 );
    new_text        = new_text..separator..part_text;

  end;

  local first = string.sub( new_text, 1, 1 );

  if ( first == separator ) then

    new_text = string.sub( new_text, 2 );

  end;

  return sign..new_text..postfix;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Button_Create ( placeholder, params, func_Exec )
  
  --[[
    USAGE EXAMPLE:
    
    int.Button_Create(
      "btn_int_dialog_fireflies_booster_buy",
      {
        title  = "str_dialog_fireflies_booster_buy",
        color  = "yellow",
        design = "small",
        price  = true
      },
      function()
        private.Tap_Buy();
      end
    );
  ]]

  params = params or {};

  -- JK: button may clear window outer tap or something
  if ( params.clear ) then

    if ( params.clear == "window_outer_tap" ) then

      local func_ExecSrc = func_Exec;

      func_Exec = function ()

        int_window.OuterTap_Clear();
        func_ExecSrc();

      end;

    end;

  end;

  --JK: button may exist, so just reset funcExec
  if ( private.button[ placeholder ] ) then
    
    int.Button_EventSet( placeholder, func_Exec );
    
    return;
    
  end;
  
  local multiset = {};
  local input    = {};
  local animfunc = "";  
  
  params.res  = params.res or "assets/content/interface/buttons/anm_buttons";
  params.name = placeholder:gsub( "btn", "anm" );

  if ( params.color ) then
  
    animfunc = params.color.."_"..params.design;
  
  else
    
    animfunc = params.design;
    
  end;
  
  if ( params.title ) then
   
    if ( type( params.title ) == "string" ) then
    
      params.title = StringGet( params.title );
      
    elseif ( type( params.title ) == "number" ) then
      
      params.title = tostring( params.title );
      
    end;
 
  else
    
    params.title = "Button";
    
  end;
  
  int.Button_EventSet( placeholder, func_Exec );
  
  m_object.Clone(
    "anm_btn",
    placeholder,
    "_"..placeholder
  );
  
  if ( params.price ) then
    
    m_object.Clone(
      "anm_btn_price",
      placeholder,
      "_"..placeholder
    );
    
    if ( type( params.price ) == "boolean" ) then

      params.price = "price_gold_top";

    end;

    m_object.Set(
      "anm_btn_price_"..placeholder,
      {
        name     = placeholder:gsub( "btn", "anm_price" ),
        animfunc = params.price
      },
      multiset
    );

    params.price = nil;
    
  end;

  if ( params.timer ) then
    
    local pos_y = 
    {
      default =    0,
      up      = - 55,
      down    =   35
    };

    m_object.Clone(
      "anm_btn_timer",
      placeholder,
      "_"..placeholder
    );
    
    local timer_pos_y = pos_y[ params.timer ] or pos_y.default;
    timer_pos_y = tern( params.design == "buy" )( pos_y.up, timer_pos_y );
    
    m_object.Set(
      "anm_btn_timer_"..placeholder,
      {
        name     = placeholder:gsub( "btn", "anm_timer" ),
        animfunc = tern( params.timer == "" )( "timer", "timer_"..params.timer ),
        pos_y    = timer_pos_y
      },
      multiset
    );

    params.timer = nil;
    
  end;
  
  if ( params.info ) then

    m_object.Clone(
     "anm_btn_info",
      placeholder,
      "_"..placeholder, true
    );

    m_object.Set(
      "anm_btn_info_"..placeholder,
      {
        name     = placeholder:gsub( "btn", "anm_info" ),
        animfunc = "save_info"
      },
      multiset
    );
    
    m_object.Set(
      "anm_btn_info_stage_"..placeholder,
      {
        name     = placeholder:gsub( "btn", "anm_info_stage" ),
        animfunc = "save_info"
      },
      multiset
    );    

    params.info = nil;
    
  end;  
  
  if ( params.icon ) then
    
    m_object.Clone(
      "spr_btn_icon",
      "anm_btn_"..placeholder,
      "_"..placeholder
    );
    
    m_object.Set(
      "spr_btn_icon_"..placeholder,
      {
        name     = placeholder:gsub( "btn", "spr_icon" ),
        animfunc = params.icon 
      },
      multiset
    );

    params.icon = nil;
    
  end;  
  
  -- JK: Button Down
  local func_BtnDown = function ()
  
    if ( not private.button[ placeholder ] ) then return; end;
  
    if ( private.button.any.pressed             ) or 
       ( private.button[ placeholder ].pressed  ) or 
       ( private.button[ placeholder ].animated ) then
      
      return;
      
    end;
  
    private.button.any.pressed            = true;
    private.button[ placeholder ].pressed = true;
    
    if ( private.button[ placeholder ].idle ) then
      
      AnimStop( params.name );
      
      m_object.Set(
        params.name,
        {
          animfunc = animfunc,
          playing  = false, 
          frame    = 0
        }
      );
    
    end;
  
    ObjAnimate( params.name, "frame", 0, 0, "",
    {
      0.00, 0, 0,
      0.25, 0, 15
    } );

    mediator.Notify(
      "Interface_ButtonPressed",
      placeholder
    );
  
  end;
  
  -- JK: Button Up
  local func_BtnUp = function ()
  
    if ( not private.button[ placeholder ]          ) or
       (     private.button[ placeholder ].animated ) or
       ( not private.button[ placeholder ].pressed  ) then
      
      return;
      
    end;
    
    private.button[ placeholder ].animated = true;
  
    if  ( private.button[ placeholder ].sound ) then
      
      if ( private.button[ placeholder ].sound ~= "no" ) then
      
        cmn.PlaySnd( private.button[ placeholder ].sound );
      
      end;
      
    else
      
      cmn.PlaySnd( "click_wood" );
      
    end;
  
    ObjAnimate( params.name, "frame", 0, 0, function ()
    
      if ( private.button[ placeholder ].idle ) then
        
        m_object.Set(
          params.name,
          {
            animfunc = animfunc.."_idle",
            playing  = true,
            frame    = 0
          }
        );
      
      end;
      
      private.button[ placeholder ].animated = nil;
      private.button[ placeholder ].pressed  = nil;
      private.button.any.pressed             = false;
      
      private.button[ placeholder ].func_Exec();

      mediator.Notify(
        "Interface_ButtonAction",
        placeholder
      );
  
    end,
    {
      0.00, 0, 15,
      0.05, 0, 30
    } );
  
  end;
  
  -- JK: Button On Leave
  local func_BtnLeave = function ()
  
    if ( IsPC() ) then
      
      SetCursor( CURSOR_DEFAULT );
      
    end;
  
    if ( not private.button[ placeholder ] ) then return; end;
  
    if (     private.button[ placeholder ].pressed  ) and
       ( not private.button[ placeholder ].animated ) then
      
      ObjAnimate( params.name, "frame", 0, 0, function()
      
        if ( private.button[ placeholder ].idle ) then
          
          m_object.Set(
            params.name,
            {
              animfunc = animfunc.."_idle",
              playing  = true,
              frame    = 0
            }
          );
        
        end;
        
        private.button[ placeholder ].pressed = nil;
        private.button.any.pressed            = false;
      
      end,
      {
        0.00, 0, 15,
        0.25, 0, 30
      } );

      mediator.Notify(
        "Interface_ButtonReleased",
        placeholder
      );
      
    end;
  
  end;
  
  local func_BtnEnter = function ()
  
    if ( IsPC() ) then
      
      SetCursor( CURSOR_HAND );
      
    end;
      
  end;

  input.event_mdown  = function () func_BtnDown();  end;
  input.event_mup    = function () func_BtnUp();    end;
  input.event_mleave = function () func_BtnLeave(); end;
  input.event_menter = function () func_BtnEnter(); end;

  -- JK: auto-inputrect settings
  local input_rects = {
    default           = { 60,  60  },
    close             = { 70,  70  },
    small             = { 120, 84  },
    medium            = { 220, 84  },
    large             = { 320, 84  },
    facebook          = { 220, 84  },
    facebook_large    = { 320, 84  },
    price             = { 220, 84  },
    heart             = { 320, 84  },
    delete            = { 58,  58  },
    heart_send        = { 98,  84  },
    heart_receive     = { 98,  84  },
    wheel             = { 320, 84  },
    buy_resource      = { 150, 200 },
    upgrade_resource1 = { 200, 70  },
    upgrade_resource2 = { 200, 70  },
    upgrade_resource3 = { 200, 70  },
    add               = { 120, 120 },
    buy               = { 220, 120 }
  };
 
  local input_rect = 
    params.input_rects           or 
    input_rects[ params.design ] or 
    input_rects.default;

  local input_shift   = params.input_shift or {};
  local input_shift_x = input_shift[ 1 ]   or 0;
  local input_shift_y = input_shift[ 2 ]   or 0;
  
  input.inputrect_init =   true;
  input.inputrect_w    =   input_rect[ 1 ];
  input.inputrect_h    =   input_rect[ 2 ];
  input.inputrect_x    = - input_rect[ 1 ] / 2 + input_shift_x;
  input.inputrect_y    = - input_rect[ 2 ] / 2 + input_shift_y;
  
  --JK: position
  if ( params.pos_x ) then
    
    input.pos_x  = params.pos_x;
    params.pos_x = nil;
    
  end;
  
  if ( params.pos_y ) then
    
    input.pos_y  = params.pos_y;
    params.pos_y = nil;
    
  end;
  
  if ( params.pos_z ) then
    
    input.pos_z  = params.pos_z;
    params.pos_z = nil;
    
  end;
  
  -- JK: button may have idle animation
  params.animfunc = animfunc;
  params.playing  = false;
  private.button[ placeholder ].idle = nil;
  
  if ( params.idle ) then
    
    params.animfunc = animfunc.."_idle";
    params.playing  = true;
    private.button[ placeholder ].idle = true;
    
  end;
  
  if ( params.sound ) then
    
    private.button[ placeholder ].sound = params.sound;
    
  end;
  
  -- JK: convert title to text tag
  params.tt_title = params.title;

  -- JK: remove junk properties
  params.title       = nil;
  params.idle        = nil;
  params.color       = nil;
  params.design      = nil;
  params.sound       = nil;
  params.input_rects = nil;
  params.input_shift = nil;
  
  m_object.Set(
    "anm_btn_"..placeholder,
    params,
    multiset
  );
  
  m_object.Set(
    placeholder,
    input,
    multiset
  );
  
  m_object.Multiset( multiset );
  
end;
--------------------------------------------------------------------------------
function int.Button_SetPrice ( placeholder, price, hide_zero, multiset )
  
  if ( type( price ) == "number" ) then
    
    price = int.NumberForm( price );
    
  end;
  
  local anm_name = placeholder:gsub( "btn", "anm_price" );
  
  if ( price == "0" ) and ( hide_zero ) then
    
    m_object.Disable( anm_name, true, multiset );
    
  else
  
    m_object.Set(
      anm_name,
      {
        active   = true,
        visible  = true,
        tt_count = price
      },
      multiset
    );
    
  end;
  
end;
--------------------------------------------------------------------------------
function int.Button_SetTime ( placeholder, time, multiset )

  local anm_name = placeholder:gsub( "btn", "anm_timer" );
  local show = tern( time )( true, false );
  
  m_object.Set(
    anm_name,
    {
      active  = show,
      visible = show,
      time    = time or "",
      tt_time = time or ""
    },
    multiset
  );

end;
--------------------------------------------------------------------------------
function int.Button_SetInfo ( placeholder, info_table, multiset )

  local anm_name = placeholder:gsub( "btn", "anm_info" );
  local show = tern( info_table )( true, false );
  
  info_table = info_table or {};
  
  m_object.Set(
    anm_name,
    {
      active  = show,
      visible = show,
      tt_gold = info_table.gold or "0",
      tt_section = "str_systemmessage_"..( info_table.section or "section_1" )
    },
    multiset
  );
  
  local anm_stage_name = placeholder:gsub( "btn", "anm_info_stage" );
  
  m_object.Set(
    anm_stage_name,
    {
      active  = show,
      visible = show,
      animfunc = tern( info_table.stage == "bonus" )( "stage_bonus", "stage" ),
      tt_stage = info_table.stage_last
    },
    multiset
  );  

end;
--------------------------------------------------------------------------------
function int.Button_SetText ( placeholder, text, multiset )

  local title    = tostring( text );
  local anm_name = placeholder:gsub( "btn", "anm" );

  if ( string.find( text, "str_" ) ) then

    title = ne.strings.Get( text );

  end;

  m_object.Set(
    anm_name,
    {
      tt_title = title
    },
    multiset
  );
  
end;
--------------------------------------------------------------------------------
function int.Button_SetTextTag ( placeholder, tag_name, tag_text, multiset )

  local text     = tostring( tag_text );
  local anm_name = placeholder:gsub( "btn", "anm" );

  if ( string.find( tag_text, "str_" ) ) then

    text = ne.strings.Get( tag_text );

  end;
  
  m_object.Set(
    anm_name,
    {
      [ tag_name ] = text
    },
    multiset
  );
  
end;
--------------------------------------------------------------------------------
function int.Button_Enable ( placeholder, multiset )

  m_object.Enable( placeholder, false, multiset );
  
end;
--------------------------------------------------------------------------------
function int.Button_Disable ( placeholder, multiset )
  
  m_object.Disable( placeholder, false, multiset );
  
end;
--------------------------------------------------------------------------------
function int.Button_SetInput ( placeholder, input )
  
  ObjSet( placeholder, { input = input } );
  
end;
--------------------------------------------------------------------------------
function int.Button_Delete ( placeholder )
  
  local anm_name = placeholder:gsub( "btn", "anm" );
  
  common.TapClear( placeholder );
  
  ObjDelete( anm_name );
  
  private.button[ placeholder ] = nil;
  
end;
--------------------------------------------------------------------------------
function int.Button_EventSet ( placeholder, func_Exec )

  if not ( private.button[ placeholder ] ) then
    
    private.button[ placeholder ] = {};
    
  end;
  
  private.button[ placeholder ].func_Exec = func_Exec;
  
end;
--------------------------------------------------------------------------------
function int.Button_EventGet ( placeholder )
  
  return private.button[ placeholder ].func_Exec;
  
end;
--------------------------------------------------------------------------------
function int.Button_Exists ( placeholder )
  
  return ( private.button[ placeholder ] ~= nil );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Buttons_Enable ( placeholders )
  
  local set = {};
  
  for p, placeholder in pairs( placeholders ) do
    
    if ( multiset ) then

      m_object.Enable( placeholder, false, multiset );

    else

      m_object.Enable( placeholder, false, set );

    end;
      
  end;

  if ( not multiset ) then
    
    m_object.Multiset( set );

  end;
  
end;
--------------------------------------------------------------------------------
function int.Buttons_Disable ( placeholders, multiset )
  
  local set = {};
  
  for p, placeholder in pairs( placeholders ) do
    
    if ( multiset ) then

      m_object.Disable( placeholder, false, multiset );

    else

      m_object.Disable( placeholder, false, set );

    end;
      
  end;

  if ( not multiset ) then
    
    m_object.Multiset( set );

  end;
  
end;
--------------------------------------------------------------------------------
function int.Buttons_Delete ( module_name, whitelist )
  
  for placeholder, func_Exec in pairs( private.button ) do
    
    if ( string.find( placeholder, module_name ) ) then
    
      if ( not cmn.IsEntityInList( whitelist, placeholder ) ) then

        int.Button_Delete( placeholder );

      end;
      
    end;
    
  end;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Control_EventsSet ( control_name, events, multiset )

  -- CAUTION! use multiset wisely

  -- allowed events:
  -- func_on_mouse_enter, func_on_mouse_leave
  -- func_on_click_start, func_on_click

  -- TODO: add hold start/end timer

  private.control_event_data[ control_name ] = {
    events = events
  };

  m_object.Set(
    control_name, 
    { 
      -- input        = true,
      event_menter = function () private.OnMouseIn ( control_name ); end,
      event_mleave = function () private.OnMouseOut( control_name ); end,
      event_mdown  = function () private.ClickStart( control_name ); end,
      event_mup    = function () private.ClickEnd  ( control_name ); end 
    },
    multiset
  );

end;
--------------------------------------------------------------------------------
function int.Control_EventsGet ( control_name )

  if ( private.control_event_data[ control_name ] ) then
  
    return private.control_event_data[ control_name ].events;
  
  end;

  return nil;

end;
--------------------------------------------------------------------------------
function int.Control_EventsClear ( control_name )

  local data = private.control_event_data[ control_name ] or {};

  private.control_event_data[ control_name ] = nil;

  m_object.Set(
    control_name,
    {
      event_menter = function () end,
      event_mleave = function () end,
      event_mdown  = function () end,
      event_mup    = function () end 
    }
  );

  -- JK: mouse out didn't trigger before int.Control_EventsClear
  if ( data.mouse_in ) then

    if ( IsPC() ) then SetCursor( CURSOR_DEFAULT ); end;

  end;

end;
--------------------------------------------------------------------------------
function private.OnMouseIn ( control_name )

  local data = private.control_event_data[ control_name ];

  if ( not data ) then

    -- JK: Control events may be cleared at the moment
    DbgTrace( "[ INTERFACE ] Attempt to call empty MouseIn for "..control_name );

    return;

  end;

  data.mouse_in = true;

  if ( IsPC() ) then

    SetCursor( CURSOR_HAND );

  end;

  if ( data.events.func_on_mouse_enter ) then

    data.events.func_on_mouse_enter();

  end;

end;
--------------------------------------------------------------------------------
function private.OnMouseOut ( control_name )

  local data = private.control_event_data[ control_name ];

  if ( not data ) then

    -- JK: Control events may be cleared at the moment
    DbgTrace( "[ INTERFACE ] Attempt to call empty MouseOut for "..control_name );

    return;

  end;

  data.click_started = false;
  data.mouse_in      = false;

  if ( IsPC() ) then

    SetCursor( CURSOR_DEFAULT );

  end;

  if ( data.events.func_on_mouse_leave ) then

    data.events.func_on_mouse_leave();

  end;

end;
--------------------------------------------------------------------------------
function private.ClickStart ( control_name )

  local data   = private.control_event_data[ control_name ];

  if ( not data ) then

    DbgTrace( "[ INTERFACE ] Attempt to call empty ClickStart for "..control_name..", input = "..tostring( ObjGet( control_name ).input ) );

    return;

  end;

  local events = data.events;

  if ( events.func_on_click_start ) then events.func_on_click_start(); end;
  if ( events.func_on_click       ) then
  
    data.click_started = true;
    data.click_obj_x, data.click_obj_y = private.ClickPos( control_name );
    data.click_cur_x, data.click_cur_y = CursorPosX() or 0, CursorPosY() or 0;
    
  end;

end;
--------------------------------------------------------------------------------
function private.ClickEnd ( control_name )

  local data = private.control_event_data[ control_name ];

  if ( not data ) then

    DbgTrace( "[ INTERFACE ] Attempt to call empty ClickEnd for "..control_name..", input = "..tostring( ObjGet( control_name ).input ) );

    return;

  end;

  local events = data.events;

  if ( events.func_on_click and data.click_started ) then
  
    local obj_x, obj_y = private.ClickPos( control_name );
    local cur_x, cur_y = CursorPosX() or 0, CursorPosY() or 0;
    
    local obj_moved = obj_x ~= data.click_obj_x or obj_y ~= data.click_obj_y;
    local cur_moved = cur_x ~= data.click_cur_x or cur_y ~= data.click_cur_y;
    
    if not ( obj_moved and cur_moved ) then
    
      data.click_obj_x, data.click_obj_y = nil, nil;
      data.click_cur_x, data.click_cur_y = nil, nil;
      events.func_on_click();
    
    end;

  end;

end;
--------------------------------------------------------------------------------
function private.ClickPos ( control_name )

  local pos = int.Position( control_name );

  return pos[ 1 ], pos[ 2 ];

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Dialog_Open ( dialog_name, func_end )

  local anm_name = "anm_int_dialog_"..dialog_name;
  local wnd_name = "obj_int_dialog_"..dialog_name;

  int.Input_Off( wnd_name );

  AnimPlay(
    anm_name,
    "open_medium",
    function ()

      AnimPlay( anm_name, "static_medium", "" );

      int_window.OuterTap_Set( "int_dialog_"..dialog_name );

      int.Input_On( wnd_name );

      if ( func_end ) then

        func_end();

      end;

    end
  );

end;
--------------------------------------------------------------------------------
function int.Dialog_Close ( dialog_name, func_end )

  local anm_name = "anm_int_dialog_"..dialog_name;
  local wnd_name = "obj_int_dialog_"..dialog_name;

  int_window.OuterTap_Clear();

  int.Input_Off( wnd_name );

  AnimPlay(
    anm_name,
    "close_medium",
    function ()

      int.Input_On( wnd_name );

      if ( func_end ) then

        func_end();

      end;

    end
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Scroller_Next ( scroller, max, loop )
  
  local scrollpos = ObjGet( scroller ).scrollpos;
  local next_pos  = math.min( scrollpos + 1, max - 1 );
  
  if ( next_pos == scrollpos ) then
    
    if ( not loop ) then return next_pos; end;
    
    next_pos = 0;
    
  end;
  
  ObjSet( scroller, { scrollpos = next_pos } );
  
  return next_pos;
  
end;
--------------------------------------------------------------------------------
function int.Scroller_Prev ( scroller, max, loop )
  
  local scrollpos = ObjGet( scroller ).scrollpos;
  local prev_pos  = math.max( scrollpos - 1, 0 );
  
  if ( prev_pos == scrollpos ) then
    
    if ( not loop ) then return prev_pos; end;
    
    prev_pos = max - 1;
    
  end;
  
  ObjSet( scroller, { scrollpos = prev_pos } );
  
  return prev_pos;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Timer_Show (
    timer_name,
    seconds,
    func_exec
  )

  --DbgTrace( "int.Timer_Show: "..timer_name..", "..seconds );
  
  local resolution = 10;
  local time_mode  = "h:m";
  
  -- JK: 1 h + 2 mins & 1 d + 2 mins
  if ( seconds <= 3720  ) then time_mode = "m:s"; resolution = 1;  end;
  if ( seconds >= 86520 ) then time_mode = "d";   resolution = 60; end;

  local tmr_timer = "tmr_"..timer_name;
  local anm_timer = "anm_"..timer_name.."_timer";
  
  local status = m_object.Get( anm_timer ).animfunc;

  if ( seconds <= 0 ) then
    
    int.Timer_Hide( timer_name, true );

  else

    local time = common.GetTimeFromSec( seconds );

    m_object.Multiset( {
      { tmr_timer,
        {
          playing = true,
          time    = resolution,
          endtrig = function() func_exec(); end
        }
      },  
      { anm_timer,
        {
          timer = int.Text_TimerFormat( time, time_mode )
        }
      }
    } );

    if ( status == "timer_static" ) then

      return;

    end;

    AnimPlay(
      anm_timer,
      "timer_show",
      function ()

        AnimPlay( anm_timer, "timer_static", "" );

      end
    );

  end;

end;
--------------------------------------------------------------------------------
function int.Timer_Hide ( timer_name, animated )

  local multiset = {};
  
  local tmr_timer = "tmr_"..timer_name;
  local anm_timer = "anm_"..timer_name.."_timer";
  
  local status = m_object.Get( anm_timer ).animfunc;
  
  if ( status == "timer_hide" ) then

    return;

  end;
  
  if ( animated ) then
    
    AnimPlay( anm_timer, "timer_hide", "" );
    
  else
  
    m_object.Set(
      anm_timer,
      {
        playing  = false,
        animfunc = "timer_hide",
        frame    = 24
      },
      multiset
    );
  
  end;
  
  m_object.Set(
    tmr_timer,
    {
      playing = false
    },
    multiset
  );
  
  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Swipe_Create ( placeholder, params )

  private.swipe[ placeholder ] = params;

  ObjSet( placeholder,
    {
      drag            = true,
      realdrag        = true,
      event_startdrag = function () private.Swipe_Start( placeholder ); end,
      event_dragdrop  = function () private.Swipe_Stop(  placeholder ); end
    }
  );

end;
--------------------------------------------------------------------------------
function private.Swipe_Start ( placeholder )

  private.swipe[ placeholder ].start = GetGameCursorPos();

end;
--------------------------------------------------------------------------------
function private.Swipe_Stop ( placeholder )

  local sensitivity = private.w / 6;

  local swipe_start = private.swipe[ placeholder ].start;
  local swipe_stop  = GetGameCursorPos();

  private.swipe[ placeholder ].start = nil;

  local delta_x = swipe_start[ 1 ] - swipe_stop[ 1 ];
  local delta_y = swipe_start[ 2 ] - swipe_stop[ 2 ];

  if ( math.abs( delta_x ) >= math.abs( delta_y ) ) then

    if ( math.abs( delta_x ) < sensitivity ) then

      return;

    end;

    if ( delta_x < 0 ) then

      -- JK: swipe RIGHT

      if ( private.swipe[ placeholder ].right ) then

        private.swipe[ placeholder ].right();

      end;

    else

      -- JK: swipe LEFT

      if ( private.swipe[ placeholder ].left ) then

        private.swipe[ placeholder ].left();

      end;

    end;

  else

    if ( math.abs( delta_y ) < sensitivity ) then

      return;

    end;

    if ( delta_y < 0 ) then

      -- JK: swipe DOWN

      if ( private.swipe[ placeholder ].down ) then

        private.swipe[ placeholder ].down();

      end;

    else

      -- JK: swipe UP

      if ( private.swipe[ placeholder ].up ) then

        private.swipe[ placeholder ].up();

      end;

    end;    

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.AspectRatio ()
  
  return private.aspect;
  
end;
--------------------------------------------------------------------------------
function int.Landscape ()
  
  return ( private.w > private.h );
  
end;
--------------------------------------------------------------------------------
function int.Orientation ()
  
  if ( int.Landscape() ) then
    
    return "landscape";
    
  end;
  
  return "portrait";
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.W ()
  
  return private.w;
  
end;
--------------------------------------------------------------------------------
function int.H ()
  
  return private.h;
  
end;
--------------------------------------------------------------------------------
function int.Dimension_Max ()

  return math.max( private.w, private.h );

end;
--------------------------------------------------------------------------------
function int.Dimension_Min ()

  return math.min( private.w, private.h );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Percent_W ()
  
  return private.w / 100;
  
end;
--------------------------------------------------------------------------------
function int.Percent_H ()
  
  return private.h / 100;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Center_Pos ()
  
  return {
    pos_x = 0.5 * private.w,
    pos_y = 0.5 * private.h
  };
  
end;
--------------------------------------------------------------------------------
function int.Center_Object ( object_name, shift_x, shift_y, multiset )
  
  shift_x = shift_x or 0;
  shift_y = shift_y or 0;
  
  m_object.Set(
    object_name,
    {
      pos_x = 0.5 * private.w + shift_x,
      pos_y = 0.5 * private.h + shift_y
    },
    multiset
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Position ( object, relative )

  relative = relative or InterfaceName;
  
  local position = {};
  
  local func_IsValid = function ( position )
  
    return ( type( position ) == "table"                      ) and
           ( #position > 1                                    ) and
           ( ( position[ 1 ] ~= 0 ) or ( position[ 2 ] ~= 0 ) );

  end;

  -- JK: 1 variant
  position = wiz.obj.GetPosByObjOnMatrix( object, relative );
  
  if ( func_IsValid( position ) ) then

    --DbgTrace( "wiz.obj.GetPosByObjOnMatrix: "..m_object.." is @ "..position[ 1 ].." x "..position[ 2 ] );
    return position;

  end;
  
  -- JK: 2 variant
  position = GetObjPosByObj( object, relative );
  
  if ( func_IsValid( position ) ) then
 
    --DbgTrace( "GetObjPosByObj: "..m_object.." is @ "..position[ 1 ].." x "..position[ 2 ] );
    return position;
 
  end;
  
  -- JK: fallback to relative pos
  if ( relative ) then
    
    local rel_obj = ObjGet( relative );
    
    position[ 1 ] = rel_obj.pos_x;
    position[ 2 ] = rel_obj.pos_y;
    
    --DbgTrace( "ObjGet: "..m_object.." is @ "..position[ 1 ].." x "..position[ 2 ] );
    return position;
    
  end;
  
  -- JK: fallback to 0
  --DbgTrace( "Fallback: "..m_object.." is @ 0 x 0" );
  return { 0.0, 0.0 };

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Input_On( object_name, multiset )

  local properties =
  {
    input = true
  };

  m_object.Set( object_name, properties, multiset );

end;
--------------------------------------------------------------------------------
function int.Input_Off( object_name, multiset )

  local properties =
  {
    input = false
  };

  m_object.Set( object_name, properties, multiset );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Text_FontsizeByHeight ( object, text_height, default_fontsize )

  local text_parms = ObjGet( object );
  local fontsize   = default_fontsize;

  if ( text_parms.draw_height > text_height ) then

    fontsize =
      math.ceil( text_parms.fontsize * text_height / text_parms.draw_height );

    ObjSet( object, { fontsize = fontsize } );

  end;

end;
--------------------------------------------------------------------------------
function int.Text_CenterIconized ( spr_name, txt_name, spr_width, margin )

  local sprite_w = spr_width or 48;
  local margin_w = margin    or 0;
  local text_w   = ObjGet( txt_name ).draw_width;

  local half_width = ( sprite_w + text_w + margin_w ) / 2;

  local sprite_pos = - half_width + sprite_w / 2;
  local text_pos   = - half_width + sprite_w + margin_w + text_w / 2;

  ObjMultiSet( {
    { spr_name, { pos_x = sprite_pos } },
    { txt_name, { pos_x = text_pos   } }
  } );

end;
--------------------------------------------------------------------------------
function int.Text_TimerFormat ( time, mode ) -- modes "d", "h:m:s", "h:m", "m:s"

  --JK: default mode is "h:m:s"
  --JK: use it after common.GetTimeFromSec()

  if ( mode == "d" ) then
  
    return ne.strings.Get( "str_time_d", time.d );
  
  elseif ( mode == "h:m" ) then

    return ne.strings.Get( "str_time_hm", time.h, time.m );

  elseif ( mode == "m:s" ) then

    return ne.strings.Get( "str_time_ms", time.m, time.s );

  end;

  return ne.strings.Get( "str_time_hms", time.h, time.m, time.s );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Receive ( receiver, object, func_end )

  local position   = int.Position( receiver, object );
  local properties = ObjGet( object );

  local func_OnReceived = function ()

    ObjSet( object, { 
      pos_x   = properties.pos_x, 
      pos_y   = properties.pos_y
    } );

    if ( func_end ) then

      func_end();

    end;

  end;

  ObjAnimate( object, "pos_xy", 0, 0, func_OnReceived,
  { 
    0.0, 3, properties.pos_x, properties.pos_y, 
    1.0, 3, position[ 1 ],    position[ 2 ]
  } );

end;
--------------------------------------------------------------------------------
function int.Receive_Gold ( object, func_end )

  int.Receive ( "obj_int_panel_map_gold", object, func_end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Icon_Animfunc ( icon_name )
  
  local animfunc = {
    gold      = "049_icon_gold",
    resource1 = "054_icon_resource1",
    resource2 = "055_icon_resource2",
    resource3 = "056_icon_resource3",
    bubble    = "icon_bubble",
    bug       = "icon_bug",
    dirt      = "icon_dirt",
    key       = "icon_key"
  };
  
  return animfunc[ icon_name ];
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.AspectRatio_Detect ()

  local ratio = tern( int.Landscape() )( ( private.w / private.h ), ( private.h / private.w ) );
  
  ratio = math.floor( ratio * 100 );
  
  local asp;
  
  asp = tern( ratio == 100 )( "1:1",  nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 126 )( "5:4",  nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 134 )( "4:3",  nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 151 )( "3:2",  nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 161 )( "8:5",  nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 167 )( "5:3",  nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 178 )( "16:9", nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 189 )( "17:9", nil ); if ( asp ) then return asp; end;
  asp = tern( ratio <= 234 )( "21:9", nil ); if ( asp ) then return asp; end;
  
  return asp;
  
end;
--------------------------------------------------------------------------------
function private.WaitingInit ()

  ObjCreate( private.waiting_name, "anim" );
  ObjSet(    private.waiting_name, {
    res      = "assets/content/interface/anm_waiting",
    animfunc = "static",
    playing  = true,
    pos_x    = private.w / 2,
    pos_y    = private.h / 2,
    pos_z    = private.int_pos_z + 1,
    tt_wait  = "str_loader_message_wait"
  } );
  ObjAttach( private.waiting_name, "ne_storage" );
  
  local bg_name = string.format( "spr_%s_black", 
                  cmn.GetObjectName( private.waiting_name ) );
  
  ObjCreate( bg_name, "spr" );
  ObjSet(    bg_name, {
    res      = "assets/resources/default",
    anim_tag = "bg",
    scale_x  = 32,
    scale_y  = 36
  } );
  ObjAttach( bg_name, private.waiting_name );

end;
--------------------------------------------------------------------------------
function private.LockInit ()

  int_control.Add( "interface_lock" );

  ObjCreate( "ng_interface_lock",     "obj" );
  ObjAttach( "ng_interface_lock",     "hub_interface_lock" );

  ObjSet( "ng_interface_lock",
  { 
    pos_x          = ( private.w / 2 ),
    pos_y          = ( private.h / 2 ),
    pos_z          = 1,
    input          = false,
    inputrect_init = true,
    inputrect_x    = - ( private.w / 2 ),
    inputrect_y    = - ( private.h / 2 ),
    inputrect_w    = private.w,
    inputrect_h    = private.h
  } );
  
  ObjCreate( "ng_interface_lock_spr", "grid" );
  ObjAttach( "ng_interface_lock_spr", "ng_interface_lock" );

  ObjSet( "ng_interface_lock_spr",
  { 
    res     = "assets/content/interface/grd_stage_background",
    active  = false,
    visible = false,
    input   = false,
    pos_x   = - ( private.w / 2 ),
    pos_y   = - ( private.h / 2 ),
    scale_x = 10,
    scale_y = 10,
    alp     = 0.0,
    color_r = 1,
    color_g = 0.5,
    color_b = 0.8
  } );  

  local grid_params = 
  {
     0,  { pos_x = 0 },
     1,  { pos_x = 1 },
     2,  { pos_x = private.w - 1 },
     3,  { pos_x = private.w },

     4,  { pos_x = 0 },
     5,  { pos_x = 1 },
     6,  { pos_x = private.w - 1 },
     7,  { pos_x = private.w },

     8,  { pos_x = 0, pos_y = private.h - 1 },
     9,  { pos_x = 1, pos_y = private.h - 1 },
     10, { pos_x = private.w - 1, pos_y = private.h - 1 },
     11, { pos_x = private.w,     pos_y = private.h - 1 },

     12, { pos_x = 0, pos_y = private.h },
     13, { pos_x = 1, pos_y = private.h },
     14, { pos_x = private.w - 1, pos_y = private.h },
     15, { pos_x = private.w,     pos_y = private.h }
  };

  GridSet( "ng_interface_lock_spr", 0, grid_params );  
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function int.Grid_Resize ( name, size, border, frame )
  
  frame = frame or 0;
  
  if ( border ) then
    
    GridSet( name, frame,
    {
       0,  { pos_x = -size.left,                 pos_y = -size.top },
       1,  { pos_x = -size.left  + border.left,  pos_y = -size.top },
       2,  { pos_x =  size.right - border.right, pos_y = -size.top },
       3,  { pos_x =  size.right,                pos_y = -size.top },
      
       4,  { pos_x = -size.left,                 pos_y = -size.top + border.top },
       5,  { pos_x = -size.left  + border.left,  pos_y = -size.top + border.top },
       6,  { pos_x =  size.right - border.right, pos_y = -size.top + border.top },
       7,  { pos_x =  size.right,                pos_y = -size.top + border.top },
      
       8,  { pos_x = -size.left,                 pos_y = size.bottom - border.bottom },
       9,  { pos_x = -size.left  + border.left,  pos_y = size.bottom - border.bottom },
      10,  { pos_x =  size.right - border.right, pos_y = size.bottom - border.bottom },
      11,  { pos_x =  size.right,                pos_y = size.bottom - border.bottom },
      
      12,  { pos_x = -size.left,                 pos_y = size.bottom },
      13,  { pos_x = -size.left  + border.left,  pos_y = size.bottom },
      14,  { pos_x =  size.right - border.right, pos_y = size.bottom },
      15,  { pos_x =  size.right,                pos_y = size.bottom },
    } );
    
    return;
    
  end;
  
  GridSet( name, frame,
  {
    0,  { pos_x = -size.left,  pos_y = -size.top },
    1,  { pos_x =  size.right, pos_y = -size.top },
    2,  { pos_x = -size.left,  pos_y =  size.bottom },
    3,  { pos_x =  size.right, pos_y =  size.bottom }
  } );
  
end;
--------------------------------------------------------------------------------