-- name=loading
--------------------------------------------------------------------------------
function public.Init () 

  int.Center_Object( "obj_loading" );
  
  ObjMultiSet( {
    { "obj_loading", {
      event_mup      = function () end,
      input          = true,
      inputrect_init = true,
      inputrect_x    = - int.W() / 2,
      inputrect_y    = - int.H() / 2,
      inputrect_w    =   int.W(),
      inputrect_h    =   int.H()
    } },
    { "obj_loading_progressbar", { 
      pos_y = 0.5 * int.H() - 50
    } },
    { "obj_loading_hint", { 
      pos_y = 0.5 * int.H() - 230
    } }    
  } );  
  
end;
--------------------------------------------------------------------------------
function public.Show ( need_fade, func_continue )

  if ( private.on_screen ) then

    need_fade = false;
    
  else
    
    private.Generate_Hint();

  end;

  func_continue = func_continue or function () end;

  private.on_screen = true;
  public.SetProgressBar( 0, 0, 0 );

  ObjAttach( "obj_loading", "ng_application" );

  ObjSet( "obj_loading", { active = true, visible = true, pos_z = 1001 } );

  ShowLoadingWheel( true );

  local func_next = function ()

    ObjSet( "obj_loading", { alp = 1.0 } );

    func_continue();

  end;

  if ( not need_fade ) then

    func_next();

    return;

  end;

  ObjAnimate( "obj_loading", "alp", 0, 0, func_next,
  {
    0.0, 0, 0.0,
    0.3, 0, 1.0
  } );

end;
--------------------------------------------------------------------------------
function public.Hide ( need_fade, func_continue )

  func_continue = func_continue or function () end;
  
  if not ( private.on_screen ) then

    func_continue();
    return;

  end;

  local func_next = function ()

    ShowLoadingWheel( false );

    ObjSet( "obj_loading", { active = false, visible = false, alp = 0.0 } );
    ObjDetach( "obj_loading" );

    private.on_screen = false;

    func_continue();

  end;

  if ( not need_fade ) then

    func_next();

    return;

  end;

  ObjAnimate( "obj_loading", "alp", 0, 0, func_next,
  {
    0.0, 0, 1.0,
    0.3, 0, 0.0
  } );

end;
--------------------------------------------------------------------------------
-- Set progress bar to specified value in range from 'from_abs' to 'to_abs'
-- For example: public.SetProgressBar( 0.5, 0.8, 0.9 ) will set bar to 0.85
--------------------------------------------------------------------------------
function public.SetProgressBar ( relative, from_abs, to_abs )
  
  if ( not private.on_screen ) then return; end;
  
  local crop_current = ObjGet( "grd_loading_progress" ).croprect_w;
  
  local crop_new = 352 * ( from_abs + ( to_abs - from_abs ) * relative ) + 0.1;
  
  ObjAnimate( "grd_loading_progress", "croprect_w", 0, 0, "", 
  {
    0.000, 0, crop_current,
    0.017, 0, crop_new
  } );
  
end;
--------------------------------------------------------------------------------
function private.Generate_Hint ()
  
  local random_index = math.random( 1, 17 );
  local help_show = tern( random_index > 7 )( false, true );
  
  ObjMultiSet( {
    { "txt_loading_hint",    { visible = ( not help_show ), 
                               text = "str_loader_hint_"..random_index } },
    { "anm_loading_hint",    { visible = help_show,
                               animfunc = "help_"..random_index,
                               help_text = "str_loader_hint_"..random_index } }
  } );
  
end;
--------------------------------------------------------------------------------