-- name=m_achievements
--[[
  Types:

  MineCollected
  TurnWheel
  UseBooster
  DestroyChip
  DestroyBlocker
  DestroyBonus
  StageTurns    --commented
  FightTurns    --commented
  WinDiffStage
  AddFriend
  WinFriend
]]
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ ACHIEVEMENTS ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()
  
  private.achievement_types = {};

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "achievements"
  };

  local achievements = m_storage.Execute( query ).result;

  for a = 1, #achievements do

    local achievement = achievements[ a ];

    if ( IsMobile() or ( not achievement.type:find( "Friend" ) ) ) then
    
      query = 
      {
        query      = "find",
        group      = "player",
        collection = "achievements",
        indice     = "name",
        criteria   = { like = achievement.name }
      };

      if ( not m_storage.Execute( query ).result ) then

        query =
        {
          query      = "insert",
          group      = "player",
          collection = "achievements",
          value      = 
          {
            name     = achievement.name,
            level    = 1,
            count    = 0,
            reward   =
            {
              { get = false, done = false },
              { get = false, done = false },
              { get = false, done = false }
            }
          }
        };

        m_storage.Execute( query );
        
        private.Android_IncrementAchievement( achievement.name, 0, true );
        
      end;

      if ( not private.achievement_types[ achievement.type ] ) then

        private.achievement_types[ achievement.type ] = {};

      end;
      
      if ( achievement.type == "WinDiffStage" ) then
        local stage_stats           = m_stage.GetStagesStat();
        achievement.bonus_count     = stage_stats.bonus_count;
        achievement.completed_count = stage_stats.completed_count;
      end;

      table.insert( private.achievement_types[ achievement.type ], achievement );
      
    end;

  end;
  
  private.caches = {
    achievements = cmn.ConvertToAssociated( achievements, "name" )
  };
  
  private.InitSubscribes();

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
function private.InitSubscribes ()
  
  mediator.Subscribe( "Model_Mines_MineCollected",     private.CheckMineCollected );
  mediator.Subscribe( "View_Stage_ContinueWheelTurn",  private.IncreaseCounterWheel );
  --mediator.Subscribe( "View_Stage_ContinueWheelClose", private.ResetCounterWheel );
  mediator.Subscribe( "Model_Boosters_CountChanged",   private.CheckBoostersUse );
  mediator.Subscribe( "Model_Stage_StageWon",       function ( params )
    --private.CheckStageTurns( params );
    private.CheckDifferentStage( params.name );
  end );
  mediator.Subscribe( "Model_Stage_CellDestroyed",     private.CheckBlocker );
  mediator.Subscribe( "Model_Stage_ChainActivated",    function ( params )
    private.CheckChips( params.chips_list );
  end );
  mediator.Subscribe( "Model_Stage_BonusesSwapped",    function ( params )
    private.CheckBonus( params.combo_type );
  end );
  mediator.Subscribe( "Model_Match3_BonusRemoved",     function ( params )
    private.CheckBonus( params.type );
  end );
  --mediator.Subscribe( "Model_Shop_LotBought",          private.CheckLotBought );
  mediator.Subscribe( "View_Stage_FriendsWon",         private.IncreaseCounterWonFriends );
  
  if ( IsMobile() ) then
  
    mediator.Subscribe( 
      "Model_Social_FriendsCollectionsUpdated", 
      private.CheckFriendsCount 
    );
    
  end;

end;
--------------------------------------------------------------------------------
function private.Android_IncrementAchievement ( achievement, count, unlock )

  if not ( IsAndroid() and wiz.Android.GetPlatform() == "google" ) then
  
    return;
    
  end;

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "achievements",
    indice     = "name",
    criteria   = { like = achievement }
  };
  
  local data = m_storage.Execute( query ).result;
  
  if ( data and data.google_play_ids ) then
    
    for _, id in pairs( data.google_play_ids ) do
    
      wiz.Android.IncrementAchievement( unlock, id, count );
    
    end;
  
  end;

end;
--------------------------------------------------------------------------------
function public.GetReward ( achievement_name, reward_type, reward_count, level )

  local result = false;

  if ( reward_type == "gold" ) then

    result = m_player.TryChangeGoldCount( reward_count );

  end;

  if ( result ) then

    local query = 
    {
      query      = "find",
      group      = "player",
      collection = "achievements",
      indice     = "name",
      criteria   = { like = achievement_name }
    };

    local achievement = m_storage.Execute( query ).result;

    local reward = achievement.reward;

    reward[ level ][ "done" ] = true;

    query =
    {
      query      = "update",
      group      = "player",
      collection = "achievements",
      indice     = "name",
      criteria   = { like = achievement_name },
      set        = 
      {
        count  = achievement.count,
        level  = achievement.level,
        name   = achievement.name,
        reward = reward
      }
    };

    m_storage.Execute( query );    

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_collect",
      category    = "achievement",
      value       = reward_count,
      name        = achievement_name
    } );

    mediator.Notify( "Model_Achievements_Rewarded" ); 

  end;

end;
--------------------------------------------------------------------------------
function public.GetAll ()

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "achievements"
  };

  local achievements = m_storage.Execute( query ).result;
  
  local result = {};
  
  for a = 1, #achievements do

    local achievement = achievements[ a ];

    if ( IsMobile() or ( not achievement.type:find( "Friend" ) ) ) then

      query = 
      {
        query      = "find",
        group      = "player",
        collection = "achievements",
        indice     = "name",
        criteria   = { like = achievement.name }
      };    

      local answer = m_storage.Execute( query ).result;

      if ( answer ) then

        achievement.count  = answer.count;
        achievement.level  = answer.level;
        achievement.reward = answer.reward;

      end;
      
      table.insert( result, achievement );
      
    end;

  end;

  --return private.Sort( achievements );
  return result;

end;
--------------------------------------------------------------------------------
function private.Sort ( achievements )

  for a = 1, #achievements do

    local achievement = achievements[ a ];

    if ( achievement.reward ) then

      for b = 1, #achievement.reward do
        
        if ( achievement.reward[ b ][ "get" ] ) and ( not achievement.reward[ b ][ "done" ] ) then

          table.remove( achievements, a );

          table.insert( achievements, 1, achievement );

          break;

        end;

      end;

    end;

  end;

  return achievements;

end;
--------------------------------------------------------------------------------
function public.GetRewardData ( achievement_name )

  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "achievements",
    indice     = "name",
    criteria   = { like = achievement_name }
  };    

  return m_storage.Execute( query ).result.reward;

end;
--------------------------------------------------------------------------------
function public.GetAchievementsStat ()

  local achievements = public.GetAll();

  local total_count = 3 * #achievements;

  local completed_count = 0;

  for i = 1, #achievements do

    local achievement = achievements[ i ];

    for r = 1, #achievement.reward do

      if ( achievement.reward[ r ].get ) then

        completed_count = completed_count + 1;

      end; 

    end;

  end;

  return { completed_count = completed_count, total_count = total_count };

end;
--------------------------------------------------------------------------------
function private.GetAchievementData ( achievement_name )
  
  return private.caches.achievements[ achievement_name ];
  
end;
--------------------------------------------------------------------------------
function private.ChangeCount ( achievement_name, count_delta, reverse )

  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "achievements",
    indice     = "name",
    criteria   = { like = achievement_name }
  };

  local achievement = m_storage.Execute( query ).result;

  local levels = private.GetAchievementData( achievement_name ).levels;

  if ( achievement.level > #levels ) then

    return;

  end;

  local new_count;

  if ( not reverse ) then
    new_count = achievement.count + count_delta;
  elseif ( count_delta < achievement.count ) or ( achievement.count == 0 ) then
    new_count = count_delta;
  end;

  query =
  {
    query      = "update",
    group      = "player",
    collection = "achievements",
    indice     = "name",
    criteria   = { like = achievement_name },
    set        = 
    {
      count = new_count
    }
  };

  m_storage.Execute( query );
  
  private.Android_IncrementAchievement( achievement_name, count_delta, false );

  private.TryComplete( achievement_name, reverse );

end;
--------------------------------------------------------------------------------
function private.TryComplete ( achievement_name, reverse )

  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "achievements",
    indice     = "name",
    criteria   = { like = achievement_name }
  };

  local achievement = m_storage.Execute( query ).result;

  local levels = private.GetAchievementData( achievement_name ).levels;

  if ( achievement.level > #levels ) then return; end;

  local count_need = levels[ achievement.level ].value;

  if (( not reverse ) and ( achievement.count >= count_need )) or 
  (( reverse ) and ( achievement.count <= count_need ) and ( achievement.count > 0 )) then

    local new_count;

    if ( not reverse ) then new_count = achievement.count; --achievement.count - count_need; 
    else                    new_count = achievement.count;
    end;
  
    local new_level = achievement.level + 1;

    if ( new_level > #levels ) then

      new_count = 0;

    end;

    local reward = achievement.reward;

    reward[ achievement.level ][ "get" ] = true;

    query =
    {
      query      = "update",
      group      = "player",
      collection = "achievements",
      indice     = "name",
      criteria   = { like = achievement_name },
      set        = 
      {
        level  = new_level,
        count  = new_count,
        reward = reward
      }
    };

    m_storage.Execute( query );

    private.TryComplete( achievement_name, reverse );

    mediator.Notify( "Model_Achievements_Completed", { name = achievement_name, level = achievement.level } ); 

  end;

end;
--------------------------------------------------------------------------------
function private.IncreaseCounterWheel ( params )

  private.ChangeCount( private.achievement_types[ "TurnWheel" ][ 1 ].name, 1 );

end;
--------------------------------------------------------------------------------
function private.ResetCounterWheel ( params )

  local achievement_name = private.achievement_types[ "TurnWheel" ][ 1 ].name;

  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "achievements",
    indice     = "name",
    criteria   = { like = achievement_name }
  };
  
  local achievement = m_storage.Execute( query ).result;

  for i = 1, #achievement.reward do
    if not ( achievement.reward[ i ].get == achievement.reward[ i ].done ) then
      private.ChangeCount( achievement_name, - achievement.count );
      break;
    end;
  end;

end;
--------------------------------------------------------------------------------
function private.CheckMineCollected ( mine_name )

  for a = 1, #private.achievement_types[ "MineCollected" ] do

    local achievement = private.achievement_types[ "MineCollected" ][ a ];

    if ( achievement.target == "none"    ) or
       ( mine_name == achievement.target ) then

      private.ChangeCount( achievement.name, 1 );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.CheckBoostersUse ( params )

  if ( params.delta >= 0 ) then

    return;

  end;

  for a = 1, #private.achievement_types[ "UseBooster" ] do

    local achievement = private.achievement_types[ "UseBooster" ][ a ];

    if ( params.name == achievement.target ) then

      private.ChangeCount( achievement.name, - params.delta );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.CheckStageTurns ( params )

  local ach_type;

  if ( m_stage.IsCurrentStageFight() ) then
    ach_type = "FightTurns"; 
  else
    return; --ach_type = "StageTurns";
  end;

  local moves = m_stage.MovesDone_GetCount();

  for a = 1, #private.achievement_types[ ach_type ] do

    local achievement = private.achievement_types[ ach_type ][ a ];

    if ( achievement.target == "none" ) or ( params.stage == achievement.target ) then

      private.ChangeCount( private.achievement_types[ ach_type ][ a ].name, moves, true );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.CheckBlocker ( chip )

  for a = 1, #private.achievement_types[ "DestroyBlocker" ] do

    local achievement = private.achievement_types[ "DestroyBlocker" ][ a ];

    if ( chip.name == achievement.target ) then

      private.ChangeCount( achievement.name, 1 );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.CheckChips ( chips )

  for i = 1, #chips do

    local chip = chips[ i ];

    for a = 1, #private.achievement_types[ "DestroyChip" ] do

      local achievement = private.achievement_types[ "DestroyChip" ][ a ];

      if ( achievement.target == "none" ) or ( chip.type == achievement.target ) then

        private.ChangeCount( achievement.name, 1 );

      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.CheckBonus ( chip_type )

  for a = 1, #private.achievement_types[ "DestroyBonus" ] do

    local achievement = private.achievement_types[ "DestroyBonus" ][ a ];

    if ( achievement.target == "none" ) or 
    ( chip_type:find( achievement.target ) and not chip_type:find( "Second" )) then

      private.ChangeCount( achievement.name, 1 );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.CheckDifferentStage ( stage_name )

  if ( stage_name:find( "daily" )) then
  
    return;
    
  end;

  local stage_num   = tonumber(stage_name:match( "%d+" ));
  local achievement = private.achievement_types[ "WinDiffStage" ][ 1 ];

  local field;
  if ( stage_name:find( "bonus" )) then
    field = "bonus_count";
  else
    field = "completed_count";
  end;

  local diff = stage_num - achievement[ field ];
  if ( diff > 0 ) then
    private.ChangeCount( achievement.name, diff );
    private.achievement_types[ "WinDiffStage" ][ 1 ][ field ] = stage_num;
  end;

end;
--------------------------------------------------------------------------------
function private.CheckFriendsCount ( params )

  --You can get collection count from _params_, but it's not correct
  local achievement_name = private.achievement_types[ "AddFriend" ][ 1 ].name;
  local friends_count = #m_social.GetFriends();

  if ( private.achievement_types[ "AddFriend" ][ 1 ].friends_count == friends_count ) then
    return;
  end;

  private.achievement_types[ "AddFriend" ][ 1 ].friends_count = friends_count;

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "achievements",
    indice     = "name",
    criteria   = { like = achievement_name }
  };
  local achievement_info  = m_storage.Execute( query ).result or {};
  local achievement_count = achievement_info.count or 0;
  local achievement_data  = private.GetAchievementData( achievement_name );

  for i = 1, #achievement_info.reward do

    local is_get  = achievement_info.reward[ i ].get;
    local is_done = achievement_info.reward[ i ].done;

    if ( is_get and ( not is_done )) then
      return;
    elseif ( is_get and ( is_done )) then
      achievement_count = achievement_count + achievement_data.levels[ i ].value;
    end;

  end;

  local diff = friends_count - achievement_count;
  if ( diff > 0 ) then
    private.ChangeCount( achievement_name, diff );
  end;

end;
--------------------------------------------------------------------------------
function private.CheckLotBought ( params )

  for a = 1, #private.achievement_types[ "BuyLot" ] do

    local achievement = private.achievement_types[ "BuyLot" ][ a ];

    if ( params.lot_type == achievement.target ) then

      private.ChangeCount( achievement.name, 1 );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.IncreaseCounterWonFriends ( count )

  local achievement = private.achievement_types[ "WinFriend" ][ 1 ].name;
  private.ChangeCount( achievement, count );

end;
--------------------------------------------------------------------------------