-- name=m_boosters
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ BOOSTER ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.booster_selected = nil;

  private.unlocked_list = {};

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
function public.Update ( stage_name )

  local booster_list = private.GetList();

  for b = 1, #booster_list do

    local booster = booster_list[ b ];

    for s = 1, #booster.unlock_stages do

      if ( booster.unlock_stages[ s ] == stage_name ) then

        local boostert_unlocked = false;

        local query = 
        {
          query      = "find",
          group      = "player",
          collection = "boosters",
          indice     = "name",
          criteria   = { like = booster.name }
        };

        local booster_exist = m_storage.Execute( query ).result;

        if ( not booster_exist ) then

          booster_exist =
          {
            name     = booster.name,
            count    = 0,
            unlocked = true
          };

          query =
          {
            query      = "insert",
            group      = "player",
            collection = "boosters",
            value      = booster_exist
          };

          m_storage.Execute( query );

          boostert_unlocked = true;

        end;

        if ( not booster_exist.unlocked ) then

          query =
          {
            query      = "update",
            group      = "player",
            collection = "boosters",
            indice     = "name",
            criteria   = { like = booster.name },
            set        =
            {
              unlocked = true
            }
          };

          m_storage.Execute( query );

          boostert_unlocked = true;

        end;

        if ( boostert_unlocked ) then

          table.insert( private.unlocked_list, booster.name );

        end;

        break;

      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.GetBoostersData ( boosters_list )

  local boosters_data = {}; 

  for b = 1, #boosters_list do

    local booster = {};

    booster.name  = boosters_list[ b ].name;

    local query =
    {
      query      = "find",
      group      = "static",
      collection = "boosters",
      indice     = "name",
      criteria   = { like = booster.name }
    };

    local database_answer = m_storage.Execute( query ).result;

    booster.packs    = database_answer.packs;
    booster.type     = database_answer.type;
    booster.count    = 0;
    booster.unlocked = false;

    query =
    {
      query      = "find",
      group      = "player",
      collection = "boosters",
      indice     = "name",
      criteria   = { like = booster.name }
    };

    local booster_data = m_storage.Execute( query ).result;

    if ( booster_data ) then

      booster.count    = booster_data.count    or booster.count;
      booster.unlocked = booster_data.unlocked or booster.unlocked;

    end;

    table.insert( boosters_data, booster );

  end;

  return boosters_data;

end;
--------------------------------------------------------------------------------
function public.StripUnknown ( boost_list )
  
  local known_boosters = {};
  
  for i = 1, #boost_list do
    
    local boost_name = boost_list[ i ];
    
    if ( public.IsUnlocked( boost_name ) ) then
      
      table.insert( known_boosters, boost_name );
      
    end;
    
  end;
  
  return known_boosters;
  
end;
--------------------------------------------------------------------------------
function public.GetList ( type )

  return private.GetList( type );

end;
--------------------------------------------------------------------------------
function private.GetList ( type )

  local query;

  if ( type ) then

    query =
    {
      query      = "find_all",
      group      = "static",
      collection = "boosters",
      indice     = "type",
      criteria   = { like = type }
    };

  else

    query =
    {
      query      = "find_all",
      group      = "static",
      collection = "boosters"
    };

  end;

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.GetCount ( booster_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "boosters",
    indice     = "name",
    criteria   = { like = booster_name }
  };

  local booster = m_storage.Execute( query ).result;
  
  if ( booster ) and ( booster.count ) then
    
    return booster.count;
    
  end;

  return 0;

end;
--------------------------------------------------------------------------------
function public.TryBuy ( booster_name, pack_level, source )

  local pack_data = public.GetPacks( booster_name )[ pack_level ];

  local booster_price = pack_data.price;
  local booster_count = pack_data.count;

  if ( m_player.TryChangeGoldCount( - booster_price ) ) then
    
    if ( source ) and ( string.find( source, "stage" ) ) then
      local _, stage_type = string.match( source, "stage_(%d+)(_%w+)" );
      source = "level"..( stage_type or "" );
    end;
    
    if ( booster_name == "extramoves" or 
         booster_name == "extralives" ) then
      common.Kontagent_LogEvent( "economy", 
      {
        name = booster_name,
        description = "extra_moves_offer",
        purchase = 1,
        level = 1,
        value = - booster_price
      } );
    end;

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "boosts",
      value       = - booster_price,
      name        = booster_name,
      source      = source
    } );

    return public.TryChangeCount( booster_name, booster_count, source );

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.TryUse ( booster_name, source )

  return public.TryChangeCount( booster_name, - 1, source );

end;
--------------------------------------------------------------------------------
function public.TryChangeCount ( booster_name, count_delta, source )
  
  local count_new = count_delta + public.GetCount( booster_name );
  
  if ( count_new < 0 ) then return false; end;

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "boosters",
    indice     = "name",
    criteria   = { like = booster_name }
  };

  local booster_exist = m_storage.Execute( query ).result;

  if ( not booster_exist ) then

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "boosters",
      value      =
      {
        name  = booster_name,
        count = 0
      }
    };
 
    m_storage.Execute( query );

  end;

  query =
  {
    query      = "update",
    group      = "player",
    collection = "boosters",
    indice     = "name",
    criteria   = { like = booster_name },
    set        = 
    {
      count = count_new
    }
  };

  m_storage.Execute( query );
  
  local description = "item_collected";
  
  if ( count_delta < 0 ) then
    
    description = "item_used";
    
  end;
  
  if ( source ) and ( string.find( source, "stage" ) ) then
    local _, stage_type = string.match( source, "stage_(%d+)(_%w+)" );
    source = "level"..( stage_type or "" );
  end;  
  
  common.Kontagent_LogEvent( "economy", 
  {
    description = description,
    category    = "boosts",
    value       = count_delta,
    name        = booster_name,
    source      = source
  } );  
  
  mediator.Notify(
    "Model_Boosters_CountChanged",
    {
      name   = booster_name,
      delta  = count_delta,
      count  = count_new,
      source = source } );
  
  return true;

end;
--------------------------------------------------------------------------------
function public.Unlock ( booster_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "boosters",
    indice     = "name",
    criteria   = { like = booster_name }
  };

  if ( not m_storage.Execute( query ).result ) then

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "boosters",
      value      =
      {
        name     = booster_name,
        count    = 0,
        unlocked = true
      }
    };
 
    m_storage.Execute( query );

  end;

  query =
  {
    query      = "update",
    group      = "player",
    collection = "boosters",
    indice     = "name",
    criteria   = { like = booster_name },
    set        = 
    {
      unlocked = true
    }
  };

  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
function public.IsUnlocked ( booster_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "boosters",
    indice     = "name",
    criteria   = { like = booster_name }
  };
  
  local booster = m_storage.Execute( query ).result;
  
  if ( booster ) and ( booster.unlocked ) then
    
    return booster.unlocked;
    
  end;
  
  return false;

end;
--------------------------------------------------------------------------------
function public.SetSelected ( boosters_list )

  private.selected = boosters_list;

end;
--------------------------------------------------------------------------------
function public.GetSelected ()

  if ( private.selected ) then

    local selected = {};

    for i = 1, #private.selected do

      table.insert( selected, private.selected[ i ] );

    end;

    private.selected = nil;

    return selected;

  end;

  return {};

end;
--------------------------------------------------------------------------------
function public.GetPrice ( booster_name, level )

  return private.GetPrice( booster_name, level );

end;
--------------------------------------------------------------------------------
function private.GetPrice ( booster_name, level )

  return public.GetPacks( booster_name )[ level or 1 ].price;

end;
--------------------------------------------------------------------------------
function public.GetPacks ( booster_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "boosters",
    indice     = "name",
    criteria   = { like = booster_name }
  };

  return m_storage.Execute( query ).result.packs;

end;
--------------------------------------------------------------------------------