-- name=m_builds
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ BUILDS ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.Trace( "Initialized." );
  
  private.puzzle_unlockers = {};
  
  local puzzles = public.GetBuildsByTypeAndSection( "puzzle" );
  
  for p = 1, #puzzles do
    
    if ( puzzles[ p ].reward ) then
      
      private.puzzle_unlockers[ puzzles[ p ].unlock ] = puzzles[ p ];
      
    end;
    
  end;

end;
--------------------------------------------------------------------------------
function public.TryUnlockBuildByStage ( stage_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "builds",
    indice     = "unlock",
    criteria   = { like = stage_name }
  };

  local build_data = m_storage.Execute( query ).result;

  if ( build_data ) then

    query =
    {
      query      = "find",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = build_data.name }
    };
    
    local saved_data = m_storage.Execute( query ).result;

    if ( saved_data and not saved_data.constructed ) or
       ( not saved_data                            ) then

      query =
      {
        query      = "update",
        group      = "player",
        collection = "builds",
        indice     = "name",
        criteria   = { like = build_data.name },
        set        =
        {
          unlocked    = true,
          constructed = false
        }
      };

      if ( not m_storage.Execute( query ).result ) then

        query =
        {
          query      = "insert",
          group      = "player",
          collection = "builds",
          value      =
          {
            name        = build_data.name,
            unlocked    = true,
            constructed = false
          }
        };

        m_storage.Execute( query );

      end;

      mediator.Notify( "Model_Builds_BuildUnlocked", build_data.name );

      -- Autocinstruct island
      if private.IsAutoConstructed( build_data ) then

        public.TryConstructBuild( build_data.name );

      end;
      
      -- Construct completed puzzle or island
      if ( private.IsCompleted( build_data.name ) ) then
      
        public.TryConstructBuild( build_data.name );
      
      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.GetBuildNameByFragment ( fragment_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "builds"
  };
  local db_answer = m_storage.Execute( query ).result;

  if ( db_answer ) then

    for i = 1, #db_answer do

      if ( db_answer[ i ].fragments                                      ) and
         ( cmn.IsEntityInList( db_answer[ i ].fragments, fragment_name ) ) then

        return db_answer[ i ].name;

      end;

    end;

  end;

  return nil;
  
end;
--------------------------------------------------------------------------------
function public.GetPuzzleUnlockedByStage ( stage_name )
  
  if ( private.puzzle_unlockers[ stage_name ] ) then
    
    local puzzle_name = private.puzzle_unlockers[ stage_name ].name;
    
    if ( public.GetBuildData( puzzle_name ).constructed ) then

      return false;

    end;
  
    return puzzle_name;
    
  end;

  return nil;
  
end;
--------------------------------------------------------------------------------
function public.TryObtainFragment ( fragment_name, stage_name )

  local build_name = public.GetBuildNameByFragment( fragment_name );

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "builds",
    indice     = "name",
    criteria   = { like = build_name }
  };
  
  local build_data = m_storage.Execute( query ).result;

  if ( build_data ) then

    query = 
    {
      query      = "find",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = build_data.name }
    };

    local collection_data = m_storage.Execute( query ).result;

    local collected_fragments = {};

    if not ( collection_data ) then

      local unlocked = false;

      if ( not build_data.unlock           ) or
         ( build_data.unlock == ""         ) or
         ( build_data.unlock == stage_name ) then

        unlocked = true;

      end;

      private.Unlock( build_data.name, unlocked );

    else

      collected_fragments = collection_data.collected_fragments or {};

    end;

    if ( cmn.IsEntityInList( collected_fragments, fragment_name ) ) then

      DbgTrace(
        "[ ERROR ] m_builds -> TryObtainFragment -> "..
        fragment_name.." already in save for "..build_data.name
      );
      return false;

    end;

    table.insert( collected_fragments, fragment_name );

    query =
    {
      query      = "update",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = build_data.name },
      set        =
      {
        collected_fragments = collected_fragments
      }
    };

    m_storage.Execute( query );
    
    -- Construct completed puzzle or island
    if ( private.IsCompleted( build_name ) ) then
    
      public.TryConstructBuild( build_name );
    
    end;

    return collected_fragments;

  else

    DbgTrace(
      "[ ERROR ] m_builds -> TryObtainFragment -> No build for "..
      fragment_name
    );
    return false;

  end;

end;
--------------------------------------------------------------------------------
function public.TryConstructBuild ( build_name, func_exec )

  local build_data = public.GetBuildData( build_name );
  local price      = build_data.price;
  local fragments  = build_data.fragments;
  local unlock     = build_data.unlock;

  if ( unlock ) and not ( unlock == "" ) then

    local stage = m_stage.GetSavedData( unlock ) or {};

    if not ( stage.completed ) then

      return false;

    end;

  end;

  if ( fragments ) then

    if ( not build_data.collected_fragments           ) or
       ( #build_data.collected_fragments < #fragments ) then

      for f = 1, #fragments do

        local lost_fragment = fragments[ f ];

        if ( not cmn.IsEntityInList( build_data.collected_fragments, lost_fragment ) ) then

          mediator.Notify( "Model_Builds_NoFragments", { 
            name = build_name, deficient = {
              name    = lost_fragment,
              count   = 1,
              section = build_data.section
            }
          } );

          return false;

        end;

      end;

    end;

  end;

  if ( price ) then

    local resources_check_table = {};

    for i = 1, #price do

      table.insert(
        resources_check_table,
        {
          name  = price[ i ].resource,
          delta = - price[ i ].count
        }
      );

    end;

    local resource_sufficient = m_player.Resources_TryDecrease(
      resources_check_table
    );

    if ( not resource_sufficient.result ) then

      return false;

    end;

    for i = 1, #resource_sufficient.resources do

      common.Kontagent_LogEvent( "economy", 
      {
        description = "soft_currency_spend",
        category    = "build",
        value       = resource_sufficient.resources[ i ].delta,
        name        = build_name,
        type        = resource_sufficient.resources[ i ].name
      } );

    end;

  end;

  local update_query =
  {
    query      = "update",
    group      = "player",
    collection = "builds",
    indice     = "name",
    criteria   = { like = build_name },
    set        =
    {
      constructed = true
    }
  };
  
  m_storage.Execute( update_query );

  local stage_name = "stage_"..build_data.unlocklist.min;

  if ( build_data.unlocklist.type == "bonus" ) then 

    stage_name = "stage_"..build_data.unlocklist.min.."_bonus";

  end;

  m_stage.TryEnable( stage_name );
  
  -- NOTE: Build rewards has no unified format!
  if ( build_data.reward ) then
  
    local reward;
  
    if     ( build_data.type == "build"  ) then reward = build_data.reward[ 1 ];
    elseif ( build_data.type == "puzzle" ) then reward = build_data.reward;
    elseif ( IsEditor()                  ) then
    
      local msg = "Error! Can't obtain reward for build %q of type %q";
      DbgTrace( msg:format( build_data.name, build_data.type ) );
    
    end;
    
    if ( reward ) then
    
      reward.category = "build";
      m_items.TryObtain( reward );
    
    end;
  
  end;

  mediator.Notify(
    "Model_Builds_BuildConstructed",
    {
      name = build_name,
      type = build_data.type
    }
  );

  return true;

end;
--------------------------------------------------------------------------------
function public.GetBuildData ( build_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "builds",
    indice     = "name",
    criteria   = { like = build_name }
  };

  local build_data = m_storage.Execute( query ).result;

  if ( build_data ) then

    public.AppendSavedData( build_data );

  end;

  return build_data;

end;
--------------------------------------------------------------------------------
function public.AppendSavedData ( build_data )

  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "builds",
    indice     = "name",
    criteria   = { like = build_data.name }
  };

  local saved_data = m_storage.Execute( query ).result;

  if ( saved_data ) then

    for key, value in pairs( saved_data ) do

      if ( not build_data[ key ] ) then

        build_data[ key ] = value;

      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.EnableBuildNext ( build_current )

  local current_build_data = public.GetBuildData( build_current );

  local stage_name = "stage_"..current_build_data.unlocklist.max;
  if ( current_build_data.unlocklist.type == "bonus" ) then
    stage_name = stage_name.."_bonus";
  end;

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "builds",
    indice     = "unlock",
    criteria   = { like = stage_name }
  };
  
  local next_build_data = m_storage.Execute( query ).result;

  if ( next_build_data ) then

    query =
    {
      query      = "update",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = next_build_data.name },
      set        =
      {
        --enabled = true,
        unlocked = false
      }
    };

    if ( not m_storage.Execute( query ).result ) then

      query =
      {
        query      = "insert",
        group      = "player",
        collection = "builds",
        value      = 
        {
          name      = next_build_data.name,
          --enabled   = true,
          unlocked  = false
        }
      };

      m_storage.Execute( query );

    end;

    return next_build_data.name;

  end;
  
  return nil;

end;
--------------------------------------------------------------------------------
function public.IsConstructed ( build_name )

  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "builds",
    indice     = "name",
    criteria   = { like = build_name }
  };

  local build = m_storage.Execute( query ).result;

  if ( build ) and ( build.constructed ) then

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.IsAutoConstructed ( build_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "builds",
    indice     = "name",
    criteria   = { like = build_name }
  };

  local build_data = m_storage.Execute( query ).result;
  
  return private.IsAutoConstructed( build_data );

end;
--------------------------------------------------------------------------------
function private.IsAutoConstructed ( build_data )

  if ( not build_data.price     ) and
     ( not build_data.fragments ) then

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.IsCompleted ( build_name )

  local data = public.GetBuildData( build_name );

  local puzzle   = ( data.type == "puzzle" );
  local island   = ( data.type == "island" );
  local unlocked = data.unlocked;
  
  if ( not unlocked ) or not ( puzzle or island ) then
  
    return false;
  
  end;

  local total = data.fragments;
  local frags = data.collected_fragments;
      
  if ( total and frags and ( #frags == #total ) ) then
  
    return true;
  
  end

  return false;

end;
--------------------------------------------------------------------------------
function private.Unlock ( build_name, unlocked )

  local query =
  {
    query      = "insert",
    group      = "player",
    collection = "builds",
    value      =
    {
      name        = build_name,
      unlocked    = unlocked,
      constructed = false
    }
  };

  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
function public.GetBuildsByTypeAndSection ( build_type, section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "builds",
    indice     = "type",
    criteria   = { like = build_type }
  };

  local builds = m_storage.Execute( query ).result;

  if not ( section_name ) then

    return builds;

  end;

  local builds_by_section = {};

  for i = 1, #builds do

    if ( builds[ i ].section == section_name ) then

      table.insert( builds_by_section, builds[ i ] );

    end;

  end;

  return builds_by_section;

end;
--------------------------------------------------------------------------------
function public.GetBuildUnlockedByStage ( stage_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "builds",
    indice     = "unlock",
    criteria   = { like = stage_name }
  };

  local builds = m_storage.Execute( query ).result or {};
  local build  = builds[ 1 ];
  
  if ( build ) then public.AppendSavedData( build ); end;
  
  return build;

end;
--------------------------------------------------------------------------------
function public.GetUnlockList ( build_data, construct )

  local stages_list = {};

  local unlocked    = build_data.unlocked or false;
  local constructed = build_data.constructed or construct;
  local build_type  = build_data.type;

  if ( build_data.first ) then

    stages_list.stages_previous =
    { 
      build_data.unlocklist.min - 11,
      build_data.unlocklist.min - 1 
    };

  end;

  if ( build_type == "island" ) or ( build_type == "puzzle" ) then

    if ( constructed and build_data.unlocklist ) then

      stages_list.stages_type = build_data.unlocklist.type;
      stages_list.stages_next =
      { 
        build_data.unlocklist.min,
        build_data.unlocklist.max
      };

    end;

  elseif ( unlocked and constructed and build_data.unlocklist ) then

    stages_list.stages_type = build_data.unlocklist.type;
    stages_list.stages_next =
    { 
      build_data.unlocklist.min,
      build_data.unlocklist.max
    };

  end;

  -- Cheater stages displaying hook
  if not ( stages_list.stages_next ) then

    local unlock_min   = build_data.unlocklist.min;
    local unlock_max   = build_data.unlocklist.max;
    local unlock_type  = build_data.unlocklist.type;

    local stage_name = "stage_"..unlock_max;

    if ( unlock_type == "bonus" ) then
      stage_name = stage_name.."_bonus"
    end;

    local stage_info = m_stage.GetSavedData( stage_name );

    if ( stage_info and stage_info.enabled ) then
      stages_list.stages_type = unlock_type;
      stages_list.stages_next = { unlock_min, unlock_max };
    end;

  end;

  return stages_list;

end;
--------------------------------------------------------------------------------