-- name=m_characters
--[[ Char states: 
  locked (for external usage), 
  unlocked, 
  active, 
  inactive
]]
--------------------------------------------------------------------------------
function public.Init ()

  mediator.Subscribe( "Model_Quests_QuestStarted",     private.TryActivateCharacterByQuest );
  mediator.Subscribe( "Model_Quests_QuestCompleted",   private.TryDeactivateCharacterByQuest );
  mediator.Subscribe( "Model_Builds_BuildConstructed", function ( params )
    private.TryUnlockCharacterByBuild( params.name );
  end );

  mediator.Subscribe( "Model_Quests_PregiveStarted",   function ( quest_name ) 

    local query =
    {
      query      = "find",
      group      = "static",
      collection = "characters",
      indice     = "quest_cage",
      criteria   = { like = quest_name }
    };
    
    local character = m_storage.Execute( query ).result;

    if ( not character ) then

      query =
      {
        query      = "find",
        group      = "static",
        collection = "characters",
        indice     = "quest_free",
        criteria   = { like = quest_name }
      };
      
      character = m_storage.Execute( query ).result;

    end;

    if ( character ) then

      mediator.Notify( "Model_Characters_PregiveStarted", character.name );

    end;

  end );

end;
--------------------------------------------------------------------------------
function public.GetCharacterState ( character_name )

  return private.GetCharacterState( character_name ) or "locked";

end;
--------------------------------------------------------------------------------
function public.IsFirstState ( character_name )

  local data = private.GetCharacterData( character_name );

  local has_qst_free = data.quest_free ~= nil;
  local has_qst_cage = data.quest_cage ~= nil;

  if   ( has_qst_free and has_qst_cage ) then

    local free_current  = private.GetQuestCurrent(  data.quest_free );
    local free_complete = private.GetQuestComplete( data.quest_free );

    if ( free_current or free_complete ) then return false; end;

  elseif ( has_qst_free and not has_qst_cage ) then

    return true;

  else

    DbgTrace( "[ MODEL ][ CHARACTERS ] Wrong quests for " .. character_name );

  end;

  return true;

end;
--------------------------------------------------------------------------------
function public.GetStageUnlocker ( build_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "unlock",
    criteria   = { like = build_name }
  };
  
  local character = m_storage.Execute( query ).result;

  if ( character ) then

    local quest = m_quests.GetQuestData( character.quest_cage or character.quest_free );

    if ( quest ) then

      return tonumber( string.match( quest.condition.name, "%d+" ) );

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.GetQuestByItemFromStage ( item_name, stage_name )

  local query = {
    query      = "find_all",
    group      = "static",
    collection = "quests",
    indice     = "target.name",
    criteria   = { like = item_name }
  };

  local quest_list = m_storage.Execute( query ).result;

  for q = 1, #quest_list do

    if ( cmn.IsEntityInList( quest_list[ q ].target.list, stage_name ) ) then

      local quest_reward = quest_list[ q ].reward;

      if ( quest_reward ) then

        for r = 1, #quest_reward do

          quest_reward[ r ].content = 
            m_content.GetContent( quest_reward[ r ].type, quest_reward[ r ].name );

        end;

      end;

      return quest_list[ q ];

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.IsQuestForItemFromStageDone ( item_name, stage_name )

  local quest = m_characters.GetQuestByItemFromStage( item_name, stage_name );

  if ( quest.reward ) then

    local query =
    {
      query      = "find",
      group      = "player",
      collection = "quests_completed",
      indice     = "name",
      criteria   = { like = quest.name }
    };
    
    local quest_completed = m_storage.Execute( query ).result;

    if ( quest_completed ) then

      return true;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.GetCharacterByItemFromStage ( item_name, stage_name )

  local target_quest = public.GetQuestByItemFromStage( item_name, stage_name ).name;

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "quest_cage",
    criteria   = { like = target_quest }
  };
  
  local character = m_storage.Execute( query ).result;

  if ( character ) and ( character.name ) then return character.name; end;

  query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "quest_free",
    criteria   = { like = target_quest }
  };
  
  return m_storage.Execute( query ).result.name;

end;
--------------------------------------------------------------------------------
function public.GetCharacterQuest ( character_name, check_pregive )

  local use_pregive_list = false;

  local character_data = private.GetCharacterData( character_name );

  local quest_cage = private.GetQuestCurrent( character_data.quest_cage or "" );
  local quest_free = private.GetQuestCurrent( character_data.quest_free or "" );

  local quest_current;

  if quest_cage then 

    if ( not check_pregive and ( not quest_cage.pregive )) then 
      quest_current = quest_cage;
    elseif ( check_pregive ) then
      use_pregive_list = true;
      quest_current = quest_cage;
    end;
  end;

  if ( not quest_current ) and quest_free then
    if ( not check_pregive and ( not quest_free.pregive )) then 
      quest_current = quest_free;
    elseif ( check_pregive ) then 
      use_pregive_list = true;
      quest_current = quest_free;
    end;
  end;

  if quest_current then

    local quest_data = m_quests.GetQuestData( quest_current.name );

    if ( not quest_current.count ) and ( not quest_current.list ) then      

      quest_current.list = {};

    end;

    local result = {
      name         = quest_current.name,
      count        = quest_current.count,
      list         = quest_current.list,
      target       = quest_data.target
    };

    if ( use_pregive_list and quest_data.target.pregive ) then

      result.target.list = quest_data.target.pregive;

    end;

    return result;

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function public.GetCharacterQuestByItem ( character_name, item_name )

  local character_data = private.GetCharacterData( character_name );

  local quest_cage_data = m_quests.GetQuestData( character_data.quest_cage or "" );
  local quest_free_data = m_quests.GetQuestData( character_data.quest_free or "" );

  local quest_name;
  local quest_data;
  local quest_current;

  if ( quest_cage_data ) and ( quest_cage_data.target.name == item_name ) then 

    quest_name = character_data.quest_cage;
    quest_data = quest_cage_data;

  elseif ( quest_free_data ) and ( quest_free_data.target.name == item_name ) then 

    quest_name = character_data.quest_free;
    quest_data = quest_free_data;

  end;

  local quest_current = private.GetQuestCurrent( quest_name );

  if ( not quest_current ) then

    quest_current = { name = quest_name };

    if ( quest_data.target.list ) then quest_current.list = quest_data.target.list; end;

    if ( quest_data.target.count ) then quest_current.count = quest_data.target.count; end;

  end;

  return {
    name    = quest_name,
    count   = quest_current.count,
    list    = quest_current.list,
    target  = quest_data.target
  };

end;
--------------------------------------------------------------------------------
function public.GetContent ( character_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "name",
    criteria   = { like = character_name }
  };

  return m_storage.Execute( query ).result.content;

end;
--------------------------------------------------------------------------------
function public.GetUnlockedType ( character_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "name",
    criteria   = { like = character_name }
  };

  return m_storage.Execute( query ).result.unlocked_type;

end;
--------------------------------------------------------------------------------
function public.Section_Enable ( section_name )

  local characters = 
  {
    section_1 = { "wizard" },
    section_2 = { "vanhelsinggirl" },
    section_3 = { "watergirl" },
    section_4 = { "aborigine" },
    section_5 = { "sailor", "quince" }
  };

  if ( characters[ section_name ] ) then

    for s = 1, #characters[ section_name ] do

      local char_name = characters[ section_name ][ s ];

      local state = private.GetCharacterState( char_name );

      if not ( state ) then

        local query =
        {
          query      = "update",
          group      = "player",
          collection = "characters",
          indice     = "name",
          criteria   = { like = char_name },
          set        =
          {
            state = "unlocked"
          }
        };

        if ( not m_storage.Execute( query ).result ) then

          query =
          {
            query      = "insert",
            group      = "player",
            collection = "characters",
            value      = 
            {
              name  = char_name,
              state = "unlocked"
            }
          };
          
          m_storage.Execute( query );

        end;

      end;  

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.GetMainEnemy ( character_name )

  local section_number = private.GetCharacterData( character_name ).section:match( "%d+" );

  local stage_name = "stage_"..math.ceil( 110 * section_number );

  local stage_data = m_stage.GetStaticData( stage_name );
  
  local enemy_name = stage_data.state[ 1 ].enemy.name;

  return enemy_name.."_"..stage_name;

end;
--------------------------------------------------------------------------------
function public.IsCharactersQuest ( quest_name )
  
  return public.GetCharacterByQuest( quest_name ) ~= nil;
  
end;
--------------------------------------------------------------------------------
function public.GetCharacterByQuest ( quest_name )

  local character = nil;

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "quest_cage",
    criteria   = { like = quest_name }
  };

  character = m_storage.Execute( query ).result;
  
  if ( not character ) then

    query =
    {
      query      = "find",
      group      = "static",
      collection = "characters",
      indice     = "quest_free",
      criteria   = { like = quest_name }
    };
    
    character = m_storage.Execute( query ).result;

  end;
  
  return character;

end;
--------------------------------------------------------------------------------
function public.GetCharacterByFragment ( fragment_name )

  local quest = m_quests.GetCurrentQuestNameByItem( "fragment", fragment_name );

  if ( quest ) then

    return public.GetCharacterByQuest( quest );

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function private.TryUnlockCharacterByBuild ( build_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "unlock",
    criteria   = { like = build_name }
  };
  
  local character = m_storage.Execute( query ).result;

  if ( character ) then

    private.SetCharacterState( character.name, "unlocked" );

  end;

end;
--------------------------------------------------------------------------------
function private.TryActivateCharacterByQuest ( quest_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "quest_cage",
    criteria   = { like = quest_name }
  };
  
  local character = m_storage.Execute( query ).result;

  if ( not character ) then

    query =
    {
      query      = "find",
      group      = "static",
      collection = "characters",
      indice     = "quest_free",
      criteria   = { like = quest_name }
    };
    
    character = m_storage.Execute( query ).result;

  end;

  if ( character ) then

    private.SetCharacterState( character.name, "active" );

  end;

end;
--------------------------------------------------------------------------------
function private.TryDeactivateCharacterByQuest ( quest_data )
  
  local character = public.GetCharacterByQuest( quest_data.name );
  
  if ( character ) then
  
    local deactivate = false;
    local quest_cage = character.quest_cage;
    local quest_free = character.quest_free;
    
    if ( quest_cage and quest_free ) then
    
      local quest_cage_done = m_quests.IsQuestComplete( quest_cage );
      local quest_free_done = m_quests.IsQuestComplete( quest_free );
      
      deactivate = quest_cage_done and quest_free_done;
      
    elseif ( quest_cage ) then
    
      deactivate = m_quests.IsQuestComplete( quest_cage );
    
    elseif ( quest_free ) then
    
      deactivate = m_quests.IsQuestComplete( quest_free );
    
    elseif ( IsEditor() ) then
    
      local msg = "Error! Can't check deactivation of %q by quest %q!";
      DbgTrace( msg:format( character.name, quest_data.name ) );
    
    end;
    
    if ( deactivate ) then
    
      private.SetCharacterState( character.name, "inactive" );
    
    end;
    
    -- DG: Characters quest complete event
    common.Kontagent_LogEvent( "quest", 
    {
      section = quest_data.section,
      name    = quest_data.name
    } );
  
  end;

end;
--------------------------------------------------------------------------------
function private.GetCharacterState ( name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "characters",
    indice     = "name",
    criteria   = { like = name }
  };
  
  local character = m_storage.Execute( query ).result;
  
  if ( character ) and ( character.state ) then
    
    return character.state;
    
  end;
  
  return false;

end;
--------------------------------------------------------------------------------
function private.SetCharacterState ( name, state )

  local query;
  
  if ( private.GetCharacterState( name ) ) then

    query =
    {
      query      = "update",
      group      = "player",
      collection = "characters",
      indice     = "name",
      criteria   = { like = name },
      set        = { state = state }
    };
    
    m_storage.Execute( query );

  else

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "characters",
      value      = 
      { 
        name     = name,
        state    = state 
      }
    };
    
    m_storage.Execute( query );

  end;

  mediator.Notify(
    "Model_Characters_CharacterStateChanged",
    { 
      name  = name,
      state = state
    }
  );

end;
--------------------------------------------------------------------------------
function private.GetQuestCurrent ( quest_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "quests_current",
    indice     = "name",
    criteria   = { like = quest_name }
  };
  
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function private.GetQuestComplete ( quest_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "quests_completed",
    indice     = "name",
    criteria   = { like = quest_name }
  };
  
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function private.GetCharacterData ( character_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "name",
    criteria   = { like = character_name }
  };
  
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.GetCharacterData ( character_name )

  return private.GetCharacterData( character_name );

end;
--------------------------------------------------------------------------------
function public.GetQuestPopupData ( quest_data )

  local quest_current = private.GetQuestCurrent( quest_data.name ) or 
                        { count = #quest_data.target.list };

  local data =
  {
    name    = quest_data.name,
    count   = quest_current.count,
    list    = quest_current.list or {},
    target  = quest_data.target,
    content = quest_data.content
  };
  
  return data;

end;
--------------------------------------------------------------------------------
function public.GetCurrentQuest ( character_name )

  local data = private.GetCharacterData( character_name ) or {};

  local quest_cage = private.GetQuestCurrent( data.quest_cage or "" );
  local quest_free = private.GetQuestCurrent( data.quest_free or "" );
  
  if     ( quest_cage ) then return m_quests.GetQuestData( quest_cage.name );
  elseif ( quest_free ) then return m_quests.GetQuestData( quest_free.name );
  end;
  
  return quest_cage or quest_free;

end;
--------------------------------------------------------------------------------
function public.GetCharacterByBuild ( build_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "characters",
    indice     = "unlock",
    criteria   = { like = build_name }
  };
  
  return m_storage.Execute( query ).result;

end;