-- name=m_chests
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ CHESTS ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.Trace( "Initialized." );

  private.section_chests = {};

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.List_OnSection ( section_name )

  local query =
  {
    query      = "find_all",
    group      = "static",
    collection = "chests",
    indice     = "section",
    criteria   = { like = section_name }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.List_Unlocked ( section_name )

  local query =
  {
    query      = "find_all",
    group      = "player",
    collection = "chests"
  };
  
  local unlocked_full_list  = m_storage.Execute( query ).result;
  local unlocked_on_section = {};
  
  for c, chest in pairs( unlocked_full_list ) do
  
    if ( private.Origin_Section( chest.name ) == section_name ) then
      
      table.insert( unlocked_on_section, chest );
      
    end;
    
  end;
  
  return unlocked_on_section;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Status ( chest_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "chests",
    indice     = "name",
    criteria   = { like = chest_name }
  };

  local chest = m_storage.Execute( query ).result;

  if ( chest and chest.opened ) then

    return "opened";

  end;

  if ( chest and not chest.opened ) then

    return "available";

  end;

  return "not_available";

end;
--------------------------------------------------------------------------------
function public.Content ( chest_name )
  
  return private.Content( chest_name );
  
end;
--------------------------------------------------------------------------------
function private.Content ( chest_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "chests",
    indice     = "name",
    criteria   = { like = chest_name }
  };

  local chest = m_storage.Execute( query ).result;

  if ( chest ) and ( chest.content ) then

    -- JK: check if we have fox bonus
    local bonus = m_hero.GetStats().fox;

    chest.content.count    = chest.content.count + bonus;
    chest.content.category = "gift";

    return chest.content;

  end;

  return false;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.TryUnlock ( stage_name )

  local chest_name = private.Origin_Stage( stage_name );
  
  if ( chest_name ) then

    private.Unlock( chest_name );

  end;

end;
--------------------------------------------------------------------------------
function public.Open ( chest_name )

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "chests",
    indice     = "name",
    criteria   = { like = chest_name },
    set        = { opened = true }
  };

  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
function private.Unlock( chest_name )

  if not ( chest_name ) then return; end;

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "chests",
    indice     = "name",
    criteria   = { like = chest_name }
  }
  
  if ( m_storage.Execute( query ).result ) then

    DbgTrace( "[ MODEL ][ CHESTS ][ ERROR ] Second unlock for " .. chest_name );
    return;

  end;
  
  query =
  {
    query      = "insert",
    group      = "player",
    collection = "chests",
    value      = {
      name   = chest_name,
      opened = false
    }
  };

  m_storage.Execute( query );

  local content = private.Content( chest_name );

  if ( content.type == "booster" ) then

    m_boosters.Unlock( content.name );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Origin_Stage ( stage_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "chests",
    indice     = "stage",
    criteria   = { like = stage_name }
  };

  local chest = m_storage.Execute( query ).result;

  if ( chest ) and ( chest.name ) then

    return chest.name;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Origin_Section ( chest_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "chests",
    indice     = "name",
    criteria   = { like = chest_name }
  };

  local chest = m_storage.Execute( query ).result;

  if ( chest ) and ( chest.section ) then

    return chest.section;

  end;

  return false;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetChestByStage ( stage_name )

  return private.Origin_Stage( stage_name );

end;
--------------------------------------------------------------------------------