-- name=m_client
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ CLIENT ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.list = {};

  private.subscribed = {};

  private.no_connection_counter = 0;

  private.is_online = nil;
  private.is_PC = IsPC();

  common.MessageSubscribe( Event_HeartBeat, private.ConnectionRestored );
  
  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
function public.Start ()
  
  common.MessageSubscribe( Event_Application_InternetError, private.ConnectionLost );  
  common.MessageSubscribe( Event_Content_Obsolete,      private.ContentObsolete );
  common.MessageSubscribe( Event_Storage_Conflict,      private.StorageConflict );
  common.MessageSubscribe( Event_Application_GainFocus, private.ApplicationGainFocus );
  common.MessageSubscribe( Event_Application_ClientVersionOutdated, private.ClientVersionOutdated );

end;
--------------------------------------------------------------------------------
function public.StartWaiting ( parent, event )

  private.Trace( "StartWaiting: "..parent.." -> "..event );
  mediator.Notify( "Model_Client_WaitingStarted" );

  private.waiting = true;

end;
--------------------------------------------------------------------------------
function public.StopWaiting ( parent, event )

  private.Trace( "StopWaiting: "..parent.." -> "..event );
  mediator.Notify( "Model_Client_WaitingFinished" );

  private.waiting = false;

end;
--------------------------------------------------------------------------------
function public.IsOnline ()

  return tern( private.is_online == nil )( loader.IsOnline(), private.is_online );

end;
--------------------------------------------------------------------------------
function public.NotifyOffline ()

  mediator.Notify( "Model_Client_ShowOfflineMode", { message = true } );

end;
--------------------------------------------------------------------------------
function public.WaitingForAnswer ( request_target, request_method, request_params, answer_handler, waiting_silent )

  table.insert( private.list,
  {
    request_target = request_target,
    request_method = request_method,
    request_params = request_params,
    answer_handler = answer_handler,
    waiting_silent = waiting_silent
  } );

  if ( not waiting_silent ) then

    mediator.Notify( "Model_Client_WaitingStarted" );

  end;

  local answer_message = private._GetAnswerMessageByRequest( request_target, request_method );

  if ( not private.subscribed[ answer_message ] ) then
    private.subscribed[ answer_message ] = true;
    common.MessageSubscribe( answer_message, private.OnWaitingFinished );
  end;

  private.SendRequest( request_target, request_method, request_params, 1 );

end;
--------------------------------------------------------------------------------
function private.OnWaitingFinished ( answer_message, answer_params )

  common.MessageUnsubscribe( answer_message, private.OnWaitingFinished );
  private.subscribed[ answer_message ] = false;

  for h = #private.list, 1, -1 do

    local element = private.list[ h ];

    if ( answer_message == private._GetAnswerMessageByRequest( element.request_target, element.request_method ) ) then

      if ( element.answer_handler ) then

        element.answer_handler( answer_message, answer_params );

      end;

      table.remove( private.list, h );

      if ( not element.waiting_silent ) then

        mediator.Notify( "Model_Client_WaitingFinished" );

      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.SendRequest ( request_target, request_method, request_params )

  NetInvokeServerMethod( private._FormatRequestString( request_target ), 
                         private._FormatRequestString( request_method ), request_params, 1 );

end;
--------------------------------------------------------------------------------
function private._GetAnswerMessageByRequest ( request_target, request_method )

  return "Server_"..request_target.."_on"..request_method;

end;
--------------------------------------------------------------------------------
function private._FormatRequestString ( request_string )

  return string.lower( string.sub( request_string, 1, 1 ) )..string.sub( request_string, 2 );

end;
--------------------------------------------------------------------------------
function private.ConnectionLost ( msg, params )

  private.Trace( "ConnectionLost" );

  if ( params and params.error == "ServerTemporarilyUnavailable" ) then

    private.Trace( "Event_Application_InternetError: ServerTemporarilyUnavailable!!!" );
    return;

  end;
  if ( params and params.error == "KickedByOthers" ) then

    private.Trace( "Event_Application_InternetError: KickedByOthers!!!" );
    private.PrioritySessionConflict();
    return;

  end;

  if private.no_connection_counter < 3 then

    private.no_connection_counter = private.no_connection_counter + 1;
    private.ResendRequests();
    parts.net.heartbeat.Restart();
    --private.Trace( "parts.net.heartbeat.Restart "..tostring(private.no_connection_counter) );

  else

    local message = false;
    for i = 1, #private.list do
      if ( not private.list[ i ].waiting_silent ) then
        message = true;
      end;
      private.list[ i ] = nil;
    end;
    private.no_connection_counter = 0;
    private.is_online = false;
    mediator.Notify( "Model_Client_ConnectionLost", { message = message } );

  end;

end;
--------------------------------------------------------------------------------
function private.ResendRequests ()

  for h = 1, #private.list, 1 do

    local element = private.list[ h ];

    private.SendRequest( element.request_target, element.request_method, element.request_params, 1 );

  end;

end;
--------------------------------------------------------------------------------
function private.ConnectionRestored ()

  if not ( public.IsOnline() ) then

    private.Trace( "ConnectionRestored" );
    private.is_online = true;
    private.no_connection_counter = 0;
    private.ResendRequests();
    mediator.Notify( "Model_Client_ConnectionRestored" );

  end;

end;
--------------------------------------------------------------------------------
function public.ConnectionReconnect ( force_reconnect )

  if ( force_reconnect or not public.IsOnline() ) then
  
    private.Trace( "ConnectionReconnect" );
    parts.net.heartbeat.Restart();
  
  end;

end;
--------------------------------------------------------------------------------
function public.CloseApp ()

  MsgSend( Command_OpenAppStore );
  wiz.Quit();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Section_SetAllowed ( section_name )

  local result, value = parts.storage.data.Read( "sml/allow_sections" );

  if ( value and value ~= "" and value ~= "false" and value ~= "true" ) then
    
    local section_number = tonumber( section_name:match( "%d+" ) );

    if ( tonumber( value ) < section_number ) then
      
      parts.storage.data.Write( "sml/allow_sections", section_number );
      
    end;

  end;

end;
--------------------------------------------------------------------------------
function public.IsContentObsolete ( section_name ) 

  if not ( section_name ) then
  
    local packets = ContentGetPacketList();
  
    local result, allow_sections = parts.storage.data.Read( "sml/allow_sections" );
    
    -- NOTE: 0 is OK reult
    if ( result ~= 0 ) then
    
      allow_sections = private.Content_SectionCount();
      parts.storage.data.Write( "sml/allow_sections", allow_sections );
    
    end;

    for p = #packets, 1, - 1 do

      local packet = packets[ p ];
      
      local packet_number = tonumber( packet.name:match( "_(%d+)" ) ) or 0;
      
      if ( packet_number > tonumber( allow_sections ) ) then
        
        private.Trace( "[ CLIENT ] IsContentObsolete: exclude < "..packet.name.." >." );
         
        table.remove( packets, p );
        
      end;

    end;

    for p = 1, #packets do

      local packet = packets[ p ];

      if ( packet.state and packet.state == "downloading" ) then

        return packet;

      end;

    end;

  else

    if ( not ContentIsPacketReady( section_name ) ) then

      private.Trace( "[ CLIENT ] IsContentObsolete: find < "..section_name.." > to download." );

      return section_name;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.ContentObsolete ()

  if ( public.IsContentObsolete() ) then

    mediator.Notify( "Model_Client_ContentObsolete" );

  end;

end;
--------------------------------------------------------------------------------
function private.Content_SectionCount ()
  
  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "sections"
  };
  
  local sections = m_storage.Execute( query ).result or {};
  
  local count = 0;
  
  for _, section in pairs( sections ) do
  
    if ( section.enabled ) then
      count = count + 1;
    end;
  
  end;
  
  return count;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.StorageConflict ( msg, params )

  mediator.Notify( "Model_Client_StorageConflict", params );

end;
--------------------------------------------------------------------------------
function private.PrioritySessionConflict ()

  mediator.Notify( "Model_Client_PrioritySessionConflict" );

end;
--------------------------------------------------------------------------------
function private.ApplicationGainFocus ()

  if ( private.waiting ) then return; end;

  mediator.Notify( "Model_Client_ApplicationGainFocus" );

end;
--------------------------------------------------------------------------------
function private.ClientVersionOutdated ()

  mediator.Notify( "Model_Client_ClientVersionOutdated" );

end;
--------------------------------------------------------------------------------
-- function TEST () end;
--------------------------------------------------------------------------------
function public.TestPartsMessage ()

  if ( not private.test ) then
    
    --private.ConnectionLost( "Event_Application_InternetError", { error = "KickedByOthers" } );
    --private.ConnectionLost( "Event_Application_InternetError", { error = "ServerTemporarilyUnavailable" } );
    --private.StorageConflict();
    --private.ClientVersionOutdated();
   
    private.test = true; 
    
  end;
  
end; 
--------------------------------------------------------------------------------