-- name=m_dailyquest
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ DAILYQUEST ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.data                 = m_storage.Settings( "dailyquest" );
  private.states               = { "open", "beaten", "done" };
  private.data.challenge_count = #private.data.rewards;

  private.tokens = {
    dailyquest = "mte_dailyquest_refresh",
    countdown  = "mte_dailyquest_countdown_refresh",
    challenge  = "mte_dailyquest_challenge_refresh"
  };

  private.events = {
    dailyquest = "Event_DailyQuest_Refresh",
    countdown  = "Event_DailyQuest_CountdownRefresh",
    challenge  = "Event_DailyQuest_ChallengeRefresh"
  };

  common.MessageSubscribe(
    private.events.dailyquest,
    private.Refresh
  );

  common.MessageSubscribe(
    private.events.countdown,
    function ()

      private.Unschedule( "countdown" );

    end
  );

  common.MessageSubscribe(
    private.events.challenge,
    function () 

      private.Unschedule( "challenge" );

    end
  );

  if ( private.Unlocked() ) then

    if ( not private.IsScheduled( "dailyquest" ) ) then

      private.Schedule( "dailyquest" );

    end;

  else

    mediator.Subscribe(
      "Model_Stage_StageWon",
      private.Countdown_TryStart
    );

  end;

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Get ( item )
  
  return private.Data_Get( item );

end;
--------------------------------------------------------------------------------
function public.Cheater_Refresh ()

  if ( private.IsScheduled( "challenge" ) ) then

    private.Unschedule( "challenge" );
    
    return;

  end;

  if not ( public.IsReady() ) then

    private.Refresh();

  end;

end;
--------------------------------------------------------------------------------
function public.IsReady ()

  return private.Data_Get( "ready" );

end;
--------------------------------------------------------------------------------
function public.GetRemainingTime ( timer_type )

  return private.GetRemainingTime( timer_type );

end;
--------------------------------------------------------------------------------
function public.Challenge_Count ()

  return private.data.challenge_count;

end;
--------------------------------------------------------------------------------
function public.Enemies ()

  return private.data.enemies;

end;
--------------------------------------------------------------------------------
function public.States ()

  return private.states;

end;
--------------------------------------------------------------------------------
function public.GetSkipPrice ()

  return private.data.skipprice;

end;
--------------------------------------------------------------------------------
function public.CooldownSkip ( timer_type )
  
  if ( timer_type == "dailyquest" ) then
    
    if ( m_player.TryChangeGoldCount( - private.data.skipprice ) ) then

      common.Kontagent_LogEvent( "economy", 
      {
        description = "premium_currency_spend",
        category    = "dailyquest",
        value       = - private.data.skipprice,
        name        = "skip_cooldown"
      } );
      
      private.Refresh();

    end;
    
    return;
    
  end;

  private.Unschedule( timer_type );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Challenge_IsSheduled ()

  return private.IsScheduled( "challenge" );

end;
--------------------------------------------------------------------------------
function public.Challenge_GetOpenedCount ()

  return private.Challenge_GetOpenedCount();

end;
--------------------------------------------------------------------------------
function public.Challenge_GetCompletedCount ()

  return private.Challenge_GetCompletedCount();

end;
--------------------------------------------------------------------------------
function public.Challenge_SetBeaten ( challenge_id )
  
  if ( private.Challenge_SetState( challenge_id, 2 ) ) then

    -- JK: check if this challenge was last
    local completed_count = private.Challenge_GetCompletedCount();
    
    common.Kontagent_LogEvent( "arena", 
    {
      name  = "success",
      level = completed_count
    } );

    if ( completed_count == private.data.challenge_count ) then

      private.Finish();

    end;

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Reward_Get ()

  return private.Reward_Get();

end;
--------------------------------------------------------------------------------
function public.Reward_Activate ()

  local index = private.Challenge_GetCompletedCount();

  private.Reward_Set( index );

  private.Schedule( "challenge" );

end;
--------------------------------------------------------------------------------
function public.Reward_Deactivate ()

  private.Reward_Set( false );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Countdown_IsActive ()

  return private.IsScheduled( "countdown" );

end;
--------------------------------------------------------------------------------
function public.Countdown_IsNeeded ()

  return ( not private.Unlocked() );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Unlocked ()

  return m_stage.GetSavedData( private.data.unlockstage ).completed;

end;
--------------------------------------------------------------------------------
function private.Countdown_TryStart ( stage_data )

  if ( private.data.unlockstage == stage_data.name ) then

    mediator.Unsubscribe(
      "Model_Stage_StageWon",
      private.Countdown_TryStart
    );

    private.Schedule( "countdown" );

    private.Refresh();

    mediator.Notify( "Model_DailyQuest_CountdownStarted" );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Schedule ( timer_type )

  local time;

  if ( timer_type == "dailyquest" ) then

    local time_def   = { tm_hour = 0, tm_min = 0, tm_sec = 0 };
    local time_now   = parts.time.server.Now() or time_def;
    local hour_end   = private.data.dailyhour;
    local hour_delta = hour_end - time_now.tm_hour;

    if ( hour_delta <= 0 ) then
      
      hour_delta = 24 - time_now.tm_hour + hour_end;
      
    end;

    time = hour_delta * 60 * 60 - ( time_now.tm_min ) * 60 - ( time_now.tm_sec );

  elseif ( timer_type == "countdown" ) then
    
    time = private.data.unlocktime;

  elseif ( timer_type == "challenge" ) then
    
    local reward = private.Reward_Get();
 
    if ( reward ) then

      time = reward.cooldown;

    end;

  else

    DbgTrace( "[ DQ ][ ERROR ] Unknown timer:  " .. tostring( timer_type ) );

  end;

  if ( not time ) then

    return;

  end;

  parts.time.event.Shedule( 
    private.tokens[ timer_type ], 
    "GLOBAL_TIME", 
    time, 
    private.events[ timer_type ], 
    {} 
  );

  mediator.Notify( "Model_DailyQuest_EventScheduled" );

end;
--------------------------------------------------------------------------------
function private.IsScheduled ( timer_type )

  return parts.time.event.Exists( private.tokens[ timer_type ] );

end;
--------------------------------------------------------------------------------
function private.Unschedule ( timer_type, skip_notification )

  parts.time.event.Finish();
  parts.time.event.Unshedule( private.tokens[ timer_type ] );

  local notification =
  {
    challenge = "Model_DailyQuest_ChallengeCooldownEnded",
    countdown = "Model_DailyQuest_CountdownEnded"
  };

  if ( notification[ timer_type ] ) and
     ( not skip_notification      )then

    mediator.Notify( notification[ timer_type ] );

  end;

end;
--------------------------------------------------------------------------------
function private.GetRemainingTime ( timer_type )

  if ( private.IsScheduled( timer_type ) ) then

    return parts.time.event.GetRemainingTime( private.tokens[ timer_type ] );

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function private.Refresh ()

  if ( private.Unlocked() ) then

    private.Unschedule( "dailyquest" );
    private.Start();
    private.Schedule( "dailyquest" );

    mediator.Notify( "Model_DailyQuest_Ready" );

  else

    -- JK: delete save does not clear timers
    private.Unschedule( "dailyquest", true );

  end;

end;

--------------------------------------------------------------------------------
function private.Start ()

  local stage_ban_list = {};
  local enemy_ban_list = {};

  local dailyquest = {
    challenge1 = private.Challenge_Create( stage_ban_list, enemy_ban_list ),
    challenge2 = private.Challenge_Create( stage_ban_list, enemy_ban_list ),
    challenge3 = private.Challenge_Create( stage_ban_list, enemy_ban_list ),
    challenge4 = private.Challenge_Create( stage_ban_list, enemy_ban_list ),
    ready      = true
  };

  private.Data_Set( dailyquest );

end;
--------------------------------------------------------------------------------
function private.Finish ()

  if ( not public.IsReady() ) then

    DbgTrace( "[ ERROR ][ DAILYQUEST ] Repeat reward get" );

    return false;

  end;

  return private.Ready_Set( false );

end;
--------------------------------------------------------------------------------
function private.Data_Get ( item )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "dailyquest"
  };
  
  local dailyquest = m_storage.Execute( query ).result;

  if ( item ) then
  
    return dailyquest[ item ];
  
  end;
  
  return dailyquest;

end;
--------------------------------------------------------------------------------
function private.Data_Set ( data )

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "dailyquest",
    set        = data
  };
  
  if ( not m_storage.Execute( query ).result ) then

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "dailyquest",
      value      = data
    };
    
    m_storage.Execute( query );

  end;

end;
--------------------------------------------------------------------------------
function private.Ready_Set ( is_ready )

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "dailyquest",
    set        = { ready = is_ready }
  };
  
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Challenge_SetState ( challenge_num, state )

  local challenge = "challenge"..challenge_num;

  local update_set = {
    [ challenge ] = private.Data_Get( challenge )
  };

  update_set[ challenge ].state = state;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "dailyquest",
    set        = update_set
  };
  
  return m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
function private.Challenge_Create ( stage_ban_list, enemy_ban_list ) 

  local challenge = {};

  local enemy = private.Challenge_CreateUniqueObject(
    private.Enemies(),
    enemy_ban_list
  );

  table.insert( enemy_ban_list, enemy.name );

  challenge.enemy = {
    name   = enemy.name;
    health = math.random( enemy.health.min, enemy.health.max );
    attack = math.random( enemy.attack.min, enemy.attack.max );
  };

  local stage = private.Challenge_CreateUniqueObject(
    private.Stages(),
    stage_ban_list
  );

  table.insert( stage_ban_list, stage.name );
  challenge.stage = stage.name;

  challenge.state = private.states[ 1 ];

  return challenge;

end;
--------------------------------------------------------------------------------
function private.Challenge_CreateUniqueObject ( list, ban_list ) 

  local ok = false;
  local obj;

  while ( not ok ) do
    obj = list[ math.random( #list ) ];
    if ( not cmn.IsEntityInList( ban_list, obj.name ) ) then
      ok = true;
    end;
  end;

  return obj;

end;
--------------------------------------------------------------------------------
function private.Challenge_GetCompletedCount ()

  return ( private.data.challenge_count - private.Challenge_GetOpenedCount() );

end;
--------------------------------------------------------------------------------
function private.Challenge_GetOpenedCount ()

  local open_count = 0;
  local challenges = private.Data_Get();
  
  for _, challenge in pairs( challenges ) do

    if ( type( challenge ) == "table" ) and
       ( challenge.state   == "open"  ) then
      
      open_count = open_count + 1;
      
    end;
    
  end;
    
  return open_count;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Reward_Get ()

  local reward_id = private.Data_Get( "reward" );

  if ( not reward_id ) then

    -- JK: show next reward
    reward_id = private.Challenge_GetCompletedCount() + 1;

  end;

  local reward = private.data.rewards[ reward_id ];

  if ( reward ) then    

    return reward;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Reward_Set ( index )

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "dailyquest",
    set        = { reward = index }
  };
  
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Enemies () 

  local by_section = true;

  local enemies = private.data.enemies;

  if ( by_section ) then

    local sections = m_map.Section_GetList();
    local result   = {};

    for e = 1, #enemies do
      for s = 1, #sections do
        if ( enemies[ e ].section == sections[ s ].name ) then
          if ( sections[ s ].enabled ) then
            table.insert( result, enemies[ e ] );
          end;
          break;
        end;
      end;
    end;

    return result;

  else

    return enemies;

  end;

end;
--------------------------------------------------------------------------------
function private.Stages ()

  local query =
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "type",
    criteria   = { like = "dailyquest" }
  };
  
  local stages = m_storage.Execute( query ).result;
  
  for s = #stages, 1, - 1 do
    
    local stage_data = m_stage.GetSavedData( stages[ s ].unlock ) or {};

    if ( not stage_data.completed ) then
       
      table.remove( stages, s );
      
    end;
    
  end;

  return stages;
  
end;
--------------------------------------------------------------------------------