-- name=m_email
-- Model_Email_UpdatedMessageCount
-- Model_Email_MessageListUpdated ( messages )
-- Model_Email_MessageAdded ( message )
-- Model_Email_MessageRemoved ( message )
--------------------------------------------------------------------------------
-- Fields description
-- - messageId
-- - text
-- - attach
-- - expDate - array
-- - subject
-- - from
--     - name
--     - level
--     - avatar
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ EMAIL ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.messageCount          = 0;
  private.messages              = nil;
  private.msgSection            = "message";
  private.need_request_messages = false;
  private.fake_email            = false;

end;
--------------------------------------------------------------------------------
function public.Start ( is_online )

  private.userId = tonumber( m_player.GetUserId() );

  private.updateMessagesToken = "tmr_mail_update_messages";
  private.updateMessagesEvent = "Event_Mail_UpdateMessages";
  
  mediator.Subscribe( "Model_Client_ConnectionLost", function ()

    private.messages = nil;
    private.fake_email = false;
    private.update_time = 10;
    private.RestartUpdateMessages();

  end );
  
  mediator.Subscribe( "Model_Client_ConnectionRestored", function ()

    private.userId = tonumber( m_player.GetUserId() );
    private.Request_Update();

  end );

  if ( is_online ) then
    private.Request_Update();
  end;

end;
--------------------------------------------------------------------------------
function public.GetUnreadMessageCount ()

  local message_count = private.messageCount;
  local view_message_count = private.GetViewMessageCount();
  
  if ( private.fake_email ) then
    message_count = message_count + 1;
  end;
  
  if ( message_count > view_message_count ) then
    message_count = view_message_count;
  end;

  return message_count;

end;
--------------------------------------------------------------------------------
function public.StopUpdateMessages ()

  private.RestartUpdateMessages();
  private.need_request_messages = false;

end;
--------------------------------------------------------------------------------
function public.WaitFor_Messages ( func_OnAnswer )

  private.need_request_messages = true;
  private.Request_Update( func_OnAnswer );
  private.need_request_messages = false;

end;
--------------------------------------------------------------------------------
function private.Request_Update ( func_OnAnswer )

  private.StopUpdateMessages();
  private.update_time = nil;

  if ( not m_client.IsOnline() ) then
    private.SheduleUpdateMessages();

  elseif ( private.need_request_messages and 
           private.GetViewMessageCount() > private.GetCurrentMessageCount() ) then
    private.Request_GetMessages( func_OnAnswer );

  else
    mediator.Notify( "Model_Email_MessageListUpdated", private.messages_to_view or {} );
    private.Request_GetMessageCount( func_OnAnswer );

  end;

  private.FakeEmail_Add();

end;
--------------------------------------------------------------------------------
function public.SendMessage ( userId, senderId, text, attach )

  m_client.WaitingForAnswer( 
    private.msgSection, 
    "SendMessage", 
    { 
      userId   = userId,
      senderId = senderId,
      text     = text,
      attach   = attach
    }, 
    function () end, 
    true
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Request_GetMessageCount ( func_OnAnswer )

  m_client.WaitingForAnswer( private.msgSection, "GetUnreadMessagesCount", 
    { userId = private.userId }, 
    function ( ... )
      private.OnGetUnreadMessageCount( ... );
      if ( func_OnAnswer ) then
        func_OnAnswer();
      end;
    end, 
    true );
  private.SheduleUpdateMessages();

end;
--------------------------------------------------------------------------------
function private.OnGetUnreadMessageCount ( msg, args )
  
  private.Set_UnreadMessageCount( args.count );
  private.RestartUpdateMessages();

end;
--------------------------------------------------------------------------------
function private.Set_UnreadMessageCount ( count )
  
  count = tonumber( count );
  
  if ( private.messageCount ~= count ) then
    
    private.messageCount = count;

    mediator.Notify( "Model_Email_UpdatedMessageCount" );
    private.Trace( "Notify: Model_Email_UpdatedMessageCount count = "..count );

  end;

end;
--------------------------------------------------------------------------------
function private.SheduleUpdateMessages ()
  
  if ( not private.update_time ) then

    private.update_time = m_storage.Settings( "email_settings" ).update_time;

  end;

  common.MessageSubscribe(
    private.updateMessagesEvent,
    private.OnStartUpdateMessages
  );
  
  TimeSheduleEvent(
    private.updateMessagesToken,
    "SESSION_GAME_TIME",
    private.update_time,
    private.updateMessagesEvent,
    {}
  );

end;
--------------------------------------------------------------------------------
function private.StopUpdateMessages ()

  TimeUnsheduleEvent( private.updateMessagesToken );

end;
--------------------------------------------------------------------------------
function private.RestartUpdateMessages ()

  private.StopUpdateMessages();
  private.SheduleUpdateMessages();

end;
--------------------------------------------------------------------------------
function private.OnStartUpdateMessages ()

  common.MessageUnsubscribe(
    private.updateMessagesEvent,
    private.OnStartUpdateMessages
  );

  TimeFinishEvent();

  private.Request_Update();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Request_GetMessages ( func_OnAnswer )

  local viewMessageCount = private.GetViewMessageCount();
  local currentMessageCount = private.GetCurrentMessageCount();

  local params = {
    userId = private.userId,
    limit = viewMessageCount * 2 - currentMessageCount,
    startMessageId = private.GetMaxMessageId()
  };

  local silent = not func_OnAnswer; --not private.need_request_messages;
  private.Trace( "Request_GetMessages limit="..(params.limit).." start with "..(params.startMessageId).." silent="..tostring(silent)  );
  m_client.WaitingForAnswer( private.msgSection, "GetMessagesWithParams", params, 
    function ( ... )
      private.OnGetMessages( ... );
      if ( func_OnAnswer ) then
        func_OnAnswer();
      end;
    end, 
    silent 
  );

end;
--------------------------------------------------------------------------------
function private.OnGetMessages ( msg, args, fake )

  private.Trace( "OnGetMessages" );

  local msgArray = args.messages;
  if ( not private.messages ) then
    private.messages = {};
  end;
  
  for key, msgInfo in pairs( msgArray ) do
    if ( not cmn.IsEntityInList( private.messages, msgInfo, "messageId" ) ) then
      table.insert( private.messages, msgInfo );
    end;
  end;

  private.messages_to_view = {};
  for i = 1, #private.messages do
    if ( i <= private.GetViewMessageCount() ) then
      table.insert( private.messages_to_view, private.messages[ i ] );
      private.Trace( "Notify: Model_Email_MessageListUpdated -> messageId: "..private.messages[ i ].messageId );
    else
      break;
    end;
  end;

  private.PrintMessages();
  mediator.Notify( "Model_Email_MessageListUpdated", private.messages_to_view );
  
  if ( not fake ) then
    private.Set_UnreadMessageCount( #msgArray );
  end;

  private.RestartUpdateMessages();

  --TODO Внимательно посмотреть на эту строчку. Мб тут можно выставить false
  private.need_request_messages = true;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.WaitFor_ReadMessage ( messageId )

  local msg = private.GetMessageById( messageId );

  if ( msg ) then

    private.Request_ReadMessage( messageId );

  end;
  
end;
--------------------------------------------------------------------------------
function private.Request_ReadMessage ( messageId )
  
  -- Make fake email readed.
  if ( private.FakeEmail_Read( messageId ) ) then
    return;
  end;
  
  private.Trace( "Request_ReadMessage messageId="..messageId );
  local params = { msgId = messageId };
  if ( private.GetCurrentMessageCount() <= private.GetViewMessageCount() ) then

    params.userId = private.userId;
    params.limit = private.GetViewMessageCount() * 2 - private.GetCurrentMessageCount();
    params.startMessageId = private.GetMaxMessageId();
    private.Trace( "Request_ReadMessage get "..(params.limit).." start with "..(params.startMessageId) );

  end;

  m_client.WaitingForAnswer( private.msgSection, "ReadMessage", params, private.OnReadMessage, true );

end;
--------------------------------------------------------------------------------
function private.OnReadMessage ( msg, args )

  local message = private.GetMessageById( args.msgId );

  if ( message ) then

    private.ApplyAttach( message.attach );

    local msgIdx = private.GetMessageIdx( args.msgId );
    local removedMsg = table.remove( private.messages, msgIdx );
    mediator.Notify( "Model_Email_MessageRemoved", removedMsg );
    private.Trace( "Notify: Model_Email_MessageRemoved messageId="..(removedMsg.messageId) );

    for i = 1, #private.messages_to_view do
      if private.messages_to_view[ i ].messageId == args.msgId then
        table.remove( private.messages_to_view, i );
        break;
      end;
    end

  end;

  if ( tonumber( args.msgId ) ~= 0 ) then
    private.Set_UnreadMessageCount( private.messageCount - 1 );
  end;

  if ( args.messages ) then

    for key, new_message in pairs( args.messages ) do

      local msg = private.GetMessageById( new_message.messageId );
      if ( not msg ) then
        table.insert( private.messages, new_message );
        private.Trace( "OnReadMessage: message added "..(new_message.messageId));
      end;

    end;

  end;

  if ( #private.messages_to_view < private.GetViewMessageCount() ) then

    local need_count = private.GetViewMessageCount() - #private.messages_to_view;

    for i = 1, need_count do

      local message = nil;
      local all = private.messages;
      local view = private.messages_to_view;

      for j = 1, #private.messages do

        local found = false;
        for k = 1, #private.messages_to_view do

          if private.messages[ j ].messageId == private.messages_to_view[ k ].messageId then
            found = true;
            break;
          end;

        end;

        if( not found ) then
          message = private.messages[ j ];
          break;
        end;

      end;
      if ( message ) then
        table.insert( private.messages_to_view, message );
        mediator.Notify( "Model_Email_MessageAdded", message );
        private.Trace( "Notify: Model_Email_MessageAdded "..(message.messageId));
      else
        break;
      end;

    end;

  end;

  private.RestartUpdateMessages();
  private.PrintMessages();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.ApplyAttach ( attachments )

  if ( not attachments ) then

    return;

  end;

  for a = 1, #attachments, 1 do

    local attachment = attachments[ a ];

    if ( attachment.type == "heart" ) then

      m_player.TryChangeHeartsCount( attachment.count );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.PrintMessages () 

  --[[private.Trace( "PrintMessages" );
  for idx = 1, #private.messages do  
     private.Trace( "Message id = "..private.messages[ idx ].messageId..
                    " text = "..private.messages[ idx ].text..
                    " attach = "..tostring( private.messages[ idx ].attach )..
                    " subject = "..private.messages[ idx ].subject..
                    " expDate = "..tostring( private.messages[ idx ].expDate ) );
  end;]]

end;
--------------------------------------------------------------------------------
function private.GetMessageById ( messageId )

  if ( not private.messages ) then
  
    return nil;
    
  end;

  for idx = 1, #private.messages do  
    if ( tonumber( private.messages[ idx ].messageId ) == tonumber( messageId ) ) then
      return private.messages[ idx ];
    end;
  end;
  return nil;

end;
--------------------------------------------------------------------------------
function private.GetMessageIdx ( messageId )

  if ( not private.messages ) then
  
    return nil;
    
  end;

  for idx = 1, #private.messages do  
    if ( tonumber( private.messages[ idx ].messageId ) == tonumber( messageId ) ) then
      return idx;
    end;
  end;
  return nil;

end;
--------------------------------------------------------------------------------
function private.GetMaxMessageId ()

  local maxMessageId = 0;
  if ( private.messages ) then
    for idx = 1, #private.messages do  
      if ( tonumber( private.messages[ idx ].messageId ) > tonumber( maxMessageId ) ) then
        maxMessageId = private.messages[ idx ].messageId;
      end;
    end;
  end;
  return maxMessageId;

end;
--------------------------------------------------------------------------------
function private.GetCurrentMessageCount ()

  return #( private.messages or {} );

end;
--------------------------------------------------------------------------------
function private.GetViewMessageCount ()

  return m_storage.Settings( "email_settings" ).message_count;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.FakeEmail_Add ( msgArray )

  if not ( m_storage.Player.Info().isemailed ) and 
     not ( private.fake_email ) then
     
    private.fake_email = true;
      
    local fake_email = {
      text = { string = "", params = {} },
      attach = {
        name = "gold",
        type = "gold",
        count = 15
      },
      created = "",
      subject = "",
      messageId = "0",
      from = {
        name = "Administrator",
        avatar = "",
        level = 0,
        userId = 1
      },
      expDate = {
        fraction = 0,
        hour = 0,
        error_count = 0,
        minute = 0,
        is_localtime = false,
        year = 0,
        month = 0,
        warnings = {},
        day = 0,
        warning_count = 0,
        errors = {},
        second = 0
      }
    };

    private.OnGetMessages( "Fake_Add", { messages = { fake_email } }, true );
    
  end;

end;
--------------------------------------------------------------------------------
function private.FakeEmail_Read ( messageId )

  if ( tonumber( messageId ) == 0 ) and 
     not ( m_storage.Player.Info().isemailed ) then

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        =
      {
        isemailed = true
      }
    };

    if not ( m_storage.Execute( query ).result ) then

      query =
      {
        query      = "insert",
        group      = "player",
        collection = "info",
        value      = 
        {
          isemailed = true
        }
      };

      m_storage.Execute( query );

    end;
    
    private.fake_email = false;
    
    private.OnReadMessage( "", { msgId = messageId } );
    
    return true;

  end;
  
  return false;

end;
--------------------------------------------------------------------------------