-- name=m_fights
--------------------------------------------------------------------------------
function  public.Init ()

  private.Trace( "Initialized." );
  
  mediator.Subscribe( "Model_Builds_BuildConstructed", private.IsVisible );

  private.bosses = { 
    reddragon      = { enemy = "reddragon", stage = "stage_110" }, 
    reddragongolem = { enemy = "reddragon", stage = "stage_110" }, 
    banshee        = { enemy = "banshee",   stage = "stage_220" }, 
    trollking      = { enemy = "trollking", stage = "stage_330" },
    naga           = { enemy = "naga",      stage = "stage_440" }
  };

end;
--------------------------------------------------------------------------------
function  public.GetStageUnlocker ( stages_list, stages_type )

  for s = stages_list[ 1 ], stages_list[ 2 ] do

    local stage_name = "stage_"..s;

    if not ( stages_type == "common" ) then

      stage_name = "stage_"..s.."_"..stages_type;

    end;

    if ( private.IsFight( stage_name ) ) then

      return s;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function  public.Section_Enable ( section_name )

  local enemies = 
  {
    section_1 = { "stage_6",   "stage_110" },
    section_2 = { "stage_116", "stage_220" },
    section_3 = { "stage_226", "stage_330" },
    section_4 = { "stage_336", "stage_440" },
    section_5 = { "stage_446" }
  };

  if ( enemies[ section_name ] ) then

    for s = 1, #enemies[ section_name ] do

      local fight_name = enemies[ section_name ][ s ];

      local visible = public.Data( fight_name ).visible;

      if not ( visible ) then

        local query =
        {
          query      = "update",
          group      = "player",
          collection = "stages",
          indice     = "name",
          criteria   = { like = fight_name },
          set        =
          {
            visible  = true
          }
        };

        if ( not m_storage.Execute( query ).result ) then

          query =
          {
            query      = "insert",
            group      = "player",
            collection = "stages",
            value      = 
            {
              name        = fight_name,
              visible     = true,
              enabled     = false,
              completed   = false,
              state       = 1
            }
          };
          
          m_storage.Execute( query );

        end;
      
      end;  

    end;

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Enemy_SetAbsent ( enemy_name, absent )

  local fight_data = private.bosses[ enemy_name ] or nil;

  if ( fight_data ) then

    if ( absent == public.Data( fight_data.stage ).absent ) then

      return;

    end;

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "stages",
      indice     = "name",
      criteria   = { like = fight_data.stage },
      set        =
      {
        absent  = absent
      }
    };

    m_storage.Execute( query );;

  end;

end;
--------------------------------------------------------------------------------
function  public.Enemy_GetAbsent ( enemy_name )

  local fight_data = private.bosses[ enemy_name ] or nil;

  if ( fight_data ) then

    return { name = fight_data.stage, enemy = fight_data.enemy, absent = public.Data( fight_data.stage ).absent };

  end;

end;
--------------------------------------------------------------------------------
function  public.Enemy_IsBoss ( enemy_name )

  if ( private.bosses[ enemy_name ] ) then

    return private.bosses[ enemy_name ].enemy;

  end;
  
  for key, value in pairs( private.bosses ) do
    
    if ( enemy_name:find( key ) ) then
      
      return private.bosses[ enemy_name ].enemy;
      
    end;
    
  end;

  return false;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Data ( fight_name )

  local fight_data = m_stage.GetSavedData( fight_name );
  local stage_data = m_stage.GetStaticData( fight_name, fight_data );

  fight_data = fight_data or {};

  fight_data.name     = fight_name;
  fight_data.type     = stage_data.type;
  fight_data.realtype = stage_data.realtype;

  fight_data.state = fight_data.state or 1; -- FIXME: always 1?

  if ( fight_data.realtype == "misc" ) then

    fight_data.state = 4;

  end;

  local enemy = stage_data.state[ fight_data.state ].enemy;
  if ( enemy ) then

    fight_data.enemy = enemy.name;
    fight_data.content = enemy.content;
    
  end;
  
  return fight_data;

end;
--------------------------------------------------------------------------------
function  public.IsBossFight ( stage_name )
  
  for k, v in pairs( private.bosses ) do
    
    if ( v.stage == stage_name ) then
      
      return true;
      
    end;
    
  end;
  
  return false;
  
end;
--------------------------------------------------------------------------------
function private.IsVisible ( build_params )

  local build_data = m_builds.GetBuildData( build_params.name );

  if ( build_data ) and ( build_data.unlocklist ) then

    local fight_name = private.Zone_Fight( build_data.unlocklist );

    if ( fight_name ) then

      local query =
      {
        query      = "update",
        group      = "player",
        collection = "stages",
        indice     = "name",
        criteria   = { like = fight_name },
        set        =
        {
          visible  = true
        }
      };

      if ( not m_storage.Execute( query ).result ) then

        query =
        {
          query      = "insert",
          group      = "player",
          collection = "stages",
          value      = 
          {
            name        = fight_name,
            visible     = true,
            enabled     = false,
            completed   = false,
            state       = 1
          }
        };
        
        m_storage.Execute( query );

      end;
      
      mediator.Notify( "Model_Fights_FightVisible", fight_name );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.IsFight( stage_name )

  return m_stage.GetStaticData( stage_name ).realtype == "fight";

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Zone_Data ( fight_name )

  local fight_data = m_stage.GetStaticData( fight_name );

  return { name = fight_data.zone, content = fight_data.content };

end;
--------------------------------------------------------------------------------
function private.Zone_Fight ( stage_list )

  for s = stage_list.min, stage_list.max do

    local stage_name = "stage_"..s;

    if not ( stage_list.type == "common" ) then

      stage_name = "stage_"..s.."_"..stage_list.type;

    end;

    if ( private.IsFight( stage_name ) ) then

      return stage_name;

    end;

  end;    

  return false;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ FIGHTS ] "..text );

end;
--------------------------------------------------------------------------------