-- name=m_fortunewheel
--------------------------------------------------------------------------------
function public.Init ()

  local settings = m_storage.Settings( "fortunewheel" );

  private.recover_time        = settings.time_recover;
  private.fortunewheel_prizes = settings.prizes;
  private.spin_price          = settings.spin_price or 0;
  ---------------------------------------------------------

  common.MessageSubscribe( "Event_Fortunewheel_RecoverDone", private.FinishTimer );

  if ( TimeIsEventExist( "tmr_fortunewheel_recover" ) ) then

    if ( TimeGetEventRemainingTime( "tmr_fortunewheel_recover" ) == 0 ) then
      private.FinishTimer();
    else
      public.SetState( { state  = "recover" } );
    end;

  end;

end;
--------------------------------------------------------------------------------
function public.TryTurnWheel ()

  local info = public.GetState();

  if ( info.state == "ready" ) then

    local result;

    if ( public.GetState().count_try == 0 ) then

      result = 1; -- Pos of ExtraMoves

    else

      local prizes = common.DeepCopy( private.fortunewheel_prizes );

      -- Do not give hearts if player has maximum
      if ( m_player.GetHeartsCount() >= m_hearts.GetMaxHearts() ) and
         ( #prizes > 1 ) then

        -- Set heart chance to zero
        local heart_chance = 0;
        for i = 1, #prizes do
          if ( prizes[ i ].name == "heart" ) then

            heart_chance = prizes[ i ].chance;
            prizes[ i ].chance = 0;
            break;

          end;
        end;

        -- Normalize another chances
        local chance_addiction = heart_chance / ( #prizes - 1 );
        for i = 1, #prizes  do
          if ( prizes[ i ].name ~= "heart" ) then

            prizes[ i ].chance = prizes[ i ].chance + chance_addiction;

          end;
        end;

      end;

      --DbgTrace( "[ INFO ] Prizes: ", prizes );

      result = private.GetRandomPrize( prizes );

    end;

    public.SetState( { state =  "recover", count_try = info.count_try + 1 } );
    private.StartTimer();
    return result;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.TryResetCooldown ()

  local info = public.GetState();
  local price = private.spin_price;

  if ( info.state == "recover" ) and ( m_player.TryChangeGoldCount( - price ) ) then

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "fortunewheel",
      value       = - price,
      name        = "reset"
    } );

    private.FinishTimer();
    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.GetInfo ()

  local info      = {};
  info.prizes     = private.fortunewheel_prizes;
  info.spin_cost  = private.spin_price;
  info.tmr_token  = "tmr_fortunewheel_recover";

  return info;

end;
--------------------------------------------------------------------------------
function public.SetState ( set )

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "fortunewheel",
    set        = set
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.GetState ()

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "fortunewheel"
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.StartTimer ()

  private.StartTimer();

end;
--------------------------------------------------------------------------------
function private.StartTimer ()

  if ( TimeIsEventExist( "tmr_fortunewheel_recover" ) ) then

    --[[if ( TimeGetEventRemainingTime( "tmr_fortunewheel_recover" ) == 0 ) then
      private.FinishTimer();
    end;]]

    return false;

  end;

  TimeSheduleEvent(
    "tmr_fortunewheel_recover",
    "GLOBAL_TIME",
    private.recover_time,
    "Event_Fortunewheel_RecoverDone",
    {}
  );

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "fortunewheel"
  };

  if ( not m_storage.Execute( query ).result.state ) then

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "fortunewheel",
      value      = 
      {
        state  = "recover"
      }
    };
    
    m_storage.Execute( query );

  end;

  mediator.Notify( "Model_Fortunewheel_TimerStart" );

  return true;

end;
--------------------------------------------------------------------------------
function private.FinishTimer ()

  TimeFinishEvent();
  TimeUnsheduleEvent( "tmr_fortunewheel_recover" );

  public.SetState( { state  = "ready" --[[, count_try = 0]] } );

  mediator.Notify( "Model_Fortunewheel_TimerStop" );

end;
--------------------------------------------------------------------------------
function private.GetRandomPrize ( prize_list )

  local random = math.random();
  local chance_counter = 0.0;

  for i = 1, #prize_list do

    chance_counter = chance_counter + prize_list[ i ].chance;

    if ( random <= chance_counter ) then return i; end;

  end;

end;
--------------------------------------------------------------------------------