-- name=m_hearts
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ HEARTS ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.recover_time = m_storage.Settings( "hearts" ).time;

  common.MessageSubscribe( "Event_Hearts_RecoverDone", private.HeartsRecoverDone );

  private.CorrectSave();

  local hearts_info = m_storage.Player.Info().hearts;

  if ( IsMobile() ) then
  
    if ( hearts_info.facebook == 0 ) then

      if ( m_social.IsLogged() ) then

        m_player.TryChangeHeartsCount_Facebook();

      else

        local func_change_heart_count;
        func_change_heart_count = function ( result )
          if ( result ) then
            mediator.Unsubscribe( "Model_Social_LoginContinue", func_change_heart_count );
            m_player.TryChangeHeartsCount_Facebook();
          end;
        end;

        mediator.Subscribe( "Model_Social_LoginContinue", func_change_heart_count );

      end;

    end;
    
  end;

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
function private.CorrectSave ()

  local hearts_info = m_storage.Player.Info().hearts;
  local donation    = m_storage.Settings( "hearts" ).donate_count;

  local donation_now = hearts_info.donate;
  local facebook_now = hearts_info.facebook;

  local donation_correct = donation.default;
  local fecebook_correct = 1;

  if ( IsPC() ) then
    donation_correct = donation.pc;
    fecebook_correct = 0;
  end;

  local count_now = hearts_info.count;
  local count_new = count_now;

  local update = {};

  if ( donation_now > 0 and donation_now ~= donation_correct ) then

    update.donate = donation_correct;
    count_new = count_new + ( donation_correct - donation_now );

  end;

  if ( facebook_now > 0 and facebook_now ~= fecebook_correct ) then

    update.facebook = fecebook_correct;
    count_new = count_new + ( fecebook_correct - facebook_now );

  end;

  if ( count_now ~= count_new ) then

    update.count = math.max( count_new, 0 );

  end;

  if ( next( update ) ~= nil ) then

    for key, value in pairs( update ) do
      hearts_info[ key ] = value;
    end;

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        = { hearts = hearts_info }
    };

    m_storage.Execute( query );

  end;

end;
--------------------------------------------------------------------------------
function public.TryBuy ()

  local current_count = m_player.GetHeartsCount();
  local maximum_count = public.GetMaxHearts();

  if ( current_count >= maximum_count ) then

    return;

  end;

  local pack_price = public.GetHeartsPrice();

  if ( m_player.TryChangeGoldCount( - pack_price ) ) then

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "hearts",
      value       = - pack_price,
      name        = "restore"
    } );

    m_player.TryChangeHeartsCount( maximum_count );

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.TryIncreaseMaxHeartsCount ()

  local donate_price = public.GetHeartsDonate();
  
  if ( IsPC() ) then
    
    if ( m_client.IsOnline() ) then

      local query_name_ios =
      {
        query      = "find",
        group      = "static",
        collection = "lots",
        indice     = "name",
        criteria   = { like = "hearts_2" }
      };

      local lot_id = m_storage.Execute( query_name_ios ).result.name_ios;

      wiz.bfg.lot.Buy( lot_id );

    else

      v_client.SystemDialog_Show( "connectionlost", function () end );

    end;
    
    return;
    
  end;

  if ( m_player.TryChangeGoldCount( - donate_price ) ) then

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "hearts",
      value       = - donate_price,
      name        = "increase"
    } );

    m_player.TryChangeHeartsCount_Donate();

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.Premium_HeartsAdd ( lot_name, transaction_id )

  local lot_data = m_shop.Gold_GetLotByIOSName( lot_name );
  
  if ( lot_data and lot_data.price ) then
  
    m_player.PayingStatus_Set( lot_data.price );
  
  end;
  
  common.Kontagent_LogEvent( "economy", 
  {
    description = "item_collected",
    category    = "store",
    value       = 2,
    name        = lot_name,
    transaction_id = transaction_id
  } );

  m_player.TryChangeHeartsCount_Donate();

end;
--------------------------------------------------------------------------------
function public.GetMaxHearts ()

  local basic = m_storage.Settings( "hearts" ).count;
  local addon = m_storage.Player.Info().hearts;

  return basic + ( addon.facebook or 0 ) + ( addon.donate or 0 );

end;
--------------------------------------------------------------------------------
function public.GetHeartsPrice ()

  local single = m_storage.Settings( "hearts" ).price.single;

  local count  = m_storage.Player.Info().hearts.count;
  local max    = public.GetMaxHearts()

  local price  = ( max - count ) * single;

  return price;

end;
--------------------------------------------------------------------------------
function public.GetHeartsDonate ()

  if ( m_storage.Player.Info().hearts.donate == 0 ) then
    
    local lots = GetLotsInfo().lots or {};
    local hearts_lot = private.HeartsLot_Get();
    
    if ( lots ) and ( #lots > 0 ) then

      for l = 1, #lots do

        local lot = lots[ l ];

        if ( lot.name_ios == hearts_lot.name_ios ) then

          hearts_lot.price = lot.price;

          break;

        end;

      end;

    end;

    if ( IsPC() ) then

      return hearts_lot.price;
      
    end;

    return m_storage.Settings( "hearts" ).price.donate;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.HeartsLot_Get ()

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "lots",
    indice     = "name",
    criteria   = { like = "hearts_2" }
  };
  
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
--[[function public.GetHeartsFacebook ()

  return m_storage.Player.Info().hearts.facebook ~= 0;

end;]]
--------------------------------------------------------------------------------
function public.HeartsRecoverStart ()

  TimeSheduleEvent( "tmr_hearts_recover", "GLOBAL_TIME", private.recover_time, "Event_Hearts_RecoverDone", {} );

  TimeStartWatch( "wth_hearts_recover", "GLOBAL_TIME" );

  mediator.Notify( "Model_Player_HeartsRecoverStart" );

end;
--------------------------------------------------------------------------------
function public.HeartsRecoverStop ()

  TimeUnsheduleEvent( "tmr_hearts_recover" );

  TimeStopWatch( "wth_hearts_recover" );

end;
--------------------------------------------------------------------------------
function private.HeartsRecoverDone ()

  TimeFinishEvent();

  local count_passed = 1;

  local time_remain = private.recover_time;

  local time_passed = TimeGetWatchTime( "wth_hearts_recover" );

  TimeStopWatch( "wth_hearts_recover" );

  if ( time_passed > private.recover_time ) then

    count_passed = math.floor( time_passed / private.recover_time );

    time_remain = math.fmod( time_passed, private.recover_time );

  end;

  m_player.TryChangeHeartsCount( count_passed );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.GetFullHeartsRestoreTime ()
  
  local hearts_needed = public.GetMaxHearts() - m_player.GetHeartsCount();
  local full_time = private.recover_time * hearts_needed;
  
  local time_passed = TimeGetWatchTime( "wth_hearts_recover" );
  if ( time_passed < 0 ) then time_passed = 0; end;
  local full_time_corrected = full_time - time_passed;

  private.Trace( "GetFullHeartsRestoreTime: full_time = "..full_time..
    ", time_passed = "..time_passed..
    ", full_time_corrected = "..full_time_corrected );
  
  return full_time_corrected;

end;
--------------------------------------------------------------------------------