-- name=m_items
--------------------------------------------------------------------------------
function public.Init ()

  private.show_errors = IsEditor();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.TryObtain ( ... )

  local args = {...};
  local item;
  local stage_name  = nil;
  local stage_state = nil;
  local quest_name = nil;
  local category   = nil;
  local silent     = true;

  if ( args ) then 

    if ( #args >= 1 ) and ( type( args[ 1 ] ) == "table" ) then

      item = args[ 1 ];
      stage_name  = args[ 1 ].stage_name;
      stage_state = args[ 1 ].stage_state;
      quest_name = args[ 1 ].quest_name;
      category   = args[ 1 ].category;
      if ( args[ 1 ].silent ~= nil ) then
        silent = args[ 1 ].silent;
      end;

    elseif ( #args >= 3 ) then

      item  = {
        name  = args[ 1 ],
        type  = args[ 2 ],
        count = args[ 3 ]
      };

    elseif ( private.show_errors ) then

      local error = "[ ERROR ] Model Items -> TryObtainItem -> Invalid args (count:"..#args.."):";
      for i, arg in ipairs( args ) do
        error = error..("\n"..i..":"..tostring( arg ));
      end;
      DbgTrace( error );
      return false;

    end;

  elseif ( private.show_errors ) then

    DbgTrace( "[ ERROR ] Model Items -> TryObtainItem -> No args" );
    return false;

  end;

  ------------------------------------------------------------------------------
  -- JK: chest are not directly obtainable
  if ( item.type == "chest" ) then

    return false;

  end;
  ------------------------------------------------------------------------------

  local func_try_obtain = function ( func_obtain )

    if ( func_obtain() ) then

      mediator.Notify( "Model_Items_ItemObtain", item );
      return true;

    elseif ( private.show_errors ) then

      DbgTrace( "[ ERROR ] Model Items -> TryObtainItem -> External model can't obtain item: "..
        item.name..":"..tostring( item.type )..":"..tostring( item.count ) );
      return false;

    end;

  end;

  if     ( item.type == "gold" )     then 
    return func_try_obtain( function () 

      common.Kontagent_LogEvent( "economy", 
      {
        description = "premium_currency_collect",
        category    = category or "reward",
        value       = item.count,
        name        = stage_name or quest_name or "",
        state       = stage_state
      } );

      return m_player.TryChangeGoldCount( item.count, silent );

    end );

  elseif ( item.type == "resource" ) then 
    return func_try_obtain( function () 

      common.Kontagent_LogEvent( "economy", 
      {
        description = "soft_currency_collect",
        category    = category or "reward",
        value       = item.count,
        name        = stage_name,
        type        = item.name,
        state       = stage_state
      } );

      return m_player.TryChangeResourceCount( item.name, item.count, true );

    end );

  elseif ( item.type == "skinpart" ) then 
    return func_try_obtain( function ()
      if ( category == "reward" ) then 
        category = stage_name;
      end;
      return m_skins.TryObtainParts( item.name, item.count, category );
    end );

  elseif ( item.type == "booster" )  then 
    return func_try_obtain( function () 
      return m_boosters.TryChangeCount( item.name, item.count, category );
    end );

  elseif ( item.type == "fragment" )  then 
    return func_try_obtain( function () 
      return m_builds.TryObtainFragment( item.name, stage_name );
    end );
    
  elseif ( item.type == "stickerpack" )  then 
    return func_try_obtain( function () 
      return m_fireflies.Stickerpack_TryObtain( item.count, false, stage_name or category );
    end );    

  elseif ( private.show_errors ) then
    DbgTrace( "[ ERROR ] Model Items -> TryObtainItem -> Invalid type: "..tostring( item.type ) );
    return false;

  end;

  return false;

end;
--------------------------------------------------------------------------------
--function public.TrySpend ( resource )
--
--  
--  
--  local func_try_spend = function ( func_spendn )
--
--    if ( func_spend() ) then
--
--      common.Kontagent_LogEvent( "economy", 
--      {
--        description = "premium_currency_spend",
--        category    = "spellbook",
--        value       = - currency.count,
--        name        = spell_name
--      } );
--      
--      mediator.Notify( "Model_Items_ItemSpend", item );
--      return true;
--
--    else
--
--      return false;
--
--    end;
--
--  end;
--  
--  
--  
--
--  if ( "gold" == currency.name ) then
--
--    m_player.TryChangeGoldCount( - currency.count );
--
--    common.Kontagent_LogEvent( "economy", 
--    {
--      description = "premium_currency_spend",
--      category    = "spellbook",
--      value       = - currency.count,
--      name        = spell_name
--    } );
--
--  else
--
--    m_player.TryChangeResourceCount( currency.name, - currency.count );
--
--    common.Kontagent_LogEvent( "economy", 
--    {
--      description = "soft_currency_spend",
--      category    = "spellbook",
--      value       = - currency.count,
--      name        = spell_name,
--      type        = currency.name
--    } );
--
--      end;
--        
--end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Chest_Generate ( chest_data )

  if ( chest_data.rewards ) then

    for r, reward in pairs( chest_data.rewards ) do

      if ( reward.type == "booster" ) then

        chest_data.rewards[ r ] = private.Chest_Random_BoosterOrGold( 
          chest_data.name
        );

      end;

    end;

    return chest_data.rewards;

  end;

  return private.Chest_Random_Rewards( chest_data.name );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Chest_Random_Rewards ( chest_name )

  local rewards = m_storage.Settings( "chests" )[ chest_name ];
  local random  = {};

  for r, reward in pairs( rewards ) do

    if ( r == "gold" ) then

      table.insert( random, private.Chest_Random_Gold( chest_name ) );

    end;

    if ( r == "booster" ) then

      table.insert( random, private.Chest_Random_BoosterOrGold( chest_name ) );

    end;

    if ( r == "skinpart" ) then

      table.insert( random, private.Chest_Random_SkinpartOrGold( chest_name ) );

    end;

  end;

  return random;

end;
--------------------------------------------------------------------------------
function private.Chest_Random_Gold ( chest_name )

  local list   = m_storage.Settings( "chests" )[ chest_name ].gold;
  local random = list[ math.random( 1, #list ) ];

  return random;

end;
--------------------------------------------------------------------------------
function private.Chest_Random_BoosterOrGold ( chest_name )

  local list   = m_storage.Settings( "chests" )[ chest_name ].booster;

  if ( not list ) or ( #list < 1 ) then

    return private.Chest_Random_Gold( chest_name );

  end;

  local random = list[ math.random( 1, #list ) ];

  if ( m_boosters.IsUnlocked( random.name ) ) then

    return random;

  end;

  return private.Chest_Random_Gold( chest_name );

end;
--------------------------------------------------------------------------------
function private.Chest_Random_SkinpartOrGold ( chest_name )

  local list   = m_storage.Settings( "chests" )[ chest_name ].skinpart;

  if ( not list ) or ( #list < 1 ) then

    return private.Chest_Random_Gold( chest_name );

  end;

  local random = list[ math.random( 1, #list ) ];

  local collectable_count = m_skins.Skinpart_CollectableCount( random.name );

  if ( collectable_count > 0 ) then

    random.count = math.min( random.count, collectable_count );

    return random;

  end;

  return private.Chest_Random_Gold( chest_name );

end;
--------------------------------------------------------------------------------