-- name=m_map
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ MAP ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  loader.Modules_Load_Init({
    "m_mines",
    "m_builds",
    "m_characters",
    "m_chests",
    "m_fights",
    "m_spells"
  });
  
  private.changes_list = {};
  
  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
function public.Update (
    stage_name,
    is_first,
    reward_data,
    noreward,
    enable_next
  )

  if ( is_first ) then
    
    m_mines.TryUnlockMineByStage( stage_name );
    m_chests.TryUnlock( stage_name );
    m_builds.TryUnlockBuildByStage( stage_name );
    m_spells.TryUnlockSpellByStage( stage_name );
    m_market.TryUnlock( stage_name );

  end;
  
  private.Portal_TryUnlock( stage_name );

  if ( 0 < #reward_data ) then

    -- Mine unlock ( by progress ).
    m_mines.TryUnlockMineByProgress( stage_name, reward_data );

  else

    reward_data = nil;

  end;

  table.insert( private.changes_list,
  {
    type        = "StageCompleted",
    name        = stage_name,
    is_first    = is_first,
    reward      = reward_data,
    noreward    = noreward,
    enable_next = enable_next
  } );

  local fight_data = m_fights.Data( stage_name );
  if ( fight_data.realtype == "misc" ) then

    m_fights.Enemy_SetAbsent( fight_data.enemy, false );

  end;

  if ( enable_next ) then

    local next_stage_name = private.Stage_Next_Name( stage_name );
    local next_info       = m_stage.GetSavedData( next_stage_name );

    if ( not next_info                                     ) or
       ( not next_info.completed and not next_info.enabled ) then

      if ( m_stage.TryEnable( next_stage_name ) ) then

        m_stage.CheckStateless();

        table.insert(
          private.changes_list,
          {
            type = "StageEnabled",
            name = next_stage_name
          }
        );

        local next_fight_data = m_fights.Data( next_stage_name );
        if ( next_fight_data.realtype == "misc" ) then

          m_fights.Enemy_SetAbsent( next_fight_data.enemy, true );

        end;

      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
-- function ...() end;  
--------------------------------------------------------------------------------
function public.GetChangesList ()

  local changes_list = private.changes_list;

  private.changes_list = {};

  return changes_list;

end;
--------------------------------------------------------------------------------
function public.DestroyChangesList ()

  private.changes_list = {};

end;
--------------------------------------------------------------------------------
function public.GetMinesList ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "mines",
    indice     = "section",
    criteria   = { like = section_name }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.GetBuildsList ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "builds",
    indice     = "section",
    criteria   = { like = section_name }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.GetCharactersList ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "characters",
    indice     = "section",
    criteria   = { like = section_name }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.GetFightsList ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "section",
    criteria   = { like = section_name }
  };

  local stages_list = m_storage.Execute( query ).result;

  local fights_list = {}

  for i = 1, #stages_list do

    local stage_name = stages_list[ i ].name;
    local fight_data = m_fights.Data( stage_name );

    if ( fight_data.type == "fight" ) then

      table.insert( fights_list, fight_data );

    end;

  end;

  return fights_list;

end;
--------------------------------------------------------------------------------
function public.GetStagesList ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "section",
    criteria   = { like = section_name }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.GetSpellsList ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "spells",
    indice     = "section",
    criteria   = { like = section_name }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
-- function ...() end;  
--------------------------------------------------------------------------------
function public.GetEnabledStagesList ( section_name )

  local stages_all     = public.GetStagesList( section_name ) or {};
  local stages_enabled = {};
  
  for _, stage in pairs( stages_all ) do
  
    local saved_data = m_stage.GetSavedData( stage.name );
    if ( saved_data and saved_data.enabled ) then
    
      table.insert( stages_enabled, stage.name );
    
    end;
  
  end;
  
  return stages_enabled;

end;
--------------------------------------------------------------------------------
-- function ...() end;  
--------------------------------------------------------------------------------
function public.GetUpdateData ()

  local update_data = false;
  local player_data = m_storage.Player.Info();
  
  --Uncomment for testing reward
  --player_data.update      = {};
  --player_data.update.type = "reward";
  --player_data.update.reward =
  --{
  --  { type = "booster", name = "hammer",    count = 1   },
  --  { type = "gold",    name = "gold",      count = 150 },
  --  { type = "booster", name = "magicgift", count = 2   }
  --};
  
  --Uncomment for testing replay
  --player_data.update      = {};
  --player_data.update.type = "replay";

  if ( player_data.update ) then

    local version = VersionGetComponentVersion( "game" );

    local message = "";
    local count   = 0;
    
    for key in string.gmatch( version, "%d+" ) do

      if ( count < 3 ) then

        message = message..key;

      end;

      count = count + 1;

    end;

    update_data = {
      type    = player_data.update.type,
      message = message,
      reward  = player_data.update.reward
    };

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        =
      {
        update = false
      }
    };

    m_storage.Execute( query );

  end;

  return update_data;

end;
--------------------------------------------------------------------------------
function public.Stage_Last_Name ()

  return m_storage.Player.Info().stage_last;

end;
--------------------------------------------------------------------------------
function public.Stage_Last_Set ( stage_name, section_name )

  local stage_last = m_storage.Player.Info().stage_last;

  local stage_last_section = m_stage.GetStaticData( stage_last ).section;

  if ( stage_last_section == section_name ) then

    return false;

  end;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        =
    {
      stage_last = stage_name
    }
  };

  m_storage.Execute( query );

  m_stage.TryEnable( stage_name );

  return true;

end;
--------------------------------------------------------------------------------
function public.Stage_Next_Name ( stage_name )

  return private.Stage_Next_Name( stage_name );

end;
--------------------------------------------------------------------------------
function public.Stage_WorkerState ( stage_name )
  
  local state = m_stage.GetStaticData( stage_name ).state;

  for i = 1, #state do

    if ( state[ i ].reward ) then

      for j = 1, #state[ i ].reward do

         if ( state[ i ].reward[ j ].name == "worker" ) then

          return i;

         end;

      end;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Stage_Next_Name ( stage_name )

  -- JK: check if current stage is last in the section
  local last_stage_name = private.Section_Stage_Last( private.section_name );

  if ( stage_name == last_stage_name ) then

    return stage_name;

  end;

  --JK: check if current stage is a deadlock
  if ( private.Stage_IsDeadlock( stage_name ) ) then

    return stage_name;

  end;

  local this_number = string.match( stage_name, "%d+" );
  local next_number = tonumber( this_number ) + 1;
  local stage_next  = string.gsub( stage_name, this_number, next_number );

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "builds",
    indice     = "unlock",
    criteria   = { like = stage_name }
  };

  local unlock_data = m_storage.Execute( query ).result;

  if ( unlock_data ) then

    local build_data = m_builds.GetBuildData( unlock_data.name );

    if ( not build_data.constructed ) then
      return stage_name;
    end;

  end;

  if ( private.Stage_Exists( stage_next, private.section_name ) ) then

    return stage_next;

  end;

  return stage_name;

end;
--------------------------------------------------------------------------------
function private.Stage_Exists ( stage_name, section_name )
  
  local stage_data = m_stage.GetStaticData( stage_name );
  
  return ( ( stage_data ) and ( stage_data.section == section_name ) );

end;
--------------------------------------------------------------------------------
function private.Stage_IsDeadlock ( stage_name )

  --JK: play with bonus stages only
  if not ( string.find( stage_name, "bonus" ) ) then return false; end;

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "builds",
    indice     = "deadlock",
    criteria   = { like = tonumber( string.match( stage_name, "%d+" ) ) }
  };

  return m_storage.Execute( query ).result ~= nil;

end;
--------------------------------------------------------------------------------
-- function ...() end;  
--------------------------------------------------------------------------------
function public.GetStageInfo ( stage_name )

  local saved_data = m_stage.GetSavedData( stage_name ) or {};
  local stage_data = m_stage.GetStaticData( stage_name, saved_data );
  
  if not ( stage_data ) then
    return false;
  end;

  -- Map point info
  local stage_info =
  {
    type     = stage_data.type,
    realtype = stage_data.realtype,
    hint     = stage_data.hint or false,

    state    = m_stage.GetStageState( stage_name, stage_data, saved_data ),

    enabled   = saved_data.enabled,
    completed = saved_data.completed,
    visible   = saved_data.visible,
    noreward  = saved_data.noreward, -- All states completed flag

    reward    = nil  -- displaying item reward
  };

  if ( m_stage.IsStateless() and stage_info.state > 1 ) then

    stage_info.state     = 1;
    stage_info.noreward  = true;

  end;

  -- Find first active reward item
  local states = #stage_data.state;

  for state = 1, states  do

    local rewards = stage_data.state[ state ].reward or {};

    for r = 1, #rewards do

      local reward   = rewards[ r ];
      reward.content = m_content.GetContent( reward.type, reward.name );

      if ( reward.type == "quest" ) then

        if ( m_quests.IsStageItemActive( reward.name, stage_name, "map", stage_data.section ) ) then

          stage_info.reward = reward;
          break;

        end;

      elseif ( reward.type == "fragment" ) then

        local build_name = m_builds.GetBuildNameByFragment( reward.name );

        if ( build_name ) then

          local build_data = m_builds.GetBuildData( build_name );

          if ( not cmn.IsEntityInList( build_data.collected_fragments, reward.name ) ) then

            stage_info.reward = reward;
            break;

          end;

        end;

      elseif not ( stage_info.noreward ) and ( stage_info.state <= state ) and ( reward.type ~= "gold" ) then

        if ( reward.type == "worker" ) then
          local mine_name = m_mines.GetWorkerMineName( stage_name );
          local mine_data = m_mines.GetMineData( mine_name );
          if ( ( mine_data.progress or 0 ) < mine_data.progress_max ) then
            stage_info.reward = reward;
            break;
          end;
        else
          stage_info.reward = reward;
          break;
        end;

      end;

    end;  

  end;

  return stage_info;

end;
--------------------------------------------------------------------------------
-- function ...() end;
--------------------------------------------------------------------------------
function public.Portal_GetName ( section_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name }
  };
  
  return ( m_storage.Execute( query ).result or {} ).portal;

end;  
--------------------------------------------------------------------------------
function private.Portal_GetUnlock ( section_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name }
  };
  
  return m_storage.Execute( query ).result.unlock;

end;
--------------------------------------------------------------------------------
function private.Portal_TryUnlock ( stage_name )
  
  local try_unlock = false;
  
  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = private.next_section_name }
  };

  local section_data = m_storage.Execute( query ).result;
  
  if ( section_data ) then
    
    return try_unlock;
    
  end;
  
  if ( private.portal_unlock.type == "CompleteStage" ) then
    
    if ( private.portal_unlock.name == stage_name ) then

      try_unlock = true;
      table.insert( private.changes_list, { type = "PortalUnlocked" } );
      
    end;
    
  elseif ( private.portal_unlock.type == "CompleteQuest" ) then
    
    local query =
    {
      query      = "find",
      group      = "player",
      collection = "quests_completed",
      indice     = "name",
      criteria = { like = private.portal_unlock.name }
    };
    
    local quest_completed = ( m_storage.Execute( query ).result or {} ).name;
      
    if ( quest_completed ) then

      try_unlock = true;
      table.insert( private.changes_list, { type = "PortalUnlocked" } );
      
    end;
    
  end;

  if ( try_unlock ) then
    private.Section_TryUnlock( private.next_section_name );
  end;
  
end;
--------------------------------------------------------------------------------
-- function ...() end;
--------------------------------------------------------------------------------
function public.Section_SetCurrent ( section_name )

  private.section_name        = section_name; 
  private.next_section_name   = private.Section_Next_Name( section_name );
  private.portal_unlock       = private.Portal_GetUnlock( private.next_section_name );

end;
--------------------------------------------------------------------------------
function public.Section_Current_Name ()

  return private.section_name;

end;
--------------------------------------------------------------------------------
function public.Section_Next_Status ()

  local next_section_name = private.Section_Next_Name( private.section_name );

  return private.Section_Status( next_section_name );

end;
--------------------------------------------------------------------------------
function public.Section_Prev_Status ()

  local prev_section_name = private.Section_Prev_Name( private.section_name );

  if ( prev_section_name ) then

    return private.Section_Status( prev_section_name );

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.Section_GetList ()

  return private.Section_GetList();

end;
--------------------------------------------------------------------------------
function public.Section_GetDetailedList ()

  local list = {};

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "sections"
  };

  local static_list   = m_storage.Execute( query ).result or {};
  local enabled_count = m_storage.Settings( "sections_enable" );

  for i = 1, enabled_count do

    local section = static_list[ i ];
    local name    = section.name;

    query =
    {
      query      = "find",
      group      = "player",
      collection = "sections",
      indice     = "name",
      criteria   = { like = name }
    };

    local saved = m_storage.Execute( query ).result or {};

    for key, value in pairs( saved ) do

      section[ key ] = value;

    end;
    
    query =
    {
      query      = "find_all",
      group      = "static",
      collection = "stages",
      indice     = "section",
      criteria   = { like = name }
    };
    
    local stages = m_storage.Execute( query ).result or {};
    
    section.stages_count = #stages;
    section.stages_done  = 0;
    
    for _, stage in pairs( stages ) do
    
      local stage_data = m_stage.GetSavedData( stage.name ) or {};
    
      if ( stage_data.completed ) then
        section.stages_done = section.stages_done + 1;
      end;
    
    end;

    table.insert( list, section );

  end;

  return list;

end;
--------------------------------------------------------------------------------
function public.Section_NeedIntro ( section_name )

  section_name = section_name or private.section_name;

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name }
  };

  return not m_storage.Execute( query ).result.intro;

end;
--------------------------------------------------------------------------------
function public.Section_NeedOutro ( section_name )

  section_name = section_name or private.section_name;

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name }
  };

  return not m_storage.Execute( query ).result.outro;

end;
--------------------------------------------------------------------------------
function public.Section_SetIntro ( section_name, value )

  section_name = section_name or private.section_name;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name },
    set        = { intro = value }
  };

  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
function public.Section_SetOutro ( section_name, value )

  section_name = section_name or private.section_name;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name },
    set        = { outro = value }
  };

  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
function public.Section_Next_Enable ()

  local sections     = private.Section_GetList();
  local enable_count = public.Section_GetEnabledCount();

  local func_section_enable = function ( section_name )
    
    local query =
    {
      query      = "update",
      group      = "player",
      collection = "sections",
      indice     = "name",
      criteria = { like = section_name },
      set      = { enabled = true }
    };

    m_storage.Execute( query );

    public.Stage_Last_Set( public.Section_Stage_First( section_name ), 
                           section_name );
    m_fights.Section_Enable( section_name );
    m_characters.Section_Enable( section_name );

  end;
    
  for s = 1, enable_count do

    if ( sections[ s ] ) and not ( sections[ s ].enabled ) then

      func_section_enable( sections[ s ].name );

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.Section_GetByStage ( stage_name )

  local query = {
    query      = "find",
    group      = "static",
    collection = "stages",
    indice     = "name",
    criteria   = { like = stage_name }
  };
  
  return m_storage.Execute( query ).result.section;

end;
--------------------------------------------------------------------------------
function public.Section_GetEnabledCount ()

  return m_storage.Settings( "sections_enable" );

end;
--------------------------------------------------------------------------------
function public.Section_Stage_First ( section_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "stages",
    indice     = "section",
    criteria   = { like = section_name }
  };

  return m_storage.Execute( query ).result.name;

end;
--------------------------------------------------------------------------------
function public.Section_Stage_Last ( section_name )

  return private.Section_Stage_Last( section_name );

end;
--------------------------------------------------------------------------------
function private.Section_GetList ()

  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "sections"
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function private.Section_Status ( section_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name }
  };

  local section = m_storage.Execute( query ).result;

  if ( section ) then

    return tern( section.enabled )( "enabled", "available" );
    
  end;
  
  if ( private.portal_unlock.type == "CompleteStage" ) then
    
    query =
    {
      query      = "find",
      group      = "static",
      collection = "builds",
      indice     = "deadlock",
      criteria   = {
        like = tonumber( private.portal_unlock.name:match( "%d+" ) )
      }
    };

    local build_name = ( m_storage.Execute( query ).result or {} ).name;

    if ( build_name                                      ) and
       ( m_builds.GetBuildData( build_name ).constructed ) then

      return "prepare";

    end;
    
  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Section_Next_Name ( section_name )

  local section_number    = tonumber( string.match( section_name, "%d" ) );
  local next_section_name = "section_"..( section_number + 1 );

  return next_section_name;

end;
--------------------------------------------------------------------------------
function private.Section_Prev_Name ( section_name )

  local section_number = tonumber( string.match( section_name, "%d" ) );
  local prev_number    = section_number - 1;

  if ( prev_number > 0 ) then

    return "section_"..prev_number;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Section_TryUnlock ( section_name )

  local count  = public.Section_GetEnabledCount();
  local enable = true;

  if ( count < tonumber( string.match( section_name, "%d+" ) ) ) then

    enable = false;

  end;

  private.Section_PushCollection( section_name, enable );

  common.Kontagent_LogEvent( "progress", 
  {
    name = private.Section_Prev_Name( section_name )
  } );
  
  if ( enable ) then

    public.Stage_Last_Set(
      public.Section_Stage_First( section_name ),
      section_name
    );

    m_fights.Section_Enable( section_name );
    m_characters.Section_Enable( section_name );

  end;

end;
--------------------------------------------------------------------------------
function private.Section_PushCollection ( section_name, enable )

  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = section_name }
  };

  local section_data = m_storage.Execute( query ).result;

  if not ( section_data ) then

    section_data = 
    {
      name    = section_name,
      enabled = enable
    };
  
    query =
    {
      query      = "insert",
      group      = "player",
      collection = "sections",
      value      = section_data
    };

    m_storage.Execute( query );

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Section_Stage_Last ( section_name )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "section",
    criteria   = { like = section_name }
  };

  local stages = m_storage.Execute( query ).result;

  return stages[ #stages ].name;

end;
--------------------------------------------------------------------------------
-- function ...() end;
--------------------------------------------------------------------------------
function public.Cheater_UnlockSection ( section_name )

  private.Section_TryUnlock( section_name );

end;
--------------------------------------------------------------------------------