-- name=m_mines
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ MINES ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  common.MessageSubscribe(
    "Event_Game_Mines_MineReady",
    function ( msg_type, msg_params )
      private.Cooldown_Stop( msg_params.name, true );
    end
  );

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
-- Разблокировка шахты после прохождения уровня.
function public.TryUnlockMineByStage ( stage_name )

  local mine_query = 
  {
    query      = "find",
    group      = "static",
    collection = "mines",
    indice     = "unlock",
    criteria   = { like = stage_name }
  };

  local mine_data = m_storage.Execute( mine_query ).result;

  if ( mine_data ) then

    local mine_name = mine_data.name;

    private.TryUnlockMine( mine_name );

    private.TrySetMineStateWait( mine_name );

    --private.IncMineProgress( mine_name, 0 );

    if ( public.Mine_Type( mine_name ) == "gold" ) then

      public.TrySetMineResource( mine_name, "gold" );

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.TryUnlockMineByProgress ( stage_name, reward_data )

  local count = 0;

  for r = 1, #reward_data do

    local reward = reward_data[ r ];

    if ( reward.name == "worker" ) then

      count = reward.count;

      break;

    end;

  end;

  if ( count == 0 ) then

    return;

  end;

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "mines"
  };

  local mines = m_storage.Execute( query ).result;

  for s = 1, #mines do

    local mine = mines[ s ];

    if ( mine.progress and #mine.progress > 0 ) then

      --for i = mine.progress[ 1 ], mine.progress[ 2 ] do
      for i = 1, #mine.progress do

        --local worker_stage_name = string.format( "stage_%i", i );
        local worker_stage_name = mine.progress[ i ];

        if ( worker_stage_name == stage_name ) then

          private.TryUnlockMine( mine.name );

          private.IncMineProgress( mine.name, count );

          private.TrySetMineStateWait( mine.name );

          break;

        end;

      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
function public.TrySetMineResource ( mine_name, resource )

  local mine_data  = private.GetMineData( mine_name );
  local mine_state = mine_data.state;

  if not ( mine_state == "wait" ) then

    return false;

  end;

  local build_price = mine_data.price;

  local no_price = ( build_price == 0 );

  if ( no_price or m_player.TryChangeGoldCount( - build_price ) ) then

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "farm_build",
      value       = - build_price,
      name        = mine_name
    } );

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "mines",
      indice     = "name",
      criteria   = { like = mine_name },
      set        = 
      {
        state    = "active",
        resource = resource
      }
    };

    m_storage.Execute( query );

    local mine_type  = public.Mine_Type( mine_name );
    local start_full = m_storage.Settings( "mines" )[ mine_type ].upgrades[ 1 ].start_full;

    if ( start_full ) then

      private.Cooldown_Stop( mine_name, true );

    else

      private.Cooldown_Start( mine_name );

    end;

    mediator.Notify( "Model_Mines_MineStateChanged", { 
      name       = mine_name, 
      state      = "active",
      start_full = ( start_full ~= nil )
    } );

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Usable_List ()

  local section = v_map.Section_Current();
  
  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "mines",
    indice     = "state",
    criteria   = { like = "active" }
  };

  local usable = m_storage.Execute( query ).result;
  
  if not ( usable ) then return false; end;
  
  local list = {};
  
  for m = 1, #usable do 
    
    if ( private.Mine_Section( usable[ m ].name ) == section ) then
      
      table.insert( list, usable[ m ] );
      
    end;
  
  end;
  
  if ( #list < 1 ) then return false; end;

  return list;

end;
--------------------------------------------------------------------------------
function public.Usable_Search ( resource_name )

  local usable_mines = private.Usable_List();

  if ( usable_mines ) then

    for i = 1, #usable_mines do

      local count = public.Progress_GetCount( usable_mines[ i ].name ).count;

      if ( usable_mines[ i ].resource == resource_name ) and ( count > 5 ) then

        return usable_mines[ i ];

      end;

    end;

    return false;

  end;

  return false;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Progress_GetCount ( mine_name )

  local mine_data  = private.GetMineData( mine_name );
  local mine_state = mine_data.state;
  local resource   = mine_data.resource;

  local mine_type = public.Mine_Type( mine_name );

  if not ( mine_state == "ready" ) and not ( mine_state == "active" ) then

    return { name = resource, count = 0 };

  end;

  local timer_token = string.format( "mte_map_cooldown_mines_%s", mine_name );

  local total_time     = private.GetMineTime( mine_type );
  local remaining_time = TimeGetEventRemainingTime( timer_token );
  local passed_time    = total_time - remaining_time;

  local total_count   = private.GetMineUpgradeData( mine_type )[ mine_data.level ].resources;
  local current_count = math.floor( passed_time * total_count / total_time );
 
  if ( current_count > total_count ) then

    current_count = total_count;

  end;

  return { name = resource, count = current_count, full = total_count };

end;
--------------------------------------------------------------------------------
function public.TryCollectResource ( mine_name )

  local progress = public.Progress_GetCount( mine_name );

  if ( progress.count == 0 ) then

    return false;

  end;

  local mine_type = public.Mine_Type( mine_name );

  local functions = 
  {
    gold = function () 

      if( m_player.TryChangeGoldCount( progress.count, true ) ) then

        common.Kontagent_LogEvent( "economy", 
        {
          description = "premium_currency_collect",
          category    = "farm_gather",
          value       = progress.count,
          name        = mine_name
        } );
      
        mediator.Notify( "Model_Mines_GoldMineCollected", mine_name );
        
        return true;
      
      end;
      
      return false; 

    end,
    resource = function ()
                
      if( m_player.TryChangeResourceCount( progress.name, progress.count, true ) ) then

        common.Kontagent_LogEvent( "economy", 
        {
          description = "soft_currency_collect",
          category    = "farm_gather",
          value       = progress.count,
          name        = mine_name,
          type        = progress.name
        } );

        mediator.Notify( "Model_Mines_MineCollected", mine_name );
        
        return true;
      
      end;
    
      return false;

    end
  };

  local is_collected = functions[ mine_type ]();

  if ( is_collected ) then

    private.Cooldown_Stop( mine_name );
    private.Cooldown_Start( mine_name );

    return progress;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.TryUpgradeMine ( mine_name )

  local mine_data = private.GetMineData( mine_name );

  local level_new = mine_data.level + 1;

  if ( level_new > 3 ) then

    return false;

  end;

  local mine_type = public.Mine_Type( mine_name );

  local upgrade_price = private.GetMineUpgradeData( mine_type )[ level_new ].price;

  if ( m_player.TryChangeGoldCount( - upgrade_price ) ) then

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "farm_upgrade",
      value       = - upgrade_price,
      name        = mine_name
    } );

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "mines",
      indice     = "name",
      criteria   = { like = mine_name },
      set        = 
      {
        level = level_new
      }
    };

    m_storage.Execute( query );

    mediator.Notify( "Model_Mines_MineLevelChanged", mine_name );

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.GetMineData ( mine_name )

  return private.GetMineData( mine_name );

end;
--------------------------------------------------------------------------------
function public.GetMineTime ( mine_type )

  return private.GetMineTime( mine_type );

end;
--------------------------------------------------------------------------------
function public.GetMineUpgradeData ( mine_name )

  local mine_data = private.GetMineData( mine_name );

  local mine_type = public.Mine_Type( mine_name );

  local upgrade_data = private.GetMineUpgradeData( mine_type );
  
  local return_data = {};

  return_data.price = upgrade_data[ mine_data.level + 1 ].price;
  return_data.count_next = upgrade_data[ mine_data.level + 1 ].resources;
  return_data.count = upgrade_data[ mine_data.level ].resources;

  return return_data;

end;
--------------------------------------------------------------------------------
function public.GetWorkerMineName ( stage_name )

  return private.GetWorkerMineName( stage_name );

end;
--------------------------------------------------------------------------------
function private.GetActiveMinesList ()

  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "mines"
  };

  local mine_data = m_storage.Execute( query ).result;

  local active = {};

  for i = 1, #mine_data do

    if ( mine_data.state == "active" ) then

      table.insert( active, mine_data.name );

    end;

  end;

  return active;

end;
--------------------------------------------------------------------------------
function private.TryUnlockMine ( mine_name )

  local mine_data = private.GetMineData( mine_name );

  if ( mine_data.unlocked ) then

    return;

  end;

  local state = "none";

  if ( public.Mine_Type( mine_name ) == "gold" ) then

    state = "wait";

  end;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name },
    set        = 
    {
      state    = state,
      unlocked = true,
      level    = 1,
      progress = 0
    }
  };

  if ( not m_storage.Execute( query ).result ) then

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "mines",
      value      = 
      {
        name        = mine_name,
        unlocked    = true,
        state       = state,
        level       = 1,
        progress    = 0
      }
    };

    m_storage.Execute( query );

  end;

  mediator.Notify( "Model_Mines_MineUnlocked", mine_name );

end;
--------------------------------------------------------------------------------
function public.GetResourcesTypes ( mine_name )

  local overrides = m_storage.Settings( "mines" ).resource.overrides;

  if ( overrides[ mine_name ] ) then

    return overrides[ mine_name ];

  end;

  return m_storage.Settings( "mines" ).resource.resources;

end;
--------------------------------------------------------------------------------
function private.GetMineData ( mine_name )

  local mine_data = {};

  mine_data.name     = mine_name;
  mine_data.unlocked = false;

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name }
  };
  
  local price = m_storage.Execute( query ).result.price;
  
  query =
  {
    query      = "find",
    group      = "player",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name }
  };

  local saved_data = m_storage.Execute( query ).result;

  if ( saved_data ) then

    mine_data = saved_data;

    mine_data.level        = mine_data.level    or 1;
    mine_data.state        = mine_data.state    or "none";
    mine_data.progress     = mine_data.progress or 0;

  end;
  
  mine_data.progress_max = private.GetMineProgressMax( mine_name );

  local mine_type = public.Mine_Type( mine_name );

  mine_data.type  = mine_type;
  mine_data.price = price;

  mine_data.produce = {
    resources = private.GetMineUpgradeData( mine_type )[ mine_data.level or 1 ].resources,
    time      = private.GetMineTime( mine_type )
  };

  return mine_data;

end;
--------------------------------------------------------------------------------
function private.GetMineTime ( mine_type )

  return m_storage.Settings( "mines" )[ mine_type ].time;

end;
--------------------------------------------------------------------------------
function private.GetMineUpgradeData ( mine_type )

  return m_storage.Settings( "mines" )[ mine_type ].upgrades;

end;
--------------------------------------------------------------------------------
function private.GetMineProgressMax ( mine_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name }
  };

  local progress = m_storage.Execute( query ).result.progress;

  if ( progress and #progress > 0 ) then

    --local progress_max = progress[ 2 ] - progress[ 1 ] + 1;
    local progress_max = #progress;

    return progress_max;

  end;

  return 0;

end;
--********************************************************************************************************************
--********************************************************************************************************************
function private.GetWorkerMineName ( stage_name )

  local mine_name = nil;

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "mines"
  };

  local mines = m_storage.Execute( query ).result;

  for m = 1, #mines do

    local mine = mines[ m ];

    if ( mine.progress and #mine.progress > 0 ) then

      --for i = mine.progress[ 1 ], mine.progress[ 2 ] do
      for i = 1, #mine.progress do

        --local worker_stage_name = string.format( "stage_%i", i );
        local worker_stage_name = mine.progress[ i ];

        if ( worker_stage_name == stage_name ) then

          mine_name = mine.name;

          break;

        end;

      end;

    end;

  end;

  return mine_name;

end;
--------------------------------------------------------------------------------
function public.Mine_Type ( mine_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name }
  };

  local mine = m_storage.Execute( query ).result;

  if ( not mine ) or ( not mine.resource ) then

    query =
    {
      query      = "find",
      group      = "static",
      collection = "mines",
      indice     = "name",
      criteria   = { like = mine_name }
    };

    mine = m_storage.Execute( query ).result;

    if ( mine and mine.progress and #mine.progress > 0 ) then

      return "resource", "resource1";

    end;

    return "gold", "gold";

  end;

  if ( mine.resource ~= "gold" ) then

    return "resource", mine.resource;

  end;

  return "gold", "gold";

end;
--------------------------------------------------------------------------------
function private.Mine_Section ( mine_name )
  
  local query =
  {
    query      = "find",
    group      = "static",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name }
  };
  
  return m_storage.Execute( query ).result.section;
  
end;
--------------------------------------------------------------------------------
function private.IncMineProgress ( mine_name, count )

  local progress     = private.GetMineData( mine_name ).progress or 0;
  local progress_new = progress + count;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name },
    set        = 
    {
      progress = progress_new
    }
  };

  m_storage.Execute( query );

  mediator.Notify( "Model_Mines_MineProgressChanged", {
    name         = mine_name,
    progress     = progress,
    progress_new = progress_new
  } );

end;
--------------------------------------------------------------------------------
function private.TrySetMineStateWait ( mine_name )

  local progress     = private.GetMineData( mine_name ).progress or 0;
  local progress_max = private.GetMineProgressMax( mine_name );

  if ( progress == progress_max ) then

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "mines",
      indice     = "name",
      criteria   = { like = mine_name },
      set        = 
      {
        state = "wait"
      }
    };

    m_storage.Execute( query );

    mediator.Notify(
      "Model_Mines_MineStateChanged",
      {
        name  = mine_name,
        state = "wait" } );

  end;

end;
--------------------------------------------------------------------------------
function private.TrySetMineStateActive ( mine_name )

  if ( private.GetMineData( mine_name ).state == "active" ) then return; end;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name },
    set        = 
    {
      state = "active"
    }
  };

  m_storage.Execute( query );

  mediator.Notify(
   "Model_Mines_MineStateChanged",
   {
    name  = mine_name,
    state = "active" } );

end;
--------------------------------------------------------------------------------
function private.TrySetMineStateReady ( mine_name )

  --[[FF]]
  -- Подстраховка от редкого бага.
  local active       = ( private.GetMineData( mine_name ).state == "active" );
  local progress     = private.GetMineData( mine_name ).progress or 0;
  local progress_max = private.GetMineProgressMax( mine_name );

  if not ( active ) or not ( progress == progress_max ) then

    return;

  end;
  --[[FF]]
  
  local query =
  {
    query      = "update",
    group      = "player",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name },
    set        = 
    {
      state = "ready"
    }
  };

  m_storage.Execute( query );

  mediator.Notify(
    "Model_Mines_MineStateChanged",
    {
      name  = mine_name,
      state = "ready" } );

end;
--------------------------------------------------------------------------------
function private.Cooldown_Start ( mine_name, time )

  local mine_type = public.Mine_Type( mine_name );

  local total_time = private.GetMineTime( mine_type );

  if ( total_time ) then

    time = time or total_time;

    local timer_token = string.format( "mte_map_cooldown_mines_%s", mine_name ); 

    if ( not TimeIsEventExist( timer_token ) ) then

      TimeSheduleEvent( timer_token, "GLOBAL_TIME", time, "Event_Game_Mines_MineReady", { name = mine_name, time = time } );

      local watch_token = string.format( "mte_map_cooldown_mines_%s_watch", mine_name );

      TimeStartWatch( watch_token, "GLOBAL_TIME" );

      private.TrySetMineStateActive( mine_name );

      mediator.Notify( "Model_Mines_MineCooldownStart", mine_name );
    
    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Cooldown_Stop ( mine_name, full )

  local timer_token = string.format( "mte_map_cooldown_mines_%s", mine_name );
  local watch_token = string.format( "mte_map_cooldown_mines_%s_watch", mine_name );

  TimeFinishEvent();
  TimeUnsheduleEvent( timer_token );
  TimeStopWatch( watch_token );

  if ( full ) then

    private.TrySetMineStateReady( mine_name );

  end;

  mediator.Notify( "Model_Mines_MineCooldownStop", mine_name );

end;
--------------------------------------------------------------------------------
-- function **** Local notifications **** () end;
--------------------------------------------------------------------------------
function public.GetRandomMineCooldown ()
  
  -- For local notifications only.

  local mines_active = private.GetActiveMinesList();

  if ( #mines_active > 0 ) then

    local index = math.random( 1, #mines_active );

    local token = string.format( "mte_map_cooldown_mines_%s", mines_active[ i ] );

    return { name = mines_active[ i ], time = TimeGetEventRemainingTime( token ) };

  end;

  return nil;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.GetMineByUnlockStage( stage )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "mines",
    indice     = "unlock",
    criteria   = { like = stage }
  };

  return ( m_storage.Execute( query ).result or {} ).name;

end;
--------------------------------------------------------------------------------