-- name=m_object
--------------------------------------------------------------------------------
function public.Get ( object_name )

  return ne.obj.Get( object_name );

end;
--------------------------------------------------------------------------------
function public.Set ( object_name, properties, multiset )

  if ( multiset ) then

    table.insert( multiset, { object_name, properties } );

  else

    ne.obj.Set( object_name, properties );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Create ( object_name, object_type, attach_to, properties )

  object_type = object_type or "obj";

  ne.obj.Create( object_name, object_type, attach_to, properties );

end;
--------------------------------------------------------------------------------
function public.Clone ( object_name, to, postfix, with_children )

  postfix = postfix or "_clone";

  local clone_name = object_name..postfix;

  if ( public.Exists( clone_name ) ) then
    
    DbgTrace( "[ OBJECT ] Error: Clone target already exists" );
    
    return;
    
  end;

  with_children = with_children or 0;

  ne.obj.Clone( object_name, postfix, with_children, to );

  return clone_name;

end;
--------------------------------------------------------------------------------
function public.Delete ( object_name )

  ne.obj.Delete( object_name );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Exists ( object_name )
  
  if ( ne.obj.Get( object_name ) ) then

    return true;

  end;
  
  return false;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Enable ( object_name, preserve_input, multiset )

  local properties = {
    active  = true,
    visible = true,
    input   = true
  };

  if ( preserve_input ) then

    properties.input = nil;

  end;

  public.Set( object_name, properties, multiset );

end;
--------------------------------------------------------------------------------
function public.Disable ( object_name, preserve_input, multiset )

  local properties =
  {
    active  = false,
    visible = false,
    input   = false
  };

  if ( preserve_input ) then

    properties.input = nil;

  end;

  public.Set( object_name, properties, multiset );

end;
--------------------------------------------------------------------------------
function public.IsEnabled ( object_name )
  
  return public.Get( object_name ).active;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Show ( object_name, multiset )

  local properties =
  {
    visible = true
  };

  public.Set( object_name, properties, multiset );

end;
--------------------------------------------------------------------------------
function public.Hide ( object_name, multiset )

  local properties =
  {
    visible = false
  };

  public.Set( object_name, properties, multiset );

end;
--------------------------------------------------------------------------------
function public.IsVisible ( object_name )
  
  return public.Get( object_name ).visible;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Multiset ( multiset )

  if ( multiset                    ) and
     ( type( multiset ) == "table" ) and
     ( #multiset > 0               ) then

    ne.obj.MultiSet( multiset );

  else
    
    DbgTrace( "[ OBJECT ] Multiset error" );
    common.DbgCallpoint();
    
  end;

  multiset = nil;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Attach ( object_name, to )

  ne.obj.Attach( object_name, to );

end;
--------------------------------------------------------------------------------
function public.Detach ( object_name )

  ne.obj.Detach( object_name );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Children ( object_name, search_pattern )

  local children = ObjGetRelations( object_name ).childs;

  if ( not children_type ) then

    return children;

  end;

  local children_by_pattern = {};

  for c, child in pairs( children ) do

    if ( child:find( search_pattern ) ) then

      table.insert( children_by_pattern, child );

    end;

  end;

  return children_by_pattern;

end;
--------------------------------------------------------------------------------