-- name=m_player
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ PLAYER ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  common.MessageSubscribe( Event_AddPlayhavenReward, function ( msg_type, msg_params )

    private.Playhaven_AddReward( msg_params );

  end );
  
  private.section = 1;

  private.buffer           = {};
  private.buffer.gold      = {};
  private.buffer.resource1 = {};
  private.buffer.resource2 = {};
  private.buffer.resource3 = {};

  private.default_name = StringGet( "str_player_name" );

  local donate_hearts = m_storage.Settings( "hearts" ).donate_count;
  private.donate_hearts_count = donate_hearts.default;

  if (IsPC()) then
    private.donate_hearts_count = donate_hearts.pc;
  end;

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetDonateHeartsCount ()

  return private.donate_hearts_count;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetUserId ()
  
  return loader.GetUserId();

end;
--------------------------------------------------------------------------------
function public.GetName ()

  local name = m_storage.Player.Info().name;
  
  return tern( name )( name, private.default_name );

end;
--------------------------------------------------------------------------------
function public.SetName ( new_name )

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        = 
    {
      name = new_name
    }
  };

  if ( m_storage.Execute( query ).result ) then

    mediator.Notify( "Model_PlayerNameChanged", new_name );
  
  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetGoldCount ()

  if ( private.buffer.gold[ 1 ] ) and ( private.buffer.gold[ 1 ].count >= 0 ) then

    return private.buffer.gold[ 1 ].count;

  end;

  return m_storage.Player.Info().gold;

end;
--------------------------------------------------------------------------------
function public.TryChangeGoldCount ( delta, hidden )

  if ( delta == 0 ) then

    return false;

  end;

  local count_current = m_storage.Player.Info().gold;

  local count_new = count_current + delta;

  if ( count_new < 0 ) then

    mediator.Notify( "Model_Player_GoldCountChangeFailed", { count = math.abs( count_new ) } );

    return false;

  end;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set = 
    {
      gold = count_new
    }
  };

  m_storage.Execute( query );

  if not ( hidden ) then

    if ( #private.buffer.gold > 0 ) then
      
      private.Gold_BufferShift( delta );
      
    end;
    
    mediator.Notify( "Model_Player_GoldCountChanged",  { count = count_new - delta, delta = delta } );
    
  else

    private.Gold_BufferAppend( delta, count_new - delta );

  end;

  return true;

end;
--------------------------------------------------------------------------------
function private.Gold_BufferAppend ( delta, count )
  
  local notification = "Model_Player_GoldCountBuffered";
  
  table.insert( private.buffer.gold, { count = count, delta = delta } );
  mediator.Notify( notification,     { count = count, delta = delta } );

  -- DbgTrace( private.buffer.gold );
  
end;
--------------------------------------------------------------------------------
function private.Gold_BufferShift ( delta )

  for i = 1, #private.buffer.gold do
    
    local buff_count = private.buffer.gold[ i ].count;
    
    private.buffer.gold[ i ].count = buff_count + delta;
    
  end;
  
end;
--------------------------------------------------------------------------------
function public.Gold_GetBuffered ()
  
  return table.remove( private.buffer.gold, 1 );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetResourceCount ( resource )
  
  if ( private.buffer[ resource ][ 1 ] ) and 
     ( private.buffer[ resource ][ 1 ].count >= 0 ) then

    return private.buffer[ resource ][ 1 ].count;

  end;
  
  local result = m_storage.Player.Info()[ resource ][ private.section ];
    
  if not ( result ) then
    
    local set = {};
    
    set[ resource ] = m_storage.Player.Info()[ resource ];
    set[ resource ][ private.section ] = -1;

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        = set
    };

    m_storage.Execute( query );
    
  end;
  
  return result or -1;
    
end;
--------------------------------------------------------------------------------
function public.HasEnoughResource ( resource_name, required_count )
  
  local available_count = public.GetResourceCount( resource_name );
  
  return ( available_count > required_count );
  
end;
--------------------------------------------------------------------------------
function public.TryChangeResourceCount ( resource, delta, hidden, section )

  if ( delta == 0 ) then

    return false;

  end;

  local section_mismatch = tern( section )(
    ( private.section ~= section ),
    false
  );

  section = section or private.section;

  local count_current = m_storage.Player.Info()[ resource ][ section ];
        count_current = tern( count_current < 0 )( 0, count_current );

  local count_new = count_current + delta;

  if ( count_new < 0 ) then

    mediator.Notify(
      "Model_Player_ResourceCountChangeFailed",
      {
        name  = resource,
        count = math.abs( count_new )
      }
    );

    return false;

  end;

  local set = {
    [ resource ] = m_storage.Player.Info()[ resource ]
  };

  set[ resource ][ section ] = count_new;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        = set
  };

  m_storage.Execute( query );

  -- JK: don't notify and buffer if player is another section
  if ( section_mismatch ) then

    return true;

  end;

  if ( not hidden ) then 

    if ( #private.buffer[ resource ] > 0 ) then
      
      private.Resource_BufferShift( resource, delta );
      
    end;
    
    mediator.Notify(
      "Model_Player_ResourceCountChanged",
      {
        name  = resource,
        count = count_new - delta,
        delta = delta
      }
    );
    
  else
    
    private.Resource_BufferAppend(
      resource,
      delta,
      count_new - delta
    );

  end;

  return true;

end;
--------------------------------------------------------------------------------
function public.Resources_TryDecrease ( resource_data, point )

  local deficient = {};
  
  for i = 1, #resource_data do

    local count_current = public.GetResourceCount( resource_data[ i ].name );
    
    if ( count_current < 0 ) then

      count_current = 0;

    end;

    local count_new = count_current + resource_data[ i ].delta;

    if ( count_new < 0 ) then

      table.insert(
        deficient,
        {
          name  = resource_data[ i ].name,
          count = math.abs( count_new )
        }
      );

    end;

  end;

  if ( #deficient > 0 ) then

    mediator.Notify(
      "Model_Player_ResourceCountChangeFailed",
      deficient
    );

    return { result = false };

  end;

  for i = 1, #resource_data do

    public.TryChangeResourceCount(
      resource_data[ i ].name,
      resource_data[ i ].delta
    );

  end;

  return { result = true, resources = resource_data };

end;
--------------------------------------------------------------------------------
function private.Resource_BufferAppend ( resource, delta, count )
  
  local notification = "Model_Player_ResourceCountBuffered";
  
  table.insert( private.buffer[ resource ], { count = count, delta = delta } );
  mediator.Notify( notification, 
                           { name = resource, count = count, delta = delta } );
  
end;
--------------------------------------------------------------------------------
function private.Resource_BufferShift ( resource, delta )

  for i = 1, #private.buffer[ resource ] do
    
    local buff_count = private.buffer[ resource ][ i ].count;
    
    private.buffer[ resource ][ i ].count = buff_count + delta;
    
  end;
  
end;
--------------------------------------------------------------------------------
function public.Resource_GetBuffered ( resource )
  
  local res = table.remove( private.buffer[ resource ], 1 );
  
  if ( res ) then
    
    res.name = resource;
    
  end;
  
  return res;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetHeartsCount ()

  return m_storage.Player.Info().hearts.count or 0;

end;
--------------------------------------------------------------------------------
function public.TryChangeHeartsCount_Facebook ()

  local hearts_info = m_storage.Player.Info().hearts;

  if ( hearts_info.facebook < 1 ) then

    hearts_info.facebook = 1;

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        = { hearts = hearts_info }
    };

    m_storage.Execute( query );

    public.TryChangeHeartsCount( 1, "facebook" );
    
    common.Kontagent_LogEvent( "social", 
    {
      description = "social_login",
      category    = "hearts",
      name        = "increase"
    } );

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.TryChangeHeartsCount_Donate ()

  local hearts_info = m_storage.Player.Info().hearts;

  local donate = private.donate_hearts_count;

  if ( hearts_info.donate < donate ) then

    hearts_info.donate = donate;

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        = { hearts = hearts_info }
    };

    m_storage.Execute( query );

    public.TryChangeHeartsCount( donate, "donate" );

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.TryChangeHeartsCount ( count_delta, reason )

  local hearts_info   = m_storage.Player.Info().hearts;
  local count_current = hearts_info.count;
  local count_new     = count_current + count_delta;

  if ( count_new < 0 ) then

    mediator.Notify( "Model_Player_HeartsCountChangeFailed" );

    return false;

  end;

  local count_max  = m_hearts.GetMaxHearts();
  local delta_real = count_delta;

  if ( count_new > count_max ) then

    delta_real = count_max - count_current;  
    count_new  = count_max;

  end;

  if ( delta_real == 0 ) then

    mediator.Notify( "Model_Player_HeartsCountNotChanged" );

    return false;

  end;

  hearts_info.count = count_new;

  local query =
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        = { hearts = hearts_info }
  };

  m_storage.Execute( query );

  mediator.Notify(
    "Model_Player_HeartsCountChanged",
    {
      count = count_new,
      max   = count_max,
      delta = delta_real,
      reason = reason
  } );

  if ( count_new < count_max ) then

    m_hearts.HeartsRecoverStart();

    if ( count_new == 0 ) then
  
      common.Playhaven_LogPlacement( "lives_zero" );

    end;

  else

    m_hearts.HeartsRecoverStop();

  end;

  return true;

end;
--------------------------------------------------------------------------------
--function .. () end;
--------------------------------------------------------------------------------
function public.AddItem ( item_name, count )

  m_quests.Event_CollectItem( item_name, count );

end;
--------------------------------------------------------------------------------
--function .. () end;
--------------------------------------------------------------------------------
function public.PayingStatus_Set ( price )

  local ispaying = m_storage.Player.Info().ispaying or 0;
  ispaying = tern( ispaying == true )( 0.99, ispaying );
   
  ispaying = ispaying + tonumber( price:match( "$%s*(.+)" ) );
  
  local query =
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        =
    {
      ispaying = ispaying
    }
  };

  if not ( m_storage.Execute( query ).result ) then

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "info",
      value      = 
      {
        ispaying = ispaying
      }
    };

    m_storage.Execute( query );

  end;

end;
--------------------------------------------------------------------------------
function public.PayingStatus_Get ()

  local ispaying = m_storage.Player.Info().ispaying or 0;
  ispaying = tern( ispaying == true )( 0.99, ispaying );
  
  local paying_status = 1;
  paying_status = tern( ispaying > 0 )( 2, paying_status );
  paying_status = tern( ispaying > 0.99 )( 3, paying_status );
  paying_status = tern( ispaying > 9.99 )( 4, paying_status );
  paying_status = tern( ispaying > 49.99 )( 5, paying_status );
  
  return paying_status;
  
end;
--------------------------------------------------------------------------------
--function .. () end;
--------------------------------------------------------------------------------
function public.Section_SetCurrent ( section_number )

  private.section = section_number;
  
  private.buffer.resource1 = {};
  private.buffer.resource2 = {};
  private.buffer.resource3 = {};
  
end;
--------------------------------------------------------------------------------
function public.Section_GetCurrent ()

  return private.section;
  
end;
--------------------------------------------------------------------------------
function public.Section_GetDiscoveredCount ()
  
  local section_list = m_map.Section_GetList();
  
  return #section_list;
  
end;
--------------------------------------------------------------------------------
--function .. () end;
--------------------------------------------------------------------------------
function public.Playhaven_GetPushNotificationsReward ()

  return m_storage.Settings( "notifications_reward" );

end;
--------------------------------------------------------------------------------
function public.Playhaven_RegisterPushNotifications ( reward )

  wiz.bfg.playhaven.RegisterPushNotifications();
  
  if ( reward ) then
    
    public.TryChangeGoldCount( reward );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Playhaven_AddReward ( params )

  local func_handler = 
  {
    gold_coin = public.TryChangeGoldCount
  };

  if ( params ) then

    if ( func_handler[ params.name ] ) then

      func_handler[ params.name ]( params.count );

    end;

  end;

end;
--------------------------------------------------------------------------------
--function .. () end;
--------------------------------------------------------------------------------
function public.GetStartPlayerName ()
  local player_name = public.GetPlayerName();
  if ( not player_name ) or ( player_name == "Player" or player_name == "" ) then
    local new_name    = ne.strings.Get( "str_player_name_"..math.random( 10 ) );
    local new_surname = ne.strings.Get( "str_player_surname_"..math.random( 10 ) );
    player_name = new_name.." "..new_surname;
  end;
  return player_name;
end;
--------------------------------------------------------------------------------
function public.IsPlayerNameSet ()
  local player_name = public.GetPlayerName();
  if ( not player_name ) or ( player_name == "Player" or player_name == "" ) then
    return false;
  end;
  return true;
end;
--------------------------------------------------------------------------------
function public.GetPlayerName ()

  local name_query =
  {
    query      = "find",
    collection = "player/info",
    result     = "name"
  };
  local player_name = DbExecuteQuery( name_query ).result;

  return player_name;

end;
--------------------------------------------------------------------------------
function public.SetPlayerName ( name_new, silent ) 
   m_client.WaitingForAnswer( "user", "UpdateUserName", { name = name_new }, 
   function ( message_type, message_params )
      if ( not message_params.result ) then
         mediator.Notify( "Model_Player_NameChangeFailed" );
         return;
      end;
      private._SetPlayerName( message_params.name );
      mediator.Notify( "Model_Player_NameChanged", { name = message_params.name } );
   end, 
   silent );
end;
--------------------------------------------------------------------------------
function private._SetPlayerName ( name_new )

  local update_query =
  {
    query      = "update",
    collection = "player/info",
    set        = 
    {
      name = name_new
    }
  };
  local update_answer = DbExecuteQuery( update_query );

end;

