-- name=m_shop
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ SHOP ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.platform = GetPlatform();
  
  ne.msg.Subscribe( "Event_Starterpack_Expired", function ( msg, params )
    public.Starterpack_Finish( params.name );
  end );
  
  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Gold_GetLots ( lot_type )

  local query =
  {
    query      = "find_all",
    group      = "static",
    collection = "lots"
  };

  local lot_data_list = m_storage.Execute( query ).result;

  local lots_list = {};

  for i = 1, #lot_data_list do

    local lot = lot_data_list[ i ];

    if ( lot.type == "shop" ) and 
    lot.name:find( lot_type ) and private.IsLotBelongsPlatform( lot.name_ios ) then

      table.insert( lots_list, lot );

    end;

  end;

  return lots_list;

end;
--------------------------------------------------------------------------------
function public.Gold_GetLotByIOSName ( name_ios )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "lots",
    indice     = "name_ios",
    criteria   = { like = name_ios }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.Gold_TryBuyLot ( lot_name, lot_type, source, transaction_id )

  if ( lot_type == "gold" ) then

    local query =
    {
      query      = "find",
      group      = "static",
      collection = "lots",
      indice     = "name",
      criteria   = { like = lot_name }
    };

    local info = m_storage.Execute( query ).result;
    
    local lot_data = {
      name_ios = info.name_ios,
      count    = info.currency_count,
      bonus    = info.currency_count_bonus,
      price    = info.price
    };

    m_player.PayingStatus_Set( info.price );

    m_player.TryChangeGoldCount( lot_data.count + lot_data.bonus );

    common.Playhaven_LogPlacement( lot_data.name_ios );
    
    if ( source.name == "shop" ) then
      
      local category = "store";
      
      if ( info.type == "playhaven" ) then
        category = "playhaven";
      end;
  
      common.Kontagent_LogEvent( "economy", 
      {
        description = "premium_currency_collect",
        category    = category,
        value       = ( lot_data.count + lot_data.bonus ),
        name        = lot_data.name_ios,
        transaction_id = transaction_id
      } );      
      
    elseif ( source.name == "minibank" ) then  

      common.Kontagent_LogEvent( "economy", 
      {
        description = "context_offer",
        category    = lot_data.name_ios,
        value       = ( lot_data.count + lot_data.bonus ),
        name        = source.lots[ 1 ].name_ios,
        level       = 1,
        transaction_id = transaction_id
      } );
      
    end;

  end;

  common.Save(true);
  NetRestartHeartBeats();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Boosters_GetLots ()

  local boosters_list = m_boosters.GetList();
  local boosters_data = m_boosters.GetBoostersData( boosters_list );

  for i = #boosters_data, 1, - 1 do

    if ( boosters_data[ i ].type == "continue" ) then

      table.remove( boosters_data, i );

    end;

  end;

  return boosters_data;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Starterpack_GetLotByIndex ( starterpack_index )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "lots",
    indice     = "name",
    criteria   = { like = "starterpack_"..starterpack_index }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function public.Starterpack_GetActualPrice ( starterpack_name_ios )

  local lots = ( GetLotsInfo() or {} ).lots or {};

  for l = 1, #lots do
    if ( lots[ l ].name_ios == starterpack_name_ios ) then
      return lots[ l ].price;
    end;
  end;

  return nil;

end;
--------------------------------------------------------------------------------
function public.Starterpack_IsAvailable ( starterpack_name )
  
  if ( public.Starterpack_IsCompleted( starterpack_name ) ) then

    return false;

  end;
  
  local timer_exists = TimeIsEventExist( "mte_starterpack_cooldown" );
  
  local unlock_stage = m_storage.Settings( "starterpacks" ).unlock_stage;
  
  local stage_completed = ( m_stage.GetSavedData( unlock_stage ) or {} ).completed;
  
  if ( not stage_completed ) then
    if ( timer_exists ) then
      public.Starterpack_Finish( starterpack_name );
      public.Starterpack_Clear( starterpack_name );
    end;
    return false;
  else
    if ( not timer_exists ) then
      public.Starterpack_Start( starterpack_name );
    end;
    return true;
  end;
  
  return false;

end;
--------------------------------------------------------------------------------
function public.Starterpack_Buy ( lot_id, transaction_id )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "lots",
    indice     = "name_ios",
    criteria   = { like = lot_id }
  };

  local starterpack = m_storage.Execute( query ).result;
  
  if ( starterpack and type( starterpack.currency_count ) == "table" ) then
  
    m_player.PayingStatus_Set( starterpack.price );
    
    common.Kontagent_LogEvent( "economy", 
    {
      description = "starterpack_collect",
      category    = "store",
      value       = 1,
      name        = starterpack.name_ios,
      transaction_id = transaction_id
    } );  
    
    for i = 1, #starterpack.currency_count do
      
      local starterpack_content = starterpack.currency_count[ i ];
      
      m_items.TryObtain( {
        name       = starterpack_content.name,
        type       = starterpack_content.type,
        count      = starterpack_content.count,
        category   = "starterpack",
        stage_name = "starterpack",
        silent     = false
      } );
 
    end;
    
    public.Starterpack_Finish( starterpack.name );
    
  end;
  
end;
--------------------------------------------------------------------------------
function public.Starterpack_Start ( starterpack_name )

  --DbgTrace( "Starterpack Start: "..starterpack_name );

  local timer_token = "mte_starterpack_cooldown";
  local expire_time = m_storage.Settings( "starterpacks" ).expire_time;

  TimeSheduleEvent(
    timer_token,
    "GLOBAL_TIME",
    expire_time,
    "Event_Starterpack_Expired",
    {
      name = starterpack_name
    }
  );

  mediator.Notify( "Model_Shop_Starterpack_Start", { name = starterpack_name } );

end;
--------------------------------------------------------------------------------
function public.Starterpack_Finish ( starterpack_name )

  --DbgTrace( "Starterpack Finish: "..starterpack_name );

  local timer_token = "mte_starterpack_cooldown";

  TimeFinishEvent();
  TimeUnsheduleEvent( timer_token );
  
  if ( public.Starterpack_IsCompleted( starterpack_name ) ) then

    return;

  end;
  
  local query =
  {
    query      = "insert",
    group      = "player",
    collection = "starterpacks",
    value      = 
    {
      name = starterpack_name
    }
  };

  m_storage.Execute( query );
  
  mediator.Notify( "Model_Shop_Starterpack_Finish", { disable = true } );

end;
--------------------------------------------------------------------------------
function public.Starterpack_IsCompleted ( starterpack_name )

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "starterpacks",
    indice     = "name",
    criteria   = 
    {
      like = starterpack_name
    }
  };

  if ( m_storage.Execute( query ).result ) then

    return true;
    
  end;
  
  return false;

end;
--------------------------------------------------------------------------------
function public.Starterpack_Clear ( starterpack_name )
  
  --DbgTrace( "Starterpack Clear: "..starterpack_name );

  local query =
  {
    query      = "delete",
    group      = "player",
    collection = "starterpacks",
    indice     = "name",
    criteria   = 
    {
      like = starterpack_name
    }
  };

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Tab_Available ( tab )

  local tabs = 
  {
    market = m_market.IsUnlocked
  };
  
  if ( tabs[ tab ] ) then
    
    return tabs[ tab ]();
    
  end;
  
  return true;

end;
--------------------------------------------------------------------------------
function public.Tab_GetUnlock ( tab )

  local tabs = 
  {
    market = function () return m_market.Settings( "unlock" ); end
  };
  
  if ( tabs[ tab ] ) then
    
    return tabs[ tab ]();
    
  end;
  
  return false;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.IsLotBelongsPlatform ( lot_name )

  local lots = {
    ios     = { "coins2tiern", "coins5tiern1", "coins10tiern",
            "coinssku4", "coinssku05", "coins60tiern" },
    pc      = { "coins2tiern", "coins5tiern1", "coins10tiern",
           "coins20tiern", "coins40tiern", "coins60tiern" },
    android = { "coins2tier", "coins5tier", "coins10tier",
            "coins20tier", "coins40tier", "coins60tier" }
  };
  
  if ( cmn.IsEntityInList( lots[ private.platform ], lot_name ) ) then
    
    return true;
    
  end;
  
  return false;

end;
--------------------------------------------------------------------------------