-- name=m_skins
--------------------------------------------------------------------------------
function public.Init ()

  private.empty_manifest = false;

  local query =
  {
    query      = "find_all",
    group      = "parts",
    collection = "content_packets"
  };

  local packet_list = m_storage.Execute( query ).result;

  if ( ( #packet_list == 1 ) and ( packet_list[ 1 ].hash == '0' ) ) or
    ( #packet_list == 0 ) then

    private.empty_manifest = true;

  end;
  
  private.platform = GetPlatform();
  private.platform = tern( private.platform == "android" )( 
                            wiz.Android.GetPlatform(), 
                            private.platform );

  private.UpdateSkinBonus();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.TryObtainPlatformSkins ()

  local query =
  {
    query      = "find_all",
    group      = "static",
    collection = "skins",
    indice     = "platform",
    criteria   = { like = private.platform }
  };

  local platform_skins = m_storage.Execute( query ).result;
  
  for i = 1, #platform_skins do
    
    local platform_skin = platform_skins[ i ];
    
    if ( not platform_skin.parts ) or ( platform_skin.parts == 0 ) then
      
      query = 
      {
        query      = "find",
        group      = "player",
        collection = "skins",
        indice     = "name",
        criteria   = { like = platform_skin.name }
      };
      
      if not ( m_storage.Execute( query ).result ) then
        
        query = 
        {
          query      = "insert",
          group      = "player",
          collection = "skins",
          value      = {
            name    = platform_skin.name
          }
        };
        m_storage.Execute( query );
     
        common.Kontagent_LogEvent( "skins", 
        {
          description = "skin_collect",
          category    = "",
          value       = 1,
          name        = platform_skin.name
        } );

        mediator.SubscribeOnce(
          "View_Map_InterfaceVisible",
          function ()

            -- JK: send messages strictly in this order

            mediator.Notify(
              "Model_Skins_PlatformSkinCollected",
              private.platform
            );

            mediator.Notify(
              "Model_Skins_SkinCollected",
              platform_skin.name
            );

          end
        );
        
      end;

    end;
    
  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.GetStats ()

  return private.current_skin_stats;

end;
--------------------------------------------------------------------------------
function public.GetSkin ()

  local query =
  {
    query      = "find",
    group      = "player",
    collection = "hero"
  };
  
  local skin = m_storage.Execute( query ).result.skin;
  local skin_platform = private.GetDatabaseInfo( skin ).platform;
  
  if ( not skin_platform ) then
    return skin;
  end;
  
  skin = tern( skin_platform and skin_platform == private.platform )( skin, 
                                                                     "default" );

  return skin;

end;
--------------------------------------------------------------------------------
function public.GetContent ( skin_name )

  local query =
  {
    query      = "find",
    group      = "static",
    collection = "skins",
    indice     = "name",
    criteria   = { like = skin_name }
  };

  local content = m_storage.Execute( query ).result.content;

  return content.."/"..skin_name;

end;
--------------------------------------------------------------------------------
function public.GetSkinByPart ( part_name )

  -- Part name prefix "braveheart" for "braveheart_boots" 
  return string.match( part_name, "%w+" );

end;
--------------------------------------------------------------------------------
function public.TrySetSkin ( skin_name )

  local db_info   = private.GetDatabaseInfo( skin_name );
  local save_info = private.GetSaveInfo( skin_name );

  local exists = false;
  if ( db_info ) then exists = true; end;

  local partless = exists and ( not db_info.parts or db_info.parts == 0 );

  local saved = false;
  if ( save_info ) then saved = true; end;

  local completed = exists and saved
                    and db_info.parts and save_info.parts 
                    and save_info.parts >= db_info.parts;

  if ( exists ) and ( partless or completed ) then
    
    common.Kontagent_LogEvent( "skins", 
    {
      description = "skin_switch",
      category    = public.GetSkin(),
      name        = skin_name
    } );

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "hero",
      set        = { skin = skin_name }
    };
    
    m_storage.Execute( query );

    private.UpdateSkinBonus();
    
    mediator.Notify( "Model_Skins_SkinSet", skin_name );
    
    return true;

  else

    return false;

  end;

end;
--------------------------------------------------------------------------------
function public.TryObtainParts ( part_name, count, category )

  local skin_name = public.GetSkinByPart( part_name );

  local skin_data  = private.GetDatabaseInfo( skin_name );
  local saved_data = private.GetSaveInfo( skin_name );

  if ( skin_data ) then

    local current = ( saved_data or {} ).parts or 0;
    local total   = skin_data.parts;

    if ( current >= total ) then return false; end;

    local new_count = math.min( current + count, total );

    if ( saved_data ) then

      saved_data.parts = new_count;

      local query = 
      {
        query      = "update",
        group      = "player",
        collection = "skins",
        indice     = "name",
        criteria   = { like = skin_name },
        set        = saved_data
      };
      
      m_storage.Execute( query );

    else

      local query = 
      {
        query      = "insert",
        group      = "player",
        collection = "skins",
        value      = {
          name    = skin_name,
          parts   = new_count
        }
      };
      
      m_storage.Execute( query );

    end;

    mediator.Notify( "Model_Skins_PartCollected",
    {
      skin = skin_name,
      part = part_name
    } );

    local kontagent_description = "token_collect";
    local kontagent_category = category or "unknown";    
    local kontagent_value = count;
    
    common.Kontagent_LogEvent( "skins", 
    {
      description = kontagent_description,
      category    = kontagent_category,
      value       = kontagent_value,
      name        = skin_name,
      tokens      = new_count..", "..total 
    } );
    
    if ( new_count >= total ) then
      
      kontagent_description = "skin_collect";
      kontagent_category = "";
      kontagent_value = 1;
      
      common.Kontagent_LogEvent( "skins", 
      {
        description = kontagent_description,
        category    = kontagent_category,
        value       = kontagent_value,
        name        = skin_name
      } );      

      mediator.Notify( "Model_Skins_SkinCollected", skin_name );

    end;
   
    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.GetSkinInfo ( skin_name )

  local skin;

  local player_skin = public.GetSkin();
  local db_info     = private.GetDatabaseInfo( skin_name );
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  if ( db_info.parts ) then

    skin = {
      name   = db_info.name,
      bonus  = db_info.bonus,
      parts  = db_info.parts,
      weared = ( player_skin == skin_name )
    };

    local save_info = private.GetSaveInfo( db_info.name );

    if ( save_info ) and ( save_info.parts ) then
      skin.collected_parts = save_info.parts;
    else
      skin.collected_parts = 0;
    end;

    if ( skin.collected_parts >= skin.parts ) then
      skin.wearable = true;
    else
      skin.wearable = false;
    end;

  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  else

    skin = {
      name  = db_info.name,
      bonus = db_info.bonus
    };

    skin.wearable = true;

  end;
  
  skin.primary = db_info.primary;

  return skin;

end;
--------------------------------------------------------------------------------
function public.GetAllSkinsInfo ()

  local skins          = {};
  local player_skin    = public.GetSkin();
  local skins_database = private.GetDatabaseInfo();

  local result, value = parts.storage.data.Read( "sml/allow_sections" );

  for s = 1, #skins_database do

    local db_info = skins_database[ s ];

    if ( tonumber( db_info.content:match( "%d+" ) ) <=
         tonumber( value ) ) then

      if ( not m_content.IsPacketReady( db_info.content ) ) then
        return false, "obsolete";
      end;

      local skin = {
        name     = db_info.name,
        bonus    = db_info.bonus,
        content  = public.GetContent( db_info.name ),
        wearable = false,
        weared   = ( player_skin == db_info.name )
      };
      
      db_info.parts = db_info.parts or 0;
      
      if ( db_info.parts ) then

        skin.parts           = db_info.parts;
        skin.collected_parts = 0;

        local save_info = private.GetSaveInfo( db_info.name );

        if ( save_info ) and ( save_info.parts ) then
          
          skin.collected_parts = save_info.parts;
          
        end;

        if ( skin.collected_parts >= skin.parts ) then
          
          skin.wearable = true;
          
        end;

      end;
      
      if ( not db_info.platform                 ) or
         ( db_info.platform == private.platform ) then

        table.insert( skins, skin );

      end;
  
    end;

  end;

  return skins, false;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Skinpart_CollectableCount ( part_name )

  local skin_name = public.GetSkinByPart( part_name );

  local skin_data  = private.GetDatabaseInfo( skin_name );
  local saved_data = private.GetSaveInfo( skin_name ) or {};

  if ( skin_data ) then

    local current = saved_data.parts or 0;
    local total   = skin_data.parts  or 0;

    return ( total - math.min( current, total ) );

  end;

  return 0;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.UpdateSkinBonus ()

  private.current_skin_stats = private.GetStats();

end;
--------------------------------------------------------------------------------
function private.GetStats ()

  local skin_name  = public.GetSkin();
  local skin_bonus = private.GetDatabaseInfo( skin_name ).bonus or {};
  return {
    gold          = skin_bonus.gold         or 0,
    moves         = skin_bonus.moves        or 0,
    score         = skin_bonus.score        or 0,
    attack        = skin_bonus.attack       or 0,
    health        = skin_bonus.health       or 0,
    resource1     = skin_bonus.resource1    or 0,
    resource2     = skin_bonus.resource2    or 0,
    resource3     = skin_bonus.resource3    or 0,
    enemy_attack  = skin_bonus.enemy_attack or 0,
    enemy_health  = skin_bonus.enemy_health or 0,
    fox           = skin_bonus.fox          or 0
  };

end;
--------------------------------------------------------------------------------
function private.GetDatabaseInfo ( skin_name )

  local query;

  if ( skin_name ) then

    query =
    {
      query      = "find",
      group      = "static",
      collection = "skins",
      indice     = "name",
      criteria   = { like = skin_name }
    };

  else

    query =
    {
      query      = "find_all",
      group      = "static",
      collection = "skins"
    };

  end;

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function private.GetSaveInfo ( skin_name )

  local query;

  if ( skin_name ) then

    query =
    {
      query      = "find",
      group      = "player",
      collection = "skins",
      indice     = "name",
      criteria   = { like = skin_name }
    };

  else

    query =
    {
      query      = "find_all",
      group      = "player",
      collection = "skins"
    };

  end;

  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------