-- name=m_social
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ SOCIAL ] "..text );

end;
--------------------------------------------------------------------------------
function  public.Init ()
  if ( IsMobile() ) then

    private.Trace( "Initialized." );
    private.Section = "Social";
    private.userSection = "user";

    private.update_cooldown     = private.GetUpdateCooldown();
    private.update_token        = "tmr_social_update_friends";
    private.friends             = {};
    private.friends_stages = {};
    private.friends_info = {};

    mediator.Subscribe( "Model_Client_ConnectionLost",     private.UpdateOnline );
    mediator.Subscribe( "Model_Client_ConnectionRestored", private.UpdateOnline );
    
    MsgSubscribe( "Message_FixedUrlReceived_Self", 
      function ( message, reply )
      
        if ( reply.data and reply.data.data and reply.data.data.url ) then
        
          private.fixed_self_avatar_url = 
            reply.data.data.url:gsub( "https", "http" );
            
          private.Server_SetSocialAvatar( private.fixed_self_avatar_url );
        
        end;
    
      end
    );
    
  end;
  
end;
--------------------------------------------------------------------------------
function  public.Start ()

  if ( IsMobile() ) then
  
    private.is_online = m_client.IsOnline();

    common.MessageSubscribe( "Event_Social_FriendsCollectionsCooldownEnd", private.UpdateFriendsCollections );
    common.MessageSubscribe( Event_Social_FirstAuthenticatesRaveId, function ()
      common.Kontagent_LogEvent( "social",
      {
        description = "account_profile",
        name        = "create_raveid"
      } );
    end );

    common.MessageSubscribe( Event_Social_RaveLoginFail, function ()
      common.Kontagent_LogEvent( "social",
      {
        description = "social_login",
        name        = "login_fail"
      } );
    end );

    mediator.Subscribe( "Model_Social_LoginComplete", private.RequestFriendsCollections );

    if ( private.is_online ) then

      if ( public.IsLogged() ) then
        private.GetFriendList();
      end;

    else

      local func_try_get_friends = function ()

        if ( public.IsLogged() ) then
          private.GetFriendList();
        end;

      end;

      mediator.SubscribeOnce( "Model_Client_ConnectionRestored", func_try_get_friends );

    end;
    
  end;

end;
--------------------------------------------------------------------------------
function private.UpdateOnline ()

  private.is_online = m_client.IsOnline();

end;
--------------------------------------------------------------------------------
-- function ***** AUTH ***** ()
--------------------------------------------------------------------------------
function  public.IsLogged ()

  local answer = SocialIsLogged();

  return answer.result;

end;
--------------------------------------------------------------------------------
function  public.Login ()

  private.Trace( "Login" );

  local func_wait;

  func_wait = function ( message_name, message_params )

    common.MessageUnsubscribe( "Event_Social_Login_Complete", func_wait );

    m_client.StopWaiting( "social", "login" );

    if ( not message_params.result ) then

      private.Trace( "Login: failed." );

      mediator.Notify( "Model_Social_LoginComplete", message_params.result );

      return;
  
    end;

    common.Kontagent_LogEvent( "social",
    {
      description = "social_login",
      name        = "login"
    } );

    if not ( message_params.socialId == message_params.previousSocialId ) then

      private.Trace( "Login: replace." );

      private.LoginReplace( message_params.socialId );

      return;
  
    end;

    private.LoginContinue( message_params.socialId );

  end;

  common.MessageSubscribe( "Event_Social_Login_Complete", func_wait );

  m_client.StartWaiting( "social", "login" );
  SocialLogin();

end;
--------------------------------------------------------------------------------
function  public.Logout ()

  private.Trace( "Logout" );

  local func_wait;

  func_wait = function ( message_name, message_params )

    common.MessageUnsubscribe( "Event_Social_Logout_Complete", func_wait );

    m_client.StopWaiting( "social", "logout" );

    if ( message_params.result ) then

      if ( message_params.socialId ~= message_params.previousSocialId ) then

        parts.application.RestartWithLocalUserIdReplace( message_params.socialId, true );

      end;
      
      common.Kontagent_LogEvent( "social", 
      {
        description = "social_login",
        name        = "logout"
      } );

    end;

  end;

  common.MessageSubscribe( "Event_Social_Logout_Complete", func_wait );

  m_client.StartWaiting( "social", "logout" );

  SocialLogout();

end;
--------------------------------------------------------------------------------
function private.LoginContinue ( social_id )

  private.Trace( "LoginContinue: [ "..social_id.." ]" );

  local social_info = public.GetSocialInfo();

  private.Server_SetSocialId( social_id );
  private.Server_SetSocialName( social_info.name );
  private.Server_SetSocialAvatar( social_info.url );

  private.GetFriendList();

  mediator.Notify( "Model_Social_LoginContinue" );

end;
--------------------------------------------------------------------------------
function private.LoginReplace ( social_id )

  private.Trace( "LoginReplace: [ "..social_id.." ]" );

  m_client.WaitingForAnswer( private.userSection, "tryReplaceLocalUserId", { newLocalUserId = social_id }, function ( message_name, message_params )

    private.Trace( "LoginReplace: [ "..social_id.." ] "..tostring( message_params.result ) );
  
    if ( not message_params.result ) then
       
      local conflict_params = {};
      conflict_params.func_local = function () private.LoginSelect( social_id, true ); end;
      conflict_params.func_server = function () private.LoginSelect( social_id, false ); end;
      conflict_params.info_local = message_params.my_profile;
      conflict_params.info_server = message_params.new_profile;
      mediator.Notify( "Event_Social_RaveLoginConflict", conflict_params );

      return;

    end;

    common.MessageSend( "Event_Application_LocalUserIdReplaced", {} );

    private.LoginContinue( social_id );

  end );

end;
--------------------------------------------------------------------------------
function private.LoginSelect ( social_id, keepLocalSave )

  private.Trace( "LoginSelect: restart with id "..social_id );

  mediator.Notify( "Model_Social_LoginComplete", true );

  parts.application.RestartWithLocalUserIdReplace( social_id, true, keepLocalSave );

end;
--------------------------------------------------------------------------------
function private.Server_SetSocialId ( socialId )

  socialId = socialId or "";

  private.Trace( "Server_SetSocialId: "..socialId );
  m_client.WaitingForAnswer( private.Section, "SetSocialId", 
    { socialId = socialId }, 
    function () end, true );

end;
--------------------------------------------------------------------------------
function private.Server_SetSocialName ( name )

  name = name or "";

  private.Trace( "Server_SetSocialName: "..name );
  m_client.WaitingForAnswer( private.userSection, "updateUserName", 
    { name = name }, 
    function () end, true );

end;
--------------------------------------------------------------------------------
function private.Server_SetSocialAvatar ( avatar )

  avatar = avatar or "";

  private.Trace( "Server_SetSocialAvatar: "..avatar );
  m_client.WaitingForAnswer( private.userSection, "updateUserAvatar", 
    { avatar = avatar }, 
    function () end, true );

end;
--------------------------------------------------------------------------------
function  public.GetSocialInfo()

  if ( public.IsLogged() ) then
  
    local info = SocialGetInfo();
    
    local fb = "graph.facebook.com";
    
    if ( IsAndroid() and info.url and info.url:find( fb ) ) then
    
      local get = tern( info.url:find( "?" ) )( "&", "?" ).."redirect=false";
      
      NetSendHttpRequest(
        {
          url = info.url..get
        },
        "Message_FixedUrlReceived_Self"
      );
      
      info.url = private.fixed_self_avatar_url;
    
    end;
    
    return info;
  
  end;
  
  return {
    userId   = m_player.GetUserId(),
    socialId = 0,
    name     = m_player.GetName(),
    score    = 0,
    url      = nil,
    res      = "assets/content/interface/int_dialog_stage_friends/default_0"
  };

end;
--------------------------------------------------------------------------------
-- function ***** FRIENDS ***** ()
--------------------------------------------------------------------------------
function private.UpdateFriendsCollections () 
  private.Trace( "private.UpdateFriendsCollections" );
  if ( private.is_online ) then

    private.UpdateCooldownFinish();
    private.GetFriendList();

  else

    mediator.SubscribeOnce( "Model_Client_ConnectionRestored", private.GetFriendList );

  end;

end;
--------------------------------------------------------------------------------
function  public.GetFriendsInZone ( section )

  private.Trace( "GetFriendsInZone: " );

  local collection_table = private.friends_info or {};

  local friend_list = {};

  if ( #collection_table > 0 ) then

    for u = 1, #collection_table do

      local user_data = collection_table[ u ];
      if ( user_data.collection and #user_data.collection > 0 ) then

        local stage_max = private.GetStageLast( user_data.collection );
        
        if ( stage_max ) then

          local isBonus = stage_max:find( "_bonus" );
          
          if ( not isBonus ) or 
             ( isBonus and section == m_stage.GetStaticData( stage_max ).section ) then
            
            local friend_data = private.GetFriend( user_data.social_id );
            table.insert( friend_list, { 
              userId   = user_data.user_id,
              socialId = user_data.social_id,
              name     = friend_data.name,
              url      = friend_data.url,
              stage    = stage_max
            } );             
             
          end;

        end;

      end;

    end;

  end;

  return friend_list;

end;
--------------------------------------------------------------------------------
function private.GetStageLast ( user_collection )

  return user_collection[ 1 ].stage_last;

end;
--------------------------------------------------------------------------------
function  public.GetFriendsStageScores ( stage_name ) 

  local friends_scores = {};

  if ( IsMobile() ) then
  
    if ( private.is_online and public.IsLogged() ) then
    
      local collection_table = private.friends_stages;
    
      for u = 1, #collection_table do
        local user_data = collection_table[ u ];
        for s = 1, #user_data.collection do
          local stage_data = user_data.collection[ s ];
          if ( not stage_name ) or ( stage_data.name == stage_name and stage_data.score and stage_data.score > 0 ) then
            local friend_data = private.GetFriend( user_data.social_id );
            table.insert( friends_scores, { 
              userId   = user_data.user_id,
              socialId = user_data.social_id,
              name     = friend_data.name,
              url      = friend_data.url,
              score    = user_data.collection[ s ].score
            } );
            break;
          end;
        end;
      end;
      
    end;
    
    local bots = private.GetBotsForStage( stage_name );
    
    for i = 1, #bots do
    
      table.insert( friends_scores, bots[ i ] );
    
    end;

  end;

  return friends_scores;

end;
--------------------------------------------------------------------------------
function private.GetFriendList ()

  private.Trace( "GetFriendList" );
  MsgSubscribe( "Event_Social_Friends_Received",  private.OnFriendsReceived );
  SocialGetFriends();

end;
--------------------------------------------------------------------------------
function private.OnFriendsReceived ( msg, params )

  private.rave_list = {};

  private.Trace( "OnFriendsReceived: "..os.clock() );

  MsgUnsubscribe( "Event_Social_Friends_Received", private.OnFriendsReceived );
  if ( params and params.friends and #params.friends > 0 ) then
    private.Trace( "OnFriendsReceived:"..common.TableToString( params.friends ) );
    local socialId_list = {};
    for i = 1, #params.friends do
      table.insert( socialId_list, params.friends[i].social_id );
      private.rave_list[ params.friends[i].social_id ] = {
        url = params.friends[i].picture_url,
        name = params.friends[i].display_name
      };
    end;
    private.Server_GetFriends( socialId_list );
  else
    mediator.Notify( "Model_Social_LoginComplete", true );
  end;

end;
--------------------------------------------------------------------------------
function public.GetRandomPlayers()
  private.Trace( "GetRandomPlayers");
  private.Server_GetRandomPlayers();
end;
--------------------------------------------------------------------------------
function private.Server_GetRandomPlayers ()
  private.Trace( "Server_GetRandomPlayers" .. m_player.GetUserId());
  m_client.WaitingForAnswer(
    private.Section,
    "GetRandomPlayers", 
    { 
      userId  = m_player.GetUserId()
    },
    private.OnGetRandomPlayers,
    true
  );
end;
--------------------------------------------------------------------------------
function private.OnGetRandomPlayers ( msg, params )
  local gameFriends = params.players;
  private.Trace( "OnGetRandomPlayers: " .. #gameFriends);
  private.Trace( "OnGetRandomPlayers:" .. common.TableToString( gameFriends ) );
end;
--------------------------------------------------------------------------------
function private.Server_GetFriends ( facebookFriends )

  private.Trace( "Server_GetFriends: "..m_player.GetUserId() );

  m_client.WaitingForAnswer(
    private.Section,
    "GetGameFriends", 
    {
      userId  = m_player.GetUserId(),
      friends = facebookFriends
    },
    private.OnGetGameFriends,
    true
  );

end;
--------------------------------------------------------------------------------
function private.OnGetGameFriends ( msg, params )

  private.Trace( "OnGetGameFriends: " );
  private.friends = {};

  local gameFriends = params.friends;
  private.Trace( "OnGetGameFriends: " .. #gameFriends);
  if ( type( gameFriends ) == "table" ) then
    
    private.Trace( "OnGetGameFriends:"..common.TableToString( gameFriends ) );
    
    for f, friend in pairs( gameFriends ) do
      
      table.insert( private.friends, friend );
      
      local url = private.rave_list[ friend.socialId ].url;
      
      --For DSprite compability
      if ( url == "" ) then
        url = nil;
      end;
      
      private.friends[ #private.friends ].url = url;
      
      if ( IsAndroid() and url and url:find( "graph.facebook.com" ) ) then
      
        private.friends[ #private.friends ].url = nil;
        private.RequestFixedAvatar( friend.socialId, url );
      
      end;
      
      private.friends[ #private.friends ].name = private.rave_list[ friend.socialId ].name;
    end;
  end;

  private.rave_list = nil;

  private.Trace( "Call RequestFriendsCollections: "..os.clock() );
  private.SaveFriends();
  mediator.Notify( "Model_Social_LoginComplete", true );

end;
--------------------------------------------------------------------------------
function private.RequestFixedAvatar( socialId, url )
  
  local message = "Message_FixedUrlReceived_"..socialId;
  MsgSubscribe( message, private.OnFixedUrlReceived );
  
  local get = tern( url:find( "?" ) )( "&", "?" ).."redirect=false";
  
  NetSendHttpRequest( { url = url..get }, message );

end;
--------------------------------------------------------------------------------
function private.OnFixedUrlReceived ( message, reply )

  MsgUnsubscribe( message, private.OnFixedUrlReceived );
  
  if ( reply.data and reply.data.data and reply.data.data.url ) then
  
    local socialId = message:match( "^Message_FixedUrlReceived_(.*)$" );
    local url      = reply.data.data.url:gsub( "https", "http" );

    local query =
    { 
      query      = "update",
      group      = "social",
      collection = "friends",
      indice     = "socialId",
      criteria   = { like = socialId },
      set        = { url  = url      }
    };
    
    m_storage.Execute( query );
  
  end;

end;
--------------------------------------------------------------------------------
-- function ***** FRIENDS - COLLECTIONS ***** ()
--------------------------------------------------------------------------------
function private.RequestFriendsCollections () 

  local friendList = private.GetFriends();

  local idList = {};
  for i = 1, #friendList do
    table.insert( idList, friendList[ i ].userId );
  end;

  private.Server_GetFriendsCollections( idList );

end;

--------------------------------------------------------------------------------
function private.Server_GetFriendsCollections ( friendArray )

  private.Trace( "Server_GetFriendsCollections: "..common.TableToString( friendArray )..": "..os.clock() );
  if(next(friendArray) ~= nil) then
    m_client.WaitingForAnswer( private.Section, "GetFriendsCollections",
      { friends = friendArray, collection = { "player/info", "player/stages" } },
      private.OnGetFriendsCollections, true );
  end;

end;
--------------------------------------------------------------------------------
function private.OnGetFriendsCollections ( msg, params )

  private.Trace( "OnGetFriendsCollections: "..#params.collections..": "..os.clock() );

  local collectionTable_stages = {};
  local collectionTable_info = {};

  for i = 1, #params.collections do

      local userData = params.collections[ i ];

      private.Trace( "get collection "..userData.userId.." : "..userData.socialId..": "..os.clock() );
      local convertedData_stages = DbConvertCollectionDataToTable( userData.data.stages );
      local convertedData_info = DbConvertCollectionDataToTable( userData.data.info );

      collectionTable_stages[ i ] = {
        user_id    = userData.userId,
        social_id  = userData.socialId,
        user_name  = userData.userName,
        collection = convertedData_stages
      };

      collectionTable_info[ i ] = {
        user_id    = userData.userId,
        social_id  = userData.socialId,
        user_name  = userData.userName,
        collection = convertedData_info
      };

  end;

  private.UpdateCooldownStart();
  private.friends_stages = collectionTable_stages;
  private.friends_info = collectionTable_info;
  mediator.Notify( "Model_Social_FriendsCollectionsUpdated", collectionTable_info );

end;
--------------------------------------------------------------------------------
function private.UpdateCooldownStart ()

  TimeSheduleEvent( 
    private.update_token,
    "SESSION_GAME_TIME",
    private.update_cooldown,
    "Event_Social_FriendsCollectionsCooldownEnd",
    {}
  );

end;
--------------------------------------------------------------------------------
function private.UpdateCooldownFinish ()

  TimeFinishEvent();
  TimeUnsheduleEvent( private.update_token );

end;
--------------------------------------------------------------------------------
-- function ***** SAVE ***** ()
--------------------------------------------------------------------------------
function  public.GetFriend ( socialId ) 

  return private.GetFriend( socialId );

end;
--------------------------------------------------------------------------------
function  public.GetFriends () 

  return private.GetFriends();

end;
--------------------------------------------------------------------------------
function private.GetFriend ( socialId ) 

  local query =
  { 
    query      = "find",
    group      = "social",
    collection = "friends",
    indice     = "socialId",
    criteria   = { like = socialId }
  };
  
  local result = m_storage.Execute( query ).result;
  
  return result or {};

end;
--------------------------------------------------------------------------------
function private.GetFriends () 

  local query =
  { 
    query      = "find_all",
    group      = "social",
    collection = "friends"
  };
  
  local result = m_storage.Execute( query ).result;
  
  return result or {};

end;
--------------------------------------------------------------------------------
function private.ClearFriends ()

  local query =
  { 
    query      = "delete_all",
    group      = "social",
    collection = "friends"
  };
  
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function private.SaveFriends ()

  if ( #private.friends == 0 ) then
 
    return;

  end;

  private.ClearFriends();
  
  m_storage.Rewrite( "social", "friends", private.friends );

--  for f = 1, #private.friends do
--
--    local query =
--    {
--      query      = "insert",
--      group      = "social",
--      collection = "friends",
--      value      =  private.friends[ f ]
--    };
--
--    m_storage.Execute( query );
--
--  end;

end;
--------------------------------------------------------------------------------
--function *** BOTS *** ()
--------------------------------------------------------------------------------
function private.GetBotsForStage ( stage_name )

  -- Get bot data
  local bot_data   = m_storage.Settings( "social" ).bot_scores;
  local move_score = m_storage.Settings( "social" ).bot_move_score;
  
  local list = {};
  for i = 1, #bot_data do table.insert( list, i ) end;
  
  -- Shuffle bot list
  local seed = stage_name:match( "%d+" );
  math.randomseed( tonumber( seed ) );
  
  for i = #list, 2, -1 do
  
    j = math.random(i);
    list[i], list[j] = list[j], list[i];
    
  end;
  
  local shifts = {};
  for i = 1, #list do table.insert( shifts, math.random( 100 ) ); end;
  
  math.randomseed( os.time() );
  
  -- Get stage state scores
  local scores = {};
  local states = m_stage.GetStaticData( stage_name ).state;
  
  for s = 1, 3 do
  
    local targets = states[ s ].targets;
    for t = 1, #targets do
    
      local target = targets[ t ];
      if ( target.type == "GainScore" ) then
      
        table.insert( scores, target.count );
        break;
      
      end;
    
    end;
  
  end;
  
  -- Generate bot list
  local bots = {};
  
  for i = 1, #list do
  
    local state = bot_data[ i ].state;
    local moves = bot_data[ i ].moves;
    local id    = list[ i ];
    local score = moves * move_score;
    
    local score_raw = ( scores[ state ] + score + shifts[ i ] );
    local score = 10 * math.ceil( score_raw / 10 );
  
    table.insert( bots,
      { 
        userId   = "bot_"..id,
        socialId = "bot_"..id,
        name     = StringGet( "str_social_bot_name_"..id ),
        score    = score,
        url      = nil,
        res      = "assets/content/interface/social/bot_avatar_"..id
      } );
  
  end;
  
  return bots;

end;
--------------------------------------------------------------------------------
--function ***** MISC ***** ()
--------------------------------------------------------------------------------
--[[function private.Fake_GetScore( stage_name, index )

  if ( private.stage and private.stage.name == stage_name ) then

    return ( private.stage.score + 1000 * index + math.random( 999 ) ); 

  end;

  local state_query = 
  {
    query = "find",
    collection = "player/stages:name",
    criteria = { like = stage_name },
    result = "state"
  };

  local state = m_storage.DbExecute( state_query ).result or 1;

  local score_query = 
  {
    query = "find",
    collection = "stages:name",
    criteria = { like = stage_name },
    result = "state."..( state - 1 )..".targets"
  };

  local targets = m_storage.DbExecute( score_query ).result;

  private.stage = { name = stage_name, score = 1000 };

  for i = 1, #targets do

    if ( targets[ i ].type == "GainScore" ) then

      private.stage.score = targets[ i ].count;

      break;

    end;

  end;

  return ( private.stage.score + 1000 * index + math.random( 999 ) ); 

end;]]
--------------------------------------------------------------------------------
function private.GetUpdateCooldown ()

  return m_storage.Settings( "social" ).friends_collections_update;

end;