-- name=m_spells
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ SPELLS ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.Trace( "Initialized." );

  private.section_spells = {};

end;
--------------------------------------------------------------------------------
function public.TryUnlockSpellByStage ( stage_name )

  local spell_name = public.GetSpellByUnlockStage( stage_name );

  if not ( spell_name ) then

    return;

  end;

  local spell_data = public.GetSpellData( spell_name );

  if not ( spell_data.enabled ) then
    
    local query =
    {
      query      = "insert",
      group      = "player",
      collection = "spells",
      value      = 
      {
        name      = spell_name,
        enabled   = true,
        completed = false
      }
    };

    m_storage.Execute( query );

    mediator.Notify( "Model_Spells_SpellEnabled", spell_name );

  end;

end;
--------------------------------------------------------------------------------
function public.GetSpellByUnlockStage ( stage_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "spells",
    indice     = "unlock",
    criteria   = { like = stage_name }
  };

  local spell = m_storage.Execute( query ).result;
  
  if ( spell ) and ( spell.name ) then
  
    return spell.name;
    
  end;
  
  return nil;

end;
--------------------------------------------------------------------------------
function public.TryExecuteSpell ( spell_name )

  local spell_data = public.GetSpellData( spell_name );

  local can_buy = true;

  local currency_count = 0;

  local func_NotifyChangeFailed = function () end;

  for i = 1, #spell_data.input do

    local currency = spell_data.input[ i ];

    if ( "gold" == currency.name ) then

      currency_count = m_player.GetGoldCount();

      local count_new = currency_count - currency.count;

      func_NotifyChangeFailed = function ()
      
        mediator.Notify( "Model_Player_GoldCountChangeFailed",
          { count = math.abs( count_new ) }
        );
     
      end;

    else

      currency_count = m_player.GetResourceCount( currency.name );

      if ( currency_count < 0 ) then

        currency_count = 0;

      end;

      local count_new = currency_count - currency.count;

      func_NotifyChangeFailed = function ()
     
        mediator.Notify( "Model_Player_ResourceCountChangeFailed",
          { name = currency.name, count = math.abs( count_new ) }
        );
     
      end;

    end;

    local price_count = currency.count;

    if ( currency_count - price_count < 0 ) then

      can_buy = false;

      func_NotifyChangeFailed();

      break;

    end;

  end;

  if ( can_buy ) then

    for i = 1, #spell_data.input do

      local currency = spell_data.input[ i ];

      if ( "gold" == currency.name ) then

        m_player.TryChangeGoldCount( - currency.count );

        common.Kontagent_LogEvent( "economy", 
        {
          description = "premium_currency_spend",
          category    = "spellbook",
          value       = - currency.count,
          name        = spell_name
        } );

      else

        m_player.TryChangeResourceCount( currency.name, - currency.count );

        common.Kontagent_LogEvent( "economy", 
        {
          description = "soft_currency_spend",
          category    = "spellbook",
          value       = - currency.count,
          name        = spell_name,
          type        = currency.name
        } );

      end;

    end;

    for i = 1, #spell_data.output do

      local currency = spell_data.output[ i ];

      if ( "gold" == currency.name ) then

        m_items.TryObtain( {
          name       = currency.name, 
          type       = currency.name, 
          count      = currency.count,
          category   = "spellbook",
          stage_name = spell_name,
          silent     = true
        } );

      else

        m_items.TryObtain( {
          name       = currency.name, 
          type       = "resource", 
          count      = currency.count,
          category   = "spellbook",
          stage_name = spell_name,
          silent     = true
        } );

      end;

    end;

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "spells",
      indice     = "name",
      criteria   = { like = spell_name },
      set        =
      {
        completed = true
      }
    };

    m_storage.Execute( query );

    mediator.Notify( "Model_Spells_SpellCompleted", spell_name );

    return spell_data;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function public.GetSpellData ( spell_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "spells",
    indice     = "name",
    criteria   = { like = spell_name }
  };

  local spell_data = m_storage.Execute( query ).result;

  if not ( spell_data ) then

    return false;

  end;

  spell_data.enabled   = false;
  spell_data.completed = false;

  query = 
  {
    query      = "find",
    group      = "player",
    collection = "spells",
    indice     = "name",
    criteria   = { like = spell_name }
  };

  local saved_data = m_storage.Execute( query ).result;

  if ( saved_data ) then

    spell_data.enabled   = saved_data.enabled;
    spell_data.completed = saved_data.completed;

  end;

  local zone_data = m_fights.Zone_Data( spell_data.unlock ); 

  spell_data.zone_name = zone_data.name;
  spell_data.content   = zone_data.content;

  return spell_data;

end;
--------------------------------------------------------------------------------
function private.GetOutput ( spell_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "spells",
    indice     = "name",
    criteria   = { like = spell_name }
  };

  local spell_data = m_storage.Execute( query ).result;

  if ( spell_data ) then

    return spell_data.output[ 1 ];

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Spell_Section ( spell_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "spells",
    indice     = "name",
    criteria   = { like = spell_name }
  };

  return m_storage.Execute( query ).result.section;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Usable_List ()

  local section = v_map.Section_Current();
  
  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "spells",
    indice     = "completed",
    criteria   = { like = "false" }
  };

  local usable = m_storage.Execute( query ).result;

  if not ( usable ) then return false; end;
  
  local list = {};
  
  for s = 1, #usable do 
    
    if ( private.Spell_Section( usable[ s ].name ) == section ) then
      
      table.insert( list, usable[ s ] );
      
    end;
  
  end;
  
  if ( #list < 1 ) then return false; end;

  return list;

end;
--------------------------------------------------------------------------------
function public.Usable_Search ( resource_name )

  local usable_spells = private.Usable_List();

  if ( usable_spells ) then

    for i = 1, #usable_spells do

      local output = private.GetOutput( usable_spells[ i ].name );

      if ( output.name == resource_name ) then

        return usable_spells[ i ];

      end;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------