-- name=m_stage
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ MODEL ] [ STAGE ] "..text );

end;
--------------------------------------------------------------------------------
function  public.Init ()

  ModLoad( "assets/content/scripts/mod_m_match3" );
  m_match3.Init();

  common.MessageSubscribe( "Command_Match3_ChainActivated",
    function ( msg, params )
      private.OnChainActivated( params );
    end );

  common.MessageSubscribe( "Event_Match3_FieldActivated",
    function ( msg, params )
      private.OnFieldActivated();
    end );

  common.MessageSubscribe( "Event_Match3_CellChanged",
    function ( msg, params )
      private.OnCellChanged( params );
    end );

  common.MessageSubscribe( "Event_Match3_BonusesSwapped",
    function ( msg, params )
      private.OnBonusesSwapped( params );
    end );

  common.MessageSubscribe( "Event_Match3_ChipsRemoved",
    function ( msg, params )
      private.OnChipsRemoved( params );
    end );
  
  common.MessageSubscribe( "Event_Match3_CombinationsNotExist",
    function ( msg, params )
      private.OnCombinationsNotExist( params );
    end );
  
  private.fight_settings = m_storage.Settings( "fights" );
  private.stage_settings = m_storage.Settings( "stages" );
  
  public.CheckStateless();
  
  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
-- function ******* Message Handlers ******* () end
--------------------------------------------------------------------------------
function private.OnChainActivated ( chain_data )

  mediator.Notify( "Model_Stage_ChainActivated", chain_data );

  private.IncChainCount();

  local score_type = "Basic";
  local chain_type = "Unknown";
  local bonus_type = "Unknown";

  for c = 1, #chain_data.chips_list do

    chain_type = chain_data.chips_list[ c ].type;

    if ( chain_data.chips_list[ c ].property ~= "Basic" ) then

      score_type = "Bonus";
      bonus_type = chain_data.chips_list[ c ].property;

      break;

    end;

  end;

  local chain_form = chain_data.chain_form;
  local basic_score = private.GetBasicChainScore( chain_form, score_type );
  local chain_score = basic_score * private.state_data.state;

  if ( private.combination.form ) and 
     ( private.combination.form == chain_data.chain_form ) then

    chain_score = chain_score + private.combination.score;

  end;

  private.combination.form  = chain_form;
  private.combination.score = chain_score;

  local x, y = chain_data.pivot_data.x, chain_data.pivot_data.y;

  private.ChangeScore( x, y, chain_type, chain_score );
  private.UpdateHeroAttackFactor( chain_data.chips_list );
  private.UpdateRewards( chain_data.chips_list );
  private.Bonus_CheckComplete( chain_type );
  private.UpdateTargetProgress_DestroyGem( chain_data.chips_list[ 1 ].type, #chain_data.chips_list );

  if ( score_type == "Bonus" ) then

    mediator.Notify( "Model_Match3_BonusRemoved", { type = bonus_type } );

  end;

end;
--------------------------------------------------------------------------------
function private.OnFieldActivated ()

  mediator.Notify( "Model_Match3_FieldActivated" );

  private.TryChangeMovesCount( - 1 );
  private.MovesDone_IncCount();
  private.Bonus_ChangeCounters(); 
  private.ResetChainCount();

  private.continue_count = 0;

end;
--------------------------------------------------------------------------------
function private.OnCellChanged ( cell_data )

  for c = 1, #cell_data.cells_list do

    local cell = cell_data.cells_list[ c ];
    
    local state = private.state_data.state;
    local cell_score = private.GetCellScore( cell.type ) * state;

    private.ChangeScore( cell.x, cell.y, cell.type, cell_score );

    local level = cell.level or 1
    private.UpdateTargetProgress_DestroyCell( cell.type, level, 1 );

    mediator.Notify( "Model_Stage_CellDestroyed", 
      { 
        name  = cell.type,
        x     = cell.x,
        y     = cell.y,
        color = cell.color,
        size  = cell.size  
      } );
  
  end;
  
end;
--------------------------------------------------------------------------------
function private.OnBonusesSwapped ( combo_data )

  -- FIXME: scores not in logic_settings

  mediator.Notify( "Model_Stage_BonusesSwapped", combo_data );

  local combo_type = combo_data.combo_type;
  local chip_type  = combo_data.type;

  if ( combo_type == "HorizontalVertical" ) then

    mediator.Notify( "Model_Match3_BonusRemoved", { type = "Horizontal" } );

  elseif ( combo_type == "NucleusNucleus" ) then

    --

  elseif ( combo_type == "HVNucleus" ) then

    --

  elseif ( combo_type == "BombBomb" ) then

    local score = 1000 * private.state_data.state;
    private.ChangeScore( combo_data.x, combo_data.y, chip_type, score );
    mediator.Notify( "Model_Match3_ComboActivated", { type = chip_type } );

  elseif ( combo_type == "NucleusBomb" ) then

    --

  elseif ( combo_type == "BombHV" ) then

    --

  elseif ( combo_type == "BasicBomb" ) then

    local score = 120 * private.state_data.state;
    private.ChangeScore( combo_data.x, combo_data.y, chip_type, score );
    mediator.Notify( "Model_Match3_ComboActivated", { type = chip_type } );

  end;

end;
--------------------------------------------------------------------------------
function private.OnChipsRemoved ( chips_data )

  local chips_list = chips_data.chips_list;

  for b = 1, #chips_list do

    local chip_data = chips_list[ b ];

    if ( chip_data.type == "Item" ) then

      private.OnItemDropped( chip_data );

    else

      local basic_score = private.GetBonusScore( chip_data.property );
      local score = basic_score * private.state_data.state;

      private.ChangeScore( chip_data.x, chip_data.y, chip_data.type, score );

      if ( chip_data.property ~= "Basic" ) then

        mediator.Notify( "Model_Match3_BonusRemoved", 
          { 
            type = chip_data.property
          } );

      end;
        
      private.UpdateTargetProgress_DestroyGem( chip_data.type, 1 );

    end;

  end;

  private.UpdateHeroAttackFactor( chips_list );

  private.UpdateRewards( chips_list );

end;
--------------------------------------------------------------------------------
function private.OnCombinationsNotExist ( params )

  private.no_combinations = true;

  mediator.Notify( "Model_Match3_CombinationsNotExist" );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.OnItemDropped ( chip_data )

  local item_name = private.state_data.items[ 1 ].name; -- FIXME (FF)

  local score = 3000 * private.state_data.state;
  private.ChangeScore( chip_data.x, chip_data.y, "Item", score );

  private.UpdateTargetProgress_DropItem( item_name, 1 );
  
  mediator.Notify( "Model_Stage_ItemDropped", 
    {
      x = chip_data.x,
      y = chip_data.y,
      name = item_name
    } );

end;
--------------------------------------------------------------------------------
-- function ******* Stateless Info ******* () end
--------------------------------------------------------------------------------
function  public.GetStatelessNum ( stage_info )

  return private.stage_settings.stateless_until;

end;
--------------------------------------------------------------------------------
function public.CheckStateless ()
  
  local query =
  {
    query      = "find",
    group      = "player",
    collection = "stages",
    indice     = "name",
    criteria   = 
    { 
      like = "stage_" .. private.stage_settings.stateless_until + 1
    }
  };

  local enabling_states_stage = m_storage.Execute( query ).result;

  private.is_stateless = ( enabling_states_stage == nil );
  
end;
--------------------------------------------------------------------------------
function  public.IsStateless ()
  
  return private.is_stateless;
  
end;
--------------------------------------------------------------------------------
-- function ******* Start / Finish ******* () end
--------------------------------------------------------------------------------
function  public.TryStart ( stage_name, ext_enemy, stage_state )

  if ( m_player.TryChangeHeartsCount( - 1 ) ) then 

    private.need_finalize = false;

    private.stage_name = stage_name;
    private.state_data = private.GetStateData( stage_name, stage_state );

    public.Kontagent_SetCurrentStage( stage_name );

    private.noreward = nil;

    private.completed         = false;
    private.bonuses_activated = false;
    private.finalized         = false;
    private.no_combinations   = nil;
    private.continue_count    = nil;
    private.boosters          = {};

    private.InitHero();
    private.InitEnemy( ext_enemy );

    private.InitScore();
    private.InitMoves();
    private.InitTime();
    private.InitChains();
    private.InitRewards();

    private.Bonus_Init();

    private.InitTargets();
    
    local saved_data = public.GetSavedData( stage_name ) or {};

    private.boosters_selected       = m_boosters.GetSelected();
    private.state_data.attempts     = saved_data.attempts or 0;
    private.state_data.max_attempts = private.stage_settings.max_attempts;
    
    public.Kontagent_SetBoostersUsed( 0 );

    if ( #private.boosters_selected > 0 ) then

      public.Kontagent_SetBoostersUsed( 1 );

    end;
    
    local start_data    = common.DeepCopy( private.state_data );
    start_data.boosters = private.boosters_selected;

    m_match3.StartData_Set( start_data );

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function  public.IsCanContinue ()

  return ( private.IsConditionFailed() or not private.IsCombinationsNotExist() );

end;
--------------------------------------------------------------------------------
function  public.TryUseContinue ( continue_name )

  if ( continue_name == "extramoves" ) then

    private.TryChangeMovesCount( 5, true );
    
  elseif ( continue_name == "extralives" ) then

    private.TryRecoverHeroHealth( math.ceil( 0.5 * private.hero.health ) );

  elseif ( continue_name == "extratime" ) then

    public.TryChangeTimeCount( 10, true );

  end;

  public.Kontagent_SetBoostersUsed( 1, continue_name );

  private.finalized = false;

end;
--------------------------------------------------------------------------------
function  public.TryContinue ()
  
  if ( private.state_data.type == "fight" ) then

    if ( private.continue_count == 0 ) then
      
      private.continue_count = 1;

      local attack_result = private.TryActivateHeroAttack();

      private.ResetHeroAttackFactor();

      if ( attack_result ) then

        return false;

      end;

    end;

    if ( private.continue_count == 1 ) then

      private.continue_count = 2;

      local attack_result = private.TryActivateEnemyAttack();

      if ( attack_result ) then

        return false;

      end;
      
      if (private.superhit) then
            
        private.superhit = false;
    
      end;

    end;

  end;
  
  private.TryCompleteTargets();

  return true;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function public.TryFinalize ( anm_playing )

  if ( private.IsConditionCompleted() ) and
     ( private.IsTargetsCompleted()   ) then

    private.Trace( "TryFinalize: all completed." );

    if ( not private.completed ) then

      private.completed = true;

      mediator.Notify(
        "Model_Stage_StageComplete",
        { moves = private.IsMovesExist() }
      );

    end;

    if ( private.IsMovesExist() ) then

      if ( not private.moves_remain ) then

        private.moves_remain = private.moves;

      end;
      
      if ( anm_playing ) then

        mediator.SubscribeOnce(
          "View_Stage_AnimationIsFinished",
          private.FinalBonuses_Spawn
        );
      
      else
        
        private.FinalBonuses_Spawn();
        
      end;

      return false;
      
    elseif not ( private.bonuses_activated ) then
      
      private.FinalBonuses_Activate();
      
      return false;

    end;

  end;

  if ( private.completed or private.IsConditionFailed() or private.IsCombinationsNotExist() ) then

    private.Trace( "TryFinalize: finalize." );
    
    if ( not private.finalized ) then

      private.finalized = true;

      private.Finalize( private.completed );

      return true;

    end;
  
    DbgTrace( "WARNING!!! Finalize recall!" );

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Finalize ( is_win )

  if ( is_win ) then

    private.Trace( "private.Finalize is_win " );
    private.need_finalize = true;
    
  end;
  private.Trace( "private.Finalize End" );

end;
--------------------------------------------------------------------------------
function  public.TryFinish ()
  
  private.Trace( "TryFinish" ); 
  
  if ( private.need_finalize ) then
    
    private._FinalizeWin();
    
  end;

  if ( private.MovesDone_GetCount() == 0 ) then 

    m_player.TryChangeHeartsCount( 1 );

  else

    if ( not private.completed ) then

      private.IncreaseAttempts();

    end;

  end;

  return true;

end;
--------------------------------------------------------------------------------
function private._FinalizeWin ()
  
  private.Trace( "_FinalizeWin" );
  
  --JK: check stateless
  public.CheckStateless( private.stage_name );
  
  local stage_name = private.stage_name;
  local stage_data = private.state_data;
  local state_max  = stage_data.state_max;

  if ( stage_data.realtype ~= "dailyquest" ) then

    local saved_info = public.GetSavedData( stage_name ) or {};
    local stage_info = {
      completed = saved_info.completed or false,
      enabled   = saved_info.enabled   or false,
      noreward  = saved_info.noreward  or false,
      state     = saved_info.state     or 1,
      score     = saved_info.score     or 0,
      attempts  = saved_info.attempts  or 0,
      type      = stage_data.type,
      realtype  = stage_data.realtype
    };

    private.noreward = stage_info.noreward;

    local stateless  = public.IsStateless();
    local update_set = {};

    local state_new   = math.min( stage_info.state + 1, state_max );
    local enabled     = true;
    local completed   = true;
    local enable_next = true;

    if ( stage_info.realtype == "misc"  ) and
       ( stage_info.type     == "fight" ) then

      stage_info.state = 4;
      state_new = 1;

      enable_next = false;
      completed   = false;

      update_set.type = "ride";

    elseif ( stage_info.realtype == "fight" ) then

      state_new = 1;
      enabled = false;

    end;

    if ( stage_info.attempts ~= 0 ) then

      update_set.attempts = 0;

    end;

    if ( stage_info.state == state_max and not stage_info.noreward ) then

      update_set.noreward = true;

    end;

    if ( completed ~= stage_info.completed ) then

      update_set.completed = completed;

    end;

    local score_new = private.score;

    if ( score_new > stage_info.score ) then

      update_set.score = score_new;

    end;

    if ( state_new ~= stage_info.state ) then

      update_set.state = state_new;

    end;

    if ( enabled ~= stage_info.enabled ) then

      update_set.enabled = enabled;

    end;

    if ( next( update_set ) ~= nil ) then

      local update_query =
      {
        query      = "update",
        group      = "player",
        collection = "stages",
        indice     = "name",
        criteria   = { like = stage_name },
        set        = update_set
      };

      m_storage.Execute( update_query );

    end;
    
    -- DG: NOTE: Say to v_map update that is first stage completion
    --  * for generic levels first complete marked by completed flag
    --  * for misc fight levels first complete marked by type = "ride"
    local is_first = update_set.completed or ( update_set.type == "ride" );
    
    if ( is_first ) then

      local query =
      {
        query      = "update",
        group      = "player",
        collection = "info",
        set        =
        {
          stage_last = stage_name
        }
      };
      m_storage.Execute( query );

    end;

    local data_for_reward = {
      name        = stage_name,
      type        = stage_info.realtype,
      state_old   = stage_info.state,
      state_new   = state_new,
      noreward    = stage_info.noreward,
      enable_next = enable_next
    }; 
    
    -- FIXME: Strange condition
    if ( stateless and state_new == 3 ) then   
      data_for_reward.state_new = 2;
      data_for_reward.state_old = 1;
      data_for_reward.noreward  = true;
    end;
    
    local reward_data = private.TryGetReward( stage_data.name, data_for_reward );

    if ( stage_info.realtype == "fight" ) then

      mediator.Notify(
        "Model_Stage_EnemyDefeated",
        {
          name = stage_data.enemy.name
        }
      );

    end;
    
    local notify_data = {
      name        = stage_name,
      type        = stage_info.realtype,
      state_old   = stage_info.state,
      state_new   = state_new,
      noreward    = stage_info.noreward,
      enable_next = enable_next
    };
    
    mediator.Notify( "Model_Stage_StageWon", notify_data );
  
    m_map.Update(
      stage_name,
      is_first,
      reward_data,
      update_set.noreward,
      enable_next
    );

  else

    local notify_data =
    {
      name = stage_data.name,
      type = stage_data.type,
      state_old = 1,
      state_new = 1,
      noreward = true
    };

    mediator.Notify( "Model_Stage_StageWon", notify_data );

  end;
  
  m_player.TryChangeHeartsCount( 1 );

end;
--------------------------------------------------------------------------------
-- function ******* Current Stage ******* () end
--------------------------------------------------------------------------------
function  public.GetCurrentStageData ()

  return private.state_data;

end;
--------------------------------------------------------------------------------
function  public.GetCurrentStageName ()

  return private.stage_name;

end;
--------------------------------------------------------------------------------
function  public.GetCurrentTargets ()

  return private.targets;

end;
--------------------------------------------------------------------------------
function  public.GetCurrentResources ()

  local resources = {};

  for key, value in pairs( private.reward ) do

    table.insert( resources, { name = key, count = value } );

  end;

  return resources;

end;
--------------------------------------------------------------------------------
function  public.GetCurrentScore ()

  return private.score;

end;
--------------------------------------------------------------------------------
function  public.GetCurrentHero ()

  return private.hero;

end;
--------------------------------------------------------------------------------
function  public.GetCurrentEnemy ()

  return private.enemy;

end;
--------------------------------------------------------------------------------
function  public.GetCurrentMoves ()

  if ( private.moves ) then

    return private.moves;

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function  public.GetTargetByType ( target_type )

  return private.GetTargetByType( target_type );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function  public.IsCurrentStageFight ()

  return ( private.state_data.type == "fight" );

end;
--------------------------------------------------------------------------------
function  public.IsWin ()

  return private.completed;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.IncreaseAttempts ()

  private.Trace( "IncreaseAttempts Start" );

  local stage_name = private.stage_name;
  local real_type  = private.state_data.realtype;

  if ( real_type ~= "dailyquest" ) then

    local attempts = ( public.GetSavedData( stage_name ) or {} ).attempts or 0;
    attempts = attempts + 1;

    private.Trace( "Set Attempts " .. attempts );
  
    local query =
    {
      query      = "update",
      group      = "player",
      collection = "stages",
      indice     = "name",
      criteria   = { like = stage_name },
      set        =
      {
        attempts = attempts
      }
    };
    
    m_storage.Execute( query ); 
    
    mediator.Notify( "Model_Stage_AttemptsIncreased", attempts );
    
  end;
  
  private.Trace( "IncreaseAttempts End" );
  
end;
--------------------------------------------------------------------------------
-- function ******* Stage & State Data ******* () end
--------------------------------------------------------------------------------
function  public.TryEnable ( stage_name )

  local result = false;
  
  local query =
  {
    query      = "update",
    group      = "player",
    collection = "stages",
    indice     = "name",
    criteria   = { like = stage_name },
    set        =
    {
      enabled = true,
      visible = true
    }
  };
  
  result = m_storage.Execute( query ).result;

  if ( not result ) then

    query =
    {
      query      = "insert",
      group      = "player",
      collection = "stages",
      value      =
      {
        name    = stage_name,
        enabled = true,
        visible = true
      }
    };
    
    result = m_storage.Execute( query ).result;

  end;

  return result;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function  public.GetStageState ( stage_name, stage_data, saved_data )

  -- NOTE: stage_data and saved_data are optional

  return private.GetStageState( stage_name, stage_data, saved_data );

end;
--------------------------------------------------------------------------------
function private.GetStageState ( stage_name, stage_data, saved_data )

  saved_data = saved_data or public.GetSavedData( stage_name );
  stage_data = stage_data or public.GetStaticData( stage_name, saved_data ); 

  saved_data  = saved_data or {};
  local state = saved_data.state or 1;

  if ( stage_data.realtype == "misc" ) and ( stage_data.type == "fight" ) then

    state = 4;

  end;

  return state;

end;
--------------------------------------------------------------------------------
function private.GetStateData ( stage_name, stage_state )

  local state_data = {};
  local db_data    = public.GetStaticData( stage_name );

  state_data.name      = stage_name;
  state_data.section   = db_data.section;
  state_data.zone      = db_data.zone;
  state_data.content   = db_data.content;
  state_data.bonus     = db_data.bonus;
  state_data.type      = db_data.type;
  state_data.realtype  = db_data.realtype;
  
  if ( public.IsStateless() ) and
     ( not stage_state      ) then

    stage_state      = 1;
    state_data.state = 1;

  else

    stage_state      = stage_state or private.GetStageState( stage_name, db_data );
    state_data.state = stage_state;

  end;
  
  state_data.state_max = math.min( #db_data.state, 3 );

  for key, value in pairs( db_data.state[ stage_state ] ) do

    state_data[ key ] = value;

  end;
  
  return state_data;
  
end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function public.GetStaticData ( stage_name, saved_data )

  local query = {
    query      = "find",
    group      = "static",
    collection = "stages",
    indice     = "name",
    criteria   = { like = stage_name }
  };

  local stage_data = m_storage.Execute( query ).result;

  if ( stage_data ) then
    private.AppendRealType( stage_data, saved_data );
  end;
  
  return stage_data;

end;
--------------------------------------------------------------------------------
function public.GetSavedData ( stage_name )

  if ( not stage_name ) then

    return false;

  end;

  local query = {
    query      = "find",
    group      = "player",
    collection = "stages",
    indice     = "name",
    criteria   = { like = stage_name }
  };
 
  return m_storage.Execute( query ).result;

end;
--------------------------------------------------------------------------------
function private.AppendRealType ( stage_data, saved_data )

  local realtype = stage_data.type;
  stage_data.realtype = realtype;

  if     ( realtype == "bonus"      ) then stage_data.type = "ride";
  elseif ( realtype == "dailyquest" ) then stage_data.type = "fight";
  elseif ( realtype == "misc"       ) then

    saved_data = saved_data or public.GetSavedData( stage_data.name ) or {};

    if ( saved_data.type == "ride" ) then stage_data.type = "ride";
    elseif ( saved_data.completed )  then stage_data.type = "ride";
    else                                  stage_data.type = "fight";
    end;
    

  elseif ( realtype ~= "ride" and realtype ~= "fight" ) then

    DbgTrace( string.format( "[ MODEL STAGE ][ ERROR ] Stage '%s' has " .. 
                             " unknown type: '%s'", stage_data.name, realtype ) );

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- function ******* Boosters ******* () end
--------------------------------------------------------------------------------
function  public.Boosters_TryUse ( func_execute )

  return private.Boosters_TryUse( func_execute );

end;
--------------------------------------------------------------------------------
function  public.Boosters_GetStageList ( stage_name, load_booster_type )

  local func_BoostersOnStage = function ( stage_name )

    local state_number = private.GetStageState( stage_name );
    
    state_number = tern( public.IsStateless() )( 1, state_number );
    
    local stage = public.GetStaticData( stage_name );

    if ( stage.state[ state_number ].boosters ) then

      return stage.state[ state_number ].boosters;

    end;

    return {};

  end;

  local boosters_on_stage = func_BoostersOnStage( stage_name );
  local boosters_list     = {};
  
  if ( boosters_on_stage ) and ( #boosters_on_stage > 0 ) then

    for i = 1, #boosters_on_stage do

      local booster = 
      {
        name     = boosters_on_stage[ i ],
        count    = m_boosters.GetCount( boosters_on_stage[ i ] ),
        unlocked = m_boosters.IsUnlocked( boosters_on_stage[ i ] )
      };
      
      local query = 
      {
        query      = "find",
        group      = "static",
        collection = "boosters",
        indice     = "name",
        criteria   = { like = booster.name }
      };
      
      local booster_type = m_storage.Execute( query ).result.type;
      
      if ( load_booster_type == "all"        ) or 
         ( load_booster_type == booster_type ) then

        table.insert( boosters_list, booster );

      end;

    end;

  end;

  return boosters_list;

end;
--------------------------------------------------------------------------------
function private.Boosters_TryUse ( func_execute )

  local boosters_list = private.boosters_selected;

  local boosters = 
  {
    movesup = private.Boosters_Use_MovesUp,
    superhit = private.Boosters_Use_SuperHit
  };
  
  if ( boosters_list ) then
    
    for b = 1, #boosters_list do

      local booster_name = boosters_list[ b ];

      if ( boosters[ booster_name ] ) then

        if ( func_execute ) then

          func_execute( 
            { 
              result = true,
              name = booster_name, 
              func_end = boosters[ booster_name ]
            }, public.GetCurrentStageName() );

        end;

      end;

    end;

    private.boosters_selected = nil;

  end;

end;
--------------------------------------------------------------------------------
function private.Boosters_Use_MovesUp ()
  
  private.TryChangeMovesCount( 5, true );

end;
--------------------------------------------------------------------------------
function private.Boosters_Use_SuperHit ()
  
  private.continue_count     = 0;
  private.bonus.activated    = true;
  private.superhit           = true;  
  private.hero.attack_factor = private.enemy.health_current / 10 / private.hero.attack;

  mediator.Notify( "Model_Stage_ViewTryContinue" );
  
  private.bonus.activated = false;
  private.ResetHeroAttackFactor();
  
end;
--------------------------------------------------------------------------------
-- function ******* Moves ******* () end
--------------------------------------------------------------------------------
function private.InitMoves ()

  private.moves = private.state_data.moves;

  if ( private.moves ) then

    private.moves = private.moves + public.GetCurrentHero().moves;
    private.moves_remain = nil; -- Счетчик оставшихся ходов, для статистики.

  end;

  private.MovesDone_Init();

end;
--------------------------------------------------------------------------------
function private.TryChangeMovesCount ( moves_delta, ignore )

  -- При отсутствии ходов изменение в любую сторону запрещено.
  if ( not private.IsMovesExist() and not ignore ) then

    return false;

  end;

  private.moves = private.moves + moves_delta;

  mediator.Notify( "Model_Stage_MovesCountChanged", private.moves );

  return true;

end;
--------------------------------------------------------------------------------
function private.IsMovesExist ()

  if ( not private.moves ) then

    return false;

  end;

  return ( private.moves > 0 );

end;
--------------------------------------------------------------------------------
function private.GetMovesCount ()

  return private.moves;

end;
--------------------------------------------------------------------------------
-- Общий счетчик ходов, сделанных на уровне.
function private.MovesDone_Init ()

  private.moves_done = 0;

end
--------------------------------------------------------------------------------
function private.MovesDone_IncCount ()

  private.moves_done = private.moves_done + 1;

  if ( private.state_data.time and private.moves_done == 1 ) then

    mediator.Notify( "Model_Stage_TimerStart" );

  end;

end;
--------------------------------------------------------------------------------
function  public.MovesDone_GetCount ()

  return private.MovesDone_GetCount();

end;
--------------------------------------------------------------------------------
function private.MovesDone_GetCount ()

  return private.moves_done;

end;
--------------------------------------------------------------------------------
-- function ******* Time ******* () end
--------------------------------------------------------------------------------
function private.InitTime ()

  private.time = private.state_data.time;
  private.time_current = private.state_data.time;

end;
--------------------------------------------------------------------------------
function  public.TryChangeTimeCount ( time_delta, ignore )

  if ( not public.IsTimeExist() and not ignore ) then

    return false;

  end;

  private.freeze_time = private.freeze_time or 0;

  if ( private.freeze_time > 0 ) then

    private.freeze_time = private.freeze_time - 1;

  else

    private.time_current = private.time_current + time_delta;

    if ( private.time_current > private.time ) then

      private.time_current = private.time;

    end;

  end;

  mediator.Notify( "Model_Stage_TimeCountChanged", private.time_current );

  return true;

end;
--------------------------------------------------------------------------------
function  public.IsTimeExist ()

  if ( not private.time_current ) then

    return false;

  end;

  return ( private.time_current > 0 );

end;
--------------------------------------------------------------------------------
function private.GetTimeCount ()

  return private.time_current;

end;
--------------------------------------------------------------------------------
function private.FreezeTime ( value )

  private.freeze_time = value;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.InitChains ()

  private.combination = { form = nil, score = nil };
  private.chain_count = 0;

end;
--------------------------------------------------------------------------------
function private.IncChainCount ()

  private.chain_count = private.chain_count + 1;

  mediator.Notify( "Model_Stage_ChainCountChanged", 
    {
      count = private.chain_count
    } );

end;
--------------------------------------------------------------------------------
function private.ResetChainCount ()

  private.combination.form = nil;

  private.chain_count = 0;

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.GetBonusScore ( bonus_property )

  local bonus_score = 
  {
    Unknown    = 1,
    Basic      = 60,  
    Horizontal = 120,
    Horizontal3 = 60,
    Vertical   = 120,
    Vertical3  = 60,
    Nucleus    = 120,
    NucleusSecond = 120,
    Bomb       = 120,
    BombSecond = 120
  };

  return bonus_score[ bonus_property or "Unknown" ];

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.InitHero ()

  private.hero = m_hero.GetStats();
  private.hero.health_current = private.hero.health;

  private.hero.attack_factor = 0;

  private.hero.bonus_attack_factor = nil;
  private.hero.bonus_protection = false;

end;
--------------------------------------------------------------------------------
function private.InitEnemy ( external_enemy )

  private.enemy = external_enemy or private.state_data.enemy;

  if ( private.enemy ) then
  
    local attack = private.hero.enemy_attack;
    private.enemy.attack = private.enemy.attack + attack;
    private.enemy.attack = math.max( private.enemy.attack, 5 );
    
    local health = private.hero.enemy_health;
    private.enemy.health = private.enemy.health + health;
    private.enemy.health = math.max( private.enemy.health, 50 );
    
    private.enemy.health_current = private.enemy.health;

    private.enemy.bonus_attack_factor = 1;
    private.enemy.bonus_protection = false;
    private.enemy.bonus_stun = false;

  end;

end;
--------------------------------------------------------------------------------
function private.IsHeroDead ()

  return ( private.hero.health_current == 0 );

end;
--------------------------------------------------------------------------------
function private.IsEnemyDead ()

  return ( private.enemy.health_current == 0 );

end;
--------------------------------------------------------------------------------
function private.UpdateHeroAttackFactor ( chips_list )

  private.IncHeroAttackFactor( #chips_list );

end;
--------------------------------------------------------------------------------
function private.IncHeroAttackFactor ( count )

  private.hero.attack_factor = private.hero.attack_factor + count;

end;
--------------------------------------------------------------------------------
function private.SetHeroBonusAttackFactor ( value )

  private.hero.bonus_attack_factor = value;

end;
--------------------------------------------------------------------------------
function private.SetHeroBonusProtection ( value )

  private.hero.bonus_protection = value;

end;
--------------------------------------------------------------------------------
function private.SetHeroBonusDoT ( value )

  private.hero.bonus_dot = value;

end;
--------------------------------------------------------------------------------
function private.ResetHeroAttackFactor ()

  private.hero.attack_factor = 0;

end;
--------------------------------------------------------------------------------
function private.TryResetHeroBonusAttackFactor ()

  if not ( private.bonus.activated ) then

    private.SetHeroBonusAttackFactor( nil );

  end;

end;
--------------------------------------------------------------------------------
function private.TryResetHeroBonusDoT ()

  if not ( private.bonus.activated ) then

    private.SetHeroBonusDoT( nil );

  end;

end;
--------------------------------------------------------------------------------
function private.TryResetHeroBonusStun ()

  if not ( private.bonus.activated ) then

    private.SetHeroBonusProtection( nil );

  end;

end;
--------------------------------------------------------------------------------
function private.TryActivateHeroAttack ()

  if ( private.enemy.bonus_stun ) then

    return false;

  end;

  if ( not private.IsEnemyDead() ) then

    local additional_damage = 0;

    if ( private.hero.bonus_dot ) then

      additional_damage = private.hero.bonus_dot;

    end;

    local bonus_attack_factor = 1;

    if ( private.hero.bonus_attack_factor ) then

      bonus_attack_factor = private.hero.bonus_attack_factor;

    end;

    local damage = bonus_attack_factor * private.hero.attack_factor *
                   private.hero.attack + additional_damage;

    if ( private.enemy.bonus_protection ) then

      damage = 0;

    end;

    private.enemy.health_current = private.enemy.health_current - damage;

    if ( private.enemy.health_current < 0 ) then

      private.enemy.health_current = 0;

    end;

    private.AchievementCheckOneShot();

    local ability = nil;

    if ( private.bonus.activated ) then

      if ( private.hero.bonus_protection ) then

        ability = "stun";

      elseif ( private.hero.bonus_dot ) then

        ability = "dot";

      elseif ( private.hero.bonus_attack_factor ) then

        ability = "damage";
        
      elseif (private.superhit) then
        
        ability = "superhit";

      end;

    end;

    mediator.Notify( "Model_Stage_HeroAttackActivated", 
    { 
      damage        = damage,
      ability       = ability, 
      ability_delta = additional_damage,
      enemy_health  = private.enemy.health_current
    } );

    if ( damage > 0 ) then

      return true;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.AchievementCheckOneShot ()

  private.enemy.one_shot = ( private.enemy.one_shot or 0 ) + 1;

  if ( private.enemy.one_shot == 1       ) and
     ( private.enemy.health_current == 0 ) then

    mediator.Notify( "Model_Stage_HeroAttackOneShot" );

  end;

end;
--------------------------------------------------------------------------------
function private.SetEnemyBonusAttackFactor ( value )

  private.enemy.bonus_attack_factor = value;

end;
--------------------------------------------------------------------------------
function private.SetEnemyBonusProtection ( value )

  private.enemy.bonus_protection = value;

end;
--------------------------------------------------------------------------------
function private.SetEnemyBonusStun ( value )

  private.enemy.bonus_stun = value;

end;
--------------------------------------------------------------------------------
function private.TryActivateEnemyAttack ()

  if ( private.IsEnemyDead() ) then

    return false;

  end;

  if ( private.hero.bonus_protection or private.superhit ) then

    return false;

  end;

  local damage = private.enemy.bonus_attack_factor * private.enemy.attack;

  private.hero.health_current = private.hero.health_current - damage;

  if ( private.hero.health_current < 0 ) then

    private.hero.health_current = 0;

  end;

  mediator.Notify( "Model_Stage_EnemyAttackActivated",
  {
    damage      = damage,
    hero_health = private.hero.health_current
  } );

  if ( damage > 0 ) then

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.TryRecoverHeroHealth ( health_count )

  private.hero.health_current = private.hero.health_current + health_count;

  if ( private.hero.health_current > private.hero.health ) then

    private.hero.health_current = private.hero.health;

  end;

  mediator.Notify( "Model_Stage_HeroHealthRecovered",
  {
    delta          = health_count,
    health_current = private.hero.health_current,
    health_total   = private.hero.health
  } );

end;
--------------------------------------------------------------------------------
-- function ******* Score  ******* () end
--------------------------------------------------------------------------------
function private.InitScore ()

  private.score = 0;

end;
--------------------------------------------------------------------------------
function private.GetBasicChainScore ( chain_form, score_type )

  local chain_score = {};

  chain_score[ "Chain3Chips" ] = {};
  chain_score[ "Chain3Chips" ][ "Basic" ] = 60;
  chain_score[ "Chain3Chips" ][ "Bonus" ] = 120;

  chain_score[ "Chain4Chips" ] = {};
  chain_score[ "Chain4Chips" ][ "Basic" ] = 120;
  chain_score[ "Chain4Chips" ][ "Bonus" ] = 240;

  chain_score[ "Chain5Chips" ] = {};
  chain_score[ "Chain5Chips" ][ "Basic" ] = 200;
  chain_score[ "Chain5Chips" ][ "Bonus" ] = 300;

  chain_score[ "Chain6Chips" ] = {};
  chain_score[ "Chain6Chips" ][ "Basic" ] = 500;
  chain_score[ "Chain6Chips" ][ "Bonus" ] = 500;

  chain_score[ "ChainTChips" ] = {};
  chain_score[ "ChainTChips" ][ "Basic" ] = 240;
  chain_score[ "ChainTChips" ][ "Bonus" ] = 540;

  chain_score[ "ChainLChips" ] = {};
  chain_score[ "ChainLChips" ][ "Basic" ] = 240;
  chain_score[ "ChainLChips" ][ "Bonus" ] = 540;

  chain_score[ "ChainCross" ] = {};
  chain_score[ "ChainCross" ][ "Basic" ] = 240;
  chain_score[ "ChainCross" ][ "Bonus" ] = 540;

  chain_score[ "ChainHVNucleus" ] = {};
  chain_score[ "ChainHVNucleus" ][ "Basic" ] = 240;
  chain_score[ "ChainHVNucleus" ][ "Bonus" ] = 540;

  chain_score[ "ChainNucleusNucleus" ] = {};
  chain_score[ "ChainNucleusNucleus" ][ "Basic" ] = 240;
  chain_score[ "ChainNucleusNucleus" ][ "Bonus" ] = 540;

  return chain_score[ chain_form ][ score_type ];

end;
--------------------------------------------------------------------------------
function private.GetCellScore ( cell_name )

  local cell_score = {};

  cell_score[ "Dirt" ] = 500;
  cell_score[ "Cage" ] = 500;
  cell_score[ "Rock" ] = 500;
  cell_score[ "Mana" ] = 500;
  cell_score[ "Bug" ]  = 500;
  cell_score[ "Bubble" ]  = 500;
  cell_score[ "Spider" ]  = 500;
  cell_score[ "Crystal" ]  = 500;
  cell_score[ "Snowflake" ]  = 500;

  return cell_score[ cell_name ];

end;
--------------------------------------------------------------------------------
function public.FinalBonuses_ChangeScore ( chip_x, chip_y, chip_type, stage )
  
  private.ChangeScore(
    chip_x,
    chip_y,
    chip_type,
    1000 * private.GetStageState( stage )
  );
  
end;
--------------------------------------------------------------------------------
function private.ChangeScore ( chip_x, chip_y, chip_type, chip_score )

  local delta_score = math.floor( chip_score * ( 1 + private.hero.score ) );

  private.score = private.score + delta_score;

  mediator.Notify(
    "Model_Stage_ScoreChanged",
    {
      chip_x     = chip_x,
      chip_y     = chip_y,
      chip_type  = chip_type,
      chip_score = delta_score,
      score      = private.score
    }
  );

  private.UpdateTargetProgress_GainScore( delta_score );

end;
--------------------------------------------------------------------------------
-- function ******* Bonus  ******* () end
--------------------------------------------------------------------------------
function  public.Bonus_GetData ()

  if ( not private.bonus ) then

    return;

  end;

  if ( private.bonus.enabled ) then

    local ability = nil;

    if ( private.state_data.type == "fight" ) then

      ability = private.bonus.ability.name;

    end;

    return { 
      chip_types = private.bonus.chip_types, 
      target = private.bonus.target, 
      ability = ability 
    };

  end;

end;
--------------------------------------------------------------------------------
function  public.Bonus_IsActivated ()

  if ( not private.bonus ) then

    return;

  end;

  local activated = private.bonus.activated;

  if not ( private.state_data.type == "fight" ) then

    private.bonus.activated = false;

  end;

  return activated;

end;
--------------------------------------------------------------------------------
function  public.Bonus_TryActivate ()

  if ( not private.bonus ) then return; end;

  if not ( private.state_data.type == "fight" ) then return; end;

  if ( private.bonus.activated ) then

    if ( private.bonus.ability.counter == 1 ) then

      private.bonus.activated = false;

      private.Bonus_Reset();
    
      mediator.Notify( "Model_Stage_BonusEffectDisabled" );

    end;

  end;

  if ( private.bonus.counter == private.bonus.move ) then

    local ability = nil;

    if ( private.state_data.type == "fight" ) then

      ability = private.bonus.ability.name;

    end;

    private.Bonus_GenerateChipTypes();

    private.bonus.enabled = true;

  end;

  if ( private.bonus.enabled ) then

    mediator.Notify( "Model_Stage_BonusEnable" );

  end;

  return false;  

end;
--------------------------------------------------------------------------------
function private.Bonus_Init ()

  local stage_data = private.state_data;

  local bonus_data = stage_data.bonus;

  if ( not bonus_data ) then

    private.bonus = nil;

    return;

  end;

  private.bonus = {};

  private.bonus.counter = 0;

  if ( private.state_data.type == "fight" ) then

    private.Bonus_GenerateTarget();   

  end;

  private.bonus.generate_range = bonus_data.range;
  private.bonus.generate_chips = stage_data.chip_types;
  private.bonus.generate_count = bonus_data.count;

  private.Bonus_GenerateMove();

  private.Bonus_GenerateChipTypes();

end;
--------------------------------------------------------------------------------
function private.Bonus_ChangeCounters ()

  if ( not private.bonus ) then

    return;

  end;

  private.Bonus_ChangeCounter( 1 );

  if ( private.bonus.activated ) then

    private.Bonus_ChangeAbilityCounter( - 1 );

  end;

end;
--------------------------------------------------------------------------------
function private.Bonus_CheckComplete ( chip_type )

  if ( not private.bonus ) then

    return;

  end;

  local count = private.bonus.generate_count;

  local bonus = false;

  for i = 1, count do

    if ( chip_type == private.bonus.chip_types[ i ] ) then

      bonus = true;

      break;

    end;

  end;

  if ( private.bonus.counter >= ( private.bonus.move + 1 ) ) then

    if ( bonus ) then

      if ( private.state_data.type == "fight" ) then

        if not ( private.bonus.activated ) then

          private.bonus.activated = true;

          private.Bonus_Activate();

          private.Bonus_Disable();

        end;

      else
      
        private.bonus.activated = true;

        private.Bonus_Reset();

        private.Bonus_Disable();

      end;

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Bonus_Activate ()
  
  mediator.Notify( "Model_Stage_BonusCountChanged", private.bonus.ability );
  
  if ( private.bonus.target == "hero" ) then

    if ( private.bonus.ability.name == "dot" ) then

      private.Bonus_Hero_AddDoT();

    elseif ( private.bonus.ability.name == "damage" ) then

      private.Bonus_Hero_AddDamage();

    elseif ( private.bonus.ability.name == "stun" ) then

      private.Bonus_Hero_AddProtection();

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Bonus_Disable ()

  private.bonus.enabled = false;

  mediator.Notify( "Model_Stage_BonusDisable" );

end;
--------------------------------------------------------------------------------
function private.Bonus_Reset ()

  private.Bonus_ChangeCounter( - private.bonus.counter );

  private.TryResetHeroBonusAttackFactor();

  private.TryResetHeroBonusDoT();

  private.TryResetHeroBonusStun();

  if ( private.state_data.type == "fight" ) then

    private.Bonus_GenerateTarget();

  end;

  private.Bonus_GenerateMove();

end;
--------------------------------------------------------------------------------
function private.Bonus_ChangeCounter ( delta )

  private.bonus.counter = private.bonus.counter + delta;

end;
--------------------------------------------------------------------------------
function private.Bonus_ChangeAbilityCounter ( delta )

  if ( private.bonus.ability ) then

    private.bonus.ability.counter = private.bonus.ability.counter + delta;
    
    mediator.Notify( "Model_Stage_BonusCountChanged", private.bonus.ability );

  end;

end;
--------------------------------------------------------------------------------
function private.Bonus_Hero_AddDoT ()

  private.SetHeroBonusDoT( private.bonus.ability.value );

end;
--------------------------------------------------------------------------------
function private.Bonus_Hero_AddDamage ()

  private.SetHeroBonusAttackFactor( private.bonus.ability.factor );

end;
--------------------------------------------------------------------------------
function private.Bonus_Hero_AddProtection ()

  private.SetHeroBonusProtection( true );

end;
--------------------------------------------------------------------------------
function private.Bonus_GenerateMove ()

  local range = private.bonus.generate_range;

  private.bonus.move = math.random( range[ 1 ], range[ 2 ] );

end;
--------------------------------------------------------------------------------
function private.Bonus_GenerateTarget ()

  -- FIXME: Obsolete

  local targets = { "hero" };
  local random_index = math.random( 1, #targets );
  local target = targets[ random_index ];

  private.bonus.target = target;
  private.bonus.ability = private.Bonus_GenerateAbility();
  private.bonus.ability.counter = private.bonus.ability.period;


end;
--------------------------------------------------------------------------------
function private.Bonus_GenerateAbility ()

  local state_data   = private.state_data;
  local ability_data = state_data.bonus[ private.bonus.target ].ability;

  return ability_data[ math.random( 1, #ability_data ) ];

end;
--------------------------------------------------------------------------------
function private.Bonus_GenerateChipTypes ()

  local count = private.bonus.generate_count;
  local chips = {};

  for i = 1, #private.bonus.generate_chips do

    table.insert( chips, private.bonus.generate_chips[ i ] );

  end;

  private.bonus.chip_types = {};

  for i = 1, count do

    local req = "Request_Match3_GetPrevalentChipsList"
    local chips_type = common.MessageSend( req ).type;

    table.insert( private.bonus.chip_types, chips_type );

  end;

end;
--------------------------------------------------------------------------------
function private.Bonus_GetAbilityProbability ( abilities_data )

  -- EK> Полная вероятность.
  local total_probability = 0;

  for i = 1, #abilities_data do

    total_probability = total_probability + abilities_data[ i ].probability;

  end;

  local factor = 1 / total_probability;

  if not ( total_probability == 1 ) then

    total_probability = 1;

  end;

  -- EK> Сортировка.
  for i = ( #abilities_data - 1 ), 1, - 1 do 

    for j = 1, i do

      if ( factor * abilities_data[ j ].probability < factor * abilities_data[ j + 1 ].probability ) then

        abilities_data[ j ], abilities_data[ j + 1 ] = abilities_data[ j + 1 ], abilities_data[ j ];

      end;

    end;

  end;

  local a = 0;

  local random = 0.1 * math.random( 1, 10 * total_probability );

  for i = 1, #abilities_data do

    if ( random > a ) and ( random <= a + factor * abilities_data[ i ].probability ) then
    
      return abilities_data[ i ];

    end;

    a = a + factor * abilities_data[ i ].probability;

  end;

end;
--------------------------------------------------------------------------------
-- function ******* Targets ******* () end
--------------------------------------------------------------------------------
function private.InitTargets ()

  -- GainScore, DropItem, DestroyCell, CollectResource, DestroyGem

  private.targets = {};

  private.targets_id = {};

  local targets = private.state_data.targets;

  for t = 1, #targets do

    private.targets_id[ targets[ t ].type ] = t;

    private.targets[ t ] = targets[ t ];

    private.targets[ t ].progress = 0;
    private.targets[ t ].done = false;

    private.ActivateTarget( t );

  end;

end;
--------------------------------------------------------------------------------
function private.GetTargetByType ( target_type )

  return private.targets[ private.targets_id[ target_type ] ];

end;
--------------------------------------------------------------------------------
function private.ActivateTarget ( target_index )

  local target = private.targets[ target_index ];

  if ( target.type == "GainScore" ) then

    private.UpdateTargetProgress_GainScore( 0 );

  elseif ( target.type == "DropItem" ) then

    private.UpdateTargetProgress_DropItem( target.name, 0 );

  elseif ( target.type == "DestroyCell" ) then

    private.UpdateTargetProgress_DestroyCell( target.name, target.level, 0 );

  elseif ( target.type == "CollectResource" ) then

    private.UpdateTargetProgress_CollectResource( target.name, 0 );
    
  elseif ( target.type == "DestroyGem" ) then

    private.UpdateTargetProgress_DestroyGem( target.name, 0 );

  end;

end;
--------------------------------------------------------------------------------
function private.UpdateTargetProgress_GainScore ( progress_delta )

  local target = private.GetTargetByType( "GainScore" );

  if ( target ) then  

    target.progress = target.progress + progress_delta;

    private.UpdateTarget( target.type, progress_delta );

    mediator.Notify(
      "Model_Stage_TargetGainScoreUpdated", 
      { 
        delta = progress_delta
      }
    );

  end;

end;
--------------------------------------------------------------------------------
function private.UpdateTargetProgress_DropItem ( item_name, progress_delta )

  local target = private.GetTargetByType( "DropItem" );

  if ( target and not target.done and item_name == target.name ) then

    local progress_allowable = target.count - target.progress;

    if ( progress_delta > progress_allowable ) then

      progress_delta = progress_allowable;

    end;

    target.progress = target.progress + progress_delta;

    private.UpdateTarget( target.type, progress_delta );

  end;

end;
--------------------------------------------------------------------------------
function private.UpdateTargetProgress_DestroyCell ( cell_name, cell_level, progress_delta )

  private.Trace( "UpdateTargetProgress_DestroyCell: "..cell_name..cell_level.." = "..progress_delta.."." );

  local target = private.GetTargetByType( "DestroyCell" );

  if ( target
       and not target.done
       and target.type == "DestroyCell"
       and target.name == cell_name
       and 1 == cell_level
  ) then

    local progress_allowable = target.count - target.progress;

    if ( progress_delta > progress_allowable ) then

      progress_delta = progress_allowable;

    end;

    target.progress = target.progress + progress_delta;

    private.UpdateTarget( target.type, progress_delta );

  end;

end;
--------------------------------------------------------------------------------
function private.UpdateTargetProgress_CollectResource ( resource_name, progress_delta )

  private.Trace( "UpdateTargetProgress_CollectResource: "..resource_name.." = "..progress_delta.."." );

  local target = private.GetTargetByType( "CollectResource" );

  if ( target and target.name == resource_name ) then

    progress_delta = progress_delta * private.GetResourceFactor();

    target.progress = target.progress + progress_delta;

    private.UpdateTarget( target.type, progress_delta );

  end;

end;
--------------------------------------------------------------------------------
function private.UpdateTargetProgress_DestroyGem ( gem_name, progress_delta )

  private.Trace( "UpdateTargetProgress_DestroyGem: "..gem_name.."." );

  local target = private.GetTargetByType( "DestroyGem" );

  if ( target
       and not target.done
       and target.type == "DestroyGem"
       and target.name == gem_name
  ) then

    local progress_allowable = target.count - target.progress;

    if ( progress_delta > progress_allowable ) then

      progress_delta = progress_allowable;

    end;

    target.progress = target.progress + progress_delta;

    private.UpdateTarget( target.type, progress_delta );

  end;

end;
--------------------------------------------------------------------------------
function private.UpdateTarget ( target_type, progress_delta )

  local target = private.GetTargetByType( target_type );

  --DbgTrace( "UpdateTarget: < "..target_type.." > "..target.progress.." / "..target.count );

  mediator.Notify( "Model_Stage_TargetUpdated",
  {
    name     = target.name,
    index    = private.targets_id[ target_type ],
    type     = target_type,
    count    = target.count,
    progress = target.progress,
    delta    = progress_delta,
    done     = target.done
  } );

end;
--------------------------------------------------------------------------------
function private.TryCompleteTargets ()

  for t = 1, #private.targets do

    local target = private.targets[ t ];

    if ( not target.done and target.progress >= target.count ) then

      target.done = true;

      private.UpdateTarget( target.type );

      --DbgTrace( "TryCompleteTargets: < "..target.type.." > successfull." );

      int_panel_stage_progress_ride.Update_TargetComplete();

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.IsConditionCompleted ()

  local stage_type = private.state_data.type;

  if ( stage_type == "ride" ) then

    return ( private.IsTargetsCompleted() );

  elseif ( stage_type == "fight" ) then

    return ( private.IsEnemyDead() );

  end;

end;
--------------------------------------------------------------------------------
function private.IsConditionFailed ()

  local stage_type = private.state_data.type;

  if ( stage_type == "ride" ) then

    local moves = private.state_data.moves;

    if ( moves ) then

      return ( not private.IsMovesExist() and not private.IsTargetsCompleted() );

    else

      return ( not public.IsTimeExist() );

    end;

  elseif ( stage_type == "fight" ) then

    return ( private.IsHeroDead() );

  end;

end;
--------------------------------------------------------------------------------
function private.IsCombinationsNotExist ()

  if ( private.no_combinations ) then

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.IsTargetsCompleted ()

  local completed = true;

  for key, value in pairs( private.targets ) do

    if ( not value.done ) then

      completed = false;

      break;

    end;

  end;

  return completed;

end;
--------------------------------------------------------------------------------
function private.FinalBonuses_Spawn ()

  private.TryChangeMovesCount( - 1 );

  local chip_data = common.MessageSend( Request_Match3_SpawnFinalBonuses );
  local moves     = private.GetMovesCount();
  
  mediator.Notify(
    "Model_Stage_FinalBonuses_StartAnimation",
    {
      chip_x    = chip_data.x,
      chip_y    = chip_data.y,
      chip_type = chip_data.type,
      stage     = private.stage_name,
      moves     = moves
    }
  );

  if ( moves == 0 ) then

    private.FinalBonuses_Activate();

    return;

  end;
  
  local tme = moves / ( private.moves_remain * 3 );

  ObjSet( "tmr_stage_finalbonuses",
  {
    time    = tme,
    playing = true,
    endtrig = function ()
      
      mediator.Notify( "View_Match3_FieldEstablished" );

    end
  } );

  mediator.Notify( "Model_Match3_BonusSpawn", { type = "Horizontal" } );

end;
--------------------------------------------------------------------------------
function private.FinalBonuses_Activate ()
  
  private.bonuses_activated = true; 

  common.MessageSend( Command_Match3_ActivateFinalBonuses );

end;
--------------------------------------------------------------------------------
-- function ******* Rewards ******* () end
--------------------------------------------------------------------------------
function private.InitRewards ()

  private.reward = {};

  local chip_types = private.state_data.chip_types;

  for i = 1, #chip_types do
  
    if ( string.lower( chip_types[ i ] ) == "resource1" or
         string.lower( chip_types[ i ] ) == "resource2" or
         string.lower( chip_types[ i ] ) == "resource3" ) then

      private.reward[ string.lower( chip_types[ i ] ) ] = 0;

    end;

  end; 

end;
--------------------------------------------------------------------------------
function private.UpdateRewards ( chips_list )

  -- FIXME
  local reward_types = {
    Resource1 = "resource1",
    Resource2 = "resource2",
    Resource3 = "resource3"
  };

  local reward_buffer = {};

  for i = 1, #chips_list do

    local resource_name = reward_types[ chips_list[ i ].type ];
    
    if ( resource_name ) then

      private.IncRewardCount(
        resource_name,
        1,
        chips_list[ i ].x,
        chips_list[ i ].y
      );

      private.UpdateTargetProgress_CollectResource( resource_name, 1 );

      table.insert( reward_buffer,
        {
          name  = resource_name,
          count = private.reward[ resource_name ],
          delta = 1,
          x     = chips_list[ i ].x,
          y     = chips_list[ i ].y
        }
      );

    end;

  end;

  if ( #reward_buffer > 0 ) then

    mediator.Notify( "Model_Stage_RewardCountChanged", reward_buffer );

  end;

end;
--------------------------------------------------------------------------------
function private.IncRewardCount ( resource_name, delta, resource_x, resource_y )

  private.reward[ resource_name ] = private.reward[ resource_name ] + delta * private.GetResourceFactor();

  --mediator.Notify( "Model_Stage_RewardCountChanged", { name = resource_name, count = private.reward[ resource_name ], delta = delta, x = resource_x, y = resource_y } );

end;
--------------------------------------------------------------------------------
-- Возвращает данные только для начисления награды!
function private.GetCurrentReward ( first )

  if ( private.stage_name ) then

    local reward_list = {};
    local stage_name = private.stage_name;

    reward_list = private.GetGoldReward(     reward_list, stage_name, first );
    reward_list = private.GetWorkerReward(   reward_list, stage_name, first );
    reward_list = private.GetQuestReward(    reward_list, stage_name, first );
    reward_list = private.GetFragmentReward( reward_list, stage_name, first );
    reward_list = private.GetSkinpartReward( reward_list, stage_name, first );
    reward_list = private.GetStickersReward( reward_list, stage_name, first );
    reward_list = private.GetResourceReward( reward_list );

    return reward_list;

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function private.GetGoldReward ( reward_list, stage_name, first )

  if ( first and first.noreward ) then

    return reward_list;

  end;

  local stage_state = first.state_old or private.GetStageState( stage_name );
  local gold_count;

  if ( private.stage_settings.rewards[ stage_state ] ) then
    
    gold_count = private.stage_settings.rewards[ stage_state ].count;
    
  end;

  if ( private.state_data.type == "fight" ) then

    gold_count = private.Fights_GetGoldReward( stage_name );
  
    if ( not gold_count ) then

      return reward_list;

    end;

  end;

  local gold_bonus  = public.GetCurrentHero().gold;

  table.insert(
    reward_list,
    {
      name  = "gold",
      type  = "gold",
      count = gold_count + gold_bonus
    }
  );

  return reward_list;

end;
--------------------------------------------------------------------------------
function private.GetWorkerReward ( reward_list, stage_name, first )

  if ( first and first.noreward ) then

    return reward_list;

  end;

  local stage_data  = public.GetStaticData( stage_name );
  local stage_state = first.state_old or private.GetStageState( stage_name );

  if ( stage_data.state[ stage_state ].reward ) then

    for i = 1, #stage_data.state[ stage_state ].reward do

      local reward = stage_data.state[ stage_state ].reward[ i ];

      if ( "worker" == reward.type ) then
        
        local mine_name = m_mines.GetWorkerMineName( stage_name );
        local mine_data = m_mines.GetMineData( mine_name );
        if ( ( mine_data.progress or 0 ) < mine_data.progress_max ) then
          table.insert( reward_list, reward );
        end;

      end;

    end;

  end;

  return reward_list;

end;
--------------------------------------------------------------------------------
function private.GetQuestReward ( reward_list, stage_name, first )

  local stage_data  = public.GetStaticData( stage_name );
  local stage_state = first.state_old or private.GetStageState( stage_name );

  for s = 1, stage_state do

    if ( stage_data.state[ s ].reward ) then

      for r = 1, #stage_data.state[ s ].reward do

        local reward = stage_data.state[ s ].reward[ r ];

        if ( reward.type == "quest" ) then

          if ( m_quests.IsStageItemActive( reward.name, stage_name, "stage", stage_data.section ) ) then

            table.insert( reward_list, reward );

          end;

        end;

      end;

    end;

  end;

  return reward_list;

end;
--------------------------------------------------------------------------------
function private.GetFragmentReward ( reward_list, stage_name, first )

  local stage_data  = public.GetStaticData( stage_name );
  local stage_state = first.state_old or private.GetStageState( stage_name );

  if ( stage_data.state[ stage_state ].reward ) then

    for r = 1, #stage_data.state[ stage_state ].reward do

      local reward = stage_data.state[ stage_state ].reward[ r ];

      if ( reward.type == "fragment" ) then

        local build_data = m_builds.GetBuildData( m_builds.GetBuildNameByFragment( reward.name ));

        if ( not cmn.IsEntityInList( build_data.collected_fragments, reward.name ) ) then

          table.insert( reward_list, reward );

        end;

      end;

    end;

  end;

  return reward_list;

end;
--------------------------------------------------------------------------------
function private.GetSkinpartReward ( reward_list, stage_name, first )

  local stage_data  = public.GetStaticData( stage_name );
  local stage_state = first.state_old or private.GetStageState( stage_name );

  if ( stage_data.state[ stage_state ].reward ) then

    for r = 1, #stage_data.state[ stage_state ].reward do

      local reward = stage_data.state[ stage_state ].reward[ r ];

      if ( reward.type == "skinpart" ) then

        table.insert( reward_list, reward );

      end;

    end;

  end;

  return reward_list;

end;
--------------------------------------------------------------------------------
function private.GetStickersReward ( reward_list, stage_name, first )

  if ( first and first.noreward ) then

    return reward_list;

  end;
  
  local stage_data  = public.GetStaticData( stage_name );
  local stage_state = first.state_old or private.GetStageState( stage_name );

  if ( stage_data.state[ stage_state ].reward ) then

    for r = 1, #stage_data.state[ stage_state ].reward do

      local reward = stage_data.state[ stage_state ].reward[ r ];

      if ( reward.type == "stickerpack" ) then

        table.insert( reward_list, reward );

      end;

    end;

  end;

  return reward_list;

end;
--------------------------------------------------------------------------------
function private.GetResourceReward ( reward_list )

  if ( private.reward ) then

    for resource_name, resource_count in pairs( private.reward ) do

      if ( resource_count > 0 ) then

        local bonus = public.GetCurrentHero()[ resource_name ];

        table.insert(
          reward_list,
          {
            type  = "resource",  
            name  = resource_name,
            count = resource_count + bonus
          }
        );

      end;

    end;

  end;

  return reward_list;

end;
--------------------------------------------------------------------------------
function private.TryGetReward ( stage_name, data_for_reward )

  local reward_data = private.GetCurrentReward( data_for_reward );
  local drop_reward = {}; 

  for i = 1, #reward_data do

    local reward_name  = reward_data[ i ].name;
    local reward_type  = reward_data[ i ].type;
    local reward_count = reward_data[ i ].count;

    if ( "quest" == reward_type ) then

      mediator.Notify( "Model_Stage_QuestItemDropped", { name = reward_name, stage = stage_name } );

    elseif ( "worker" == reward_type ) then

      --

    else
  
      reward_data[ i ].stage_name = stage_name;
      reward_data[ i ].stage_state = data_for_reward.state_old;
      reward_data[ i ].category = "stage";

      m_items.TryObtain( reward_data[ i ] );

    end;

    reward_data[ i ].content = m_content.GetContent( reward_type, reward_name );

    table.insert( drop_reward, reward_data[ i ] );

  end;

  return drop_reward;

end;
--------------------------------------------------------------------------------
function private.GetResourceFactor ()

  return private.stage_settings.resource_factor;

end;
--------------------------------------------------------------------------------
function private.Fights_GetGoldReward ( stage_name )

  local query = 
  {
    query      = "find",
    group      = "static",
    collection = "spells",
    indice     = "unlock",
    criteria   = { like = stage_name }
  };

  if ( not m_storage.Execute( query ).result ) then

    return private.fight_settings.reward;

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function private.GetRewardProbability ( reward_data )

  -- EK> Полная вероятность.
  local total_probability = 0;

  for i = 1, #reward_data do

    total_probability = total_probability + reward_data[ i ].probability;

  end;

  local factor = 100 / total_probability;

  if not ( total_probability == 100 ) then

    total_probability = 100;

  end;

  -- EK> Сортировка.
  for i = ( #reward_data - 1 ), 1, - 1 do 

    for j = 1, i do

      if ( factor * reward_data[ j ].probability < factor * reward_data[ j + 1 ].probability ) then

        reward_data[ j ], reward_data[ j + 1 ] = reward_data[ j + 1 ], reward_data[ j ];

      end;

    end;

  end;

  local a = 0;

  local random = math.random( 1, total_probability );

  for i = 1, #reward_data do

    if ( random > a ) and ( random <= a + factor * reward_data[ i ].probability ) then
    
      return reward_data[ i ];

    end;

    a = a + factor * reward_data[ i ].probability;

  end;

end;
--------------------------------------------------------------------------------
function  public.GetFriendsStageScores ( stage_name )

  local friends_data = m_social.GetFriendsStageScores( stage_name );

  if ( friends_data and #friends_data > 0 ) then

    --[[local targets = private.GetCurrentTargets();

    local stage_score = 0;

    for t = 1, #targets do
      
      if ( targets[ t ].type == "GainScore" ) then

        stage_score = targets[ t ].count;

        break;

      end;

    end;

    for f = #friends_data, 1, - 1 do

      if ( friends_data[ f ].score < stage_score ) then

        table.remove( friends_data, f );

      end;

    end;]]

    return friends_data;

  end;

  return {};

end;
--------------------------------------------------------------------------------
function  public.GetStagesStat ( external_stages )
  
  local func_get_stage_data = function ( stages_data, stage_name ) 
    for i = 1, #stages_data do
      if ( stage_name == stages_data[ i ].name ) then 
        return stages_data[ i ]; 
      end;
    end;
  end;

  local query =
  {
    query      = "find_all",
    group      = "static",
    collection = "stages"
  };
  
  local stages_data = m_storage.Execute( query ).result or {};

  query =
  {
    query      = "find_all",
    group      = "player",
    collection = "stages"
  };
  
  local save_stages = external_stages or m_storage.Execute( query ).result;

  query =
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "type",
    criteria   = { like = "bonus" }
  };
  
  local bonus_stages = m_storage.Execute( query ).result;

  local result = {
    total_states       = 0;
    total_count        = #stages_data or 0;
    completed_states   = 0;
    completed_count    = 0;
    bonus_total        = #bonus_stages or 0;
    bonus_count        = 0;
    bonus_states       = 0;
    bonus_total_states = 0;
  };

  -- -- -- count totals -- -- -- 

  for s = 1, #stages_data do

    if ( stages_data[ s ].type ~= "dailyquest" ) then

      local section = stages_data[ s ].section;

      if ( not result[ section ] ) then

        result[ section ] = {
          bonus_total        = 0;
          bonus_count        = 0;
          bonus_states       = 0;
          bonus_total_states = 0;
          total_states       = 0;
          total_count        = 0;
          completed_count    = 0;
          completed_states   = 0;
          stages_prev        = 0;
        };

      end;

      result[ section ].total_states = result[ section ].total_states + #stages_data[ s ].state;
      result[ section ].total_count  = result[ section ].total_count + 1;
      result.total_states            = result.total_states + #stages_data[ s ].state;

    else

      result.total_count = result.total_count - 1;

    end;

  end;

  -- -- -- count completed -- -- -- 

  for i = 1, #save_stages do

    local stage_data = func_get_stage_data( stages_data, save_stages[ i ].name );
    local section = stage_data.section;

    if ( stage_data.type ~= "bonus" ) then

      if ( save_stages[ i ].completed ) then

        result[ section ].completed_count = result[ section ].completed_count + 1;
        result.completed_count            = result.completed_count + 1;

      end;

      if ( save_stages[ i ].state ) then

        result[ section ].completed_states = result[ section ].completed_states + save_stages[ i ].state - 1;
        result.completed_states            = result.completed_states + save_stages[ i ].state - 1;

      end;

      if ( save_stages[ i ].noreward ) then

        result[ section ].completed_states = result[ section ].completed_states + 1;
        result.completed_states            = result.completed_states + 1;

      end;

    end;

  end;

  -- -- -- bonus stages -- -- -- 

  if ( bonus_stages and #bonus_stages > 0 ) then

    for b = 1, #bonus_stages do

      local saved_stage = func_get_stage_data( save_stages, bonus_stages[ b ].name );
      local section = bonus_stages[ b ].section;

      result[ section ].bonus_total        = result[ section ].bonus_total + 1;
      result[ section ].bonus_total_states = result[ section ].bonus_total_states + #bonus_stages[ b ].state;
      result.bonus_total_states = result.bonus_total_states + #bonus_stages[ b ].state;

      if ( saved_stage ) then

        if ( saved_stage.completed ) then

          result[ section ].bonus_count = result[ section ].bonus_count + 1;
          result.bonus_count            = result.bonus_count + 1;

        end;

        if ( saved_stage.state ) then

          result[ section ].bonus_states = result[ section ].bonus_states + saved_stage.state - 1;
          result.bonus_states            = result.bonus_states + saved_stage.state - 1;

        end;

        if ( saved_stage.noreward ) then

          result[ section ].bonus_states = result[ section ].bonus_states + 1;
          result.bonus_states           = result.bonus_states + 1;

        end;

      end;

    end;

  end;

  -- -- -- stages_prev to section results -- -- -- 
  query =
  {
    query      = "find_all",
    group      = "static",
    collection = "sections"
  };
  
  local section_list = m_storage.Execute( query ).result;

  for s = 2, #section_list do

    local current  = result[ section_list[ s ].name ];
    local previous = result[ section_list[ s - 1 ].name ];

    if ( current ) and ( previous ) then

      current.stages_prev = ( s - 1 ) * ( previous.total_count - previous.bonus_total );

    else

      break;
 
    end;

  end;

  return result;
  
end;
--------------------------------------------------------------------------------
-- function ******* Kontagent ******* () end
--------------------------------------------------------------------------------
function  public.Kontagent_SetBoostersUsed ( value, booster_name )

  private.boosters_used = value;

  if ( value == 0 ) then

    private.continue_used = 0;

  end;

  if ( booster_name ) then

    private.continue_used = private.continue_used or 0;

    private.continue_used = private.continue_used + 1;

  end;

end;
--------------------------------------------------------------------------------
function  public.Kontagent_GetBoostersUsed ()

  return { private.boosters_used, private.continue_used };

end;
--------------------------------------------------------------------------------
function  public.Kontagent_GetStageState ( stage_name )

  local stage_info  = public.GetSavedData( stage_name ) or {};
  local stage_state = stage_info.state or 1;

  if ( stage_info.noreward ) then

    stage_state = 4;

  end;

  return stage_state;

end;
--------------------------------------------------------------------------------
function  public.Kontagent_GetCellsDestroyed ()

  local target = private.GetTargetByType( "DestroyCell" );

  if ( target ) then

    return { type = target.name, value = ( target.progress..", "..target.count ) };

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function  public.Kontagent_GetItemsDropped ()

  local target = private.GetTargetByType( "DropItem" );

  if ( target ) then

    return target.progress..", "..target.count;

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function  public.Kontagent_GetResourceCollected ()

  local target = private.GetTargetByType( "CollectResource" );

  if ( target ) then

    return target.progress..", "..target.count;

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function  public.Kontagent_GetExplosivesDestroyed ()

  local explosives = public.GetCurrentStageData().explosives;

  if ( explosives ) then
    
    local explosives_remain = ( common.MessageSend( Request_Match3_GetExplosivesRemain ) or {} ).explosives;

    if ( not explosives_remain ) then
      return nil;
    end;
    
    if ( private.IsCombinationsNotExist() ) then
      
      explosives_remain = explosives_remain + 1;
      
    end;
       
    return ( #explosives - explosives_remain )..", "..#explosives;

  end;

  return nil;

end;
--------------------------------------------------------------------------------
function  public.Kontagent_GetCurrentStage ()

  return private.kontagent_stage;

end;
--------------------------------------------------------------------------------
function  public.Kontagent_SetCurrentStage ( stage_name )

  private.kontagent_stage = stage_name;

end;
--------------------------------------------------------------------------------
-- function ******* Usable Lists ******* () end
--------------------------------------------------------------------------------
function private.Usable_List ()
  
  local section = v_map.Section_Current();

  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "stages",
    indice     = "completed",
    criteria   = { like = "true" }
  };

  local usable = m_storage.Execute( query ).result;
  
  if not ( usable ) then return false; end;
  
  local list = {};
  
  for s = 1, #usable do 
    
    if ( public.GetStaticData( usable[ s ].name ).section == section ) then
      
      table.insert( list, usable[ s ] );
      
    end;
  
  end;
  
  if ( #list < 1 ) then return false; end;

  return list;

end;
--------------------------------------------------------------------------------
function  public.Usable_Search ( deficient_name, deficient_section )

  if ( deficient_name:find( "resource" )) then

    local resource_name = deficient_name;
    local usable_stages = private.Usable_List();

    if ( usable_stages ) then

      local selected = {};

      for i = 1, #usable_stages do

        local stage_name  = usable_stages[ i ].name;
        local stage_data  = public.GetStaticData( stage_name );
        local stage_info  = public.GetSavedData(  stage_name ) or {};
        local stage_state = stage_info.state or 1;
        local chip_types  = stage_data.state[ stage_state ].chip_types;

        for t = 1, #chip_types do

          if ( string.lower( chip_types[ t ] ) == resource_name ) then
            
            table.insert( selected, usable_stages[ i ]);

          end;

        end;      

      end;

      if ( #selected > 1 ) then

        local random = math.random ( 1, #selected );

        return selected[ random ];

      elseif ( #selected > 0 ) then

        return selected[ 1 ];

      else

        return false;

      end;

    end;

  else

    local item_name = deficient_name;

    local query;

    if ( deficient_section ) then

      query = 
      {
        query      = "find_all",
        group      = "static",
        collection = "stages",
        indice     = "section",
        criteria   = { like = deficient_section }
      };

    else

      query = 
      {
        query      = "find_all",
        group      = "static",
        collection = "stages"
      };

    end;

    local stage_list = m_storage.Execute( query ).result;

    for i = 1, #stage_list do

      local stage = stage_list[ i ];

      for j = 1, #stage.state do

        local state = stage.state[ j ];

        if ( state.reward ) then

          for k = 1, #state.reward do

            local reward = state.reward[ k ];

            if ( reward.name == item_name ) then

              return stage;

            end;

          end;

        end;

      end;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
-- function ******* Cheater ******* () end
--------------------------------------------------------------------------------
function  public.Cheater_SetWinState ()

  private.score     = 99999;
  private.completed = true;
  
  if ( private.state_data.type == "fight" ) then
    
    private.moves_done = 1;
    private.enemy.health_current = 0;

  else

    private.moves = 0;

    for t = 1, #private.targets do

      local target = private.targets[ t ];
      target.progress = target.count;

    end;

  end;
  
  mediator.Notify( "Model_Stage_ViewTryContinue" );

end;
--------------------------------------------------------------------------------
function  public.Cheater_SetFailState ()

  if ( private.MovesDone_GetCount() == 0 ) then

    private.moves_done = 1;

  end;

  if ( private.state_data.type == "fight" ) then

    private.hero.health_current = 0;

  else

    local moves = private.moves or 0;
    private.TryChangeMovesCount( -moves, true );

  end;
  
  mediator.Notify( "Model_Stage_ViewTryContinue" );
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.List_HasReward (
    section_name,
    reward_type,
    reward_name_pattern
  )

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "section",
    criteria   = {
      like = section_name
    }
  };

  local stage_list = m_storage.Execute( query ).result or {};

  if ( #stage_list < 1 ) then

    return stage_list;

  end;

  local list = {};

  for d, stage_data in pairs( stage_list ) do

    for state_index, stage_state in pairs( stage_data.state ) do

      if ( stage_state.reward ) then

        for r, reward_data in pairs( stage_state.reward ) do

          if ( reward_data.type == reward_type              ) and
             ( reward_data.name:find( reward_name_pattern ) ) then

            local saved_data = public.GetSavedData( stage_data.name );

            if ( saved_data                               ) and
               ( ( saved_data.state or 0 ) <= state_index ) then

              table.insert( list, stage_data.name );

            end;

          end;

        end;

      end;

    end;

  end;

  return list;

end;
--------------------------------------------------------------------------------