-- name=m_storage
--------------------------------------------------------------------------------

appdata = {};
session = {};

private.Player = {};
 public.Player = {};
 
--------------------------------------------------------------------------------
function public.Init ()
  
  private.is_debug    = wiz.IsCheatsEnabled();
  private.driver      = "ram"; --JK: "db" or "ram"
  private.readonly    = false;
  public.Execute      = private.Execute_DB;
  public.Player.Info  = private.Player.Info_DB;
  private.collections = {};
  appdata             = {
    static = {}
  };
  
  private.collections.static = {
    achievements   = { "name", "type" },
    boosters       = { "name", "type" },
    builds         = { "name", "type", "section", "unlock", "deadlock" },
    characters     = { "name", "section", "unlock", "quest_cage", "quest_free" },
    chests         = { "name", "stage", "section" },
    logic_settings = { "type" },
    lots           = { "name", "name_ios" },
    mines          = { "name", "section", "unlock" },
    quests         = { "name", "condition.type", "target.name", "target.pregive", "section" },
    sections       = { "name", "unlock" },
    skins          = { "name", "platform" },
    spells         = { "name", "unlock", "section" },
    stages         = { "section", "name", "type" },
    stickers       = { "name" }
  };
  
  private.Register( private.collections.static );
  
  if ( private.driver == "db" ) then
 
    -- JK: static is always cached
    for name, indices in pairs( private.collections.static ) do
      
      private.Load( "static", name, indices );
      
    end;
    
    return;
 
  end;
  
  public.Execute = private.Execute_RAM;

  -- JK: protected data is retrieved from cache on debug only
  if ( private.is_debug ) then

    public.Player.Info = private.Player.Info_RAM;

  end;
  
  for c, collection in pairs( ConfigGetTable( "parts.db" ).writable_collections ) do
    
    local header = common.String_Split( collection.name, "/" );
    local group  = header[ 1 ];
    local name   = header[ 2 ];
 
    if ( not private.IsProtected( group, name ) ) then
   
      if ( not private.collections[ group ] ) then
        
        private.collections[ group ] = {};
        appdata[ group ]             = {};
        
      end;
      
      private.collections[ group ][ name ] = {};
      
      for i, index in pairs( collection.indices ) do

        table.insert( private.collections[ group ][ name ], index );
        
      end;

    end;
    
  end;
  
  for group, collections in pairs( private.collections ) do
    
    for name, indices in pairs( collections ) do
      
      private.Load( group, name, indices );
      
    end;
    
  end;
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function public.Execute ( query )
  
  -- JK: see Init
  
end;
--------------------------------------------------------------------------------
function public.DbExecute ( query )
  
  return DbExecuteQuery( query );
  
end;
--------------------------------------------------------------------------------
function public.Rewrite ( group, name, data )
  
  if ( private.driver == "ram" ) then
  
    local indices = private.collections[ group ][ name ];
    local indexed = private.Index( indices, data );
  
    if ( not indexed ) then
      
      data = data[ 1 ];
      
    end;
    
    appdata[ group ][ name ] = {
      data    = data,
      indices = indexed
    };
    
  end;
  
  -- JK: insert in db immediately
  local query = 
  {
    query      = "insert",
    collection = group.."/"..name,
    value      = data
  };
  
  DbExecuteQuery( query );
  
  return true;
  
end;
--------------------------------------------------------------------------------
function public.Settings ( type_id )
  
  local query =
  {
    query      = "find",
    group      = "static",
    collection = "logic_settings",
    indice     = "type",
    criteria   = { like = type_id }
  };
  
  return public.Execute( query ).result.value;
  
end;
--------------------------------------------------------------------------------
function public.Player.Info ()
  
  -- JK: see Init
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function public.Session_Set ( key, value )

  session[ key ] = value;

end;
--------------------------------------------------------------------------------
function public.Session_Get ( key )

  if ( session[ key ] ) then
  
    return session[ key ];
  
  end;
  
  return false;

end;
--------------------------------------------------------------------------------
function public.Session_Clear ( key )

  if ( session[ key ] ) then
  
    session[ key ] = nil;
    
    return true;
    
  end;
  
  return false;

end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function public.Database_Attach ()

  if ( private.db_method_backup                       ) and
     ( type( private.db_method_backup ) == "function" ) then

    _G[ "DbExecuteQuery" ] = private.db_method_backup;

  end;

end;
--------------------------------------------------------------------------------
function public.Database_Detach ()
  
  if ( private.driver == "ram" ) then

    private.db_method_backup = _G[ "DbExecuteQuery" ];
  
    _G[ "DbExecuteQuery" ] = function () return { resut = 1 }; end;
  
    return true;
    
  end;
  
  return false;
  
end;
--------------------------------------------------------------------------------
function public.Database_SetRO ()
  
  private.readonly = true;
  
end;
--------------------------------------------------------------------------------
function public.Database_SetRW ()
  
  private.readonly = false;
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.Execute_DB ( query )
  
  local group, indice = "", "";
  
  if ( query.group ~= "static" ) then
  
    group = query.group.."/";
  
  else
    
    -- JK: static is always cached
    return private.Execute_RAM( query );
    
  end;
  
  query.group = nil;
  
  if ( query.indice ) then
    
    indice       = ":"..query.indice;
    query.indice = nil;
    
  end;
  
  query.collection = group..query.collection..indice;
  
  local response = DbExecuteQuery( query );
  
  if ( query.query ~= "find"     ) and
     ( query.query ~= "find_all" ) then
     
     return { result = ( response.result == 1 ) };
     
  end;
  
  return response;
  
end;
--------------------------------------------------------------------------------
function private.Execute_RAM ( query )

  -- JK: protected data is always retrieved from DB
  if ( private.IsProtected( query.group, query.collection ) ) then

    return private.Execute_DB( query );

  end;

  local methods = {
    find       = private.FindAll,
    find_i     = private.FindByIndex,
    find_all   = private.FindAll,
    find_all_i = private.FindAllByIndex,
    insert     = private.Insert,
    update     = private.Update,
    update_i   = private.UpdateByIndex,
    delete_i   = private.DeleteByIndex,
    delete_all = private.DeleteAll
  };
  
  local method = query.query;
  
  if ( query.indice ) then
    
    method = method .. "_i";
    
  end;
  
  return { result = methods[ method ]( query ) };
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.Player.Info_DB ()
  
  local query =
  {
    query      = "find",
    collection = "player/info"
  };
  
  return DbExecuteQuery( query ).result;
  
end;
--------------------------------------------------------------------------------
function private.Player.Info_RAM ()
  
  return private.Copy( appdata.player.info.data );
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.IsProtected ( group, name )

  local protected = {
    player = {
      info     = true,
      boosters = true
    }
  };

  if ( private.is_debug ) then

    return false;

  end;

  if ( protected[ group ]         ) and
     ( protected[ group ][ name ] ) then

    return protected[ group ][ name ];

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Register ( readonly_collections )

  local subs = {
    characters = { "1", "2", "3", "4", "5" },
    builds = { "1", "2", "3", "4", "5" },
    chests = { "1", "2", "3", "4", "5" },
    spells = { "1", "2", "3", "4", "5" },  
    stages = { "1", "2", "3", "4", "5", "dailyquest" },
    quests = { "1", "2", "3", "4", "5" },
    mines  = { "1", "2", "3", "4", "5" }
  };
  
  local register_list = {};
  
  for key, collection in pairs( readonly_collections ) do
    
    local path = "assets/content/database/"..key;
    
    if ( subs[ key ] ) then
      
      for i, index in pairs( subs[ key ] ) do
        
        table.insert( register_list, path.."_"..index );
        
      end;
      
    else
    
      table.insert( register_list, path );
    
    end;
    
  end;
  
  DbRegisterReadonlyCollections( register_list );
  
end;
--------------------------------------------------------------------------------
function private.Load ( group, name, indices )
  
  local query = {
    query      = "find_all",
    collection = group.."/"..name
  };
  
  if ( group == "static" ) then
    
    query.collection = name;
    
  end;

  local data = DbExecuteQuery( query ).result;
  
  --DbgTrace( "[ STORAGE ] "..group.."/"..name.." ("..( tostring( #data ) )..")" );
  
  local indexed = private.Index( indices, data );
  
  if ( not indexed ) then
    
    data = data[ 1 ];
    
  end;
  
  appdata[ group ][ name ] = {
    data    = data,
    indices = indexed
  };
  
  --DbgTrace( "[STORAGE] RAM consumed: "..math.floor( collectgarbage( "count" ) ).." KB" );
  
end;
--------------------------------------------------------------------------------
function private.Index ( index_names, data )

  if ( #index_names < 1 ) then return nil; end;
  
  local indices = {};
  
  for i, index in pairs( index_names ) do
    
    local indexed = {};
    
    for d, item in pairs( data ) do
      
      local value;
      
      if ( index:find( "%." ) ) then
        
        local s = common.String_Split( index, "." );
        
        if ( item[ s[ 1 ] ] ) then
   
          value = item[ s[ 1 ] ][ s[ 2 ] ];
   
        end;
        
      else
        
        value = item[ index ];
        
      end;
      
      local key = "_index";
      
      if ( value ~= nil ) and ( value ~= "" ) then
        
        if ( type( value ) ~= "table" ) then
        
          key = "_"..tostring( value );
        
        end;
        
        if ( indexed[ key ] ) then
      
          table.insert( indexed[ key ], d );
          
        else
          
          indexed[ key ] = { d };
          
        end;
        
      end;
      
    end;
    
    indices[ index ] = indexed;
    
  end;
  
  return indices;
  
end;
--------------------------------------------------------------------------------
function private.Copy ( object )
  
  if ( type( object ) == "table" ) then
    
    local copy = {};
  
    for key, value in pairs( object ) do
        
      if ( type( value ) == "table" ) then
        
        value = private.Copy( value );
        
      end;
          
      copy[ key ] = value;
        
    end;
    
    return copy;
    
  end;
    
  return object;
  
end;
--------------------------------------------------------------------------------
function private.SelectFields ( fields, data )
  
  if ( data == nil ) then
    
    return nil;
    
  end;
  
  if ( type( fields ) == "string" ) then
    
    return data[ fields ];
    
  end;
  
  local result = {};
  for key, value in pairs( fields ) do
  
    result[ key ] = data[ value ];
    
  end;
  
  return result;
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.FindByIndex ( query )

  --JK: -1 is non-existent index
  local indexes = appdata[ query.group ][ query.collection ].indices[ query.indice ][ "_"..query.criteria.like ] or { -1 };
  local data    = appdata[ query.group ][ query.collection ].data[ indexes[ 1 ] ]
  
  if ( query.result ) then
  
    return private.Copy( private.SelectFields( query.result, data ) );
  
  end;
  
  return private.Copy( data );
  
end;
--------------------------------------------------------------------------------
function private.FindAll ( query )

  local data = appdata[ query.group ][ query.collection ].data;

  if ( query.result and #data > 0 ) then
  
    local result = {};
    
    for _, value in pairs( data ) do
    
      local fields = private.SelectFields( query.result, value );
      table.insert( result, fields );
    
    end;
    
    return private.Copy( result );
  
  end;
  
  return private.Copy( data );
  
end;
--------------------------------------------------------------------------------
function private.FindAllByIndex ( query )
  
  local result  = {};
  local indexes = appdata[ query.group ][ query.collection ].indices[ query.indice ][ "_"..query.criteria.like ] or {};
  local data    = appdata[ query.group ][ query.collection ].data;
  
  for i, index in pairs( indexes ) do
  
    if ( query.result ) then
  
      local fields = private.SelectFields( query.result, data[ index ] );
      table.insert( result, fields );
    
    else
    
      table.insert( result, data[ index ] );
  
    end;
    
  end;
  
  return private.Copy( result );
  
end;
--------------------------------------------------------------------------------
function private.Insert ( query )
  
  -- JK: check for read-only
  if ( query.group == "static" ) or
     ( private.readonly        ) then

    -- JK: if readonly by flag, fake success
    return private.readonly;

  end;
  
  local index = #appdata[ query.group ][ query.collection ].data + 1;
  
  table.insert( appdata[ query.group ][ query.collection ].data, query.value );
  
  -- JK: indexing
  for i, indice in pairs( private.collections[ query.group ][ query.collection ] ) do
    
    local key  = "_index";
    local value = query.value[ indice ];
    
    if ( value ) and ( value ~= "" ) then
    
      if ( type( value ) ~= "table" ) then
      
        key = "_"..tostring( query.value[ indice ] );
        
      end;
      
      if ( appdata[ query.group ][ query.collection ].indices[ indice ][ key ] ) then
      
        table.insert( appdata[ query.group ][ query.collection ].indices[ indice ][ key ], index );
      
      else
      
        appdata[ query.group ][ query.collection ].indices[ indice ][ key ] = { index };
      
      end;
      
    end;
    
  end;
  
  -- JK: insert in db immediately
  local query = 
  {
    query      = "insert",
    collection = query.group.."/"..query.collection,
    value      = query.value
  };
  
  DbExecuteQuery( query );
  
  return true;
  
end;
--------------------------------------------------------------------------------
function private.Update ( query )
  
  -- JK: check for read-only
  if ( query.group == "static" ) or
     ( private.readonly        ) then

    -- JK: if readonly by flag, fake success
    return private.readonly;

  end;
  
  for key, value in pairs( query.set ) do
  
    appdata[ query.group ][ query.collection ].data[ key ] = value;
  
  end;
  
  --JK: update in db immediately
  local query = 
  {
    query      = "update",
    collection = query.group.."/"..query.collection,
    set        = query.set
  };
  
  DbExecuteQuery( query );
  
  return true;
  
end;
--------------------------------------------------------------------------------
function private.UpdateByIndex ( query )
  
  -- JK: check for read-only
  if ( query.group == "static" ) or
     ( private.readonly        ) then

    -- JK: if readonly by flag, fake success
    return private.readonly;

  end;
  
  --JK: -1 is non-existent index
  local indexes = appdata[ query.group ][ query.collection ].indices[ query.indice ][ "_"..query.criteria.like ] or { -1 };
  local index   = indexes[ 1 ];
  
  if ( index < 0 ) then return false; end;
  
  for key, value in pairs( query.set ) do
  
    appdata[ query.group ][ query.collection ].data[ index ][ key ] = value;
    
    if ( appdata[ query.group ][ query.collection ].indices[ key ] ) then
    
      if ( appdata[ query.group ][ query.collection ].indices[ key ][ "_"..tostring( value ) ] ) then
      
        table.insert( appdata[ query.group ][ query.collection ].indices[ key ][ "_"..tostring( value ) ], index );
        
      else
        
        appdata[ query.group ][ query.collection ].indices[ key ][ "_"..tostring( value ) ] = { index };
        
      end;
    
    end;
  
  end;
  
  --JK: update in db immediately
  local query = 
  {
    query      = "update",
    collection = query.group.."/"..query.collection..":"..query.indice,
    criteria   = query.criteria,
    set        = query.set
  };
  
  DbExecuteQuery( query );
  
  return true;
  
end;
--------------------------------------------------------------------------------
function private.DeleteByIndex ( query )
  
  -- JK: check for read-only
  if ( query.group == "static" ) or
     ( private.readonly        ) then

    -- JK: if readonly by flag, fake success
    return private.readonly;

  end;
  
  -- JK: -1 is non-existent index
  local indexes = appdata[ query.group ][ query.collection ].indices[ query.indice ][ "_"..query.criteria.like ] or { -1 };
  local index   = indexes[ 1 ];

  if ( index < 0 ) then

    -- JK: Object not found by index
    return false;

  end;

  table.remove( appdata[ query.group ][ query.collection ].data, index );
  
  -- JK: here we should re-index ALL elements
  appdata[ query.group ][ query.collection ].indices = {};
  
  appdata[ query.group ][ query.collection ].indices = private.Index(
    private.collections[ query.group ][ query.collection ],
    appdata[ query.group ][ query.collection ].data
  );
  
  --JK: delete from db immediately
  local query = 
  {
    query      = "delete",
    collection = query.group.."/"..query.collection..":"..query.indice,
    criteria   = query.criteria
  };
  
  DbExecuteQuery( query );
  
  return true;
  
end;
--------------------------------------------------------------------------------
function private.DeleteAll ( query )
  
  -- JK: check for read-only
  if ( query.group == "static" ) or
     ( private.readonly        ) then

    -- JK: if readonly by flag, fake success
    return private.readonly;

  end;
  
  appdata[ query.group ][ query.collection ].data = {};
  
  --JK: delete indexes
  for i, indice in pairs( private.collections[ query.group ][ query.collection ] ) do
    
    appdata[ query.group ][ query.collection ].indices[ indice ] = {};
    
  end;
  
  --JK: delete from db immediately
  local query = 
  {
    query      = "delete_all",
    collection = query.group.."/"..query.collection
  };
  
  DbExecuteQuery( query );
  
  return true;
  
end;
--------------------------------------------------------------------------------