-- name=migrations
--[[ function *** !README! *** ()

  Порядок работы с модулем:
  
  1) Напиши функцию-миграцию private.Migration_XXX_tttttt,
  где XXX - след. ревизия pack build из teamcity,
  tttttt - тип миграции: client, server или др.
  
  2) Проверь миграцию тестером private.Tester, например:
  
    private.Tester( private.Migration_404_client )
    
    путём раскомментирования блока кода в Init
    
  Он выводит в лог запросы к БД и результаты, но фактически
  не выполняет "update" и "insert"
  
  3) Внеси миграцию в private.migrations_list.new чтобы она 
  заработала у старых пользователей
  
  4) Внеси миграцию в config.json -> sml/migrations чтобы она 
  не заработала у новых пользователей
  
]]
--******************************************************************************
function private.Trace ( text )

  common.Trace( "[ MIGRATIONS ] "..text );

end;
--******************************************************************************
function public.Init ()

  private.migrations_list = {
    old = {
      { version = { major = 1, minor = 2 }, handler = private.Migration_1_2 },
      { version = { major = 1, minor = 3 }, handler = private.Migration_1_3 }
    },
    new = {
      --AL: Don't be hurry to add migration to this list, test it on Tester before
      { version = { revision = 360 }, name = "Migration_360_client" },
      { version = { revision = 371 }, name = "Migration_371_client" },
      { version = { revision = 385 }, name = "Migration_385_client" },
      { version = { revision = 393 }, name = "Migration_393_client" },
      { version = { revision = 400 }, name = "Migration_400_client" },
      { version = { revision = 402 }, name = "Migration_402_client" },
      { version = { revision = 403 }, name = "Migration_403_client" },
      { version = { revision = 404 }, name = "Migration_404_client" },
      { version = { revision = 437 }, name = "Migration_437_client" },
      { version = { revision = 438 }, name = "Migration_438_client" },
      { version = { revision = 450 }, name = "Migration_450_client" },
      { version = { revision = 493 }, name = "Migration_493_client" },
      { version = { revision = 494 }, name = "Migration_494_client" },
      { version = { revision = 532 }, name = "Migration_532_client" },
      { version = { revision = 553 }, name = "Migration_553_client" },
      { version = { revision = 560 }, name = "Migration_560_client" },
      { version = { revision = 572 }, name = "Migration_572_client" },
      { version = { revision = 573 }, name = "Migration_573_client" },
      { version = { revision = 574 }, name = "Migration_574_client" },
      { version = { revision = 596 }, name = "Migration_596_client" },
      { version = { revision = 637 }, name = "Migration_637_client" },
      { version = { revision = 763 }, name = "Migration_763_client" },
      { version = { revision = 777 }, name = "Migration_777_client" }
    }
  };
  
  private.version = private.GetParsedVersion( private.GetVersion() );
  
  local ignore_saved = private.OldMigrationsStart( private.version );
  private.NewMigrationsStart( private.version, ignore_saved );
  
  -- JK: uncomment below to test your migration
  -- private.Tester( private.Migration_777_client );
  
end;
--******************************************************************************
function private.GetVersion ()
  
  local result, storage_version = parts.storage.data.Read( "sml/need_run_migration" );
  
  if ( storage_version                     ) and 
     ( storage_version ~= ""               ) and 
     ( storage_version ~= "false"          ) and 
     ( not storage_version:find( "0.0.0" ) ) then 
  
    parts.storage.data.Write( "sml/need_run_migration", "false" );
    return storage_version;
    
  else
    
    return parts.version.GetSaved( "game" );
    
  end;
  
end;
--******************************************************************************
function private.GetParsedVersion ( raw_version )

  local version = {};

  version.major, version.minor, version.build, version.revision = 
    string.match( raw_version, "(%d+).(%d+).(%d+).(%d+)" );

  for k, v in pairs( version ) do
    version[ k ] = tonumber( v );
  end;

  return version;

end;
--******************************************************************************
function private.Tester ( migration )
  
  if ( migration and type( migration ) == "function" ) then
    
    DbgTrace( "***********************************" );
      
    local oldDbExecuteQuery = DbExecuteQuery;
    
    DbExecuteQuery = function ( query )
    
      DbgTrace( "+++++++++++++++++++++++++++++++++" );
        
      if ( query.query:find( "find" ) ) then
        
        local result = oldDbExecuteQuery( query );
        
        DbgTrace( 
          common.TableToString( query )..
          "-> -> -> -> -> -> -> -> -> -> -> -> ->"..
          common.TableToString( result ) 
        );
        
        return result;
        
      else
        
        DbgTrace( common.TableToString( query ) );
        
      end;
    
    end;
    
    migration();
    
    DbExecuteQuery = oldDbExecuteQuery;
    
  end;
  
end;
--******************************************************************************
-- function *** OLD *** ()
--******************************************************************************
function private.OldMigrationsStart ( version )
  
  private.Trace( "[ OLD ] Saved game version: "..
    version.major.."."..version.minor.."."..version.build );
  
  if ( version.major == 1 and version.minor < 4 ) then

    for m, migration in ipairs( private.migrations_list.old ) do

      if
      ( version.major < migration.version.major ) or 
      ( version.minor < migration.version.minor ) or 
      ( migration.version.build and ( version.build < migration.version.build )) 
      then

        private.Trace( "[ OLD ] Migrating to version: "..
          migration.version.major.."."..
          migration.version.minor.."."..
          ( migration.version.build or "X" )
        );

        migration.handler();

      elseif ( version.major == 1 and version.minor == 3 ) then
        
        local query =
        {
          query      = "find",
          group      = "player",
          collection = "sections",
          indice     = "name",
          criteria   = { like = "section_1" }
        };
        local find_answer = m_storage.Execute( query ).result.intro;
        
        if ( not find_answer ) then
          
          private.Trace ( "[ OLD ] Using 1.3 migration fix" );
          migration.handler();
          
        end;
        
      end;

    end;
    
    return true;

  else
    
    private.Trace ( "[ OLD ] No need to use old mechanics" );
    return false;
    
  end;

end;
--******************************************************************************
-- function *** OLD : LIST *** ()
--******************************************************************************
function private.Migration_1_2 ()

  -- Add 100 gold -- -- -- -- -- -- -- -- 
  local query = 
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        = 
    { 
      update = true,
      gold   = 100
    }
  };
  
  m_storage.Execute( query );

end;
--******************************************************************************
function private.Migration_1_3 ()

  -- Set section 1 intro as played -- -- -- -- -- -- -- -- -- -- -- -- -- --
  local query =
  {
    query      = "update",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = "section_1" },
    set        = { intro = true }
  };
  
  m_storage.Execute( query );
  
  -- Fix stonegate quest -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  local quest_name = "qst_collect_stonegate_fragments";
  local quest_criteria = { like = quest_name };
  local item_name = "stonegate_star";

  query =
  {
    query      = "find",
    group      = "player",
    collection = "quests_current",
    indice     = "name",
    criteria   = quest_criteria
  };
  local quest_current = m_storage.Execute( query ).result;

  query =
  {
    query      = "find",
    group      = "static",
    collection = "quests",
    indice     = "name",
    criteria   = quest_criteria
  };
  
  local quest_data = m_storage.Execute( query ).result;

  if ( quest_current ) then

    quest_current.list = quest_current.list or {};
    
    if ( not cmn.IsEntityInList( quest_current.list, item_name )) then
  
      if ( #quest_current.list == #quest_data.target.list - 1 ) then

        local quest_remove_query = 
        {
          query      = "delete",
          group      = "player",
          collection = "quests_current",
          indice     = "name",
          criteria   = quest_criteria
        };
        m_storage.Execute( quest_remove_query );

        local quest_insert_query = 
        {
          query      = "insert",
          group      = "player",
          collection = "quests_completed",
          value      = { name = quest_name }
        };
        m_storage.Execute( quest_insert_query );

      else

        local new_list = quest_current.list;
        table.insert( new_list, item_name );

        local quest_update_query = 
        {
          query      = "update",
          group      = "player",
          collection = "quests_current",
          indice     = "name",
          criteria   = quest_criteria,
          set        = { list = new_list }
        };
        m_storage.Execute( quest_update_query );

      end;

    end;

  else

    local quest_completed_query =
    {
      query      = "find",
      group      = "player",
      collection = "quests_completed",
      indice     = "name",
      criteria   = quest_criteria
    };
    local quest_completed = m_storage.Execute( quest_completed_query ).result;

    if ( not quest_completed ) then

      local build_info_query =
      {
        query      = "find",
        group      = "player",
        collection = "builds",
        indice     = "name",
        criteria   = { like = "island_1" }
      };
      local build_info = m_storage.Execute( build_info_query ).result;

      if ( 
      build_info and 
      build_info.collected_fragments and ( 
      #build_info.collected_fragments > 0 )) then

        if ( #build_info.collected_fragments >= #quest_data.target.list - 1 ) then

          local quest_insert_query = 
          {
            query      = "insert",
            group      = "player",
            collection = "quests_completed",
            value      = { name = quest_name }
          };
          m_storage.Execute( quest_insert_query );

        else

          local insert_value = { 
            name = quest_name,
            list = build_info.collected_fragments
          };

          local insert_query =
          {
            query      = "insert",
            group      = "player",
            collection = "quests_current",
            value      = insert_value
          };
          m_storage.Execute( insert_query );

        end;

      end;

    end;

  end;

end;
--******************************************************************************
-- function *** NEW *** ()
--******************************************************************************
function private.NewMigrationsStart ( version, ignore_saved )
  
  local query;
  
  local function func_OnOnline ()
  end;
  
  -- -- -- Shedule server-require migrations on 
  local function func_OnOnline ()
    mediator.Unsubscribe( "Model_Client_ConnectionRestored", func_OnOnline );
    --private.CheckAllMigrations();
  end;
  
  local is_online = true;
  if ( not m_client.IsOnline() ) then
    mediator.Subscribe( "Model_Client_ConnectionRestored", func_OnOnline );
    is_online = false;
  end;
  
  -- -- -- Get migrations and split by type
  local could_migrations;
  
  if ( ignore_saved ) then
    
    could_migrations = {};
  
  else
  
    query =
    {
      query      = "find",
      group      = "sml",
      collection = "migrations"
    };
    could_migrations = m_storage.Execute( query ).result.migration or {};
    
  end;

  for m, migration in ipairs( private.migrations_list.new ) do
    
    if ( migration.name:find( "client" ) or ( migration.name:find( "server" ) and is_online ) ) then
      could_migrations = private.NewTryMigrate( could_migrations, migration.name );
    
    --elseif ( migration.name:find( "local" )) then
    --  local_migrations = private.NewTryMigrate( local_migrations, migration.name );

    end;

  end;

  query =
  {
    query      = "update",
    group      = "sml",
    collection = "migrations",
    set        = { migration = could_migrations }
  };
  
  m_storage.Execute( query );

end;
--******************************************************************************
function private.NewTryMigrate ( collection, migration_name )
  
  if ( not cmn.IsEntityInList( collection, migration_name ) ) then 
    private.Trace( "[ NEW ] Doing " .. migration_name );
    private[ migration_name ]();
    table.insert( collection, migration_name );
  end;

  return collection;
  
end;
--******************************************************************************
-- function *** NEW : LIST *** ()
--******************************************************************************
function private.Migration_360_client ()

  -- Reduce swordstone fragments amount. Give 50 gold for each -- -- -- -- -- --
  local find_build_query =
  {
    query      = "find",
    group      = "player",
    collection = "builds",
    indice     = "name",
    criteria   = { like = "puzzle_1" }
  };
  
  local swordstone_data = m_storage.Execute( find_build_query ).result;

  if ( 
  swordstone_data and 
  swordstone_data.collected_fragments and
  #swordstone_data.collected_fragments >= 2 ) then

    local change_list = { 
      [1]  = false, -- не удалять
      [2]  = true,  -- удалять
      [3]  = 2,     -- заменить на 2
      [4]  = true,  -- ...
      [5]  = true,
      [6]  = 3,
      [7]  = true,
      [8]  = true,
      [9]  = true,
      [10] = 4,
      [11] = true,
      [12] = true,
      [13] = 5,
      [14] = true,
      [15] = 6
    };

    local new_fragment_list = {};
    --local new_gold_count    = 0;

    for f = 1, #swordstone_data.collected_fragments do

      local fragment_num = tonumber( swordstone_data.collected_fragments[ f ]:match( "%d+" ) );
      local change       = change_list[ fragment_num ];

      if ( not change ) then
        table.insert( new_fragment_list, "swordstone_part_"..fragment_num );
      elseif ( type( change ) == "number" ) then
        table.insert( new_fragment_list, "swordstone_part_"..change );
        --new_gold_count = new_gold_count + 75;
      else
        --new_gold_count = new_gold_count + 50;
      end;

    end;

    local update_build_query =
    {
      query      = "update",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = "puzzle_1" },
      set        = {
        collected_fragments = new_fragment_list
      }
    };
    m_storage.Execute( update_build_query );
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
    local quest_name = "qst_collect_swordstone_fragments";

    local find_quest_query =
    {
      query      = "find",
      group      = "player",
      collection = "quests_current",
      indice     = "name",
      criteria   = { like = quest_name }
    };
    local quest_data = m_storage.Execute( find_quest_query ).result;

    if ( quest_data ) then

      local update_quest_query =
      {
        query      = "update",
        group      = "player",
        collection = "quests_current",
        indice     = "name",
        criteria   = { like = quest_name },
        set        = {
          list     = new_fragment_list
        }
      };
      m_storage.Execute( update_quest_query );

    end;
    
  end;

end;
--******************************************************************************
function private.Migration_371_client ()

  -- Reduce swordstone fragments amount.
  local find_build_query =
  {
    query      = "find",
    group      = "player",
    collection = "builds",
    indice     = "name",
    criteria   = { like = "puzzle_1" }
  };
  local swordstone_data = m_storage.Execute( find_build_query ).result;

  if ( 
  swordstone_data and 
  swordstone_data.collected_fragments and
  #swordstone_data.collected_fragments > 0 ) then

    local change_list = { 
      [1] = true, --remove
      [2] = 1,
      [3] = 2,
      [4] = 3,
      [5] = 4,
      [6] = 5,
      [7] = 6
    };

    local new_fragment_list = {};
    
    for f = 1, #swordstone_data.collected_fragments do

      local fragment_num = tonumber( swordstone_data.collected_fragments[ f ]:match( "%d+" ) );
      local change       = change_list[ fragment_num ];

      if ( not change ) then
        table.insert( new_fragment_list, "swordstone_part_"..fragment_num );
      elseif ( type( change ) == "number" ) then
        table.insert( new_fragment_list, "swordstone_part_"..change );
      end;

    end;

    local update_build_query =
    {
      query      = "update",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = "puzzle_1" },
      set        = {
        collected_fragments = new_fragment_list
      }
    };
    m_storage.Execute( update_build_query );
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
    local quest_name = "qst_collect_swordstone_fragments";

    local find_quest_query =
    {
      query      = "find",
      group      = "player",
      collection = "quests_current",
      indice     = "name",
      criteria   = { like = quest_name }
    };
    local quest_data = m_storage.Execute( find_quest_query ).result;

    if ( quest_data ) then

      local update_quest_query =
      {
        query      = "update",
        group      = "player",
        collection = "quests_current",
        indice     = "name",
        criteria   = { like = quest_name },
        set        = {
          list     = new_fragment_list
        }
      };
      m_storage.Execute( update_quest_query );

    end;
    
  end;

end;
--******************************************************************************
function private.Migration_385_client ()

  -- insert 1st swordstone fragment
  local query =
  {
    query      = "find",
    group      = "player",
    collection = "stages",
    indice     = "name",
    criteria   = { like = "stage_1" }
  };
  
  local stage_1_data = m_storage.Execute( query ).result;
  
  if ( stage_1_data and stage_1_data.state and stage_1_data.state > 1 ) then
    
    query =
    {
      query      = "find",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = "puzzle_1" }
    };
    local swordstone_data = m_storage.Execute( query ).result;
    
    if ( not swordstone_data ) then
      
      query = 
      {
        query      = "insert",
        group      = "player",
        collection = "builds",
        value      = { 
          name                = "puzzle_1",
          constructed         = false,
          unlocked            = false,
          collected_fragments = { "swordstone_part_1" }
        }
      };
      m_storage.Execute( query );
      
      query = 
      {
        query      = "insert",
        group      = "player",
        collection = "quests_current",
        value      = { 
          name     = "qst_collect_swordstone_fragments",
          list     = { "swordstone_part_1" }
        }
      };
      m_storage.Execute( query );
      
    elseif ( swordstone_data.constructed ) then
      
      local list = swordstone_data.collected_fragments;
      if ( not cmn.IsEntityInList( list, "swordstone_part_1" )) then
      
        table.insert( list, "swordstone_part_1" );
        
        query = 
        {
          query      = "update",
          group      = "player",
          collection = "builds",
          indice     = "name",
          criteria   = { like = "puzzle_1" },
          set        = { collected_fragments = list }
        };
        m_storage.Execute( query );
      
      end;
      
    elseif ( #swordstone_data.collected_fragments > 0 ) then

      local change_list = {
        [1] = 2,
        [2] = 3,
        [3] = 4,
        [4] = 5,
        [5] = 6
      };

      local list = {};
      table.insert( list, "swordstone_part_1" );
      
      for f = 1, #swordstone_data.collected_fragments do

        local fragment_num = tonumber( swordstone_data.collected_fragments[ f ]:match( "%d+" ) );
        local change       = change_list[ fragment_num ];
        table.insert( list, "swordstone_part_"..change );

      end;
        
      query = 
      {
        query      = "update",
        group      = "player",
        collection = "builds",
        indice     = "name",
        criteria   = { like = "puzzle_1" },
        set        = { collected_fragments = list }
      };
      m_storage.Execute( query );
      
      query = 
      {
        query      = "update",
        group      = "player",
        collection = "quests_current",
        indice     = "name",
        criteria   = { like = "qst_collect_swordstone_fragments" },
        set        = { list = list }
      };
      m_storage.Execute( query );
      
    end;
    
  end;
  
end;
--******************************************************************************
function private.Migration_393_client ()

  -- Change resources type to table.
  -- "resource1" : -1 -> "resource1" : [ -1 ]
  local resources_list = { "resource1", "resource2", "resource3" };
  
  for r = 1, #resources_list do
    
    local resource = resources_list[ r ];
    
    local query =
    {
      query      = "find",
      group      = "player",
      collection = "info"
    };
    
    local count = m_storage.Execute( query ).result[ resource ];
    
    if ( type( count ) == "number" ) then
      
      query =
      {
        query      = "update",
        group      = "player",
        collection = "info",
        set = { [ resource ] = { count, count } }
      };

      m_storage.Execute( query );
      
    end;

  end;
  
end;
--******************************************************************************
function private.Migration_400_client ()
  
  -- SPECIAL FIX FOR THOSE, WHO COME 1.4 FROM 1.3 AND GAIN sml_migrations FROM config
  
  local query =
  {
    query      = "find",
    group      = "sml",
    collection = "migrations"
  };
  local migrations_answer = m_storage.Execute( query ).result.migration;

  query =
  {
    query      = "find",
    group      = "player",
    collection = "info"
  };
  local resource_answer = m_storage.Execute( query ).result.resource1;
    
  if ( 
    migrations_answer and 
    cmn.IsEntityInList( migrations_answer, "Migration_393_client" )
  ) and ( 
    resource_answer and 
    type( resource_answer ) == "number" 
  ) then
  
    private.Migration_360_client();
    private.Migration_371_client();
    private.Migration_385_client();
    private.Migration_393_client();
  
  end;
  
end;
--********************************************************************************************************************
function private.Migration_402_client ()

  -- FIX: insert 1st swordstone fragment
  local query =
  {
    query      = "find",
    group      = "player",
    collection = "stages",
    indice     = "name",
    criteria   = { like = "stage_1" }
  };
  
  local stage_1_data = m_storage.Execute( query ).result;
  
  query =
  {
    query      = "find",
    group      = "player",
    collection = "builds",
    indice     = "name",
    criteria   = { like = "puzzle_1" }
  };
  
  local swordstone_data = m_storage.Execute( query ).result;
    
  if ( stage_1_data and stage_1_data.state and stage_1_data.state > 1 ) and
  ( swordstone_data and 
  swordstone_data.collected_fragments and
  #swordstone_data.collected_fragments == 0 ) then

    local list = { "swordstone_part_1"};
    
    query = 
    {
      query      = "update",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = "puzzle_1" },
      set        = { collected_fragments = list }
    };
    
    m_storage.Execute( query );
    
    query = 
    {
      query      = "update",
      group      = "player",
      collection = "quests_current",
      indice     = "name",
      criteria   = { like = "qst_collect_swordstone_fragments" },
      set        = { list = list }
    };
    
    m_storage.Execute( query );
      
  end;
  
end;
--******************************************************************************
function private.Migration_403_client ()
  
  --FIX: goldkeys quest, stars quest
  local find_goldkey_query =
  {
    query      = "find",
    group      = "player",
    collection = "quests_current",
    indice     = "name",
    criteria   = { like = "qst_collect_gold_key" }
  };
  local goldkey_quest_data = m_storage.Execute( find_goldkey_query ).result;
  
  if ( goldkey_quest_data and goldkey_quest_data.list and
  #goldkey_quest_data.list > 0 ) then
  
    --"stage_2", "stage_3", "stage_4", "stage_5", "stage_7"
    -- to
    --"stage_2", "stage_4", "stage_6", "stage_8", "stage_10"

    local change_list = {
      [2] = 2,
      [3] = 4,
      [4] = 6,
      [5] = 8,
      [7] = 10
    };
    
    local list = {};
    
    for f = 1, #goldkey_quest_data.list do

      local stage_num = tonumber( goldkey_quest_data.list[ f ]:match( "%d+" ) );
      local change    = change_list[ stage_num ];
      table.insert( list, "stage_"..change );

    end;
    
    local update_goldkey_query = 
    {
      query      = "update",
      group      = "player",
      collection = "quests_current",
      indice     = "name",
      criteria   = { like = "qst_collect_gold_key" },
      set        = { list = list }
    };
    m_storage.Execute( update_goldkey_query );
    
  end;
  
  local find_stars_query =
  {
    query      = "find",
    group      = "player",
    collection = "quests_current",
    indice     = "name",
    criteria   = { like = "qst_collect_star" }
  };
  local stars_quest_data = m_storage.Execute( find_stars_query ).result;
  
  if ( stars_quest_data and stars_quest_data.list and
  #stars_quest_data.list > 0 ) then
    
    --"stage_7", "stage_8", "stage_9", "stage_10", "stage_11"
    -- to
    --"stage_1", "stage_3", "stage_5", "stage_7", "stage_9"
    
    local change_list = {
      [7]  = 1,
      [8]  = 3,
      [9]  = 5,
      [10] = 7,
      [11] = 9
    };
    
    local list = {};
    
    for f = 1, #stars_quest_data.list do

      local stage_num = tonumber( stars_quest_data.list[ f ]:match( "%d+" ) );
      local change    = change_list[ stage_num ];
      table.insert( list, "stage_"..change );

    end;
    
    local update_stars_query = 
    {
      query      = "update",
      group      = "player",
      collection = "quests_current",
      indice     = "name",
      criteria   = { like = "qst_collect_star" },
      set        = { list = list }
    };
    m_storage.Execute( update_stars_query );
    
  end;
  
  
end;
--******************************************************************************
function private.Migration_404_client ()
  
  -- Remove 2nd swordstone fragment
  local find_build_query =
  {
    query      = "find",
    group      = "player",
    collection = "builds",
    indice     = "name",
    criteria   = { like = "puzzle_1" }
  };
  local swordstone_data = m_storage.Execute( find_build_query ).result;

  if ( 
  swordstone_data and 
  swordstone_data.collected_fragments and
  #swordstone_data.collected_fragments > 0 ) then

    local change_list = { 
      [1] = false,
      [2] = true, --remove
      [3] = 2,
      [4] = 3,
      [5] = 4,
      [6] = 5
    };

    local new_fragment_list = {};
    
    for f = 1, #swordstone_data.collected_fragments do

      local fragment_num = tonumber( swordstone_data.collected_fragments[ f ]:match( "%d+" ) );
      local change       = change_list[ fragment_num ];

      if ( not change ) then
        table.insert( new_fragment_list, "swordstone_part_"..fragment_num );
      elseif ( type( change ) == "number" ) then
        table.insert( new_fragment_list, "swordstone_part_"..change );
      end;

    end;

    local update_build_query =
    {
      query      = "update",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = "puzzle_1" },
      set        = {
        collected_fragments = new_fragment_list
      }
    };
    m_storage.Execute( update_build_query );
    -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
    local quest_name = "qst_collect_swordstone_fragments";

    local find_quest_query =
    {
      query      = "find",
      group      = "player",
      collection = "quests_current",
      indice     = "name",
      criteria   = { like = quest_name }
    };
    local quest_data = m_storage.Execute( find_quest_query ).result;

    if ( quest_data ) then

      local update_quest_query =
      {
        query      = "update",
        group      = "player",
        collection = "quests_current",
        indice     = "name",
        criteria   = { like = quest_name },
        set        = {
          list     = new_fragment_list
        }
      };
      m_storage.Execute( update_quest_query );

    end;
    
  end;

end;
--******************************************************************************
function private.Migration_437_client ()
  
  -- Remove skin bonuses from save
  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "skins"
  };
  
  local skins_list = m_storage.Execute( query ).result;
  
  local change_list = {};
  
  for s, skin in ipairs( skins_list ) do
    
    skin.bonus = nil;
    table.insert( change_list, skin );
      
  end;
  
  if ( #change_list > 1 ) then
    
    local query = 
    {
      query      = "delete_all",
      group      = "player",
      collection = "skins"
    };
    
    m_storage.Execute( query );
    
    for c, change in ipairs( change_list ) do
      
      query = 
      {
        query      = "insert",
        group      = "player",
        collection = "skins",
        value      = change
      };
      
      m_storage.Execute( query );
      
    end;
    
  end;
  
end;
--******************************************************************************
function private.Migration_438_client ()
  
  -- Give skinpart and skin for complete wizard's quest
  local query = 
  {
    query      = "find",
    group      = "player",
    collection = "quests_completed",
    indice     = "name",
    criteria   = { like = "qst_collect_gold_key" }
  };
  
  local quest_completed = m_storage.Execute( query ).result;
  
  query = 
  {
    query      = "find",
    group      = "player",
    collection = "skins",
    indice     = "name",
    criteria   = { like = "justknight" }
  };
  
  local skin_info = m_storage.Execute( query ).result;
  
  if ( quest_completed and ( not skin_info )) then
    
    query = 
    {
      query      = "insert",
      group      = "player",
      collection = "skins",
      value      = { 
        name  = "justknight",
        parts = { "justknight_gear" }
      }
    };
    
    m_storage.Execute( query );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Migration_450_client ()
  
  --JK: This migration will fix the hives
  
  -- UPD 1.8.0: This function is empty,
  -- because Migration_573_client removes
  -- hives from the player's save.
  
end;
--------------------------------------------------------------------------------
function private.Migration_493_client ()
  
  -- JK: this migration will fix quest completion for mine 6 construction
  -- Condition: if mine 6 is constructed, the quest must be complete
  -- Action: if the quest is still not complete, this migration will force it
  -- to complete
  
  local quest_name = "qst_construct_mine_6";
  local mine_name  = "mine_6";
  
  -- 1. Check if quest is complete
  local query =
  {
    query      = "find",
    group      = "player",
    collection = "quests_completed",
    indice     = "name",
    criteria   = { like = quest_name }
  };
  
  local quest = m_storage.Execute( query ).result;
  
  -- 2. If quest is complete, finish migration
  if ( quest ) then return; end;
  
  -- 3. Quest is not complete, check mine_6 state
  query =
  {
    query      = "find",
    group      = "player",
    collection = "mines",
    indice     = "name",
    criteria   = { like = mine_name }
  };

  local mine = m_storage.Execute( query ).result;
  
  -- 4. If mine_6 is active, force the quest to complete
  if ( mine ) and ( mine.state == "active" ) then
  
    query = 
    {
      query      = "delete",
      group      = "player",
      collection = "quests_current",
      indice     = "name",
      criteria   = { like = quest_name }
    };
    
    m_storage.Execute( query );

    query = 
    {
      query      = "insert",
      group      = "player",
      collection = "quests_completed",
      value      = { name = quest_name }
    };
    
    m_storage.Execute( query );
    
  end;
  
  -- 5. If mine_6 is not active and quest is not complete,
  -- it's OK, doing nothing :)
  
end;
--------------------------------------------------------------------------------
function private.Migration_494_client ()
  
  --AL> Give stage_220-quest to guys, who already done start conditions
  local quest_name = "qst_complete_stage_220";
  
  local saved_quest_query = 
  {
    query      = "find",
    group      = "player",
    collection = "quests_current",
    indice     = "name",
    criteria   = { like = quest_name }
  };
  local saved_quest = m_storage.Execute( saved_quest_query ).result;
  
  if ( saved_quest ) then return; end; -- RET: Quest already in list
    
  saved_quest_query = 
  {
    query      = "find",
    group      = "player",
    collection = "quests_completed",
    indice     = "name",
    criteria   = { like = quest_name }
  };
  
  saved_quest = m_storage.Execute( saved_quest_query ).result;
  
  if ( saved_quest ) then return; end; -- RET: Quest already done
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  local quest_data_query = 
  {
    query      = "find",
    group      = "static",
    collection = "quests",
    indice     = "name",
    criteria   = { like = quest_name }
  };
  local quest_data = m_storage.Execute( quest_data_query ).result;
  
  if ( quest_data.target.type == "QT_StageCompleted" ) then
    
    local saved_stage_query = 
    {
      query      = "find",
      group      = "player",
      collection = "stages",
      indice     = "name",
      criteria   = { like = quest_data.target.name }
    };
    
    local saved_stage = m_storage.Execute( saved_stage_query ).result;
    
    if ( saved_stage and saved_stage.state and ( saved_stage.state > 1 )) then
      return;     -- RET: Target stage already done
    end;
    
    saved_stage_query = 
    {
      query      = "find",
      group      = "player",
      collection = "stages",
      indice     = "name",
      criteria   = { like = quest_data.condition.name }
    };
    
    saved_stage = m_storage.Execute( saved_stage_query ).result;
    
    if ( saved_stage and saved_stage.state and ( saved_stage.state > 1 )) then
      
      local quest_insert_query = 
      {
        query      = "insert",
        group      = "player",
        collection = "quests_current",
        value      = { name = quest_name }
      };
      m_storage.Execute( quest_insert_query );
      
    end;
    
  end;
  
end;
--------------------------------------------------------------------------------
-- Change hearts donation count on PC from 1 to 2
--------------------------------------------------------------------------------
function private.Migration_532_client ()

  if ( IsPC() ) then

    local hearts_info = m_storage.Player.Info().hearts;

    local donation     = m_storage.Settings( "hearts" ).donate_count.pc;
    local donation_now = hearts_info.donate;

    local need_fix = false;

    if ( donation_now > 0 and donation_now < donation ) then

      local delta = donation - donation_now;
  
      hearts_info.donate = donation;
      hearts_info.count  = hearts_info.count + delta;

      need_fix = true;

    end;

    if ( hearts_info.facebook ~= 0 ) then

      hearts_info.count    = hearts_info.count - hearts_info.facebook;
      hearts_info.facebook = 0;

      need_fix = true;

    end;

    if ( need_fix ) then

      local query =
      {
        query      = "update",
        group      = "player",
        collection = "info",
        set        = { hearts = hearts_info }
      };

      m_storage.Execute( query );

    end;
    
  end;

end;
--------------------------------------------------------------------------------
-- Change sections content downloading.
--------------------------------------------------------------------------------
function private.Migration_553_client ()
    
  local query =
  {
    query      = "find",
    group      = "player",
    collection = "sections"
  };
  
  local sections = m_storage.Execute( query ).result;
  
  local enabled_count = 0;
  
  private.Trace( "Migration_553_client: < "..#sections.." >." );
  
  for s = 1, #sections do
    
    local section_name = sections[ s ].name;
    
    if ( sections[ s ].enabled ) then

      enabled_count = enabled_count + 1;
      
    end;
    
  end;
  
  local result, value = parts.storage.data.Read( "sml/allow_sections" );

  if ( not value ) or ( value == "" ) or ( value == "false" ) or ( value == "true" ) then

    parts.storage.data.Write( "sml/allow_sections", enabled_count );
    private.Trace( "Migration_553_client: set allow_sections to < "..enabled_count.." >." );

  end;

end;
--------------------------------------------------------------------------------
-- 1.8.0 dailybonus
--------------------------------------------------------------------------------
function private.Migration_560_client ()

  -- JK: remove keys, add day

  local query = {
    query      = "delete_all",
    collection = "player/dailybonus"
  };

  m_storage.DbExecute( query );

  query = {
    query      = "insert",
    collection = "player/dailybonus",
    value      = {
      day   = 0,
      ready = false
    }
  };

  m_storage.DbExecute( query );

  m_storage.Database_Detach();

  query =
  {
    query      = "update",
    group      = "player",
    collection = "dailybonus",
    set        = {
      day   = 0,
      ready = false,
      keys  = nil,
      first = nil
    }
  };

  m_storage.Execute( query );

  m_storage.Database_Attach();

end;
--------------------------------------------------------------------------------
-- Change completed misc fights saved state
-- 'completed, fight' -> 'not completed, ride'
--------------------------------------------------------------------------------
function private.Migration_572_client ()

  local query = 
  {
    query      = "find_all",
    group      = "static",
    collection = "stages",
    indice     = "type",
    criteria   = { like = "misc" }
  };

  local misc_stages = m_storage.Execute( query ).result or {};

  for i = 1, #misc_stages do

    local name = misc_stages[ i ].name;
    query = 
    {
      query      = "find",
      group      = "player",
      collection = "stages",
      indice     = "name",
      criteria   = { like = name }
    };

    local saved = m_storage.Execute( query ).result;

    if ( saved and saved.type == "fight" and saved.completed ) then

      query = 
      {
        query      = "update",
        group      = "player",
        collection = "stages",
        indice     = "name",
        criteria   = { like = name },
        set        = { type = "ride", completed = false }
      };

      m_storage.Execute( query );

    end;

  end;

end;
--------------------------------------------------------------------------------
-- Drop player hives data
--------------------------------------------------------------------------------
function private.Migration_573_client ()

  local query = 
  {
    query      = "find_all",
    group      = "player",
    collection = "hives"
  };

  local hives = m_storage.Execute( query ).result or {};

  for i = 1, #hives do

    local hive_name = hives[ i ].name;
    local token = string.format( "mte_map_cooldown_hives_%s", hive_name );

    if ( TimeIsEventExist( token ) ) then

      TimeFinishEvent();
      TimeUnsheduleEvent( token );

    end;

  end;

  local query = {
    query      = "delete_all",
    collection = "player/hives"
  };

  m_storage.DbExecute( query );

end;
--------------------------------------------------------------------------------
-- 1.8.0 Update Reward
--------------------------------------------------------------------------------
function private.Migration_574_client ()

  local query = 
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        = 
    { 
      update = true,
      reward =
      {
        { type = "booster", name = "hammer",    count = 1   },
        { type = "gold",    name = "gold",      count = 150 },
        { type = "booster", name = "magicgift", count = 2   }
      }
    }
  };
  
  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
-- Change saved skins data: store skinparts count instead of array
--------------------------------------------------------------------------------
function private.Migration_596_client ()

  local query =
  {
    query      = "find_all",
    group      = "player",
    collection = "skins"
  };

  local skins = m_storage.Execute( query ).result or {};

  for i, skin in pairs( skins ) do

    if ( skin.parts ) then

      skin.parts = #skin.parts;

      local query = 
      {
        query      = "update",
        group      = "player",
        collection = "skins",
        indice     = "name",
        criteria   = { like = skin.name },
        set        = skin
      };
      
      m_storage.Execute( query );

    end;

  end;

end;
--------------------------------------------------------------------------------
-- Fix for worker reward at section 1 stage 44.
--------------------------------------------------------------------------------
function private.Migration_600_client ()
  
  local update_query = 
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        = 
    { 
      update = false,
      reward = false
    }
  };
  
  m_storage.Execute( update_query );
  
  local query =
  {
    query      = "find",
    group      = "player",
    collection = "stages",
    indice     = "name",
    criteria   = { like = "stage_44" }
  };
  
  local stage = m_storage.Execute( query ).result;
 
  if ( stage and stage.noreward ) then
    
    query =
    {
      query      = "find",
      group      = "player",
      collection = "mines",
      indice     = "name",
      criteria   = { like = "mine_2" }
    };
    
    local mine = m_storage.Execute( query ).result or {};
    
    query =
    {
      query      = "find",
      group      = "static",
      collection = "mines",
      indice     = "name",
      criteria   = { like = "mine_2" }
    };

    local progress = m_storage.Execute( query ).result.progress;
    
    if ( mine ) and ( mine.progress ) and ( mine.progress < #progress ) then
     
      mine.progress = mine.progress + 1;
      if ( mine.progress == #progress ) then
        mine.state = "wait";
      end;
      
      query = 
      {
        query      = "update",
        group      = "player",
        collection = "mines",
        indice     = "name",
        criteria   = { like = "mine_2" },
        set        = mine
      };
      
      m_storage.Execute( query );
      
    end;
    
  end;
    
end;
--------------------------------------------------------------------------------
function private.Migration_637_client ()

  -- JK: 1. dailyquest reworked
  local unlock_stage = m_storage.Settings( "dailyquest" ).unlockstage;

  local query = {
    query      = "find",
    group      = "player",
    collection = "stages",
    indice     = "name",
    criteria   = { like = unlock_stage }
  };
 
  local unlocked = m_storage.Execute( query ).result.completed;

  if ( not unlocked ) then

    if ( parts.time.event.Exists( "mte_dailyquest_refresh" ) ) then

      parts.time.event.Finish();
      parts.time.event.Unshedule( "mte_dailyquest_refresh" );

    end;

    query =
    {
      query      = "update",
      group      = "player",
      collection = "dailyquest",
      set        = {
        ready  = false,
        reward = false
      }
    };
    
    m_storage.Execute( query );

  else

    -- JK: set all challenges beaten -> state = 2
    query =
    {
      query      = "find",
      group      = "player",
      collection = "dailyquest"
    };
    
    local dailyquest = m_storage.Execute( query ).result;
    
    -- JK: Additional check for dailyquest data
    if ( dailyquest            ) and
       ( dailyquest.challenge1 ) then

      for c = 1, 4 do

        dailyquest[ "challenge"..c ].state = 2;

      end;      

    -- JK: 1.2.0 has no dailyquest, but unlock stage may be complete
    -- Set fake challenge data
    else

      dailyquest = {
        challenge1 = {
          enemy = {
            attack = 11,
            health = 155,
            name   = "bandit",
          },
          stage = "stage_1_daily",
          state = 2
        },
        challenge2 = {
          enemy = {
            attack = 10,
            health = 290,
            name   = "pirate",
          },
          stage = "stage_2_daily",
          state = 2
        },
        challenge3 = {
          enemy = {
            attack = 13,
            health = 281,
            name   = "yeti",
          },
          stage = "stage_3_daily",
          state = 2
        },
        challenge4 = {
          enemy = {
            attack = 12,
            health = 176,
            name   = "spider",
          },
          stage = "stage_4_daily",
          state = 2
        }
      };

    end;

    query =
    {
      query      = "update",
      group      = "player",
      collection = "dailyquest",
      set        = {
        challenge1 = dailyquest.challenge1,
        challenge2 = dailyquest.challenge2,
        challenge3 = dailyquest.challenge3,
        challenge4 = dailyquest.challenge4,
        ready      = false,
        reward     = false
      }
    };
    
    m_storage.Execute( query );

  end;

  if ( parts.time.event.Exists( "mte_dailyquest_countdown_refresh" ) ) then

    parts.time.event.Finish();
    parts.time.event.Unshedule( "mte_dailyquest_countdown_refresh" );

  end;

  if ( parts.time.event.Exists( "mte_dailyquest_challenge_refresh" ) ) then

    parts.time.event.Finish();
    parts.time.event.Unshedule( "mte_dailyquest_challenge_refresh" );

  end;

  query = 
  {
    query      = "update",
    group      = "player",
    collection = "info",
    set        = 
    {
      update = { type = "replay" },
      reward = false
    }
  };
  
  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------
-- Set default player name to empty string (need for localized display)
--------------------------------------------------------------------------------
function private.Migration_763_client ()

  local name = m_storage.Player.Info().name;
  
  if ( name and ( name == "Player" ) ) then
  
    local query = 
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        = { name = false }
    };
    
    m_storage.Execute( query );
  
  end;

end;
--------------------------------------------------------------------------------
function private.Migration_777_client ()

  -- JK: Clear WinSameStage achievement

  local query = 
  {
    query      = "delete",
    group      = "player",
    collection = "achievements",
    indice     = "name",
    criteria   = {
      like = "favourite_level"
    }
  };

  m_storage.Execute( query );

end;
--------------------------------------------------------------------------------