-- name=strict
--------------------------------------------------------------------------------
-- Undefined global variables notifier.
-- Has no public interface, just run Init for enable error tracing.
-- Debug module required.
-- @module strict
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Setup traces about non-declared global variables usage.
--------------------------------------------------------------------------------
function public.Init ()
  
  -- Set new global variables metatable
  local mt = getmetatable( _G );
  if ( mt == nil ) then
    mt = {}; setmetatable( _G, mt );
  end;

  -- Declared global varibales array
  mt.__declared = {};
  
  -- Redefine access to global variables table indexes
  mt.__index = function (t, n)
  
    local info = private.GetInfo();
    if ( not mt.__declared[n] and info.what ~= "C" ) then
      private.TraceError( info.src, info.line, "'" .. n .. "' is undefined." );
    end;
    
    return rawget( t, n );
    
  end;

  -- Redefine new index creation in global variables table
  mt.__newindex = function (t, n, v)

    if not ( mt.__declared[n] ) then
    
      local info = private.GetInfo();
      local w = info.what;
      
      if ( w ~= "main" and w ~= "C" and v == nil ) then
        private.TraceError( info.src, info.line, 
                            "'" .. n .. "' : nil-initialization rejected!" );
        return;
      end;
      
      -- Set field declared
      mt.__declared[n] = true;
      
    end;
    
    rawset( t, n, v );

  end;
  
end;

--------------------------------------------------------------------------------
-- Get info about function with nil-declaration of new global variable.
-- @return table with fields: 'what', 'source', 'currentline'.
--------------------------------------------------------------------------------
function private.GetInfo ()
  
  -- getinfo arguments
  local level = 3;    -- 0 = getinfo(),
                      -- 1 = private.GetInfo(),
                      -- 2 = __index(),
                      -- 3 = function, that contains the variable

  local what  = "Sl"; -- "S" select t += { source, short_src, linedefined, 
                      --                   lastlinedefined, what           }
                      -- "l" select t += { currentline }
                      -- t.what can be: 
                      --   "Lua"  -- a regular Lua function, 
                      --   "C"    -- it is a C function, 
                      --   "main" -- it is the main part of a Lua chunk

  -- NOTE: debug functions so slow
  local info = debug.getinfo( level, what );
  
  if not ( info ) then info = {}; end;
  
  local ret_info = {
    what = info.what        or "C",
    src  = info.source      or "Unknown source",
    line = info.currentline or -1
  };
  
  return ret_info;

end;

--------------------------------------------------------------------------------
-- Trace error with known callpoint (module:string).
-- @param src  source of error (module name).
-- @param line line in source.
-- @param msg  error message.
--------------------------------------------------------------------------------
function private.TraceError ( src, line, msg )
  
  DbgTrace( "[ STRICT ][ ERROR ] " .. src .. ":" .. line .. " " .. msg );
  
end;

--------------------------------------------------------------------------------