-- name=v_builds
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ BUILDS ] "..text );

end;
--------------------------------------------------------------------------------
function public.__InitModule ()

  loader.Module_Load( "int_dialog_map_build_construct" );

  mediator.Subscribe( "Model_Builds_NoFragments", private.Deficient_Show );

  private.fragments = {};
  private.visible   = {};

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
-- function ... Create and Set ... () end;
--------------------------------------------------------------------------------
function public.Create ( build_data )

  local build_name = build_data.name;
  local obj_name   = "obj_map_builds_"..build_name;
  
  ObjSet( obj_name, { anim_tag = build_name } );
  ObjClone( "btn_map_builds", "_"..build_name, 1, obj_name );
  
  int.Control_EventsSet( obj_name,
    { 
      func_on_click = function () private.Tap( build_name ); end
    }
  );

  if ( build_data.type == "puzzle" ) then
    
    ObjSet( "anm_map_builds_icon_state_"..build_name, 
    { 
      res = "assets/content/"..build_data.section.."/map/anm_"..build_data.icon.."_states",
      anim_tag = "states"
    } );

    m_object.Attach(
      "anm_map_builds_icon_state_"..build_name,
      "anm_map_builds_icon_"..build_name
    );

    if ( build_data.fragments ) then

      for p = 1, #build_data.fragments do

        local fragment_name = "anm_map_builds_"..build_name.."_part_"..p;
        if ( m_object.Exists( fragment_name ) ) then
          
          ObjSet( fragment_name, 
          { 
            res = "assets/content/"..build_data.section.."/map/anm_"..build_data.icon.."_states",
            anim_tag = "defeat"..p,
            animfunc = "defeat"
          } );
        
          int.Control_EventsSet( fragment_name,
            { 
              func_on_click = function ()
                private.Tap( build_name, fragment_name );
              end
            }
          );

        end;

      end;
      
      if ( ObjGet( "anm_map_builds_"..build_name.."_dummy" ) ) then
        
        ObjSet( "anm_map_builds_"..build_name.."_dummy", 
        { 
          res = "assets/content/"..build_data.section.."/map/anm_"..build_data.icon.."_states",
          animfunc = "dummy"
        } );
        
      end;

    end;

  elseif ( ( build_data.type == "island" ) and build_data.fragments ) then

    for i = 1, #build_data.fragments do

      local fragment_object = "obj_map_builds_" .. build_name .. 
                              "_fragment_" .. build_data.fragments[ i ];
      
      if ( m_object.Exists( fragment_object ) ) then
        ObjAttach( fragment_object, "anm_map_builds_icon_"..build_name );
      end;

    end;

  end;

  private.Set( build_data );

end;
--------------------------------------------------------------------------------
function private.Set ( build_data )

  local name    = build_data.name;
  local section = build_data.section;
  local icon    = build_data.icon;

  private.visible[ name ] = true;

  local obj_name = "obj_map_builds_"..name;
  local anm_name = "anm_map_builds_icon_"..name;
  
  local anm_res = tern( icon == "dummy" )(
    "assets/content/map/builds/anm_build_"..icon,
    "assets/content/"..section.."/map/anm_"..icon
  );
  
  local build_type  = build_data.type;
  local fragments   = build_data.fragments or {};
  local unlocked    = build_data.unlocked or false;
  local constructed = build_data.constructed or false;
  local collected   = build_data.collected_fragments or {};

  local set = {};
  local animfunc = tern( constructed )( "normal", "corrupted" );

  m_object.Show( obj_name, set );
  m_object.Set(
    anm_name, 
    {
      res      = anm_res,
      animfunc = animfunc,
      playing  = true
    },
    set
  );

  -- Inputs
  if ( build_type == "island" ) or
     ( build_type == "puzzle" ) then
   
    m_object.Set(
      obj_name,
      {
        input = not constructed
      },
      set
    );
    
  end;
  
  -- Fragments
  if ( #collected > 0 ) then

    private.fragments[ name ] = collected;

  end;

  if ( build_type == "island" ) then

    -- Single fragment island is a fork
    if ( ( #fragments == 1 ) ) then

      -- Show / hide fork sign
      if not ( constructed ) then

        local sign_sprite = "spr_map_build_island_item_"..name;
        if not ( m_object.Exists( sign_sprite ) ) then

          ObjClone(
            "spr_map_build_island_item",
            "_"..name,
            true,
            "obj_map_builds_indicator_"..name
          );

          local res = 
            common.GetItemResource( "fragment", fragments[1] ).."_small";
          
          table.insert( set, { "spr_map_build_island_item_icon_"..name, 
          {
            res     = res,
            color_r = 0.0,
            color_g = 0.0,
            color_b = 0.0
          } } );

        end;

      else

        m_object.Disable( "obj_map_builds_indicator_"..name, true, set );

      end;

    else

      for i = 1, #fragments do

        if ( cmn.IsEntityInList( collected, fragments[ i ] ) ) then

          m_object.Enable(
            obj_name.."_fragment_"..fragments[ i ],
            true,
            set
          );
          
        else

          m_object.Disable(
            obj_name.."_fragment_"..fragments[ i ],
            true,
            set
          );

        end;

      end;

    end;

  elseif ( build_type == "puzzle" ) then

    local states_name = "anm_map_builds_icon_state_"..name;
    local state = #collected;

    -- common.TapClear( "anm_map_builds_"..name.."_part_"..state );

    table.insert( set, { states_name,
    {
      active   = true,
      visible  = true,
      res      = "assets/content/"..section.."/map/anm_"..icon.."_states",
      animfunc = "state"..state,
      playing  = true
    } } );

  -- Build indicator
  elseif ( build_type == "build" ) then

    local obj_indicator = "obj_map_builds_indicator_"..name;
    local spr_indicator = "spr_map_build_indicator_"..name;

    if ( unlocked and not constructed ) then
      if ( not ObjGet( spr_indicator ) ) then

        ObjClone( "spr_map_build_indicator", "_"..name, 1, obj_indicator );
        table.insert( set, { obj_indicator, 
        {
          active  = true,
          visible = true
        } } );

      end;
    end;

  end;

  ObjMultiSet( set );

end;
--------------------------------------------------------------------------------
function private.BuildIndicator_Show ( build_name )

  local parent = "obj_map_builds_indicator_"..build_name;
  local sprite = "spr_map_build_indicator_"..build_name;
  
  if m_object.Exists( sprite ) then return; end;
  
  ObjClone( "spr_map_build_indicator", "_"..build_name, 1, parent );
  ObjSet( sprite, { active = true, visible = true } );

  ObjAnimate( sprite, "scale_xy", 0, 0, "", 
  { 
    0.0, 0, 0.0, 0.0,  
    0.3, 0, 1.0, 1.0
  } );

end;
--------------------------------------------------------------------------------
function private.BuildIndicator_Hide ( build_name )

  local sprite = "spr_map_build_indicator_"..build_name;
  
  if not m_object.Exists( sprite ) then return; end;

  ObjAnimate( sprite, "scale_xy", 0, 0, function () ObjDelete( sprite ); end, 
  { 
    0.0, 0, 1.0, 1.0,  
    0.3, 0, 0.0, 0.0
  } );

end;
--------------------------------------------------------------------------------
-- function ... Tap ... () end;
--------------------------------------------------------------------------------
function private.Tap ( build_name, build_part )
  
  local build_data = m_builds.GetBuildData( build_name );
  
  -- Play tap sound
  local sounds =
  {
    build  = "tap_build",
    island = "tap_build",
    puzzle = "tap_build"
  };
  
  cmn.PlaySnd( sounds[ build_data.type ] );
  
  -- Animate click
  private.Animate_Click( build_data );
  
  -- Try show construct dialog or popup
  if not ( build_data.constructed ) then
  
    if ( build_data.type == "build" and build_data.unlocked ) then
  
      private.Dialog_Build_Show( build_name, function ()
      
        local contructed_data = m_builds.GetBuildData( build_name );
      
        private.BuildIndicator_Hide( build_name );
        
        local animation = private.GetAnimData_Construct( contructed_data );
        local func_anim = v_map.Callback_Animate( animation );
        
        v_map.Builds_UpdateStages( contructed_data, func_anim );
        
      end );
    
    else

      local func_after;
    
      if ( build_data.type == "puzzle" ) then

        func_after = function ()

          private.Puzzle_HighlightStage( build_data.fragments[ 1 ] );

        end;

      end;

      private.Popup_Tap( build_data, build_part, func_after );
    
    end;
    
  else
  
    private.Focus_Build( build_name );
  
  end;
  
end;
--------------------------------------------------------------------------------
-- function ... Animations ... () end;
--------------------------------------------------------------------------------
function private.Animate_Click ( build_data, state )

  state = state or tern( build_data.constructed )( "normal", "corrupted" );
  
  local anm_build = "anm_map_builds_icon_"..build_data.name;

  if ( ObjGet( anm_build ).animfunc ~= "click_"..state ) then

    AnimPlay( anm_build, "click_"..state, function () 
      AnimPlay( anm_build, state, "" );
    end );

  end;

end;
--------------------------------------------------------------------------------
function private.Animate_Construct ( build_data )

  local build_name = build_data.name;
  local build_type = build_data.type;

  local obj_name = "obj_map_builds_"..build_name;
  local anm_name = "anm_map_builds_icon_"..build_name;

  AnimStop( anm_name );

  if ( build_type == "puzzle" ) or
     ( build_type == "island" ) then

    int.Input_Off( obj_name );

  else

    int.Input_On( obj_name );

  end;

  AnimPlay( anm_name, "normal", "" );

end;
--------------------------------------------------------------------------------
function private.GetAnimData_Obtain ( build_data, fragment )

  local anim = {};
  
  anim.name   = "obj_map_builds_"..build_data.name;
  anim.coords = { pos_x = 0, pos_y = 0 };
  
  local anm_name = "anm_map_builds_icon_"..build_data.name;
  
  if     ( build_data.type == "build" ) then
  
    anim.func_action = function ( func_after )
    
      AnimPlay( anm_name, "click_corrupted", function ()
      
        private.Set( build_data );
        AnimPlay( anm_name, "corrupted", "" );
        if ( func_after ) then func_after(); end;
        
      end );
    
    end;
  
  elseif ( build_data.type == "island" ) then
  
    anim.func_action = function ( func_after )
    
      AnimPlay( anm_name, "action", function ()
      
        private.Set( build_data );
        AnimPlay( anm_name, "corrupted", "" );
        
      end );
    
      -- Fadeout item indicator if exists
      local obj_indicator = "obj_map_builds_indicator_"..build_data.name;
      local spr_item_icon = "spr_map_build_island_item_icon_"..build_data.name;
      local obj_item = "obj_map_builds_"..build_data.name.."_item";
        
      if ( ObjGet( obj_item ) ) then

        ObjSet( spr_item_icon,
        {
          color_r = 1.0, 
          color_g = 1.0,
          color_b = 1.0
        } );

        local function func_fade_indicator ()

          ObjAnimate( obj_indicator, "alp", 0, 0, "",
          {
            0.0, 0, 1.0,
            0.3, 0, 0.0
          } )

        end;

        common.Animation_Bounce( obj_indicator, func_fade_indicator );

      end;
        
      -- Hide sign of fork or animate fragment on place
      if ( ObjGet( spr_item_icon) ) then
          
        ObjAnimate( obj_indicator, "alp", 0, 0,
        function ()
          ObjSet( obj_indicator, { active  = false, visible = false } );
        end,
        {
          0.0, 3, 1.0,
          0.3, 3, 0.0
        } );
          
      else
      
        local obj_fragment = anim.name.."_fragment_"..fragment.name;

        ObjSet( obj_fragment,
        {
          active  = true,
          visible = true,
          alp     = 0.0
        } );

        ObjAnimate( obj_fragment, "alp", 0, 0, "",
        {
          0.0, 3, 0.0,
          0.3, 3, 1.0
        } );
      
      end;
      
      if ( func_after ) then func_after(); end;
      
    end;
  
  elseif ( build_data.type == "puzzle" ) then
  
    anim.func_action = function ( func_after )
    
      private.Set( build_data );
      AnimPlay( anm_name, "action", function ()
      
        AnimPlay( anm_name, "corrupted", "" );
        
      end );
      
      local pat_part  = "anm_map_builds_%s_part_%s";
      local pat_dummy = "anm_map_builds_%s_dummy";
      
      local count = #build_data.collected_fragments;
      
      local anm_part  = pat_part:format( build_data.name, count );
      local anm_dummy = pat_dummy:format( build_data.name );

      if ( ObjGet( anm_part ) ) then

        AnimPlay( anm_dummy, "dummy", "" );
        AnimPlay( anm_part, "defeat", "" );

      end;
      
      if ( func_after ) then func_after(); end;
    
    end;
  
  else
  
    local msg = "Error! Can't animate obtain for build %q of type %q!"
    DbgTrace( msg:format( build_data.name, build_data.type ) );
  
  end;
  
  return anim;

end;
--------------------------------------------------------------------------------
function private.GetAnimData_Construct ( build_data, func_ShowPortal )

  local anim = {};
  anim.name  = "obj_map_builds_"..build_data.name;
  
  local anm_name = "anm_map_builds_icon_"..build_data.name;
  
  if ( m_builds.IsAutoConstructed( build_data.name ) ) then
  
    anim.skip_focus  = true;
    anim.func_action = function ( func_after )
    
      local anm_name = "anm_map_builds_icon_"..build_data.name;
      
      AnimPlay(
        anm_name,
        "appear",
        function ()
      
          AnimPlay( anm_name, "normal", "" );
          
          if ( func_after ) then func_after(); end;
      
        end
      );
    
    end;
    
  else
  
    anim.res    = "assets/content/map/effects/anm_appearance_build";
    anim.coords = { pos_x = 0, pos_y = 0, scale_x = 0.5, scale_y = 0.5 };
    
    anim.func_preset = function ()
    
      private.Animate_Click( build_data, "corrupted" );
    
    end;
    
    anim.func_dummy = function ()
    
      if ( func_ShowPortal ) then
      
        func_ShowPortal();
      
      end;
    
      cmn.PlaySnd( "unspell" );
      
      private.Set( build_data );
      private.Animate_Construct( build_data );
    
    end;
    
    anim.func_action = function ( func_after )
  
      if ( func_after ) then func_after(); end;
    
    end;
  
  end;
  
  return anim;

end;
--------------------------------------------------------------------------------
-- function ... Popups ... () end;
--------------------------------------------------------------------------------
function private.Popup_Tap ( build_data, build_part, func_after )

  local target, mode, data;
  local functions = { func_preclose = func_after };
  
  local obj_ind = build_part or "obj_map_builds_indicator_"..build_data.name;
  local obj_bld = "obj_map_builds_"..build_data.name;
  
  target = tern( m_object.Exists( obj_ind ) )( obj_ind, obj_bld );
  
  local fragments = #( build_data.fragments or {} );
  local collected = #( build_data.collected_fragments or {} );
  
  local collect_done = ( collected == fragments ) and ( fragments > 0 );
  
  local text = "str_int_popup_cloud_builds_"..build_data.name;
  
  if ( fragments > 0 ) then
  
    data =
    {
      count   = collected,
      content = m_content.GetContent( "fragment" ),
      target  =
      {
        count = fragments,
        name  = build_data.fragments[ 1 ]:gsub( "_%d+$", "" )
      }
    };
  
  end;
  
  if ( build_data.type == "puzzle" ) then
  
    if ( collect_done and not build_data.constructed ) then
    
      mode = "monolog";
      text = "str_int_popup_cloud_builds_stagelock_"..build_data.name;
    
    else
    
      mode = "complex"
    
    end;
  
  elseif ( build_data.type == "island" ) then
  
    mode = "monolog";
  
  elseif ( build_data.type == "build"  ) then
  
    if ( fragments > 0 ) then
    
      mode = "complex"
    
    else
    
      mode = "monolog";
      
    end;
    
  else
  
    local msg = "Error! Can't show tap popup for build %q of type %q!"
    DbgTrace( msg:format( build_data.name, build_data.type ) );
    
    if ( func_after ) then func_after(); end;
    return;
      
  end;
  
  v_map.ShowObjectPopup( mode, target, text, data, nil, functions );

end;
--------------------------------------------------------------------------------
function private.Popup_ItemReceive ( build_data, fragment, func_after )

  local target, mode, text, data;
  local functions = { func_preclose = func_after };
  
  local obj_ind = "obj_map_builds_indicator_"..build_data.name;
  local obj_bld = "obj_map_builds_"..build_data.name;
  
  target = tern( m_object.Exists( obj_ind ) )( obj_ind, obj_bld );

  if ( build_data.type == "puzzle" ) then
  
    mode = "counter";
    text = "";
    
    data         = {};
    data.count   = #( build_data.collected_fragments or {} );
    data.content = m_content.GetContent( "fragment" );
    data.target  =
    {
      count = #build_data.fragments,
      name  =  build_data.fragments[ 1 ]:gsub( "_%d+$", "" )
    };
    
    if ( data.count == data.target.count ) then
    
      mode = "monolog";
      text = "str_int_popup_cloud_builds_stagelock_"..build_data.name;
    
    end;
    
    local p = "anm_map_builds_%s_part_%s";
    local obj_part = p:format( build_data.name, fragment.name:match( "%d+$" ) );
    
    if ( ObjGet( obj_part ) ) then target = obj_part; end;
    
  elseif ( build_data.type == "island" ) then
  
    mode = "monolog";
    text = "str_int_popup_cloud_builds_"..build_data.name;
  
  else
  
    local msg = "Error! Can't show receive popup for build %q of type %q!"
    DbgTrace( msg:format( build_data.name, build_data.type ) );
    
    if ( func_after ) then func_after(); end;
    return;
  
  end;
  
  v_map.ShowObjectPopup( mode, target, text, data, nil, functions );

end;
--------------------------------------------------------------------------------
-- function ... Reactions ... () end;
--------------------------------------------------------------------------------
--[[
  Build reaction format:
  {
    -- Basic data:
    name       = "build_name",
    func_popup = function ( func_after ) end,
    func_anim  = function ( func_after ) end,
    
    -- Only for reward drop:
    object          = "obj_build_name", -- receiver-object
    func_drop       = function () end,  -- on-drop function
    call_drop_first = true, -- true : func_drop on focus, func_anim on drop
                            -- false: func_anim on focus, func_drop on drop
    
    -- Rewards:
    skinpart = "skn_part_name",
    
    -- Other:
    show_stages = { min = 1, max = 3, type = "bonus" },
    move_marker = true
  }
]]
--------------------------------------------------------------------------------
function public.GetRewardReaction ( rewards, func_ShowPortal )

  local reaction = nil;
  
  if not ( rewards ) then return reaction; end;
  
  -- Find fragment
  local fragment = nil;
  for i, reward in pairs( rewards ) do
  
    if ( reward.type == "fragment" ) then
    
      fragment = reward;
      break;
    
    end;
  
  end;
  
  if not ( fragment ) then return reaction; end;
  
  -- Try setup reaction
  local build_name = m_builds.GetBuildNameByFragment( fragment.name );

  if ( build_name ) then
  
    local build_data  = m_builds.GetBuildData( build_name );
    local constructed = build_data.constructed or false;
  
    reaction        = {};
    reaction.name   = build_name;
    reaction.object = "obj_map_builds_"..build_name;
    
    reaction.call_drop_first = true;
    reaction.func_drop       = nil;
    
    if ( constructed ) then
    
      local animation = private.GetAnimData_Construct(
        build_data,
        func_ShowPortal
      );
      
      reaction.func_anim = v_map.Callback_Animate( animation );
      
      if ( build_data.reward                    ) and
         ( build_data.reward.type == "skinpart" ) then
      
        reaction.skinpart = build_data.reward.name;
      
      end;
      
      reaction.show_stages = build_data.unlocklist;
      reaction.move_marker = build_data.last or 
                             ( build_data.unlocklist.type == "common" );
      
    else
      
      if ( build_data.type ~= "build" ) then
    
        reaction.func_popup = function ( func_after )
          private.Popup_ItemReceive( build_data, fragment, func_after );
        end;
      
      end;
      
      local animation    = private.GetAnimData_Obtain( build_data, fragment );
      reaction.func_anim = v_map.Callback_Animate( animation );
    
    end;
  
  end;
  
  return reaction;

end;
--------------------------------------------------------------------------------
function public.GetStageCompleteReaction ( stage )

  local reaction = nil;
  
  local build_data = m_builds.GetBuildUnlockedByStage( stage );
  if ( build_data ) then
  
    if ( build_data.constructed ) then
    
      reaction           = {};
      reaction.name      = build_data.name;
      
      local animation    = private.GetAnimData_Construct( build_data );
      reaction.func_anim = v_map.Callback_Animate( animation );
      
      if ( build_data.reward and build_data.reward.type == "skinpart" ) then
      
        reaction.skinpart = build_data.reward.name;
      
      end;
      
      reaction.show_stages = build_data.unlocklist;
    
    elseif ( build_data.type == "build" and build_data.unlocked ) then
    
      reaction           = {};
      reaction.name      = build_data.name;
      reaction.func_anim = function ( func_after )
          
        private.BuildIndicator_Show( build_data.name );
        if ( func_after ) then func_after(); end;
      
      end;
    
    end;
  
  end;
  
  return reaction;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Dialog_Build_Show ( build_name, func_construct )

  local func_start = function ()

    local build_data = m_builds.GetBuildData( build_name );
    local deficient  = private.Deficient_Detect( build_data.price );
     
    ----------------------------------------------------------------------------
    local func_accept = function ()
    
      if ( m_builds.TryConstructBuild( build_name ) ) then
      
        private.Dialog_Build_Hide();
        func_construct();
        
      end;
      
    end;

    local func_reject = private.Dialog_Build_Hide;

    ----------------------------------------------------------------------------
    local func_find = function ( fragment )

      local stage_data = m_stage.Usable_Search( fragment, build_data.section );
      if  ( stage_data ) then
      
        private.Focus_Stage( stage_data.name );
        return;
        
      end;

      local character = m_characters.GetCharacterByFragment( fragment );
      if  ( character ) then
        
        private.Focus_Character( character.name );
        return;

      end;

    end;

    local build_content     = build_data.section;
    local construct_price   = build_data.price;
    local construct_rewards = build_data.reward;

    local construct_fragments = nil;

    if ( build_data.fragments and ( #build_data.fragments > 0 ) ) then
    
      local collected = build_data.collected_fragments or {};

      construct_fragments = {};

      for f = 1, #build_data.fragments do

        local fragment  = build_data.fragments[ f ];
        
        table.insert( construct_fragments, {
          name      = fragment,
          collected = cmn.IsEntityInList( collected, fragment );
        } );

      end;

    end;
    
    ----------------------------------------------------------------------------
    local func_top_panel_show = function () v_map.TopPanel_Show(); end;
    local func_top_panel_hide = function () v_map.TopPanel_Hide(); end;
    local func_top_panel_lock = function () v_map.TopPanel_Lock(); end;
    
    ----------------------------------------------------------------------------
    local function func_resolve_deficient ()
      
      return private.Deficient_TryResolve( deficient, build_name );
      
    end;

    ----------------------------------------------------------------------------
    int_dialog_map_build_construct.Show(
      build_name,
      build_content,
      construct_price,
      deficient,
      construct_fragments,
      construct_rewards,
      func_accept,
      func_reject,
      func_find,
      func_resolve_deficient,
      func_top_panel_show,
      func_top_panel_hide,
      func_top_panel_lock
    );

  end;

  int_window.AddWindowToQueue( "int_dialog_map_build_construct", func_start );

end;
--------------------------------------------------------------------------------
function private.Dialog_Build_Hide ()

  --local window = "int_dialog_map_build_construct"
  --local func_hide = function () int_window.Hide( window ); end

  int_dialog_map_build_construct.Hide( --[[func_hide]] );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Puzzle_HighlightStage ( fragment_name )

  int.Lock( true );

  local pattern = fragment_name:gsub( "_part_(%d+)", "" );

  local stage_list = v_stage.List_HasReward(
    "fragment",
    pattern
  );

  if ( #stage_list < 1 ) then

    int.Lock( false );
    return;

  end;

  local random     = math.random( 1, #stage_list );
  local stage_name = stage_list[ random ];
  local obj_name   = "obj_map_point_"..stage_name;

  local function func_after ()

    common.Animation_Spin(
      obj_name,
      function ()

        int.Lock( false );

      end
    );

  end;

  v_map.Focus(
    obj_name,
    {
      func_after = func_after,
      scale      = v_map.Scale_GetMax()
    }
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Focus_Build ( build_name, func_after )

  v_map.Focus( "obj_map_builds_"..build_name, { func_after = func_after } );

end;
--------------------------------------------------------------------------------
function private.Focus_Stage ( target_name )

  local func_get_blocker_name = function ( stage_name )

    local stage_data    = m_stage.GetStaticData( stage_name );
    local stage_type    = stage_data.realtype;
    local stage_section = stage_data.section;
    local stage_num     = tonumber(stage_name:match( "%d+" ));

    local query = 
    {
      query      = "find_all",
      group      = "static",
      collection = "builds",
      indice     = "section",
      criteria   = { like = stage_section }
    };
    
    local build_data = m_storage.Execute( query ).result;

    for b = 1, #build_data do

      local list = build_data[ b ].unlocklist;

      if ( list and ( ( list.type == stage_type ) or 
                      ( list.type == "common"   ) ) ) then

        if ( stage_num >= list.min ) and ( stage_num <= list.max ) then

          return build_data[ b ].name;

        end;

      end;

    end;

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  local func_is_blocker_constructed = function ( build_name )

    local query = 
    {
      query      = "find",
      group      = "player",
      collection = "builds",
      indice     = "name",
      criteria   = { like = build_name },
      result     = "constructed"
    };
    
    return m_storage.Execute( query ).result;    

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  local func_focus_stage = function ( stage_name ) 

    local obj_name = "obj_map_point_"..stage_name;
    
    v_map.Focus( obj_name, { func_after = function ()
    
      common.Animation_Spin( obj_name, function()

        local focus_name = string.format( "fcs_map_point_%s", stage_name );

        ObjAnimate( focus_name, "alp", 0, 0, "", 
        {
          0.0, 2, 0,
          0.3, 2, 1,
          0.6, 2, 0,
          0.9, 2, 1,
          1.2, 2, 0
        } );

      end );
        
    end } );

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
  local func_focus_build = function ( name )
  
    private.Focus_Build( name, function () private.Tap( name ); end );

  end;
  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  int_window.HideAll( function ()

    local build_name = func_get_blocker_name( target_name );

    if ( build_name and not func_is_blocker_constructed( build_name ) ) then

      func_focus_build( build_name );

    else

      func_focus_stage( target_name );

    end;

  end );

end;
--------------------------------------------------------------------------------
function private.Focus_Character ( character )
  
  local obj_name  = "obj_map_characters_"..character;
  
  int_window.HideAll( function ()

    v_map.Focus( obj_name, 
      { 
        func_after = function () v_characters.Tap( character ); end
      }
    );

  end );  
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Deficient_Show ( params )

  local build_name = params.name;
  local deficient  = params.deficient;

  if ( #deficient == 0 ) then

    deficient = { deficient };

  end;
  
  local first = deficient[ 1 ];

  local func_solution  = function () end;
  local solution       = "stage";
  local stage_data     = m_stage.Usable_Search( first.name, first.section );
  local character_data = m_characters.GetCharacterByFragment( first.name );

  if ( stage_data ) then

    func_solution = function ()

      private.Focus_Stage( stage_data.name );

    end;
    
  elseif ( character_data ) then
    
    func_solution = function ()

      private.Focus_Character( character_data.name );

    end;

  end;
  
  local func_accept = function ()
  
    private.Deficient_Hide( func_solution );
  
  end;

  local func_reject = function ()

    private.Deficient_Hide();
  
  end;

  local func_skin_attach = function ( destination )

    return v_hero.Skin_Attach( destination );

  end;

  local func_skin_detach = function ( anm_name )

    v_hero.Skin_Detach( anm_name );

  end;

  int_window.Show( "int_dialog_deficient", function()
 
    int_dialog_deficient.Show(
      deficient,
      func_accept,
      func_reject,
      func_skin_attach,
      func_skin_detach,
      solution
    );
 
  end );

end;
--------------------------------------------------------------------------------
function private.Deficient_Hide ( func_end )
  
  int_dialog_deficient.Hide( function()
  
    int_window.Hide( "int_dialog_deficient", func_end );

  end );
  
end;
--------------------------------------------------------------------------------
function private.Deficient_Detect ( build_price )
  
  local deficient =
  {
    list  = {},
    delta = {},
    gold  = 0
  };
  
  local market_prices   = {};
  local market_settings = m_market.Settings( "buy" );
  
  for l, lot in pairs( market_settings ) do
    
    market_prices[ lot.name ] = { value = lot.value, price = lot.price };
    
  end;
  
  for p, price in pairs( build_price ) do
    
    local available_count = m_player.GetResourceCount( price.resource );
    
    if ( available_count < price.count ) then
      
      local delta = price.count - available_count;
      
      table.insert( deficient.list,  price.resource );
      table.insert( deficient.delta, delta );
      
      deficient.gold = deficient.gold + delta / market_prices[ price.resource ].value * market_prices[ price.resource ].price;
      
    end;
    
  end;
  
  deficient.gold = math.ceil( deficient.gold );
   
  return tern( #deficient.list < 1 )( false, deficient );
  
end;
--------------------------------------------------------------------------------
function private.Deficient_TryResolve ( deficient, build_name )
  
  if ( m_player.TryChangeGoldCount( - deficient.gold ) ) then
    
    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "build",
      value       = - deficient.gold,
      name        = build_name
    } );    
    
    for d, delta in pairs( deficient.delta ) do
      
      m_player.TryChangeResourceCount( deficient.list[ d ], delta );
      
      common.Kontagent_LogEvent( "economy", 
      {
        description = "soft_currency_collect",
        category    = "build",
        value       = delta,
        name        = build_name,
        type        = deficient.list[ d ]
      } );
      
    end;
    
    return true;
    
  end;
  
  return false;
  
end;
--------------------------------------------------------------------------------