-- name=v_chests
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ CHESTS ] "..text );

end;
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.Trace( "Initialized." );

  private.chests_available = {};
  
  private.app_w = int.W();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Create ( chest_name, section_name )

  local obj_name = "obj_map_chests_"..chest_name;
  local btn_name = "btn_map_chests_"..chest_name;
  
  ObjCreate( obj_name, "obj" );
  ObjSet(    obj_name, { anim_tag = chest_name } );
  ObjAttach( obj_name, "anm_map_"..section_name );

  ObjClone( "btn_map_chests", "_"..chest_name, 1, obj_name );
  
  int.Control_EventsSet( btn_name,
    {
      func_on_click = function ()
      
        int.Input_Off( btn_name );

        mediator.Notify( "View_Chests_ChestCollected", chest_name );

        local gift = m_chests.Content( chest_name );
        m_items.TryObtain( gift );
        
        m_chests.Open( chest_name );

        local func_after_hide = function ()

          private.Dialog_Show( chest_name, gift,
            function ()
            
              int.Lock( false );
              
            end,
            function ()
            
              v_items.Reward_Show( gift,
                function ()
            
                  AnimPlay(
                    "anm_map_chests_paw_"..chest_name,
                    "dynamic",
                    function ()
                      private.Set_Opened( chest_name );
                    end
                  );
            
                end
              );
              
            end
          );

        end;
        
        int.Lock( true );
        cmn.PlaySnd( "button_click" );
        private.Chest_Hide( chest_name, func_after_hide );

      end;
    }
  );
  
  private.Set( chest_name );

end;
--------------------------------------------------------------------------------
function private.Set ( chest_name )

  local state = m_chests.Status( chest_name );
  
  if     ( state == "available" ) then private.Set_Active( chest_name );
  elseif ( state == "opened"    ) then private.Set_Opened( chest_name );
  else                                 private.Set_Hidden( chest_name );
  end;

end;
--------------------------------------------------------------------------------
function private.Set_Active ( chest_name )

  local btn_gift = "btn_map_chests_"..chest_name;
  local anm_gift = "anm_map_chests_gift_"..chest_name;
  local anm_paw  = "anm_map_chests_paw_"..chest_name;
  
  local set = {};
  
  m_object.Set( btn_gift, { input = true }, set );
  
  m_object.Set( anm_gift,
    {
      visible  = true,
      animfunc = "static",
      playing  = true
    }, 
    set
  );
  
  m_object.Set( anm_paw,
    { 
      visible  = true,
      animfunc = "static",
      playing  = true
    },
    set
  );

  m_object.Multiset( set );

end;
--------------------------------------------------------------------------------
function private.Set_Opened ( chest_name )

  local btn_gift = "btn_map_chests_"..chest_name;
  local anm_gift = "anm_map_chests_gift_"..chest_name;
  local anm_paw  = "anm_map_chests_paw_"..chest_name;
  
  local set = {};
  
  m_object.Set( btn_gift, { input   = false                  }, set );
  m_object.Set( anm_gift, { visible = false, playing = false }, set );
  m_object.Set( anm_paw,  { visible = false, playing = false }, set );

  m_object.Multiset( set );

end;
--------------------------------------------------------------------------------
function private.Set_Hidden ( chest_name )

  local btn_gift = "btn_map_chests_"..chest_name;
  local anm_gift = "anm_map_chests_gift_"..chest_name;
  local anm_paw  = "anm_map_chests_paw_"..chest_name;
  
  local set = {};
  
  m_object.Set( btn_gift, { input   = false                  }, set );
  m_object.Set( anm_gift, { visible = false, playing = false }, set );
  m_object.Set( anm_paw,
    {
      visible  = true,
      playing  = true,
      animfunc = "static"
    },
    set
  );

  m_object.Multiset( set );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Chest_Show ( chest_name, func_after )

  local anm_gift = "anm_map_chests_gift_"..chest_name;

  AnimPlay( anm_gift, "appear", function ()
  
    AnimPlay( anm_gift, "static", "" );
    if ( func_after ) then func_after(); end;
  
  end );

end;
--------------------------------------------------------------------------------
function private.Chest_Hide ( chest_name, func_after )

  cmn.PlaySnd( "click_present" );
  AnimPlay( "anm_map_chests_gift_"..chest_name, "get", func_after or "" );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Dialog_Show ( chest_name, gift, func_open, func_hide )

  local func_close = function ()

    int_dialog_map_chest.Hide( func_hide, function ()

      int_window.Hide( "int_dialog_map_chest" );

    end );

  end;

  local func_start = function ()

    int_dialog_map_chest.Show( chest_name, gift, func_open, func_close );

  end;

  int_window.AddWindowToQueue( "int_dialog_map_chest", func_start );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Fox_Show ( chest_name, func_after )

  local obj_chest = "obj_map_chests_"..chest_name;

  cmn.PlaySnd( "flying_fox" );

  private.ZoomFocus( obj_chest, "out", 1.0, { x = 0, y = 200 } );

  ObjSet( "obj_map_foreground_fox", { visible = true, active = true } );
  ObjAttach( "obj_map_foreground", InterfaceName );

  local func_end = function()

    ObjSet( "obj_map_foreground_fox", { visible = false, active = false } );
    ObjAttach( "obj_map_foreground", "obj_map_body" );

  end;

  AnimPlay( "anm_map_foreground_fox_actor", "action", func_end );
  AnimPlay( "anm_map_foreground_fox_dummy", "dummy",  function()

    private.Fox_Sparks( chest_name, func_after );

  end );
  
  common.Defer( "foreground_fox", 1.3, function () 
  
    private.ZoomFocus( obj_chest, "in", 2.8, { x = 0, y = 200 } );
  
  end );

end;
--------------------------------------------------------------------------------
function private.Fox_Sparks ( chest_name, func_after )

  AnimPlay( "anm_map_foreground_fox_sparks", "sparks", "" );

  local chest_pos = int.Position( "btn_map_chests_"..chest_name );

  ObjAnimate( "anm_map_foreground_fox_sparks", "pos_xy", 0, 0, function()

    private.Fox_Blast( chest_name, chest_pos );
    private.ZoomFocus( "obj_map_chests_"..chest_name, "in", 1.0 );
    
    private.Set_Active( chest_name );
    private.Chest_Show( chest_name, func_after );

  end,
  {
    0, 0, 0.5 * private.app_w - 50, 530,
    1, 0, chest_pos[ 1 ], chest_pos[ 2 ]
  } );

end;
--------------------------------------------------------------------------------
function private.Fox_Blast ( chest_name, pos )

  local anm_blast  = "anm_map_foreground_fox_blast";
  local anm_sparks = "anm_map_foreground_fox_sparks";

  m_object.Set( anm_blast,  { pos_x = pos[ 1 ], pos_y = pos[ 2 ]            } );
  m_object.Set( anm_sparks, { pos_x = 0.5 * private.app_w - 50, pos_y = 530 } );

  AnimPlay( anm_blast, "blast", "" );

end;
--------------------------------------------------------------------------------
function private.ZoomFocus( object, zoom, time, offset )

  time   = time or 0.6;

  if ( zoom == "in" ) then

    v_map.Focus( object, { scale = 1.0, time = time, offset = offset } );

  elseif ( zoom == "out" ) then

    v_map.Focus( object, { scale = 0.5, time = time, offset = offset } );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Callback_AnimatePlace ( stage )

  local chest_name = m_chests.GetChestByStage( stage );
  if  ( chest_name ) then
  
    return function ( func_after )
    
      int.Lock( true );
      private.Fox_Show( chest_name, function ()
      
        int.Lock( false );
        if ( func_after ) then func_after(); end;
      
      end );
    
    end;
  
  end;
  
  return nil;

end;
--------------------------------------------------------------------------------