-- name=v_client
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ CLIENT ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.app_url =
  {
    ios    = "https://itunes.apple.com/app/id905121703",
    google = "https://play.google.com/store/apps/details?id=com.bigfishgames.sirmatchgoogle",
    amazon = "http://www.amazon.com/gp/mas/dl/android?p=com.bigfishgames.sirmatchamazon"
  };

  private.strings = {
    connection_lost     = ne.strings.Get( "str_ingame_notification_connection_lost" ),
    connection_restored = ne.strings.Get( "str_ingame_notification_connection_restored" )
  };
  
  mediator.Subscribe( "Model_Client_WaitingStarted",  int.WaitingShow );
  mediator.Subscribe( "Model_Client_WaitingFinished", int.WaitingHide );

  private.Connection_Init();
  private.SystemMessages_Init();
  
  private.is_online = true;

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.TryDownloadContent ( section_name )

  if ( section_name ) then
    
    m_client.Section_SetAllowed( section_name );

  end;

  if ( not m_client.IsContentObsolete( section_name ) ) then
    
    return false;

  else

    if ( IsMobile() and not m_client.IsOnline() ) then

      m_client.NotifyOffline();

      return true;

    end;

  end;

  private.ContentObsolete( true );

  return true;

end;
--------------------------------------------------------------------------------
function public.SystemDialog_Show (
    dialog_type,
    func_btn1,
    func_btn2,
    text_params_btn1,
    text_params_btn2
  )

  local func_hide = function ()

    private.is_on_screen_system_msg = false;

  end;
  
  local is_ingame = ( private.sysmsg_type == "ingame" ) or 
                    ( dialog_type == "conflictstorage" );

  local buttons = {};
  local postfix = tern( is_ingame )( "_pc", "" ); -- FIXME: _ingame

  if ( func_btn1 ) then

    text_params_btn1 = text_params_btn1 or {};
    
    table.insert( buttons, {
      title = ne.strings.Get(
        "str_systemmessage_"..dialog_type.."_button1"..postfix,
        text_params_btn1[ 1 ],
        text_params_btn1[ 2 ]
      ),
      action = function()

        func_hide();
        func_btn1();

      end,
      params = text_params_btn1
    } );

  end;

  if ( func_btn2 ) then
    
    text_params_btn2 = text_params_btn2 or {};

    table.insert( buttons, {

      title = ne.strings.Get(
        "str_systemmessage_"..dialog_type.."_button2"..postfix,
        text_params_btn2[ 1 ],
        text_params_btn2[ 2 ]
      ),
      action = function()

        func_hide();
        func_btn2();

      end,
      params = text_params_btn2
    } );

  end;

  private.is_on_screen_system_msg = true;
  
  local str    = "str_systemmessage_"..dialog_type;

  local user_id = m_player.GetUserId();
  local user_id_message = "";
  if ( user_id ~= "0" ) then
    user_id_message = ne.strings.Get( "str_loader_userid", user_id );
  end;
  
  local params =
  {
    title   = ne.strings.Get( str.."_title" ),
    body    = ne.strings.Get( str.."_body", user_id_message ),
    buttons = buttons 
  };
  
  if ( is_ingame ) then
  
    int_dialog_info.ShowLikeSysMsg( params );
    
  else
  
    wiz.ShowSystemMessage( params );
  
  end;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Connection_Init ()

  ObjCreate( "tmr_client_connection", "timer" );
  ObjAttach( "tmr_client_connection", InterfaceName );
  
  mediator.Subscribe(
    "Model_Client_ShowOfflineMode",
    private.Connection_Show
  );
  
  mediator.Subscribe(
    "Model_Client_ConnectionLost",
    private.Connection_Lost
  );

end;
--------------------------------------------------------------------------------
function private.Connection_Show ( params )
  
  if ( not params.message ) then
    
    return;
    
  end;
  
  local func_connectionlost, func_reconnect;
  
  func_reconnect = function ()

    -- У Визарда сообщние о том, что окошко закрылось.
    -- mediator.Notify( "View_Client_ConnectionLostClosed" );
    -- Потребуется для фич, диалоги в которых должны открываться
    -- после нотификации об оффлайне (или в онлайне)

  end;
  
  func_connectionlost = function ()

    public.SystemDialog_Show( "connectionlost", func_reconnect );
   
  end; 
  
  func_connectionlost();

end;
--------------------------------------------------------------------------------
function private.Connection_Lost ( params )
 
  m_client.StopWaiting( "client", "connection_lost" );
  
  if ( params.message ) then

    private.Connection_Show( params );

  end;

  local is_online = m_client.IsOnline();
  
  -- JK: if connection state changed
  if ( private.is_online ~= is_online ) then
    
    private.is_online = is_online;
    
    if ( not private.is_online ) then
    
      int_panel_map_top.ShowNotification(
        private.strings.connection_lost,
        "offline",
        "fail"
      );
      
      mediator.SubscribeOnce(
        "Model_Client_ConnectionRestored",
        private.Connection_Restored
      );
    
    end;
    
  end;

end;
--------------------------------------------------------------------------------
function private.Connection_Restored ()
 
  private.is_online = true;
  
  int_panel_map_top.ShowNotification(
    private.strings.connection_restored,
    "online",
    "success"
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
-------------------------------------------------------------------------------- 
function private.SystemMessages_Init ()

  private.sysmsg_type = loader.GetSystemMessageType();

  mediator.Subscribe(
    "Model_Client_ContentObsolete",
    private.ContentObsolete
  );
  
  mediator.Subscribe(
    "Model_Client_StorageConflict",
    function ( params )
     
      private.StorageConflict( params.clientSaveTime, params.serverSaveTime );
      
    end
  );
  
  mediator.Subscribe(
    "Model_Client_PrioritySessionConflict",
    private.PrioritySessionConflict
  );
  
  mediator.Subscribe(
    "Model_Client_ApplicationGainFocus",
     private.ApplicationGainFocus
  );
  
  mediator.Subscribe(
    "Model_Client_ClientVersionOutdated",
    private.ClientVersionOutdated
  );
  
  mediator.Subscribe( 
    "Event_Social_RaveLoginConflict", 
    private.RaveLoginConflict
  );
  
end;
--------------------------------------------------------------------------------
function public.ContentObsolete ( force_message )
  private.ContentObsolete( force_message );
end;
--------------------------------------------------------------------------------
function private.ContentObsolete ( force_message )

  -- Show "obsolete" message only on MAP.
  if ( not force_message and ( int_window.IsAnyWindowOnScreen() or
       screen.GetCurrentName() == "scr_stage" ) ) then

    return;

  end;

  local func_show_contentobsolete = function ()

    common.Return();

  end;

  public.SystemDialog_Show( "contentobsolete", func_show_contentobsolete );  

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.StorageConflict ( clientTime, serverTime )

  local func_message_main, func_message_local, func_message_server;

  func_message_main = function ()
    
    local clientTimeFormatted = os.date( "%Y-%m-%d", clientTime );
    local serverTimeFormatted = os.date( "%Y-%m-%d", serverTime );
    
    if clientTimeFormatted == serverTimeFormatted then
      clientTimeFormatted = os.date( "%H:%M:%S", clientTime );
      serverTimeFormatted = os.date( "%H:%M:%S", serverTime );
    end;

    local clientInfoData = private.StorageConflict_GetClientInfo();
    local serverInfoData = private.StorageConflict_GetServerInfo();

    public.SystemDialog_Show( "conflictstorage", 
      func_message_local, 
      func_message_server,
      { clientTimeFormatted, clientInfoData },
      { serverTimeFormatted, serverInfoData }
    );
  
  end;

  func_message_local = function ()

    public.SystemDialog_Show(
      "conflictstorage_local",
      function ()
        parts.storage.ResolveConflict( "local" );
      end,
      function ()
        func_message_main();
      end
    );

  end;

  func_message_server = function ()

    public.SystemDialog_Show(
      "conflictstorage_server",
      function ()
        parts.storage.ResolveConflict( "server" );
        common.Return();
      end,
      function ()
        func_message_main();
      end
    );

  end;

  func_message_main();

end;
--------------------------------------------------------------------------------
function private.StorageConflict_GetClientInfo ()
  
  local _, player_info_string = parts.storage.data.Read( "parts/cloudstorage/player/info" );
  local player_info = wiz.json.Decode( player_info_string );
  
  if not ( player_info ) then
    player_info = m_storage.Player.Info() or {};
  end;
  
  local client_data = {};

  client_data.stage_last = private.StorageConflict_GetStage( player_info[ 1 ].stage_last );
  
  local _, sections_string = parts.storage.data.Read( "parts/cloudstorage/player/sections" );
  local sections = wiz.json.Decode( sections_string );
  
  if not ( sections ) then
    sections = m_map.Section_GetList() or {};
  end;
  
  client_data.section = private.StorageConflict_GetSection( sections );
  client_data.gold = player_info[ 1 ].gold;
  
  return client_data;
  
end;
--------------------------------------------------------------------------------
function private.StorageConflict_GetServerInfo ()
  
  local conflictData = parts.storage.GetConflictSave();
  
  local player_info = wiz.json.Decode( conflictData[ "player/info" ] );
  
  local server_data = {};
  
  server_data.stage_last = private.StorageConflict_GetStage( player_info[ 1 ].stage_last );

  local sections = wiz.json.Decode( conflictData[ "player/sections" ] );
  
  server_data.section = private.StorageConflict_GetSection( sections );
  server_data.gold = player_info[ 1 ].gold;
  
  return server_data;
  
end;
--------------------------------------------------------------------------------
function private.StorageConflict_GetStage ( stage_name )
  
  local stage_number = tonumber( stage_name:match( "_(%d+)" ) );
  
  if ( stage_name:match( "_bonus" ) ) then
    return "bonus";
  end;
  
  if ( stage_number > 110 ) then
    stage_number = stage_number - 110 * math.modf( stage_number / 110 );
  end;
  
  return stage_number;
  
end;
--------------------------------------------------------------------------------
function private.StorageConflict_GetSection ( sections )
  
  local section = "section_1";
  
  for s = #sections, 1, -1  do
    if ( sections[ s ].enabled ) then
      section = sections[ s ].name;
      break;
    end;
  end;
  
  --section = tonumber( section:match( "_(%d+)" ) );
  
  return section;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PrioritySessionConflict ()

  parts.net.heartbeat.Stop();

  local func_show_prioritysession = function ()

    common.Return();

  end;

  public.SystemDialog_Show( "prioritysession", func_show_prioritysession );

end;
--------------------------------------------------------------------------------
function private.ApplicationGainFocus ()

  m_client.ConnectionReconnect( false );

  wiz.ChartBoost.LocationStartup();
  
  if ( IsPC() ) then
    
    return;
    
  end;

  local platform = GetPlatform();
  if ( platform == "android" ) then
  
    platform = wiz.Android.GetPlatform();
  
  end;

  m_client.WaitingForAnswer( "version",
                            "IsNewClientVersionAvailable",
                            { platform = platform },
                            private.OnIsNewClientVersionAvailable, true );

end;
--------------------------------------------------------------------------------
function private.OnIsNewClientVersionAvailable ( msg, params )
  
  if ( params.available ) then
    
    public.SystemDialog_Show( "newversion", function ()

      local platform = GetPlatform();
      if ( platform == "android" ) then
      
        platform = wiz.Android.GetPlatform();
      
      end;

      wiz.bfg.OpenUrl( private.app_url[ platform ] );

    end, function () end );
    
  end;

end;
--------------------------------------------------------------------------------
function private.ClientVersionOutdated ()

  if ( IsPC() ) then
    
    return;
    
  end;
  
  local func_continue = function ()

    local platform = GetPlatform();
    if ( platform == "android" ) then
    
      platform = wiz.Android.GetPlatform();
    
    end;

    wiz.bfg.OpenUrl( private.app_url[ platform ] );
    common.Return();

  end;

  public.SystemDialog_Show( "oldclientversion", func_continue );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.RaveLoginConflict ( params )
  
  private.Trace( "RaveLoginConflict: received", true );

  local func_message_main, func_message_local, func_message_server;

  func_message_main = function ()
    
    local clientTimeFormatted = os.date( "%Y-%m-%d", params.info_local.saveTime or clientTime );
    local serverTimeFormatted = os.date( "%Y-%m-%d", params.info_server.saveTime or serverTime );
    
    if clientTimeFormatted == serverTimeFormatted then
      clientTimeFormatted = os.date( "%H:%M:%S", params.info_local.saveTime or clientTime );
      serverTimeFormatted = os.date( "%H:%M:%S", params.info_server.saveTime or serverTime );
    end;
    
    local clientInfoData = { 
      stage_last = private.StorageConflict_GetStage( params.info_local.collections[ "player/info" ][ 1 ].stage_last ),
      section = private.StorageConflict_GetSection( params.info_local.collections[ "player/sections" ] ),
      gold = params.info_local.collections[ "player/info" ][ 1 ].gold
    };
    local serverInfoData = { 
      stage_last = private.StorageConflict_GetStage( params.info_server.collections[ "player/info" ][ 1 ].stage_last ),
      section = private.StorageConflict_GetSection( params.info_server.collections[ "player/sections" ] ),
      gold = params.info_server.collections[ "player/info" ][ 1 ].gold
    };

    public.SystemDialog_Show( "conflictstorage",
      func_message_local, 
      func_message_server,
      { clientTimeFormatted, clientInfoData },
      { serverTimeFormatted, serverInfoData }
    );
  
  end;

  func_message_local = function ()

    public.SystemDialog_Show(
      "conflictstorage_local",
      function ()
        params.func_local();
      end,
      function ()
        func_message_main();
      end
    );

  end;

  func_message_server = function ()

    public.SystemDialog_Show(
    "conflictstorage_server",
      function ()
        params.func_server();
        common.Return();
      end,
      function ()
        func_message_main();
      end
    );

  end;
  
  func_message_main();
  
end;