-- name=v_dailyquest
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ DAILYQUEST ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  loader.Module_Load( "int_dialog_dailyquest" );

  mediator.Subscribe(
    "Model_DailyQuest_EventScheduled",
    private.Panel_UpdateTimer
  );

  mediator.Subscribe(
    "Model_DailyQuest_ChallengeCooldownEnded",
    function ()
      
      private.OnCooldownEnd( "challenge" );
      
    end
  );
  
  mediator.Subscribe(
    "Model_DailyQuest_Ready",
    function ()
      
      private.OnCooldownEnd( "dailyquest" );
      
    end
  );

  private.dailyquest_data  = {};
  private.challenge_states = m_dailyquest.States();
  private.challenge_count  = m_dailyquest.Challenge_Count();

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Panel_Show ()
  
  wiz.ClearUsedResource();
  
  if ( not m_dailyquest.Countdown_IsNeeded() ) then

    int_panel_map_left.Dailyquest_Show(
      private.Dialog_Show,
      private.Panel_UpdateTimer
    );
    
  end;

end;
--------------------------------------------------------------------------------
function public.Panel_Hide ()

  int_panel_map_left.Dailyquest_Hide();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.OnMapUpdate ()

  return private.OnMapUpdate;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Dialog_Show ( winned_stage_name, func_after )
  
  private.func_ContinueMapUpdate = func_after;
  
  private.Update();

  local func       = {};
  local dailyquest = private.dailyquest_data;

  --- Stage --------------------------------------------------------------------

  func.OnStageComplete = function ( stage_name )

    local challenge_num;

    for num = 1, private.challenge_count do

      if ( dailyquest[ "challenge"..num ].stage == stage_name ) then
        challenge_num = num;
        break;
      end;
    end;

    if ( challenge_num ) then

      func.AnimateChallenge( challenge_num, private.challenge_states[ 2 ] );

    end;

  end;

  --- Reward -------------------------------------------------------------------

  func.Reward_Tap    = private.Reward_Tap;
  func.Reward_Notify = function ( challenge_num )

    mediator.Notify( "View_DailyQuest_RewardShown", challenge_num );

  end;

  --- Challenge ----------------------------------------------------------------

  func.Challenge_Tap = function ( challenge_id )

    local challenge = dailyquest[ "challenge"..challenge_id ];

    -- JK: open
    if ( challenge.state == private.challenge_states[ 1 ] ) then

      private.Dialog_Hide(
        function ()
 
          if ( v_stage.TryStart( challenge.stage, challenge ) ) then
            
            private.Dialog_Wait( challenge_id, challenge.stage );
            
          else
            
            -- JK: wait for hearts dialog closed
            mediator.SubscribeOnce(
              "Interface_Window_NoWindows",
              function()
            
                -- JK: if stage not started, continue map update
                if ( private.func_ContinueMapUpdate ) then
                  
                  private.func_ContinueMapUpdate();
                  private.func_ContinueMapUpdate = nil;
                  
                end;
              
              end
            );
            
          end;

        end
      );

    -- JK: beaten
    elseif ( challenge.state == private.challenge_states[ 2 ] ) then

      --

    end;

  end;

  func.AnimateChallenge = function ( challenge_id )    

    local old_dailyquest = private.dailyquest_data;

    private.Update();

    local new_dailyquest = private.dailyquest_data;
    
    int_dialog_dailyquest.Challenge_Set( 
      challenge_id, 
      new_dailyquest,
      old_dailyquest
    );

  end;

  --- Cooldown -----------------------------------------------------------------

  func.Cooldown_GetTime = function ( timer_type )

    return m_dailyquest.GetRemainingTime( timer_type );

  end;
  
  func.Cooldown_Skip = function ( timer_type )

    m_dailyquest.CooldownSkip( timer_type );

  end;

  ------------------------------------------------------------------------------

  func.DialogHide = function () private.Dialog_Hide( func_after ); end;
  
  func.OnShow = function ()

    private.dialog_active = true;
  
    int_dialog_dailyquest.Show(
      private.challenge_states,
      dailyquest,
      func
    );

    if ( winned_stage_name ) then

      func.OnStageComplete( winned_stage_name );

    end;
    
  end;
  
  v_map.TopPanel_Lock();
  
  if ( not v_map.TopPanel_IsVisible() ) then
  
    v_map.TopPanel_Show();
    v_map.TopPanel_Unlock();
    
  end;
    
  int_window.Show( "int_dialog_dailyquest", func.OnShow );

end;
--------------------------------------------------------------------------------
function private.Dialog_Hide ( func_end )
  
  private.wait_map_opened = nil;
  
  v_map.TopPanel_Hide( true );
      
  int_dialog_dailyquest.Hide( function ()

    private.dialog_active = false;

    int_window.Hide( "int_dialog_dailyquest" );

    if ( func_end ) then
   
      func_end();
    
    end;

  end );

end;
--------------------------------------------------------------------------------
function private.Dialog_Wait ( challenge_id, stage_name )
  
  local func_OnStageWon;
  local func_OnStageFailed;
  local func_OnStageAbandon;
  
  local func_OnMapUpdate = function ( stage_name )
    if not ( private.wait_map_opened ) then
      private.OnMapUpdate = function ( func_after )
      
        private.wait_map_opened = nil;
        mediator.Unsubscribe( "View_Stage_StageFailed",  func_OnStageFailed );
        mediator.Unsubscribe( "View_Stage_StageAbandon", func_OnStageAbandon );
        mediator.Unsubscribe( "Model_Stage_StageWon", func_OnStageWon );
        private.Dialog_Show( stage_name, func_after );
        private.OnMapUpdate = nil;

      end;
    end;
    private.wait_map_opened = true;
  end;
  
  func_OnStageFailed = function ()

    mediator.Unsubscribe( "View_Stage_StageFailed", func_OnStageFailed );
    common.Kontagent_LogEvent( "arena", 
    {
      name  = "fail",
      level = ( m_dailyquest.Challenge_Count() - 
                m_dailyquest.Challenge_GetOpenedCount() + 1 )
    } );
    func_OnMapUpdate();

  end;

  func_OnStageAbandon = function ()

    mediator.Unsubscribe( "View_Stage_StageAbandon", func_OnStageAbandon );
    common.Kontagent_LogEvent( "arena",
    {
      name  = "abandon",
      level = ( m_dailyquest.Challenge_Count() -
                m_dailyquest.Challenge_GetOpenedCount() + 1 )
    } );
    func_OnMapUpdate();

  end;

  func_OnStageWon = function ( stage_data )

    if ( stage_name ~= stage_data.name ) then

      return;

    end;

    mediator.Unsubscribe( "Model_Stage_StageWon", func_OnStageWon );

    func_OnMapUpdate( stage_data.name );

    m_dailyquest.Challenge_SetBeaten( challenge_id );
    m_dailyquest.Reward_Activate();

    int_dialog_dailyquest.Reward_Enable();

  end;

  mediator.Subscribe( "Model_Stage_StageWon", func_OnStageWon );
  mediator.Subscribe( "View_Stage_StageFailed", func_OnStageFailed );
  mediator.Subscribe( "View_Stage_StageAbandon", func_OnStageAbandon );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Update ()

  local dailyquest = m_dailyquest.Get();
  local enemies    = m_dailyquest.Enemies();

  dailyquest.reward_id = dailyquest.reward;
  
  for c = 1, private.challenge_count do

    for e = 1, #enemies do

      if ( enemies[ e ].name == dailyquest[ "challenge"..c ].enemy.name ) then

        dailyquest[ "challenge"..c ].enemy.content = enemies[ e ].content;

        break;

      end;

    end;

  end;

  dailyquest.reward    = m_dailyquest.Reward_Get();
  dailyquest.skipprice = m_dailyquest.GetSkipPrice();

  private.dailyquest_data = dailyquest;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.OnCooldownEnd ( timer_type )

  if ( not int_dialog_dailyquest.IsActive() ) then
    
    return;
    
  end;
  
  if ( timer_type == "challenge" ) then
    
    int_dialog_dailyquest.Timer_Hide();
    
  elseif( timer_type == "dailyquest" ) then
    
    int.Lock( true );
    
    private.Dialog_Hide(
      function ()
     
        private.Dialog_Show( nil, private.func_ContinueMapUpdate );
        
        int.Lock( false );
     
      end
    );
    
  end;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Panel_UpdateNotification ( cooldown, timer_type )
  
  if ( not private.dialog_active ) then

    local count = tern( cooldown > 0 )( 0, m_dailyquest.Challenge_GetOpenedCount() );
    count = tern( private.dailyquest_data.reward_id or 
      m_dailyquest.Get().reward )( 1, count );

    int_panel_map_left.Dailyquest_UpdateNotification( count );

  end;
  
end;
--------------------------------------------------------------------------------
function private.Panel_UpdateTimer ()

  local time = private.Time_GetRemaining();
  local step = private.Time_GetStep();

  int_panel_map_left.Dailyquest_UpdateTimer(
    time,
    step,
    private.Panel_UpdateTimer,
    function ()

      private.Panel_UpdateNotification( time.sec, time.type );

    end
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Time_GetRemaining ()

  local data = { sec = 0, mode = "m:s", type = "unknown" };

  if ( m_dailyquest.Countdown_IsActive() ) then

    data = {
      sec  = m_dailyquest.GetRemainingTime( "countdown" ) or 0,
      mode = "m:s",
      type = "countdown"
    };

  elseif ( not m_dailyquest.IsReady()              ) and
         ( not m_dailyquest.Challenge_IsSheduled() ) then

    data = {
      sec  = m_dailyquest.GetRemainingTime( "dailyquest" ) or 0,
      mode = "h:m",
      type = "dailyquest"
    };

  elseif ( m_dailyquest.Challenge_IsSheduled() ) then

    data = {
      sec  = m_dailyquest.GetRemainingTime( "challenge" ) or 0,
      mode = "m:s",
      type = "challenge"
    };

  end;

  return data;

end;
--------------------------------------------------------------------------------
function private.Time_GetStep ()

  if ( m_dailyquest.Countdown_IsActive()   ) or
     ( m_dailyquest.Challenge_IsSheduled() ) then

    return 1.0;

  else

    return 5.0;

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Reward_Tap ()

  if ( not private.dailyquest_data.reward_id ) then

    int_dialog_dailyquest.Reward_ShowPopup();

  else

    private.Reward_Show();

  end;

end;
--------------------------------------------------------------------------------
function private.Reward_Show ()

  local reward = private.dailyquest_data.reward;

  local chest = {
    type  = reward.type,
    name  = reward.name,
    count = reward.count
  };

  local params = {
    category   = "dailyquest",
    stage_name = chest.name,
    silent     = true
  };

  v_items.Chest_Show(
    chest,
    params,
    nil,
    function()

      if ( m_dailyquest.Challenge_GetOpenedCount() < 1 ) then

        int_dialog_dailyquest.Timer_TryShow( "dailyquest", m_dailyquest.GetSkipPrice() );

      else

        int_dialog_dailyquest.Timer_TryShow( "challenge" );
        int_dialog_dailyquest.Reward_Disable();
        
        local multiset = {};
        
        for id = 1, private.challenge_count do

          int_dialog_dailyquest.Challenge_Update( id, multiset );

        end;
        if ( #multiset > 0 ) then
          m_object.Multiset( multiset );
        end;

      end;

    end,
    function()

      local reward_id = private.dailyquest_data.reward_id;
      local reward    = private.dailyquest_data.reward;

      int_dialog_dailyquest.Reward_Set( reward_id, reward );

    end
  );
  
  common.Kontagent_LogEvent( "arena", 
  {
    name  = "collect_reward",
    level = private.dailyquest_data.reward_id
  } );

  int_dialog_dailyquest.Reward_Disable( true );

  m_dailyquest.Reward_Deactivate();

  private.Update();

end;
--------------------------------------------------------------------------------