-- name=v_fireflies
--------------------------------------------------------------------------------
function public.Init ()

  private.Init_Modules();
  private.Init_Subscribes();

  private.stickerpack_price = m_fireflies.Stickerpack_Price();
  
  private.Panel_UpdateCount( m_fireflies.Stickerpack_Count().total );  
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Panel_TryShow ()

  if ( m_fireflies.Unlocked() ) then
  
    local func_tap = function ()
    
      private.Dialog_Show();
      cmn.PlaySnd( "bottom_button" );
    
    end;

    int_panel_map_bottom.Stickers_Show( func_tap );
    
  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Init_Modules ()
  
  loader.Module_Load( "int_dialog_fireflies" );
  
end;
--------------------------------------------------------------------------------
function private.Init_Subscribes ()
  
  mediator.Subscribe(
    "Model_Fireflies_BoosterCountChanged",
    function ( params )
  
      private.Panel_UpdateCount( params.count );
    
    end
  );
  
  mediator.Subscribe(
    "Model_Fireflies_AchievementComplete",
    function ( reward )
  
      private.Achievement_Complete( reward );
    
    end
  );

  mediator.Subscribe(
    "Model_Fireflies_AchievementCompleteAll",
    function ( reward )
  
      private.Achievement_CompleteAll( reward );
    
    end
  );
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function public.Dialog_Show ( tab_number )

  private.Dialog_Show( tab_number );

end;
--------------------------------------------------------------------------------
function private.Dialog_Show ( tab_number )

  tab_number = tab_number or 1;

  ------------------------------------------------------------------------------
  local func_StickerpackGetCount = function ()
 
    return m_fireflies.Stickerpack_Count().total;
 
  end;
  ------------------------------------------------------------------------------
  local func_top_panel_show = function ()
  
    if ( not v_map.TopPanel_IsVisible() ) then

      int_panel_map_top.Highlight( "gold" );
      v_map.TopPanel_Show();

    end;
  
  end;
  ------------------------------------------------------------------------------
  local func_top_panel_hide = function ()

    if ( v_map.TopPanel_IsVisible() ) then

      v_map.TopPanel_Hide( false, int_panel_map_top.Restore );

    end;
  
  end;
  ------------------------------------------------------------------------------
  local func_show = function ()
  
    int_dialog_fireflies.Show(
      tab_number,
      m_fireflies.Firefly_Data,      
      m_fireflies.Firefly_GetAll,
      private.Stickerpack_Data,
      private.Duplicates_Data,
      func_StickerpackGetCount,
      m_fireflies.Sticker_TryStick,
      m_fireflies.Stats,
      private.Dialog_Hide,
      func_top_panel_show,
      func_top_panel_hide,
      private.Reward_TryShow,
      private.Reward_IsPending,
      private.Reward_GetData
    );
    
  end;
    
  int_window.Show( "int_dialog_fireflies", func_show );
  
end;
--------------------------------------------------------------------------------
function private.Dialog_Hide ( func_end )
  
  int_dialog_fireflies.Hide(
    function ()

      int_window.Hide( "int_dialog_fireflies" );

      if ( func_end ) then
  
        func_end();

      end;

    end
  );
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.Duplicates_Data ()

  return {
    duplicates      = m_fireflies.Sticker_Duplicates(),
    func_TrySell    = m_fireflies.Sticker_TrySell,
    func_TrySellAll = m_fireflies.Sticker_TrySellAll
  };  
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.Stickerpack_Data ()
  
  local price = private.stickerpack_price;

  local func_Open = function ()

    local stickers = private.Stickerpack_TryOpen();

    if ( stickers ) then

      return stickers;

    end;

    return false;

  end;
  ------------------------------------------------------------------------------
  local func_Get = function ()

    return func_Open();

  end;
  ------------------------------------------------------------------------------
  local func_Buy = function ()
  
    if ( m_player.TryChangeGoldCount( - price ) ) then
      
      common.Kontagent_LogEvent( "economy", 
      {
        description = "premium_currency_spend",
        category    = "fireflies_album",
        value       = - price,
        name        = "stickerpack"
      } );
  
      if ( m_fireflies.Stickerpack_TryObtain( 1, true, "album" ) ) then
      
        return func_Open();

      end;

      return false;
      
    end;
    
    return false;
  
  end;
  ------------------------------------------------------------------------------
  return {
    price       = price,
    packs_total = m_fireflies.Stickerpack_Count(),
    func_Get    = func_Get,
    func_Buy    = func_Buy
  };
  
end;
--------------------------------------------------------------------------------
function private.Stickerpack_TryOpen ()
  
  return m_fireflies.Stickerpack_TryOpen();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Panel_UpdateCount ( count )
  
  int_panel_map_bottom.Stickers_SetNotification( count );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Achievement_Complete ( reward )

  private.reward = reward;
    
end;
--------------------------------------------------------------------------------
function private.Achievement_CompleteAll ( reward )

  private.reward_all = reward;

end;
--------------------------------------------------------------------------------
function private.Reward_TryShowForAll ()
  
  if ( not private.reward_all ) then

    return;

  end;

  int.Lock( true );

  int_dialog_fireflies.Tab_Show( 4 );
  
  local reward = private.reward_all.custom;
  local chest  =
  {
    type  = private.reward_all.type,
    name  = private.reward_all.name,
    count = private.reward_all.count
  };

  private.reward_all = nil;

  local obtain_params = 
  {
    category   = "album",
    stage_name = "album",
    silent     = true
  };

  common.Defer(
    "album_reward",
    1,
    function()

      v_items.Chest_Show(
        chest,
        obtain_params,
        reward,
        nil,
        function ()

          int.Lock( false );

        end
      );

    end
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Reward_TryShow ( func_exec )

  if ( not private.reward ) then

    return;

  end;

  local reward   = private.reward;
  private.reward = nil;

  if ( reward.type == "chest" ) then
    
    local chest_data =
    {
      type  = reward.type,
      name  = reward.name,
      count = 1
    };

    local obtain_params = 
    {
      category   = "stickers",
      stage_name = chest_data.name,
      silent     = true
    };

    v_items.Chest_Show(
      chest_data,
      obtain_params,
      reward.custom,
      private.Reward_TryShowForAll,
      func_exec
    );

    return;

  end;

  v_items.Reward_Show( reward );

end;
--------------------------------------------------------------------------------
function private.Reward_IsPending ()

  return private.reward ~= nil;

end;
--------------------------------------------------------------------------------
function private.Reward_GetData ()

  return {
    index  = private.reward.index,
    layout = private.reward.layout
  };

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ FIREFLIES ] "..text );
  
end;
--------------------------------------------------------------------------------