-- name=v_hero
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ HERO ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  loader.Modules_Load( {
    "int_hero",
    "int_dialog_hero",
    "int_panel_map_achievement",
    "int_dialog_map_changeskin"
  } );

  private.Hero_SetDialogHandlers();
  
  private.Notification_Reset();
  
  private.Init_Subscribes();
  
  private.Hero_UpdatePanel();
  
  private.Trace( "Initialized." );
  
end;
--------------------------------------------------------------------------------
function private.Init_Subscribes ()
  
  mediator.Subscribe(
    "Model_Skins_SkinCollected",
    function ( skin_name ) 
      private.Notification_Skins_Add( skin_name );
      private.Hero_UpdatePanel();
    end
  );
  
  mediator.Subscribe(
    "Model_Skins_PartCollected",
    function ( params )
  
      private.Notification_Skins_AddPart( params.skin );
      private.Hero_UpdatePanel();

    end
  );
  
  mediator.Subscribe(
    "Model_Skins_PlatformSkinCollected",
    function ( platform )
  
      int_panel_map_bottom.Hero_SetPlatformNotification( platform );

    end
  );

  mediator.Subscribe(
    "Model_Achievements_Rewarded",
    private.Hero_UpdatePanel
  );
  
  mediator.Subscribe(
    "Model_Achievements_Completed",
    function ( achievement_data )

      private.Notification_Achievements_Add( achievement_data.level );
      private.Hero_UpdatePanel();
    
    end
  );
  
  m_skins.TryObtainPlatformSkins();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Achievement_TryShowPanel ()
  
  if ( #private.notifications[ "achievements" ] > 0 ) then
    
    local achievement_level = private.notifications[ "achievements" ][ #private.notifications[ "achievements" ] ]; 

    private.notifications[ "achievements" ] = {};
   
    local func_OnHide = function () end;
    
    local func_Tap = function ()
  
      v_map.TopPanel_Lock();
      private.Hero_ShowDialog();
    
    end;
    
    int_panel_map_achievement.Show(
      "btn_int_panel_map_hero",
      achievement_level,
      func_OnHide,
      func_Tap
    );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Achievement_HidePanel ()
  
  int_panel_map_achievement.Hide();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Hero_ShowPanel ()
  
  local func_tap = function ( force_tab )

    v_map.TopPanel_Lock();

    private.Hero_ShowDialog( force_tab );

    cmn.PlaySnd( "bottom_button" );

  end;

  int_panel_map_bottom.Hero_Show( func_tap );

end;
--------------------------------------------------------------------------------
function public.Hero_FocusSkin ( skin_name, func_end )

  int_dialog_hero.Skins_Focus( skin_name, func_end );

end;
--------------------------------------------------------------------------------
function private.Hero_UpdatePanel ()
    
  int_panel_map_bottom.Hero_SetNotification(
    private.Notification_Total_Count()
  );

end;
--------------------------------------------------------------------------------
function private.Hero_SetDialogHandlers ()

  -- Skins
  local function func_change_skin ( skin_name )
    local skins, reason = m_skins.GetAllSkinsInfo();

    if ( not skins and reason == "obsolete" ) then
      v_client.ContentObsolete( true );
      return;
    end;

    if ( m_skins.TrySetSkin( skin_name ) ) then
      skins = m_skins.GetAllSkinsInfo();
      int_dialog_hero.Skins_Update( skins );
    end;

  end;

  int_dialog_hero.SetSkinHandlers(
    public.Skin_Attach,
    public.Skin_Detach, 
    func_change_skin
  );

  -- Achievments
  int_dialog_hero.SetAchievHandlers(
    m_achievements.GetReward,
    m_achievements.GetRewardData,
    private.Notification_Achievements_Count,
    private.OpenGooglePlayAchievements
  );

  -- Info
  local function func_is_need_login ()
    return ( IsMobile() and m_client.IsOnline() and not m_social.IsLogged() );
  end;

  int_dialog_hero.SetInfoHandlers( func_is_need_login, m_social.Login );

  -- Generic
  local function func_top_panel_toggle ( tab_name )

    if ( tab_name == "achievements" ) then
      v_map.TopPanel_Show();
    else
      v_map.TopPanel_Hide( true );
    end;

  end;

  int_dialog_hero.SetGenericHandlers(
    func_top_panel_toggle, 
    private.Hero_UpdatePanel
  );

end;
--------------------------------------------------------------------------------
function private.Hero_ShowDialog ( force_tab )

  local statistic     = m_hero.GetGameStatistic();
  local skins, reason = m_skins.GetAllSkinsInfo();
  local achievements  = m_achievements.GetAll();

  if ( not skins and reason == "obsolete" ) then
    v_client.ContentObsolete( true );
    return;
  end;

  local tab_name = "skins";

  if ( private.Notification_Achievements_Count( achievements ) > 0 ) then

    tab_name = "achievements";

  end;
  
  if ( private.Notification_Skins_Count() > 0 ) then

    tab_name = "skins";

  end;
  
  if ( force_tab ) then

    tab_name = force_tab;

  end;
  
  int_dialog_hero.Show( 
    tab_name, 
    skins, 
    achievements, 
    statistic, 
    private.notifications[ "skins" ][ 1 ] or m_skins.GetSkin(),
    private.notifications[ "skins" ],
    private.notifications[ "skinparts" ]
  );

  private.Notification_Reset();
  private.Achievement_HidePanel();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.ChangeSkin_ShowDialog ( new_skin_part, func_OnClose )

  local old_skin      = m_skins.GetSkin(); 
  local new_skin      = m_skins.GetSkinByPart( new_skin_part );
  local new_skin_info = m_skins.GetSkinInfo( new_skin );
  
  if not ( m_skins.GetSkinInfo( new_skin ).primary ) then
    
    ( func_OnClose or function () end )();
    return;
    
  end;

  local func_skins_attach = function ( parent, skin_name )

    return public.Skin_Attach( parent, "fight", skin_name );

  end;

  local func_skins_detach = function ( anm_name )

    public.Skin_Detach( anm_name );

  end;

  local func_close = function ()
    
    ( func_OnClose or function () end )();
    m_skins.TrySetSkin( new_skin ); 

    private.ChangeSkin_HideDialog();

  end;

  local func_start = function ()

    v_map.TopPanel_Hide();

    int_dialog_map_changeskin.Show( 
      old_skin,
      new_skin,
      new_skin_info, 
      func_skins_attach, 
      func_skins_detach, 
      func_close 
    );

  end;

  int_window.AddWindowToQueue( "int_dialog_map_changeskin", func_start );

end;
--------------------------------------------------------------------------------
function private.ChangeSkin_HideDialog ( func_end )

  int_dialog_map_changeskin.Hide( function ()

    int_window.Hide( "int_dialog_map_changeskin", func_end );

  end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Notification_Total_Count ()

  local count =
    private.Notification_Achievements_Count() +
    private.Notification_Skins_Count();

  return count;

end;
--------------------------------------------------------------------------------
function private.Notification_Skins_Count ()
  
  return #private.notifications[ "skins" ];
  
end;
--------------------------------------------------------------------------------
function private.Notification_Skins_Add ( skin_name )

  local skins = private.notifications[ "skins" ];

  if not ( cmn.IsEntityInList( skins, skin_name ) ) then

    table.insert( skins, skin_name );

  end;
  
end;
--------------------------------------------------------------------------------
function private.Notification_Skins_AddPart ( skin_name )

  local parts = private.notifications[ "skinparts" ];

  if not ( parts[ skin_name ] ) then

    parts[ skin_name ] = 1;

  else

    parts[ skin_name ] = parts[ skin_name ] + 1;

  end;

  private.Notification_Skins_Add( skin_name );
  
end;
--------------------------------------------------------------------------------
function private.Notification_Achievements_Count ( ext_achievement_list )

  local achievements = ext_achievement_list or m_achievements.GetAll();
  local count = 0;

  for a = 1, #achievements do

    for r = 1, #achievements[ a ].reward do

      if  ( achievements[ a ].reward[ r ].get      )
      and ( not achievements[ a ].reward[ r ].done ) then

        count = count + 1;
        break;

      end;

    end;

  end;

  return count;

end;
--------------------------------------------------------------------------------
function private.Notification_Achievements_Add ( achievement_level )
  
  table.insert( private.notifications[ "achievements" ], achievement_level );
  
end;
--------------------------------------------------------------------------------
function private.Notification_Reset ()

  private.notifications = {
    skins        = {},
    skinparts    = {},
    achievements = {}
  };
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Skin_Attach ( destination, skin_type, skin_name )

        skin_type    = skin_type or "ride";
        skin_name    = skin_name or m_skins.GetSkin();
  local skin_content = m_skins.GetContent( skin_name );

  return int_hero.Skin_Attach(
    destination,
    skin_type,
    skin_name,
    skin_content
  );

end;
--------------------------------------------------------------------------------
function  public.Skin_Detach ( anm_name )

  int_hero.Skin_Detach( anm_name );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.OpenGooglePlayAchievements ()

  if not ( IsAndroid() and ( wiz.Android.GetPlatform() == "google" ) ) then
  
    return;
  
  end;

  if ( wiz.Android.GooglePlayIsConnected() ) then
    wiz.Android.AndroidShowAchievements();
  else
    wiz.Android.GooglePlaySignIn();
  end;

end;
--------------------------------------------------------------------------------