-- name=v_items
--------------------------------------------------------------------------------
function public.Init ()

  private.dialog_chest  = "int_dialog_items_chest";
  private.dialog_reward = "int_dialog_reward";

  loader.Modules_Load( {
    private.dialog_chest,
    private.dialog_reward
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Chest_Show (
    chest_data,
    obtain_params,
    ext_rewards,
    func_after,
    func_before,
    ignore_panel
  )

  -- JK: default is wood chest
  chest_data = chest_data or
  {
    type  = "chest",
    name  = "woodchest",
    count = 1
  };

  local rewards = ext_rewards or m_items.Chest_Generate( chest_data );

  if ( obtain_params ) then

    -- JK: Obtain rewards immediately
    for index, reward in pairs( rewards ) do

      m_items.TryObtain( {
        name       = reward.name,
        type       = reward.type,
        count      = reward.count,
        category   = obtain_params.category,
        stage_name = obtain_params.stage_name,
        silent     = obtain_params.silent
      } );

    end;

  end;

  ------------------------------------------------------------------------------
  local func_show_top_panel = function ( highlight_gold )
  
    if ( highlight_gold ) then
  
      int_panel_map_top.Highlight( "gold" );
    
    end;
  
    v_map.TopPanel_Show();
      
  end;
  ------------------------------------------------------------------------------
  local func_hide_top_panel = function ()
  
    if ( v_map.TopPanel_IsVisible() ) then
  
      v_map.TopPanel_Hide( false, function()
      
        int_panel_map_top.Restore();
        
      end );
      
    end;
      
  end;
  ------------------------------------------------------------------------------
  local func_get_rewards = function ()
  
    local highlight_gold = false;
    local show_top_panel = false;

    for r, reward in ipairs( rewards ) do

      if ( reward.type == "gold"     ) or
         ( reward.type == "resource" ) then

        show_top_panel = true;

      end;

      if ( reward.type == "gold" ) and ( #rewards < 2 ) then
        
        highlight_gold = true;

        break;
        
      end;      
      
    end;

    if ( show_top_panel ) then

      func_show_top_panel( highlight_gold );

    end;
    
    return rewards;
  
  end;
  ------------------------------------------------------------------------------
  local func_count_rewards = function ()
       
    for r, reward in pairs( rewards ) do
    
      if ( reward.type == "gold" ) then
        
        v_map.Gold_Buffer_Out();
        
      end;
      
      if ( reward.type == "resource" ) then
        
        v_map.Resource_Buffer_Out( reward.name );
        
      end;
      
    end;
  
  end;
  ------------------------------------------------------------------------------
  local func_hide = function ()

    private.Chest_Hide( function ()
    
      if ( not ignore_panel ) then
      
        func_hide_top_panel();
      
      end;
      
      if ( func_after ) then func_after(); end;
      
    end );
  
  end;
  ------------------------------------------------------------------------------
  local func_show = function ()

    int_dialog_items_chest.Show(
      chest_data.name,
      func_get_rewards,
      func_count_rewards,
      func_hide,
      func_before
    );

  end;
  ------------------------------------------------------------------------------

  int_window.Show( private.dialog_chest, func_show );

end;
--------------------------------------------------------------------------------
function private.Chest_Hide ( func_end )

  int_dialog_items_chest.Hide( function()

    int_window.Hide( private.dialog_chest, func_end );

  end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Reward_Show ( reward, func_end )

  local res = m_content.Res( reward.type, reward.name );

  local func_close = function ()

    int_dialog_reward.Hide(
      function ()

        int_window.Hide( private.dialog_reward, func_end );

      end
    );

  end;

  int_window.AddWindowToQueue(
    private.dialog_reward,
    function ()

      int_dialog_reward.Show( res, reward.count, func_close );

    end
  );

end;
--------------------------------------------------------------------------------