-- name=v_map
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ MAP ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()

  private.Init_Modules();
  private.Init_Variables();
  private.Init_Objects();
  private.Init_Subscribes();
  
  private.Trace( "Initialized." );

  private.DEV_GetPointPos();
  
  -- common.DbgWatchTable( private, "v_map", "marker_point_current", false );

end;
--------------------------------------------------------------------------------
function private.Init_Modules ()

  loader.Modules_Load({
    "v_mines",
    "v_builds",
    "v_characters",
    "v_chests",
    "v_fights",
    "v_spells",
    "v_friends",
    "int_panel_map_top",
    "int_panel_map_left",
    "int_panel_map_right",
    "int_panel_map_bottom",
    "int_dialog_map_build_market",
    "int_dialog_map_comingsoon",
    "int_dialog_map_chest",
    "int_dialog_map_reward",
    "int_dialog_worldmap"
  });

end;
--------------------------------------------------------------------------------
function private.Init_Variables ()

  -- JK: ask for screen dimensions once
  private.app_h = int.H();
  private.app_w = int.W();
  private.app_r = int.AspectRatio();
  
  -- Первый запуск.
  private.first_load = true;

  -- Доп. обработчики состояния.
  private.point_handler = {};

  -- Созданные секции.
  private.sections = {};

  private.map_scale_current = 1.0;
  private.map_scale_max = 1.0;
  private.map_scale_min = 0.5;
  
  if ( private.app_h < 960 ) then
    
    private.map_scale_current = 0.75;
    private.map_scale_max = 0.75;
    
  end; 

  private.scale_modifier = 2;
  private.offset         = 256;
  private.map_w          = 2048;
  private.map_h          = 1280;

  private.reward          = {};
  private.resource_buffer = {};

  private.interface         = {};
  private.interface.enabled = true;
  private.interface.anm_tme = 0.3;
  private.interface.bot_tme = 0.8; -- Bottom panel animation time

  private.popup_shift = { 0, - 100 };
  
  private.opened = false;

  private.appear_queue = {};

  private.this = {
    background = "obj_map_background",
    body       = "obj_map_body",
    marker     = "obj_map_marker"
  };

end;
--------------------------------------------------------------------------------
function private.Init_Objects ()

  m_object.Attach( "tmr_map_ground_update", InterfaceName );

  --[[
  ObjSet( "obj_map_drag", 
  { 
    event_startdrag = private.DragStart,
    event_drag      = private.DragUpdate,
    event_dragdrop  = private.DragDrop,
  } );
  ]]

  local screen_w = private.app_w;
  local screen_h = private.app_h;
  local scroll_h = private.scale_modifier * private.map_h;

  local multiset = {};

  m_object.Set(
    "zmv_map",
    {
      scale         = 0.75,
      min_scale     = private.map_scale_min,
      max_scale     = private.map_scale_max,
      scrollrect_x  = 0,
      scrollrect_y  = - scroll_h,
      scrollrect_w  = private.scale_modifier * private.map_w,
      scrollrect_h  = scroll_h,
      screen_w      = screen_w,
      screen_h      = screen_h,
      begindragtrig = private.DragStart,
      dragtrig      = private.DragUpdate,
      enddragtrig   = private.DragDrop
    },
    multiset
  );

  GridSet( "grd_map_foreground_movie_vignette", 0, 
  {
    2,  { pos_x = screen_w - 1 },
    3,  { pos_x = screen_w     },
    6,  { pos_x = screen_w - 1 },
    7,  { pos_x = screen_w     },

    8,  { pos_y = screen_h - 4 },
    9,  { pos_y = screen_h - 4 },
    10, { pos_x = screen_w - 1, pos_y = screen_h - 4 },
    11, { pos_x = screen_w,     pos_y = screen_h - 4 },

    12, { pos_y = screen_h },
    13, { pos_y = screen_h },
    14, { pos_x = screen_w - 1, pos_y = screen_h },
    15, { pos_x = screen_w,     pos_y = screen_h }
  } );
  
  screen.Setup( "scr_map",
  {
    func_pre_open    = private.PreOpen,
    func_open        = private.Open,
    func_close       = private.Close,
    func_after_close = private.AfterClose
  } );

  m_object.Attach( "scr_map", "ne_storage" );

  if ( not m_tutorial.IsCompleted( "intro" ) ) then

    local pos_x = 
    {
      [ 640  ] = - 1152,
      [ 768  ] = - 1024,
      [ 1024 ] = - 800
    };

    m_object.Set(
      private.this.body,
      {
        pos_x   = pos_x[ screen_w ],
        pos_y   = screen_h,
        scale_x = 0.5,
        scale_y = 0.5
      },
      multiset
    );

  end;
  
  if ( wiz.IsCheatsEnabled() ) then
    
    m_object.Enable( "obj_map_point_DEV", false, multiset );
    
  end;

  m_object.Set(
    private.this.background,
    {
      pos_x = 0,
      pos_y = 0
    },
    multiset
  );

  m_object.Set(
    "anm_map_foreground_fox_actor",
    {
      pos_x = 0.5 * private.app_w
    },
    multiset
  );

  m_object.Set(
    "obj_map_anims",
    {
      pos_x =  private.offset,
      pos_y = -private.offset
    },
    multiset
  );

  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.Init_Subscribes ()

  mediator.Subscribe( "Model_Player_HeartsCountChanged",    private.LeftPanel_UpdateHeartsCount );
  mediator.Subscribe( "Model_Player_GoldCountChanged",      private.Gold_UpdatePanel );
  mediator.Subscribe( "Model_Skins_SkinSet",                private.Marker_SetSkin );
  mediator.Subscribe( "Model_Player_ResourceCountChanged",  private.Resource_UpdatePanel );
  mediator.Subscribe( "Model_DailyQuest_Ready",             private.LeftPanel_TryShowDailyquest );
  
  -- QuestList update subscribtions
  mediator.Subscribe( "View_Map_MapOpened",            private.TryExecuteQuestEvents );
  mediator.Subscribe( "View_Map_Section_Switched",     private.TryExecuteQuestEvents );
  mediator.Subscribe( "Model_Mines_MineStateChanged",  private.TryExecuteQuestEvents );
  mediator.Subscribe( "Model_Builds_BuildUnlocked",    private.TryExecuteQuestEvents );
  mediator.Subscribe( "Model_Builds_BuildConstructed", private.TryExecuteQuestEvents );
  mediator.Subscribe( "Model_Spells_SpellCompleted",   private.TryExecuteQuestEvents );
  mediator.Subscribe( "Model_Market_Constructed",      private.TryExecuteQuestEvents );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.IsOpened ()

  return private.opened;
  
end;
--------------------------------------------------------------------------------
function public.IsActive ()

  return private.IsActive();
  
end;
--------------------------------------------------------------------------------
function private.IsActive ()

  return screen.GetCurrentName() == "scr_map";

end;
--------------------------------------------------------------------------------
function public.FocusAtCurrentPoint ( func_after )

  if ( private.marker_point_current ) then
  
    local target = private.Marker_Target( private.marker_point_current );
    public.Focus( target, { func_after = func_after } );
  
  elseif ( func_after ) then
  
    func_after();
  
  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Scale_GetMax ()

  return private.map_scale_max;

end;
--------------------------------------------------------------------------------
function public.Scale_GetMin ()

  return private.map_scale_min;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Interface_IsVisible ()
  
  return private.interface.visible;
  
end;
--------------------------------------------------------------------------------
function public.Interface_Enable ( show, point, func_end )
  
  --DbgTrace( "Interface_Enable: "..tostring( point ) );

  if ( not private.opened                          ) or
     ( private.IsPending() == "ConstructBuildAnim" ) then

    --DbgTrace( "Interface_Enable: blocked" );

    return;

  end;

  private.interface.enabled = true;
  
  if ( private.interface.visible ) then
    
    int.Lock( false );
    
  end;
  
  if ( show ) then

    private.Interface_Show( private.TopPanel_Unlock, func_end );

  end;

end;
--------------------------------------------------------------------------------
function public.Interface_Disable ( hide )

  if ( hide ) then

    private.Interface_Hide();

  end;

  private.interface.enabled = false;

end;
--------------------------------------------------------------------------------
function private.Interface_Show ( func_OnStart, func_OnEnd )

  if ( private.interface.enabled ) then
    
    if not ( private.interface.visible ) then

      private.interface.visible = true;

      if ( func_OnStart ) then

        func_OnStart();

      end;

      local func_End = function ()

        mediator.Notify( "View_Map_InterfaceVisible" );
        int.Lock( false );
        
        if ( func_OnEnd ) then func_OnEnd(); end;

      end;

      private.TopPanel_Show();
      private.BottomPanel_Show( func_End );

      private.SidePanels_Show();
    
    else
    
      -- Check this!
      ( func_OnEnd or function () end )();
        
    end;
      
  elseif ( IsEditor() and func_OnEnd ) then

    common.DbgCallpoint( "Interface disabled, but tries to show!" );
      
  end;

end;
--------------------------------------------------------------------------------
function private.Interface_Hide ( func_OnStart, func_OnEnd )

  if --[[( private.interface.enabled ) and]] ( private.interface.visible ) then

    private.interface.visible = false;
    
    if ( func_OnStart ) then

      func_OnStart();

    end;

    local func_End = function ()

      if( func_OnEnd ) then func_OnEnd(); end;
    
      mediator.Notify( "View_Map_InterfaceInvisible" );

    end;

    private.TopPanel_Hide();
    private.BottomPanel_Hide( func_End );

    private.SidePanels_Hide();

  end;

end;
--------------------------------------------------------------------------------
function private.Interface_Off ()

  int_panel_map_top.Off();
  int_panel_map_left.Off();
  int_panel_map_bottom.Off();
  int_panel_map_right.Off();

  public.Interface_Disable();

end;
--------------------------------------------------------------------------------
function private.Interface_Input_On ()

  int_panel_map_top.Input_On();
  int_panel_map_left.Input_On();
  int_panel_map_bottom.Input_On();
  int_panel_map_right.Input_On();

end;
--------------------------------------------------------------------------------
function private.Interface_Input_Off ()

  int_panel_map_top.Input_Off();
  int_panel_map_left.Input_Off();
  int_panel_map_bottom.Input_Off();
  int_panel_map_right.Input_Off();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.SidePanels_Show ()

  private.LeftPanel_Show();
  private.RightPanel_Show();

end;
--------------------------------------------------------------------------------
function private.SidePanels_Hide ()

  private.LeftPanel_Hide();
  private.RightPanel_Hide();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Intro_Prepare ( func_end )

  local section_name = private.Section_Current_GetName();
  
  public.TopPanel_Hide( true );

  private.intro_active = true;

  private.interface.enabled = false;
  private.interface.visible = false;

  ObjAttach( "obj_map_spawn", "obj_map_anims" );
  ObjAttach( "obj_map_foreground", InterfaceName );

  local objects = {};

  table.insert( objects, { "obj_map_foreground", { alp = 0 } } );

  for i = 1, 9 do

    AnimStop( "anm_map_builds_icon_build_"..i );

    table.insert( objects, { "anm_map_builds_icon_build_"..i, { animfunc = "normal", playing = true } } );
  
  end;
  
  for i = 1, 11 do

    table.insert( objects, { "btn_map_point_stage_"..i,  { alp = 0, scale_x = 0 } } );
    
  end;
 
  table.insert( objects, { private.this.marker, { alp = 0 } } );
  table.insert( objects, { "obj_map_spawn",  { active = true, visible = true } } );

  table.insert( objects, { "obj_map_foreground_intro",  { active = true, visible = true } } );
  table.insert( objects, { "anm_map_builds_icon_state_puzzle_1", { animfunc = "intro", playing = false } } );
  table.insert( objects, { "anm_map_point_enemy_reddragon_stage_110", { animfunc = "action", playing = false } } );

  ObjMultiSet( objects );

  local screen_w = private.app_w;
  local screen_h = private.app_h;

  GridSet( "grd_map_foreground_vignette", 0, 
  {
    2,  { pos_x = screen_w - 1 },
    3,  { pos_x = screen_w     },
    6,  { pos_x = screen_w - 1 },
    7,  { pos_x = screen_w     },

    8,  { pos_y = screen_h - 4 },
    9,  { pos_y = screen_h - 4 },
    10, { pos_x = screen_w - 1, pos_y = screen_h - 4 },
    11, { pos_x = screen_w,     pos_y = screen_h - 4 },

    12, { pos_y = screen_h },
    13, { pos_y = screen_h },
    14, { pos_x = screen_w - 1, pos_y = screen_h },
    15, { pos_x = screen_w,     pos_y = screen_h }
  } );

  ObjAnimate( "obj_map_foreground", "alp", 0, 0, "", 
  { 
    0.0, 0, 0.0, 
    0.6, 0, 1.0 
  } );

  local func_skip = function ( func_OnAnimEnd )

    private.Intro_Animate_Skip( func_OnAnimEnd );

  end;
  
  local func_after_skip = function ()
  
    private.intro_active = false;

    --public.Interface_Enable( true, "intro tutorial skip" );
    int.Lock( false );
    
    if ( func_end ) then

      func_end();

    end;
  
  end;
  
  int.Subtitles_Show( "intro", section_name, func_skip, func_after_skip );

end;
--------------------------------------------------------------------------------
function public.Intro_Animate_Dragon ( func_end )

  int.Lock( true );
  
  local section_name = private.Section_Current_GetName();

  local func_OnAnimationEnd = function ()

    local func_anim_end = function ()
      
      ObjSet( "obj_map_foreground_intro", { active = false, visible = false } );
      ObjAttach( "obj_map_foreground", private.this.body );

      private.Music_Start();
      
      private.Intro_Animate_Hero( func_end );

    end;

    public.Focus( "obj_map_builds_puzzle_1", { time = 1.0, scale = 0.8 } );

    ObjAnimate( "obj_map_foreground_intro", "alp", 0, 0, func_anim_end, 
    { 
      0.0, 0, 1.0, 
      2.0, 0, 0.0 
    } );

  end;

  local func_OnDummyEnd = function ()

    local indexes = { { 4, 1 }, { 3, 2 }, { 2, 3 }, { 8, 4 }, { 9, 4.3 }, { 1, 2 }, { 5, 2 }, { 6, 2 }, { 7, 2 } };    

    local func_anim_end = function ( index )
      
      private.Intro_Animate_Builds( index ); 

    end;

    for i = 1, #indexes do

      ObjAnimate( "anm_map_builds_icon_build_"..indexes[ i ][ 1 ], "alp", 0, 0, function () func_anim_end( indexes[ i ][ 1 ] ) end, 
      { 
        0.0                    , 0, 1,
        0.0 + indexes[ i ][ 2 ], 0, 1,   
        0.3 + indexes[ i ][ 2 ], 0, 1 
      } );
    
    end;

    local offset = 
    {
      default    = -160,
      [ "16:9" ] = - 70
    };

    local offset_y = offset[ private.app_r ] or offset.default;
    
    public.Focus( "obj_map_point_stage_110",
      {
        time       = 9.0,
        scale      = 1.0,
        offset     = { x = 0, y = offset_y },
        func_after = func_OnAnimationEnd
      }
    );

  end;

  public.Focus( "obj_map_point_stage_25", { time  = 18.0, scale = 0.8 } );

  AnimStop( "anm_map_point_enemy_reddragon_stage_110" );
  AnimPlay( "anm_map_foreground_dragon",       "action", function () 

    AnimPlay( "anm_map_point_enemy_reddragon_stage_110", "static", "" );
    AnimPlay( "anm_map_point_enemy_reddragon_stage_110", "action", function () 

      AnimPlay( "anm_map_point_enemy_reddragon_stage_110", "static", "" );

      AnimPlay( "anm_map_builds_icon_state_puzzle_1", "intro", "" );

    end ); 

  end  );
  AnimPlay( "anm_map_foreground_dragon_dummy", "dummy",  func_OnDummyEnd );

  private.Music_Stop();
  cmn.PlaySnd( "intro" );

end;
--------------------------------------------------------------------------------
function private.Intro_Animate_Builds ( index )

  local anm_action = "anm_map_builds_icon_build_"..index.."_action";
  local anm_dummy  = "anm_map_builds_icon_build_"..index.."_dummy";

  m_object.Create(
    anm_action,
    "anim",
    "obj_map_builds_build_"..index,
    {
      pos_x = 0,
      pos_y = 0,
      pos_z = 10
    }
  );

  m_object.Create(
    anm_dummy,
    "anim",
    "obj_map_builds_build_"..index,
    {
      pos_x = 0,
      pos_y = 0,
      pos_z = 10
    }
  );

  local multiset = {};

  m_object.Set(
    anm_action,
    { 
      res      = "assets/content/map/effects/anm_appearance_build",
      animfunc = "action",
      color_r  = 0.7,
      color_g  = 0.2,
      color_b  = 0.8, 
      playing  = true,
      endtrig  = function () 

        m_object.Delete( anm_action );

      end
    },
    multiset
  );

  m_object.Set(
    anm_dummy,
    { 
      res      = "assets/content/map/effects/anm_appearance_build",
      animfunc = "dummy", 
      playing  = true,
      endtrig  = function () 

        m_object.Delete( anm_dummy );

        m_object.Set(
          "anm_map_builds_icon_build_"..index,
          {
            animfunc = "corrupted",
            playing  = true
          }
        );

      end
    },
    multiset
  );

  m_object.Multiset( multiset ); 

end;
--------------------------------------------------------------------------------
function private.Intro_Animate_Hero ( func_end )

  local func_OnAnimationEnd = function ()

    local func_OnSpawnEnd = function ()

      private.intro_active = false;
      
      public.Focus( "obj_map_builds_puzzle_1",
        {
          time       = 1.5,
          scale      = 1.0,
          func_after = function ()
            
            --public.Interface_Enable( true, "animate hero" );
            
            int.Lock( false );
            
            if ( func_end ) then
            
              func_end();
           
            end;
            
          end
        }
      );

    end;

    private.Stages_Show_Cascade( 1, 11, func_OnSpawnEnd );
    
    ObjAnimate( private.this.marker, "alp", 0, 0, "",
    {
      0.0, 0, 0,
      0.3, 0, 1
    } );
    
    public.Focus( "obj_map_point_stage_10", { time = 1.5, scale = 0.6 } );

    public.HideObjectPopup();

  end;

  local target = private.Marker_Target( "stage_2" );
  public.Focus( target, { time = 1.0 } );

  AnimPlay( "anm_map_spawn_hero", "action", func_OnAnimationEnd );
  AnimPlay( "anm_map_spawn_hero_dummy", "dummy", function () 

    private.Hero_ShowPopup( "str_int_popup_cloud_intro" );

  end );

  cmn.PlaySnd( "intro_hero" );

end;
--------------------------------------------------------------------------------
function private.Intro_Animate_Skip ( func_OnAnimEnd )

  local objects      = {};
  local section_name = private.Section_Current_GetName();

  for i = 1, 9 do

    table.insert( objects, { "anm_map_builds_icon_build_"..i, { animfunc = "corrupted", playing = true } } );

    local anm_action = "anm_map_builds_icon_build_"..i.."_action";
    local anm_dummy = "anm_map_builds_icon_build_"..i.."_dummy";

    ObjStopAnimate( "anm_map_builds_icon_build_"..i, "alp" );

    AnimStop( anm_action );
    AnimStop( anm_dummy );

    ObjDelete( anm_action );
    ObjDelete( anm_dummy );
  
  end;
  
  for i = 1, 11 do
    
    table.insert( objects, { "btn_map_point_stage_"..i,  { alp = 1, scale_x = 1 } } );
    
  end;

  AnimPlay( "anm_map_builds_icon_state_puzzle_1", "state0", "" );
  AnimPlay( "anm_map_point_enemy_reddragon_stage_110", "static", "" );

  ObjMultiSet( objects );

  private.Stages_Show_Instant( 1, 11, "common" );
  
  ObjSet( "obj_map_foreground_intro", { active = false, visible = false } );
  ObjAttach( "obj_map_foreground", private.this.body );
  ObjAttach( "obj_map_spawn", "hub_map_anims" );
  
  public.Focus( "obj_map_point_stage_1", { time = 0.3, scale = 1.0 } );

  public.HideObjectPopup();

  cmn.StopSnd( "intro" );
  cmn.StopSnd( "intro_hero" );

  private.Music_Start();

  AnimStop( "anm_map_spawn_hero" );
  AnimStop( "anm_map_spawn_hero_dummy" );
  
  ObjAnimate( private.this.marker, "alp", 0, 0,
  function ()
    func_OnAnimEnd();
  end,
  {
    0.0, 0, 0,
    0.3, 0, 1
  } );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Hero_ShowPanel ()

  v_hero.Hero_ShowPanel();

end;
--------------------------------------------------------------------------------
function private.Hero_ShowPopup ( string_id, stage_name, funcs )

  if ( stage_name ) then

    mediator.Notify(
      "View_Map_HeroPopupActivated",
      stage_name
    );

  end;

  public.ShowObjectPopup(
    "monolog",
    "anm_map_marker_icon",
    string_id,
    nil,
    { 0, -50 },
    funcs,
    true
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.LeftPanel_Show ( highlight_hearts )

  private.LeftPanel_Show( highlight_hearts );

end;
--------------------------------------------------------------------------------
function public.LeftPanel_Hide ( func_end )

  private.LeftPanel_Hide( func_end );

end;
--------------------------------------------------------------------------------
function private.LeftPanel_Show ( highlight_hearts )

  private.LeftPanel_ShowHearts( highlight_hearts );
  private.LeftPanel_TryShowSocial();
  private.LeftPanel_TryShowDailyquest();
  
  local anim_time = private.interface.anm_tme;
  local full_time = private.interface.bot_tme;

  int_panel_map_left.Show( anim_time, full_time );

end;
--------------------------------------------------------------------------------
function private.LeftPanel_Hide ( func_end )

  int_panel_map_left.Hide( private.interface.anm_tme, func_end );

end;
--------------------------------------------------------------------------------
function private.LeftPanel_ShowHearts ( highlight_hearts )

  local hearts_count = m_player.GetHeartsCount();

  local func_seconds = function ()

    return TimeGetEventRemainingTime( "tmr_hearts_recover" );

  end;

  int_panel_map_left.Hearts_Show(
    hearts_count,
    v_hearts.Show,
    func_seconds,
    highlight_hearts
  );

end;
--------------------------------------------------------------------------------
function private.LeftPanel_UpdateHeartsCount ( params )
  
  int_panel_map_left.Hearts_UpdateCount( params.count );

end;
--------------------------------------------------------------------------------
function private.LeftPanel_TryShowSocial ()
  
  v_social.TryShowPanel();
  
end;
--------------------------------------------------------------------------------
function private.LeftPanel_TryShowDailyquest ()
  
  v_dailyquest.Panel_Show();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.RightPanel_Show ()

  private.RightPanel_Show();

end;
--------------------------------------------------------------------------------
function public.RightPanel_Hide ( func_end )

  private.RightPanel_Hide( func_end );

end;
--------------------------------------------------------------------------------
function private.RightPanel_Show ()

  private.QuestList_TryShowPanel();
  --private.IngameFriends_TryShowPanel();
  private.Sale_TryShowPanel();
  
  local anim_time = private.interface.anm_tme;
  local full_time = private.interface.bot_tme;

  int_panel_map_right.Show( anim_time, full_time );

end;
--------------------------------------------------------------------------------
function private.RightPanel_Hide ( func_end )

  int_panel_map_right.Hide( private.interface.anm_tme, func_end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.DailyBonus_TryShowDialog ( func_close )

  v_dailybonus.Dialog_TryShow( func_close );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Worldmap_ShowDialog ( unlock )

  local section_name  = private.Section_Current_GetName();
  local sections_list = m_map.Section_GetDetailedList();
  
  local skin_name = m_skins.GetSkin();
  local skin_cont = m_skins.GetContent( skin_name );

  int_dialog_worldmap.Show(
    section_name,
    sections_list,
    { 
      name    = skin_name,
      content = skin_cont
    },
    function ( section_name ) 
  
      return v_client.TryDownloadContent( section_name );

    end,
    function ( destination, func_hide ) -- OnButtonClick

      if ( destination == private.Section_Current_GetNumber() ) then

        func_hide();

      else

        loading.Show( true,
        function ()
        
          public.Interface_Disable( true );
          func_hide();
          
          private.Section_Switch( destination, 
          function()

            loading.Hide( true );
            private.Update();

          end );
        
        end );

      end;
    
  end,
  unlock );

end;
--------------------------------------------------------------------------------
function private.Worldmap_ShowPanel ()

  local func_tap = function()

    private.Worldmap_ShowDialog();

  end;

  int_panel_map_bottom.Worlds_Show( func_tap );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.DialogNotifications_Show ( func_after )
  
  local push_reward = m_player.Playhaven_GetPushNotificationsReward();
  
  int_dialog_notifications.Show(
    push_reward,
    function()
      
      m_player.Playhaven_RegisterPushNotifications( push_reward );
      
    end,
    function()
      
      int_dialog_notifications.Hide( func_after );
      
    end
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Achievement_TryShowPanel ()

  v_hero.Achievement_TryShowPanel();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Gold_Buffer_Out ()

  private.Gold_Buffer_Out();

end;
--------------------------------------------------------------------------------
function private.Gold_ShowPanel ()

  local gold_count = m_player.GetGoldCount();
  local gold_panel = "obj_int_panel_map_gold";

  local func_show_popup = function( text_string )

    text_string = text_string or "str_int_panel_map_gold_popup_text";

    int.ShowPopupFlat(
      "str_int_panel_map_gold_popup_caption",
      text_string,
      gold_panel,
      { 0, -130 },
      true,
      true
    );

  end;
  
  local func_hide_popup = function( instant )

    int.HidePopupFlat( instant );

  end;
  
  local func_check_popup = function ()
    
    return ( int.Popup_Flat_AttachedTo() == gold_panel );
    
  end;
  
  int_panel_map_top.Gold_Show(
    gold_count,
    func_show_popup,
    func_hide_popup,
    func_check_popup
  );

end;
--------------------------------------------------------------------------------
--function private.Gold_HidePanel ()
--
--  int_panel_map_top.Gold_Hide();
--
--end;
--------------------------------------------------------------------------------
function private.Gold_UpdatePanel ( params )

  int_panel_map_top.Gold_Update( params.count, params.count + params.delta );

end;
--------------------------------------------------------------------------------
function private.Gold_Buffer_Out ()

  local buffer_data = m_player.Gold_GetBuffered();
  
  if ( buffer_data ) then

    private.Gold_UpdatePanel( buffer_data );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Resource_Buffer_Out ( resource )

  private.Resource_Buffer_Out( resource );
  
end;
--------------------------------------------------------------------------------
function private.Resource_ShowPanel ( resource, count )
  
  count = count or m_player.GetResourceCount( resource );
  
  local resource_panel = "obj_int_panel_map_"..resource;

  local func_show_popup = function( text_string )

    text_string = text_string or "str_int_panel_map_"..resource.."_popup_text";

    int.ShowPopupFlat(
      "str_int_panel_map_"..resource.."_popup_caption",
      text_string,
      resource_panel,
      { 0, -130 },
      true,
      true
    );

  end;
  
  local func_hide_popup = function( instant )

    int.HidePopupFlat( instant );

  end;
  
  local func_check_popup = function ()
    
    return ( int.Popup_Flat_AttachedTo() == resource_panel );
    
  end;
  
  int_panel_map_top.Resource_Show(
    resource,
    count,
    func_show_popup,
    func_hide_popup,
    func_check_popup
  );

end;
--------------------------------------------------------------------------------
function private.Resource_HidePanel ( resource )

  int_panel_map_top.Resource_Hide( resource );

end;
--------------------------------------------------------------------------------
function private.Resource_IsPanelActive ( resource )

  int_panel_map_top.IsActive( resource );

end;
--------------------------------------------------------------------------------
function private.Resource_UpdatePanel ( params )

  local resource    = params.name;
  local start_count = params.count;
  local stop_count  = params.count + params.delta;

  if ( not int_panel_map_top.IsActive( resource ) ) then

    private.Resource_ShowPanel( resource, start_count );

  end;

  local func_end;

  if ( int_panel_map_top.IsVisible()         ) and
     ( not int_panel_map_top.IsHighlighted() ) then
  
    for i = 1, 3 do

      int_panel_map_top.Resource_GrayIn( "resource"..i );

    end;
    
    func_end = function ()
    
      for i = 1, 3 do

        int_panel_map_top.Resource_GrayOut( "resource"..i );

      end;
    
    end;
  
  end;
  
  int_panel_map_top.Resource_Update(
    resource,
    start_count,
    stop_count,
    func_end
  );

end;
--------------------------------------------------------------------------------
function private.Resource_Buffer_Out ( resource )
  
  local buffer_data = m_player.Resource_GetBuffered( resource );
  
  if ( buffer_data ) then

    private.Resource_UpdatePanel( buffer_data );

  end;

end;
--------------------------------------------------------------------------------
function private.Resource_Reload ()
  
  local resources_list = { "resource1", "resource2", "resource3" };
  
  for r = 1, #resources_list do
    
    local resource = resources_list[ r ];
    local count    = m_player.GetResourceCount( resource );
    
    if ( count > 0 ) then
      
      private.Resource_UpdatePanel( 
      { 
        name  = resource, 
        count = 0, 
        delta = count
      } );
      
    else
    
      private.Resource_HidePanel( resource );
      
    end;
    
  end;

end;   
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.BottomPanel_Show ( func_end )

  int_panel_map_bottom.Show( func_end );

end;
--------------------------------------------------------------------------------
function private.BottomPanel_Hide ( func_end )

  int_panel_map_bottom.Hide( func_end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.TopPanel_Show ( lock, highlight_resource )

  private.TopPanel_Show( lock, highlight_resource );

end;
--------------------------------------------------------------------------------
function public.TopPanel_Hide ( unlock, func_exec )

  private.TopPanel_Hide( unlock, func_exec );

end;
--------------------------------------------------------------------------------
function public.TopPanel_Lock ()

  private.TopPanel_Lock();

end;
--------------------------------------------------------------------------------
function public.TopPanel_Unlock ()

  private.TopPanel_Unlock();

end;
--------------------------------------------------------------------------------
function public.TopPanel_IsVisible ()
  
  return private.TopPanel_IsVisible();
  
end;
--------------------------------------------------------------------------------
function public.TopPanel_IsLocked ()
  
  return private.TopPanel_IsLocked();
  
end;
--------------------------------------------------------------------------------
function private.TopPanel_Show ( lock, highlight_resource )

  if not ( private.TopPanel_IsVisible() ) then

    if ( highlight_resource ) then
      
      int_panel_map_top.Highlight( highlight_resource );
      
    end;
    
    int_panel_map_top.Show();

    if ( lock ) then

      private.TopPanel_Lock();

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.TopPanel_Hide ( unlock, func_exec )

  if ( unlock ) then

    private.TopPanel_Unlock();

  end;

  if  ( not private.TopPanel_IsLocked() ) and
      ( private.TopPanel_IsVisible()    ) then

    int_panel_map_top.Hide( func_exec );

  end;

end;
--------------------------------------------------------------------------------
function private.TopPanel_Lock ()

  int_panel_map_top.Lock( true );

end;
--------------------------------------------------------------------------------
function private.TopPanel_Unlock ()

  int_panel_map_top.Lock( false );

end;
--------------------------------------------------------------------------------
function private.TopPanel_IsVisible ()
  
  return int_panel_map_top.IsVisible();
  
end;
--------------------------------------------------------------------------------
function private.TopPanel_IsLocked ()
  
  return int_panel_map_top.IsLocked();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Settings_ShowPanel ()

  local func_tap = function ()
    
    v_settings.Dialog_Show();

  end;

  int_panel_map_bottom.Options_Show( func_tap );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Market_Update ()

  v_shop.Market_Update( private.Market_SetOnMap, private.Market_SetTap );

end;
--------------------------------------------------------------------------------
function private.Market_SetOnMap ( unlocked, built )

  local objects = {};

  if ( unlocked ) and not ( built ) then

    table.insert( objects, { "obj_map_builds_market_sign",
    {
      active  = true,
      visible = true } } );
 
  else

    table.insert( objects, { "obj_map_builds_market_sign",
    {
      active  = false,
      visible = false } } );

  end;

  if ( built ) then

    table.insert( objects, { "anm_map_builds_market",
    {
      animfunc = "static",
      playing  = true } } );

  else

    table.insert( objects, { "anm_map_builds_market",
    {
      animfunc = "base",
      playing  = true } } );

  end;

  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------
function private.Market_Dialog_Build_Show ()

  local gold_count = m_market.Settings( "build" ).gold;

  local func_accept = function()
  
    v_shop.Market_TryBuild(
      gold_count,
      private.Market_SetOnMap,
      private.Market_SetTap
    );

  end;

  local func_start = function ()

    local func_top_panel_show = function ()

      if ( gold_count > 0 ) then
    
        int_panel_map_top.Highlight( "gold" );
        private.TopPanel_Show();
      
      end;

    end;

    local func_top_panel_hide = function ()

      if ( gold_count > 0 ) then
    
        private.TopPanel_Hide();
      
      end;

    end;

    int_dialog_map_build_market.Show(
      gold_count,
      func_accept,
      private.Market_Dialog_Build_Hide,
      func_top_panel_show,
      func_top_panel_hide
    );

  end;

  int_window.Show( "int_dialog_map_build_market", func_start );

  cmn.PlaySnd( "bottom_button" );

end;
--------------------------------------------------------------------------------
function private.Market_Dialog_Build_Hide ()

  int_dialog_map_build_market.Hide( function () 

    int_window.Hide( "int_dialog_map_build_market" );

  end );

end;
--------------------------------------------------------------------------------
function private.Market_SetTap ( action ) -- "popup", "build" or "open"

  if ( action == "popup" ) then

    common.TapSet( "obj_map_builds_market",
    {      
    func_true = function ( sender, skip_focus )

        public.ShowObjectPopup(
          "monolog",
          "obj_map_builds_market",
          "str_int_popup_cloud_builds_market",
          nil,
          nil,
          nil,
          skip_focus
        );

      end
    } );

  end;

  if ( action == "build" ) then

    common.TapSet( "obj_map_builds_market",
    {
      func_true = function()

        private.Market_Dialog_Build_Show();

      end
    } );

  end;

  if ( action == "open" ) then

    common.TapSet( "obj_map_builds_market",
    {      
      func_true = function ()

        private.TopPanel_Lock();

        v_shop.Show( "market" );

      end
    } );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Shop_ShowPanel ()

  local func_tap = function ( tab )

    v_shop.Show( tab );
    cmn.PlaySnd( "bottom_button" );

  end;

  int_panel_map_bottom.Shop_Show( func_tap );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Fireflies_TryShowPanel ()
  
  v_fireflies.Panel_TryShow();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.QuestList_Init ()
  
  v_quests.Questlist.SetHandlers( 
    function () public.Interface_Disable( true ) end, 
    function () public.Interface_Enable(  true ) end
  );
  
end;
--------------------------------------------------------------------------------
function private.QuestList_TryShowPanel ()

  v_quests.Panel.Show();
  
end;
--------------------------------------------------------------------------------
function private.QuestList_TryHidePanel ()
  
  v_quests.Panel.Hide();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.IngameFriends_TryShowPanel ()

  v_social.Panel.Show( "friends", private.interface.anm_tme );
  
end;
--------------------------------------------------------------------------------
function private.IngameFriends_TryHidePanel ()
  
  v_quests.Panel.Hide( "friends" );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Sale_TryShowPanel ()

  v_shop.Panel.Show( "starterpack" );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.PreOpen ( func_after )

  mediator.Subscribe( "Interface_Window_WindowOpened", private.OnWindowOpen );
  mediator.Subscribe( "Interface_Window_NoWindows",    private.OnWindowsClose );
  
  -- PreOpen end callback
  local function func_preopened ()
  
    if ( private.first_load ) then
  
      private.first_load = false;
      private.Marker_SetAtLastPoint();
    
    end;

    if ( m_map.Section_NeedIntro() ) then

      private.Intro_Prepare();

    end;

    if ( func_after ) then func_after(); end;
  
  end;

  if ( private.first_load ) then

    private.Section_Switch( private.Section_Current_GetNumber(), func_preopened );

  else

    func_preopened();

  end;

end;
--------------------------------------------------------------------------------
function private.Open ( func_after )
  
  cmn.StopSnd( "assets/content/hero/aud_hero_win_ride" );
  cmn.StopSnd( "level_completed" );

  if not m_map.Section_NeedIntro() and m_tutorial.IsCompleted( "intro" ) then

    private.Music_Start();

  end;
  
  private.opened = true;

  private.Update();
    
  private.Resource_ShowPanel( "resource1" );
  private.Resource_ShowPanel( "resource2" );
  private.Resource_ShowPanel( "resource3" );

  private.Gold_ShowPanel();

  private.Settings_ShowPanel();
  private.Hero_ShowPanel();
  private.Shop_ShowPanel();
  private.Fireflies_TryShowPanel();
  private.Worldmap_ShowPanel();

  common.Playhaven_LogPlacement( "map_open" );
  wiz.ChartBoost.LocationHomeScreen();
  
  common.LocalNotifications_SetData( "comeback_level", nil );

  mediator.Notify( "View_Map_MapOpened" );
  
  if ( func_after ) then func_after() end;
  
end;
--------------------------------------------------------------------------------
function private.Close ( func_after )
  
  mediator.Unsubscribe(
    "Interface_Window_WindowOpened",
    private.OnWindowOpen
  );

  mediator.Unsubscribe(
    "Interface_Window_NoWindows",
    private.OnWindowsClose
  );

  private.Interface_Off();

  private.opened = false;
  
  if ( func_after ) then

    func_after();

  end;

end;
--------------------------------------------------------------------------------
function private.AfterClose ( func_after )

  private.Music_Stop();

  mediator.Notify( "View_Map_MapClosed" );

  private.Reward_Collect();
  
  if ( func_after ) then

    func_after();

  end;

end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.OnWindowOpen ( window_name )

  private.Interface_Hide();

end;
--------------------------------------------------------------------------------
function private.OnWindowsClose ()

  if ( private.opened ) and not ( private.intro_active ) then --optional expression

    private.Interface_Show( private.TopPanel_Unlock );

  end;

end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function public.Section_Current ()

  return private.Section_Current_GetName();

end;
--------------------------------------------------------------------------------
function public.Section_Current_GetNumber ()

  return private.Section_Current_GetNumber();

end;
--------------------------------------------------------------------------------
function private.Section_Current_SetNumber ( sec_num )

  private.section_current = sec_num;
  
end;
--------------------------------------------------------------------------------
function private.Section_Current_GetNumber ()

  return private.section_current or private.Section_LastNumber();
  
end;
--------------------------------------------------------------------------------
function private.Section_Current_GetName ()

  return "section_" .. private.Section_Current_GetNumber();

end;
--------------------------------------------------------------------------------
function private.Section_LastNumber ()
  
  local last_section_name   = private.Section_LastName();
  local last_section_number = string.match( last_section_name, "%d+" );

  return tonumber( last_section_number );

end;
--------------------------------------------------------------------------------
function private.Section_LastName ()

  local last_stage_name   = m_map.Stage_Last_Name();
  local last_section_name = m_map.Section_GetByStage( last_stage_name );

  return last_section_name;

end;
--------------------------------------------------------------------------------
function public.Section_IsCurrentLoaded ()

  return private.section_current;
  
end;
--------------------------------------------------------------------------------
function private.Section_IsEnabled ( num )
  
  local query =
  {
    query      = "find",
    group      = "player",
    collection = "sections",
    indice     = "name",
    criteria   = { like = "section_"..num }
  }
  
  return m_storage.Execute( query ).result.enabled or false;
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.Section_Switch ( destination, func_after_switch )

  if not ( m_content.IsPacketReady( private.Section_Current_GetName() ) ) then

    destination = destination - 1;

  end;

  local function func_switch_finished ()
    
    mediator.Notify( "View_Map_Section_Switched", destination );
    
    if ( m_map.Section_NeedIntro() ) then

      private.Intro_Prepare();

    end;
    
    if ( func_after_switch ) then func_after_switch(); end;
  
  end;

  -- Do not switch to not enabled sections  
  local section_enabled = private.Section_IsEnabled( destination );

  if not ( section_enabled ) then

    private.open_map_deferred = false;
    
    if ( func_after_switch ) then func_after_switch(); end;
    return;

  end;
  
  -- Do nothing if section already loaded and equal with destination
  local prew_section_loaded = public.Section_IsCurrentLoaded();
  local prew_section_number = private.Section_Current_GetNumber();
  local prew_section_name   = private.Section_Current_GetName();
  
  if ( prew_section_loaded ) and ( prew_section_number == destination ) then
    
    private.open_map_deferred = false;
    
    func_switch_finished();
    return;
    
  end;
  
  private.open_map_deferred = true;
  
  -- Detach old objects, if any section already loaded
  if ( private.sections[ prew_section_name ] ) then
    
    ObjDetach( "anm_map_section_"    .. prew_section_number );
    ObjDetach( "obj_map_anims_"      .. prew_section_number );
    
  end;

  -- Update current section number and load section
  private.Section_Current_SetNumber( destination );
  
  local section_number = private.Section_Current_GetNumber();
  local section_name   = private.Section_Current_GetName();

  private.Section_LoadStrings( section_number );
  m_player.Section_SetCurrent( section_number );
  
  m_content.Section_SetCurrent( section_name );
  m_map.Section_SetCurrent( section_name );
  
  private.Resource_Reload();
  
  -- private.Section_LoadCurrent using timer for processing long cycles,
  -- all next operations passed as callbacks (prevention of EventLoop locking)
  
  local function func_marker_transfer ( func_after )

    if ( prew_section_number ~= section_number ) then

      private.Marker_Transfer( prew_section_number, section_number );
      
    end;
    
    func_after();
  
  end;
  
  common.RunCallbackList( { private.Section_LoadCurrent,
                            func_marker_transfer,
                            func_switch_finished        } );
  
end;
--------------------------------------------------------------------------------
function private.Section_LoadStrings ( section_number )
  
  private.strings_loaded = private.strings_loaded or {};
  
  for s = 1, section_number do
    if ( not private.strings_loaded[ s ] ) then
      private.strings_loaded[ s ] = true;
      ne.strings.Load( "assets/content/section_"..s.."/strings_section_"..s );
    end;
  end;
  
end;
--------------------------------------------------------------------------------
function private.Section_LoadCurrent ( func_after_load )
  
  local section_number = private.Section_Current_GetNumber();
  local section_name   = private.Section_Current_GetName();

  m_map.Section_Next_Enable(); -- WUT?
  
  local function func_load_ground ( func_after )
  
    ObjSet( "spr_map_background",
    {
      res = "assets/content/section_"..section_number.."/background",
      pos_x = 0.5 * ( private.map_w ),
      pos_y = - 0.5 * ( private.map_h )
    } );

    mediator.Notify( "View_Map_GroundLoaded", section_name );
    
    func_after();
    
  end;
  
  common.RunCallbackList( { private.Section_CreateObjects, 
                            func_load_ground,
                            func_after_load               } );
  
end;
--------------------------------------------------------------------------------
function private.Section_CreateObjects ( func_after )
  
  local section_name = private.Section_Current_GetName();

  -- Just reattach animation if section already created
  if ( private.sections[ section_name ] ) then

    -- NOTE: Attach map and wait for object placing
    ObjAttach( "anm_map_"..section_name, private.this.body );
    common.Defer( "map_attach_wait", 0.1, func_after );
    return;

  end;

  private.sections[ section_name ] = true;

  local function func_attach_map ()

    ObjSet( "anm_map_"..section_name, {
      res      = "assets/content/"..section_name.."/anm_map_"..section_name,
      animfunc = section_name,
      playing  = true,
      pos_x    =  private.offset,
      pos_y    = -private.offset,
      pos_z    = 1000
    } );

    ObjAttach( "anm_map_"..section_name, private.this.body );

  end;
  
  local function func_create_other ( func_after )

    local creation_funcs =
    {
      private.Portal_Show,
      private.Portal_Next_SetTap,
      private.PortalSign_Create,
      func_attach_map,
      private.QuestList_Init,
      private.Marker_SetSkin
    };
    
    local function func_exec ( func, progress )
      func();
      loading.SetProgressBar( progress, 0.95, 1.0 );
    end;

    common.ProcessArrayByTimer( creation_funcs, func_exec, func_after );
    
  end;
  
  common.RunCallbackList( {
                            private.Create_Environment, -- 00 - 10%
                            private.Create_Stages,      -- 10 - 50%
                            private.Create_Builds,      -- 50 - 70%
                            private.Create_Mines,       -- 70 - 75%
                            private.Create_Fights,      -- 75 - 80%
                            private.Create_Characters,  -- 80 - 85%
                            private.Create_Spells,      -- 85 - 90%
                            private.Create_Chests,      -- 90 - 95%
                            func_create_other,          -- 95 - 100%
                            func_after
                          } );

end;
--------------------------------------------------------------------------------
function private.Create_Environment ( func_after )

  local section_name     = private.Section_Current_GetName();
  local environment_list = {};

  -- JK: 5 environment animations per section
  for z = 1, 5 do

    table.insert( environment_list, "environment_"..z );

  end;

  -- Create and attach environment animations
  local function func_create ( env_name, progress )

    local map_name = "anm_map_"..section_name;
    local anm_name = map_name.."_"..env_name;
    local res_name = "assets/content/"..section_name.."/map/anm_"..env_name;

    m_object.Create(
      anm_name, 
      "anim", 
      map_name,
      {
       res      = res_name,
       animfunc = "static",
       playing  = true,
       anim_tag = env_name,
       input    = false
      }
    );
    
    loading.SetProgressBar( progress, 0.0, 0.1 );
  
  end;
  
  common.ProcessArrayByTimer( environment_list, func_create, func_after );

end;
--------------------------------------------------------------------------------
function private.Create_Stages ( func_after )
  
  local section_name = private.Section_Current_GetName();

  local query =
  {   
      query      = "find_all",
      group      = "static",
      collection = "stages",
      indice     = "section",
      criteria   = { like = section_name }
  };

  local stages_list = m_storage.Execute( query ).result;
  local multiset    = {};
  
  local function func_create_stage ( stage, progress )
  
    private.Stages_PointCreate( stage.name, section_name, multiset );
    loading.SetProgressBar( progress, 0.1, 0.5 );
  
  end;
  
  common.ProcessArrayByTimer(
    stages_list,
    func_create_stage,
    function ()
    
      m_object.Multiset( multiset );
      func_after();
   
    end
  );

end;
--------------------------------------------------------------------------------
function private.Create_Builds ( func_after )

  local section_name = private.Section_Current_GetName();
  local builds_list  = m_map.GetBuildsList( section_name );

  -- Update build_list by saved data
  for i = 1, #builds_list do
    m_builds.AppendSavedData( builds_list[ i ] );
  end;

  -- Builds creation funcs
  local function func_create_build ( build, progress )

    v_builds.Create( build );
    loading.SetProgressBar( progress, 0.5, 0.55 );

  end;

  local function func_create_builds ( func_next )

    common.ProcessArrayByTimer( builds_list, func_create_build, func_next );

  end;

  -- Update stages by builds
  local function func_set_zone ( build, progress )

    private.Stages_SetByBuild( build );
    loading.SetProgressBar( progress, 0.55, 0.7 );

  end;

  local function func_update_stages ( func_next )

    common.ProcessArrayByTimer( builds_list, func_set_zone, func_next );

  end;

  -- Run sequence
  common.RunCallbackList( { func_create_builds,
                            func_update_stages,
                            func_after } );
  
end;
--------------------------------------------------------------------------------
function private.Create_Mines ( func_after )

  local section_name = private.Section_Current_GetName();
  local mines_list   = m_map.GetMinesList( section_name );
  
  local function func_create_mine ( mine, progress )

    v_mines.Create( mine );
    loading.SetProgressBar( progress, 0.7, 0.75 );

  end;
  
  common.ProcessArrayByTimer( mines_list, func_create_mine, func_after );

end;
--------------------------------------------------------------------------------
function private.Create_Fights ( func_after )

  local section_name = private.Section_Current_GetName();
  local fights_list  = m_map.GetFightsList( section_name );

  local function func_create_fight ( fight_data, progress )

    v_fights.Create( fight_data, section_name );
    loading.SetProgressBar( progress, 0.75, 0.8 );

  end;
  
  common.ProcessArrayByTimer( fights_list, func_create_fight, func_after );

end;
--------------------------------------------------------------------------------
function private.Create_Characters ( func_after )

  local section_name    = private.Section_Current_GetName();
  local characters_list = m_map.GetCharactersList( section_name );

  local function func_char_create ( character, progress )

    v_characters.Create( character.name, section_name );
    loading.SetProgressBar( progress, 0.8, 0.85 );

  end;
  
  common.ProcessArrayByTimer( characters_list, func_char_create, func_after );

end;
--------------------------------------------------------------------------------
function private.Create_Spells ( func_after )

  local section_name = private.Section_Current_GetName();
  local spells_list  = m_map.GetSpellsList( section_name );

  local function func_spell_create ( spell, progress )

    v_spells.Create( spell.name, section_name );
    loading.SetProgressBar( progress, 0.85, 0.9 );

  end;
  
  common.ProcessArrayByTimer( spells_list, func_spell_create, func_after );

end;
--------------------------------------------------------------------------------
function private.Create_Chests ( func_after )

  local section_name = private.Section_Current_GetName();
  local chest_list   = m_chests.List_OnSection( section_name );

  local function func_create_chest ( chest, progress )
  
    v_chests.Create( chest.name, section_name );
    loading.SetProgressBar( progress, 0.9, 0.95 );

  end;

  common.ProcessArrayByTimer( chest_list, func_create_chest, func_after );
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function public.Marker_CurrentPoint ()

  return private.marker_point_current;

end;
--------------------------------------------------------------------------------
function private.Marker_SetSkin ( skin_name )
  
  skin_name     = skin_name or m_skins.GetSkin();
  local content = m_skins.GetContent( skin_name );

  m_object.Set(
    "anm_map_marker_icon_skin",
    {
      res      = "assets/content/"..content.."/anm_skin_"..skin_name,
      animfunc = "marker",
      playing  = true
    }
  );

end;
--------------------------------------------------------------------------------
function private.Marker_Transfer ( from_screen, to_screen )

  local section_name = "section_"..to_screen;

  local marker_at_stage;
  local marker_is_visible = true;

  if ( from_screen <= to_screen ) then

    local current_stage_name   = m_map.Stage_Last_Name();
    local check_section_name   = m_map.Section_GetByStage( current_stage_name );
    local check_section_number = tonumber( string.match( check_section_name, "%d+" ) );

    if ( check_section_number == to_screen ) then
      
      marker_at_stage = current_stage_name;

    else

      marker_at_stage = m_map.Section_Stage_First( section_name );
      m_map.Stage_Last_Set( marker_at_stage, section_name );

    end;

  elseif ( from_screen > to_screen ) then

    marker_at_stage   = m_map.Section_Stage_Last( section_name );
    marker_is_visible = false;

  end;
  
  private.Marker_SetAtPoint( marker_at_stage );

  m_object.Set( private.this.marker, { visible = marker_is_visible } );

end;
--------------------------------------------------------------------------------
function private.Marker_SetAtLastPoint ()

  local last_stage   = m_map.Stage_Last_Name();
  local last_section = m_map.Section_GetByStage( last_stage );
  
  local section_name = private.Section_Current_GetName();

  if not ( section_name == last_section ) then

    return;

  end;

  private.Marker_SetAtPoint( last_stage );

end;
--------------------------------------------------------------------------------
function private.Marker_Target ( stage_name )

  local target_point = "obj_map_point_"..stage_name;
  local stage_info   = m_map.GetStageInfo( stage_name );

  if ( stage_info.completed ) then

    if not ( stage_info.realtype == "misc" ) or ( stage_info.realtype == "misc" and stage_info.type ~= "fight" ) then

      stage_name   = m_map.Stage_Next_Name( stage_name );
      target_point = "obj_map_point_"..stage_name;

    end;

  end;

  return target_point, stage_name;

end;
--------------------------------------------------------------------------------
function private.Marker_SetAtPoint ( stage_name )

  local target_point;
  local multiset = {};
  
  target_point, stage_name     = private.Marker_Target( stage_name );
  private.marker_point_current = stage_name;
  
  if ( m_tutorial.IsCompleted( "intro" ) ) then

    public.Focus( target_point, { instant = true } );

  end;
  
  local point_params = private.Object_Position( target_point );
  
  int.Input_On( target_point, multiset );
  
  m_object.Set(
    private.this.marker,
    { 
      pos_x = point_params[ 1 ] + private.offset,
      pos_y = point_params[ 2 ] - private.offset 
    },
    multiset
  );
  
  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.Marker_MoveToPoint ( stage_name, execute_tap, no_focus, func_end )
  
  if ( not no_focus ) then

    public.Focus( private.Marker_Target( stage_name ), { time = 0.5 } );

  end;

  private.marker_point_current  = stage_name;
  private.marker_target_execute = execute_tap;
  
  local target_object = private.Marker_Target( stage_name );
  local target_params = private.Object_Position( target_object );
  
  local button = "btn_"..cmn.GetObjectName( target_object );
  local events = int.Control_EventsGet( button );
  
  local func_tap = function () end;
  if ( events ) then func_tap = events.func_on_click; end;
   
  ObjSet( target_object, { input = true } ); 
  
  private.marker_target_params =
  {
    pos_x    = target_params[ 1 ],
    pos_y    = target_params[ 2 ],
    func_tap = func_tap, 
    name     = target_object
  };

  private.Marker_MoveToPoint_Animate( func_end );

end;
--------------------------------------------------------------------------------
function private.Marker_MoveToPoint_Animate ( func_end )

  local func_OnEnd = function ()

    private.Marker_MoveToPoint_Done( func_end );

  end;

  local marker_object = private.this.marker;

  ObjStopAnimate( marker_object, "pos_xy" );

  local marker_params = ObjGet( marker_object );
  local target_params = private.marker_target_params;
  
  ObjAnimate( marker_object, "pos_xy", 0, 0, func_OnEnd,
  {
    0.0, 3, marker_params.pos_x, marker_params.pos_y, 
    0.2, 3, marker_params.pos_x, marker_params.pos_y, 
    0.5, 3, target_params.pos_x + private.offset, target_params.pos_y - private.offset
  } );

end;
--------------------------------------------------------------------------------
function private.Marker_MoveToPoint_Done ( func_end )
  
  PartSysRestart( "pfx_map_marker_done" );

  cmn.PlaySnd( "open_new_level" );

  if ( func_end ) then

    func_end();

  end;

  if ( private.marker_target_execute ) then

    private.marker_target_params.func_tap();

  end;

end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.Point_Tap ( stage_name, skip_focus )

  local stage_info = m_map.GetStageInfo( stage_name );

  if ( not stage_info.enabled ) then
    
    return;

  end;

  if ( stage_name ~= private.marker_point_current ) and
     ( not stage_info.completed                   ) then
  
    private.Marker_MoveToPoint( stage_name, true );

    return;

  end;
  
  private.Interface_Input_Off();
  
  if ( not skip_focus ) then
  
    -- JK: map scale normalization
    public.Focus( "obj_map_point_"..stage_name,
      {
        scale      = 1.0,
        func_after = function ()
        
          v_stage.DialogStart_Show( stage_name, "stages" );
          
        end;
      }
    );
    
  else
 
    v_stage.DialogStart_Show( stage_name, "stages" );
    
  end;

end;
--------------------------------------------------------------------------------
function public.Point_UpdateState ( stage_name, noreward )

  private.Stages_PointUpdate( stage_name, noreward );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.TryExecuteQuestEvents ()
  
  local current_section_name = private.Section_Current_GetName();
  
  v_quests.Questlist.Update( current_section_name );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.DragStart ()

  common.TapBreak();
  
  int.HidePopupCloud();

  --[[
  local tLocPosMap = ObjGet( private.this.body );
  local tLocCursor = GetGameCursorPos();

  private.tPosObjMap = { tLocPosMap.pos_x,   tLocPosMap.pos_y };
  private.tPosCursor = { tLocCursor[ 1 ], tLocCursor[ 2 ] };
  ]]

end;
--------------------------------------------------------------------------------
function private.DragUpdate ()

  --[[
  local tLocCursor = GetGameCursorPos();
  local tDelta = {
    private.tPosCursor[ 1 ] - tLocCursor[ 1 ],
    private.tPosCursor[ 2 ] - tLocCursor[ 2 ]
  };

  private.tPosCursor = { tLocCursor[ 1 ], tLocCursor[ 2 ] };

  private.tPosObjMap = private.CheckPosition( { private.tPosObjMap[ 1 ] - tDelta[ 1 ], private.tPosObjMap[ 2 ] - tDelta[ 2 ] }, private.map_scale_current );

  ObjSet( private.this.body, { pos_x = private.tPosObjMap[ 1 ], pos_y = private.tPosObjMap[ 2 ] } );
  ]]

  --private.GroundUpdate();

end;
--------------------------------------------------------------------------------
function private.DragDrop ()

  private.map_scale_current = ObjGet( private.this.body ).scale_x;

  --ObjSet( "obj_map_drag", { pos_x = 0, pos_y = 0 } );

end;
--------------------------------------------------------------------------------
function private.CheckPosition ( position, scale )

  if ( position[ 1 ] > 0 ) then

    position[ 1 ] = 0;

  end;

  if ( position[ 1 ] < ( - private.scale_modifier * private.map_w * scale ) + private.app_w ) then

    position[ 1 ] = ( - private.scale_modifier * private.map_w * scale ) + private.app_w;

  end;

  local screen_height = private.scale_modifier * private.map_h;

  if ( position[ 2 ] > screen_height ) then

    position[ 2 ] = screen_height;

  end;

  local app_height = private.app_h;

  if ( position[ 2 ] < app_height ) then

    position[ 2 ] = app_height;

  end;
  
  return position;

end;
--------------------------------------------------------------------------------
function private.GroundUpdateProcess ( update_time, func_end )

  if ( update_time <= 0 ) then

    return;

  end;

  if ( func_end ) then

    func_end();

  end;

  update_time = update_time - 0.05;

  ObjSet( "tmr_map_ground_update",
  {
    playing = true,
    time = 0.05,
    endtrig = function ()
      private.GroundUpdateProcess( update_time, func_end );
    end
  } );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function public.AppearInsertToQueue ( obj_type, animation )

  if ( not private.appear_queue[ obj_type ] ) then
    private.appear_queue[ obj_type ] = {};
  end;

  table.insert( private.appear_queue[ obj_type ], animation );

end;
--------------------------------------------------------------------------------
function public.AppearClearQueue ( obj_type )

  if ( obj_type ) then
    private.appear_queue[ obj_type ] = {};
  end;

end;
--------------------------------------------------------------------------------
function public.AppearClearAllQueue ()

  private.appear_queue = {};

end;
--------------------------------------------------------------------------------
-- function ******* Map Scale ******* () end
--------------------------------------------------------------------------------
function private.ShowFocusPoint ( stage_name )

  local stage_info = m_map.GetStageInfo( stage_name );

  if ( stage_info.enabled ) then

    local focus_name = string.format( "fcs_map_point_%s", stage_name );

    local tme = 0.3;

    local alp = ObjGet( focus_name ).alp;

    ObjAnimate( focus_name, "alp", 0, 0, "", 
    {
      0.0, 2, alp, 
      tme, 2, 1
    } );

  end

end;
--------------------------------------------------------------------------------
function private.HideFocusPoint ( stage_name )

  local focus_name = string.format( "fcs_map_point_%s", stage_name );

  local tme = 0.3;

  local alp = ObjGet( focus_name ).alp;

  ObjAnimate( focus_name, "alp", 0, 0, "", 
  {
    0.0, 2, alp, 
    tme, 2, 0
  } );

end;
--------------------------------------------------------------------------------
function private.Object_Position ( object, relative )

  relative = relative or "anm_map_"..private.Section_Current_GetName();
  
  return int.Position( object, relative );
  
end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function public.Builds_UpdateStages ( build_data, func_anim, func_after )
  
  cmn.PlaySnd( "unspell_button" );
  
  local function func_end ()
  
    public.Interface_Enable( true );
    if ( func_after ) then func_after(); end;
  
  end;
  
  public.Interface_Disable( true );

  if ( build_data.type == "build" ) then

    private.Builds_TakeResources( build_data, function ()
    
      cmn.PlaySnd( "catch_glow" );
      if ( func_anim ) then
      
        func_anim( function ()
          private.Stages_UpdateByBuild( build_data, func_end );
        end );
      
      else
      
        private.Stages_UpdateByBuild( build_data, func_end );
        
      end;
      
    end );

  else

    private.Stages_UpdateByBuild( build_data, func_end );

  end;

end;
--------------------------------------------------------------------------------
function private.Builds_TakeResources ( build_data, func_end )

  local build_name = build_data.name;
  local price      = build_data.price;

  if ( not price ) then

    func_end();
    return;

  end;

  int.Lock( true, "Builds_TakeResources" );

  local build_pos = private.Object_Position( "obj_map_builds_"..build_name, "obj_map_anims" );

  local flying_objects = 0;

  for j = 1, #price do

    local max_objects = math.min( price[ j ].count, 15 );
    local current_obj = 0;
    local resource    = price[ j ].resource;

    local res_pos = private.Object_Position(
      "obj_int_panel_map_"..resource,
      "obj_map_anims"
    );

    for i = 1, max_objects do      

      current_obj = current_obj + 1;

      flying_objects = flying_objects + 1; 

      local obj_name = "obj_map_flying_resources_"..resource.."_"..current_obj;
      local anm_name = "anm_map_flying_resources_"..resource.."_"..current_obj;
      local res_name = "anm_map_flying_resources_icon_"..resource.."_"..current_obj;
      local pfx_name = "pfx_map_flying_resources_"..resource.."_"..current_obj;

      ObjClone( "obj_map_flying_resources", "_"..resource.."_"..current_obj, 1, "obj_map_anims" );

      ObjSet( obj_name, { pos_x = res_pos[ 1 ], pos_y = res_pos[ 2 ], pos_z = ( 10 + current_obj ) } );
      ObjSet( res_name, { animfunc = resource, playing = true } );

      local animfunc = math.random( 1, 8 );

      local resource_type = "resource";

      if ( string.find( resource, "gold" ) ) then

        resource_type = "gold";

      end;

      AnimPlay( anm_name, resource_type..animfunc, "" );

      local shift = math.random( 1, max_objects );
      local type  = math.random( 0, 2 );
      local delay = math.random( 1, max_objects ) / 100;

      local func_OnAnimationEnd = function ()

        ObjDelete( obj_name );

        flying_objects = flying_objects - 1;

        if ( flying_objects < 1 ) and ( func_end ) then

          int.Lock( false, "Builds_TakeResources" );
          func_end();

        end;

      end;

      ObjSet( pfx_name, { visible = true, active = true } );

      PartSysStart( pfx_name );

      ObjAnimate( obj_name, "pos_x", 0, 0, func_OnAnimationEnd, 
      {
        0.0, type, res_pos[ 1 ],
        0.5, type, res_pos[ 1 ] + 15 * shift,
        1.0 + delay, type, res_pos[ 1 ] + 15 * shift,
        1.5 + delay, type, build_pos[ 1 ]
      } );

      shift = math.random( 1, max_objects );
      type  = math.random( 0, 2 );
      
      ObjAnimate( obj_name, "pos_y", 0, 0, "", 
      {
        0.0, type, res_pos[ 2 ],
        0.5, type, res_pos[ 2 ] + 150 + 15 * shift,
        1.0 + delay, type, res_pos[ 2 ] + 150 + 15 * shift,
        1.5 + delay, type, build_pos[ 2 ]
      } );

      ObjAnimate( obj_name, "alp", 0, 0, "", 
      {
        0.0, 0, 1,
        0.5, 0, 1,
        1.0 + delay, 0, 1,
        1.5 + delay, 1, 0
      } );

      ObjAnimate( obj_name, "scale_xy", 0, 0, "", 
      {
        0.0,         0, 1.0, 1.0,
        0.5,         0, 1.0, 1.0,
        1.0 + delay, 0, 1.0, 1.0,
        1.5 + delay, 1, 0.5, 0.5
      } );

    end;

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Stages_PointCreate ( stage_name, section_name, multiset )

  -- Create and attach base object
  local point_name = "obj_map_point_"..stage_name;
  
  ObjCreate( point_name, "obj" );
  m_object.Set( point_name, { anim_tag = stage_name }, multiset );
  
  m_object.Attach( point_name, "anm_map_"..section_name );
  m_object.Clone( "btn_map_point", point_name, "_"..stage_name, 1 );

  local funcs =
  {
    func_on_click = function ()
      private.Point_Tap( stage_name );
    end
  };

  if ( IsPC() ) then
    
    funcs.func_on_mouse_enter = function ()
      private.ShowFocusPoint( stage_name )
    end;
    
    funcs.func_on_mouse_leave = function ()
      private.HideFocusPoint( stage_name )
    end;
      
  end;
  
  int.Control_EventsSet( "btn_map_point_"..stage_name, funcs );

  -- Set text label
  if not ( string.match( stage_name, "bonus" ) ) then

    local number = private.Stages_RelativeNumber( stage_name );
    
    m_object.Set(
      "anm_map_point_label_"..stage_name,
      {
        number = tostring( number )
      },
      multiset
    );

  end;

end;
--------------------------------------------------------------------------------
function private.Stages_PointSet ( stage_name, hide_quest_reward )

  local obj_point = "btn_map_point_"..stage_name;
  local active    = {
    [ true  ] = { active  = true,  visible = true  },
    [ false ] = { active  = false, visible = false }
  };
  
  local set = {};

  table.insert( set, { obj_point,
  {
    active  = true,
    visible = true,
    input   = true
  } } );

  local stage_info = m_map.GetStageInfo( stage_name );

  local enabled   = stage_info.enabled or false;

  local ride      = stage_info.type     == "ride";
  local fight     = stage_info.type     == "fight";
  local realfight = stage_info.realtype == "fight";
  local realbonus = stage_info.realtype == "bonus";

  local state     = ( stage_info.state or 1 ) - 1;

  local completed = stage_info.noreward or false;
  local disabled  = stage_info.completed and realfight;
  
  -- Setup input
  table.insert( set, { obj_point, { input = enabled } } );

  -- Setup background
  local anm_bg = "anm_map_point_bg_"..stage_name;
  local bg_af  =
  {
    [ "default"   ] = "bg_default",
    [ "active"    ] = "bg_active",
    [ "completed" ] = "bg_completed",
    [ "disabled"  ] = "bg_disabled"
  };

  if ( disabled ) then

    table.insert( set, { anm_bg, { animfunc = bg_af.disabled } } );

  elseif ( completed ) then

    table.insert( set, { anm_bg, { animfunc = bg_af.completed } } );

  elseif ( fight and enabled ) then

    table.insert( set, { anm_bg, { animfunc = bg_af.active } } );

  elseif ( not enabled or ( state <= 0 ) ) then

    table.insert( set, { anm_bg, { animfunc = bg_af.default } } );

  else

    table.insert( set, { anm_bg, { animfunc = bg_af.active } } );

  end;
 
  -- Setup label
  local anm_label = "anm_map_point_label_"..stage_name;
  local label_af  =
  {
    [ "empty"  ] = "label_empty",
    [ "bonus"  ] = "label_bonus",
    [ "ride"   ] = "label_number",
    [ "fight"  ] = "label_fight"
  };

  if ( disabled ) then

    table.insert( set, { anm_label, { animfunc = label_af.fight } } );

  elseif not ( enabled ) then

    table.insert( set, { anm_label, { animfunc = label_af.empty } } );

  elseif ( realbonus ) then

    table.insert( set, { anm_label, { animfunc = label_af.bonus } } );

  else

    local stage_type = stage_info.type;
    table.insert( set, { anm_label, { animfunc = label_af[ stage_type ] } } );

  end;

  -- Setup state
  local anm_state = "anm_map_point_state_"..stage_name;
  local state_af  =
  {
    [ 0 ] = "state_0",
    [ 1 ] = "state_1",
    [ 2 ] = "state_2",
    [ 3 ] = "state_3"
  };

  if ( fight or not enabled ) then

    table.insert( set, { anm_state, { animfunc = state_af[ 0 ] } } );

  elseif ( completed ) then

    table.insert( set, { anm_state, { animfunc = state_af[ 3 ] } } );

  else

    table.insert( set, { anm_state, { animfunc = state_af[ state ] } } );

  end;

  -- Setup reward
  local obj_reward  = "obj_map_point_reward_"..stage_name;
  local anm_reward  = "anm_map_point_reward_"..stage_name;
  
  local reward = stage_info.reward;
  local hide_reward = hide_quest_reward and reward and reward.type == "quest";
  
  if ( stage_info.reward and not hide_reward ) then

    local spr_reward = "spr_map_point_reward_icon_"..stage_name;
    local resource = common.GetItemResource(
      stage_info.reward.type,
      stage_info.reward.name
    ).."_small";

    table.insert( set, { obj_reward, active[ true ] } );
    table.insert( set, { spr_reward, { res = resource } } );
    table.insert( set, { anm_reward, 
                       { animfunc = "static", playing  = true } } );

  else

    table.insert( set, { obj_reward, active[ false ] } );

  end;

  ObjMultiSet( set );

end;
--------------------------------------------------------------------------------
function private.Stages_PointUpdate ( stage_name, noreward, hide_quest_reward )

  if ( noreward ) then

    local function func_swap ()

      ObjAnimate( "btn_map_point_"..stage_name, "scale_x", 0, 0, "", 
      {
        0.0, 0, 0.0,
        0.3, 0, 1.0
      } );

      ObjAnimate( "anm_map_point_state_"..stage_name, "alp", 0, 0, "", 
      {
        0.0, 0, 0,
        0.3, 0, 0, 
        0.6, 0, 1
      } );

      ObjAnimate( "anm_map_point_state_"..stage_name, "scale_xy", 0, 0, "", 
      {
        0.0, 0, 1.5, 1.5, 
        0.3, 0, 1.5, 1.5, 
        0.6, 0, 1.0, 1.0
      } );

      private.Stages_PointSet( stage_name, hide_quest_reward );
    
    end;

    ObjAnimate( "btn_map_point_"..stage_name, "scale_x", 0, 0, func_swap, 
    {
      0.0, 0, 1.0,
      0.3, 0, 0.0
    } );

    return;
  
  end;
  
  if not ( ObjGet( "btn_map_point_"..stage_name ).visible ) then
    return;
  end;
  
  local rew_obj        = ObjGet( "obj_map_point_reward_"..stage_name );
  local animate_reward = not ( rew_obj and rew_obj.visible );

  private.Stages_PointSet( stage_name, hide_quest_reward );

  if ( animate_reward ) then

    AnimPlay(
      "anm_map_point_reward_"..stage_name,
      "appear", 
      function () 

        AnimPlay(
          "anm_map_point_reward_"..stage_name,
          "static",
          ""
        );

      end
    );

  end;

end;
--------------------------------------------------------------------------------
function private.Stages_PointEnable ( stage_name )

  local set = {};
  local enable = { active = true, visible = true, input = true };

  local anm_label = "anm_map_point_label_"..stage_name;

  table.insert( set, { "btn_map_point_"..stage_name, enable } );

  local stage_data = m_stage.GetStaticData( stage_name );

  if ( stage_data.realtype == "bonus" ) then

    table.insert( set, { anm_label, { animfunc = "label_bonus" } } );

  elseif ( stage_data.type == "ride" ) then

    table.insert( set, { anm_label, 
    {
      animfunc = "label_number",
      number   = tostring( private.Stages_RelativeNumber( stage_name ) )
    } } );

  end;

  ObjMultiSet( set );

  common.Animation_Fade_In( 0.15, anm_label );

end;
--------------------------------------------------------------------------------
function private.Stages_GetName ( stage_num, stage_type )

  local result = "stage_"..stage_num;
  if ( stage_type == "bonus" ) then 
    result = result.."_bonus";
  end;
  return result;

end;
--------------------------------------------------------------------------------
function private.Stages_RelativeNumber ( stage_name )

  local sectoin_current = private.Section_Current_GetNumber();
  
  return tonumber( string.match( stage_name, "%d+" ) ) - 110 * ( sectoin_current - 1 );

end;
--------------------------------------------------------------------------------
function private.Stages_UpdateByBuild ( build_data, func_after )

  local list = m_builds.GetUnlockList( build_data, true );

  if ( list and list.stages_next and #list.stages_next > 0 ) then

    private.AnimateZone(
      build_data.name, 
      list.stages_next,
      list.stages_type,
      function ()
      
        private.Stages_Show_Instant(
          list.stages_next[ 1 ],
          list.stages_next[ 2 ],
          list.stages_type
        );
        
        if ( func_after ) then func_after(); end;
      
      end
    );
    
  elseif ( func_after ) then
  
    func_after();

  end;  

end;
--------------------------------------------------------------------------------
function private.Stages_SetByBuild ( build_data )

  local list = m_builds.GetUnlockList( build_data, false );

  if ( list ) then

    if ( list.stages_previous and #list.stages_previous > 0 ) then

      private.Stages_Show_Instant( list.stages_previous[ 1 ],
                                   list.stages_previous[ 2 ] );

    end;

    if ( list.stages_next and #list.stages_next > 1 ) then

      private.Stages_Show_Instant( list.stages_next[ 1 ], 
                                   list.stages_next[ 2 ], 
                                   list.stages_type );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Stages_TrySetWorkerSign ( stage_name, objects )

  local worker_state = m_map.Stage_WorkerState( stage_name );

  if not ( worker_state ) then

    return;

  end;

  local stage_data = m_map.GetStageInfo( stage_name );

  if ( stage_data.state <= worker_state ) and not ( stage_data.noreward ) then

    table.insert( objects, { "obj_map_point_reward_"..stage_name,
    {
      active  = true,
      visible = true } } );
      
    table.insert( objects, { "anm_map_point_reward_"..stage_name,
    {
      animfunc = "static",
      playing  = true } } );
      
    table.insert( objects, { "spr_map_point_reward_icon_"..stage_name,
    {
      res = "assets/content/items/worker_small" } } );

  end;

end;
--------------------------------------------------------------------------------
function private.Stages_Show_Instant ( start_id, stop_id, stage_type )

  for i = start_id, stop_id do

    local stage_name = private.Stages_GetName( i, stage_type );
    private.Stages_PointSet( stage_name );

  end;

end;
--------------------------------------------------------------------------------
function private.Stages_Show_Cascade ( start_id, stop_id, func_end, stages_type )

  local stage_type = stages_type or "common";
  local stage_name = private.Stages_GetName( start_id, stage_type );
  
  local func_OnAnimationEnd = function ()

    if ( ObjGet( "obj_map_point_reward_"..stage_name ).visible ) then

      common.Animation_Bounce( "obj_map_point_reward_"..stage_name );

    end;

    -- ObjSet( "btn_map_point_"..stage_name, { input = true } );

    if ( start_id == stop_id ) then

      if ( func_end ) then func_end(); end;
      return;
      
    end;

    private.Stages_Show_Cascade( start_id + 1, stop_id, func_end, stages_type );

  end;

  local objects = {};

  private.Stages_TrySetWorkerSign( stage_name, objects );

  table.insert( objects, { "btn_map_point_"..stage_name,
  {
    active  = true,
    visible = true,
    alp     = 0 } } );

  ObjMultiSet( objects );

  private.Stages_PointUpdate( stage_name );

  ObjAnimate( "btn_map_point_"..stage_name, "alp", 0, 0, func_OnAnimationEnd,
  {
    0.0, 0, 0,
    0.2, 0, 1
  } );

  ObjAnimate( "btn_map_point_"..stage_name, "scale_x", 0, 0, "",
  {
    0.0, 0, 0,
    0.2, 0, 1
  } );

  local zone_id  = math.floor( start_id / 11 );
  local sound_id = start_id - zone_id * 11;

  if ( sound_id == 0 ) then

    sound_id = 11;

  end;

  cmn.PlaySnd( "level_app"..sound_id );

end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.Fights_Enable ( stage_name )

  local stage_data = m_fights.Data( stage_name );

  if ( stage_data.type == "fight" ) then

    if ( stage_data.visible ) and ( stage_data.enabled ) then

      if ( stage_data.realtype == "misc" ) then

        if not ( ObjGet( "obj_map_enemy_"..stage_data.enemy.."_"..stage_name ) ) then

          ObjClone(
            "obj_map_enemy_"..stage_data.enemy,
            "_"..stage_name,
            1,
            "obj_map_enemy_"..stage_name
          );

          local obj_params = ObjGet( "obj_map_enemy_"..stage_data.enemy.."_"..fight_name );

          ObjSet( "obj_map_enemy_"..stage_data.enemy.."_"..fight_name,
          {
            input       = false,
            inputrect_x = 0.5 * obj_params.inputrect_x,
            inputrect_y = 0.5 * obj_params.inputrect_y,
            inputrect_w = 0.5 * obj_params.inputrect_w,
            inputrect_h = 0.5 * obj_params.inputrect_h
          } );

        end;

      end;

      v_fights.Fight_Enable_Animate( stage_name, stage_data );

      return true;

    end;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Fights_Disable ( stage_name )

  local fight_data = m_fights.Data( stage_name );
  local completed  = fight_data.completed or false;

  local fight_done      = ( fight_data.realtype == "fight" ) and completed;

  local misc_fight_done = ( fight_data.realtype == "misc"  ) and 
                          ( fight_data.type     == "ride"  ) and not completed;

  if ( fight_done or misc_fight_done ) then

    v_fights.Fight_Disable_Animate( stage_name, fight_data );

  end;

end;
--------------------------------------------------------------------------------
--function ... ()
--------------------------------------------------------------------------------
function private.PortalSign_Create ()

  local section = private.Section_Current_GetName();

  local hub_obj = "anm_map_portal_next";
  local suffix  = "_"..section;
  local parent  = "obj_map_portal_"..section.."_sign";

  ObjClone( hub_obj, suffix, true, parent );
  
  m_object.Set( hub_obj..suffix, { label = "str_map_portal_next_sign" } );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.Portal_Unlock ( func_after )
  
  local section_name = private.Section_Current_GetName();

  int.Lock( true );
  ObjAnimate( private.this.marker, "alp", 0, 0, function()

    m_object.Hide( private.this.marker );

    AnimPlay(
      "anm_map_portal_"..section_name.."_next",
      "ready_start",
      function ()

        AnimPlay(
          "anm_map_portal_"..section_name.."_next",
          "ready",
          ""
        );

        -- JK: workaround for section 3
        if ( section_name == "section_3" ) then

          ObjSet( "btn_map_portal_"..section_name.."_next",
          {
            inputrect_h = 240,
            inputrect_w = 240,
            inputrect_y = -200,
            inputrect_x = -120
          } );

        end;

      end
    );

    public.ShowObjectPopup(
      "hint",
      "obj_map_portal_"..section_name.."_next_indicator",
      "str_int_popup_portal_"..section_name.."_next_hero",
      nil,
      nil,
      { 
        func_preclose = function ()
        
          int.Lock( false );
          if ( func_after ) then func_after(); end;
        
        end
      }
    ); 

  end,
  {
    0.0, 0, 1,
    1.0, 0, 0
  } );  

end;
--------------------------------------------------------------------------------
function private.Portal_Show ()

  local status       = m_map.Section_Next_Status();
  local section_name = private.Section_Current_GetName();

  local multiset = {};
  
  local portal_icon = m_map.Portal_GetName( section_name );
  
  if ( portal_icon ) then
    m_object.Set(
      "anm_map_portal_"..section_name.."_next",
      {
        res = "assets/content/"..section_name.."/map/anm_"..portal_icon
      },
      multiset
    );
  else
    DbgTrace( "[ WARNING ] Portal icon is not set!" );
  end;
  
  if ( status == "available" ) or
     ( status == "enabled"   ) then

     m_object.Hide( private.this.marker, multiset );

    if ( m_map.Section_NeedOutro( section_name ) ) then

      m_object.Set(
        "anm_map_portal_"..section_name.."_next",
        {
          animfunc = "ready",
          playing  = true
        },
        multiset
      );

      -- JK: workaround for section 3
      if ( section_name == "section_3" ) then

        m_object.Set(
          "btn_map_portal_"..section_name.."_next",
          {
            inputrect_h = 240,
            inputrect_w = 240,
            inputrect_y = -200,
            inputrect_x = -120
          },
          multiset
        );

      end;

    else

      m_object.Disable(
        "btn_map_portal_"..section_name.."_next",
        false,
        multiset
      );

    end;

  elseif ( status == "prepare" ) then

    m_object.Set(
      "anm_map_portal_"..section_name.."_next",
      {
        animfunc = "prepare",
        playing  = true
      },
      multiset
    );

  else

    m_object.Set(
      "anm_map_portal_"..section_name.."_next",
      {
        animfunc = "normal",
        playing  = true
      },
      multiset
    );

    m_object.Set(
      private.this.marker,
      {
        active  = true,
        visible = true
      },
      multiset
    );

  end;
  
  m_object.Multiset( multiset );

end;
--------------------------------------------------------------------------------
function private.Portal_ShowComingSoon ()
  
  local section_name = private.Section_Current_GetName();

  local func_OnShow = function ()

    local func_accept = function ()

      int_dialog_map_comingsoon.Hide( function () 

        int_window.Hide( "int_dialog_map_comingsoon" );

      end );

    end;

    int_dialog_map_comingsoon.Show( section_name, func_accept );

  end;

  int_window.Show( "int_dialog_map_comingsoon", func_OnShow );

end;
--------------------------------------------------------------------------------
function private.Portal_ShowWorldMap ()

  private.Worldmap_ShowDialog( true );

end;
--------------------------------------------------------------------------------
function private.Portal_Next_Tap ( status )
  
  -- JK: debug coming soon
  --private.Portal_ShowComingSoon();
  
  local section_name = private.Section_Current_GetName();

  if ( status == "available" ) then

    AnimPlay( "anm_map_portal_"..section_name.."_next", "ready_click", function()

      private.Portal_ShowComingSoon();

      ObjSet( "anm_map_portal_"..section_name.."_next",
      {
        animfunc = "ready",
        playing  = true
      } );

    end );

  elseif ( status == "enabled" ) then

    ObjSet( "btn_map_portal_"..section_name.."_next",
    {
      visible = false,
      active  = false,
      input   = false
    } );

    if ( m_map.Section_NeedOutro( section_name ) ) then

      private.Outro_Show( private.Portal_ShowWorldMap );

      m_map.Section_SetOutro( section_name, true );

    end;

  elseif ( status == "prepare" ) then

    private.Portal_Next_ShowPopup();

    AnimPlay( "anm_map_portal_"..section_name.."_next", "prepare_click", function()

      ObjSet( "anm_map_portal_"..section_name.."_next",
      {
        animfunc = "prepare",
        playing  = true
      } );

    end );

  else

    private.Portal_Next_ShowPopup();

    AnimPlay( "anm_map_portal_"..section_name.."_next", "normal_click", function()

      ObjSet( "anm_map_portal_"..section_name.."_next",
      {
        animfunc = "normal",
        playing  = true
      } );

    end );

  end;

end;
--------------------------------------------------------------------------------
function private.Portal_Next_SetTap ()
  
  local section_name = private.Section_Current_GetName();

  common.TapSet( "btn_map_portal_"..section_name.."_next",
  {
    func_true = function ()

      local status = m_map.Section_Next_Status();

      private.Portal_Next_Tap( status ); 

    end
  } );

end;
--------------------------------------------------------------------------------
function private.Portal_Next_ShowPopup ()
  
  local section_name = private.Section_Current_GetName();

  public.ShowObjectPopup(
    "monolog",
    "obj_map_portal_"..section_name.."_next_indicator",
    "str_int_popup_portal_"..section_name.."_next",
    nil
  );

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function public.Pending_Start ( pending_token, func_exec )
  
  private.Pending_Start( pending_token, func_exec );
  
end;
--------------------------------------------------------------------------------
function private.IsPending ()
  
  return private.pending;
  
end;
--------------------------------------------------------------------------------
function public.Pending_Complete ( pending_token )
  
  private.Pending_Complete( pending_token );
  
end;
--------------------------------------------------------------------------------
function private.Pending_Start ( pending_token, func_exec )
  
  -- DbgTrace( "[ VIEW ][ MAP ] Pending started: "..pending_token );

  private.pending = pending_token;
  
  if ( func_exec ) then
    
    private.Pending_AddOnComplete( pending_token, func_exec );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Pending_AddOnComplete ( pending_token, func_exec )
  
  if not ( func_exec ) then
    
    return;
    
  end;
  
  if not ( private.pending_oncomplete ) then
    
    private.pending_oncomplete = {};
    
  end;
  
  if not ( private.pending_oncomplete[ pending_token ] ) then
    
    private.pending_oncomplete[ pending_token ] = {};
    
  end;
  
  table.insert( private.pending_oncomplete[ pending_token ], func_exec );
  
end;
--------------------------------------------------------------------------------
function private.Pending_Complete ( pending_token )
 
  -- DbgTrace( "[ VIEW ][ MAP ] Pending complete: "..pending_token );
  
  private.pending = nil;
  
  private.Achievement_TryShowPanel();
  
  if ( not private.pending_oncomplete                   ) or
     ( not private.pending_oncomplete[ pending_token ]  ) or 
     ( #private.pending_oncomplete[ pending_token ] < 1 ) then
     
     -- DbgTrace( "[ VIEW ][ MAP ] Pending complete: nothing executed" );
    
    return;
    
  end;
  
  for f = 1, #private.pending_oncomplete[ pending_token ] do
    
    private.pending_oncomplete[ pending_token ][ f ]();
    
  end;
  
  private.pending_oncomplete[ pending_token ] = nil;
  
end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.Intro_Prepare ()

  local section_name = m_map.Section_Current_Name();

  local objects = {};

  local stage_first = tonumber(
    string.match( m_map.Section_Stage_First( section_name ), "%d+" ) );

  local stage_last  = stage_first + 10;

  table.insert( objects, { private.this.marker, { alp = 0 } } );

  for i = stage_first, stage_last do

    table.insert( objects, { "btn_map_point_stage_"..i,
    {
      alp     = 0,
      scale_x = 0
    } } );

  end;

  ObjMultiSet( objects );

end;
--------------------------------------------------------------------------------
function private.Intro_Show ( func_OnEnd )

  private.intro_active = true;

  local section_name = m_map.Section_Current_Name();
  
  int.Lock( true, "Intro_Show" );
  public.Interface_Disable( true, "animate_intro" );

  m_map.Section_SetIntro( section_name, true );

  local stage_first = tonumber(
    string.match( m_map.Section_Stage_First( section_name ), "%d+" ) );
    
  local stage_last  = stage_first + 10;
  
  local objects = {};

  local func_CascadeShow = function ()

    ObjAttach( "obj_map_foreground_movie", "obj_map_foreground" );
    ObjAttach( "obj_map_spawn", "hub_map_anims" );

    ObjSet( "obj_map_foreground_movie", { active = false, visible = false } );
    ObjSet( "obj_map_spawn",            { active = false, visible = false } );

    local func_OnCascadeEnd = function ()
    
      public.Focus( "obj_map_point_stage_"..stage_first,
        {
          time = 1.5,
          scale = 1.0,
          func_after = function ()
          
            private.Stages_PointEnable( "stage_"..stage_first );
            private.intro_active = false;
            public.Interface_Enable( true, "animate_intro hero" );
            if ( func_OnEnd ) then func_OnEnd(); end;
          
          end
        }
      );

    end;

    public.Interface_Enable( false, "animate_intro" );

    private.Stages_Show_Cascade( stage_first, stage_last, func_OnCascadeEnd );
    
    ObjAnimate( private.this.marker, "alp", 0, 0, "",
    {
      0.0, 0, 0,
      0.3, 0, 1
    } );
    
    public.Focus( "obj_map_point_stage_"..stage_last,
      {
        time  = 1.5,
        scale = 0.6
      }
    );

  end;

  ObjAttach( "obj_map_spawn", "obj_map_anims" );
  ObjAttach( "obj_map_foreground_movie", InterfaceName );
  
  local portal_pos =
    private.Object_Position( "obj_map_portal_"..section_name.."_prev" );

  public.Focus( "obj_map_portal_"..section_name.."_prev",
    {
      instant = true,
      scale   = 1.0
    }
  );

  table.insert( objects, { "obj_map_foreground_movie",
  {
    active  = true,
    visible = true
  } } );
  
  table.insert( objects, { "anm_map_foreground_movie",
  {
    active  = false,
    visible = false,
    res     = "assets/content/"..section_name.."/movie/anm_intro",
    pos_x   = portal_pos[ 1 ],
    pos_y   = portal_pos[ 2 ]
  } } );

  table.insert( objects, { "obj_map_spawn",
  {
    active  = true,
    visible = true,
    pos_x   = portal_pos[ 1 ],
    pos_y   = portal_pos[ 2 ]
  } } );
  
  table.insert( objects, { "anm_map_spawn_hero",
  {
    active  = false,
    visible = false,
    res     = "assets/content/"..section_name.."/movie/anm_intro"
  } } );

  ObjMultiSet( objects );

  local func_skip = function ( func_OnSkipEnd )

    private.Intro_Skip( func_OnSkipEnd );

  end;
  
  local func_after_skip = function ()
  
    private.intro_active = false;
    public.Interface_Enable( true, "skip map intro" );
  
    if ( func_OnEnd ) then func_OnEnd(); end;
  
  end;

  int.Subtitles_Show( "intro", section_name, func_skip, func_after_skip );

  private.Intro_Play( 1, section_name, func_CascadeShow );

end;
--------------------------------------------------------------------------------
function private.Intro_Play ( intro_step, section_name, func_end )

  --DbgTrace( "Intro_Play: "..intro_step );

  local intro = 
  {
    section_2 = 
    { 
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_1" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_enemy_stage_220",
        params = { time = 1.0, scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_point_enemy_banshee_stage_220",
        params = {
          animfunc_start = "action",
          animfunc_end   = "static",
          animfunc_dummy = "dummy" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_point_stage_111",
        params = {
          time  = 1.0,
          scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_2" }
      }
    },
    section_3 = 
    { 
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_1" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_enemy_stage_330",
        params = { time = 1.0, scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_point_enemy_trollking_stage_330",
        params = {
          animfunc_start = "action",
          animfunc_end   = "static",
          animfunc_dummy = "dummy" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_point_stage_221",
        params = {
          time  = 1.0,
          scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_2" }
      }
    },
    section_4 = 
    { 
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_1" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_enemy_stage_440",
        params = { time = 1.0, scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_point_enemy_naga_stage_440",
        params = {
          animfunc_start = "action",
          animfunc_end   = "static",
          animfunc_dummy = "dummy" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_point_stage_331",
        params = {
          time  = 1.0,
          scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_2" }
      }
    },
    section_5 = 
    { 
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_1" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_characters_quince",
        params = { time = 1.0, scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_characters_icon_quince",
        params = {
          animfunc_start = "action_quince",
          animfunc_end   = "static_quince" }
      },
      { func   = private.Intro_Scale,
        object = "obj_map_point_stage_441",
        params = {
          time  = 1.0,
          scale = 1.0 }
      },
      { func   = private.Intro_Animate,
        object = "anm_map_spawn_hero",
        params = { animfunc_start = "intro_2" }
      }
    }
  };

  if ( intro[ section_name ] ) then

    local steps = intro[ section_name ];

    if ( intro_step == 1 ) then

      private.Music_Stop();

      ne.snd.Play(
        "assets/content/"..section_name.."/movie/aud_intro",
        "sfx", 0, "", 0
      );

    end;

    if ( steps[ intro_step ] ) then

      steps[ intro_step ].func(
        intro_step,
        steps[ intro_step ].object,
        steps[ intro_step ].params,
        function ()

        private.Intro_Play( ( intro_step + 1 ), section_name, func_end );

        end
      );

    else

      private.Music_Start();
      func_end();

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Intro_Animate ( intro_step, object_name, params, func_end )

  if not ( object_name ) then

    object_name = "anm_map_foreground_movie";

  end;

  --DbgTrace( "Intro_Animate: "..intro_step..", "..object_name );

  local animfunc_start = "intro_"..intro_step;
  local animfunc_end = nil;
  local animfunc_dummy = nil;

  if ( params ) then

    if ( params.animfunc_start ) then

      animfunc_start = params.animfunc_start;

    end;

    if ( params.animfunc_dummy ) then

      animfunc_dummy = params.animfunc_dummy;

    end;

    if ( params.animfunc_end ) then

      animfunc_end = params.animfunc_end;

    end;

  end;

  local func_end_anim = func_end;

  if ( animfunc_end ) then

    func_end_anim = function ()

      AnimPlay( object_name, animfunc_end, "" );

      if not ( animfunc_dummy ) then

        func_end();

      end;

    end;

  end;

  if ( animfunc_dummy ) then

    ObjClone( object_name, "_dummy", 1, ObjGetRelations( object_name ).parent );

    AnimPlay( object_name.."_dummy", animfunc_dummy, function ()

      ObjDelete( object_name.."_dummy" );

      func_end();

    end );

  end;

  ObjSet( object_name, { active = true, visible = true } );

  AnimPlay( object_name, animfunc_start, func_end_anim );

end;
--------------------------------------------------------------------------------
function private.Intro_Scale ( intro_step, object_name, params, func_end )

  --DbgTrace( "Intro_Scale: "..intro_step..", "..object_name );

  local scale = params.scale or 1.0;
  local time  = params.time or 1.0;

  public.Focus(
    object_name,
    {
      time       = time,
      scale      = scale,
      func_after = func_end
    }
  );

end;
--------------------------------------------------------------------------------
function private.Intro_Skip ( func_OnSkipEnd )

  --DbgTrace( "Intro_Skip" );

  local intro = 
  {
    section_2 = 
    { 
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_point_enemy_banshee_stage_220",
        params = { animfunc_end = "static" }
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      }
    },
    section_3 = 
    { 
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_point_enemy_trollking_stage_330",
        params = { animfunc_end = "static" }
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      }
    },
    section_4 = 
    { 
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_point_enemy_naga_stage_440",
        params = { animfunc_end = "static" }
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      }
    },
    section_5 = 
    { 
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_characters_icon_quince",
        params = { animfunc_end = "static" }
      },
      { func   = private.Intro_StopAnimate,
        object = "anm_map_spawn_hero",
        params = {}
      }
    }
  };

  local section_name = private.Section_Current_GetName();
  
  if ( intro[ section_name ] ) then

    for i = 1, #intro[ section_name ] do

      local action = intro[ section_name ][ i ];

      action.func( action.object, action.params.animfunc_end );

    end;

  end;

  local stage_first = tonumber(
    string.match( m_map.Section_Stage_First( section_name ), "%d+" ) );
    
  local stage_last = stage_first + 10;

  local points = {}; 

  for i = stage_first, stage_last do

    table.insert( points, { "btn_map_point_stage_"..i,
    {
      alp     = 1,
      scale_x = 1 } } );

  end;

  ObjMultiSet( points );

  private.Stages_Show_Instant( stage_first, stage_last, "common" );
  
  public.Focus( "obj_map_point_stage_"..stage_first,
    {
      time       = 0.3,
      scale      = 1.0,
      func_after = function ()
        private.Stages_PointEnable( "stage_"..stage_first );
      end
    }
  );

  ObjAttach( "obj_map_foreground_movie", "obj_map_foreground" );
  ObjAttach( "obj_map_spawn", "hub_map_anims" );

  ObjSet( "obj_map_foreground_movie", { active = false, visible = false } );
  ObjSet( "obj_map_spawn", { active = false, visible = false } );
   
  ObjAnimate( private.this.marker, "alp", 0, 0,
  function ()
    func_OnSkipEnd();
  end,
  {
    0.0, 0, 0,
    0.3, 0, 1
  } );

  ne.snd.Stop( "assets/content/"..section_name.."/movie/aud_intro", "sfx", 0, "", 0 );
  private.Music_Start();

end;
--------------------------------------------------------------------------------
function private.Intro_StopAnimate ( object_name, animfunc_end )

  --DbgTrace( "Intro_StopAnimate: "..object_name );

  AnimStop( object_name );

  if ( animfunc_end ) then

    AnimPlay( object_name, animfunc_end, "" );

  end;

end;
--------------------------------------------------------------------------------
function private.Outro_Show ( func_OnEnd )

  local outro = 
  {
    section_1 =
    { 
      {
        func   = private.Outro_Scale,
        object = "obj_map_enemy_stage_85",
        params = { time = 4.0, scale = 0.8 }
      }
    },
    section_2 =
    { 
      {
        func   = private.Outro_Scale,
        object = "obj_map_enemy_stage_220",
        params = { time = 4.0, scale = 0.8 }
      }
    },
    section_3 =
    { 
      {
        func   = private.Outro_Scale,
        object = "obj_map_enemy_stage_330",
        params = { time = 4.0, scale = 0.8 }
      }
    },
    section_4 =
    { 
      {
        func   = private.Outro_Scale,
        object = "obj_map_enemy_stage_440",
        params = { time = 4.0, scale = 0.8 }
      }
    }
  };

  local section_name = private.Section_Current_GetName();
  
  local objects = {};

  int.Lock( true );

  ObjAttach( "obj_map_foreground_movie", InterfaceName );

  local portal_pos = private.Object_Position(
    "obj_map_portal_"..section_name.."_next",
    InterfaceName
  );

  table.insert( objects, { "obj_map_foreground_movie",
  {
    active  = true,
    visible = true
  } } );
  
  table.insert( objects, { "anm_map_foreground_movie",
  {
    res   = "assets/content/"..section_name.."/movie/anm_outro",
    pos_x = portal_pos[ 1 ],
    pos_y = portal_pos[ 2 ]
  } } );

  ObjMultiSet( objects );

  AnimPlay( "anm_map_foreground_movie", "outro", function ()

    int.Lock( false );

    ObjSet( "obj_map_foreground_movie", { active = false, visible = false } );
    ObjAttach( "obj_map_foreground_movie", "obj_map_foreground" );

    if ( func_OnEnd ) then
  
      func_OnEnd();

    end;

  end );

  private.Music_Stop();
  cmn.PlaySnd(   "assets/content/"..section_name.."/aud_leave_section" );

  if ( outro[ section_name ] ) then

    outro[ section_name ][ 1 ].func(
      outro[ section_name ][ 1 ].object,
      outro[ section_name ][ 1 ].params
    );

  end;

  local func_skip = function ( func_after )
  
      private.Outro_Skip( func_after, outro[ section_name ] )
      
  end;
  
  local func_after_skip = function ()

    int.Lock( false );
    if ( func_OnEnd ) then func_OnEnd(); end;

  end;

  int.Subtitles_Show( "outro", section_name, func_skip, func_after_skip );

  private.Interface_Hide();

end;
--------------------------------------------------------------------------------
function private.Outro_Scale ( object_name, params )

  local scale = params.scale or 1.0;
  local time  = params.time or 1.0;

  public.Focus( object_name, { time = time, scale = scale } );

end;
--------------------------------------------------------------------------------
function private.Outro_Skip ( func_OnSkipEnd, outro_steps )
  
  local section_name = private.Section_Current_GetName();

  cmn.StopSnd( "leave_"..section_name );

  AnimStop( "anm_map_foreground_movie" );
  ObjSet( "obj_map_foreground_movie", { active = false, visible = false } );
  ObjAttach( "obj_map_foreground_movie", "obj_map_foreground" );
  
  if ( outro_steps ) then
  
    last_step = outro_steps[ #outro_steps ];
    last_step.params.time = 0.1;
    
    last_step.func( last_step.object, last_step.params );
  
  end;

  func_OnSkipEnd();

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function public.ShowObjectPopup (
    mode,
    indicator_name,
    text,
    data,
    offset,
    functions,
    skip_focus
  )

  offset    = offset or { 0, 0 };
  functions = functions or {};

  local object_name = indicator_name:gsub( "_indicator", "" );
  local obj_pos     = private.Object_Position( object_name );

  if ( not skip_focus ) then

    local scale   = nil;
    local panel_h = 100;
    local popup_h = 240;
    local map_top = -( private.map_w + 2 * private.offset );
    
    local space = obj_pos[2] - ( map_top + ( panel_h + popup_h ) / private.map_scale_current );
    
    if ( space < popup_h + panel_h ) then

      scale = math.abs( ( popup_h / private.map_scale_current ) / space );

    end;

    functions.func_scale = function( end_func )

      public.Focus(
        object_name, 
        {
          scale      = scale,
          func_after = end_func,
          offset = { x = 0, y = 100 }
        }
      );

    end;
  
  end;

  int.ShowPopupCloud(
    mode,
    indicator_name,
    text,
    data,
    offset,
    1.0,
    functions
  );

end;
--------------------------------------------------------------------------------
function  public.HideObjectPopup ()
  
  int.HidePopupCloud(); 

end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.Sound_Play ( sound_id )
  
  local path = "assets/content/map/sounds/aud_"..sound_id;
  
  cmn.PlaySnd( path );
  
end;
--------------------------------------------------------------------------------
-- function ... () end
--------------------------------------------------------------------------------
function private.Music_Start ()
  
  if ( not private.music ) then
    cmn.PlayMusic( "assets/content/audio/music/aud_map_mus", 
                    nil, nil, true );
    private.music = true;
  end;

end;
--------------------------------------------------------------------------------
function private.Music_Stop ()
  
  if ( private.music ) then
    cmn.StopMusic( "assets/content/audio/music/aud_map_mus" );
    private.music = false;
  end;

end;
--------------------------------------------------------------------------------
-- function ******* DEV ******* () end
--------------------------------------------------------------------------------
function  public.Cheater_Update ()

  private.Update();

end;
--------------------------------------------------------------------------------
function private.DEV_GetPointPos ()

  ObjSet( "obj_map_point_DEV", { event_dragdrop = function ()

    local pos = private.Object_Position(
      "obj_map_point_DEV",
      private.this.background
    );

    DbgTrace( "pos_x = "..tostring( private.scale_modifier * pos[ 1 ] ) );
    DbgTrace( "pos_y = "..tostring( private.scale_modifier * pos[ 2 ] ) );
    DbgTrace( "" );

    ObjSet( "obj_map_point_DEV",
    {
      pos_x = private.scale_modifier * pos[ 1 ] + private.offset,
      pos_y = private.scale_modifier * pos[ 2 ] - private.offset
    } );

  end; } );

end;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- function *** NEW FOCUSING *** ()
--------------------------------------------------------------------------------
function public.Focus ( obj, params )

  -- common.DbgCallpoint( obj );

  params = params or {};
  
  -- NOTE: all parameters are optional:
  -- time       - focus time, default = ( distance / map_w )^( 1/3 )
  -- speed      - focus speed factor, default = 1
  -- instant    - instant focus flag, default = false
  -- scale      - new scale, default = current scale
  -- func_after - after focus callback
  -- offset     - target position offset, default = { x = 0, y = 0 }
  
  -- Warn about breaking solid focus
  if ( IsEditor() and private.focusing_solid ) then
  
    local msg = "Warning! Solid focusing at '%s' breaked by '%s'";
    common.DbgCallpoint( msg:format( private.focusing_solid, obj ) );
  
  end;
  
  local func_after = params.func_after;
  
  private.focusing_solid = tern( func_after )( obj, false );

  local function func_focus_end ()
    
    ObjSet( "zmv_map", { input = true } );
    if ( func_after ) then
      private.focusing_solid = false;
      func_after();
    end;
  
  end;

  -- Stop map animations
  ZoomViewStopVelocity( "zmv_map" );
  ObjStopAnimate( private.this.body, "pos_xy" );
  ObjStopAnimate( private.this.body, "scale_xy" );
  
  -- Define map state changes
  local scale_now = private.Focus_ScaleCurrent();
  local scale_new = params.scale or scale_now;
  
  scale_new = math.min( scale_new, private.map_scale_max );
  scale_new = math.max( scale_new, private.map_scale_min );
  
  local scale_factor = scale_now / scale_new;
  
  local pos_now  = private.Focus_PositionCurrent();
  local offset   = params.offset;
  local pos_new  = private.Focus_PositionTarget( obj, offset, scale_new );
  
  local distance = private.Focus_DistanceTo( pos_now, pos_new );
  local speed    = params.speed or 1;
  local time_opt = ( distance / private.map_w )^( 1/3 ) * scale_factor / speed;
  local time     = params.time or math.max( time_opt, 0.2 );
  
  local hold     = ( distance < 30 ) and not params.time;
  local instant  = params.instant or ( time < 1 / 25 );
  
  -- local tr = "\tp = %.2f,%.2f -> %.2f,%.2f; d = %.2f; "..
             -- "s = %.2f -> %.2f; t = %.2f; i = %s; h = %s";
  -- DbgTrace( tr:format( 
    -- pos_now.x, pos_now.y, pos_new.x, pos_new.y, distance,
    -- scale_now, scale_new, time, instant, hold
  -- ) );
  
  if ( hold ) then
  
    pos_new = pos_now;
    instant = true;
  
  end;
  
  if ( instant ) then
  
    local map_params =
    {
      pos_x = pos_new.x,
      pos_y = pos_new.y
    };
    
    if ( scale_now ~= scale_new ) then
    
      map_params.scale_x = scale_new;
      map_params.scale_y = scale_new;
    
    end;
  
    ObjSet( private.this.body, map_params );
    func_focus_end();
  
  else
  
    ObjSet( "zmv_map", { input = false } );
    
    if ( scale_now ~= scale_new ) then
    
      ObjAnimate( private.this.body, "scale_xy", 0, 0, "", 
      { 
        0.00, 3, scale_now, scale_now,
        time, 3, scale_new, scale_new
      } );
    
    end;
  
    ObjAnimate( private.this.body, "pos_xy", 0, 0, func_focus_end, 
    {
      0.00, 3, pos_now.x, pos_now.y,
      time, 3, pos_new.x, pos_new.y
    } );
  
  end;

end;
--------------------------------------------------------------------------------
function private.Focus_ScaleCurrent ()

  local params = ObjGet( private.this.body ) or {};
  return params.scale_x;

end;
--------------------------------------------------------------------------------
function private.Focus_PositionCurrent ()

  local params = ObjGet( private.this.body ) or {};
  return { x = params.pos_x or 0, y = params.pos_y or 0 };

end;
--------------------------------------------------------------------------------
function private.Focus_PositionTarget ( obj, offset, scale_new )

  -- Get raw position
  local pos_raw = int.Position( obj, private.this.body );
  local pos = { x = pos_raw[ 1 ] or 0, y = pos_raw[ 2 ] or 0 };
  
  -- Get zoomview params
  local params = m_object.Get( "zmv_map" );
  local scr_x, scr_y = params.screen_x, params.screen_y;
  local scr_w, scr_h = params.screen_w, params.screen_h;
  
  -- Shift by offset and move to screen center
  offset = offset or { x = 0, y = 0 };
  local center = { x = scr_w / 2 + scr_x, y = scr_h / 2 + scr_y };
  
  -- DbgTrace( ("Focus center: %s, %s"):format( center.x, center.y ) );
  
  pos.x = center.x + offset.x - pos.x * scale_new;
  pos.y = center.y + offset.y - pos.y * scale_new;
  
  -- Check bounds
  local map_s        = private.scale_modifier;
  local map_h, map_w = private.map_h, private.map_w;
  -- local app_h, app_w = private.app_h, private.app_w;
  
  local x_min, x_max = scr_w - map_s * map_w * scale_new, 0;
  local y_min, y_max = scr_h,  map_s * map_h * scale_new;
  
  pos.x = math.min( pos.x, x_max );
  pos.x = math.max( pos.x, x_min );
  
  pos.y = math.min( pos.y, y_max );
  pos.y = math.max( pos.y, y_min );
  
  return pos;

end;
--------------------------------------------------------------------------------
function private.Focus_DistanceTo ( a, b )

  local dx, dy = b.x - a.x, b.y - a.y;
  return math.sqrt( dx^2 + dy^2 );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Update ()

  mediator.Notify( "View_Map_Update_Triggered" );

  local changes = m_map.GetChangesList();
  local update  = {}; -- callback list
  
  -- Notify about update started
  local begin_msg = "View_Map_Update_Started";
  local func_notify = private.Callback_NotifyForPending( begin_msg );
  table.insert( update, func_notify );

  private.Update_TryShowIntro( update );
  private.Update_TryShowUpdateReward( update );
  
  -- Internal update flags
  local enable_notifications = false;
  local unlock_portal = false;
  
  -- Begin changes
  local ch_begin_msg = "View_Map_Update_ChangesBegin";
  local func_ch_begin = private.Callback_NotifyForPending( ch_begin_msg );
  table.insert( update, func_ch_begin );
  
  for id, change in pairs( changes ) do
  
    local stage = change.name;
    local rewards = change.reward;
    local noreward = change.noreward;
  
    if ( change.type == "StageCompleted" ) then
    
      -- Try set notifications flag
      local need_push = wiz.bfg.playhaven.NeedRegisterPushNotifications();
      if ( need_push.result and change.is_first ) then
      
        local num = tonumber( stage:match( "%d+" ) );
        if ( num >= 6 ) and ( math.fmod( num - 10, 4 ) == 0 ) then
          enable_notifications = true;
        end;
      
      end;
      
      -- Setup reactions
      local reactions =
      {
        stage_reward = {},
        stage_done   = {},
        quest_reward = {},
        quest_done   = {} -- DG: Used for start second character quest
      };
      
      reactions.stage_reward =
      {
        build     = v_builds.GetRewardReaction(
          rewards,
          private.Portal_Show
        ),
        character = v_characters.GetRewardReaction( stage, rewards ),
        mine      = v_mines.GetRewardReaction( stage, rewards )
      };
      
      if ( change.is_first ) then
      
        reactions.stage_done =
        {
          build     = v_builds.GetStageCompleteReaction( stage );
          character = v_characters.GetStageCompleteReaction( stage );
          mine      = v_mines.GetStageCompleteReaction( stage );
        };
        
        if ( reactions.stage_reward.build ) and
           ( reactions.stage_done.build   ) then
          
          local build_name = {
            reactions.stage_reward.build.name,
            reactions.stage_done.build.name
          };
          
          if ( build_name[ 1 ] == build_name[ 2 ] ) then
            
            reactions.stage_done.build = nil;
            
          end;
          
        end;
      
      end;
      
      if ( reactions.stage_reward.character ) then
      
        local reward = reactions.stage_reward.character.reward;
        local quest  = reactions.stage_reward.character.quest_done;
      
        reactions.quest_reward.build = v_builds.GetRewardReaction(
          reward,
          private.Portal_Show
        );
        
        reactions.quest_done.character = 
          v_characters.GetQuestCompleteReaction( quest );
      
      end;
      
      --DbgTrace( reactions );
      
      -- Check for characters will update stages
      local hide_quest_item = false;
      for _, reaction in pairs( reactions ) do
      
        if ( reaction.character and reaction.character.update_stages ) then
        
          local list = reaction.character.update_stages;
          if ( cmn.IsEntityInList( list, stage ) ) then
          
            hide_quest_item = true;
            break;
          
          end;
        
        end;
      
      end;
      
      -- Process reactions
      private.Update_FocusAtCompletedPoint(
        update, stage, noreward, hide_quest_item
      );
      private.Update_TryDisableBossFight( update, stage );
      private.Update_TryShowStageReward(
        update, stage, rewards, reactions.stage_reward
      );
      
      if ( reactions.stage_reward.build ) then
      
        local reaction = reactions.stage_reward.build;
      
        private.Update_Build_TryAnimate( update, reaction );
        private.Update_Build_TryShowPopup( update, reaction );
        private.Update_Build_TryShowStages( update, reaction );
        private.Update_Build_TryChangeSkin( update, reaction );
      
      end;
      
      if ( reactions.stage_reward.mine ) then
      
        local reaction = reactions.stage_reward.mine;
      
        private.Update_Mine_TryAnimate( update, reaction );
        private.Update_Mine_TryShowPopup( update, reaction );
      
      end;
      
      if ( reactions.stage_reward.character ) then
      
        local reaction    = reactions.stage_reward.character;
        local quest_react = reactions.quest_reward;
      
        private.Update_Character_TryShowPopup( update, reaction );
        private.Update_Character_TryAnimate( update, reaction );
        private.Update_Character_TryShowStory( update, reaction );
        private.Update_Character_TryShowReward( update, reaction, quest_react );
        private.Update_Character_TryChangeSkin( update, reaction );
        
      end;
      
      if ( reactions.quest_reward.build ) then
      
        local reaction = reactions.quest_reward.build;
      
        private.Update_Build_TryAnimate( update, reaction );
        private.Update_Build_TryShowPopup( update, reaction );
        private.Update_Build_TryShowStages( update, reaction );
        private.Update_Build_TryChangeSkin( update, reaction );
      
      end;
      
      if ( reactions.quest_done.character ) then
      
        local reaction = reactions.quest_done.character;
      
        private.Update_Character_TryAnimate( update, reaction );
        private.Update_Character_TryUpdateStages( update, reaction );
        private.Update_Character_TryShowPopup( update, reaction );
        
      end;
      
      if ( reactions.stage_done.build ) then
      
        local reaction = reactions.stage_done.build;
        
        private.Update_Build_TryAnimate( update, reaction );
        private.Update_Build_TryShowStages( update, reaction );
        private.Update_Build_TryChangeSkin( update, reaction );
      
      end;
      
      if ( reactions.stage_done.mine ) then
      
        local reaction = reactions.stage_done.mine;
      
        private.Update_Mine_TryAnimate( update, reaction );
        private.Update_Mine_TryShowPopup( update, reaction );
      
      end;
      
      if ( reactions.stage_done.character ) then
      
        local reaction = reactions.stage_done.character;
      
        private.Update_Character_TryAnimate( update, reaction );
        private.Update_Character_TryUpdateStages( update, reaction );
        private.Update_Character_TryShowPopup( update, reaction );
      
      end;
      
      private.Update_TryShowResourcesReward( update, stage, rewards );
      
      if ( change.is_first ) then
      
        private.Update_TryDisableFight( update, stage );
        private.Update_TryAppearSpell(  update, stage );
        
        private.Update_TryPlaceChest( update, stage );
        
      end;
  
    elseif ( change.type == "StageEnabled" ) then
    
      private.Update_FocusAtEnabledPoint( update, stage );
      private.Update_TryEnableFight( update, stage );
      private.Update_TryShowHint( update, stage );
    
    elseif ( change.type == "PortalUnlocked" ) then
    
      unlock_portal = true;
    
    end;
    
  end;
  
  local ch_done_msg = "View_Map_Update_ChangesDone";
  local func_ch_done = private.Callback_NotifyForPending( ch_done_msg );
  table.insert( update, func_ch_done );
  
  private.Update_TryEnableNotifications( update, enable_notifications );
  private.Update_TryShowDailyQuest( update );
  private.Update_TryShowDailyBonus( update );
  
  private.Update_TryUnlockPortal( update, unlock_portal );
  private.Update_Finish( update, unlock_portal );
  
  common.RunCallbackList( update );

  private.Market_Update(); -- FIXME
  
end;
--------------------------------------------------------------------------------
function private.Callback_NotifyForPending ( notification, params )

  return function ( func_after )
  
    -- DbgTrace( notification, params );
    mediator.Notify( notification, params );
  
    if ( func_after ) then
  
      local pending_token = private.IsPending();
      if  ( pending_token ) then
      
        local function func_after_pending ()
        
          if ( int_window.IsAnyWindowOnScreen() ) then
            public.Interface_Disable();
            mediator.SubscribeOnce( "Interface_Window_NoWindows", func_after );
          else
            func_after();
          end;
          
        end;
        
        private.Pending_AddOnComplete( pending_token, func_after_pending );
        
      else
      
        func_after();
        
      end;
      
    end;
  
  end;

end;
--------------------------------------------------------------------------------
--[[ 
  Animation format
  {
    res = "resource", -- optional
    coords = {}, -- parameters for anim_action
    name = "parent_name",
    func_preset = function () end,
    func_dummy  = function () end,
    func_action = function ( func_after ) end 
  }
]]
--------------------------------------------------------------------------------
function public.Callback_Animate ( anim )

  if not ( anim ) then return false; end;

  local function func_anim ( func_after )

    if ( anim.res and anim.res ~= "" ) then

      local anm_dummy  = string.format( "anm_%s_dummy", anim.name );
      local anm_action = string.format( "anm_%s_action", anim.name );
      local anm_coords =
      { 
        pos_x = anim.coords.pos_x, 
        pos_y = anim.coords.pos_y, 
        pos_z = 10,
        scale_x = anim.coords.scale_x or 1,
        scale_y = anim.coords.scale_y or 1,
      };

      -- Set animations
      ObjCreate( anm_dummy,  "anim" );
      ObjCreate( anm_action, "anim" );
      ObjSet(    anm_action, anm_coords );
      ObjAttach( anm_dummy,  anim.name );
      ObjAttach( anm_action, anim.name );
      
      ObjSet( anm_dummy,  { res = anim.res } );
      ObjSet( anm_action, { res = anim.res } );
      
      -- Run animations
      AnimPlay( anm_dummy, "dummy", function ()
      
        anim.func_dummy();
        ObjDelete( anm_dummy );
      
      end );
      
      local animfunc = anim.animfunc or "action";
      AnimPlay( anm_action, animfunc, function ()
      
        anim.func_action( func_after );
        ObjDelete( anm_action );
      
      end );

    else
    
      if ( anim.func_dummy ) then
  
        anim.func_dummy();
        
      end;

      anim.func_action( func_after );

    end;

  end;

  return function ( func_after )
    
    int.Lock( true );
    
    local function func_end ()
      int.Lock( false );
      if ( func_after ) then func_after(); end;
    end;
    
    if ( anim.func_preset ) then
    
      anim.func_preset();
    
    end;
    
    if ( anim.skip_focus ) then
    
      func_anim( func_end );
      
    else
    
      public.Focus( anim.name, 
        { 
          func_after = function () func_anim( func_end ) end
        }
      );
      
    end;

  end;
  
end;
--------------------------------------------------------------------------------
function private.Update_TryAddStep ( update, func_step, notification, params )

  if ( func_step ) then
  
    local func_notify =
      private.Callback_NotifyForPending( notification, params );
    
    -- DbgTrace( "Add to update: " .. notification, params );
    
    table.insert( update, func_step );
    table.insert( update, func_notify );
  
  end;

end;
--------------------------------------------------------------------------------
function private.Update_TryShowIntro ( update )

  local func_intro = nil;
  
  if ( not m_tutorial.IsCompleted( "intro" ) ) then
  
    func_intro = function ( func_after )
    
      private.Interface_Hide();
      mediator.Notify( "View_Tutorial_StartIntro", func_after );
    
    end;
  
  elseif ( m_map.Section_NeedIntro() ) then

    func_intro = private.Intro_Show;

  end;
  
  local notification = "View_Map_Update_IntroDisplayed";
  private.Update_TryAddStep( update, func_intro, notification );

end;
--------------------------------------------------------------------------------
function private.Update_TryShowUpdateReward ( update )

  local func_reward = nil;
  
  local update_data = m_map.GetUpdateData();
  if  ( update_data ) then
  
    func_reward = function ( func_after )

      if ( update_data.type == "replay" ) then
        
        int_dialog_update.Show(
          update_data.type,
          update_data.message,
          nil,
          nil,
          func_after
        );
        
      else
      
        local chest = 
        { 
          type  = "chest",
          name  = "goldchest",
          count = 1
        };

        local params =
        {
          category   = "reward",
          stage_name = "update",
          silent     = true
        };
      
        local function func_obtain ()
          v_items.Chest_Show( chest, params, update_data.reward, func_after );
        end;
      
        int_dialog_update.Show(
          update_data.type,
          update_data.message,
          v_hero.Skin_Attach,
          v_hero.Skin_Detach,
          func_obtain
        );
        
      end;
    
    end;
  
  end;
  
  local notification = "View_Map_Update_UpdateRewardDisplayed";
  private.Update_TryAddStep( update, func_reward, notification, update_data );

end;
--------------------------------------------------------------------------------
function private.Update_FocusAtCompletedPoint ( 
          update,
          stage,
          noreward,
          hide_quest_reward
        )

  local function func_focus ( func_after )
  
    private.Stages_PointUpdate( stage, noreward, hide_quest_reward );
    public.Focus( "obj_map_point_"..stage, 
      { 
        instant    = true, 
        func_after = func_after
      }
    );
  
  end;

  local notification = "View_Map_Update_CompletedStageFocused";
  private.Update_TryAddStep( update, func_focus, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryShowStageReward ( update, stage, rewards, reactions )

  local func_reward = nil;
  local has_drops = false;
  
  if ( rewards ) then
  
    for r, reward in pairs( rewards ) do
      if ( reward.type ~= "gold" and reward.type ~= "resource" ) then 
          has_drops = true; 
          break;
      end;
    end;
  
    func_reward = function ( func_after )

      local function func_reward_accept ()

        if ( has_drops ) then

          int_dialog_map_reward.Hide(
            function ()
              public.Reward_Show( true, rewards, reactions, func_after );
            end,        
            function ()    
              int_window.Hide( "int_dialog_map_reward" );
            end );

        else

          int_dialog_map_reward.Hide(
            function ()
              public.Reward_Show( true, rewards ); -- Just for gold
            end,        
            function ()    
              int_window.Hide( "int_dialog_map_reward", func_after );
            end );

        end;

        mediator.Notify( "View_Map_Update_StageRewardCollected", stage );

      end; -- end func_reward_accept

      private.TopPanel_Show();
      int_window.Show(
        "int_dialog_map_reward", 
        function ()
          int_dialog_map_reward.Show( rewards, func_reward_accept );
        end, 
        { map_active = true }
      );

    end; -- end func_reward
  
  end;
  
  local notification = "View_Map_Update_StageRewardDisplayed";
  private.Update_TryAddStep( update, func_reward, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_Build_TryShowPopup ( update, reaction )

  local notification = "View_Map_Update_Build_PopupDisplayed";
  local build = reaction.name;
  
  private.Update_TryAddStep( update, reaction.func_popup, notification, build );

end;
--------------------------------------------------------------------------------
function private.Update_Build_TryAnimate ( update, reaction )

  local func_anim    = reaction.func_anim;
  local notification = "View_Map_Update_Build_Animated";
  local build        = reaction.name;
  
  private.Update_TryAddStep( update, func_anim, notification, build );

end;
--------------------------------------------------------------------------------
function private.Update_Build_TryChangeSkin ( update, reaction )

  local func_skin = nil;

  if ( reaction.skinpart ) then
  
    func_skin = function ( func_after )
      v_hero.ChangeSkin_ShowDialog( reaction.skinpart, func_after );
    end;
  
  end;
  
  local notification = "View_Map_Update_Build_SkinChanged";
  local params =
  {
    build = reaction.name,
    skinpart = reaction.skinpart
  }
  
  private.Update_TryAddStep( update, func_skin, notification, params );

end;
--------------------------------------------------------------------------------
function private.Update_Build_TryShowStages ( update, reaction )

  local func_stages = nil;
  
  if ( reaction.show_stages ) then
  
    local stage_min  = reaction.show_stages.min;
    local stage_max  = reaction.show_stages.max;
    local stage_type = reaction.show_stages.type;
  
    func_stages = function ( func_after )
      
      local func_move_marker = function ()
        
        if ( reaction.move_marker ) then
          private.Marker_MoveToPoint( private.Stages_GetName( stage_min, 
                                                              stage_type ), 
                                      false, 
                                      false, 
                                      func_after );
        elseif ( func_after ) then
          func_after();
        end;
        
      end;
      
      private.Stages_Show_Cascade(
        stage_min, stage_max, func_move_marker, stage_type
      );
    end;
  
  end;
  
  local notification = "View_Map_Update_Build_StagesAppeared";
  local build = reaction.name;
  
  private.Update_TryAddStep( update, func_stages, notification, build );

end;
--------------------------------------------------------------------------------
function private.Update_Mine_TryShowPopup ( update, reaction )

  local notification = "View_Map_Update_Mine_PopupDisplayed";
  local func_popup   = reaction.func_popup;
  local mine         = reaction.name;
  
  private.Update_TryAddStep( update, func_popup, notification, mine );

end;
--------------------------------------------------------------------------------
function private.Update_Mine_TryAnimate ( update, reaction )

  local func_anim    = reaction.func_anim;
  local notification = "View_Map_Update_Mine_Animated";
  local mine         = reaction.name;
  
  private.Update_TryAddStep( update, func_anim, notification, mine );

end;
--------------------------------------------------------------------------------
function private.Update_Character_TryShowPopup ( update, reaction )

  local notification = "View_Map_Update_Character_PopupDisplayed";
  local func_popup   = reaction.func_popup;
  local character    = reaction.name;
  
  private.Update_TryAddStep( update, func_popup, notification, character );

end;
--------------------------------------------------------------------------------
function private.Update_Character_TryAnimate ( update, reaction )

  local func_anim    = reaction.func_anim;
  local notification = "View_Map_Update_Character_Animated";
  local character    = reaction.name;
  
  private.Update_TryAddStep( update, func_anim, notification, character );

end;
--------------------------------------------------------------------------------
function private.Update_Character_TryShowReward ( update, reaction, reactions )

  local func_reward;

  if ( reaction.func_reward ) then
  
    func_reward = function ( func_after )  
      reaction.func_reward( reactions, func_after );
    end;
  
  end;
  
  local quest = reaction.quest_done;
  
  local notification = "View_Map_Update_Quest_RewardDisplayed";
  private.Update_TryAddStep( update, func_reward, notification, quest );

end;
--------------------------------------------------------------------------------
function private.Update_Character_TryChangeSkin ( update, reaction )

  local func_skin = nil;

  if ( reaction.skinpart ) then
  
    func_skin = function ( func_after )
      v_hero.ChangeSkin_ShowDialog( reaction.skinpart, func_after );
    end;
  
  end;
  
  local notification = "View_Map_Update_Character_SkinChanged";
  local params =
  {
    character = reaction.name,
    skinpart  = reaction.skinpart
  }
  
  private.Update_TryAddStep( update, func_skin, notification, params );

end;
--------------------------------------------------------------------------------
function private.Update_Character_TryShowStory ( update, reaction )

  local func_story = v_story.Callback_ShowStory( reaction.story_data )

  local notification = "View_Map_Update_Character_StoryDisplayed";
  local quest = reaction.quest_done;
  
  private.Update_TryAddStep( update, func_story, notification, quest );

end;
--------------------------------------------------------------------------------
function private.Update_Character_TryUpdateStages ( update, reaction )

  local func_update = nil;
  
  if ( reaction.update_stages ) then
  
    func_update = function ( func_after )
    
      for _, stage in pairs( reaction.update_stages ) do
        private.Stages_PointUpdate( stage );
      end;
      
      if ( func_after ) then func_after(); end;
    
    end;
  
  end;
  
  local notification = "View_Map_Update_Character_StagesUpdated";
  local character    = reaction.name;
  
  private.Update_TryAddStep( update, func_update, notification, character );

end;
--------------------------------------------------------------------------------
function private.Update_TryShowResourcesReward ( update, stage, rewards )

  local func_res = nil;
  
  if ( rewards ) then
  
    local resources = {};
  
    for r, reward in pairs( rewards ) do
    
      if ( reward.type == "resource" ) then 
          table.insert( resources, reward );
      end;
      
    end;
    
    if ( #resources > 0 ) then
    
      func_res = function ( func_after )
      
        public.Focus( "obj_map_point_"..stage,
          { 
            func_after = function ()
        
              private.TopPanel_Show();
              ObjSet( "zmv_map", { input = false } );
            
              for r = 1, #resources do
            
                local reward = resources[ r ];
                private.Reward_Show_Resource( stage, reward, 
                  function ()
                  
                    private.Resource_Buffer_Out( reward.name );
                    if ( r == #resources ) then
                      ObjSet( "zmv_map", { input = true } );
                      func_after();
                    end;
                    
                  end
                );
              
              end;
              
            end
          }
        );
      
      end;
    
    end;
  
  end;

  local notification = "View_Map_Update_ResourcesRewardDisplayed";
  private.Update_TryAddStep( update, func_res, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryDisableFight ( update, stage )

  local func_fight = v_fights.Callback_AnimateDisable( stage );

  local notification = "View_Map_Update_FightDisabled";
  private.Update_TryAddStep( update, func_fight, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryDisableBossFight ( update, stage )

  local func_fight = v_fights.Callback_AnimateBossDisable( stage );

  local notification = "View_Map_Update_BossFightDisabled";
  private.Update_TryAddStep( update, func_fight, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryAppearSpell ( update, stage )

  local func_spell = v_spells.Callback_AnimateAppear( stage );

  local notification = "View_Map_Update_SpellAppeared";
  private.Update_TryAddStep( update, func_spell, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_FocusAtEnabledPoint ( update, stage )

  local function func_focus ( func_after )
  
    private.Marker_MoveToPoint( stage, false, true,
      function ()
        private.Stages_PointEnable( stage );
      end
    );
    
    public.Focus( "obj_map_point_"..stage, 
      { 
        time       = 0.6,
        func_after = func_after
      }
    );
  
  end;

  local notification = "View_Map_Update_EnabledStageFocused";
  private.Update_TryAddStep( update, func_focus, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryEnableFight ( update, stage )

  local func_fight = v_fights.Callback_AnimateEnable( stage );

  local notification = "View_Map_Update_FightEnabled";
  private.Update_TryAddStep( update, func_fight, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryPlaceChest ( update, stage )

  local func_chest = v_chests.Callback_AnimatePlace( stage );
  local notification = "View_Map_Update_ChestPlaced";
  
  private.Update_TryAddStep( update, func_chest, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryShowHint ( update, stage )

  local stage_info = m_map.GetStageInfo( stage ) or {};
  local func_hint = nil;
  
  if ( stage_info.hint and  stage_info.hint == stage_info.state ) then
  
    func_hint = function ( func_after )
    
      local function func_popup ()
        local str = "str_int_popup_cloud_"..stage;
        private.Hero_ShowPopup( str, stage, { func_preclose = func_after } );
      end;
    
      public.Focus( "obj_map_point_"..stage, { func_after = func_popup } );
    
    end;
  
  end;

  local notification = "View_Map_Update_HintDisplayed";
  private.Update_TryAddStep( update, func_hint, notification, stage );

end;
--------------------------------------------------------------------------------
function private.Update_TryEnableNotifications ( update, show_dialog )

  local func_push = nil;
  
  if ( show_dialog ) then
    func_push = private.DialogNotifications_Show;
  end;

  local notification = "View_Map_Update_NotificationsDisplayed";
  private.Update_TryAddStep( update, func_push, notification );

end;
--------------------------------------------------------------------------------
function private.Update_TryShowDailyQuest ( update )

  local func_daily = v_dailyquest.OnMapUpdate();
  
  local notification = "View_Map_Update_DailyQuestDisplayed";
  private.Update_TryAddStep( update, func_daily, notification );

end;
--------------------------------------------------------------------------------
function private.Update_TryShowDailyBonus ( update )

  -- Uncomment for testing
  -- m_dailybonus.Refresh();
  
  local func_daily = nil;

  if ( m_dailybonus.IsReady() ) then 
  
    func_daily = private.DailyBonus_TryShowDialog;
  
  end;
  
  local notification = "View_Map_Update_DailyBonusDisplayed";
  private.Update_TryAddStep( update, func_daily, notification );

end;
--------------------------------------------------------------------------------
function private.Update_TryUnlockPortal ( update, unlock_portal )

  local func_portal = nil;
  
  if ( unlock_portal ) then
  
    func_portal = private.Portal_Unlock;
  
  end;
  
  local section_name = private.Section_Current_GetName();
  local notification = "View_Map_Update_PortalUnlocked";
  private.Update_TryAddStep( update, func_portal, notification, section_name );

end;
--------------------------------------------------------------------------------
function private.Update_Finish ( update, unlock_portal )

  local function func_finish ( func_after )
  
    local function func_end ()
    
      int.Lock( true );
      public.Interface_Enable( true, "v_map.Update_OnFinish", function ()
      
        private.Achievement_TryShowPanel();
        if ( func_after ) then func_after(); end;
      
      end );
    
    end;
  
    if ( not unlock_portal ) then
      public.FocusAtCurrentPoint( func_end );
    else
      func_end();
    end;
  
  end;

  local notification = "View_Map_Update_Finished";
  private.Update_TryAddStep( update, func_finish, notification );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Reward_ShowCollected ( emitter, count, name, func_after )
  
  local reward =
  {
    name  = name,
    count = count,
    type  = tern( name == "gold" )( "gold", "resource" );
  };
  
  local funcs_buffer_out =
  {
    gold     = function() private.Gold_Buffer_Out() end,
    resource = function() private.Resource_Buffer_Out( name ) end
  };
  
  local function func_end ()
  
    funcs_buffer_out[ reward.type ](); 
    if ( func_after ) then func_after(); end;
  
  end;

  private.Reward_Show_Resource( emitter, reward, func_end );

end;
--------------------------------------------------------------------------------
function public.Reward_Show ( is_map_reward, rewards, reactions, func_after )

  -- Set drop sender
  local stage_sender = "spr_int_dialog_map_reward_item_icon";
  local quest_sender = "spr_int_dialog_map_quest_reward_item_icon";
  
  local sender = tern( is_map_reward )( stage_sender, quest_sender );

  -- Prepare for animation
  int.Lock( true );
  local function func_end ()
  
    int.Lock( false );
    if ( func_after ) then func_after(); end;
  
  end;

  -- Show gold reward
  for i = 1, #rewards do
    if ( rewards[ i ].type == "gold" ) then

      private.Gold_Buffer_Out();
      break;

    end;
  end;

  local func = {
    quest = function ( reward_name )

      private.Reward_DropToMap( sender, reactions.character, func_end );

    end,
    worker = function ()

      private.Reward_DropToMap( sender, reactions.mine, func_end );

    end,
    fragment = function ()

      private.Reward_DropToMap( sender, reactions.build, func_end );

    end,
    skinpart = function ()

      private.Reward_DropToPanelBottom( sender, "hero", func_end );

    end,
    stickerpack = function ()

      private.Reward_DropToPanelBottom( sender, "fireflies", func_end );

    end
  };

  -- Show first item reward
  for i = 1, #rewards do
  
    local reward_type = rewards[ i ].type;

    if ( func[ reward_type ] ) then

      func[ reward_type ]();

      return;

    end;

  end;
  
  func_end();

end;
--------------------------------------------------------------------------------
function private.Reward_DropToMap ( sender, reaction, func_after )

  local target    = reaction.object;
  local func_drop = reaction.func_drop;
  
  if ( reaction.call_drop_first ) then
  
    private.Reward_AnimateDrop( sender, target, func_drop, func_after );
  
  else
  
    private.Reward_AnimateDrop( sender, target, func_after, func_drop );
  
  end;
  
end;
--------------------------------------------------------------------------------
function private.Reward_DropToPanelBottom ( sender, button, func_after )

  local receiver = "btn_int_panel_map_"..button;
  
  local function func_lock ()
    int.Lock( true );
  end;
  
  public.Interface_Enable( true, "RewardDrop", func_lock );
  
  local function func_end ()
  
    common.Animation_Bounce( receiver, function ()
    
      int.Lock( false );
      if ( func_after ) then func_after(); end;
    
    end );
  
  end;
  
  private.Reward_AnimateDrop( sender, receiver, nil, func_end, true );

end;
--------------------------------------------------------------------------------
function private.Reward_AnimateDrop ( 
    sender, 
    receiver, 
    func_start,
    func_end, 
    skip_focus
  )

  -- Setup sound
  local sound = "reward_drop_common";

  if     ( string.find( receiver, "puzzle" ) ) then

    sound = "reward_drop_puzzle";

  elseif ( string.find( receiver, "island" ) ) then

    sound = "reward_drop_island";

  end;

  private.Sound_Play( sound );

  -- Setup animation
  ObjClone( sender, "_drop", 0, "obj_map_drop" );

  local sender_pos = private.Object_Position( sender, InterfaceName );

  ObjSet( "anm_map_drop",
  {
    pos_x = sender_pos[ 1 ],
    pos_y = sender_pos[ 2 ]
  } );
  
  ObjAttach( "anm_map_drop", InterfaceName );

  local func_OnFocusEnd = function ()

    if ( func_start ) then func_start(); end;

    local receiver_pos;

    sender_pos   = private.Object_Position( "anm_map_drop", InterfaceName );
    receiver_pos = private.Object_Position( receiver,       InterfaceName );

    local func_OnAnimationEnd = function ()

      local func_OnAnimationEnd = function ()

        ObjAttach( "anm_map_drop", "hub_map_drop" );
        ObjDelete( sender.."_drop" );

        if ( func_end ) then func_end(); end;

      end;

      ObjAnimate( sender.."_drop", "alp", 0, 0, func_OnAnimationEnd,
      {
        0.0, 0, 1,
        0.3, 0, 0
      } );

      ObjAnimate( sender.."_drop", "scale_xy", 0, 0, "",
      {
        0.0, 0, 0.5, 0.5,
        0.3, 0, 0.0, 0.0
      } );

    end;

    local sender_props = ObjGet( sender.."_drop" );

    ObjAnimate( sender.."_drop", "scale_xy", 0, 0, "",
    {
      0.0, 0, sender_props.scale_x, sender_props.scale_y,
      0.5, 0, 0.5,                  0.5
    } );

    ObjAnimate( "anm_map_drop", "pos_xy", 0, 0, func_OnAnimationEnd,
    {
      0.0, 0, sender_pos[ 1 ],   sender_pos[ 2 ],
      0.5, 0, receiver_pos[ 1 ], receiver_pos[ 2 ]
    } );

  end;

  -- Play animation
  if not ( skip_focus ) then

    AnimPlay( "anm_map_drop", "item_apply", "" );
    public.Focus( receiver,
      {
        scale      = 1.0,
        func_after = func_OnFocusEnd
      }
    );

  else

    AnimPlay( "anm_map_drop", "item_apply", func_OnFocusEnd );

  end;

end;
--------------------------------------------------------------------------------
function private.Reward_Show_Resource ( emitter, reward_data, func_end )

  -- JK: attach anims layer to interface to solve zoom problem
  ObjAttach( "obj_map_anims", InterfaceName );

  ObjSet( "obj_map_anims", { pos_x = 0, pos_y = 0 } );

  local count        = 0;
  local flight_times = {};
  local max_count    = math.min( reward_data.count, 15 );
  local tme          = { 0.45, 0.47, 0.51, 0.54, 0.58, 0.60, 0.64, 0.66 }; 

  -- default: point
  local emitter_obj = "obj_map_point_"..emitter;

  -- spell
  if ( string.find( emitter, "spell" ) ) then

    emitter_obj = "obj_map_spells_"..emitter;

  end;

  -- mine
  if ( string.find( emitter, "mine" ) ) then

    emitter_obj = "obj_map_mines_"..emitter;

  end;
  
  local emitter_pos = private.Object_Position( emitter_obj, InterfaceName );
  
  if ( string.find( emitter, "mine" ) ) then
      
    local positions = private.Object_Position( "obj_map_mines_"..emitter );

    mediator.Notify( "View_Map_MineGathered", 
    { 
      resource    = reward_data.name, 
      delta       = reward_data.count,
      posx        = positions[ 1 ], 
      posy        = positions[ 2 ],
      fontcolor_r = 1,
      fontcolor_g = 1,
      fontcolor_b = 1
    } );

  end;

  for j = 1, max_count do

    count = count + 1;

    local obj_name = "obj_map_flying_resources_"..emitter.."_"..reward_data.name.."_"..count;
    local anm_name = "anm_map_flying_resources_"..emitter.."_"..reward_data.name.."_"..count;
    local res_name = "anm_map_flying_resources_icon_"..emitter.."_"..reward_data.name.."_"..count;

    private.reward[ obj_name ] = true;

    local index = math.random( 1, 8 );
    local pos_x = math.random( - 150, 150 );
    local pos_y = math.random( - 80, 80 );

    ObjClone( "obj_map_flying_resources", "_"..emitter.."_"..reward_data.name.."_"..count, 1, "obj_map_anims" );

    ObjMultiSet( {
      { obj_name,
        { pos_x   = emitter_pos[ 1 ],
          pos_y   = emitter_pos[ 2 ],
          pos_z   = ( 10 + index ),
          scale_x = 0,
          scale_y = 0 } },
      { res_name,
        { animfunc = reward_data.name,
          playing  = true } }
    } );

    local delay = math.random( 100, 500 ) / 1000;    

    local tmr_name = "tmr_map_stage_reward_"..emitter.."_"..reward_data.name.."_"..count;

    ObjCreate( tmr_name, "timer" );
    ObjAttach( tmr_name, obj_name );

    local resource_type = "resource";

    if ( string.find( reward_data.name, "gold" ) ) then

      resource_type = "gold";

    end;

    ObjSet( tmr_name,
    {
      time    = delay,
      playing = true,
      endtrig = function()

        AnimPlay( anm_name, resource_type..index, "" );

      end;
    } );

    local flight_time = tme[ index ] + delay + 0.5;

    table.insert( flight_times, flight_time );

    ObjAnimate( obj_name, "pos_xy", 0, 0,
    function ( params ) 

      local index = cmn.GetObjectPostfix( params.sender );

      private.Reward_Get( emitter, reward_data, index );

    end, 
    {
      0.0,               1, emitter_pos[ 1 ],         emitter_pos[ 2 ],
      0.0 + delay,       1, emitter_pos[ 1 ],         emitter_pos[ 2 ],
      flight_time - 0.5, 1, emitter_pos[ 1 ] + pos_x, emitter_pos[ 2 ] + pos_y,
      flight_time,       1, emitter_pos[ 1 ] + pos_x, emitter_pos[ 2 ] + pos_y
    } );

    ObjAnimate( obj_name, "scale_xy", 0, 0, "", 
    {
      0.00, 0, 0.0, 0.0,
      0.10, 0, 1.2, 1.2,
      0.20, 0, 0.8, 0.8,
      0.25, 0, 1.1, 1.1,
      0.3,  0, 1.0, 1.0      
    } );

  end;

  table.sort( flight_times );

  local max_flight_time = flight_times[ #flight_times ] + 0.5;

  ObjAttach( "tmr_map_reward_"..reward_data.name, InterfaceName );

  local func_OnTimerEnd = function()

    ObjDetach( "tmr_map_reward_"..reward_data.name );

    -- JK: attach anims layer back
    ObjAttach( "obj_map_anims", private.this.body );

    ObjSet( "obj_map_anims", { pos_x = private.offset, pos_y = - private.offset } );

    if ( func_end ) then

      func_end();

    end;

  end;

  ObjSet( "tmr_map_reward_"..reward_data.name,
  {
    time    = max_flight_time,
    playing = true,
    endtrig = func_OnTimerEnd } );

  private.Sound_Play( "fireflies" );

end;
--------------------------------------------------------------------------------
function private.Reward_Get ( stage_name, reward_data, index )

  local resource_name = reward_data.name;

  private.reward_count = private.reward_count or {};

  private.reward_count[ resource_name ] = ( private.reward_count[ resource_name ] or 0 ) + 1;

  local obj_name = "obj_map_flying_resources_"..stage_name.."_"..resource_name.."_"..index;
  local pfx_name = "pfx_map_flying_resources_"..stage_name.."_"..resource_name.."_"..index;
  local anm_name = "anm_map_flying_resources_"..stage_name.."_"..resource_name.."_"..index;

  if ( not private.reward[ obj_name ] ) then

    return;

  end;

  private.reward[ obj_name ] = nil;

  ObjStopAnimate( obj_name, "pos_x" );

  local start_pos = ObjGet( obj_name );

  if ( not private.Resource_IsPanelActive ( resource_name ) ) then

    if ( string.find( reward_data.name, "gold" ) ) then

      private.Gold_ShowPanel();

    else

      private.Resource_ShowPanel( resource_name );

    end;

  end;

  local stop_pos  = private.Object_Position(
    "obj_int_panel_map_"..resource_name,
    "obj_map_anims"
  );

  if ( string.find( resource_name, "resource" ) ) then

    stop_pos[ 2 ] = stop_pos[ 2 ] + 140;

  end;

  local func_end = function ()

    ObjDelete( obj_name );

  end;

  ObjSet( pfx_name, { visible = true, active = true } );

  PartSysStart( pfx_name );

  ObjAnimate( obj_name, "pos_xy", 0, 0, func_end, 
  {
    0.0, 0, start_pos.pos_x, start_pos.pos_y,
    0.5, 0, stop_pos[ 1 ], stop_pos[ 2 ]
  } );

  ObjAnimate( obj_name, "alp", 0, 0, "", 
  {
    0.00, 1, 1,
    0.45, 1, 1,
    0.50, 1, 0,
  } );

end;
--------------------------------------------------------------------------------
function private.Reward_Collect ()

  if ( private.reward ) then

    for key, value in pairs( private.reward ) do

      ObjDelete( key );

      private.reward[ key ] = nil;

    end;

  end;

  if ( private.reward_count ) then

    private.reward_count = {};

  end;

end; 
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.AnimateZone (
    build_name,
    stages_list,
    stages_type,
    func_end
  )
  
  -- Animation callbacks
  local function get_func_appear_stages ( stage_from, stage_to )
  
    return function ( func_after )
    
      private.Stages_Show_Cascade( stage_from, stage_to, nil, stages_type );
      
      local tme = 0.3 * ( stage_to - stage_from );
      
      local stage = private.Stages_GetName( stage_to, stages_type );
      local point = "obj_map_point_"..stage;
      
      public.Focus( point, { time = tme, func_after = func_after } );
    
    end;
  
  end;
  
  -- Build animations list
  local anims = {};
  
  local stage_fight = m_fights.GetStageUnlocker( stages_list, stages_type );
  local stage_char  = m_characters.GetStageUnlocker( build_name );

  local stage_min = stages_list[ 1 ];
  local stage_max = stages_list[ 2 ];
  local stage_cur = stage_min;

  for s = stage_min, stage_max do
  
    if ( s == stage_fight ) or ( s == stage_char ) then
    
      table.insert( anims, get_func_appear_stages( stage_cur, s ) );
      stage_cur = s + 1;
      
      if ( s == stage_fight ) then
      
        local stage = private.Stages_GetName( s, stages_type );
        local func_appear = v_fights.Callback_AnimateAppear( stage );
        table.insert( anims, func_appear );
      
      end;
      
      if ( s == stage_char ) then
      
        local func_appear = v_characters.Callback_AnimateAppear( build_name );
        table.insert( anims, func_appear );
      
      end;
    
    elseif ( s == stage_max ) then
    
      table.insert( anims, get_func_appear_stages( stage_cur, s ) );
    
    end;
  
  end;
  
  -- End animations func
  local function func_finish_anims ( func_after )
  
    local stage = private.Stages_GetName( stage_min, stages_type );
    local autoconstruct = m_builds.IsAutoConstructed( build_name );
    
    if not ( autoconstruct ) then
    
      local tap, no_focus = false, true;
      private.Marker_MoveToPoint( stage, tap, no_focus );
      
      public.Focus( "obj_map_point_"..stage, { func_after = func_after } );
    
    elseif ( func_after ) then
    
      func_after();
    
    end;
  
  end;
  
  table.insert( anims, func_finish_anims );
  
  if ( func_end ) then
    table.insert( anims, func_end );
  end;
  
  common.RunCallbackList( anims );
  
end;
--------------------------------------------------------------------------------