-- name=v_match3
--------------------------------------------------------------------------------
function public.__InitModule ()

  private.app_w = int.W();
  private.app_h = int.H();
  
  private.pos_x = private.app_w / 2;
  private.pos_y = private.app_h / 2;
  
  private.max_size = 500;

  private.active = false;

  private.Trace( "Code initialized." );
  
end;
--------------------------------------------------------------------------------
function public.__InitObjects ()

  private.Trace( "Objects initialized." );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.SetPos ( x, y )
  
  private.pos_x = x;
  private.pos_y = y;
  
end;
--------------------------------------------------------------------------------
function public.SetMaxSize ( size )
  
  private.max_size = size;
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Start ()

  if ( private.active ) then

    public.Finish();

  end;

  private.active = true;

  wiz.ClearUsedResource();

  local data = m_match3.StartData_Get();
  
  private.field_width  = 70 * #data.cells[ 1 ];
  private.field_height = 70 * #data.cells;
  
  private.scale = private.CalcFieldScale( private.field_width, private.field_height );

  ObjAttach( "obj_match3", screen.GetCurrentName() );

  private.Border_Create( data.cells );
  private.Match3_Create( data );

end;
--------------------------------------------------------------------------------
function public.Finish ()

  ObjDetach( "grd_match3_bg" );
  ObjDelete( "obj_match3_field_cells_border" );
  ObjDelete( "match" );

  ObjDetach( "obj_match3" );

  private.active = false;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.GetFieldCenter ()

  return private.pos_x, private.pos_y;

end;
--------------------------------------------------------------------------------
function public.GetFieldScale ()

  return private.scale;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Match3_Create ( stage_data )

  local field_width  = private.field_width;
  local field_height = private.field_height;
  
  local scale = private.scale;

  ObjCreate( "match", "matchbox" );
  ObjSet( "match", 
  { 
    pos_x     = common.Round( private.pos_x - ( 0.5 * field_width  * scale ) ), 
    pos_y     = common.Round( private.pos_y - ( 0.5 * field_height * scale ) ),
    scale_x   = scale,
    scale_y   = scale,
    stage     = stage_data.stage, 
    state     = stage_data.state,
    preset    = stage_data.preset,
    boosters  = stage_data.boosters,
    section   = stage_data.section,
    attempts  = stage_data.attempts,
    max_attempts  = stage_data.max_attempts
  } );
  
  ObjAttach( "match", "obj_match3" );

end;
--------------------------------------------------------------------------------
function private.Border_Create ( cells_table )

  local field_width  = private.field_width;
  local field_height = private.field_height;
  
  local scale = private.scale;

  ObjCreate( "obj_match3_field_cells_border", "obj" );
  
  local properties = ObjGet( "obj_match3_field_cells_default" );
  properties.name = "obj_match3_field_cells_border";
  
  local multiset_table = {};
  
  table.insert( multiset_table, { "obj_match3_field_cells_border", properties } );
  table.insert( multiset_table, { "obj_match3_field_cells_border", 
  { 
    pos_x   = common.Round( private.pos_x - ( 0.5 * field_width  * scale ) ), 
    pos_y   = common.Round( private.pos_y - ( 0.5 * field_height * scale ) ),
    scale_x = scale,
    scale_y = scale
  } } );
  table.insert( multiset_table, { "grd_match3_bg", { pos_x = - 5, pos_y = - 5 } } );
  
  ObjMultiSet( multiset_table );
  
  ObjAttach( "obj_match3_field_cells_border", "obj_match3" );
  ObjAttach( "grd_match3_bg", "obj_match3_field_cells_border" );
  
  local grid_params = 
  {
     0,  { pos_x = 0 },
     1,  { pos_x = 10 },
     2,  { pos_x = field_width },
     3,  { pos_x = field_width + 10 },

     4,  { pos_x = 0 },
     5,  { pos_x = 10 },
     6,  { pos_x = field_width },
     7,  { pos_x = field_width + 10 },

     8,  { pos_x = 0,                pos_y = field_height },
     9,  { pos_x = 10,               pos_y = field_height },
     10, { pos_x = field_width,      pos_y = field_height },
     11, { pos_x = field_width + 10, pos_y = field_height },

     12, { pos_x = 0,                pos_y = field_height + 10 },
     13, { pos_x = 10,               pos_y = field_height + 10 },
     14, { pos_x = field_width,      pos_y = field_height + 10 },
     15, { pos_x = field_width + 10, pos_y = field_height + 10 }
  };

  GridSet( "grd_match3_bg", 0, grid_params );  

  --[[local cols_count = #cells_table[ 1 ];
  local rows_count = #cells_table;

  local multiset_table = {};

  ObjClone( "anm_match3_cell_border", "_corner_1", 1, "obj_match3_field_cells_border" );
  table.insert( multiset_table, { "anm_match3_cell_border_corner_1", 
  { 
    pos_x = - 70 + 70 * 1 - 4,
    pos_y = - 70 + 70 * 1 - 4,  
    scale_x = - 1,
    animfunc = "border_corner",
    playing = true 
  } } );

  ObjClone( "anm_match3_cell_border", "_corner_2", 1, "obj_match3_field_cells_border" );
  table.insert( multiset_table, { "anm_match3_cell_border_corner_2", 
  { 
    pos_x =   70 * cols_count + 4,
    pos_y = - 70 + 70 * 1 - 4,  
    scale_x = 1,
    animfunc = "border_corner",
    playing = true 
  } } );

  ObjClone( "anm_match3_cell_border", "_corner_3", 1, "obj_match3_field_cells_border" );
  table.insert( multiset_table, { "anm_match3_cell_border_corner_3", 
  { 
    pos_x = - 70 + 70 * 1 - 4,
    pos_y =        70 * rows_count + 4,
    scale_x = - 1,   
    scale_y = - 1,
    animfunc = "border_corner",
    playing = true 
  } } );

  ObjClone( "anm_match3_cell_border", "_corner_4", 1, "obj_match3_field_cells_border" );
  table.insert( multiset_table, { "anm_match3_cell_border_corner_4", 
  { 
    pos_x = 70 * cols_count + 4,
    pos_y = 70 * rows_count + 4,  
    scale_y = - 1,
    animfunc = "border_corner",
    playing = true 
  } } );

  for row = 1, rows_count do

    for col = 1, cols_count do

      if ( row == 1 ) then

        ObjClone( "anm_match3_cell_border", "_top_"..row.."_"..col, 1, "obj_match3_field_cells_border" );
        table.insert( multiset_table, { "anm_match3_cell_border_top_"..row.."_"..col, 
        { 
          pos_x = - 35 + 70 * col,
          pos_y = - 70 + 70 * row - 8,
          animfunc = "border",
          playing = true 
        } } );

      end;

      if ( row == rows_count ) then

        ObjClone( "anm_match3_cell_border", "_bot_"..row.."_"..col, 1, "obj_match3_field_cells_border" );
        table.insert( multiset_table, { "anm_match3_cell_border_bot_"..row.."_"..col, 
        { 
          pos_x = - 35 + 70 * col,
          pos_y =        70 * row + 8,
          scale_y = - 1,
          animfunc = "border",
          playing = true 
        } } );

      end;

      if ( col == 1 ) then

        ObjClone( "anm_match3_cell_border", "_left_"..row.."_"..col, 1, "obj_match3_field_cells_border" );
        table.insert( multiset_table, { "anm_match3_cell_border_left_"..row.."_"..col, 
        { 
          pos_x = - 70 + 70 * col - 8,
          pos_y = - 35 + 70 * row, 
          ang = 1.570707,
          animfunc = "border",
          playing = true 
        } } );

      end;

      if ( col == cols_count ) then

        ObjClone( "anm_match3_cell_border", "_right_"..row.."_"..col, 1, "obj_match3_field_cells_border" );
        table.insert( multiset_table, { "anm_match3_cell_border_right_"..row.."_"..col, 
        { 
          pos_x =        70 * col + 8,
          pos_y = - 35 + 70 * row,
          ang = - 1.570707,
          animfunc = "border",
          playing = true 
        } } );

      end;

    end;

  end;

  ObjMultiSet( multiset_table );]]

end;
--------------------------------------------------------------------------------
function private.CalcFieldScale ( w, h )
  
  local max_dim = math.max( w, h );
  local scale = private.max_size / max_dim;
  
  scale = private.Round( scale, 10 );
  
  return math.min( scale, 1.0 );
  
end;
--------------------------------------------------------------------------------
function private.Round ( value, precision )

  precision = precision or 1;
  return math.floor( ( value * precision ) + 0.5 ) / precision;
  
end;
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ MATCH3 ] "..text );

end;
--------------------------------------------------------------------------------