-- name=v_mines
--------------------------------------------------------------------------------
function public.__InitModule ()

  loader.Module_Load( "int_dialog_map_mine" );
  
  -- Active signs array
  private.signs = {};
  
  mediator.Subscribe( "Model_Mines_MineCooldownStop", function ( mine_name )
  
    if ( private.signs[ mine_name ] ) then
    
      private.Sign_Full( mine_name );
    
    end;
  
  end );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Create ( mine )

  local obj_name = ( "obj_map_mines_%s"      ):format( mine.name );
  local btn_name = ( "btn_map_mines_%s"      ):format( mine.name );
  local sgn_name = ( "obj_map_mines_sign_%s" ):format( mine.name );

  ObjCreate( obj_name, "obj" );
  ObjSet( obj_name, { anim_tag = mine.name } );
  ObjAttach( obj_name, "anm_map_"..mine.section );

  ObjClone( "btn_map_mines", "_"..mine.name, 1, obj_name );
  
  local func_click = function () private.Tap( mine.name ); end
  
  int.Control_EventsSet( btn_name, { func_on_click = func_click } );
  int.Control_EventsSet( sgn_name, { func_on_click = func_click } );
  
  private.Set( mine.name, true );

end;
--------------------------------------------------------------------------------
function private.Set ( mine_name, instant )

  local mine = m_mines.GetMineData( mine_name );
  mine.state = mine.state or "none";
  
  local anm_mine = "spr_map_mines_icon_"..mine_name;
  
  private.Progress_Hide( mine.name );
  private.Upgrade_Hide( mine.name );
  private.Sign_Hide( mine.name, instant );
  private.Indicator_Hide( mine.name, instant );
  
  -- States:
  -- none   - collecting
  -- wait   - ready to build
  -- active - producing resources
  -- ready  - full of resources
  
  if ( mine.state == "none" or mine.state == "wait" ) then
  
   if ( mine.progress and mine.progress > 0 ) then
    
      m_object.Set( anm_mine,
        {
          res      = "assets/content/map/mines/anm_mine_base",
          animfunc = "state_"..mine.progress,
          playing  = true 
        }
      );
    
      private.Progress_Show( mine.name, mine.progress, mine.progress_max );
    
    else
    
      m_object.Set( anm_mine,
        {
          res      = "assets/content/map/mines/anm_mine_base",
          animfunc = "static_"..mine.type,
          playing  = true 
        }
      );
    
    end;
    
    if ( mine.state == "wait" ) then
    
      private.Indicator_Show( mine.name, instant );
    
    end;
  
  elseif ( mine.state == "active" or mine.state == "ready" ) then
  
    local level    = mine.level or 1;
    local resource = mine.resource or "resource1";
    
    m_object.Set( anm_mine,
      {
        res      = "assets/content/map/mines/anm_mine_"..resource, 
        animfunc = "upgrade_"..level.."_static",
        playing  = true 
      }
    );
    
    private.Sign_TryShow( mine.name );
    private.Sign_CheckerCreate( mine.name );
  
    if ( mine.type ~= "gold" and level < 3 ) then
    
      private.Upgrade_Show( mine.name, resource );
    
    end;
  
  end;

end;
--------------------------------------------------------------------------------
function private.Progress_Show ( mine_name, current, total )

  current = int.NumberForm( current );
  total   = int.NumberForm( total   );

  local obj_progress = ( "obj_map_mines_progress_%s" ):format( mine_name );
  local txt_progress = ( "txt_map_mines_progress_%s" ):format( mine_name );
  
  local set = {};
  
  m_object.Enable( obj_progress, true, set );
  m_object.Set( txt_progress, 
    {
      active  = true,
      visible = true,
      text    = ( "%s/%s" ):format( current, total )
    }, 
    set
  );
  
  m_object.Multiset( set );

end;
--------------------------------------------------------------------------------
function private.Progress_Hide ( mine_name )

  local obj_progress = ( "obj_map_mines_progress_%s" ):format( mine_name );
  m_object.Disable( obj_progress, true );

end;
--------------------------------------------------------------------------------
function private.Indicator_Show ( mine_name, instant )

  local indicator = "anm_map_mines_indicator_"..mine_name;

  if not ( instant ) then
  
    m_object.Enable( indicator );
    ObjAnimate( indicator, "scale_xy", 0, 0, "", 
      { 
        0.0, 0, 0.0, 0.0,  
        0.3, 0, 1.0, 1.0
      }
    );
  
  else
  
    m_object.Enable( indicator );
  
  end;
  
end;
--------------------------------------------------------------------------------
function private.Indicator_Hide ( mine_name, instant )

  local indicator = "anm_map_mines_indicator_"..mine_name;
  
  if not ( instant ) then
  
    ObjAnimate( indicator, "scale_xy", 0, 0,
      function ()
        m_object.Disable( indicator );
      end, 
      { 
        0.0, 0, 0.0, 0.0,  
        0.3, 0, 1.0, 1.0
      }
    );
  
  else
  
    m_object.Disable( indicator );
  
  end;

end;
--------------------------------------------------------------------------------
function private.Upgrade_Show ( mine_name, resource )

  local button = "btn_map_mines_upgrade_"..mine_name;
  
  int.Button_Create( button,
    {
      title  = "str_int_dialog_map_mine_upgrade_accept",
      design = "upgrade_"..resource
    },
    function ()
      private.DialogUpgrade_Show( mine_name );
    end
  );
  
  m_object.Enable( button );

end;
--------------------------------------------------------------------------------
function private.Upgrade_Hide ( mine_name )

  m_object.Disable( "btn_map_mines_upgrade_"..mine_name );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Sign_CheckerCreate ( mine_name )

  local obj_name = ( "obj_map_mines_%s" ):format( mine_name );
  local tmr_name = ( "tmr_map_mines_%s" ):format( mine_name );
  
  ObjCreate( tmr_name, "timer"  );
  ObjAttach( tmr_name, obj_name );
  
  ObjSet( tmr_name,
  {
    time    = 1.0,
    playing = true,
    endtrig = function()
    
      private.Sign_TryShow( mine_name );
      ObjSet( tmr_name, { time = 1.0, playing = true } );

    end;
  } );

end;
--------------------------------------------------------------------------------
function private.Sign_TryShow ( mine_name )

  local progress = m_mines.Progress_GetCount( mine_name );

  if ( not private.signs[ mine_name ] ) and ( progress.count >= 1 ) then
  
    private.signs[ mine_name ] = true;

    ObjMultiSet( { 
      { "spr_map_mines_sign_icon_"..mine_name, { 
        active   = true, 
        visible  = true, 
        animfunc = int.Icon_Animfunc( progress.name ), 
        playing  = true, 
        alp      = 0.0 
      } }
    } );

    AnimPlay( "anm_map_mines_sign_bg_"..mine_name, "resource_popup_open", "" );

    ObjAnimate( "spr_map_mines_sign_icon_"..mine_name, "alp", 0, 0, "",
    {
      0.0, 0, 0.0,
      0.3, 0, 1.0
    } );
    
    if ( ( progress.count / progress.full ) > 0.99 ) then

      private.Sign_Full( mine_name );

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.Sign_Full ( mine_name )

  ObjAnimate( "obj_map_mines_sign_"..mine_name, "scale_xy", 1, 0, "",
  { 
    0.0, 3, 1.0, 1.0,
    1.0, 3, 1.5, 1.5,
    2.0, 3, 1.0, 1.0
  } );

end;
--------------------------------------------------------------------------------
function private.Sign_Hide ( mine_name, instant )

  private.signs[ mine_name ] = false;

  if not ( instant ) then

    AnimPlay( "anm_map_mines_sign_bg_"..mine_name,
      "resource_popup_close",
      function ()
        private.Sign_Disable( mine_name );
      end
    );

    local alp = ObjGet( "spr_map_mines_sign_icon_"..mine_name ).alp;

    ObjAnimate( "spr_map_mines_sign_icon_"..mine_name, "alp", 0, 0, "",
    {
      0.0, 0, alp,
      0.3, 0, 0.0
    } );

  else

    private.Sign_Disable( mine_name );

  end;

end;
--------------------------------------------------------------------------------
function private.Sign_Disable ( mine_name )

  local obj_sign = "obj_map_mines_sign_"..mine_name;
  local spr_sign = "spr_map_mines_sign_icon_"..mine_name

  ObjStopAnimate( obj_sign, "scale_xy" );
  
  m_object.Disable( spr_sign, true );
  m_object.Set( obj_sign, { scale_x = 1, scale_y = 1 } );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
--[[
  Mine reaction format:
  {
    -- Basic data:
    name       = "build_name",
    func_popup = function ( func_after ) end,
    func_anim  = function ( func_after ) end,
    
    -- Only for reward drop:
    object          = "obj_build_name", -- receiver-object
    func_drop       = function () end,  -- on-drop function
    call_drop_first = true, -- true : func_drop on focus, func_anim on drop
                            -- false: func_anim on focus, func_drop on drop
  }
]]
--------------------------------------------------------------------------------
function public.GetRewardReaction ( stage, rewards )

  local reaction = nil;
  
  if not ( rewards ) then return reaction; end;
  
  -- Find worker
  local worker = nil;
  for i, reward in pairs( rewards ) do
  
    if ( reward.type == "worker" ) then
    
      worker = reward;
      break;
    
    end;
  
  end;
  
  local mine_name = m_mines.GetWorkerMineName( stage );
  
  if not ( worker and mine_name ) then return reaction; end;
  
  local mine = m_mines.GetMineData( mine_name );
  
  reaction =
  {
    name            = mine.name,
    object          = "obj_map_mines_"..mine.name,
    call_drop_first = true,
    func_anim       = function ( func_after )
    
      int.Lock( true );
      
      m_object.Set( "spr_map_mines_icon_"..mine.name,
        {
          res      = "assets/content/map/mines/anm_mine_base",
          animfunc = "state_"..mine.progress,
          playing  = true,
          endtrig  = function ()
          
            int.Lock( false );
            if ( func_after ) then func_after(); end;
          
          end;
        }
      );
    
      private.Progress_Show( mine.name, mine.progress, mine.progress_max );
      
      if ( mine.progress == mine.progress_max ) then
        
          private.Indicator_Show( mine.name );
        
      end;
    
    end
  };
  
  -- Add popup for first seed
  if ( mine.progress == 1 ) then
  
    reaction.func_popup = function ( func_after )
    
      private.Popup_Collect( mine.name, func_after );
    
    end;
  
  end;
  
  return reaction;

end;
--------------------------------------------------------------------------------
function public.GetStageCompleteReaction ( stage )

  local reaction = nil;
  
  local mine_name = m_mines.GetMineByUnlockStage( stage );
  if not ( mine_name ) then return reaction; end;
  
  if ( m_mines.Mine_Type( mine_name ) == "gold" ) then
  
    reaction = 
    {
      name   = mine_name,
      object = "obj_map_mines_"..mine_name,
      call_drop_first = true,
      func_anim       = function ( func_after )
      
        private.Animate_Upgrade( mine_name );
        if ( func_after ) then func_after(); end;
      
      end
    };
  
  end;
  
  return reaction;

end;
--------------------------------------------------------------------------------
function public.Mine_Type ( mine_name )
  
  return m_mines.Mine_Type( mine_name );
  
end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Tap ( mine_name )

  local collected = m_mines.TryCollectResource( mine_name );
  local mine_data = m_mines.GetMineData( mine_name );
  local obj_name  = "obj_map_mines_"..mine_name;
  
  mine_data.state = mine_data.state or "none";

  if ( collected ) then

    local func_OnFocus = function ()

      private.Sign_Hide( mine_name );
      private.Animate_Get( mine_data );
      v_map.Reward_ShowCollected( mine_name, collected.count, collected.name );
      
    end;
    
    v_map.Focus( obj_name, { func_after = func_OnFocus } );

  else

    if ( mine_data.state == "none" ) then

      private.Popup_Collect( mine_name );

    elseif ( mine_data.state == "wait" ) then

      if ( mine_data.price > 0 ) then v_map.TopPanel_Lock(); end;

      v_map.Focus( obj_name,
        {
          func_after = function () private.DialogSelect_Show( mine_name ); end
        }
      );

    else

      private.Popup_Wait( mine_name );

    end;

  end;

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.DialogUpgrade_Show ( mine_name )

  local mine_data    = m_mines.GetMineData( mine_name );
  local upgrade_data = m_mines.GetMineUpgradeData( mine_name );

  local func_accept = function ()

    if ( m_mines.TryUpgradeMine( mine_name ) ) then

      private.Animate_Upgrade( mine_name );

    end;

  end;

  local func_close = function ()

    private.DialogUpgrade_Hide();

  end;

  local func_start = function ()

    local func_top_panel_show = function ()

      int_panel_map_top.Highlight( "gold" );
      v_map.TopPanel_Show();

    end;

    local func_top_panel_hide = function ()

      v_map.TopPanel_Hide();

    end;

    int_dialog_map_mine.Upgrade_Show(
      mine_data.resource,
      upgrade_data,
      func_accept,
      func_close,
      func_top_panel_show,
      func_top_panel_hide
    );

  end;

  v_map.Focus( "obj_map_mines_"..mine_name, { func_after = function () 

    int_window.Show( "int_dialog_map_mine", func_start );

  end } );

end;
--------------------------------------------------------------------------------
function private.DialogUpgrade_Hide ()

  local func_close = function ()

    int_window.Hide( "int_dialog_map_mine" );

  end;

  int_dialog_map_mine.Upgrade_Hide( func_close );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.DialogSelect_Show ( mine_name )

  local mine_data      = m_mines.GetMineData( mine_name ); 
  local mine_price     = mine_data.price;
  local produce_data   = mine_data.produce;

  local resource_types = { "gold" };

  if ( mine_data.type == "resource" ) then
  
    resource_types = m_mines.GetResourcesTypes( mine_name );
    
  end;

  local func_accept = function ( resource )

    if ( resource ) then

      if ( m_mines.TrySetMineResource( mine_name, resource ) ) then

        private.Animate_Upgrade( mine_name );

      end;

    end;

  end;

  local func_close = function ()

    private.DialogSelect_Hide();

  end;

  local func_start = function ()

    int_dialog_map_mine.Select_Show(
      mine_data.type,
      resource_types,
      mine_price,
      produce_data,
      func_accept,
      func_close
    );  

  end;

  int_window.Show( "int_dialog_map_mine", func_start );

end;
--------------------------------------------------------------------------------
function private.DialogSelect_Hide ()

  local func_close = function ()

    int_window.Hide( "int_dialog_map_mine" );

  end;

  int_dialog_map_mine.Select_Hide( func_close );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Animate_Upgrade ( mine_name, func_after )

  local mine_data = m_mines.GetMineData( mine_name ); 
  local level     = mine_data.level or 1;
  local resource  = mine_data.resource or "resource1";
  local anm_name = "spr_map_mines_icon_"..mine_name;
  
  cmn.PlaySnd( "unspell" );
  
  m_object.Set( anm_name,
    {
      visible = true,
      res     = "assets/content/map/mines/anm_mine_"..resource
    }
  );
  
  local appear = "upgrade_"..level.."_appear";
  local static = "upgrade_"..level.."_static";
  
  -- Preanim
  if ( mine_data.type == "resource" ) then
  
    private.Upgrade_Hide( mine_name );
  
    if ( level == 1 ) then
    
      private.Indicator_Hide( mine_name, true );
      private.Progress_Hide( mine_name );
      
    end;
  
  end;
  
  if ( level == 1 ) then
  
    private.Sign_CheckerCreate( mine_name );
  
  end;
  
  -- Postanim
  local function func_anim_end ()
  
    if ( mine_data.type == "resource" and level < 3 ) then
    
      private.Upgrade_Show( mine_name, resource );
      
    end;
    
    AnimPlay( anm_name, static, "" );
    private.Sign_TryShow( mine_name );
    
    if ( func_after ) then func_after(); end;
  
  end;
  
  AnimPlay( anm_name, appear, func_anim_end );

end;
--------------------------------------------------------------------------------
function private.Animate_Get ( mine_data, func_after )
  
  local level    = mine_data.level or 1;
  local resource = mine_data.resource or "resource1";
  local anm_name = "spr_map_mines_icon_"..mine_data.name;
  
  m_object.Set( anm_name,
    {
      visible = true,
      res     = "assets/content/map/mines/anm_mine_"..resource
    }
  );
  
  local action = "upgrade_"..level.."_get";
  local static = "upgrade_"..level.."_static";
  
  AnimPlay( anm_name, action, function ()
  
    AnimPlay( anm_name, static, "" );
    if ( func_after ) then func_after(); end;
  
  end );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Popup_Collect ( mine_name, func_after )

  local text      = "str_int_popup_cloud_mines_"..mine_name;
  local indicator = "obj_map_mines_indicator_"..mine_name;
  
  local funcs = { func_preclose = func_after };
  
  v_map.ShowObjectPopup( "monolog", indicator, text, nil, nil, funcs );

end;
--------------------------------------------------------------------------------
function private.Popup_Wait ( mine_name )

  local text      = "str_int_popup_cloud_mines_started";
  local indicator = "obj_map_mines_indicator_"..mine_name;

  if ( m_mines.Mine_Type( mine_name ) == "gold" ) then

    text = text.."_gold";

  end;

  v_map.ShowObjectPopup( "monolog", indicator, text );

end;
--------------------------------------------------------------------------------