-- name=v_settings
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ SETTINGS ] "..text );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Init ()

  loader.Module_Load( "int_dialog_options" );
  loader.Module_Load( "int_dialog_language" );  
  
  common.MessageSubscribe(
    "Event_Application_ToggleFullScreen",
    function ( msg, params )
      
      private.Toggle_FullScreen(params);
      
    end
  );
  
  common.MessageSubscribe(
    "Command_OptionsMenuShow",
    function ( msg )
   
      if ( v_map.Interface_IsVisible() ) then
        
        private.DialogOptions_Show();
      
      end;
    
    end
  );

  mediator.Subscribe(
    "Model_Social_LoginComplete",
    int_dialog_options.OnLogin
  );
  
  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Dialog_Show ( func_on_hide )

  private.DialogOptions_Show( func_on_hide );

end;
--------------------------------------------------------------------------------
function private.DialogOptions_Show ( func_on_hide )

  -- States
  local funcs_state = {};

  funcs_state.music      = cmn.GetMusic;
  funcs_state.sound      = cmn.GetSound;
  funcs_state.fullscreen = function () return GetFullScreen();    end;
  funcs_state.widescreen = function () return GetWideScreen();    end;
  funcs_state.cursor     = function () return not GetSysCursor(); end; 
  funcs_state.language   = function ()

    private.Language_Show();
    
  end;      
  
  -- Toggles
  local funcs_toggle = {
    music = cmn.SetMusic,
    sound = cmn.SetSound
  };

  funcs_toggle.fullscreen = function ()

    local is_fullscreen = GetFullScreen();
    
    SetFullScreen( not is_fullscreen );
    return not is_fullscreen;

  end;
  
  funcs_toggle.widescreen = function ()

    local func_ws_yes = function()
      
      int_dialog_info.Hide();
      SetWideScreen( not GetWideScreen() );
    
    end;
    
    local func_ws_no = function()
      
      int_dialog_info.Hide();
      
    end;
    
    local message_type = tern( v_map.IsActive() )( "widescreen_map", 
                                                   "widescreen_stage" );
    
    int_dialog_info.Show( message_type, func_ws_yes, func_ws_no );

  end;

  funcs_toggle.cursor = function ()

    SetSysCursor( not GetSysCursor() );
    return not GetSysCursor();

  end;

  -- Social
  local funcs_social = {
    is_logged = m_social.IsLogged,
    login     = v_social.Login,
    logout    = v_social.Logout,
    get_info  = m_social.GetSocialInfo
  };

  -- Other
  local funcs_other = {};

  funcs_other.close = function ()

    int_dialog_options.Hide( func_on_hide );

  end;

  funcs_other.reset = function ()

    int_dialog_info.Show(
      "savedelete",
      m_settings.DeleteSave,
      int_dialog_info.Hide
    );

  end;
  
  funcs_other.quit = function ()

    int_dialog_info.Show( "exitconfirm", wiz.Quit, int_dialog_info.Hide );
    
  end;
  
  funcs_other.is_online = function ()
  
    if ( m_client.IsOnline() ) then return true; end;
  
    v_client.SystemDialog_Show( "connectionlost", function () end );
    return false;
  
  end;

  int_dialog_options.Show(
    funcs_state,
    funcs_toggle,
    funcs_social,
    funcs_other
  );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Toggle_FullScreen ( params )

  local togglefullscreen = params.togglefullscreen; 
  local togglewidescreen = params.togglewidescreen;

  if ( togglefullscreen == false ) and
     ( togglewidescreen == false ) then

    private.Trace( "[ ! ] Change screen mode: impossible" );

    int_dialog_info.Show( "togglescreenmode", function () end, function () end );

  elseif ( togglewidescreen == true ) then
      
    ObjSet( "ng_application", { color_r = 0.0, color_g = 0.0, color_b = 0.0 } );
    common.Return( true );
      
  else

    int_dialog_options.OnFullScreenToggle( params );

  end;
    
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Language_Show ()
    
  local language = wiz.settings.GetLanguage();
  local list = { "fr", "it", "de", "es", "ja", "ru", "nl", "pt", "ko", "en" };  
  
  local func_select = function ( language )
    int_dialog_info.Show( "languageconfirm", function ()
      wiz.settings.SetLanguage( language );
      common.Return( true );
    end )
  end;  
  
  int_dialog_language.Show( language, list, func_select );
  
end;
--------------------------------------------------------------------------------