-- name=v_shop
 public.Panel = {};
--------------------------------------------------------------------------------
function public.Init ()

  private.Trace( "Initialized." );
  
  loader.Modules_Load({
    "int_dialog_shop",
    "int_dialog_starterpack"
  });
  
  private.Subscribe();
  private.SetDialogHandlers();

  wiz.bfg.lot.InitializePurchases();

  private.dialog_notify = false;

  --local is_sale = GetSaleInfo().sale;  -- Sales on PC

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Panel.Show ()
  
  local starterpack = private.Starterpack_Data();
  
  if ( starterpack and m_shop.Starterpack_IsAvailable( starterpack.name ) ) then
    
    int_panel_map_right.Sale_Enable( 
      private.Starterpack_DialogShow,
      private.Starterpack_GetRemainingTime
    );

    int_panel_map_right.Sale_Warn( 0 );
    
  end;
  
end;
--------------------------------------------------------------------------------
function private.Starterpack_GetRemainingTime ()

  if ( not TimeIsEventExist( "mte_starterpack_cooldown" ) ) then
    
    return 0;
    
  end;
  
  return TimeGetEventRemainingTime( "mte_starterpack_cooldown" );
  
end;
--------------------------------------------------------------------------------
function public.Show ( tab, deficient )

  tab = tab or "gold";
  
  int_panel_map_bottom.Shop_SetNotification( 0 );
  
  local answer = GetLotsInfo();

  local purchase_lots = nil;

  if ( answer ) then purchase_lots = answer.lots; end;

  local gold_lots = m_shop.Gold_GetLots( "coins" );

  if ( purchase_lots ) and ( #purchase_lots > 0 ) then

    for p = 1, #purchase_lots do

      local purchase_lot = purchase_lots[ p ];

      for i = 1, #gold_lots do

        local lot = gold_lots[ i ];

        if ( lot.name_ios == purchase_lot.name_ios ) then

          lot.price = purchase_lot.price;

          break;

        end;

      end;

    end;

  end;

  local boosters_list = m_shop.Boosters_GetLots();
  
  m_client.ConnectionReconnect( false );

  int_dialog_shop.Show( tab, gold_lots, boosters_list );

end;
--------------------------------------------------------------------------------
function public.Notify ( flag )

  private.dialog_notify = flag;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Cheat_Starterpack ()

  local starterpack = private.Starterpack_Data();

  if ( starterpack                                        ) and 
     ( m_shop.Starterpack_IsCompleted( starterpack.name ) ) then

    m_shop.Starterpack_Clear( starterpack.name );
    m_shop.Starterpack_Start( starterpack.name );

    return;

  end;

  m_shop.Starterpack_Finish( starterpack.name );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ BANK ] "..text );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Subscribe ()

  mediator.Subscribe(
    "Model_Player_GoldCountChangeFailed",
    private.Gold_OnDeficient
  );
  
  mediator.Subscribe(
    "Model_Player_ResourceCountChangeFailed",
    private.Resources_OnDeficient
  );

  ne.msg.Subscribe( Event_Purchase_Finished, private.OnPurchaseFinish );
  ne.msg.Subscribe( Event_Purchase_Disabled, private.OnPurchaseDisable );
  ne.msg.Subscribe( 
    "Event_Purchase_ProductInformationReady",
    private.OnProductInformationReady
  );

  mediator.Subscribe( "Model_Market_Refresh", function()
 
    if ( int_dialog_shop.IsActive() ) then
      
      if ( int_dialog_shop.CurrentTab() == "market" ) then

        int_dialog_shop.Market_Sell_SetDefaults();

      else
        
        int_dialog_shop.Market_SetNotification( true );
        
      end;
    
    else
    
      int_panel_map_bottom.Shop_SetNotification( 1 );
      int_dialog_shop.Market_SetNotification( true );
    
    end;
 
  end );

  mediator.Subscribe(
    "Model_Shop_Starterpack_Start",
    private.Starterpack_OnStart
  );

  mediator.Subscribe(
    "Model_Shop_Starterpack_Finish",
    private.Starterpack_OnFinish
  );

end;
--------------------------------------------------------------------------------
function private.SetDialogHandlers ()

  local func_buy_gold = function ( lot_name )

    if ( m_client.IsOnline() ) then
      
      private.shop_purchase = true;

      local query_name_ios =
      {
        query      = "find",
        group      = "static",
        collection = "lots",
        indice     = "name",
        criteria   = { like = lot_name }
      };

      local lot_id = m_storage.Execute( query_name_ios ).result.name_ios;

      private.PurchaseLot( lot_id );

    else

      m_client.ConnectionReconnect( false );
      v_client.SystemDialog_Show( "nointernetconnection", function () end );

    end;

  end;

  int_dialog_shop.SetGoldHandlers( func_buy_gold );


  --
  local func_buy_booster = function ( booster_name, pack_level )

    if ( m_boosters.TryBuy( booster_name, pack_level, "shop" ) ) then

      cmn.PlaySnd( "buy" );
      return true;

    end;

    return false;

  end;

  local func_info_booster = function ( booster_name ) 

    return v_boosters.Tap( "shop", booster_name );

  end;

  int_dialog_shop.SetBoostersHandlers( func_buy_booster, 
                                       func_info_booster );

  --
  int_dialog_shop.SetMarketHandlers( private.Market_Share_Value,
                                     private.Market_Share_Price,
                                     private.Market_Share_Limit,
                                     private.Market_GetShares_Sell,
                                     private.Market_TryBuy,
                                     private.Market_TrySell,
                                     private.Market_RefreshTime );

  --
  local func_on_tab_open = function ( tab_name )
    
    local events = 
    {
      gold     = "hc_enter",
      boosters = "boost_purchase_open"
    };
    
    if ( events[ tab_name ] ) then
      
      common.Playhaven_LogPlacement( events[ tab_name ] );
      
    end;

    mediator.Notify( "View_Shop_TabOpened", tab_name );

  end;

  local func_on_tab_close = function ( tab_name )

    local events = 
    {
      gold     = "hc_exit",
      boosters = "boost_purchase_close"
    };
    
    if ( events[ tab_name ] ) then
      
      common.Playhaven_LogPlacement( events[ tab_name ] );
      
    end;    

  end;

  local func_is_market_available = function ()
    return ( m_market.IsUnlocked() and m_market.IsBuilt() );
  end;
  
  int_dialog_shop.SetGenericHandlers( v_map.TopPanel_Lock,
                                      v_map.TopPanel_Unlock,
                                      func_on_tab_open,
                                      func_on_tab_close,
                                      func_is_market_available,
                                      int.ShowPopupFlat,
                                      int.HidePopupFlat );

end;
--------------------------------------------------------------------------------
function private.PurchaseLot ( lot_id )

  mediator.Notify(
    "View_Shop_PurchaseStart",
    lot_id
  );

  private.connection_pending = true;
  int.WaitingShow();

  wiz.bfg.lot.Buy( lot_id );

end;
--------------------------------------------------------------------------------
function private.OnPurchaseFinish ( msg, params )

  private.connection_pending = false;
  int.WaitingHide();

  private.Premium_TryBuyLot( params );

  mediator.Notify(
    "View_Shop_PurchaseFinish",
    params
  );

end;
--------------------------------------------------------------------------------
function private.OnPurchaseDisable ( msg, params )

  private.connection_pending = false;

  local str   = "str_systemmessage_inapp_purchase_disabled_";

  wiz.ShowSystemMessage(
  {
    title   = StringGet( str.."title" ),
    body    = StringGet( str.."body" ),
    buttons =
    {
      {
        title  = StringGet( str.."button1" ),
        action = function () int.WaitingHide(); end;
      }
    }
  } );

end;
--------------------------------------------------------------------------------
function private.OnProductInformationReady ( msg, params )

  private.product_info_ready = true;

  local purchase_lots = GetLotsInfo().lots;
  local database_lots = m_shop.Gold_GetLots( "coins" );
  
  for _, database_lot in pairs( database_lots ) do
    for _, purchase_lot in pairs( purchase_lots ) do
    
      if ( database_lot.name_ios == purchase_lot.name_ios ) then
      
        database_lot.price = purchase_lot.price;
        break;
      
      end;
    
    end;
  end;
  
  int_dialog_shop.UpdateLots( database_lots );
  int_dialog_minibank.UpdateLots( database_lots );
  
  local starterpack = private.Starterpack_Data();
  local sp_name_ios = starterpack.name_ios;
  
  for _, purchase_lot in pairs( purchase_lots ) do
    if ( sp_name_ios == purchase_lot.name_ios ) then
      
      starterpack.price = purchase_lot.price;
      break;
    
    end;
  end;
  
  int_dialog_starterpack.UpdatePrice( starterpack.price );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Premium_TryBuyLot ( params )
  
  local source = { name = "shop" };

  if ( private.minibank_purchase ) then
    
    source = private.minibank_purchase;
    source.name = "minibank";

    if ( params.result ) then

      int_dialog_minibank.Hide();

    end;

  end;

  if ( not params.result ) or 
     ( not params.id )   or 
     ( params.id == "" ) then

    private.shop_purchase = false;

    -- PC: params.canceled always false.
    if ( IsMobile() ) then
      
      local purchase_message = tern( params.canceled )( "purchasecanceled", 
                                                        "purchasefailed" );

      if not ( private.product_info_ready or loader.IsProductInfoReady() ) then
        purchase_message = "nointernetconnection";
      end;

      v_client.SystemDialog_Show( purchase_message, function () end );

    end;

    return;

  end;

  cmn.PlaySnd( "buy" );
  
  if ( params.id:find( "lives" ) ) then
    
    m_hearts.Premium_HeartsAdd( params.id, params.transaction_id );
    
    return;
    
  end;

  if ( params.id:find( "starterpack" ) ) then
    
    m_shop.Starterpack_Buy( params.id, params.transaction_id );
    
    return;
    
  end;
  
  local lot_data = m_shop.Gold_GetLotByIOSName( params.id );

  m_shop.Gold_TryBuyLot( 
    lot_data.name, 
    lot_data.currency_type, 
    source, 
    params.transaction_id
  );
  private.shop_purchase = false;
  private.minibank_purchase = false;

end;
--------------------------------------------------------------------------------
function private.Gold_OnDeficient ( params )

  local deficient = params;
  deficient.name = "gold";

  local answer = GetLotsInfo();

  local purchase_lots = nil;

  if ( answer ) then

    purchase_lots = answer.lots;

  end;

  local gold_lots = m_shop.Gold_GetLots( "coins" );
  
  deficient.lots = {};
  
  if ( purchase_lots ) and ( #purchase_lots > 0 ) then

    for p = 1, #purchase_lots do

      local purchase_lot = purchase_lots[ p ];

      for i = 1, #gold_lots do

        local lot = gold_lots[ i ];

        if ( lot.currency_count >= deficient.count ) and
           ( #deficient.lots < 2 ) then
          if ( not deficient.lots[ 1 ] ) or
             ( deficient.lots[ 1 ] and deficient.lots[ 1 ].currency_count < lot.currency_count ) then
            lot.number = i;
            table.insert( deficient.lots, lot );
          end;
        end;

        if ( lot.name_ios == purchase_lot.name_ios ) then

          lot.price = purchase_lot.price;

          break;

        end;

      end;

    end;

  end;
  
  if ( #deficient.lots == 1 ) or ( #deficient.lots == 2 and
     ( deficient.lots[ 1 ] == deficient.lots[ 2 ] ) ) then
     deficient.lots[ 2 ] = gold_lots[ #gold_lots - 1 ];
     deficient.lots[ 2 ].number = #gold_lots - 1;
  end;

  local func_accept = function ( lot_name )
    
    if ( m_client.IsOnline() ) then

      local query_name_ios =
      {
        query      = "find",
        group      = "static",
        collection = "lots",
        indice     = "name",
        criteria   = { like = lot_name }
      };

      local lot_id = m_storage.Execute( query_name_ios ).result.name_ios;

      private.PurchaseLot( lot_id );

    else

      m_client.ConnectionReconnect( false );
      v_client.SystemDialog_Show( "nointernetconnection", function () end );

    end;

  end;

  local func_reject = function () 

    if ( private.minibank_purchase ) then

      common.Kontagent_LogEvent( "economy", 
      {
        description = "context_offer",
        category    = "",
        value       = 0,
        name        = private.minibank_purchase.lots[ 1 ].name_ios,
        level       = 0
      } );
      
      private.minibank_purchase = false;

    end;
    
  end;

  private.minibank_purchase = { lots = deficient.lots };

  m_client.ConnectionReconnect( false );
  
  int_dialog_minibank.Show( deficient, func_accept, func_reject );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Resources_OnDeficient ( params )

  local deficient = params;

  if ( #params == 0 ) then

    deficient = { params };

  end;

  local func_solution = function () end;
  local solution      = "stage";

  local solved = false;

  -- JK: Solution 1. Market
  -- GD: Hided because of possibility to purchase 
  -- resources in the BuildConstruct window.
  --[[if ( m_market.IsBuilt() ) then

    solved        = true;
    solution      = "market";
    func_solution = function () 

      public.Show( "market" );

    end;

  end;]]

  -- JK: Solution 2. Spell
  if not ( solved ) then

    local spell_data = m_spells.Usable_Search( deficient[ 1 ].name );

    if ( spell_data ) then

      solved        = true;
      solution      = "spell";
      func_solution = function ()

        private.Resources_FocusOnSolution( "obj_map_spells_"..spell_data.name, 
          function()

            common.Animation_Bounce( "obj_map_spells_"..spell_data.name,
              function()
             
                v_map.ShowObjectPopup(
                  "monolog",
                  "obj_map_spells_"..spell_data.name,
                  "str_int_popup_cloud_spell_usable",
                  nil,
                  { 0, -100 }
                );
              
              end );

          end );

      end;

    end;

  end;

  -- JK: Solution 3. Mine
  if not ( solved ) then

    local mine_data = m_mines.Usable_Search( deficient[ 1 ].name );

    if ( mine_data ) then

      solved        = true;
      solution      = "mine";
      func_solution = function ()

        private.Resources_FocusOnSolution( "obj_map_mines_"..mine_data.name );

      end;

    end;

  end;

  -- JK: Solution 4. Stage
  if not ( solved ) then

    local stage_data = m_stage.Usable_Search( deficient[ 1 ].name );

    if ( stage_data ) then

      solved        = true;
      solution      = "stage";
      func_solution = function ()

        private.Resources_FocusOnSolution( "obj_map_point_"..stage_data.name,
          function ()

            common.Animation_Spin( "obj_map_point_"..stage_data.name,
              function()

                local focus_name = string.format( "fcs_map_point_%s",
                                                  stage_data.name );

                ObjAnimate( focus_name, "alp", 0, 0, "", 
                {
                  0.0, 2, 0, 
                  0.3, 2, 1,
                  0.6, 2, 0,
                  0.9, 2, 1,
                  1.2, 2, 0
                } );

            end );

        end );

      end;

    end;

  end;

  if not ( solved ) then

    DbgTrace( "[ VIEW SHOP ] Deficient solution not found!" );

  end;

  
  local func_accept = function ()
  
    private.Deficient_Hide( func_solution );
  
  end;

  local func_reject = function ()

    private.Deficient_Hide();

  end;

  local func_skin_attach = function ( destination )

    return v_hero.Skin_Attach( destination );

  end;

  local func_skin_detach = function ( anm_name )

    v_hero.Skin_Detach( anm_name );

  end;

  private.Deficient_Show(
    deficient,
    func_accept,
    func_reject,
    func_skin_attach,
    func_skin_detach,
    solution
  );

end;
--------------------------------------------------------------------------------
function private.Resources_FocusOnSolution ( obj_name, func_end )

  int_window.HideAll( function ()

    v_map.Focus( obj_name, { func_after = func_end } );

  end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Deficient_Show (
    deficient,
    func_accept,
    func_reject,
    func_skin_attach,
    func_skin_detach,
    solution
  )
  
  int_window.Show( "int_dialog_deficient", function()
 
    int_dialog_deficient.Show(
      deficient,
      func_accept,
      func_reject,
      func_skin_attach,
      func_skin_detach,
      solution
    );
 
  end );
  
end;
--------------------------------------------------------------------------------
function private.Deficient_Hide ( func_end )
  
  int_dialog_deficient.Hide( function()
  
    int_window.Hide( "int_dialog_deficient", func_end );

  end );
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function public.Market_TryBuild ( gold_count, func_SetObject, func_SetTap )

  if ( m_market.TryBuild( gold_count ) ) then

    private.Market_Update( func_SetObject, func_SetTap );

  end;

end;
--------------------------------------------------------------------------------
function public.Market_Update ( func_SetObject, func_SetTap )

  private.Market_Update( func_SetObject, func_SetTap );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Market_Update ( func_SetObject, func_SetTap )

  local unlocked = m_market.IsUnlocked();
  local built    = m_market.IsBuilt();

  if ( unlocked ) then

    func_SetObject( unlocked, built );

  else

    func_SetObject( unlocked, built );
    func_SetTap( "popup" );

    return;

  end;

  if ( built ) then

    func_SetTap( "open" );

  else

    func_SetTap( "build" );

  end;

end;
--------------------------------------------------------------------------------
function private.Market_TryBuy ( resources, subtotal )

  if ( m_player.TryChangeGoldCount( - subtotal ) ) then

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_spend",
      category    = "market",
      value       = - subtotal,
      name        = "buy"
    } );

    for i = 1, #resources do

      local share_value = private.Market_Share_Value( "buy", i );

      if ( m_player.TryChangeResourceCount( "resource"..i, resources[ i ] ) ) then

        common.Kontagent_LogEvent( "economy", 
        {
          description = "soft_currency_collect",
          category    = "market",
          value       = resources[ i ],
          name        = "buy",
          type        = "resource"..i
        } );

      end;
  
    end;    

    return true;

  end;

  return false;

end;
--------------------------------------------------------------------------------
function private.Market_TrySell ( resource_index )

  local resources = private.Market_GetShares_Sell();

  if ( m_player.TryChangeResourceCount( "resource"..resource_index, - resources[ "resource"..resource_index ] ) ) then

    common.Kontagent_LogEvent( "economy", 
    {
      description = "soft_currency_spend",
      category    = "market",
      value       = - resources[ "resource"..resource_index ],
      name        = "sell",
      type        = "resource"..resource_index
    } );

    local share_value = private.Market_Share_Value( "sell", resource_index );
    local share_price = private.Market_Share_Price( "sell", resource_index );
    local gold_amount = math.floor( resources[ "resource"..resource_index ] / share_value * share_price );

    m_player.TryChangeGoldCount( gold_amount );

    common.Kontagent_LogEvent( "economy", 
    {
      description = "premium_currency_collect",
      category    = "market",
      value       = gold_amount,
      name        = "sell"
    } );

    resources[ "resource"..resource_index ] = 0;

    m_market.Offers_Save( resources );

    return true;

  else

    --DbgTrace( "not enough res: "..resources[ "resource"..resource_index ] );
    return false;

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Market_Share_Value ( type, index )

  local shares = m_market.Settings( type );

  return shares[ index ].value;

end;
--------------------------------------------------------------------------------
function private.Market_Share_Price ( type, index )

  local shares = m_market.Settings( type );

  return shares[ index ].price;

end;
--------------------------------------------------------------------------------
function private.Market_Share_Limit ( type, index )

  local shares = m_market.Settings( type );

  return shares[ index ].max;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Market_GetShares_Sell ()

  return m_market.GetShares_Sell();

end;
--------------------------------------------------------------------------------
function private.Market_RefreshTime ()

  local secs = m_market.Refresh_Time();
  
  if ( secs < 1 ) then return false; end;
  
  local time = cmn.GetTimeFromSec( secs );

  return string.format( "%s:%s:%s", time.h, time.m, time.s );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Starterpack_Data ()

  -- NOTE: Do not check paying status until we have just single lot
  local paying_status = 1; -- m_player.PayingStatus_Get();
  local starterpack   = m_shop.Starterpack_GetLotByIndex( paying_status );

  return starterpack;

end;
--------------------------------------------------------------------------------
function private.Starterpack_DialogShow ()

  local starterpack = private.Starterpack_Data();
  starterpack.price = m_shop.Starterpack_GetActualPrice( starterpack.name_ios ) or starterpack.price;
  
  local func_accept = function ()
    
    if ( m_client.IsOnline() ) then
      
      private.shop_purchase = true;

      local query_name_ios =
      {
        query      = "find",
        group      = "static",
        collection = "lots",
        indice     = "name",
        criteria   = { like = starterpack.name }
      };

      local lot_id = m_storage.Execute( query_name_ios ).result.name_ios;

      private.PurchaseLot( lot_id );

    else
      
      m_client.ConnectionReconnect( false );
      v_client.SystemDialog_Show( "nointernetconnection", function () end );

    end;

  end;
  
  local func_reject = private.Starterpack_DialogHide;
  
  m_client.ConnectionReconnect( false );
  
  int_window.Show(
    "int_dialog_starterpack",
    function ()
      
      int_dialog_starterpack.Show( starterpack, func_accept, func_reject );

    end
  );

end;
--------------------------------------------------------------------------------
function private.Starterpack_DialogHide ()

  local func_window_hide = function ()
    
    int_window.Hide( "int_dialog_starterpack" );
    
  end;
  
  int_dialog_starterpack.Hide( func_window_hide );
  
end;
--------------------------------------------------------------------------------
function private.Starterpack_OnStart ()

  public.Panel.Show();

end;
--------------------------------------------------------------------------------
function private.Starterpack_OnFinish ( params )

  if ( params.disable ) then

    int_panel_map_right.Sale_Disable();
    int_panel_map_right.Sale_Warn( 0 );

  end;

  if ( int_window.IsOnScreen( "int_dialog_starterpack" ) ) then

    private.Starterpack_DialogHide();

  end;

end;
--------------------------------------------------------------------------------