-- name=v_social
public = {
  Mail = {};
  Hearts = {};
  Panel = {
    Mail = {};
    Social = {};
  };
  Score = {};
  FriendsChoose = {};
};
private = {
  Mail = {};
  Hearts = {};
  FriendsChoose = {};
  Panel = {};
};
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ SOCIAL ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()
  
  loader.Modules_Load( {
    "int_dialog_social_mail"
  } );
  
  mediator.Subscribe( "Model_Client_ConnectionLost",     private.UpdateOnline );
  mediator.Subscribe( "Model_Client_ConnectionRestored", private.UpdateOnline );

  private.userId = tonumber( m_player.GetUserId() );
    
  if ( IsMobile() ) then
    
    loader.Modules_Load( {
      "int_dialog_social_friends_choose",
      "int_panel_social_score",
      --[["int_dialog_ingame_friends_nickname",
      "int_dialog_ingame_friends"]]
    } );
      
    local settings_data       = m_storage.Settings( "hearts" );
    private.heart_cooldown    = settings_data.social_cooldown;
    private.heart_max_friends = settings_data.social_max_friends_asked;

    private.ask_heart_token = "tmr_social_heart_cooldown_asked";

    private.Trace( "Initialized." );
    
    --int_panel_map_right.Friends_Enable( private.Panel.FriendsTap );
    
  end;
  
end;
--------------------------------------------------------------------------------
function public.Start ()
  
  private.is_logged = m_social.IsLogged();
  private.is_online = m_client.IsOnline();

  private.Mail.messages = {};
  
  mediator.Subscribe(
    "Model_Email_MessageListUpdated", 
    private.Mail_UpdateList
  );
  
  mediator.Subscribe(
    "Model_Email_UpdatedMessageCount", 
    private.Mail_TryShowPanel
  );
  
  if ( IsMobile() ) then
    
    private.Hearts.AskTimerTryFinish();

    mediator.Subscribe(
      "Model_Social_LoginComplete",
      private.OnLoginComplete
    );
    
    common.MessageSubscribe(
      "Event_Social_AskHeartCooldownEnd",
      private.Hearts.AskTimerFinish
    );
    
  end;
  
  m_email.Start( private.is_online );
  
  if ( private.is_online ) then
    
    if ( private.is_logged ) then
      int_panel_social_score.Friends_Create( m_social.GetFriends() );
    else
      mediator.SubscribeOnce(
        "Model_Social_LoginComplete",
        function ()
      
          int_panel_social_score.Friends_Create( m_social.GetFriends() );
          
        end
      );
    end;
    
  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.IsLogged ()

  return private.is_logged and private.is_online;

end;
--------------------------------------------------------------------------------
function public.TryShowPanel ()

  if ( not private.is_online ) then

    return;

  end;
  
  private.Mail_TryShowPanel();
  private.Social_TryShowPanel();
  
end;
--------------------------------------------------------------------------------
function  public.Logout ()

  if ( IsMobile() ) then
    
    if ( private.is_online ) then

      m_social.Logout();

    else

      private.ShowOfflineNote();

    end;
    
  end;

end;
--------------------------------------------------------------------------------
function public.Login ()

  private.Login();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Social_TryShowPanel ()

  if ( IsMobile() ) and ( not private.is_logged ) then

    local func_tap = function ()
    
      int_panel_map_left.Social_Disable();
      private.Login();

    end;

    int_panel_map_left.Social_Show( func_tap );
      
  end;
  
end;
--------------------------------------------------------------------------------
function private.Login ()

  if ( private.is_online ) then

    if ( not private.is_logged ) then

      mediator.Notify( "View_Social_LoginStart" );

      m_social.Login();

    end;

  else

    private.ShowOfflineNote( int_panel_map_left.Social_Enable );

  end;

end;
--------------------------------------------------------------------------------
function private.OnLoginComplete ( success )

  int_panel_map_left.Social_Enable();
  
  if ( success ) then

    private.is_logged = true;
    
    private.HidePanel();
    
  end;

  mediator.Notify( "View_Social_LoginFinish" );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function  public.Panel.Show ( panel_name, time )
  
  local panels = {
    friends = int_panel_map_right.Friends_Warn
  };
  
  panels[ panel_name ]( 0 );
  
end;
--------------------------------------------------------------------------------
function  public.Panel.Hide ( panel_name )
  
  local panels = {
    friends = int_panel_map_right.Friends_Hide
  };
  
  panels[ panel_name ]();

end;
--------------------------------------------------------------------------------
function private.Panel.FriendsTap ()
  
  int_panel_map_right.Friends_Warn( 0 );
  
  tern( m_player.IsPlayerNameSet() )( private.Friends_ShowDialog, 
                                       private.Nickname_ShowDialog )();
  
end;
--------------------------------------------------------------------------------
function private.Friends_ShowDialog ()

  DbgTrace( "[ DEBUG ] Friends_ShowDialog" );

  m_social.GetRandomPlayers();
  
  local friends = m_social.GetFriendsStageScores( "stage_1" );

  local players = m_social.GetFriendsStageScores( "stage_1" );
  
  local func_login;
  
  if not ( public.IsLogged() ) then
    
    func_login = private.Login;
    
  end;
  
  local func_addfriend = function ( friend_id )
    DbgTrace( "[ DEBUG ] Friends_ShowDialog: add friend < "..friend_id.." >." );  
  end;  
  
  int_dialog_ingame_friends.Show( 
    "players",
    friends,
    players,
    func_login,
    func_addfriend
  );

end;
--------------------------------------------------------------------------------
function private.WaitForNameChangeResult ( player_name, func_continue )

  local func_wait;

  func_wait = function ( msg, params )
    DbgTrace( "[ DEBUG ]  start func_wait in WaitForNameChangeResult" );
    local result_name;
    if ( params ) then
      result_name = params.name;
    end;
    
    mediator.Unsubscribe( "Model_Player_NameChanged",      func_wait );
    mediator.Unsubscribe( "Model_Player_NameChangeFailed", func_wait );

    if ( func_continue ) then

      func_continue( result_name );

    end;

  end;

  mediator.Subscribe( "Model_Player_NameChanged",      func_wait );
  mediator.Subscribe( "Model_Player_NameChangeFailed", func_wait );

  m_player.SetPlayerName( player_name );

end;
--------------------------------------------------------------------------------
function private.Nickname_ShowDialog ()
  
  DbgTrace( "[ DEBUG ] Nickname_ShowDialog" );
  local func_accept = function ( player_name )
    DbgTrace( "[ DEBUG ]  start private.WaitForNameChangeResult .. ".. player_name );
    private.WaitForNameChangeResult( player_name, function ( result_name )
      private.Friends_ShowDialog();
    end );
  end;
  
  local func_close = function ()
    int_dialog_ingame_friends_nickname.Hide( function ()
      int_window.Hide( "int_dialog_ingame_friends_nickname" );
    end );
  end;

  local func_start = function ()
    local preset_name = m_player.GetStartPlayerName();
    int_dialog_ingame_friends_nickname.Show( preset_name, func_accept, func_close );
  end;
  
  int_window.Show( "int_dialog_ingame_friends_nickname", func_start );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Mail_ShowDialog ()

  if ( private.is_online ) then

    local func_read_message = function ( messageId )
      m_email.WaitFor_ReadMessage( messageId );
    end;

    local func_send_heart = function ( targetId ) 
      private.Hearts.SendHeart( targetId, private.userId );
    end;

    local func_get_heart = function ( count )
      return m_player.TryChangeHeartsCount( count );
    end;
    
    local func_obtain_attachment = private.ObtainAttachment;

    local func_send_thanks = function ( targetId ) 
      private.Hearts.SendThanks( targetId, private.userId );
    end;

    local func_show_mail = function ()

      local func_start = function ()
        local mail_list     = private.Mail_GetList();
        local friend_list   = m_social.GetFriends();
        local gold_attached = false;

        for i = 1, #mail_list do
          
          local sender      = mail_list[ i ].from;
          local friend_data = friend_list[ 
            cmn.IsEntityInList( 
              friend_list, 
              { socialId = sender.socialId },
              "socialId"
            )
          ];
        
          if ( friend_data ) then
            mail_list[ i ].from.avatar = friend_data.url;
            mail_list[ i ].from.name   = friend_data.name or sender.name;
          end;

          if ( not mail_list[ i ].from.name   ) or 
             ( mail_list[ i ].from.name == "" ) then
          
            mail_list[ i ].from.name = "id"..sender.userId;
            
          end;

          if ( mail_list[ i ].attach                ) and
             ( mail_list[ i ].attach.type == "gold" ) then

            gold_attached = true;

          end;
          
        end;

        local function func_TopPanel_Show ()

          if ( gold_attached ) then

            v_map.TopPanel_Show( nil, "gold" );

          end;

        end;

        local function func_Hide ()

          if ( v_map.TopPanel_IsVisible() ) then

            v_map.TopPanel_Hide(
              nil,
              function ()

                int_panel_map_top.Restore();
                private.Mail_HideDialog();

              end
            );

          else

            private.Mail_HideDialog();

          end;

        end;
        
        int_dialog_social_mail.Show( 
          mail_list,
          func_read_message,
          func_send_heart,
          func_get_heart,
          func_obtain_attachment,
          func_send_thanks,
          func_Hide,
          func_TopPanel_Show
        );
        
      end;
    
      int_window.Show( "int_dialog_social_mail", func_start );
      
    end;
    
    if ( private.Mail_GetCount() == m_email.GetUnreadMessageCount() ) then
      func_show_mail();
    else
      m_email.WaitFor_Messages( func_show_mail );
    end;

  else

    private.ShowOfflineNote();

  end;

end;
--------------------------------------------------------------------------------
function private.Mail_HideDialog ()

  int_dialog_social_mail.Hide( function ()
    
    int_window.Hide( "int_dialog_social_mail" );

  end );

end;
--------------------------------------------------------------------------------
function private.Mail_UpdateList ( mail )

  private.Mail.messages = {};

  for key, msgInfo in pairs( mail ) do
    if ( not cmn.IsEntityInList( private.Mail.messages, msgInfo, "messageId" )) then
      table.insert( private.Mail.messages, msgInfo );
    end;
  end;
  
  --private.Mail.messages = private.GetUnboundList( mail );

end;
--------------------------------------------------------------------------------
function private.Mail_GetCount ()

  return #private.Mail.messages;

end;
--------------------------------------------------------------------------------
function private.Mail_GetList ()

  return private.GetUnboundList( private.Mail.messages );

end;
--------------------------------------------------------------------------------
function private.Mail_TryShowPanel ( animated )
  
  local mail_count = m_email.GetUnreadMessageCount();
  
  if ( private.is_online           ) and
     ( v_map.Interface_IsVisible() ) then
     
    int_panel_map_left.Mail_Show(
      mail_count,
      private.Mail_ShowDialog,
      animated
    );

  end;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.FriendsChoose.TryShow ( func_OnClose )
    
  if ( private.is_online ) and ( private.is_logged ) then

    local func_start = function ()

      local friend_list = m_social.GetFriends();

      local func_send_hearts = function ( choosen_list )

        private.Hearts.Ask( choosen_list );

      end;

      local func_close = function ()

        private.FriendsChoose.Hide();

        if ( func_OnClose ) then func_OnClose(); end;

      end;

      int_dialog_social_friends_choose.Show( 
        friend_list,
        private.heart_max_friends,
        func_send_hearts,
        func_close
      );

    end;

    int_window.Show( "int_dialog_social_friends_choose", func_start );

  else

    private.ShowOfflineNote();

  end;
  
end;
--------------------------------------------------------------------------------
function private.FriendsChoose.Hide ()

  int_dialog_social_friends_choose.Hide(
    function ()
    
      int_window.Hide( "int_dialog_social_friends_choose" );

    end
  );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Score.TryShow ( point, stage_name, stage_score, score_old )

  if ( IsMobile() ) then

    local func_show_panel = function ()

      local friends_scores = m_social.GetFriendsStageScores( stage_name );

      if ( friends_scores and #friends_scores > 0 ) then

        local func_send_heart = function ( targetId ) 
          --private.Hearts.SendHeart( targetId, private.userId );
          --m_email.WaitFor_ReadMessage( private.Hearts.GetAskingMessageId( targetId ) );
        end;

        local player_score = stage_score;
        score_old = score_old or 0;
        if ( point == "win" ) and ( score_old > stage_score ) then
          point = "win_low";
          player_score = score_old;
        end;

        local social_info = m_social.GetSocialInfo();
        table.insert( friends_scores, { 
          userId   = private.userId,
          socialId = social_info.id,
          name     = social_info.name,
          url      = social_info.url,
          score    = player_score,
          heart    = false,
          res      = "assets/content/interface/social/"..m_skins.GetSkin().."_avatar"
        } );

        table.sort( friends_scores, function( a, b ) 
          return a.score > b.score; 
        end );

        int_panel_social_score.Show( 
          point,
          friends_scores,
          private.userId,
          func_send_heart
        );
        
      end;

    end;

    func_show_panel();

    --[[if ( private.Mail_GetCount() == m_email.GetUnreadMessageCount() ) then
      func_show_panel();
    else
      m_email.WaitFor_Messages( func_show_panel );
    end;]]

  end;

end;
--------------------------------------------------------------------------------
function  public.Score.Hide ()

  if ( IsMobile() ) then
    
    int_panel_social_score.Hide();

  end;
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function  public.Hearts.GetAskToken () 
  
  return private.ask_heart_token;

end;
--------------------------------------------------------------------------------
function private.Hearts.Ask ( friend_list )

  if ( private.is_online ) then

    if ( not TimeIsEventExist( private.ask_heart_token ) ) then

      if ( friend_list and #friend_list > 0 ) then

        TimeSheduleEvent( 
          private.ask_heart_token,
          "GLOBAL_TIME",
          private.heart_cooldown,
          "Event_Social_AskHeartCooldownEnd",
          {}
        );

        for i = 1, #friend_list do
          private.Hearts.SendAsk( friend_list[i], private.userId );
        end;

        mediator.Notify( "View_Social_HeartAsked" );

      end;

    end;

  else

    private.ShowOfflineNote();

  end;

end;
--------------------------------------------------------------------------------
function private.Hearts.Add ( friend_list )

  local out_list = {};
  local mail_list = private.Mail_GetList();

  for f = 1, #friend_list do

    local friend = friend_list[f];
    friend.heart = ( private.Hearts.GetAskingMessageId( friend.userId, mail_list ) ~= false );
    table.insert( out_list, friend );

  end;

  return out_list;

end;
--------------------------------------------------------------------------------
function private.Hearts.GetAskingMessageId ( targetId, mail_list )

  mail_list = mail_list or private.Mail_GetList();
  if ( mail_list and #mail_list > 0 ) then
    for m = 1, #mail_list do
      if ( mail_list[m].text == "str_int_dialog_social_mail_message_help" ) and
         ( mail_list[m].from.userId == targetId ) then
        return mail_list[m].messageId;
      end;
    end;
  end;
  return false; 

end;
--------------------------------------------------------------------------------
function private.Hearts.SendAsk ( targetId, userId )

  m_email.SendMessage( 
    targetId, 
    userId, 
    "str_int_dialog_social_mail_message_help", 
    { name = "heart", type = "heart", count = 1 } 
  );

end;
--------------------------------------------------------------------------------
function private.Hearts.SendHeart ( targetId, userId )

  m_email.SendMessage( 
    targetId, 
    userId, 
    "str_int_dialog_social_mail_message_gift", 
    { name = "heart", type = "heart", count = 1 } 
  );

  --[[
    GD: Если игроку на почту приходит письмо с запросом Сердец, 
        и он отправляет сердце в ответ,
        то помимо отправки сердца другу, 
        он оправляет сообщение с запросом Сердца.
  
    UPD: Убрано по просьбе издателя (FSFTP-605). 
    private.Hearts.SendAsk( targetId, userId );
  ]]

end;
--------------------------------------------------------------------------------
function private.Hearts.SendThanks ( targetId, userId )

  m_email.SendMessage( 
    targetId, 
    userId, 
    "str_int_dialog_social_mail_message_thanks", 
    { name = "heart", type = "heart", count = 1 } 
  );

end;
--------------------------------------------------------------------------------
function private.Hearts.AskTimerTryFinish ()

  if ( TimeIsEventExist( private.ask_heart_token ) and
     ( TimeGetEventRemainingTime( private.ask_heart_token ) == 0 )) then
    private.Hearts.AskTimerFinish();
  end;

end;
--------------------------------------------------------------------------------
function private.Hearts.AskTimerFinish ()

  TimeFinishEvent();
  TimeUnsheduleEvent( private.ask_heart_token );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.ObtainAttachment( attachment )

  if not ( attachment ) then return false; end;

  local category, stage = "mailgift", "mailgift";
  
  if ( attachment.type ~= "chest" ) then
  
    if ( attachment.type == "stickerpack" ) then
      
      if ( not ( m_stage.GetSavedData( "stage_5" ) or {} ).completed ) then
        
        return false, "wait";
        
      end;
      
    end;
    
    local item =
    {
      name       = attachment.name,
      type       = attachment.type,
      count      = attachment.count,
      category   = category,
      stage_name = stage,
      silent     = false
    };
    
    return m_items.TryObtain( item );
  
  end;
  
  local name = attachment.name;
  if ( name:find( "event" ) ) then
  
    -- TODO: Fix it in 1.12
    name = "woodchest";
  
  end;
  
  local chest = 
  { 
    type  = "chest",
    name  = name,
    count = 1
  };
  
  local params =
  {
    category   = category,
    stage_name = stage,
    silent     = true
  };
  
  v_items.Chest_Show( chest, params, attachment.content );
  
  return true;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.HidePanel ( func_end )

  int_panel_map_left.Social_Hide( func_end );

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.UpdateOnline ()

  private.userId    = tonumber( m_player.GetUserId() );
  private.is_online = m_client.IsOnline();

  if ( not private.is_online ) then
    
    -- JK: bots are available even offline, comment below
    --public.Score.Hide();
    
    private.Mail_HideDialog();
    
  end;

end;
--------------------------------------------------------------------------------
function private.GetUnboundList ( list )

  local out_list = {};
  for i = 1, #list do
    table.insert( out_list, list[i] );
  end;
  return out_list;

end;
--------------------------------------------------------------------------------
function private.ShowOfflineNote ( func_close )

  func_close = func_close or function () end;

  v_client.SystemDialog_Show( "connectionlost", func_close );

end;
--------------------------------------------------------------------------------