-- name=v_spells
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ SPELLS ] "..text );

end;
--------------------------------------------------------------------------------
function public.__InitModule ()

  ne.mod.LoadLazily( "assets/content/scripts/mod_int_dialog_map_spell" );

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Create ( spell_name, section_name )

  local obj_name = ( "obj_map_spells_%s" ):format( spell_name );
  local btn_name = ( "btn_map_spells_%s" ):format( spell_name );
  
  ObjCreate( obj_name, "obj" );
  ObjSet(    obj_name, { anim_tag = spell_name } );
  ObjAttach( obj_name, "anm_map_"..section_name );
  
  ObjClone( "btn_map_spells", "_"..spell_name, 1, obj_name );
  
  int.Control_EventsSet( btn_name,
    { 
      func_on_click = function () private.Dialog_Show( spell_name ) end
    }
  );
  
  private.Set( spell_name );

end;
--------------------------------------------------------------------------------
function private.Set ( spell_name )

  local spell_data = m_spells.GetSpellData( spell_name );

  if ( spell_data.enabled ) then

    private.ShowSpell( spell_name );

  else

    private.HideSpell( spell_name );

  end;

  if ( spell_data.completed ) then

    private.CompleteSpell( spell_name );

  end;

end;
--------------------------------------------------------------------------------
function private.ShowSpell ( spell_name )

  m_object.Enable( "btn_map_spells_"..spell_name );

end;
--------------------------------------------------------------------------------
function private.HideSpell ( spell_name )

  m_object.Disable( "btn_map_spells_"..spell_name );

end;
--------------------------------------------------------------------------------
function private.CompleteSpell ( spell_name )

  private.Trace( "CompleteSpell: "..spell_name );

  local btn_name = "btn_map_spells_"..spell_name;
  local anm_name = "anm_map_spells_"..spell_name;
  
  m_object.Set( btn_name, { input = false } );
  m_object.Set( anm_name, { animfunc = "static", playing = true } );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Dialog_Show ( spell_name )

  local func_accept = function ()
  
    return m_spells.TryExecuteSpell( spell_name );
  
  end;

  local func_close = function ( spell_data )

    int_dialog_map_spell.Hide( function ()

      if ( spell_data ) then
      
        private.CompleteSpell( spell_data.name );
        
        if ( spell_data.output ) then
        
          for _, reward in pairs( spell_data.output ) do
          
            if     ( reward.name:find( "gold"     ) ) then
            
              v_map.Gold_Buffer_Out();
            
            elseif ( reward.name:find( "resource" ) ) then
            
              v_map.Reward_ShowCollected( 
                spell_data.name, reward.count, reward.name
              );
            
            elseif ( IsEditor()                     ) then
            
              local msg = "Warning! No visual reward (%s) for spell %q!"
              DbgTrace( msg:format( reward.name, spell_data.name ) );
            
            end;
          
          end;
        
        end;

      end;
    
    end );
    
  end;
  
  local func_top_panel_show = function () v_map.TopPanel_Show(); end;
  local func_top_panel_hide = function () v_map.TopPanel_Hide(); end;
  
  local func_start = function ()
  
    local spell_data = m_spells.GetSpellData( spell_name );

    int_dialog_map_spell.Show(
      spell_data,
      func_accept,
      func_close,
      func_top_panel_show,
      func_top_panel_hide
    );

  end;

  int_window.AddWindowToQueue( "int_dialog_map_spell", func_start );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function public.Callback_AnimateAppear ( stage_name )

  local spell_name = m_spells.GetSpellByUnlockStage( stage_name );
  
  if not ( spell_name ) then 
  
    return nil;
    
  end;
  
  local obj_name = "obj_map_spells_"..spell_name;
  
  local anim =
  {
    name   = obj_name,
    res    = "assets/content/map/effects/anm_appearance",
    coords = { pos_x = 0, pos_y = 0, scale_x = 0.5, scale_y = 0.5 },

    func_preset = function () 
      ObjSet( obj_name, { visible = true, alp = 0.0 } );
    end,

    func_dummy = function () 
      
      private.Sound_Play( "new_bandit" );
      
      ObjAnimate( obj_name, "alp", 0, 0, function ()
        private.ShowSpell( spell_name );
      end, 
      { 
        0.0, 0, 0.0, 
        0.2, 0, 1.0
      } );
    end,

    func_action = function ( func_after )

      mediator.Notify( "View_Spells_SpellbookAppear", spell_name );
      if ( func_after ) then func_after(); end;

    end
  };
  
  return v_map.Callback_Animate( anim );

end;
--------------------------------------------------------------------------------
-- function ... ()
--------------------------------------------------------------------------------
function private.Sound_Play ( sound_id )
  
  local path = "assets/content/map/sounds/aud_"..sound_id;
  
  cmn.PlaySnd( path );
  
end;
--------------------------------------------------------------------------------