-- name=v_stage
--------------------------------------------------------------------------------
function private.Trace ( text )

  common.Trace( "[ VIEW ] [ STAGE ] "..text );

end;
--------------------------------------------------------------------------------
function public.Init ()
  
  private.is_active = false;
  private.popup_show = false;
  private.prev_popup_stage = nil;
  private.fortunewheel_popup_enabled = true;

  loader.Modules_Load({
    "v_match3",
    "int_panel_stage_top",
    "int_panel_stage_progress_ride",
    "int_panel_stage_progress_fight",
    "int_panel_stage_score",
    "int_panel_stage_time",
    "int_panel_stage_resources",
    "int_panel_stage_boosters",
    "int_panel_stage_settings",
    "int_panel_stage_win",
    "int_popup_stage_score",
    "int_popup_stage_combo",
    "int_popup_stage_bonus",
    "int_dialog_stage_shuffle",
    "int_dialog_stage_start",
    "int_dialog_stage_exit",
    "int_dialog_stage_win",
    "int_dialog_stage_fail",
    "int_dialog_stage_continue",
    "int_dialog_stage_targets",
    "int_dialog_stage_friends"
  });

  common.MessageSubscribe(
    "Event_Match3_FieldEstablished",
    function ( msg, params )

      private.TryContinueNEW( params );

    end
  );

  common.MessageSubscribe(
    "Event_Match3_BoosterUsed",
    function ( msg, params )

      private.PanelBoosters_Use( params );

    end
  );

  mediator.Subscribe(
    "Model_Fortunewheel_TimerStart",
    function ()

      int_dialog_stage_continue.CooldownStarted(
        m_fortunewheel.GetState().state
      );

    end
  );

  mediator.Subscribe(
    "Model_Fortunewheel_TimerStop",
    function ()

    int_dialog_stage_continue.CooldownFinished(
      m_fortunewheel.GetState().state
    );

    end
  );

  mediator.Subscribe(
    "View_Boosters_BoosterUsedAnimEnd",
    private.PanelBoosters_HideAnim
  );

  mediator.Subscribe(
    "View_Match3_FieldEstablished",
    private.TryContinue
  );

  common.MessageSubscribe(
    "Event_Match3_FieldNeedShuffle",
    private.OnFieldNeedShuffle
  );

  mediator.Subscribe( "Model_Match3_FieldActivated",  private.OnFieldActivated );
  mediator.Subscribe( "View_Match3_FieldEstablished", private.OnFieldEstablished );
  mediator.Subscribe( "Model_Match3_BonusRemoved",    private.OnBonusRemoved );
  mediator.Subscribe( "Model_Match3_BonusSpawn",      private.OnBonusSpawn );
  --mediator.Subscribe( "Model_Match3_ComboActivated",        private.OnComboActivated );
  mediator.Subscribe( "Model_Match3_CombinationsNotExist",  private.TryContinue );
  
  mediator.Subscribe( "Model_Stage_BonusCountChanged", private.OnBonusCountChanged );

  mediator.Subscribe( "Model_Stage_ScoreChanged", function ( params )
  
    private.PopupScore_Show( params.chip_x, params.chip_y, params.chip_type, params.chip_score );

  end );

  mediator.Subscribe( "Model_Stage_TargetUpdated", private.TopPanel_UpdateGoals );
  --mediator.Subscribe( "Model_Stage_TargetUpdated", private.ShowPanelWin );

  mediator.Subscribe( "Model_Stage_ChainCountChanged", private.PopupCombo_Show );

  --mediator.Subscribe( "Model_Stage_Finalized", private.DialogComplete_Show );

  mediator.Subscribe( "Model_Stage_MovesCountChanged", private.OnMovesCountChanged );
  --mediator.Subscribe( "Model_Stage_MovesCountChanged", private.PanelBoosters_UpdateButton );
  --mediator.Subscribe( "Model_Stage_TimeCountChanged", private.UpdatePanelDistance );
  --mediator.Subscribe( "Model_Stage_TimeCountChanged", private.UpdatePanelProgressVisible );

  mediator.Subscribe( "Model_Stage_CellDestroyed",  private.OnCellDestroyed );
  mediator.Subscribe( "Model_Stage_ItemDropped",    private.OnItemDropped );
  mediator.Subscribe( "Model_Stage_TargetUpdated",  private.OnTargetUpdated );
  mediator.Subscribe( "Model_Stage_StageComplete",  private.OnStageComplete );

  --mediator.Subscribe( "Model_Stage_HeroHealthRecovered", private.OnHeroHealthRecovered );
  --mediator.Subscribe( "Model_Stage_HeroAbilityIdleChanged", private.OnHeroAbilityIdleChanged );
  --mediator.Subscribe( "Model_Stage_HeroAbilityActivated", private.OnHeroAbilityActivated );
  mediator.Subscribe( "Model_Stage_HeroAttackActivated", private.OnHeroAttackActivated );

  --mediator.Subscribe( "Model_Stage_EnemyAbilityIdleChanged", private.OnEnemyAbilityIdleChanged );
  mediator.Subscribe( "Model_Stage_EnemyAttackActivated", private.OnEnemyAttackActivated );

  --mediator.Subscribe( "Model_Stage_GatherCountChanged", private.UpdatePanelGathers );

  mediator.Subscribe( "Model_Stage_RewardCountChanged", private.OnResourceRewardCountChanged );
  mediator.Subscribe( "Model_Stage_RewardCountChanged", private.PanelResources_Show );

  -- Panel score.
  mediator.Subscribe( "Model_Stage_ScoreChanged", private.PanelScore_Update );

  mediator.Subscribe( "Model_Stage_BonusEnable",          private.PopupBonus_Show );
  mediator.Subscribe( "Model_Stage_BonusDisable",         private.PopupBonus_Hide );
  mediator.Subscribe( "Model_Stage_BonusEffectDisabled",  private.BonusAbility_Hide );

  -- Panel time.
  mediator.Subscribe( "Model_Stage_TimeCountChanged",   private.PanelTime_Update );
  mediator.Subscribe( "Model_Stage_TimerStart",         private.PanelTime_StartTimer );
  mediator.Subscribe( "Interface_Window_WindowOpened",  private.PanelTime_StopTimer );
  mediator.Subscribe( "Interface_Window_WindowClosed",  private.PanelTime_StartTimer );
  
  mediator.Subscribe(
    "Model_Stage_ViewTryContinue",
    private.TryContinue
  );
  
  screen.Setup( "scr_stage",
  {
    func_pre_open    = private.PreOpen,
    func_open        = private.Open,
    func_close       = private.Close,
    func_after_close = private.AfterClose
  } );

  ObjAttach( "scr_stage", "ne_storage" );

  ObjAttach( "tmr_stage_finalize", InterfaceName );
  ObjAttach( "tmr_stage_finalbonuses", InterfaceName );
  ObjAttach( "tmr_stage_resources", InterfaceName );

  private.item_dropped_target_offset_x = 0;
  private.item_dropped_target_offset_y = -100;

  if( int.Landscape() ) then

    private.item_dropped_target_offset_x = 100;
    private.item_dropped_target_offset_y = -70;

  end;

  private.this = {
    effect = {
      cell_destroyed = "anm_cell_destroyed"
    }
  };

  private.Trace( "Initialized." );

end;
--------------------------------------------------------------------------------
function public.Finish ()

  private.Finish();

end;
--------------------------------------------------------------------------------
function public.TryStart ( stage_name, dailyquest, stage_state )

  mediator.Subscribe(
    "Model_Boosters_CountChanged",
    private.DialogStart_Update
  );

  local stage_data = m_stage.GetStaticData( stage_name );

  private.score_old = 0;

  m_boosters.SetSelected( {} );

  local stage_started;

  if ( stage_data.realtype == "dailyquest" ) then

    private.ext_enemy = dailyquest.enemy;

    stage_started = m_stage.TryStart(
      stage_name,
      dailyquest.enemy
    );

  elseif ( stage_data.realtype == "fight" ) then

    stage_started = m_stage.TryStart(
      stage_name,
      nil,
      1
    );

  elseif ( stage_data.realtype == "misc" ) then

    stage_started = m_stage.TryStart(
      stage_name,
      nil,
      4
    );

  else

    stage_started = m_stage.TryStart(
      stage_name,
      nil,
      stage_state
    );

  end;

  if ( stage_started ) then

    int.Input_Off( "zmv_map" );

    common.Playhaven_LogPlacement( "load_level" ); 

    private.Start();
    
    return true;

  end;
  
  return false;
  
end;
--------------------------------------------------------------------------------
function public.IsActive ()
  
  return private.is_active;
  
end;
--------------------------------------------------------------------------------
function public.IsStageComplete ()
  
  return m_stage.IsWin();
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Match3_SetInput ( value )

  ObjSet( "obj_match3", { input = value } );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Match3_ShadesOn ( forced )

  private.match3_shades_forced = private.match3_shades_forced or forced;

  if ( private.match3_shades ) then

    return;

  end;

  private.match3_shades = true;
  
  local function func_notify ()
  
    mediator.Notify( "View_Stage_Match_ShadesOn" );
  
  end;
  
  local tme = 0.3;
  
  int_panel_stage_progress_fight.Move_Near( tme );

  ObjAnimate( "obj_match3", "color_rgb", 0, 0, func_notify, 
  { 
    0.0, 0, 1.0, 1.0, 1.0, 
    tme, 0, 0.5, 0.5, 0.5 
  } );
  
  if ( private.match_custom_shade ) then
    
    --local cur_y = int.Position( "obj_match3" )[2];
    --local shift = private.match_custom_shade_shift or 100;
    --
    --ObjAnimate( "obj_match3", "pos_y", 0, 0, "", 
    --{ 
    --  0.0, 0, cur_y, 
    --  tme, 0, cur_y + shift
    --} );
    --
    --private.PanelBoosters_Hide();
    
    ObjAnimate( "obj_match3", "alp", 0, 0, func_notify, 
    { 
      0.0, 0, 1.0,
      tme, 0, 0.0
    } );
    
  end;

end;
--------------------------------------------------------------------------------
function private.Match3_ShadesOff ( forced )

  if not ( private.match3_shades ) and not ( forced ) then

    return;

  end;

  if ( private.match3_shades_forced ) and not ( forced ) then

    return;

  end;
  
  local function func_notify ()
  
    mediator.Notify( "View_Stage_Match_ShadesOff" );
  
  end;

  private.match3_shades_forced = false;
  private.match3_shades = false;
  
  local tme = 0.3;
  
  int_panel_stage_progress_fight.Move_Far( tme );

  ObjAnimate( "obj_match3", "color_rgb", 0, 0, func_notify, 
  { 
    0.0, 0, 0.5, 0.5, 0.5,
    tme, 0, 1.0, 1.0, 1.0
  } );
  
  if ( private.match_custom_shade ) then
    
    --local cur_y = int.Position( "obj_match3" )[2];
    --local shift = private.match_custom_shade_shift or 100;
    --
    --ObjAnimate( "obj_match3", "pos_y", 0, 0, "", 
    --{ 
    --  0.0, 0, cur_y, 
    --  tme, 0, cur_y - shift
    --} );
    --
    --private.PanelBoosters_Show();
    
    ObjAnimate( "obj_match3", "alp", 0, 0, func_notify, 
    { 
      0.0, 0, 0.0,
      tme, 0, 1.0
    } );
    
  end;

end;
--------------------------------------------------------------------------------
function private.Match3_ShadesReset ()

  ObjStopAnimate( "obj_match3", "color_rgb" );
  ObjStopAnimate( "obj_match3", "alp" );
  ObjSet( "obj_match3", { alp = 1, color_r = 1, color_g = 1, color_b = 1 } );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.DialogStart_Show ( stage_name, database, dailyquest )

  private.Trace( "DialogStart_Show" );

  mediator.Subscribe(
    "Model_Boosters_CountChanged",
    private.DialogStart_Update
  );

  local start_data = private.GetStartData( stage_name, database, dailyquest );

  private.score_old = start_data.score_old;

  local func_accept = function ( booster_list )

    local boosters = {};

    for key, value in pairs( booster_list ) do

      table.insert( boosters, key );

    end;

    m_boosters.SetSelected( boosters );

    if ( m_stage.TryStart( stage_name, start_data.enemy ) ) then

      local func_end = function ()

        private.Start();

      end;

      private.DialogStart_Hide( func_end, true );

      int.Input_Off( "zmv_map" );

      mediator.Notify(
        "View_Stage_DialogStartDeactivated",
        {
          name  = stage_name,
          start = true
        }
      );

    end;

  end;

  local func_reject = function ()

    local func_end = function ()

      private.Finish();

    end;

    mediator.Notify( "View_Stage_DialogStartDeactivated",
    {
      name  = stage_name,
      start = false } );

    private.DialogStart_Hide( func_end );

  end;

  local func_boost_buy = function ( booster_name )

    return v_boosters.Tap( "dialog", booster_name, stage_name );

  end;

  local func_popup_show = function ( caption, text, pos, parent, inverse )

    int.ShowPopupFlat( caption, text, pos, parent, inverse );

  end;

  local func_popup_hide = function ( instant )

    int.HidePopupFlat( instant );

  end;
  
  local func_OnShow = function ()
    
    private.Social_Score_TryShow(
      "start",
      start_data.type,
      stage_name, 
      start_data.score_old
    );    

    local boosters = {};
    
    for b, booster in pairs( start_data.boosters ) do
      
      table.insert( boosters, booster.name );
      
    end;
  
    mediator.Notify( "View_Stage_DialogStartActivated",
    {
      name     = stage_name,
      boosters = boosters } );

  end;  

  local func_start = function ()
  
    -- TODO: Fix it in 1.12
    local show_event_marker = false;
    
    int_dialog_stage_start.Show( 
      stage_name,
      start_data,
      func_accept,
      func_reject,
      func_boost_buy,
      func_popup_show,
      func_popup_hide,
      func_OnShow,
      show_event_marker
    );

    common.Playhaven_LogPlacement( "load_level" );

  end; 

  int_window.AddWindowToQueue( "int_dialog_stage_start", func_start );

  cmn.PlaySnd( "level_click" );

end;
--------------------------------------------------------------------------------
function private.DialogStart_Update ( booster_data )

  int_dialog_stage_start.Update( booster_data );

end;
--------------------------------------------------------------------------------
function private.DialogStart_Hide ( func_end, skip_map_enable )

  mediator.Unsubscribe( "Model_Boosters_CountChanged", private.DialogStart_Update );

  if ( IsMobile() ) then
    
    private.Social_Score_Hide();
    
  end;

  int_dialog_stage_start.Hide( func_end, skip_map_enable );

end;
--------------------------------------------------------------------------------
function private.GetStartData (
    stage_name,
    database,
    dailyquest,
    stage_state
  )

  local saved_data = m_stage.GetSavedData(  stage_name ) or {};
  local db_data    = m_stage.GetStaticData( stage_name, saved_data );

  stage_state = stage_state or m_stage.GetStageState(
    stage_name,
    db_data,
    saved_data
  );

  local stateless = m_stage.IsStateless();
  local db_index;
  
  if ( stateless ) then db_index = 1;
  else                  db_index = stage_state;
  end
  
  -- Data for stage start dialog
  local start_data = {
    section     =  db_data.section,
    type        =  db_data.type,
    realtype    =  db_data.realtype,
    zone        =  db_data.zone,
    content     =  db_data.content,

    moves       =  db_data.state[ db_index ].moves,
    time        =  db_data.state[ db_index ].time,
    targets     =  db_data.state[ db_index ].targets,
    resources   =  db_data.state[ db_index ].chip_types,
    enemy       =  db_data.state[ db_index ].enemy,

    boosters    =  m_stage.Boosters_GetStageList( stage_name, "start" ),

    states      = #db_data.state,
    state       =  stage_state,
    stateless   =  stateless,

    noreward    =  saved_data.noreward,

    score_old   =  nil,
    attempts    =  nil,
    color       =  nil, -- Dialog bg color

    reward      =  {}   -- List of reward lists for each state
  };

  -- FIXME: Is it used?
  if not ( dailyquest ) then 

    start_data.score_old = saved_data.score    or 0;
    start_data.attempts  = saved_data.attempts or 0;

  end;

  -- State and noreward
  if ( stateless and saved_data.completed ) then

    start_data.state     = 1;
    start_data.noreward  = true;

  end;

  if ( start_data.type == "fight" ) then

    start_data.state = 1;

  end;

  -- Dialog BG color
  local enemy    = ( start_data.enemy or {} ).name;
  local zone     =   start_data.zone;
  local stg_type =   start_data.type;

  start_data.color = private.GetDialogBackgroundColor( zone, stg_type, enemy );

  -- Trim not resources from chip_types
  for i = #start_data.resources, 1, - 1 do

    if not ( string.find( start_data.resources[ i ], "Resource" ) ) then

      table.remove( start_data.resources, i );

    end;

  end;

  -- Generate rewards list
  local state_max = math.min( start_data.states, 3 );

  for i = 1, state_max do
    table.insert( start_data.reward, {} );
  end;

  -- Get unique rewards
  for s = 1, state_max do

    local state_rewards = db_data.state[ s ].reward or {};

    for r = 1, #state_rewards do

      local reward   = state_rewards[ r ];
      reward.content = m_content.GetContent( reward.type, reward.name );

      if     ( reward.type == "quest" ) then

        local index = math.max( s, start_data.state );
        if ( m_quests.IsStageItemActive( reward.name, stage_name, "stage", start_data.section ) ) then

          table.insert( start_data.reward[ index ], reward );

        end;
        
      elseif ( reward.type ~= "gold"  ) then
        
        if ( reward.type == "worker" ) then
          local mine_name = m_mines.GetWorkerMineName( stage_name );
          local mine_data = m_mines.GetMineData( mine_name );
          if ( ( mine_data.progress or 0 ) < mine_data.progress_max ) then
            table.insert( start_data.reward[ s ], reward );
          end;
        else
          table.insert( start_data.reward[ s ], reward );
        end;

      end;

    end;
    
  end;

  -- Append generic rewards
  local gold_bonus = m_hero.GetStats().gold;

  if ( db_data.type == "ride" ) then

    local rewards = m_storage.Settings( "stages" ).rewards or {};

    for r = 1, #rewards do

      if ( start_data.reward[ r ] ) then

        local gold_count = rewards[ r ].count;

        table.insert( start_data.reward[ r ], 
        { 
          name    = "gold", 
          type    = "gold", 
          count   = gold_count + gold_bonus,
          content = m_content.GetContent()
        } );

      end;

    end;

  elseif ( db_data.type == "fight" ) then

    local fight_gold_count = m_storage.Settings( "fights" ).reward;
    local fight_reward = 
    {
      name    = "gold",
      type    = "gold",
      count   = fight_gold_count + gold_bonus,
      content = m_content.GetContent()
    };

    if ( db_data.realtype == "misc" ) then

      -- Replace misc fight reward by generic
      start_data.reward[ start_data.state ] = {};
      table.insert( start_data.reward[ start_data.state ], fight_reward );

    else

      local reward = fight_reward;

      local query = 
      {
        query      = "find",
        group      = "static",
        collection = "spells",
        indice     = "unlock",
        criteria   = { like = stage_name }
      };

      if ( m_storage.Execute( query ).result ) then

        reward = 
        {
          name    = "spellbook";
          content = m_content.GetContent()
        }

      end;

    table.insert( start_data.reward[ start_data.state ], reward );

    end;

  end;

  return start_data;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.DialogExit_Show ( func_on_reject, func_on_accept )
  
  private.Trace( "DialogExit_Show" );
  
  local stage_data = m_stage.GetCurrentStageData();
  local targets    = m_stage.GetCurrentTargets();
  local moves_done = m_stage.MovesDone_GetCount();
  local hearts_cnt = m_player.GetHeartsCount();
  local hearts_max = m_hearts.GetMaxHearts();

  local func_accept = function ()
    
    if ( func_on_accept ) then 
      func_on_accept();
    end;

    private.Match3_SetInput( false );
    private.PanelBoosters_SetInput( false, "DialogExit_Show" );
    --private.PanelSettings_SetInput( false );

    private.DialogFriends_Hide();

    private.DialogExit_Hide( function ()

      mediator.Notify( "View_Stage_StageAbandon", { stage = stage_data.name } );
      private.Finish();

    end );

    local score_target = 0;
    local moves_target = 0;

    if ( stage_data.type == "ride" ) or ( stage_data.type == "bonus" ) then

      score_target = m_stage.GetTargetByType( "GainScore" ).count;
      moves_target = stage_data.moves;

    end;

    local getscore = function ( stage_type )

      if ( stage_type == "fight" ) then
        
        local hero_data = m_stage.GetCurrentHero();

        return { ( hero_data.health_current..", "..hero_data.health ), 1 };

      end;

      return { ( m_stage.GetCurrentScore()..", "..score_target ), 0 };

    end;
    
    local scoredata  = getscore( stage_data.type );
    local daily = tern( stage_data.realtype == "dailyquest" )( 
          ( m_dailyquest.Challenge_Count() - m_dailyquest.Challenge_GetOpenedCount() + 1 ), 
          false );

    common.Kontagent_LogEvent( "stage",
      {
        section = stage_data.section,
        stage   = stage_data.name,
        daily   = daily,
        name    = "abandon",
        state   = m_stage.Kontagent_GetStageState( stage_data.name ),
        score   = scoredata[ 1 ],
        is_boss = scoredata[ 2 ],
        moves   = moves_done..", "..moves_target,
        boost   = m_stage.Kontagent_GetBoostersUsed(),
        cells   = m_stage.Kontagent_GetCellsDestroyed(),
        items   = m_stage.Kontagent_GetItemsDropped(),
        flies   = m_stage.Kontagent_GetResourceCollected(),
        bombs   = m_stage.Kontagent_GetExplosivesDestroyed()
    } );

    common.Playhaven_LogPlacement( "level_quit" );

  end;

  local func_reject = function ()

    private.DialogExit_Hide( func_on_reject, true );

  end;
  ------------------------------------------------------------------------------
  local func_skin_attach = function ( parent )

    return v_hero.Skin_Attach( parent );

  end;
  ------------------------------------------------------------------------------
  local func_skin_detach = function ( anm_name )

    v_hero.Skin_Detach( anm_name );

  end;
  ------------------------------------------------------------------------------
  local func_start = function ()

    int_dialog_stage_exit.Show(
      stage_data.name,
      moves_done,
      hearts_cnt,
      hearts_max,
      func_accept,
      func_reject,
      func_skin_attach,
      func_skin_detach
    );

  end;

  int_window.Show( "int_dialog_stage_exit", func_start, { vignette = true } );

end;
--------------------------------------------------------------------------------
function private.DialogExit_Hide ( func_end, no_animation )

  int_dialog_stage_exit.Hide( function ()

    int_window.Hide( "int_dialog_stage_exit", func_end );

  end, no_animation );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.DialogComplete_Show ()
  
  private.finalized = false;

  if ( m_stage.IsWin() ) then

    private.DialogWin_Show();
    private.DialogFriends_Hide();

  else

    if ( m_stage.IsCanContinue() ) then

      private.Dialog_Continue_Show();

    else

      private.Dialog_Fail_Show();
      private.DialogFriends_Hide();

    end;

  end;

  private.PopupBonus_Hide();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.DialogTargets_Show ( state, targets, enemy_name )

  local func_start = function ()
  
    local section = m_stage.GetCurrentStageData().section;
    
    int_dialog_stage_targets.Show( state, 
                                   targets, 
                                   enemy_name,
                                   section,
                                   private.DialogTargets_Hide );

  end;

  int_window.Show( "int_dialog_stage_targets", func_start, { vignette = true } );

end;
--------------------------------------------------------------------------------
function private.DialogTargets_Hide ()

  local func_hide = function ()

    common.MessageSend( Command_Match3_DidBecomeActive );

    m_stage.Boosters_TryUse( v_boosters.Use );

  end;

  int_window.Hide( "int_dialog_stage_targets", func_hide );

  mediator.Notify( "View_Stage_DialogTargetsHided" );
  
  local stage_data = m_stage.GetCurrentStageData();
  local stage_name = stage_data.name;
  
  if ( private.popup_show and private.prev_popup_stage ~= stage_name ) then
    
    int_panel_stage_boosters.Popup_Show();
    
    private.prev_popup_stage = stage_name;
    
    private.popup_show = false;
  
  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Dialog_Continue_SetPopupEnabled ( value )
  private.fortunewheel_popup_enabled = value;
end;
--------------------------------------------------------------------------------
function private.Dialog_Continue_Show ()

  local continue_name = "";
  local stage_name    = m_stage.GetCurrentStageName();
  local stage_type    = private.GetCurrentStageType();

  if ( stage_type == "fight" ) then

    continue_name = "extralives";

  else

    if ( m_stage.GetCurrentStageData().time ) then

      continue_name = "extratime";

    else

      continue_name = "extramoves";

    end;

  end;
  
  local booster_cost  = m_boosters.GetPrice( continue_name, 1 );
  local booster_count = m_boosters.GetCount( continue_name );

  -- -- -- -- -- -- -- wheel -- -- -- -- -- -- -- 
  local wheel_info = m_fortunewheel.GetInfo();
  wheel_info.state = m_fortunewheel.GetState().state;
  wheel_info.notify_ready = function ()
    mediator.Notify( "View_Stage_ContinueWheelReadyToSpin" );
  end;

  local func_turn = function ()
    mediator.Notify( "View_Stage_ContinueWheelTurn" );
    return m_fortunewheel.TryTurnWheel();
  end;

  local func_reset_cooldown = function ()
    return m_fortunewheel.TryResetCooldown();
  end;

  -- -- -- -- -- -- -- boosters -- -- -- -- -- -- -- 
  local func_use_continue_booster = function ( booster_name )

    private.PopupBonus_Show();

    private.Match3_SetInput( true );
  
    m_boosters.TryUse( booster_name, stage_name );

    m_stage.TryUseContinue( continue_name );

    private.PanelBoosters_SetInput( true, "Use Continue" );

    if ( m_stage.GetCurrentStageData().time ) then

      private.PanelTime_StartTimer();

    end;

  end;

  local func_item_get = function ( item, func_end )

    if ( item.name == "heart" ) then

      m_player.TryChangeHeartsCount( item.count or 1 );
      common.Defer(
        "hide_left_panel",
        0.7,
        function()
          
          v_map.LeftPanel_Hide( func_end );
         
        end
      );

    elseif ( item.name == "extramoves" ) then

      m_boosters.TryChangeCount( item.name, item.count or 1, stage_name );
      private.Dialog_Continue_Hide();
      func_use_continue_booster( item.name );

    elseif ( item.name == "gold" ) then

      if ( booster_count > 0 ) then
      
        v_map.TopPanel_Hide( nil, func_end );
        
      else
        
        if ( func_end ) then
          
          func_end();
          
        end;
        
      end;
      
      m_player.TryChangeGoldCount( item.count or 1 );

      local stage_data = m_stage.GetCurrentStageData();
      local stage_name = stage_data.name;

      common.Kontagent_LogEvent( "economy", 
      {
        description = "premium_currency_collect",
        category    = "fortunewheel",
        value       = item.count or 1,
        name        = "reward"
      } );

    end;

    mediator.Notify( "View_Stage_ContinueWheelReward" );

  end;

  local func_buy_booster = function ()

    if ( m_boosters.GetCount( continue_name ) <= 0 ) then

      if ( m_boosters.TryBuy( continue_name, 1, stage_name ) ) then

        private.Dialog_Continue_Hide();
        func_use_continue_booster( continue_name );

      end;

    else

      private.Dialog_Continue_Hide();
      func_use_continue_booster( continue_name );

    end;

  end;

  -- -- -- -- -- -- -- interface -- -- -- -- -- -- -- 
  local func_detach_res_panel = function ()

    v_map.TopPanel_Hide( true );

  end;

  local func_attach_res_panel = function ()

    int_panel_map_top.Highlight( "gold" );
    v_map.TopPanel_Show();

  end;

  local func_get_flight_target = function ( reward_name )

    if ( reward_name == "extramoves" ) or ( reward_name == "extralives" ) then 

      func_detach_res_panel();

      if ( continue_name == "extralives" ) then

        return "anm_int_panel_stage_progress_fight_healthbars_hero_icon"; 

      else

        return "anm_int_panel_stage_top_moves"; 

      end;

    elseif ( reward_name == "gold" ) then

      func_attach_res_panel();
      return "obj_int_panel_map_gold";

    elseif ( reward_name == "heart" ) then

      func_detach_res_panel();
      
      v_map.LeftPanel_Show( true );

      return "btn_int_panel_map_hearts";

    end;

  end;

  local func_popup_show = function ( source, offset, scale )
    
    if ( private.fortunewheel_popup_enabled ) then
      int.Popup_Cloud_Show(
        "hint", 
        "obj_int_dialog_stage_continue_wheel", 
        "str_int_popup_cloud_wheel_"..source,
        nil,
        offset,
        scale,
        nil,
        false,
        true
      );
    end;

  end;

  local func_reject = function ( extramoves )

    mediator.Notify( "View_Stage_ContinueWheelClose" );

    if ( extramoves ) then

      private.Dialog_Continue_Hide();

    else

      private.Dialog_Continue_Hide( private.Dialog_Fail_Show );
      
      -- Send message only if purchase available.
      if ( m_boosters.GetCount( continue_name ) <= 0 ) then
        
        common.Kontagent_LogEvent( "economy", 
        {
          name = continue_name,
          description = "extra_moves_offer",
          level = 1,
          value = 0,
          purchase = 0
        } );
        
      end;

    end;

  end;

  local func_hero_fun = function ()
  
    if ( stage_type == "fight" ) then
      int_panel_stage_progress_fight.Hero_Fun_Wheel();
    else
      int_panel_stage_progress_ride.Hero_Fun_Wheel();
    end;

  end;
  
  local func_notify = function ()
  
    mediator.Notify( "View_Stage_DialogContinueOpened" );
  
  end;
  
  local function func_show_panel_stage_top ()
    
    int_panel_stage_top.SetVisible();
    
  end;
  
  local function func_hide_panel_stage_top ()
    
    int_panel_stage_top.SetInvisible();
    
  end;

  local func_start = function ()

    int_dialog_stage_continue.Show( 
      wheel_info,              continue_name, 
      booster_cost,            booster_count, 
      func_buy_booster,        func_turn, 
      func_reset_cooldown,     func_item_get, 
      func_attach_res_panel,   func_detach_res_panel, 
      func_get_flight_target,  func_popup_show,
      func_reject,             func_hero_fun,
      func_notify,             func_show_panel_stage_top,
      func_hide_panel_stage_top
    );

  end;

  if ( private.GetCurrentStageType() == "ride" ) or 
     ( private.GetCurrentStageType() == "bonus" ) then

    int_panel_stage_progress_ride.Update_StageFailed();

  end;

  int_window.AddWindowToQueue(
    "int_dialog_stage_continue",
    func_start,
    { vignette = true }
  );

end;
--------------------------------------------------------------------------------
function private.Dialog_Continue_Hide ( func_end )

  if ( private.GetCurrentStageType() == "ride" ) or ( private.GetCurrentStageType() == "bonus" )then

    int_panel_stage_progress_ride.Update_HeroRespawn();
        
    private.popup_show = true;

  end;

  int_dialog_stage_continue.Hide( function ()

    v_map.TopPanel_Hide( true );

    int_window.Hide( "int_dialog_stage_continue", func_end );

  end );

end;
--------------------------------------------------------------------------------
function public.Dialog_Continue_Hide ( func_end )

  private.Dialog_Continue_Hide( func_end );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Dialog_Fail_Show ()
  
  private.Trace( "Dialog_Fail_Show" );
  
  local stage_data = m_stage.GetCurrentStageData();
  local stage_name = stage_data.name;

  local database = "stages";

  if ( string.find( stage_name, "fight" ) ) then

    database = "fights";

  end;

  local targets     = m_stage.GetCurrentTargets();
  local heart_count = m_player.GetHeartsCount();
  local heart_max   = m_hearts.GetMaxHearts();

  ------------------------------------------------------------------------------
  local func_close = function ()

    private.Dialog_Fail_Hide( function () private.Finish(); end );

  end;
  ------------------------------------------------------------------------------
  local func_try_show_dialog_start = function ()

    if ( string.find( stage_name, "daily" ) ) then

      public.TryStart( stage_name, { enemy = m_stage.GetCurrentEnemy() } );

    else

      public.DialogStart_Show( stage_name, database ); 

    end;

  end;
  ------------------------------------------------------------------------------
  local func_retry = function ()

    private.Dialog_Fail_Hide( function () 
      
      m_stage.TryFinish();
    
      func_try_show_dialog_start();

    end );

    cmn.PlaySnd( "simply_click" );

  end;
  ------------------------------------------------------------------------------
  local func_restore = function ()

    private.Dialog_Fail_Hide( function ()

      local func_OnHide = function ( result )

        m_stage.TryFinish();
      
        if ( string.find( stage_name, "daily" ) ) and not ( result ) then
          private.Finish();
          return;
        end;
        func_try_show_dialog_start();

      end;

      v_hearts.Show( func_OnHide );

    end );

    cmn.PlaySnd( "simply_click" );

  end;
  ------------------------------------------------------------------------------
  local func_skin_attach = function ( parent )

    return v_hero.Skin_Attach( parent );

  end;
  ------------------------------------------------------------------------------
  local func_skin_detach = function ( anm_name )

    v_hero.Skin_Detach( anm_name );

  end;
  ------------------------------------------------------------------------------
  local func_start = function ()

    int_dialog_stage_fail.Show( 
      stage_data, 
      targets, 
      heart_count,
      heart_max,
      func_close, 
      func_retry, 
      func_restore,
      func_skin_attach,
      func_skin_detach
    );

  end;
  ------------------------------------------------------------------------------

  int_window.Show( "int_dialog_stage_fail", func_start );

  public.Soundtrack_Stop();
  
  cmn.PlaySnd( "assets/content/hero/aud_hero_win_ride" );

  common.Playhaven_LogPlacement( "level_fail" );
  wiz.ChartBoost.LocationGameOver();

  local score_target = 0;
  local moves_target = 0;

  if ( stage_data.type == "ride" ) then

    score_target = m_stage.GetTargetByType( "GainScore" ).count;
    moves_target = stage_data.moves;

  end;

  local getscore = function ( stage_type )

    if ( stage_type == "fight" ) then
      
      local hero_data = m_stage.GetCurrentHero();

      return { ( hero_data.health_current..", "..hero_data.health ), 1 };

    end;

    return { ( m_stage.GetCurrentScore()..", "..score_target ), 0 };

  end;
  
  local scoredata = getscore( stage_data.type );
  local daily = tern( stage_data.realtype == "dailyquest" )( 
      ( m_dailyquest.Challenge_Count() - m_dailyquest.Challenge_GetOpenedCount() + 1 ), 
      false );
      
  common.Kontagent_LogEvent( "stage",
  {
    section = stage_data.section,
    stage   = stage_data.name,
    daily   = daily,
    name    = "fail",
    state   = m_stage.Kontagent_GetStageState( stage_data.name ),
    score   = scoredata[ 1 ],
    is_boss = scoredata[ 2 ],
    moves   = m_stage.MovesDone_GetCount()..", "..moves_target,
    boost   = m_stage.Kontagent_GetBoostersUsed(),
    cells   = m_stage.Kontagent_GetCellsDestroyed(),
    items   = m_stage.Kontagent_GetItemsDropped(),
    flies   = m_stage.Kontagent_GetResourceCollected(),
    bombs   = m_stage.Kontagent_GetExplosivesDestroyed()
  } );

  local section_name = m_stage.GetStaticData( stage_name ).section;

  -- FIXME: Is it used?
  if ( stage_data.type == "fight" ) then

    mediator.Notify( "View_Stage_StageFailed", { stage = stage_name, section = section_name } );

  end;

end;
--------------------------------------------------------------------------------
function private.Dialog_Fail_Hide ( func_end )

  int_dialog_stage_fail.Hide( function ()

    int_window.Hide( "int_dialog_stage_fail", func_end );

  end );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.DialogFriends_Prepare ( stage_name )
  
  if ( m_stage.GetCurrentStageData().type == "fight" ) or ( IsPC() ) then
    
    return;
    
  end;
  
  local friends_data = m_stage.GetFriendsStageScores( stage_name );

  int_dialog_stage_friends.Prepare( friends_data );

end;
--------------------------------------------------------------------------------
function private.DialogFriends_Update ( score, is_win )
  
  if ( m_stage.GetCurrentStageData().type == "fight" ) or ( IsPC() ) then
    
    return;
    
  end;

  int_dialog_stage_friends.Update( score, is_win );

end;
--------------------------------------------------------------------------------
function private.DialogFriends_Hide ()

  int_dialog_stage_friends.Hide();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.DialogWin_Show ()
  
  private.Trace( "DialogWin_Show" );
  
  local stage_data  = m_stage.GetCurrentStageData();
  local stage_score = m_stage.GetCurrentScore();
  local stage_moves = m_stage.MovesDone_GetCount() or 0;
  local score_old   = private.score_old;
  private.score_old = nil;

  local func_show_hero = function ( stage_type )

    int_dialog_stage_win.ShowHero( stage_type );

  end;

  local func_accept = function ()

    cmn.PlaySnd( "simply_click" );

    private.DialogWin_Hide( function ()

      local section_name = stage_data.section;
      local stage_name   = stage_data.name;
      
      local hide_panel = v_story.TryShow( 
        section_name,
        "CompleteStage", 
        stage_name,
        private.Finish
      );

      if ( hide_panel ) then
        
        private.TopPanel_Hide();
        
      end;

    end );

  end;

  local func_skin_attach = function ( parent )

    return v_hero.Skin_Attach( parent, stage_data.type );

  end;

  local func_skin_detach = function ( anm_name )

    v_hero.Skin_Detach( anm_name );

  end;

  local func_start = function ()

    local social_panel = false;
    
    if not ( stage_data.type == "fight" ) then
      
      social_panel = v_social.IsLogged();
      
      local friends_data = m_stage.GetFriendsStageScores( stage_data.name );
      
      if ( #friends_data > 0 ) then
        
        local outrun_count = 0;
      
        for f = 1, #friends_data do
        
          local friend_score = friends_data[ f ].score;
          local friend_id    = tostring( friends_data[ f ].userId );
          
          local friend_was_ahead  = friend_score > score_old;
          local friend_now_behind = stage_score  > friend_score;
          
          local friend_real = ( friend_id:match("^bot_.*$") == nil );
          
          if ( friend_real and friend_was_ahead and friend_now_behind ) then
            outrun_count = outrun_count + 1;
          end;
        
        end;
        
        --DbgTrace( "Outrun Count: " .. outrun_count );
          
        if ( outrun_count > 0 ) then
          mediator.Notify( "View_Stage_FriendsWon", outrun_count );
        end;
      
      end;
      
    end;
    
    int_dialog_stage_win.Show(
      stage_data.name,
      stage_data.type,
      tern( stage_data.type == "ride" )( stage_score, stage_moves ),
      func_skin_attach,
      func_skin_detach,
      func_accept,
      social_panel
    );
    
    private.Social_Score_TryShow( 
      "win", 
      stage_data.type, 
      stage_data.name, 
      stage_score, 
      score_old 
    );

    local is_win_anim_ended = false;

    if ( stage_data.type == "ride" ) then

      is_win_anim_ended = int_panel_stage_progress_ride.IsWinAnimEnded();

    elseif ( stage_data.type == "fight" ) then

      is_win_anim_ended = int_panel_stage_progress_fight.IsWinAnimEnded();

    end;

    if ( is_win_anim_ended ) then

      func_show_hero( stage_data.type );

    else

      mediator.SubscribeOnce(
        "Interface_PanelStageProgress_WinAnimEnd",
        func_show_hero
      );

    end;

  end;
  
  local func_WinDialogShow = function()
    
    int_window.AddWindowToQueue(
      "int_dialog_stage_win",
      func_start,
      { vignette = true }
    );
    
  end;
  
  if ( stage_data.type == "ride" ) then

    int_panel_stage_progress_ride.Update_StageComplete( function()
  
      if ( not int_window.IsAnyWindowOnScreen() ) then

        int_panel_stage_win.Show(
          "level_complete",
          func_WinDialogShow,
          func_skin_attach,
          func_skin_detach
        );

      else

        func_WinDialogShow();

      end;

    end );
  
  else
    
    func_WinDialogShow();
    
  end;

  public.Soundtrack_Stop();
  cmn.PlayMusic( "splash" );

  if ( stage_data.type == "ride" ) then

    cmn.PlaySnd( "level_win" );

  elseif ( stage_data.type == "fight" ) then

    cmn.PlaySnd( "assets/content/hero/aud_hero_win_fight" );

  end;

  common.Playhaven_LogPlacement( "level_success" );
  wiz.ChartBoost.LocationLevelComplete();  

  local getnoreward = function ()

    local stage_info = m_stage.GetSavedData( stage_data.name ) or {};
    return stage_info.noreward;

  end;

  local score_target = 0;
  local moves_target = 0;

  if ( stage_data.type == "ride" ) then

    score_target = m_stage.GetTargetByType( "GainScore" ).count;
    moves_target = stage_data.moves;

  end;

  local getscore = function ( stage_type )

    if ( stage_type == "fight" ) then
      
      local hero_data = m_stage.GetCurrentHero();

      return { ( hero_data.health_current..", "..hero_data.health ), 1 };

    end;

    return { ( m_stage.GetCurrentScore()..", "..score_target ), 0 };

  end;

  local scoredata = getscore( stage_data.type );
  local daily = tern( stage_data.realtype == "dailyquest" )( 
                      ( m_dailyquest.Challenge_GetCompletedCount() + 1 ), 
                      false );

  common.Kontagent_LogEvent( "stage",
  {
    section  = stage_data.section,
    stage    = stage_data.name,
    daily    = daily,
    name     = "success",
    state    = m_stage.Kontagent_GetStageState( stage_data.name, "success" ),
    noreward = getnoreward(),
    score    = scoredata[ 1 ],
    is_boss  = scoredata[ 2 ],
    moves    = m_stage.MovesDone_GetCount()..", "..moves_target,
    boost    = m_stage.Kontagent_GetBoostersUsed(),
    cells    = m_stage.Kontagent_GetCellsDestroyed(),
    items    = m_stage.Kontagent_GetItemsDropped(),
    flies    = m_stage.Kontagent_GetResourceCollected(),
    bombs    = m_stage.Kontagent_GetExplosivesDestroyed()
  } );

  mediator.Notify( "View_Stage_StageWon", { stage = stage_data.name } );

end;
--------------------------------------------------------------------------------
function private.DialogWin_Hide ( func_end )

  if ( IsMobile() ) then
    
    private.Social_Score_Hide();
    
  end;

  cmn.StopSnd( "level_win" );
  cmn.StopMusic( "splash" );

  --int_window.Hide( "int_dialog_stage_win", function ()
  int_dialog_stage_win.Hide( func_end );
  --end );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Social_Score_TryShow (
    point,
    stage_type,
    stage_name,
    stage_score,
    score_old
  )

  if ( stage_type == "fight" ) or ( IsPC() ) then 
    
    return; 

  end;

  private.score_data   = {
    point       = point,
    stage_name  = stage_name,
    stage_score = stage_score,
    score_old   = score_old
  };

  mediator.Subscribe(
    "Interface_Window_WindowOpened",
    private.Social_Score_WindowOpened
  );
  
  mediator.Subscribe(
    "Interface_Window_WindowClosedEnd",
    private.Social_Score_WindowClosed
  );

  v_social.Score.TryShow( point, stage_name, stage_score, score_old );

end;
--------------------------------------------------------------------------------
function private.Social_Score_Hide ()

  private.score_data = nil;

  mediator.Unsubscribe(
    "Interface_Window_WindowOpened",
    private.Social_Score_WindowOpened
  );
  
  mediator.Unsubscribe(
    "Interface_Window_WindowClosedEnd",
    private.Social_Score_WindowClosed
  );

  v_social.Score.Hide();

end;
--------------------------------------------------------------------------------
function private.Social_Score_WindowOpened ( name )

  local upper_name = int_window.Upper_Name();

  if ( upper_name ~= "int_dialog_stage_start" ) and
     ( upper_name ~= "int_dialog_stage_win" ) then

    v_social.Score.Hide();

  end;

end;
--------------------------------------------------------------------------------
function private.Social_Score_WindowClosed ( name )

  if ( int_window.Upper_Name() == "int_dialog_stage_start" ) then

    v_social.Score.TryShow( 
      private.score_data.point,
      private.score_data.stage_name,
      private.score_data.stage_score,
      private.score_data.score_old
    );

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.SetupLayout ( is_fight, has_res )
  
  local round  = common.Round;
  
  local h, w         = int.H(), int.W();
  local mid_x, mid_y = round( 0.50 * w ), round( 0.50 * h );
  
  -- ride interface position
  local hero_x     = mid_x;
  local hero_y     = round( 0.23 * h );
  local hero_scale = round( hero_y / 350, 2 );
  
  local hero_smooth_border = false;
  
  -- fight interface positions
  local fight_x     = hero_x;
  local fight_y     = hero_y;
  
  local fight_offset = hero_y;
  local fight_scale  = hero_scale;
  
  local fight_near = 350; -- will be scaled
  local fight_far  = fight_near;
  
  private.bg_offset = hero_y;
  
  -- set bonus popup position
  local bonus_x = mid_x;
  local bonus_y = math.max( round( fight_y - 330 * fight_scale ), 75 );
  
  -- booster panel size and position
  local boost_scale  = 1.0; --round( h / 1536, 2 );
  
  local boost_x      = mid_x;
  local boost_y      = round( 0.94 * h );
  local boost_hide_x = mid_x;
  local boost_hide_y = round( 1.06 * h );
  
  local boost_hideable = true;
  
  local boost_space  = round( boost_scale * 70 * 1.3 );
  local boost_margin = round( h - boost_y + boost_space / 2 );
  
  -- matchbox position
  local match_x = mid_x;
  local match_y = round( hero_y + ( ( h - hero_y - boost_margin ) / 2 ) );
  
  local match_size_h = round( 0.90 * w );
  local match_size_v = round( 0.90 * ( h - hero_y - boost_margin ) );
  
  local match_size   = math.min( match_size_h, match_size_v );
  
  -- score visible/hidden position
  local score_scale  =  hero_scale;
  local score_x      =  mid_x;
  local score_y      =  hero_y - 10;
  
  -- resources panel position
  local res_x      =  w - 45;
  local res_y      =  135;
  local res_hide_x =  w + 55;
  local res_hide_y =  res_y;
  
  -- top panels
  local is_landscape = false;
  
  -- stage number
  local num_x      = w - 35;
  local num_y      = h - 40;
  local num_hide_x = num_x;
  local num_hide_y = num_y + 50;
  
  local num_scale = 1.0;
  
  local num_hideable = true;
  
  -- custom shading flag for mtach-3 field
  private.match_custom_shade = false;
  
  if ( int.Landscape() ) then
    
    local match_space = round( 0.9 * h );
    
    match_size = round( 0.90 * match_space );
    match_x    = round( w - ( match_space / 2 ) );
    match_y    = round( mid_y + ( ( h - match_space ) / 2 ) - 10 );
    
    hero_scale = 1.2;
    hero_x     = round( ( w - match_space ) / 2 );
    hero_y     = round( 0.65 * h );
    
    hero_smooth_border = true;
    
    private.bg_offset = hero_y;
    
    score_scale  = 1.0;
    score_x      = hero_x;
    score_y      = hero_y - 10;
    
    boost_scale  = 1.0;--0.7;
    boost_x      = hero_x;
    boost_y      = round( 0.75 * h );
    
    boost_hide_x = boost_x;
    boost_hide_y = boost_y;
    
    boost_hideable = false;
    
    -- Shift boost pos if resources are exist
    if ( has_res ) then
      
      boost_y      = round( 0.84 * h );
      boost_hide_y = boost_y;
      
    end;
    
    res_x      =  hero_x;
    res_y      =  round( 0.72 * h );
    res_hide_x =  hero_x;
    res_hide_y =  round( 0.72 * h );
    
    is_landscape = true;
    
    num_x      =  35;
    num_y      =  35;
    num_hide_x =  num_x;
    num_hide_y = -num_y;
    
    num_scale = 0.8;
    
    num_hideable = boost_hideable;
    
    if ( is_fight ) then
      
      match_x = mid_x;
      match_y = round( 0.47 * h );
      
      fight_y     = hero_y;
      fight_scale = 1.0;
      
      fight_near = round( 0.35 * w / fight_scale );
      fight_far  = round( 0.90 * w / fight_scale );
      
      private.bg_offset = fight_y;
      
      bonus_x = round( 0.12 * w );
      bonus_y = round( fight_y - 330 * fight_scale );
      
      boost_x      = match_x;
      boost_y      = round( 0.92 * h );
      boost_hide_x = boost_x;
      boost_hide_y = round( 1.10 * h );
      
      boost_hideable = true;
      
      num_x      = w - 35;
      num_y      = h - 40;
      num_hide_x = num_x;
      num_hide_y = num_y + 50;
      
      num_scale = 1.0;
      
      num_hideable = true;
      
      -- See private.Match3_ShadesOn, private.Match3_ShadesOff
      private.match_custom_shade       = true;
      private.match_custom_shade_shift = h - match_y;
      
    end;
      
  end;
  
  v_match3.SetMaxSize( match_size );
  v_match3.SetPos( match_x,  match_y );
  
  int_panel_stage_progress_ride.SetScale( hero_scale );
  int_panel_stage_progress_ride.SetFgPos( hero_x, hero_y );
  int_panel_stage_progress_ride.SetBgOffset( hero_y );
  int_panel_stage_progress_ride.SetSmoothBorder( hero_smooth_border );
  
  int_panel_stage_progress_fight.SetScale( fight_scale );
  int_panel_stage_progress_fight.SetOffset( fight_y );
  int_panel_stage_progress_fight.SetDistances( fight_near, fight_far );
  int_panel_stage_progress_fight.SetSmoothBorder( hero_smooth_border );
  
  int_popup_stage_bonus.SetPos( bonus_x, bonus_y );
  
  int_panel_stage_score.SetScale( score_scale );
  int_panel_stage_score.SetPos( score_x, score_y );
  
  int_panel_stage_boosters.SetScale( boost_scale );
  int_panel_stage_boosters.SetPosVisible(      boost_x,      boost_y );
  int_panel_stage_boosters.SetPosHidden(  boost_hide_x, boost_hide_y );
  int_panel_stage_boosters.SetHideable( boost_hideable );
  
  int_panel_stage_resources.SetPosVisible(      res_x,      res_y );
  int_panel_stage_resources.SetPosHidden(  res_hide_x, res_hide_y );
  int_panel_stage_resources.SetLandscape( is_landscape );
  
end;
--------------------------------------------------------------------------------
function private.Start ()

  mediator.Subscribe(
    "Model_Stage_FinalBonuses_StartAnimation",
    private.FinalBonuses_StartAnimation
  );
  
  private.Match3_SetInput( true );

  if ( private.started ) then

    private.PreOpen();

    return;

  end;

  private.started   = true;
  private.finalized = false;

  screen.Switch( "scr_stage" );

--  if ( v_map.TopPanel_IsVisible() ) then
--
--    DbgTrace( "[ VIEW ][ STAGE ] Map top panel is still visible!" );
--    v_map.TopPanel_Hide();
--
--  end;

  common.Playhaven_LogPlacement( "level_start" );

end;
--------------------------------------------------------------------------------
function private.Finish ()
  
  private.Trace( "Finish" );
  
  m_stage.Kontagent_SetCurrentStage( nil );
  
  if ( private.started ) then

    m_stage.TryFinish();
  
    private.started = false;
    
    mediator.Unsubscribe(
      "Model_Stage_FinalBonuses_StartAnimation",
      private.FinalBonuses_StartAnimation
    );

    public.Soundtrack_Stop();

    screen.Switch(
      "scr_map",
      function () 
        
        ObjSet( "zmv_map", { input = true } );
        
      end
    );

  end;

end;
--------------------------------------------------------------------------------
function private.PreOpen ( func_after )

  local stage_data = m_stage.GetCurrentStageData(); 
  local is_fight   = ( stage_data.type == "fight" );
  local has_res    = ( #m_stage.GetCurrentResources() > 0 );

  -- HACK: add enemy name to dailyquest
  if ( is_fight ) then

    if ( not stage_data.enemy ) then

      stage_data.enemy = private.ext_enemy; -- see public.TryStart

    end;

  end;

  common.LocalNotifications_SetData( "comeback_level", stage_data.name );
  private.DialogFriends_Prepare( stage_data.name );

  public.Soundtrack_Start();

  private.SetupLayout( is_fight, has_res );
  private.SetStageBackgroundColor( stage_data.zone, stage_data.type, ( stage_data.enemy or {} ).name );

  v_match3.Start();
  private.Match3_ShadesReset();

  int_popup_stage_score.Attach();
  int_popup_stage_combo.Attach();

  private.PanelProgress_Show();
  private.PanelScore_Show();

  private.PanelSettings_Show();
  --private.PanelSettings_SetInput( true );

  private.PanelResources_Attach();
  private.PanelBoosters_Show();
  
  private.TopPanel_TryShow();

  local enemy = stage_data.enemy or {};
  
  private.DialogTargets_Show(
    stage_data.state,
    stage_data.targets,
    enemy.name
  );
  
  private.targets = stage_data.targets;

  if ( func_after ) then
 
    func_after();

  end;

end;
--------------------------------------------------------------------------------
function private.Open ( func_after ) -- After splash fade

  private.is_active = true;

  if ( func_after ) then 
  
    func_after();
 
  end;

end;
--------------------------------------------------------------------------------
function private.Close ( func_after )

  if ( private.boosters_panel_shown ) then

    private.PanelBoosters_Hide();

  end;

  int_popup_stage_score.Detach();
  int_popup_stage_combo.Detach();

  private.PopupBonus_Hide();
  private.TopPanel_Hide();

  private.Cleanup();

  if ( func_after ) then 
  
    func_after();
 
  end;

end;
--------------------------------------------------------------------------------
function private.Cleanup ()

  for _, target_data in pairs( private.targets ) do

    if ( target_data.type == "DestroyCell" ) then

      for i = 1, target_data.count do

        m_object.Delete( private.this.effect.cell_destroyed.."_"..i );

      end;

    end;

  end;

  private.targets = nil;

end;
--------------------------------------------------------------------------------
function private.AfterClose ( func_after )
  
  if ( private.panel_score_shown ) then
    private.PanelScore_Hide();
  end;

  private.PanelSettings_Hide();
  private.PanelProgress_Hide();
  v_match3.Finish();

  private.is_active = false;
  
  if ( func_after ) then
    
    func_after();
    
  end;
  
  private.popup_show = false;

end;
--------------------------------------------------------------------------------
function private.TryContinueNEW ()

  mediator.Notify( "View_Match3_FieldEstablished" );
                           
end;
--------------------------------------------------------------------------------
function private.TryContinue ()

  if ( private.finalized ) then

    return;

  end;
  
  local stage_type = private.GetCurrentStageType();
  
  -- JK: wait for attacks finish
  if ( stage_type == "fight"                                    ) and
     ( not int_panel_stage_progress_fight.Attack_IsQueueEmpty() ) then
     
     return;
     
  end;

  if ( not m_stage.TryContinue() ) then

    private.Match3_SetInput( false );
    
    if ( stage_type == "fight" ) then

      private.Match3_ShadesOn();

    end;

    return;

  end;
  
  private.Match3_SetInput( true );

  -- Unlock field.
  private.Match3_ShadesOff();
  --private.PanelBoosters_SetInput( true );

  mediator.Notify( "View_Stage_FieldUnlocked" );

  local anm_playing = public.IsAnimationPlaying();
  
  if ( m_stage.TryFinalize( anm_playing ) ) then

    private.Match3_SetInput( false );

    -- Lock field.
    private.PanelBoosters_SetInput( false, "TryContinue" );

    private.finalized = true;

    local time_finalize = 0.6;

    if ( not m_stage.IsCanContinue() ) then

      time_finalize = 2.0;

    end;

    ObjSet( "tmr_stage_finalize",
    {
      time    = time_finalize,
      playing = true,
      endtrig = function ()

        private.DialogComplete_Show();

      end,
    } );

    m_stage.Bonus_TryActivate();

  else

    if ( m_stage.IsWin() ) then

      private.Match3_SetInput( false );
      --private.PanelSettings_SetInput( false );
      private.PanelBoosters_SetInput( false );

      private.PopupBonus_Hide();

    else

      m_stage.Bonus_TryActivate();

      private.PopupBonus_Show();

    end;

  end;

  --DbgTrace( "TryContinue Finish" );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.IsAnimationPlaying ()
  
  return int_panel_stage_win.IsActive();
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.OnStageComplete ( params )

  private.Trace( "OnStageComplete." );

  if ( params.moves ) then
    
    local func_end = function ()

      int_panel_stage_progress_ride.Update_BonusRun();

      mediator.Notify( "View_Stage_AnimationIsFinished" );

    end;

    if ( not int_window.IsAnyWindowOnScreen() ) then

      int_panel_stage_win.Show( "fox_appear", func_end );
      
      cmn.PlaySnd( "you_win" );

    else

      func_end();

    end;

  end;

end;
--------------------------------------------------------------------------------
function private.OnTargetUpdated ( params )

  private.Trace( "OnTargetsUpdated." );

  local target = params;

  if ( target.type == "GainScore" ) then

    local action_data = { type = "HeroRide", score_current = params.progress --[[, score_total = target.count]] };
  
    int_panel_stage_progress_ride.Update( action_data );

    private.DialogFriends_Update( action_data.score_current, m_stage.IsWin() );

  end;

end;
--------------------------------------------------------------------------------
function private.OnFieldActivated ()
  
  --

end;
--------------------------------------------------------------------------------
function private.OnFieldEstablished ()

  --

end;
--------------------------------------------------------------------------------
function private.OnFieldNeedShuffle ()
  
  local function func_shuffle ()
  
    common.MessageSend( "Command_Match3_FieldShuffle" );
    
    private.Shuffle_EffectShow();
  
  end;
  
  local function func_shuffle_dialog ()
  
    if ( not m_stage.IsWin() ) then
      
      local x, y = v_match3.GetFieldCenter();
      
      int_dialog_stage_shuffle.Show( x, y, func_shuffle );

    end;
  
  end;
  
  if ( private.match3_shades ) then
    
    mediator.SubscribeOnce( "View_Stage_Match_ShadesOff", func_shuffle_dialog );
    
  else
    
    func_shuffle_dialog();
    
  end;

end;
--------------------------------------------------------------------------------
function private.Shuffle_EffectShow ()

  ObjAttach( "anm_shuffle", "obj_match3" );
  
  local x, y = v_match3.GetFieldCenter();

  ObjSet( "anm_shuffle", { pos_x = x, pos_y = y } );

  AnimPlay( "anm_shuffle", "shuffle", function ()

    ObjAttach( "anm_shuffle", "hub_stage" );

  end );

end;
--------------------------------------------------------------------------------
function private.OnCellDestroyed ( params )

  -- Bug
  if ( params.name == "Bug" ) then

    local colors = 
    {
      Magenta = 4,
      Orange  = 5,
      Yellow  = 6,
      Blue    = 3,
      Red     = 1,
      Green   = 2     
    };

    params.animfunc = "bug_landing_"..colors[ params.color ];
    
    private.OnCellDestroyedReceive( params );

  end;
  
  -- Snowflake
  if ( params.name == "Snowflake" ) then

    params.animfunc = "snowflake_"..params.size.."_get";
    
    private.OnCellDestroyedReceive( params );

  end;

end;
--------------------------------------------------------------------------------
function private.OnBonusRemoved ( params )

  if ( params.type ) then

    local sound_name = "chip_"..string.lower( params.type ).."_remove";

    if ( params.type == "Vertical" ) then

      sound_name = "chip_horizontal_remove";

    end;

    if ( params.type == "NucleusSecond" ) then

      sound_name = "chip_nucleus_remove";

    end;

    cmn.PlaySnd( sound_name );

  end;

end;
--------------------------------------------------------------------------------
function private.OnBonusSpawn ( params )

  if ( params.type ) then

    local sound_name = "chip_"..string.lower( params.type ).."_create";

    if ( params.type == "Vertical" ) then

      sound_name = "chip_horizontal_create";

    end;

    cmn.PlaySnd( sound_name );

  end;

end;
--------------------------------------------------------------------------------
function private.OnBonusCountChanged ( ability )
  
  local ability_count = ability.counter - 1;
      
  local left_count  = 0;
  local right_count = 0;
  
  if ( ability.name == "stun" ) or ( ability.name == "dot" ) then
    right_count = ability_count;
  else
    left_count = ability_count;
  end;
  
  int_panel_stage_progress_fight.ShowDots( left_count, right_count );
  
end;
--------------------------------------------------------------------------------
function private.OnComboActivated ( params )

  --

end;
--------------------------------------------------------------------------------
function private.OnCellDestroyedReceive ( params )

  local target_list = m_stage.GetCurrentTargets();
  local target_data = m_stage.GetTargetByType( "DestroyCell" );

  int_panel_stage_top.Goals_AddToQueue( target_data );

  local effect_index = target_data.progress;
  local effect_name  = private.this.effect.cell_destroyed.."_"..effect_index;
  
  m_object.Clone(
    private.this.effect.cell_destroyed,
    "int_control",
    "_"..effect_index
  );

  local function func_end ()

    int_panel_stage_top.Goals_Receive(
      target_data.type,
      function ()

        int_panel_stage_top.Goals_Update(
          nil, -- JK: use queue
          target_data.type,
          function ()

            m_object.Delete( effect_name );

          end
        );

      end
    );

  end;

  local receiver_name = int_panel_stage_top.Goals_GetReceiver( target_data.type );
  local receiver_pos  = int.Position( receiver_name );

  m_object.Set(
    effect_name, 
    {
      animfunc = params.animfunc,
      playing  = true,
      pos_x    = params.x,
      pos_y    = params.y,
      pos_z    = 20  
    }
  );
  
  ObjAnimate( effect_name, "pos_xy", 0, 0, func_end,
  { 
    0.0, 3, params.x + 35,          params.y + 35,
    2.0, 3, receiver_pos[ 1 ] + 50, receiver_pos[ 2 ]
  } );

  ObjAnimate( effect_name, "scale_xy", 0, 0, "", 
  { 
    0.0, 3, 1.0, 1.0,
    2.0, 3, 0.5, 0.5
  } );

  ObjAnimate( effect_name, "alp", 0, 0, "", 
  { 
    0.0, 3, 1.0,
    1.5, 3, 1.0,
    2.0, 3, 0.0,      
  } );

end;
--------------------------------------------------------------------------------
function private.OnItemDropped ( params )

  private.Trace( "OnItemDropped: name = "..params.name.."." );

  local target_data = m_stage.GetTargetByType( "DropItem" );
  
  int_panel_stage_top.Goals_Update( target_data, target_data.type );
  
  local index = private.EffectIndex_Get();

  local effect_name = string.format( "anm_item_drop_effect_%s", index );

  ObjClone( "anm_item_drop_effect", "_"..index, 1, "int_control" );

  ObjSet( effect_name, 
  { 
    active  = true, 
    visible = true,
    pos_x   = params.x + 35,
    pos_y   = params.y + 35,
    pos_z   = 899
  } );

  local item_name = string.format( "anm_item_drop_icon_%s", index );

  ObjClone( "anm_item_drop_icon", "_"..index, 1, "int_control" );

  local section = string.match( m_stage.GetCurrentStageData().section, "%d+" );

  ObjSet( item_name, 
  { 
    active   = true, 
    visible  = true,
    pos_x    = params.x + 35,
    pos_y    = params.y + 35,
    animfunc = "item_"..string.lower( params.name ).."_"..section,
    playing  = false,
    res      = "assets/content/match3/items/anm_item_"..string.lower( params.name ),
    pos_z    = 898
  } );

  local func_anim_end = function ()

    local func_end = function ( effect_name )

      ObjDelete( effect_name );

      AnimPlay( item_name, "item_"..string.lower( params.name ).."_"..section, function () 

        ObjDelete( item_name );
        
      end );

      local target_x = 0.5 * int.W() + private.item_dropped_target_offset_x;
      local target_y = 0.5 * int.H() + private.item_dropped_target_offset_y;

      ObjAnimate( item_name, "pos_xy", 0, 0, "", 
      { 
        0.0, 3, params.x + 35, params.y + 130,
        1.0, 3, target_x,      target_y
      } );

    end;

    AnimPlay( effect_name, "appear", function () func_end( effect_name ) end );

    cmn.PlaySnd( "pet_fly" );

  end;

  ObjAnimate( effect_name, "pos_xy", 0, 0, func_anim_end, 
  { 
    0.0, 3, params.x + 35, params.y + 35,
    0.2, 3, params.x + 35, params.y + 95,
    0.3, 3, params.x + 35, params.y + 95
  } );

  ObjAnimate( item_name, "pos_xy", 0, 0, "", 
  { 
    0.0, 3, params.x + 35, params.y + 35,
    0.2, 3, params.x + 35, params.y + 95,
    0.3, 3, params.x + 35, params.y + 95
  } );

end;
--------------------------------------------------------------------------------
function private.OnResourceRewardCountChanged ( resources_list )

  local resources_animfuncs = 
  {
    resource1 = { "match_7", "collect_7" },
    resource2 = { "match_8", "collect_8" },
    resource3 = { "match_9", "collect_9" }
  };
  
  cmn.PlaySnd( "glowworms_match" );

  for j = 1, #resources_list do

    local index     = private.EffectIndex_Get();
    local anim_name = string.format( "anm_resource_remove_%s", index );

    ObjClone( "anm_resource_remove", "_"..index, 1, "int_control" );

    ObjSet( anim_name, 
    { 
      active  = true, 
      visible = true,
      pos_x   = resources_list[ j ].x,
      pos_y   = resources_list[ j ].y,
      pos_z   = 20  
    } );

    local func_anim_end = function ( anim_name )

      ObjStopAnimate( anim_name, "pos_y" );

      local func_end = function ( anim_name )

        local resources_data = m_stage.GetCurrentResources();
  
        for i = 1, #resources_data do

          if ( resources_data[ i ].name == resources_list[ j ].name ) then

            private.PanelResources_Update( resources_data[ i ] );

          end;

        end;
  
        ObjDelete( anim_name );

      end;

      private.Resource_FlyToPanel(
        anim_name,
        resources_list[ j ].name,
        function ()
          func_end( anim_name )
        end
      );

    end;

    AnimPlay( anim_name, resources_animfuncs[ resources_list[ j ].name ][ 1 ], function () 

      AnimPlay( anim_name, resources_animfuncs[ resources_list[ j ].name ][ 2 ], "" );

      local timer_name = "tmr_stage_resources_"..index;

      ObjClone( "tmr_stage_resources", "_"..index, 1, InterfaceName );

      ObjSet( timer_name,
      {
        time    = 0.01,
        playing = true,
        endtrig = function ()

          ObjDelete( timer_name );

          func_anim_end( anim_name );

        end
      } );

    end );

  end;

end;
--------------------------------------------------------------------------------
function private.Resource_FlyToPanel ( anim_name, resource_name, func_end )

  local target_name = 
    "spr_int_panel_stage_resources_resource_"..
    string.match( resource_name, "%d+" ).."_icon";

  local current_pos     = GetObjPosByObj( anim_name );
  local target_pos      = int_panel_stage_resources.GetPosVisible();
  local offset_x        = math.random( -80, 80 );
  local tme             = 0.6 + math.random( 1, 3 ) / 10;
  local target_offset_x = 0;
  local target_offset_y = 10;

  ObjSet( anim_name,
  {
    pos_x = current_pos[ 1 ],
    pos_y = current_pos[ 2 ],
    pos_z = 20
  } );

  ObjAnimate( anim_name, "scale_xy", 0, 0, "", 
  {
    0.0,       3, 1.0, 1.0,
    tme - 0.1, 3, 1.0, 1.0,
    tme,       3, 0.3, 0.3
  } );

  ObjAnimate( anim_name, "alp", 0, 0, "", 
  {
    0.0,       1, 1.0,
    tme - 0.1, 1, 1.0,
    tme,       1, 0.0
  } );

  ObjAnimate( anim_name, "pos_y", 0, 0, func_end, 
  {
    0.0, 1, current_pos[ 2 ],
    tme, 1, target_pos[ 2 ] + target_offset_y
  } );

  ObjAnimate( anim_name, "pos_x", 0, 0, "", 
  {
    0.0,       1, current_pos[ 1 ],
    0.5 * tme, 1, current_pos[ 1 ] + offset_x,
    tme,       1, target_pos[ 1 ] + target_offset_x
  } );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PanelScore_Show ( stage_name )  

  if ( private.GetCurrentStageType() == "fight" ) then

    return;

  end;

  local color = private.GetStageBackgroundColor();

  int_panel_stage_score.Show( color );

  private.panel_score_shown = true;

end;
--------------------------------------------------------------------------------
function private.PanelScore_Update ( params )

  if ( private.GetCurrentStageType() == "fight" ) then

    return;

  end;

  int_panel_stage_score.Update( params.score );

end;
--------------------------------------------------------------------------------
function private.PanelScore_Hide ()

  if ( private.GetCurrentStageType() == "fight" ) then

    return;

  end;

  int_panel_stage_score.Hide();

  private.panel_score_shown = false;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.TopPanel_TryShow ()
  
  local stage_type = m_stage.GetCurrentStageData().type;
  
  private.TopPanel_SetupLevel();
  
  if ( stage_type == "fight" ) then
    
    private.TopPanel_ShowLevelStandalone();
    
    return;
    
  end;
  
  if ( not int.Landscape() ) then
    
    private.TopPanel_ShowLevelStandalone();
    
  end;
  
  private.TopPanel_SetupMoves();
  private.TopPanel_SetupGoals();
  
  int_panel_stage_top.Show();

end;
--------------------------------------------------------------------------------
function private.TopPanel_ShowLevelStandalone ()
  
  int_panel_stage_top.Level_Show();
  
end;
--------------------------------------------------------------------------------
function private.TopPanel_Hide ()

  int_panel_stage_top.Hide();

  private.PanelResources_Hide();

end;
--------------------------------------------------------------------------------
function private.TopPanel_SetupLevel ()

  local stage_name   = m_stage.GetCurrentStageName();
  local stage_data   = m_stage.GetCurrentStageData();
  
  if ( stage_data.realtype == "dailyquest" ) then

    int_panel_stage_top.Level_Disable();
    
    return;

  end;

  local stage_section = stage_data.section;
  local stage_number  = private.Number_Relative( stage_section, stage_name );
  
  int_panel_stage_top.Level_Setup(
    stage_number,
    stage_data.state,
    tern( stage_data.realtype == "bonus" )( stage_data.realtype, stage_data.type ),
    m_stage.IsStateless()
  );

end;
--------------------------------------------------------------------------------
function private.TopPanel_SetupMoves ()

  local moves = m_stage.GetCurrentMoves() or
                m_stage.GetCurrentStageData().moves;

  if ( moves ) then

    int_panel_stage_top.Moves_Update( moves );

  end;

end;
--------------------------------------------------------------------------------
function private.TopPanel_UpdateMoves ( moves )

  int_panel_stage_top.Moves_Update( moves, m_stage.IsWin() );

end;
--------------------------------------------------------------------------------
function private.TopPanel_SetupGoals ()

  local goals = m_stage.GetCurrentTargets();

  if ( goals      ) and
     ( #goals > 0 ) then
    
    int_panel_stage_top.Goals_Setup(
      goals,
      m_stage.GetCurrentStageData().section
    );

  end;

end;
--------------------------------------------------------------------------------
function private.TopPanel_UpdateGoals ( goal_data )

  if ( goal_data.type ~= "DropItem"  ) and
     ( goal_data.name ~= "Bug"       ) and
     ( goal_data.name ~= "Snowflake" ) then

     int_panel_stage_top.Goals_Update( goal_data, goal_data.type );

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.OnMovesCountChanged ( moves )
  
  private.TopPanel_UpdateMoves( moves );
  private.PanelBoosters_Appearance( moves );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PanelTime_StartTimer ()
  
  int_panel_stage_time.StartTimer();

end;
--------------------------------------------------------------------------------
function private.PanelTime_StopTimer ()
  
  int_panel_stage_time.StopTimer();

end;
--------------------------------------------------------------------------------
function private.PanelTime_Show ()

  local time = m_stage.GetCurrentStageData().time;

  local func_change_time = function ( time_delta )

    return m_stage.TryChangeTimeCount( time_delta );

  end;

  local func_continue = function ()

    private.TryContinue();

  end;

  if ( time ) then

    int_panel_stage_time.Show( time, func_change_time, func_continue );

  end;

end;
--------------------------------------------------------------------------------
function private.PanelTime_Update ( time )

  int_panel_stage_time.Update( time );

end;
--------------------------------------------------------------------------------
function private.PanelTime_Hide ()

  int_panel_stage_time.Hide();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PanelResources_Attach ()

  local resources_data = m_stage.GetCurrentResources();

  if ( resources_data ) and ( #resources_data > 0 ) then

    int_panel_stage_resources.Attach( resources_data );

  end;

end;
--------------------------------------------------------------------------------
function private.PanelResources_Show ()

  int_panel_stage_resources.Show();

end;
--------------------------------------------------------------------------------
function private.PanelResources_Update ( resource_data )

  int_panel_stage_resources.Update( resource_data );

end;
--------------------------------------------------------------------------------
function private.PanelResources_Hide ()

  int_panel_stage_resources.Hide();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PanelProgress_Show ()

  local stage_data = m_stage.GetCurrentStageData();
  local hero_data  = m_stage.GetCurrentHero();
  local enemy_data = m_stage.GetCurrentEnemy();

  stage_data.color = private.GetStageBackgroundColor();
  
  if ( stage_data.type == "ride"  ) or
     ( stage_data.type == "bonus" ) then

    local func_skin_attach = function ( destination )

      return v_hero.Skin_Attach( destination );

    end;

    int_panel_stage_progress_ride.Show(
      stage_data,
      func_skin_attach,
      v_hero.Skin_Detach
    );

  end;

  if ( stage_data.type == "fight" ) then

    local func_skin_attach = function ( destination )

      return v_hero.Skin_Attach( destination, "fight" );

    end;
    
    local skin_name = m_skins.GetSkin();
    
    hero_data.skin_name = skin_name;
    hero_data.skin_res  = m_skins.GetContent( skin_name );

    int_panel_stage_progress_fight.Show(
      stage_data,
      hero_data,
      enemy_data,
      func_skin_attach,
      v_hero.Skin_Detach
    );

  end;

end;
--------------------------------------------------------------------------------
function private.PanelProgress_Hide ()

  if ( private.GetCurrentStageType() == "ride"  ) or
     ( private.GetCurrentStageType() == "bonus" ) then

    int_panel_stage_progress_ride.Hide();

  end;

  if ( private.GetCurrentStageType() == "fight" ) then

    int_panel_stage_progress_fight.Hide();

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PanelBoosters_Show ()

  local stage         = m_stage.GetCurrentStageData();
  local boosters_list = m_stage.Boosters_GetStageList( stage.name, "stage" );

  if( #boosters_list < 1 ) then

    private.boosters_panel_shown = false;
    return;

  end;

  private.boosters_panel_shown = true;

  mediator.Subscribe( "Model_Boosters_CountChanged", private.PanelBoosters_Update );

  local boosters_data = m_boosters.GetBoostersData( boosters_list );

  local func_tap = function ( booster_name )

    private.PanelBoosters_SetInput( false, "Tap" );
    private.PanelSettings_SetInput( false );

    local tap = v_boosters.Tap( "stage", booster_name, stage.name );

    return tap;

  end;

  local func_popup_show = function ( caption, text, pos, parent, inverse )

    int.ShowPopupFlat( caption, text, pos, parent, inverse );

  end;

  local func_popup_hide = function ( instant )

    int.HidePopupFlat( instant );

  end;

  private.PanelBoosters_SetInput( true, "PanelBoosters_Show" );

  int_panel_stage_boosters.Show( boosters_data, func_tap, func_popup_show, func_popup_hide );

end;
--------------------------------------------------------------------------------
function private.PanelBoosters_Update ()

  local stage = m_stage.GetCurrentStageData();

  local boosters_list = m_stage.Boosters_GetStageList( stage.name, "stage" );
  local boosters_data = m_boosters.GetBoostersData( boosters_list );

  int_panel_stage_boosters.Update( boosters_data );

end;
--------------------------------------------------------------------------------
function private.PanelBoosters_Hide ()

  mediator.Unsubscribe(
    "Model_Boosters_CountChanged",
    private.PanelBoosters_Update
  );

  int_panel_stage_boosters.Hide();

  v_boosters.ResetTapCounter();

end;
--------------------------------------------------------------------------------
function private.PanelBoosters_Use ( params )

  local booster_name = params.name;

  int_panel_stage_boosters.StopAnim( booster_name );

  v_boosters.Use( params, m_stage.GetCurrentStageName() );

  if not ( params.result ) then

    private.PanelBoosters_SetInput( true, "Not Use" );
    private.PanelSettings_SetInput( true );

  else
  
    m_stage.Kontagent_SetBoostersUsed( 1 );

  end;

end;
--------------------------------------------------------------------------------
function private.PanelBoosters_HideAnim ( params )

  local booster_name = params.name;

  if ( int_panel_stage_boosters.StopAnim( booster_name ) ) then
    
    private.PanelBoosters_SetInput( true, "HideAnim" );
    private.PanelSettings_SetInput( true );
    
  end;

end;
--------------------------------------------------------------------------------
function private.PanelBoosters_SetInput ( value, point )

  --DbgTrace( "Booster panel input: "..tostring( value )..", point: "..point );
  
  int_panel_stage_boosters.SetInput( value );

end;
--------------------------------------------------------------------------------
function private.PanelBoosters_Appearance ( moves )
  
  if  ( moves < 5 and not m_stage.IsWin() ) then
  
    int_panel_stage_boosters.EmergencyMode_TryStart();
  
  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PanelSettings_Show ()

  local function func_hide_others ()

    ObjSet( "scr_stage", { active = false } );
    --mediator.Notify( "View_Stage_Pause", true );

  end;

  local function func_show_others ()

    ObjSet( "scr_stage", { active = true } );
    --mediator.Notify( "View_Stage_Pause", false );

  end;

  local function func_show_options ( func_after )

    v_settings.Dialog_Show( func_after );

  end;

  local function func_exit_stage ( func_reject, func_accept )

    private.DialogExit_Show( func_reject, func_accept );

  end;

  int_panel_stage_settings.SetHandlers( func_show_options, func_exit_stage,
                                        func_show_others, func_hide_others );
                                        
  int_panel_stage_settings.Show(); 

end;
--------------------------------------------------------------------------------
function private.PanelSettings_Hide ()
  
  int_panel_stage_settings.Hide();

end;
--------------------------------------------------------------------------------
function private.PanelSettings_SetInput ( value )

  int_panel_stage_settings.SetInput( value );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PopupBonus_Show ()

  local stage_type = private.GetCurrentStageType(); 

  local bonus_data = m_stage.Bonus_GetData();

  if ( bonus_data and not private.finalized ) then
    
    local func_OnShow = function ()
    
      mediator.Notify( "View_Stage_PopupBonus_Activated" );
    
    end;

    int_popup_stage_bonus.Show(
      bonus_data.chip_types,
      stage_type,
      bonus_data.target,
      bonus_data.ability,
      func_OnShow
    );

  end;

end;
--------------------------------------------------------------------------------
function private.PopupBonus_Hide ()

  local activated = m_stage.Bonus_IsActivated();

  if ( activated ) then

    int_panel_stage_progress_ride.Update_PopupHit();

  end;

  if not ( activated ) then

    activated = nil;

  end;

  int_popup_stage_bonus.Hide( activated );

end;
--------------------------------------------------------------------------------
function private.BonusAbility_Hide ()

  int_panel_stage_progress_fight.Update_Ability();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.OnHeroAttackActivated ( params )

  private.Trace( "OnHeroAttackActivated: damage = "..params.damage..", health = "..params.enemy_health.."." );

  local action_data =
  {
    type           = "HeroAttackActivated",
    delta          = - params.damage,
    ability        = params.ability,
    ability_delta  = - params.ability_delta,
    health_current = params.enemy_health,
    health_total   = m_stage.GetCurrentEnemy().health
  };

  int_panel_stage_progress_fight.Update( action_data, private.TryContinue );

  private.PopupBonus_Hide();

end;
--------------------------------------------------------------------------------
function private.OnEnemyAttackActivated ( params )

  private.Trace( "OnEnemyAttackActivated: damage = "..params.damage..", hero_health = "..params.hero_health.."." );

  local action_data = 
  {
    type           = "EnemyAttackActivated",
    delta          = - params.damage,
    health_current = params.hero_health,
    health_total   = m_stage.GetCurrentHero().health
  };  

  int_panel_stage_progress_fight.Update( action_data, private.TryContinue );

  private.PopupBonus_Hide();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PopupScore_Show ( chip_x, chip_y, chip_type, chip_score )

  local popup_color =
  {
    Unknown = { r = 0.500, g = 0.500, b = 0.500 },
    Magenta = { r = 0.922, g = 0.000, b = 0.922 },
    Orange  = { r = 1.000, g = 0.432, b = 0.040 },
    Yellow  = { r = 1.000, g = 0.863, b = 0.000 },
    Blue    = { r = 0.118, g = 0.412, b = 1.000 },
    Red     = { r = 1.000, g = 0.000, b = 0.000 },
    Green   = { r = 0.000, g = 1.000, b = 0.197 },
    Heart   = { r = 1.000, g = 1.000, b = 1.000 },
    Sword   = { r = 1.000, g = 0.000, b = 0.000 },
    Bomb    = { r = 0.197, g = 0.000, b = 0.197 },
    Dirt    = { r = 0.500, g = 0.500, b = 0.560 },
    Cage    = { r = 1.000, g = 1.000, b = 1.000 },
    Rock    = { r = 1.000, g = 1.000, b = 1.000 },
    Resource1    = { r = 1.000, g = 0.863, b = 0.000 },
    Resource2    = { r = 0.118, g = 0.412, b = 1.000 },
    Resource3    = { r = 0.922, g = 0.000, b = 0.922 }
  };
  
  if ( not chip_x ) and ( not chip_y ) then
    
    chip_x, chip_y = v_match3.GetFieldCenter();   
    
  end;

  local text_color = popup_color[ chip_type ] or popup_color[ "Unknown" ];

  int_popup_stage_score.Show( chip_x, chip_y, text_color, chip_score );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.FinalBonuses_StartAnimation ( params )

  if ( params.moves == 0 ) then
    
    cmn.PlaySnd( "count_moves_off" );
    
  end;
  
  local pfx_name  = "pfx_spawn_bonuses_"..params.moves;

  m_object.Clone(
    "pfx_spawn_bonuses",
    "int_control",
    "_"..params.moves,
    1
  );
  
  local func_end = function ()
  
    m_stage.FinalBonuses_ChangeScore(
      params.chip_x,
      params.chip_y,
      params.chip_type,
      params.stage
    );
    
    m_object.Delete( pfx_name );

  end;

  m_object.Enable( pfx_name, true );

  PartSysStart( pfx_name );
  
  local pos = int.Position( "anm_int_panel_stage_top_moves" );
  
  --DbgTrace( pos );

  ObjAnimate( pfx_name, "pos_xy", 0, 0, func_end, 
  {
    0.0, 0, pos[ 1 ],      pos[ 2 ],
    0.5, 0, params.chip_x, params.chip_y
  } );

  ObjAnimate( pfx_name, "alp", 0, 0, "", 
  {
    0.00, 1, 1,
    0.45, 1, 1,
    0.50, 1, 0,
  } );
  
  local sound_id = ( 7 - params.moves % 7 );
    
  cmn.PlaySnd( "part_"..sound_id );

end;
--------------------------------------------------------------------------------
function private.PopupCombo_Show ( chain_data )

  local stage = m_stage.GetCurrentStageData();

  local combo =
  {
    [ "ride" ] =
    {
      [ 3 ] = 1,
      [ 5 ] = 2,
      [ 7 ] = 3,
      [ 9 ] = 4
    },
    [ "bonus" ] =
    {
      [ 3 ] = 1,
      [ 5 ] = 2,
      [ 7 ] = 3,
      [ 9 ] = 4
    },
    [ "fight" ] =
    {
      [ 2 ] = 1,
      [ 3 ] = 2,
      [ 5 ] = 3
    }
  };

  if ( combo[ stage.type ][ chain_data.count ] ) then
    
    local x, y = v_match3.GetFieldCenter();

    int_popup_stage_combo.Show( x, y, combo[ stage.type ][ chain_data.count ] );

    if ( stage.type == "ride" ) or ( stage.type == "bonus" ) then

      int_panel_stage_progress_ride.Update_ComboSpeed(
        combo[ stage.type ][ chain_data.count ]
      );

    elseif ( stage.type == "fight" ) then
      
      int_panel_stage_progress_fight.Update_ComboHit(
        combo[ stage.type ][ chain_data.count ]
      );
      
    end;

  end;

  cmn.PlaySnd( "combo_sound"..math.min( chain_data.count, 12 ) );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Soundtrack_Start ( stage_data )

  private.soundtrack = m_stage.GetCurrentStageData().type;

  cmn.PlayMusic( private.soundtrack, 1 );

end;
--------------------------------------------------------------------------------
function public.Soundtrack_Name ()

  return private.soundtrack;

end;
--------------------------------------------------------------------------------
function public.Soundtrack_Stop ()

  cmn.StopMusic( private.soundtrack );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Number_Absolute ( stage_name )

  local stage_number = string.match( stage_name, "%d+" ) or nil;

  return stage_number;

end;
--------------------------------------------------------------------------------
function private.Number_Relative ( section_name, stage_name )

  local section_number  = tonumber( string.match( section_name, "%d+" ) );
  local relative_number = tonumber( string.match( stage_name, "%d+" ) ) - 110 * ( section_number - 1 );

  return relative_number;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Pending_Start ( tutorial_name )
  
  --DbgTrace( "Tutorial started: "..tutorial_name );
  
  int_panel_stage_settings.SetInput( false );
  
end;
--------------------------------------------------------------------------------
function public.Pending_Complete ( tutorial_name )
  
  --DbgTrace( "Tutorial complete: "..tutorial_name );
  
  int_panel_stage_settings.SetInput( true );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.GetCurrentStageType ()

  return m_stage.GetCurrentStageData().type;

end;
--------------------------------------------------------------------------------
function private.GetDialogBackgroundColor ( zone, stage_type, enemy_name )

  local colors = 
  {
    default = { 
      ride  = { r = 0.5484, g = 0.6617, b = 0.1562 }, 
      fight = { r = 0.5484, g = 0.6617, b = 0.1562 }
    },
    forest = { 
      ride  = { r = 0.1058, g = 0.6666, b = 0.4784 }, 
      fight = { r = 0.4196, g = 0.4823, b = 0.0941 }
    },
    hills = { 
      ride  = { r = 0.2666, g = 0.4823, b = 0.4000 }, 
      fight = { r = 0.3254, g = 0.4235, b = 0.0941 }
    },
    bay = { 
      ride  = { r = 0.1686, g = 0.6000, b = 0.6588 },
      fight = { r = 0.2941, g = 0.5176, b = 0.0745 }
    },
    mountains = { 
      ride  = { r = 0.1921, g = 0.5294, b = 0.9647 },
      fight = { r = 0.7921, g = 0.8294, b = 1.0000 }
    },
    wasteland = { 
      ride      = { r = 0.4058, g = 0.4843, b = 0.5470 },
      fight     = { r = 0.5484, g = 0.6617, b = 0.1562 },
      reddragon = { r = 0.2851, g = 0.0937, b = 0.3476 }
    },
    town = { 
      ride  = { r = 0.6901, g = 0.3372, b = 0.4862 },
      fight = { r = 0.5137, g = 0.2980, b = 0.2352 }
    },
    cemetery = { 
      ride  = { r = 0.3647, g = 0.2196, b = 0.5686 },
      fight = { r = 0.2392, g = 0.1764, b = 0.4274 }
    },
    bridge = { 
      ride  = { r = 0.1803, g = 0.6156, b = 0.6705 },
      fight = { r = 0.1803, g = 0.6156, b = 0.6705 }
    },
    fair = { 
      ride  = { r = 0.2392, g = 0.6039, b = 0.5725 },
      fight = { r = 0.8980, g = 0.4627, b = 0.1137 }
    },
    livingtrees = { 
      ride    = { r = 0.8549, g = 0.5372, b = 0.1450 }, 
      fight   = { r = 0.7254, g = 0.3960, b = 0.1215 }, 
      banshee = { r = 0.0000, g = 0.4941, b = 0.4156 }
    },
    greenland = { 
      ride  = { r = 0.7215, g = 0.9058, b = 0.2941 }, 
      fight = { r = 0.3607, g = 0.5686, b = 0.1254 }
    },
    library = { 
      ride  = { r = 0.5960, g = 0.4784, b = 0.5176 }, 
      fight = { r = 0.5960, g = 0.4784, b = 0.5176 }
    },
    kitchen = { 
      ride  = { r = 0.8862, g = 0.7019, b = 0.3490 },
      fight = { r = 0.8862, g = 0.7019, b = 0.3490 }
    },
    garden = { 
      ride  = { r = 0.4941, g = 0.7372, b = 0.3450 },
      fight = { r = 0.3058, g = 0.6039, b = 0.1960 }
    },
    throneroom = { 
      ride  = { r = 0.7490, g = 0.9372, b = 0.8745 },
      fight = { r = 0.5960, g = 0.4784, b = 0.5176 }
    },
    tropic = { 
      ride  = { r = 0.7215, g = 0.9058, b = 0.2941 }, 
      fight = { r = 0.7215, g = 0.9058, b = 0.2941 }
    },
    aboriginal = { 
      ride  = { r = 0.2666, g = 0.4823, b = 0.4000 }, 
      fight = { r = 0.8862, g = 0.7019, b = 0.3490 }
    },
    vulcanic = { 
      ride  = { r = 0.8862, g = 0.7019, b = 0.3490 },
      fight = { r = 0.8862, g = 0.7019, b = 0.3490 }
    },
    beach = { 
      ride  = { r = 0.1686, g = 0.6000, b = 0.6588 },
     fight  = { r = 0.8862, g = 0.7019, b = 0.3490 }
    },
    atlantis = { 
      ride  = { r = 0.1686, g = 0.6000, b = 0.6588 },
      fight = { r = 0.1686, g = 0.6000, b = 0.6588 }
    },
    waterfalls = { 
      ride  = { r = 0.9647, g = 0.7411, b = 0.2196 },
      fight = { r = 0.9647, g = 0.7725, b = 0.2274 }
    },
    kaleyard = { 
      ride  = { r = 0.9803, g = 0.6196, b = 0.0588 },
      fight = { r = 0.9098, g = 0.7568, b = 0.1490 }
    },
    dungeon = { 
      ride  = { r = 0.0000, g = 0.1921, b = 0.3607 },
      fight = { r = 0.2705, g = 0.4117, b = 0.8588 }
    },
    swamp = { 
      ride  = { r = 0.0627, g = 0.4784, b = 0.4705 },
      fight = { r = 0.1372, g = 0.5647, b = 0.2549 }
    },
    oldtreeland = { 
      ride  = { r = 0.7411, g = 0.7411, b = 0.0666 },
      fight = { r = 0.6588, g = 0.6431, b = 0.1607 }
    },
    wintersweets = { 
      ride  = { r = 0.8352, g = 1, b = 1 },
      fight = { r = 0.4745, g = 0.7450, b = 1 }
    },
    icelake = { 
      ride  = { r = 0.0509, g = 0.1843, b = 0.7058 },
      fight = { r = 0.4862, g = 0.5764, b = 0.7098 }
    }
  
  };

  if not ( colors[ zone ] ) then

    zone = "default";

  end;

  local color = colors[ zone ][ stage_type ];

  if ( enemy_name ) then

    color = colors[ zone ][ enemy_name ];

  end;

  if not ( colors[ zone ][ enemy_name ] ) then

    color = colors[ zone ][ stage_type ];

  end;

  return color;

end;
--------------------------------------------------------------------------------
function private.GetStageBackgroundColor ()

  return private.color;

end;
--------------------------------------------------------------------------------
function private.SetStageBackgroundColor ( zone, stage_type, enemy_name )

  local colors = 
  {
    default     = { 
      ride  = { r = 0.6484, g = 0.7617, b = 0.1562, decoration = 1, angle = {  0,  0 } },
      fight = { r = 0.6484, g = 0.7617, b = 0.1562, decoration = 1, angle = {  0,  0 } }
    },
    arena       = {
      ride  = { r = 0.7686, g = 0.5882, b = 0.2392, decoration = 4, angle = { 10, 30 } },
      fight = { r = 0.7686, g = 0.5882, b = 0.2392, decoration = 4, angle = { 10, 30 } },
    },
    forest      = {
      ride  = { r = 0.2275, g = 0.5098, b = 0.1098, decoration = 1, angle = {  0,  0 } },
      fight = { r = 0.4157, g = 0.4745, b = 0.0706, decoration = 1, angle = {  0,  0 } }
    },
    hills       = {
      ride  = { r = 0.3373, g = 0.6235, b = 0.1255, decoration = 1, angle = {  0,  0 } },
      fight = { r = 0.2588, g = 0.3686, b = 0.0353, decoration = 1, angle = {  0,  0 } }
    },
    bay         = {
      ride  = { r = 0.9568, g = 0.8313, b = 0.2941, decoration = 6, angle = {  0,  0 } },
      fight = { r = 0.4431, g = 0.6314, b = 0.1569, decoration = 5, angle = {  0,  0 } }
    }, 
    mountains   = {
      ride  = { r = 0.5569, g = 0.8549, b = 0.9255, decoration = 3, angle = {  0,  0 } },
      fight = { r = 0.8510, g = 0.8706, b = 0.8863, decoration = 3, angle = {  0,  0 } }
    },
    wasteland   = {
      ride  = { r = 0.8235, g = 0.6157, b = 0.2196, decoration = 4, angle = { 10, 30 } },
      fight = { r = 0.4275, g = 0.4745, b = 0.0745, decoration = 4, angle = { 10, 30 } },
      reddragon = { r = 0.2851, g = 0.0937, b = 0.3476, decoration = 4, angle = { 10, 30 } }
    },
    town        = {
      ride  = { r = 0.6980, g = 0.4431, b = 0.4706, decoration = 4, angle = { 10, 30 } },
      fight = { r = 0.6510, g = 0.4471, b = 0.3961, decoration = 4, angle = { 10, 30 } }
    },
    cemetery    = {
      ride  = { r = 0.5373, g = 0.3020, b = 0.5412, decoration = 4, angle = { 10, 30 } },
      fight = { r = 0.5020, g = 0.2588, b = 0.5255, decoration = 4, angle = { 10, 30 } }
    },
    bridge      = {
      ride  = { r = 0.1647, g = 0.3725, b = 0.5647, decoration = 2, angle = {  0,  0 } },
      fight = { r = 0.4196, g = 0.5765, b = 0.5922, decoration = 4, angle = { 10, 30 } }
    },
    fair        = {
      ride  = { r = 0.9255, g = 0.5843, b = 0.2196, decoration = 4, angle = { 10, 30 } },
      fight = { r = 0.9333, g = 0.6275, b = 0.3255, decoration = 4, angle = { 10, 30 } }
    },
    livingtrees = {
      ride  = { r = 0.8902, g = 0.5569, b = 0.2510, decoration = 4, angle = { 10, 30 } },
      fight = { r = 0.8980, g = 0.5294, b = 0.2471, decoration = 4, angle = { 10, 30 } }
    },
    greenland   = {
      ride  = { r = 0.6431, g = 0.8588, b = 0.1961, decoration = 1, angle = {   0, 0 } },
      fight = { r = 0.2863, g = 0.4118, b = 0.0784, decoration = 4, angle = { 10, 30 } }
    },
    library     = {
      ride  = { r = 0.5294, g = 0.2941, b = 0.0549, decoration = 5, angle = {   0, 0 } },
      fight = { r = 0.6784, g = 0.4275, b = 0.4039, decoration = 4, angle = { 10, 30 } }
    },
    kitchen     = {
      ride  = { r = 1,0000, g = 0.8588, b = 0.1843, decoration = 6, angle = { 10, 30 } },
      fight = { r = 0.9608, g = 0.7843, b = 0.4510, decoration = 4, angle = { 10, 30 } }
    },
    garden      = {
      ride  = { r = 0.4196, g = 0.8118, b = 0.4627, decoration = 1, angle = {   0, 0 } },
      fight = { r = 0.3529, g = 0.6314, b = 0.2510, decoration = 4, angle = { 10, 30 } }
    },
    throneroom  = {
      ride  = { r = 0.3882, g = 0.5490, b = 0.6980, decoration = 5, angle = {   0, 0 } },
      fight = { r = 0.3725, g = 0.4118, b = 0.6039, decoration = 4, angle = { 10, 30 } }
    },
    tropic      = {
      ride  = { r = 0.5608, g = 0.8078, b = 0.1843, decoration = 1, angle = {   0, 0 } },
      fight = { r = 0.4980, g = 0.8118, b = 0.3647, decoration = 4, angle = { 10, 30 } }
    },
    aboriginal  = {
      ride  = { r = 0.7451, g = 0.7608, b = 0.1686, decoration = 1, angle = {   0, 0 } },
      fight = { r = 0.8431, g = 0.6000, b = 0.2588, decoration = 4, angle = { 10, 30 } }
    },
    vulcanic    = {
      ride  = { r = 0.9451, g = 0.6510, b = 0.3451, decoration = 5, angle = {   0, 0 } },
      fight = { r = 0.9647, g = 0.7725, b = 0.4824, decoration = 4, angle = { 10, 30 } }
    },
    beach       = {
      ride  = { r = 0.9647, g = 0.8314, b = 0.2471, decoration = 2, angle = {   0, 0 } },
      fight = { r = 0.9529, g = 0.7843, b = 0.5137, decoration = 4, angle = { 10, 30 } }
    },
    atlantis    = {
      ride  = { r = 0.1020, g = 0.6314, b = 0.8588, decoration = 2, angle = {   0, 0 } },
      fight = { r = 0.1765, g = 0.6431, b = 0.7333, decoration = 4, angle = { 10, 30 } }
    },
    waterfalls  = {
      ride  = { r = 0.9686, g = 0.7803, b = 0.1333, decoration = 6, angle = {   0, 0 } },
      fight = { r = 0.9647, g = 0.7725, b = 0.2274, decoration = 5, angle = { 10, 30 } }
    },
    kaleyard = {
      ride  = { r = 1.0000, g = 0.7607, b = 0.0509, decoration = 1, angle = {   0, 0 } },
      fight = { r = 0.9098, g = 0.7568, b = 0.1490, decoration = 4, angle = { 10, 30 } }
    },
    dungeon = {
      ride  = { r = 0.0274, g = 0.4705, b = 0.9568, decoration = 5, angle = { 10, 30 } },
      fight = { r = 0.2705, g = 0.4117, b = 0.8588, decoration = 4, angle = { 10, 30 } }
    },
    swamp = {
      ride  = { r = 0.3294, g = 0.9137, b = 0.6705, decoration = 5, angle = { 10, 30 } },
      fight = { r = 0.1372, g = 0.5647, b = 0.2549, decoration = 4, angle = { 10, 30 } }
    },
    oldtreeland = {
      ride  = { r = 0.8117, g = 0.8666, b = 0.2431, decoration = 1, angle = {   0, 0 } },
      fight = { r = 0.6588, g = 0.6431, b = 0.1607, decoration = 4, angle = { 10, 30 } }
    },
    wintersweets = {
      ride  = { r = 0.3490, g = 0.7176, b = 1, decoration = 5, angle = { 10, 30 } },
      fight = { r = 0.3529, g = 0.6666, b = 1, decoration = 4, angle = { 10, 30 } }
    },
    icelake = {
      ride  = { r = 0.3215, g = 0.8627, b = 0.9686, decoration = 1, angle = {   0, 0 } },
      fight = { r = 0.8235, g = 0.8470, b = 0.8862, decoration = 4, angle = { 10, 30 } }
    }
  
  };

  if not ( colors[ zone ] ) then

    zone = "default";

  end;

  private.color = tern( enemy_name and colors[ zone ][ enemy_name ] )( colors[ zone ][ enemy_name ], colors[ zone ][ stage_type ] );
  
  local multiset_table = {};

  -- Setup stage BG grid
  table.insert( multiset_table, 
  {
    "grd_stage_background", { color_r = private.color.r, 
                              color_g = private.color.g, 
                              color_b = private.color.b }
  } );
  
  local h, w = GetAppHeight(), GetAppWidth();
  
  local grd_size = {
    top    = 0,
    left   = 0,
    bottom = h,
    right  = w
  };
  
  local grd_border = {
    top    = 0,
    left   = 0,
    bottom = 0,
    right  = 0
  };
  
  int.Grid_Resize( "grd_stage_background", grd_size, grd_border );
  
  -- Setup background objects
  local deco_h = 85;
  local deco_w = 450;
  
  local offset = private.bg_offset or 0;
  
  local rows = math.floor( ( h - offset ) / deco_h );
  local cols = math.ceil( w / deco_w );
  
  for row = 1, rows  do
    
    local cols_here = cols;
    
    if ( ( row % 2 ) == 0 ) then
      
      cols_here = cols - 1;
      
    end;
    
    for col = 1, cols_here do
      
      local deco_name = "anm_stage_background_decoration_"..col.."_"..row;
      
      if not ( ObjGet( deco_name ) ) then

        ObjCreate( deco_name, "anim", "obj_stage_background_decoration" );

      end;
      
      local x_shift = common.Round( 0.5 * ( w - cols_here * deco_w ) );
      local y_shift = common.Round( offset - 0.15 * deco_h );
      
      local x_rand = common.Round( 0.015 * deco_w * math.random( -10, 10 ) );
      local y_rand = common.Round( 0.020 * deco_h * math.random( -10, 10 ) );
      
      if ( row == 1 ) then 
        
        y_rand = common.Round( 0.020 * deco_h * math.random( 0, 10 ) );
      
      end;
      
      local x = common.Round( x_shift + ( ( col - 0.5 ) * deco_w ) + x_rand );
      local y = common.Round( y_shift + ( ( row - 0.5 ) * deco_h ) + y_rand);
      
      local scale_pm = ( - 1 ) ^ math.random( 1, 2 );
      local scale    = math.random( 6, 10 ) / 10;
      
      local ang_pm = ( - 1 ) ^ math.random( 1, 2 );
      
      table.insert( multiset_table, 
      {
        deco_name, 
        { 
          res      = "assets/content/interface/tex_stage_a",
          animfunc = "bg_decoration_"..private.color.decoration,
          playing  = true,
          input    = false,
          pos_x    = x,
          pos_y    = y,
          scale_x  = scale_pm * scale,
          scale_y  = scale,
          ang      = ang_pm * 0.01 * math.random( private.color.angle[ 1 ], private.color.angle[ 2 ] ) or 0,
          color_r  = ( math.abs( private.color.r - 0.01 * math.random( 10, 15 ) ) ),
          color_g  = ( math.abs( private.color.g - 0.01 * math.random( 10, 15 ) ) ),
          color_b  = ( math.abs( private.color.b - 0.01 * math.random( 10, 15 ) ) )
        }
      } );
      
    end;
    
  end;

  table.insert( multiset_table, 
  {
    "obj_stage_background_decoration", { pos_x = 0, pos_y = 0 }
  } );

  ObjMultiSet( multiset_table );

end;
--------------------------------------------------------------------------------
function private.EffectIndex_Get ()

  private.effect_index = ( private.effect_index or 0 ) + 1;

  return private.effect_index;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Cheater_GetStartData (
    stage_name,
    database,
    dailyquest,
    stage_state
  )

  return private.GetStartData( stage_name, nil, nil, stage_state );

end;
--------------------------------------------------------------------------------
function public.Cheater_DialogStart_Hide ( func_end, skip_map_enable )

  return private.DialogStart_Hide( func_end, skip_map_enable );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.List_HasReward ( reward_type, reward_name_pattern )

  local section_name = v_map.Section_Current();

  local stage_list = m_stage.List_HasReward(
    section_name,
    reward_type,
    reward_name_pattern
  );

  local quest_stages = v_quests.StageList_HasReward(
    section_name,
    reward_type,
    reward_name_pattern
  );

  for s, stage_name in pairs( quest_stages ) do

    table.insert( stage_list, stage_name );

  end;

  return stage_list;

end;
--------------------------------------------------------------------------------