-- name=view_game
--------------------------------------------------------------------------------
function public.Init ()

  if ( IsIOS() ) then
  
    if ( m_client.IsOnline() ) then
    
      model_game.RequestIsCCSAvailable();
    
    else
    
      mediator.SubscribeOnce(
        "Model_Client_ConnectionRestored",
        model_game.RequestIsCCSAvailable
      );
    
    end;
  
  end;

end;
--------------------------------------------------------------------------------
function public.Start ()

  --ModLoad( "assets/content/scripts/mod_test" );
  --test.Init();

  private.Subscribe();

  local cheater_enabled = wiz.IsCheatsEnabled();

  if ( cheater_enabled ) then

    ModLoad( "assets/cheater/mod_cheater" );
    cheater.Init();

  end;

  private.TryStartTutorial();

  screen.Switch( "scr_map" );

end;
--------------------------------------------------------------------------------
function private.Subscribe ()

  if ( IsAndroid() ) then

    common.MessageSubscribe(
      "Event_Android_BackPressed",
      private.OnAndroidBackButtonPressed
    );

  end;
  
  if ( IsIOS() ) then
    
    mediator.Subscribe(
      "Interface_RightPanel_Visible",
      private.CCS_ShowButton
    );
    
    mediator.Subscribe(
      "Interface_RightPanel_Invisible",
      private.CCS_HideButton
    );
    
    mediator.Subscribe(
      "Interface_RightPanel_Updated",
      private.CCS_UpdateButton
    );
    
  end;
  
  if ( IsAndroid() and ( wiz.Android.GetPlatform() == "google" ) ) then
  
    local is_connected = wiz.Android.GooglePlayIsConnected();
    
    if ( not is_connected ) then
    
      int_panel_map_right.Google_Enable( private.GooglePlayTap );
      
    end;
    
    int_dialog_options.GooglePlay_Setup( is_connected, private.GooglePlayTap );
    
    ne.msg.Subscribe(
      "Event_GooglePlay_LoggedStateChanged",
      private.OnGooglePlayLoggedStateChanged
    );
  
  end;

end;
--------------------------------------------------------------------------------
function private.OnAndroidBackButtonPressed ()

  -- JK: On screen locked - ignore
  if ( int.IsLocked() ) then

    --private.Minimize();
    return;

  end;
  
  if ( int_dialog_info.IsActive() ) then
  
    local func_close = int_dialog_info.GetCloseFunc();
    local func_ok    = int_dialog_info.GetOkFunc();
    
    if ( func_close ) then
      func_close();
    elseif ( func_ok ) then
      func_ok();
    else
      private.Minimize();
    end;
    return;
  
  end;

  local function func_unsubscribe ()

    common.MessageUnsubscribe(
      "Event_Android_BackPressed",
      private.OnAndroidBackButtonPressed
    );

  end;

  local function func_subscribe ()

    common.MessageSubscribe(
      "Event_Android_BackPressed",
      private.OnAndroidBackButtonPressed
    );

  end;

  if ( not int_window.IsAnyWindowOnScreen() ) then

    if ( screen.GetCurrentName() == "scr_stage" ) then

      -- JK: On stage screen if stage is complete do nothing
      if ( v_stage.IsStageComplete() ) then
        
        return;
        
      end;
      
      -- JK: On stage screen with no windows
      -- set pause on and show exit dialog
      
      local pause_button = int.Control_EventsGet(
        "btn_int_panel_stage_settings_panel"
      );
      
      if ( not int_panel_stage_settings.IsOpened() ) then
      
        if ( pause_button.func_on_click ) then
          pause_button.func_on_click();  
        end;
      
      end;

      local exit_button = int.Control_EventsGet(
        "btn_int_panel_stage_settings_exit"
      );
      
      if ( not int_dialog_stage_exit.IsActive() ) then
      
        if ( exit_button.func_on_click ) then
          exit_button.func_on_click();  
        end;
      
      end;

      return;

    end;

    -- JK: On map screen with no windows - try to quit
    if ( screen.GetCurrentName() == "scr_map" ) then
    
      if ( v_map.IsOpened() ) then
      
        if ( int_panel_questlist.IsActive() ) then
          int_panel_questlist.Hide();
          
          -- HACK: int_panel_questlist behavior so strange
          int_panel_darkhole.Hide();
        else
          private.TryQuit();
        end;
        return;
      
      end;
    
    end;
    
    return;

  end;

  local closed, window_name = int_window.TryCloseUpperWindow();

  if ( closed ) then

    func_unsubscribe();

    mediator.SubscribeOnce(
      "Interface_Window_WindowClosedEnd",
      function ()

        func_subscribe();

      end
    );

    return;

  end;

  -- JK: Default action is ignore
  --private.TryQuit();

end;
--------------------------------------------------------------------------------
function private.TryQuit ( func_unsubscribe )

  int_dialog_info.Show(
    "exitconfirm",
    function ()

      ne.msg.Unsubscribe(
        "Event_Android_BackPressed",
        private.OnAndroidBackButtonPressed
      );
      int_dialog_info.Hide(
        function ()
          private.Minimize();
          ne.msg.Subscribe(
            "Event_Application_GainFocus",
            private.OnFocusGainedAfterMinimize
          );
        end
      );

    end,
    function ()

      int_dialog_info.Hide();

    end
  );

end;
--------------------------------------------------------------------------------
function private.Minimize ()

  wiz.app.Minimize();

end;
--------------------------------------------------------------------------------
function private.OnFocusGainedAfterMinimize ( message, params )

  ne.msg.Unsubscribe( message, private.OnFocusGainedAfterMinimize );
  ne.msg.Subscribe(
    "Event_Android_BackPressed",
    private.OnAndroidBackButtonPressed
  );

end;
--------------------------------------------------------------------------------
function private.TryStartTutorial ()

  if ( m_storage.Settings( "tutorial_enable" ) ) then 

    v_tutorial.Start();

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.CCS_ShowButton ()

  if ( not model_game.IsCCSAvailable() ) then return; end;
  
  local screen = {
    width  = int.W(),
    height = int.H()
  };
  
  local is_sale_enabled = int_panel_map_right.Button_IsEnabled( "sale" );
  
  local target_name = tern( is_sale_enabled )(
    "btn_int_panel_map_ccs",
    "dmy_int_panel_map_sale"
  );
  
  local target_pos = int.Position( target_name );
  
  local button = {
    width  = 100,
    height = 100,
    pos_x  = target_pos[ 1 ],
    pos_y  = target_pos[ 2 ]
  };
  
  if ( IsEditor() ) then
    
    ObjMultiSet( {
      {
        "obj_loader_ccs_placeholder",
        {
          active  = true,
          visible = true,
          pos_x   = button.pos_x,
          pos_y   = button.pos_y
        }
      },
      { "spr_loader_ccs_placeholder",
        {
          scale_x = button.width  / 2,
          scale_y = button.height / 2
        }
      }
    } );
    
    ObjAttach( "obj_loader_ccs_placeholder", InterfaceName );
    
  end;
  
  local def_params = { scale_x = 1, scale_y = 1 };
  local app_params = ne.obj.Get( "ng_application" ) or def_params;

  wiz.bfg.ccs.ShowButton(
    {
      x = ( button.pos_x - button.width  / 2 ) * app_params.scale_x,
      y = ( button.pos_y - button.height / 2 ) * app_params.scale_y,
      w = button.width  * app_params.scale_x,
      h = button.height * app_params.scale_y
    }
  );
  
end;
--------------------------------------------------------------------------------
function private.CCS_HideButton ()
  
  if ( IsEditor() ) then
    
    ObjSet(
      "obj_loader_ccs_placeholder",
      {
        active  = false,
        visible = false
      }
    );
    
    ObjAttach( "obj_loader_ccs_placeholder", "scr_loader" );
    
  end;
  
  wiz.bfg.ccs.HideButton();
  
end;
--------------------------------------------------------------------------------
function private.CCS_UpdateButton ( params )

  if ( not params.visible ) then return; end;
  
  private.CCS_HideButton();
  private.CCS_ShowButton();
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.GooglePlayTap ()

  if ( wiz.Android.GooglePlayIsConnected() ) then
  
    wiz.Android.GooglePlaySignOut();
    SetGooglePlayLogged( false );
  
  else
  
    mediator.Notify( "View_Game_GooglePlayLoginStart" );

    wiz.Android.GooglePlaySignIn();
  
  end;

end;
--------------------------------------------------------------------------------
function private.OnGooglePlayLoggedStateChanged ( msg, params )
  
  local is_connected = params.logged;
  
  if ( is_connected ) then
  
    int_panel_map_right.Google_Disable();
    
    mediator.Notify( "View_Game_GooglePlayLoggedIn" );
  
  else
    
    int_panel_map_right.Google_Enable( private.GooglePlayTap );
    
    mediator.Notify( "View_Game_GooglePlayLoggedOut" );
    
  end;
  
  int_dialog_options.GooglePlay_Setup( is_connected, private.GooglePlayTap );

end;
--------------------------------------------------------------------------------