-- name=loader
--------------------------------------------------------------------------------
function private.Trace ( text )

  local allow_trace = false;

  if ( allow_trace ) then

    DbgTrace( "[ LOADER ]: "..text );

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function public.Start ()

  math.randomseed( os.time() );
  
  private.app_url =
  {
    ios    = "https://itunes.apple.com/app/id905121703",
    google = "https://play.google.com/store/apps/details?id=com.bigfishgames.sirmatchgoogle",
    amazon = "http://www.amazon.com/gp/mas/dl/android?p=com.bigfishgames.sirmatchamazon"
  };

  local platform = GetPlatform();

  public.simulate_ios     = false;
  public.simulate_android = false;

  if ( public.simulate_ios     ) then platform = "ios";     end;
  if ( public.simulate_android ) then platform = "android"; end;
  
  if ( public.simulate_ios     ) or
     ( public.simulate_android ) then
  
    private.SetupSimulation( platform );
     
  end;

  private.is_pc      = ( platform == "pc" );
  private.is_android = ( platform == "android" );
  private.is_ios     = ( platform == "ios" );

  private.free_space_needed = 200;
  if ( private.is_pc ) then private.free_space_needed = 1000; end;

  private.is_online = true;
  private.gonext = false;
  private.request_list = {};

  if ( private.is_android ) then
    ne.msg.Subscribe( "Event_Purchase_ProductInformationReady", function ()
      private.product_info_ready = true;
    end );
  end;

  private.LoadStandardCursors();
  
  private.SystemMessages_Init( platform );
  
  private.PartsLoad();

  private.SetForm();

  ObjMultiSet({
  { "spr_loader_logo_publisher",
    {
      input     = false,
      event_mup = private.HideLogoPublisher,
      alp       = 0.0
    }
  },
  { "spr_loader_logo_developer",
    {
      input     = false,
      event_mup = private.HideLogoDeveloper,
      alp       = 0.0
    }
  },
  { "spr_loader_logo_game",
    {
      input = false,
      alp   = 0.0
    }
  }
  });

  ObjAttach( "scr_loader", "ng_application" );

  if ( not wiz.HasFreeDiskspace() ) then

    local func_continue = function ()
        ne.msg.Send( Event_RestartEngine );
    end;
    private.SystemDialog_Show( "nofreespace", func_continue );

  end;

  if ( not parts_app_wasrestarted ) then

    private.ShowLogoPublisher();

  else

    private.ShowLogoGame();

  end;
  
  if ( IsEditor() ) then
  
    _PRIVATE = {};
    _PRIVATE[ "loader" ] = _G[ "loader" ].__Private();
  
  end;
  
  if ( private.is_android and ( wiz.Android.GetPlatform() == "google" ) ) then

    public.Module_Load( "int_dialog_game_pause" );
    int_dialog_game_pause.CreateHub( "ng_pause", "ne_origin" );
    int_dialog_game_pause.SetHandlers(
      private.Pause,
      private.Resume
    );
    
    -- JK: delayed, see private.Splash_Keep()
    --ne.msg.Subscribe( "Event_Android_OnPause", private.OnPause );

  end;
  
  if ( private.is_ios ) then
  
    wiz.bfg.ccs.HideButton();
  
  end;

end;
--------------------------------------------------------------------------------
function public.GetUserId ()
  
  local user_id = AuthGetUserId();
  
  if ( user_id ~= "" ) then
    
    return user_id;
    
  end;

  return "0";
  
end;
--------------------------------------------------------------------------------
function public.IsOnline ()

  return private.is_online;
  
end;
--------------------------------------------------------------------------------
function public.GetSystemMessageType ()

  return private.sysmsg_type;
  
end;
--------------------------------------------------------------------------------
function public.IsProductInfoReady ()

  return private.product_info_ready;

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.LoadStandardCursors ()

  SetSysCursor( GetSysCursor() );
 
  local count = 0;

  local CursorLoad = function ( cur_name, cur_path, cur_idx )

    _G[ cur_name ] = count;
    
    CursorLoad( cur_path, cur_idx );
    
    count = count + 1;
  
  end;

  CursorLoad( "CURSOR_NULL",    "", 0 );
  CursorLoad( "CURSOR_DEFAULT", "assets/cursors/cursor_default_a", 2 );
  CursorLoad( "CURSOR_HAND",    "assets/cursors/cursor_hand_a",    4 );

  SetCursor( CURSOR_DEFAULT );

  --[[CursorLoad( "CURSOR_NULL",    "",                              0 );
  CursorLoad( "CURSOR_DEFAULT", "assets/cursors/cursor",         2 );
  CursorLoad( "CURSOR_LOUPE",   "assets/cursors/cursor_loupe",  10 );
  CursorLoad( "CURSOR_GET",     "assets/cursors/cursor_get",     4 );
  CursorLoad( "CURSOR_HAND",    "assets/cursors/cursor_hand",    4 );
  CursorLoad( "CURSOR_USE",     "assets/cursors/cursor_use",     4 );
  CursorLoad( "CURSOR_DOWN",    "assets/cursors/cursor_down",   10 );
  CursorLoad( "CURSOR_UP",      "assets/cursors/cursor_up",     10 );
  CursorLoad( "CURSOR_LEFT",    "assets/cursors/cursor_left",   10 );
  CursorLoad( "CURSOR_RIGHT",   "assets/cursors/cursor_right",  10 );
  CursorLoad( "CURSOR_FINGER",  "assets/cursors/cursor_finger", 10 );]]

end;
--------------------------------------------------------------------------------
function private.PartsLoad ()

  private.message_handlers = {};

  private.PartsWaitForResult( Event_Application_GameVersionOutdated,   private.PartsGameVersionOutdated );
  private.PartsWaitForResult( Event_Application_StateCorrupted,        private.PartsStateCorrupted        );
  private.PartsWaitForResult( Event_Application_ClientVersionOutdated, private.PartsClientVersionOutdated );
  private.PartsWaitForResult( Event_Application_InternetError,         private.PartsInternetError   );
  private.PartsWaitForResult( Event_Application_LoadingComplete,       private.PartsLoadingComplete );
  private.PartsWaitForResult( Event_Storage_Conflict,                  private.PartsStorageConflict );
  private.PartsWaitForResult( Event_Application_RunMigrations,         private.PartsRunMigrations );
  
  ne.msg.Subscribe( Event_Application_GainFocus, private.ApplicationGainFocus );

  ne.msg.Send( Event_Menu_LoadingComplete );

  private.IsNewClientVersionAvailable();

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.SystemMessages_Init ( platform )

  local system_message_type = {
    ios     = "native",
    pc      = "ingame",
    android = "ingame" 
  };

  private.sysmsg_type    = system_message_type[ platform ];
  private.messages_queue = {};

end;
--------------------------------------------------------------------------------
function private.SystemMessages_Add ( message_type, message_handler )

  for i = #private.messages_queue, 1, - 1 do

    if ( private.messages_queue[ i ].type == message_type ) then

      return;

    end;

  end;   

  table.insert( private.messages_queue, { type = message_type, handler = message_handler } );

end;
--------------------------------------------------------------------------------
function private.SystemMessages_ShowNext ()

  if ( #private.messages_queue > 0 ) then

    local message = table.remove( private.messages_queue, #private.messages_queue );

    if ( message.handler ) then

      message.handler();

    end;

  end;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PartsWaitForResult ( message_type, message_handler )

  if ( private.message_handlers[ message_type ] ) then

    private.Trace( "Error: handler for < "..message_type.." > already exist!" );

    return;

  end;

  local func_handler = function ( msg_type, msg_params )

    ne.msg.Unsubscribe( message_type, private.message_handlers[ message_type ] );

    if ( message_handler ) then

      message_handler( msg_params );

    end;

  end;

  private.message_handlers[ message_type ] = func_handler;

  ne.msg.Subscribe( message_type, func_handler );

end;
--------------------------------------------------------------------------------
function private.PartsSetStateHandler ( state, handler )

  --private.Trace( "PartsSetStateHandler: "..tostring( state ).." -> "..tostring( handler ) );

  private.parts_result_state   = state   or nil;
  private.parts_result_handler = handler or function () end;

end;
--------------------------------------------------------------------------------
function private.PartsClearStateHandler ()

  private.parts_result_state   = nil;
  private.parts_result_handler = nil;

end;
--------------------------------------------------------------------------------
function private.PartsUnsubscribeAll ()

  for message_name, message_handler in pairs( private.message_handlers ) do

    ne.msg.Unsubscribe( message_name, message_handler );

  end;
  
  ne.msg.Unsubscribe( Event_Application_GainFocus, private.ApplicationGainFocus );

  private.message_handlers = {};

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.PartsGameVersionOutdated ()

  private.Trace( "PartsGameVersionOutdated" );

  private.SystemMessages_Add( "oldclientversion", private.PartsGameVersionOutdated_Handler );

  private.PartsSetStateHandler( "oldclientversion", private.PartsGameVersionOutdated_Handler );
  
end;
--------------------------------------------------------------------------------
function private.PartsGameVersionOutdated_Handler ()

  private.Trace( "PartsGameVersionOutdated_Handler" );

  local func_continue = function ()
  
    local platform = GetPlatform();
    if ( platform == "android" ) then
    
      platform = wiz.Android.GetPlatform();
    
    end;

    wiz.bfg.OpenUrl( private.app_url[ platform ] );
    private.Loader_MainHandler();

  end;

  private.SystemDialog_Show( "oldclientversion", func_continue );

end;
--------------------------------------------------------------------------------
function private.PartsStateCorrupted ()

  private.Trace( "Event_Application_StateCorrupted" );

  -- JK: don't send corrupted save to cloud
  parts.net.heartbeat.Stop();

  private.SystemMessages_Add(
    "statecorrupted",
    function ()
    
      private.SystemDialog_Show(
        "statecorrupted",
        function ()
          wiz.RemoveLocalSave();
        end
      );

    end
  );

end;
--------------------------------------------------------------------------------
function private.PartsClientVersionOutdated ( params )

  private.Trace( "PartsClientVersionOutdated" );

  private.SystemMessages_Add( "oldclientversion", private.PartsClientVersionOutdated_Handler );

  private.PartsSetStateHandler( "oldclientversion", private.PartsClientVersionOutdated_Handler );

end;
--------------------------------------------------------------------------------
function private.PartsClientVersionOutdated_Handler ()

  private.Trace( "PartsClientVersionOutdated_Handler" );

  local func_continue = function ()

    local platform = GetPlatform();
    if ( platform == "android" ) then
    
      platform = wiz.Android.GetPlatform();
    
    end;

    wiz.bfg.OpenUrl( private.app_url[ platform ] );
    private.Loader_MainHandler();

  end;

  private.SystemDialog_Show( "oldclientversion", func_continue ); 

end;
--------------------------------------------------------------------------------
function private.PartsStorageConflict ( params )

  private.Trace( "PartsStorageConflict: "..public.GetUserId() );

  private.SystemMessages_Add( "conflictstorage", function ()
    private.PartsStorageConflict_Handler( params.clientSaveTime, params.serverSaveTime );
  end );

end;
--------------------------------------------------------------------------------
function private.PartsStorageConflict_Handler ( clientTime, serverTime )

  private.Trace( "PartsStorageConflict_Handler" );
  
  local func_message_main, func_message_local, func_message_server;

  func_message_main = function ()
    
    local clientTimeFormatted = os.date( "%Y-%m-%d", clientTime );
    local serverTimeFormatted = os.date( "%Y-%m-%d", serverTime );
    
    if clientTimeFormatted == serverTimeFormatted then
      clientTimeFormatted = os.date( "%H:%M:%S", clientTime );
      serverTimeFormatted = os.date( "%H:%M:%S", serverTime );
    end;
    
    local clientInfoData = private.PartsStorageConflict_GetClientInfo();
    local serverInfoData = private.PartsStorageConflict_GetServerInfo();

    private.SystemDialog_Show( "conflictstorage", 
      func_message_local, 
      func_message_server,
      { clientTimeFormatted, clientInfoData },
      { serverTimeFormatted, serverInfoData }
    );

  end;

  func_message_local = function ()

    private.SystemDialog_Show(
      "conflictstorage_local",
      function ()

        parts.storage.ResolveConflict( "local" );
        private.Loader_MainHandler();
        --private.SystemMessages_ShowNext();

      end,
      func_message_main
    );

  end;

  func_message_server = function ()

    private.SystemDialog_Show(
      "conflictstorage_server",
      function ()

        parts.storage.ResolveConflict( "server" );
        private.Loader_MainHandler();
        --private.SystemMessages_ShowNext();

      end,
      func_message_main
    );

  end;

  func_message_main();

end;
--------------------------------------------------------------------------------
function private.PartsStorageConflict_GetClientInfo ()
  
  local _, player_info_string = parts.storage.data.Read( "parts/cloudstorage/player/info" );
  local player_info = wiz.json.Decode( player_info_string );
  
  local query;
  
  if not ( player_info ) then
    query = {
      query = "find_all",
      collection = "player/info"
    };
    player_info = DbExecuteQuery( query ).result or {};
  end;
  
  local client_data = {};
  
  client_data.stage_last = private.PartsStorageConflict_GetStage( player_info[ 1 ].stage_last );
  
  local _, sections_string = parts.storage.data.Read( "parts/cloudstorage/player/sections" );
  local sections = wiz.json.Decode( sections_string );

  if not ( sections ) then
    query = {
      query = "find_all",
      collection = "player/sections"
    };
    sections = DbExecuteQuery( query ).result or {};
  end;
 
  client_data.section = private.PartsStorageConflict_GetSection( sections );
  client_data.gold = player_info[ 1 ].gold;
  
  return client_data;
  
end;
--------------------------------------------------------------------------------
function private.PartsStorageConflict_GetServerInfo ()
  
  local conflictData = parts.storage.GetConflictSave();
  
  local player_info = wiz.json.Decode( conflictData[ "player/info" ] );
  
  local server_data = {};
  
  server_data.stage_last = private.PartsStorageConflict_GetStage( player_info[ 1 ].stage_last );

  local sections = wiz.json.Decode( conflictData[ "player/sections" ] );
  
  server_data.section = private.PartsStorageConflict_GetSection( sections );
  server_data.gold = player_info[ 1 ].gold;
  
  return server_data;
  
end;
--------------------------------------------------------------------------------
function private.PartsStorageConflict_GetStage ( stage_name )
  
  local stage_number = tonumber( stage_name:match( "_(%d+)" ) );
  
  if ( stage_name:match( "_bonus" ) ) then
    return "bonus";
  end;
  
  if ( stage_number > 110 ) then
    stage_number = stage_number - 110 * math.modf( stage_number / 110 );
  end;
  
  return stage_number;
  
end;
--------------------------------------------------------------------------------
function private.PartsStorageConflict_GetSection ( sections )
  
  local section = "section_1";
  
  for s = #sections, 1, -1  do
    if ( sections[ s ].enabled ) then
      section = sections[ s ].name;
      break;
    end;
  end;
  
  --section = tonumber( section:match( "_(%d+)" ) );
  
  return section;
  
end;
--------------------------------------------------------------------------------
function private.PartsInternetError ( params )

  private.Trace( "PartsInternetError" );

  private.is_online = false;

  if ( params and params.error == "ServerTemporarilyUnavailable" ) then

    private.Trace( "PartsInternetError: ServerTemporarilyUnavailable" );

  end;

  if ( params and params.error == "KickedByOthers" ) then

    private.Trace( "PartsInternetError: KickedByOthers" );

    private.SystemMessages_Add( "prioritysession", private.PartsInternetError_PrioritySession_Handler );

    private.Loader_MainHandler();

    return;

  end;

  private.PartsWaitForResult( Event_HeartBeat, private.PartsInternetConnectionReturned );

end;
--------------------------------------------------------------------------------
function private.PartsInternetError_PrioritySession_Handler ()

  private.Trace( "PartsInternetError_PrioritySession_Handler" );

  parts.net.heartbeat.Stop();

  local func_continue = function ()

    RestartApplication();

  end;

  private.SystemDialog_Show( "prioritysession", func_continue ); 

end;
--------------------------------------------------------------------------------
function private.PartsLoadingComplete ( force )

  private.Trace( "PartsLoadingComplete" );
  local obsolete = private.Content_IsObsolete();

  if ( obsolete.result ) then
    
    private.Trace( "ContentObsolete" );

    if ( not private.is_online ) then

      private.PartsSetStateHandler( "nointernetconnection", function ()

        private.SystemDialog_Show( "nointernetconnection", function () 

          parts.net.heartbeat.Restart();
          private.Loader_MainHandler();
          
        end );

      end );

      private.PartsWaitForResult( Event_HeartBeat, private.PartsInternetConnectionReturned );

    else

      if ( parts_app_wasrestarted or force ) then

        private.HideLogoGame( function ()

          private.PartsUnsubscribeAll();
          private.Content_StartDownload();

        end );

      else

        private.SystemMessages_Add( "contentobsolete", function ()
          
          private.SystemDialog_Show( "contentobsolete", function()

            private.HideLogoGame( function ()

              private.PartsUnsubscribeAll();
              private.Content_StartDownload();

            end );

          end )

        end );

      end;

    end;

    return;

  elseif ( obsolete.reason ) and ( obsolete.reason == "nofreespace" ) then

    private.SystemMessages_Add( "nofreespace", function ()

      private.SystemDialog_Show( "nofreespace", function()

        private.PartsUnsubscribeAll();
        RestartApplication();

      end )

    end );

    return;
    
  end;

  if ( not private.is_pc ) and
     ( private.newversion ) then

    private.Trace( "NewClientVersionAvailable" );

    private.SystemMessages_Add( "newversion", private.PartsNewVersionAvailable_Handler );
    private.Loader_MainHandler();
    
    return;
    
  end;
 
  local result_state = "loadingcomplete";

  local result_finish = function ()

    if ( not private.modulesloaded ) then
      private.Modules_Load();
    end;

  end;

  private.PartsSetStateHandler( result_state, function ()

    result_finish();

  end );

  ShowLoadingWheel( false );

  private.Loader_MainHandler();

end;
--------------------------------------------------------------------------------
function private.IsNewClientVersionAvailable ()

  local platform = GetPlatform();
  if ( platform == "android" ) then
  
    platform = wiz.Android.GetPlatform();
  
  end;

  private.WaitingForAnswer( "version",
                            "IsNewClientVersionAvailable",
                            { platform = platform },
                            private.OnIsNewClientVersionAvailable );

end;
--------------------------------------------------------------------------------
function private.OnIsNewClientVersionAvailable ( msg, params )

  if ( params.available ) then

    private.newversion = true;

  end;

end;
--------------------------------------------------------------------------------
function private.PartsRunMigrations ( params )

  private.Trace( "PartsRunMigrations" );
  
  parts.storage.data.Write( "sml/need_run_migration", params.version );

end;
--------------------------------------------------------------------------------
function private.PartsNewVersionAvailable_Handler ()

  local func_download = function ()

    local platform = GetPlatform();
    if ( platform == "android" ) then

      platform = wiz.Android.GetPlatform();

    end;

    private.SystemMessages_Add( "newversion", function ()
      private.PartsNewVersionAvailable_Handler();
    end );
    private.Loader_MainHandler();

    wiz.bfg.OpenUrl( private.app_url[ platform ] );

  end;

  local func_continue = function ()

    local result_state = "loadingcomplete";

    local result_finish = function ()

      if ( not private.modulesloaded ) then
        private.Modules_Load();
      end;

    end;

    private.PartsSetStateHandler( result_state, function ()

    result_finish();

    end );

    private.Loader_MainHandler();

  end;

  private.SystemDialog_Show( "newversion", func_download, func_continue );

end;
--------------------------------------------------------------------------------
function private.PartsInternetConnectionReturned ()

  private.is_online = true;

  private.Info_Update();

  local func_continue = function ()
    
    private.after_close_handler = nil;

    if ( private.parts_result_state == "nointernetconnection" ) then

      private.PartsClearStateHandler();
      
      ObjSet( "spr_loader_logo_game",
      {
        input = false,
        event_mup = function ()

        end,
      } );

    end;

    private.PartsLoadingComplete( true );

  end;

  if not ( private.is_on_screen_system_msg ) then

    func_continue();

  else

    private.after_close_handler = func_continue;

  end;

end;
--------------------------------------------------------------------------------
function private.ApplicationGainFocus ()
  
  wiz.ChartBoost.LocationStartup();
  
end;
--------------------------------------------------------------------------------
function private.Loader_MainHandler ()

  if not ( private.gonext ) then

    private.Trace( "Loader_MainHandler: CAN'T GO NEXT!'");

    return;

  end;

  local func_on_click = function () end;

  if ( private.parts_result_state ) then
    
    private.Trace( "Loader_MainHandler: execute state handler -> "..private.parts_result_state );

    ShowLoadingWheel( false );
    
    if ( private.parts_result_state == "loadingcomplete" ) then
      
      private.parts_result_handler();
      private.PartsClearStateHandler();
      
      return false;
      
    end;

    func_on_click = private.parts_result_handler;

    private.Text_Show( "taptocontinue", func_on_click );
    
    return;
    
  end;
  
  if ( #private.messages_queue > 0 ) then
    
    private.Trace( "Loader_MainHandler: show message." );

    ShowLoadingWheel( false );

    func_on_click = private.SystemMessages_ShowNext;

    private.Text_Show( "taptocontinue", func_on_click );

    return;

  end;

  private.Trace( "Loader_MainHandler: loop." );

  --private.Loader_MainHandler();

end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.Module_TryGetPrivate ( module_name )

  if ( IsEditor() and _G[ module_name ] ) then
  
    if ( _G[ module_name ].__Private ) then
    
      _PRIVATE[ module_name ] = _G[ module_name ].__Private();
    
    else
    
      _PRIVATE[ module_name ] = _G[ module_name ];
    
    end;
    
  end;

end;
--------------------------------------------------------------------------------
function public.Module_Load ( module_name, instant )
  
  local path = "assets/content/scripts/";
  
  if ( instant ) then

    ne.mod.Load( path.."mod_"..module_name );
    
  else
  
    ne.mod.LoadLazily( path.."mod_"..module_name );

  end;
  
  private.Module_TryGetPrivate( module_name );
  
end;
--------------------------------------------------------------------------------
function public.Module_Load_Init ( module_name )
  
  -- JK: Load & Init is not for lazily mode

  public.Module_Load( module_name, true );
  _G[ module_name ].Init();
  
end;
--------------------------------------------------------------------------------
function public.Modules_Load ( modules )

  -- JK: lazily mode, otherwise use Modules_Load_Init

  for m, module_name in pairs( modules ) do
  
    public.Module_Load( module_name );

  end;
  
end;
--------------------------------------------------------------------------------
function public.Modules_Load_Init ( modules )

  -- JK: Load & Init is not for lazily mode

  for m, module_name in pairs( modules ) do
  
    public.Module_Load( module_name, true );
    _G[ module_name ].Init();

  end;
  
end;
--------------------------------------------------------------------------------
function private.Modules_Load ()

  private.modulesloaded = false;

  private.Trace( "Modules_Load: started." );
  
  local modules = {
    "safe_api",
    "common",
    "m_object",
    "m_storage",
    "mediator",
    "screen",
    "m_client",
    "migrations",
    "m_content",
    "int_control",
    "interface",
    "int_window",
    "model_game",
    "view_game",
    "m_settings",
    "m_story",
    "m_hero",
    "m_player",
    "m_map",
    "m_stage",
    "m_fireflies",
    "m_shop",
    "m_market",
    "m_boosters",
    "m_skins",
    "m_achievements",
    "m_social",
    "m_email",
    "m_tutorial",
    "m_hearts",
    "m_quests",
    "m_dailyquest",
    "m_dailybonus",
    "m_fortunewheel",
    "m_items",
    "v_items",
    "v_client",
    "v_story",
    "v_map",
    "v_stage",
    "v_shop",
    "v_boosters",
    "v_settings",
    "v_quests" ,
    "v_dailyquest",
    "v_dailybonus",
    "v_social",
    "v_hero",
    "v_tutorial",
    "v_hearts",
    "v_fireflies",
    "int_system_message",
    "int_dialog_info",
    "int_dialog_update",
    "int_dialog_deficient",
    "int_dialog_minibank",
    "int_dialog_notifications"
  };
  
  -- JK: load strict in debug mode only
  if ( IsEditor() and debug ) then table.insert( modules, 1, "strict" ); end;
  
  local timer_name = "tmr_module_load";
  
  ObjCreate( timer_name, "timer" );
  ObjAttach( timer_name, "scr_loader" );
  
  ObjSet( "grd_loader_logo_game", { croprect_w = 0.1 } );
  ObjSet( "spr_loader_logo_game",
  {
    input = false,
    event_mup = function ()
      
    end,
  } );  

  private.Text_Show( "wait", false, true );
  
  local modules_count = #modules;
  
  local func_load;
 
  func_load = function ()
  
    ObjSet( timer_name, { time = 0.017, playing = true, endtrig = function ()
    
      ObjSet( timer_name, { playing = false } );
      
      local module_name = table.remove( modules, 1 );
      
      ModLoad( "assets/content/scripts/mod_"..module_name );
      private.Module_TryGetPrivate( module_name );
      
      local func_init = _G[ module_name ].Init;
      
      if ( module_name == "common" ) then
        
        func_init = _G[ module_name ].Start;
        
      end;
      
      -- JK: some modules may not require to be initialized
      if ( func_init ) then

        func_init();

      end;
      
      local modules_loaded = modules_count - #modules;
      local percents = math.floor( 100 * modules_loaded / modules_count );
      
      -- load from 50% to 100%
      local crop_percents = math.floor( 50 + percents / 2 );
      local crop_current  = ObjGet( "grd_loader_logo_game" ).croprect_w;
      local crop_new      = 352 * crop_percents / 100;
      
      ObjSet( "txt_loader_logo_game_percents", { text = crop_percents.."%" } );

      ObjAnimate( "grd_loader_logo_game", "croprect_w", 0, 0, function ()
      
        if ( #modules > 0 ) then
          
          func_load();
          
        else

          private.modulesloaded = true;
          
          ObjDelete( timer_name );
          
          private.Trace( "Modules_Load: finished." );
          
          private.HideLogoGame();
          
        end;
      
      end,
      {
        0.000, 0, crop_current,
        0.017, 0, crop_new
      } );     
      
    end } );
    
  end;
  
  func_load();
  
end;
--------------------------------------------------------------------------------
--function ... () end;
--------------------------------------------------------------------------------
function private.ShowLogoPublisher ()

  local func_next = function ()

    private.KeepLogoPublisher();

  end;

  --private.Anim_ShowLogo( "publisher", func_next );
  ObjSet( "spr_loader_logo_publisher", { alp = 1.0 } );
  func_next();

end;

function private.KeepLogoPublisher ()

  local func_next = function ()

    private.HideLogoPublisher();

  end;

  private.Anim_KeepLogo( "publisher", func_next );

end;

function private.HideLogoPublisher ()

  local func_next = function ()

    private.ShowLogoDeveloper();

  end;

  private.Anim_HideLogo( "publisher", func_next );

end;

function private.ShowLogoDeveloper ()

  local func_next = function ()

    private.KeepLogoDeveloper();

  end;

  private.Anim_ShowLogo( "developer", func_next );

end;

function private.KeepLogoDeveloper ()

  local func_next = function ()

    private.HideLogoDeveloper();

  end;

  private.Anim_KeepLogo( "developer", func_next );

end;

function private.HideLogoDeveloper ()

  local func_next = function ()

    private.ShowLogoGame();

  end;

  private.Anim_HideLogo( "developer", func_next );

end;

function private.ShowLogoGame ()

  local func_next = function ()

    private.KeepLogoGame();
    --ShowLoadingWheel( false );

  end;

  ShowLoadingWheel( true );
  private.Anim_ShowLogo( "game", func_next );

end;

function private.KeepLogoGame ()
  
  private.gonext = true;

  private.Text_Show( "wait" );

  private.Loader_MainHandler();

end;

function private.HideLogoGame ( func_next )

  private.Text_Hide();

  func_next = func_next or function ()

    private.Splash_Show();

  end;

  private.Anim_HideLogo( "game", func_next );

end;

function private.Splash_Show ()

  ObjSet( "obj_loader_splash", { active = true, visible = true, input = true } );

  ObjAnimate( "obj_loader_splash", "alp", 0, 0, private.Splash_Keep, 
  { 
    0.0, 0, 0.0, 
    0.3, 0, 1.0 
  } );

  ne.snd.Play( "assets/loader/resources/aud_loader_music", "soundtrack", 1, "", 0 );

  private.Info_Show();

end;

function private.Splash_Keep ()

  if ( private.is_android                    ) and
     ( wiz.Android.GetPlatform() == "google" ) then

    ne.msg.Subscribe( "Event_Android_OnPause", private.OnPause );
    int_dialog_game_pause.Subscribe();

  end;

  ObjSet( "obj_loader_splash", { input = true } );

  AnimPlay( "anm_loader_gamelogo", "appear", function () 

    AnimPlay( "anm_loader_gamelogo", "appear2", function ()
 
      AnimPlay( "anm_loader_gamelogo", "static", "" );

    end );

  end );

  AnimPlay( "anm_loader_gamelogo_dummy", "dummy", function () 
    
    private.Splash_Buttons();

  end );

  ne.snd.Play( "assets/loader/resources/aud_loader_game", "sfx", 0, "", 0 );

end;

function private.Splash_Buttons ()

  private.Trace( "Splash_Buttons" );
  
  common.Playhaven_LogPlacement( "load_game" );
  wiz.ChartBoost.LocationMainMenu();
  
  private.BtnPlay_Show();

  if ( private.is_pc ) then
    
    private.BtnClose_Show();
    
  else
    
    if ( private.is_android                    ) and
       ( wiz.Android.GetPlatform() == "google" ) then
       
      private.BtnGoogle_Show();
      
    end;
    
    private.BtnConnect_Show();
    
  end;
  
  private.Social_Subscribe();

  if ( private.is_android ) then
  
    ne.msg.Subscribe(
      "Event_Android_BackPressed",
      private.OnAndroidBackButtonPressed
    );
  
  end;
  
end;

function private.Splash_Hide ( func_end )

  local func_onAnimEnd = function ()

    ObjSet( "obj_loader_splash", { input = false, active = false, visible = false } );

    if ( func_end ) then

      func_end();

    end;

  end;

  ObjAnimate( "obj_loader_splash", "alp", 0, 0, func_onAnimEnd, 
  { 
    0.0, 0, 1.0, 
    0.3, 0, 0.0 
  } );

  ne.snd.Stop( "assets/loader/resources/aud_loader_music", 1 );
  ne.snd.Stop( "assets/loader/resources/aud_loader_game",  1 );

  private.Info_Hide();

end;

function private.Anim_ShowLogo ( target, func_next )

  ObjAnimate( "spr_loader_logo_"..target, "alp", 0, 0, func_next, 
  {
    0.0, 0, 0.0,
    1.0, 0, 1.0
  } );

end;

function private.Anim_KeepLogo ( target, func_next )

  ObjSet( "spr_loader_logo_"..target, { input = true } );

  ObjAnimate( "spr_loader_logo_"..target, "alp", 0, 0, func_next,
  {
    0.0, 0, 1.0,
    3.0, 0, 1.0
  } );

end;

function private.Anim_HideLogo ( target, func_next )

  ObjSet( "spr_loader_logo_"..target, { input = false } );

  ObjAnimate( "spr_loader_logo_"..target, "alp", 0, 0, func_next,
  {
    0.0, 0, 1.0,
    0.5, 0, 0.0
  } );

end;

function private.BtnPlay_Show ( func_end )
  
  local func_next = function ()
    
    ObjSet( "btn_loader_play", { input = true, event_mup = private.BtnPlay_Tap } );

    private.BtnPlay_StartAnim();
    
    if ( func_end ) then
      
      func_end();
      
    end;

  end;
  
  ObjSet( "anm_loader_play", { title = "str_menu_button_play" } ); 
  ObjSet( "btn_loader_play", { active = true, visible = true, input = false, alp = 0 } );

  ObjAnimate( "btn_loader_play", "alp", 0, 0, func_next,
  {
    0.0, 0, 0.0,
    1.0, 0, 1.0
  } );

end;

function private.BtnPlay_StartAnim ()

  ObjAnimate( "btn_loader_play", "scale_xy", 1, 0, "",
  { 
    0.0, 3, 1.00, 1.00,
    1.0, 3, 0.92, 0.92,
    2.0, 3, 1.00, 1.00,
  } );

end;

function private.BtnPlay_StopAnim ()

  ObjStopAnimate( "btn_loader_play", "scale_xy" );

end;

function private.BtnPlay_Tap ()

  ne.snd.Stop( "assets/loader/resources/aud_loader_music", 1 );
  ne.snd.Stop( "assets/loader/resources/aud_loader_game",  1 );

  private.Info_Hide();

  ne.snd.Play( "assets/loader/resources/aud_loader_click", "sfx", 0, "", 0 );

  private.BtnPlay_StopAnim();

  ObjSet( "btn_loader_play", { active = false, input = false } );

  private.PartsUnsubscribeAll();

  ModLoad( "assets/content/scripts/mod_loading" );
  loading.Init();
  
  if ( not private.is_pc ) then
  
    private.BtnConnect_Unsubscribe();
  
  end;
  
  MsgUnsubscribe(
    "Event_GooglePlay_LoggedStateChanged",
    private.BtnGoogle_OnStateChanged
  );
  
  if ( private.is_android ) then
  
    ne.msg.Unsubscribe(
      "Event_Android_BackPressed",
      private.OnAndroidBackButtonPressed
    );
  
  end;

  loading.Show( true, function ()

    ObjDetach( "scr_loader" );
    --ModLoad( "assets/content/scripts/mod_common" );
    --common.Start();

    m_client.Start();
    m_social.Start();
    v_social.Start();
    view_game.Start();

  end );

end;
--------------------------------------------------------------------------------
function private.BtnClose_Show ()
  
  local func_next = function ()

    ObjSet( "btn_loader_close", { input = true, event_mup = private.BtnClose_Tap } );

    AnimPlay( "anm_loader_close", "btn_close", "" );
    
  end;

  ObjSet( "btn_loader_close", { active = true, visible = true, input = false, alp = 0 } );

  ObjAnimate( "btn_loader_close", "alp", 0, 0, func_next,
  {
    0.0, 0, 0.0,
    1.0, 0, 1.0
  } );

end;

function private.BtnClose_Tap ()
  
  local dialog_type = "exitconfirm";
  local buttons = {};
  
  table.insert( buttons, {
    title  = ne.strings.Get( "str_systemmessage_"..dialog_type.."_button1" ),
    action = function ()
    
      private.InGameMessage_Hide();
      
      if ( private.is_pc ) then
      
        wiz.Quit();
      
      elseif ( private.is_android ) then
      
        private.AndroidMinimize();
      
      end;

    end,
    params = {}
      
  } );

  table.insert( buttons, {

    title  = ne.strings.Get( "str_systemmessage_"..dialog_type.."_button2" );
    action = function ()
      
      private.exitconfirm = false;
      private.InGameMessage_Hide();

    end,
    params = {}
      
  } );
  
  private.is_on_screen_system_msg = true;
  
  private.exitconfirm = true;
  
  private.InGameMessage_Show( {
    title   = ne.strings.Get( "str_systemmessage_"..dialog_type.."_title" ),
    body    = ne.strings.Get( "str_systemmessage_"..dialog_type.."_body" ),
    buttons = buttons
  } );
    
end;
--------------------------------------------------------------------------------
function private.BtnGoogle_Show ()
  
  private.BtnGoogle_SetState( wiz.Android.GooglePlayIsConnected() );
  MsgSubscribe(
    "Event_GooglePlay_LoggedStateChanged",
    private.BtnGoogle_OnStateChanged
  );
  
  if ( GetGooglePlayLogged() ) then
  
    mediator.Notify( "View_Game_GooglePlayLoginStart" );
    wiz.Android.GooglePlaySignIn();
  
  end;
  
  local func_next = function ()

    ObjSet( "btn_loader_google", { input = true, event_mup = private.BtnGoogle_Tap } );
    
  end;

  ObjSet( "btn_loader_google", { active = true, visible = true, input = false, alp = 0 } );

  ObjAnimate( "btn_loader_google", "alp", 0, 0, func_next,
  {
    0.0, 0, 0.0,
    1.0, 0, 1.0
  } );

end;

function private.BtnGoogle_Tap ()

  if ( wiz.Android.GooglePlayIsConnected() ) then
  
    wiz.Android.GooglePlaySignOut();
    SetGooglePlayLogged( false );
  
  else

    mediator.Notify( "View_Game_GooglePlayLoginStart" );
    wiz.Android.GooglePlaySignIn();
  
  end;

end;

function private.BtnGoogle_OnStateChanged ( msg, params )
    
  private.BtnGoogle_SetState( params.logged );

end;

function private.BtnGoogle_SetState ( connected )

  if ( connected ) then
  
    AnimPlay( "anm_loader_google", "btn_google", "" );
    ObjSet(
      "anm_loader_google",
      { 
        tt_desc = wiz.Android.GooglePlayGetPlayerName()
      }
    );
    
  else
  
    AnimPlay( "anm_loader_google", "btn_google_noactive", "" );
  
  end;

end;
--------------------------------------------------------------------------------
function private.Text_Show ( text, func_on_click, hint )
  
  local msg_text = "str_loader_message_"..text;
  local hint_text = "";
  
  if ( hint ) then
    
    hint_text = "str_loader_hint_"..math.random( 1, 10 );
    
  end;

  ObjMultiSet( {
    { "txt_loader_logo_game", { active = true, visible = true, text = msg_text, alp = 1.0 } },
    { "txt_loader_logo_game_hint", { active = true, visible = true, text = hint_text, alp = 1.0 } }
  } );

  if ( func_on_click ) then

    ObjSet( "spr_loader_logo_game",
    {
      input = true,
      event_mup = function ()

        private.Text_Hide();
        func_on_click();

      end
    } );

    private.Text_Anim();

  end;

end;
--------------------------------------------------------------------------------
function private.Text_Anim ()

  ObjAnimate( "txt_loader_logo_game", "alp", 1, 0, "", 
  { 
    0.0, 3, 1.0,
    1.2, 3, 0.0,
    2.4, 3, 1.0
  } );
  
  ObjAnimate( "txt_loader_logo_game_hint", "alp", 1, 0, "", 
  { 
    0.0, 3, 1.0,
    1.2, 3, 0.0,
    2.4, 3, 1.0
  } );  

end;
--------------------------------------------------------------------------------
function private.Text_Hide ()
  
  ObjStopAnimate( "txt_loader_logo_game", "alp" );
  ObjStopAnimate( "txt_loader_logo_game_hint", "alp" );
  
  ObjMultiSet( {
    { "txt_loader_logo_game", { active = false, visible = false } },
    { "txt_loader_logo_game_hint", { active = false, visible = false } }
  } ); 

end;
--------------------------------------------------------------------------------
function private.TapToContinue_Show ()

  ObjSet( "obj_loader_taptocontinue", { active = true, visible = true } );

  ObjAnimate( "obj_loader_taptocontinue", "alp", 0, 0, function () 

    private.TapToContinue_StartAnim();

    private.TapToContinue_Set(); 

  end,
  { 
    0.0, 0, 0.0, 
    0.3, 0, 1.0 
  } );

end;

function private.TapToContinue_StartAnim ()

  ObjAnimate( "obj_loader_taptocontinue", "scale_xy", 1, 0, "",
  { 
    0.0, 3, 1.00, 1.00,
    1.0, 3, 0.92, 0.92,
    2.0, 3, 1.00, 1.00,
  } );

end;

function private.TapToContinue_Set ()

  ObjSet( "obj_loader_splash", { event_mup = function () 

    private.Splash_Hide( function ()

      private.PartsUnsubscribeAll();
      private.Content_StartDownload();

    end );

  end } );

end;
--------------------------------------------------------------------------------
function private.Info_Show ()

  private.Info_Update();

  ObjSet( "obj_loader_info", { active = true, visible = true, alp = 0 } );

  ObjAnimate( "obj_loader_info", "alp", 0, 0, "", 
  { 
    0.0, 0, 0.0, 
    0.3, 0, 1.0 
  } );

end;

function private.Info_Update ()
  
  if ( not private.modulesloaded ) then
    return;
  end;
  
  local client_version  = ConfigGetInfo( "parts.version.currentVersion" ) or "UNKNOWN";
  local content_version = parts.version.GetCurrent( "content" ) or "UNKNOWN";
  local server_version  = parts.version.GetCurrent( "server" );

  if ( "" == server_version ) then

    server_version = "UNKNOWN";

  end;

  local server_host  = private.Info_GetServer( "server.host" ) or "UNKNOWN";
  local content_host = private.Info_GetServer( "parts.content.contentHost" ) or "UNKNOWN";
  server_version = server_version.." ( "..server_host.." ) ";
  content_version = content_version.." ( "..content_host.." ) ";

  m_object.Set(
    "txt_loader_info",
    {
      text    = "str_loader_info",
      param0  = m_player.GetUserId(),
      param1  = client_version,
      param2  = server_version,
      param3  = content_version
    }
  );

end;

function private.Info_Hide ()

  ObjAnimate( "obj_loader_info", "alp", 0, 0, function ()

    ObjSet( "obj_loader_info", { active = false, visible = false, alp = 0 } );

  end, 
  { 
    0.0, 0, ObjGet( "obj_loader_info" ).alp, 
    0.3, 0, 0.0 
  } );

end;

function private.Info_GetServer ( param_type )

  local url_server = tostring( ConfigGetInfo( param_type ) );

  local server_list = 
  {
    { host = "http://dev1.fairysaga.elephant-games.com/", type = "EG_DEV1" },
    { host = "http://dev2.fairysaga.elephant-games.com/", type = "EG_DEV2" },
    { host = "http://dev3.fairysaga.elephant-games.com/", type = "EG_DEV3" },
    { host = "http://dev4.fairysaga.elephant-games.com/", type = "EG_DEV4" },
    { host = "http://f2p-qa.bigfishgames.com/FairySaga1/", type = "BFG_QA1" },
    { host = "http://f2p-qa.bigfishgames.com/FairySaga2/", type = "BFG_QA2" },
    { host = "http://f2p-st.bigfishgames.com/FairySaga/", type = "BFG_STAGE" },
    { host = "http://f2p.bigfishgames.com/FairySaga/", type = "BFG_PROD" },
    { host = "http://cdnf2pqa-a.akamaihd.net/FairySaga1/", type = "CDN_QA1" },
    { host = "http://cdnf2pqa-a.akamaihd.net/FairySaga2/", type = "CDN_QA2" },
    { host = "http://cdnf2pst-a.akamaihd.net/FairySaga/", type = "CDN_STAGE" },
    { host = "http://cdnf2p-a.akamaihd.net/FairySaga/", type = "CDN_PROD" },
    { host = "http://f2p-qa.bigfishgames.com/SirMatchPC1/", type = "BFG_QA1_PC" },
    { host = "http://f2p-qa.bigfishgames.com/SirMatchPC2/", type = "BFG_QA2_PC" },
    { host = "http://f2p-st.bigfishgames.com/SirMatchPC/", type = "BFG_STAGE_PC" },
    { host = "http://f2p.bigfishgames.com/SirMatchPC/", type = "BFG_PROD_PC" },
    { host = "http://cdnf2pqa-a.akamaihd.net/SirMatchPC1/", type = "CDN_QA1_PC" },
    { host = "http://cdnf2pqa-a.akamaihd.net/SirMatchPC2/", type = "CDN_QA2_PC" },
    { host = "http://cdnf2pst-a.akamaihd.net/SirMatchPC/", type = "CDN_STAGE_PC" },
    { host = "http://cdnf2p-a.akamaihd.net/SirMatchPC/", type = "CDN_PROD_PC" }
  };

  for i = 1, #server_list do

    if ( server_list[ i ].host:find( url_server, 1, true ) ) then

      return server_list[ i ].type;

    end;

  end;

  return "???";

end;
--------------------------------------------------------------------------------
function private.SetForm ()

  local screen_w = GetAppWidth();
  local screen_h = GetAppHeight();
  
  local multiset_table = {};
  
  table.insert( multiset_table, { "obj_loader_logo", {
    pos_x = 0.5 * screen_w,
    pos_y = 0.5 * screen_h
  } } );
  
  table.insert( multiset_table, { "obj_loader_logo_game", {
    pos_y = ( screen_h / 2 ) - 150
  } } );
  
  table.insert( multiset_table, { "obj_loader_splash", {
    pos_x = 0.5 * screen_w,
    pos_y = 0.5 * screen_h,
    alp = 0.0,
    inputrect_w = screen_w,
    inputrect_h = screen_h,
    inputrect_x = - 0.5 * screen_w,
    inputrect_y = - 0.5 * screen_h
  } } );
  
  table.insert( multiset_table, { "obj_loader_message", {
    pos_x = 0.5 * screen_w,
    pos_y = 0.5 * screen_h,
    active = false, visible = false, input = false
  } } );
  
  table.insert( multiset_table, { "btn_loader_play", {
    active = false, visible = false, input = false
  } } );
  table.insert( multiset_table, { "btn_loader_splash_connect", {
    active = false, visible = false, input = false
  } } );
  table.insert( multiset_table, { "btn_loader_connect", {
    active = false, visible = false, input = false
  } } );
  table.insert( multiset_table, { "btn_loader_close", {
    pos_x = 0.5 * screen_w - 50,
    pos_y = - 0.5 * screen_h + 50,
    active = false, visible = false, input = false
  } } );
  table.insert( multiset_table, { "btn_loader_google", {
    pos_x = 0.5 * screen_w - 50,
    pos_y = - 0.5 * screen_h + 50,
    active = false, visible = false, input = false
  } } );
  
  if ( private.is_pc ) then
    
    table.insert( multiset_table, { "btn_loader_play", {
      pos_y = math.ceil( 0.4 * screen_h ) - 20 } } );
    
  end;

  local scale = 
  {
    [  640 ] = 1.0,
    [  768 ] = 1.0,
    [ 1024 ] = 0.9
  };

  local logo_y = math.ceil( 0.15 * screen_h );

  if ( not private.is_pc ) then

    logo_y = math.ceil( 0.1 * screen_h );

    if ( screen_w == 1024 ) then

      scale[ 1024 ] = 0.8;
      logo_y        = math.ceil( 0.07 * screen_h );

    end;

  end;
  
  table.insert( multiset_table, { "obj_loader_info", {
    pos_x = 0, pos_y = 0 } } );
  table.insert( multiset_table, { "obj_loader_gamelogo", {
    pos_x = 0, 
    pos_y = logo_y, 
    scale_x = scale[ screen_w ] or 1,
    scale_y = scale[ screen_w ] or 1 } } );
  table.insert( multiset_table, { "obj_loader_taptocontinue", {
    pos_x = 0, pos_y = math.ceil( 0.35 * screen_h ) } } );
  table.insert( multiset_table, { "obj_loader_content", {
    pos_x = 0.5 * screen_w, pos_y = 0.5 * screen_h } } );
  table.insert( multiset_table, { "obj_loader_content_message", {
    pos_x = 0, pos_y = 0.5 * screen_h - 230 } } ); 

  ObjMultiSet( multiset_table );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Social_Subscribe ()
  
  if ( private.is_pc ) then
    
    return;
    
  end;

  private.PartsWaitForResult( Event_Social_FirstAuthenticatesRaveId, function ()
    common.Kontagent_LogEvent( "social",
    {
      description = "account_profile",
      name        = "create_raveid"
      } );
  end );

  private.PartsWaitForResult( Event_Social_RaveLoginFail, function ()
    common.Kontagent_LogEvent( "social",
    {
      description = "social_login",
      name        = "login_fail"
    } );
  end );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.BtnConnect_Show ()

  private.BtnConnect_Set( SocialIsLogged().result );
  private.BtnConnect_Subscribe();

  local function func_input_on ()

    ObjSet( "btn_loader_fb_connect", { input = true } );

  end;

  ObjSet( "btn_loader_fb_connect", { input = false } );
  ObjAnimate(
    "anm_loader_fb", "alp", 0, 0,
    func_input_on,
    {
      0.0, 0, 0.0,
      0.7, 0, 0.0,
      1.5, 0, 1.0
    }
  );

end;
--------------------------------------------------------------------------------
function private.BtnConnect_Set ( logged )

  if ( logged ) then
  
    ObjSet(
      "anm_loader_fb",
      {
        animfunc = "portrait_fb_connect",
        playing  = true,
        frame    = 0
      }
    );
    
    ObjSet(
      "btn_loader_fb_connect", 
      { 
        event_mup = private.BtnConnect_TapLogout
      }
    );
    
    ObjSet(
      "anm_loader_fb_connect", 
      { 
        title = "str_menu_button_f_manage",
        maxwidth = 100
      }
    );
  
  else
  
    local fb_hearts = m_storage.Player.Info().hearts.facebook or 0;
    
    local af_normal = "portrait_fb_connect";
    local af_hearts = "portrait_fb_connect_hearts"
    local animfunc = tern( fb_hearts > 0 )( af_normal, af_hearts );
    
    ObjSet(
      "anm_loader_fb",
      {
        animfunc = animfunc,
        playing  = true,
        frame    = 0,
        -- text_tags:
        heart       = "str_menu_button_f_connect_title",
        heart_count = "str_menu_button_f_connect_heart",
        description = "str_menu_button_f_connect_description"
      }
    );
    
    ObjSet(
      "btn_loader_fb_connect", 
      { 
        event_mup = private.BtnConnect_TapLogin
      }
    );
    
    ObjSet(
      "anm_loader_fb_connect", 
      { 
        title = "str_menu_button_f_connect",
        maxwidth = 100
      }
    );
  
  end;

end;
--------------------------------------------------------------------------------
function private.BtnConnect_TapLogin ()

  if ( not private.is_online ) then
    private.SystemDialog_Show( "nointernetconnection", function () end );
    return;
  end;

  mediator.Notify( "View_Social_LoginStart" );

  ObjSet( "scr_loader", { input = false } );
  SocialLogin();

end
--------------------------------------------------------------------------------
function private.BtnConnect_TapLogout ()

  if ( not private.is_online ) then
    private.SystemDialog_Show( "nointernetconnection", function () end );
    return;
  end;

  ObjSet( "scr_loader", { input = false } );
  SocialLogout();

end
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.BtnConnect_Subscribe ()

  MsgSubscribe( "Event_Social_Login_Complete",  private.BtnConnect_OnLogin  );
  MsgSubscribe( "Event_Social_Logout_Complete", private.BtnConnect_OnLogout );

end;
--------------------------------------------------------------------------------
function private.BtnConnect_Unsubscribe ()

  MsgUnsubscribe( "Event_Social_Login_Complete",  private.BtnConnect_OnLogin  );
  MsgUnsubscribe( "Event_Social_Logout_Complete", private.BtnConnect_OnLogout );

end;
--------------------------------------------------------------------------------
function private.BtnConnect_OnLogin ( msg, params )

  ObjSet( "scr_loader", { input = true } );
  
  if ( not params.result ) then return; end;
  
  common.Kontagent_LogEvent( "social",
  {
    description = "social_login",
    name        = "login"
  } );
  
  private.BtnConnect_Set( true );

  if ( params.socialId ~= params.previousSocialId ) then
    private.LoginReplace( params.socialId );
    return;
  end;

  private.LoginContinue( params.socialId );

end;
--------------------------------------------------------------------------------
function private.BtnConnect_OnLogout ( msg, params )
  
  -- NOTE: params.result can contain wrong state
  local logged = SocialIsLogged().result or false;
  
  if ( not logged ) then
    
    private.BtnConnect_Set( false );
    
    common.Kontagent_LogEvent( "social",
    {
      description = "social_login",
      name        = "logout"
    } );

    if ( params.socialId ~= params.previousSocialId ) then

      parts.application.RestartWithLocalUserIdReplace( params.socialId, true );
      return;

    end;

  end;
  
  ObjSet( "scr_loader", { input = true } );

end;

function private.TryAddHeart ()

  local hearts_info = m_storage.Player.Info().hearts;

  if ( hearts_info.facebook < 1 ) then

    hearts_info.count = hearts_info.count + 1;
    hearts_info.facebook = 1;

    local query =
    {
      query      = "update",
      group      = "player",
      collection = "info",
      set        = { hearts = hearts_info }
    };

    m_storage.Execute( query );

    common.Kontagent_LogEvent( "social",
    {
      description = "social_login",
      category    = "hearts",
      name        = "increase"
    } );

    return true;

  end;

  return false;

end;

function private.LoginReplace ( social_id )

  private.Trace( "[ LOADING ] Login. LoginReplace: "..social_id );
  
  ObjSet( "anm_loader_fb", { input = false, visible = false } );

  private.WaitingForAnswer( "user", "tryReplaceLocalUserId", { newLocalUserId = social_id }, function ( message_name, message_params )
    ObjSet( "anm_loader_fb", { input = true, visible = true } );
    if ( not message_params.result ) then
      local conflict_params = {};
      conflict_params.func_local = function () private.LoginSelect( social_id, true ); end;
      conflict_params.func_server = function () private.LoginSelect( social_id, false ); end;
      conflict_params.info_local = message_params.my_profile;
      conflict_params.info_server = message_params.new_profile;
      private.PartsRaveConflict_Handler( conflict_params );
      return;
    end;
    MsgSend( "Event_Application_LocalUserIdReplaced", {} );
    private.LoginContinue( social_id );
  end );

end;

function private.PartsRaveConflict_Handler ( params )

  local func_message_main, func_message_local, func_message_server;

  func_message_main = function ()
    
    local clientTimeFormatted = os.date( "%Y-%m-%d", params.info_local.saveTime or clientTime );
    local serverTimeFormatted = os.date( "%Y-%m-%d", params.info_server.saveTime or serverTime );
    
    if clientTimeFormatted == serverTimeFormatted then
      clientTimeFormatted = os.date( "%H:%M:%S", params.info_local.saveTime or clientTime );
      serverTimeFormatted = os.date( "%H:%M:%S", params.info_server.saveTime or serverTime );
    end;
    
    local clientInfoData = { 
      stage_last = private.PartsStorageConflict_GetStage( params.info_local.collections[ "player/info" ][ 1 ].stage_last ),
      section = private.PartsStorageConflict_GetSection( params.info_local.collections[ "player/sections" ] ),
      gold = params.info_local.collections[ "player/info" ][ 1 ].gold
    };
    local serverInfoData = { 
      stage_last = private.PartsStorageConflict_GetStage( params.info_server.collections[ "player/info" ][ 1 ].stage_last ),
      section = private.PartsStorageConflict_GetSection( params.info_server.collections[ "player/sections" ] ),
      gold = params.info_server.collections[ "player/info" ][ 1 ].gold
    };

    private.SystemDialog_Show( "conflictstorage",
      func_message_local, 
      func_message_server,
      { clientTimeFormatted, clientInfoData },
      { serverTimeFormatted, serverInfoData }
    );
  
  end;

  func_message_local = function ()

    private.SystemDialog_Show(
      "conflictstorage_local",
      function ()
        params.func_local();
      end,
      function ()
        func_message_main();
      end
    );

  end;

  func_message_server = function ()

    private.SystemDialog_Show(
    "conflictstorage_server",
      function ()
        params.func_server();
      end,
      function ()
        func_message_main();
      end
    );

  end;
  
  func_message_main();

end;

function private.LoginSelect ( social_id, keepLocalSave )

  private.Trace( "LoginSelect: restart with id "..social_id );

  mediator.Notify( "Model_Social_LoginComplete", true );

  parts.application.RestartWithLocalUserIdReplace( social_id, true, keepLocalSave );

end;

function private.LoginContinue ( social_id )

  private.Trace( "[ LOADING ] Login. LoginContinue: "..social_id );

  local social_info = SocialGetInfo();

  private.WaitingForAnswer( "Social", "SetSocialId", { socialId = social_id } );
  private.WaitingForAnswer( "user", "updateUserName", { name = social_info.name } );
  private.WaitingForAnswer( "user", "updateUserAvatar", { avatar = social_info.url } );

  private.TryAddHeart();

end;

function private.WaitingForAnswer ( request_target, request_method, request_params, answer_handler )

  table.insert( private.request_list,
  {
    request_target = request_target,
    request_method = request_method,
    request_params = request_params,
    answer_handler = answer_handler
  } );

  local answer_message = private._GetAnswerMessageByRequest( request_target, request_method );

  MsgSubscribe( answer_message, private.OnWaitingFinished );

  private.SendRequest( request_target, request_method, request_params, 1 );

end;

function private.OnWaitingFinished ( answer_message, answer_params )

  MsgUnsubscribe( answer_message, private.OnWaitingFinished );

  for h = #private.request_list, 1, -1 do

    local element = private.request_list[ h ];

    if ( answer_message == private._GetAnswerMessageByRequest( element.request_target, element.request_method ) ) then

      if ( element.answer_handler ) then

        element.answer_handler( answer_message, answer_params );

      end;

      table.remove( private.request_list, h );

    end;

  end;

end;

function private.SendRequest ( request_target, request_method, request_params )

  local function _FormatRequestString ( request_string )

    return string.lower( string.sub( request_string, 1, 1 ) )
      ..string.sub( request_string, 2 );

  end;
  
  NetInvokeServerMethod( _FormatRequestString( request_target ), 
                         _FormatRequestString( request_method ), 
                         request_params, 
                         1 );

end;

function private._GetAnswerMessageByRequest ( request_target, request_method )

  return "Server_"..request_target.."_on"..request_method;

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Content_IsObsolete ()

  local result, value = parts.storage.data.Read( "sml/allow_sections" );
  
  if ( not value        ) or 
     ( value == ""      ) or 
     ( value == "false" ) or
     ( value == "true"  ) then

    value = private.Content_SectionCount();
    parts.storage.data.Write( "sml/allow_sections", value );

  end;

  private.content = {
    strings = {
      progress  = ne.strings.Get( "str_loader_content_downloading" ),
      unpacking = ne.strings.Get( "str_loader_content_unpacking" )
    },
    sections = tonumber( value ),    
    packets = {
      list     = {},
      total    = 0,
      done     = 0
    },
    progress = {
      done  = 0,
      count = 0
    },
    fail = {
      name  = "",
      retry = 0
    },
    milestone = {
      value = 0,
      step  = 25
    }
  };

  private.content.packets.list  = private.Content_GetPacketDownloadList();
  private.content.packets.total = #private.content.packets.list;

  if ( private.content.packets.total < 1 ) then

    private.content = nil;

    return {
      result = false,
      reason = "notobsolete"
    };

  end;
  
  private.content.milestone.value = private.content.milestone.value + 
                                    private.content.milestone.step;

  --- Content is obsolete ------------------------------------------------------
  private.Trace(
    "Need to get "..private.content.packets.total.." content packets for "..
    private.content.sections.." sections."
  );

  return {
    result = true
  };

end;
--------------------------------------------------------------------------------
function private.Content_GetPacketDownloadList ()

  local packet_list = ContentGetPacketList();

  for p = #packet_list, 1, - 1 do

    local packet        = packet_list[ p ];
    local packet_number = tonumber( packet.name:match( "_(%d+)" ) ) or 0;

    if ( not packet.state                         ) or
       ( packet.state ~= "downloading"            ) or
       ( packet_number > private.content.sections ) then

      table.remove( packet_list, p );

    end;
      
  end;

  return packet_list;

end;
--------------------------------------------------------------------------------
function private.Content_SectionCount ()
  
  local query = 
  {
    query      = "find_all",
    collection = "player/sections",
  };

  local sections = DbExecuteQuery( query ).result;
  
  local enabled_count = 0;
    
  for s = 1, #sections do
    
    if ( sections[ s ].enabled ) then
      
      enabled_count = enabled_count + 1;
      
    end;
    
  end;
  
  private.Trace(
    "[ CONTENT ] Section_ContentAllowed: < "..enabled_count.." >"
  );
  
  return enabled_count;

end;
--------------------------------------------------------------------------------
function private.Content_GetNextPacket ()

  private.content.packets.list = private.Content_GetPacketDownloadList();

  return private.content.packets.list[ 1 ];

end;
--------------------------------------------------------------------------------
function private.Content_TryDownloadNext ()

  local packet = private.Content_GetNextPacket();

  if ( packet ) then

    local message = string.format(
      "Download started for < %s >",
      packet.name
    );

    private.Trace( message );

    wiz.StartBackground();

    ContentStartDownloading(
      packet.name,
      private.Content_DownloadCallback
    );

    return;

  end;

end;
--------------------------------------------------------------------------------
function private.Content_DownloadCallback (
    name,
    status,
    total,
    progress
  )

  local message = string.format(
    "%s ( %i / %i )",
    name,
    progress,
    total
  );

  local progress_data = {
    status = status,
    overall = {
      total    = private.content.packets.total,
      progress = private.content.packets.done
    },
    packet = {
      total    = total,
      progress = progress
    }
  };

  --- Downloading --------------------------------------------------------------
  if ( status == "progress" ) then

    if ( private.content.progress.done == progress ) then

      private.content.progress.count = private.content.progress.count + 1;

    end;

    if ( 20 == private.content.progress.count ) then

      private.Trace( "FAIL: "..message );

      private.Content_OnDownloadFailed();

      return;

    end;

    private.Trace( "Downloading "..message );

    private.content.progress.done = done;

    private.Content_UpdateProgressbar( progress_data );

  --- Unpacking ----------------------------------------------------------------
  elseif ( status == "unpacking" ) then

    ShowLoadingWheel( true );

    private.Trace( "Unpacking "..name );

    private.size_prev = nil;

    local strings = {
      desc  = ne.strings.Get( "str_loader_content_unpacking" ),
      count = ""
    };

    ObjSet(
      "anm_loader_content_progressbar",
      {
        tt_desc  = strings.desc,
        tt_count = strings.count
      }
    );

    private.Content_UpdateProgressbar( progress_data );

  --- Success ------------------------------------------------------------------
  elseif ( status == "success" ) then

    ShowLoadingWheel( false );

    private.content.packets.done   = private.content.packets.done   + 1;
    progress_data.overall.progress = progress_data.overall.progress + 1;
    
    local overall_ratio    = progress_data.overall.progress / progress_data.overall.total;
    local overall_percents = math.floor( overall_ratio * 100 );
    
    while ( overall_percents > private.content.milestone.value ) do
      private.Kontagent_LogTechnicalEvent( private.content.milestone.value.."_complete" );
      private.content.milestone.value = private.content.milestone.value + 
                                        private.content.milestone.step;
    end;

    private.Trace(
      "Packet ready: "..name.."( "..
      progress_data.overall.progress.." / "..progress_data.overall.total..
      " )"
    );

    if ( private.content.packets.total == private.content.packets.done ) then

      private.Content_UpdateProgressbar( progress_data );
      private.Content_FinishDownload();
      wiz.EndBackground();

    else

      -- JK: next step is downloading
      progress_data.status = "progress";

      private.Content_UpdateProgressbar( progress_data );
      private.Content_TryDownloadNext();

    end;

  --- Fail ---------------------------------------------------------------------
  elseif ( status == "failed" ) then

    if ( private.content.fail.name ~= name ) then

      private.content.fail.name  = name;
      private.content.fail.retry = 0;

    end;

    if ( private.content.fail.retry == 3 ) then

      private.Trace( "FAIL: "..message );

      private.Content_OnDownloadFailed();

      return;

    end;

    private.content.fail.retry = private.content.fail.retry + 1;

    ObjSet(
      "tmr_loader_content",
      {
        time    = 10,
        playing = true,
        endtrig = private.Content_TryDownloadNext
      }
    );

  end;

end;
--------------------------------------------------------------------------------
function private.Content_OnDownloadFailed ()

  local function func_restart ()

    ObjAnimate(
      "obj_loader_content",
      "color_rgb", 0, 0,
      function ()

        RestartApplication( true );

      end,
      {
        0.0, 0, 0.2, 0.2, 0.2,
        0.1, 0, 0.0, 0.0, 0.0
      }
    );

  end;

  ObjAnimate(
    "obj_loader_content",
    "color_rgb", 0, 0,
    function ()

      private.SystemDialog_Show(
        "contentfailed",
        func_restart
      );  

    end,
    {
      0.0, 0, 1.0, 1.0, 1.0,
      0.3, 0, 0.2, 0.2, 0.2
    }
  );

end;
--------------------------------------------------------------------------------
function private.Content_StartDownload ()

  private.Kontagent_LogTechnicalEvent( "begin" );
  
  local animfunc = "hero"..math.random( 1, 3 ).."_static";

  ObjMultiSet({
    { "obj_loader_content",             { active   = true,     visible = true } },
    { "anm_loader_content_hero",        { animfunc = animfunc, playing = true } },
    { "anm_loader_content_progressbar",
      {
        tt_count = tostring( private.content.packets.done.." / "..private.content.packets.total ),
        tt_desc  = private.content.strings.progress
      }
    }
  });

  ObjAnimate(
    "obj_loader_content",
    "color_rgb", 0, 0,
    private.Content_TryDownloadNext,
    {
      0.0, 0, 0.0, 0.0, 0.0,
      0.6, 0, 1.0, 1.0, 1.0
    }
  );

  private.Info_Hide();

end;
--------------------------------------------------------------------------------
function private.Content_UpdateProgressbar ( progress_data )
  
  local progressbar_length = 123;

  local overall_ratio    = progress_data.overall.progress / progress_data.overall.total;
  local overall_percents = math.floor( overall_ratio * 100 );

  local frame_prev = ObjGet( "anm_loader_content_progressbar" ).frame;
  local frame_next = math.floor( overall_ratio * progressbar_length );    

  local packet_ratio, packet_percents, packet_segment, packet_add = 0, 0, 0, 0;

  local description = private.content.strings[ progress_data.status ];

  if ( progress_data.status == "progress" ) then

    if ( progress_data.packet.total    > 0 ) and
       ( progress_data.packet.progress > 0 ) then

      packet_segment  = math.floor( progressbar_length / private.content.packets.total );
      packet_ratio    = progress_data.packet.progress / progress_data.packet.total;
      packet_percents = math.floor( packet_ratio * 100 );
      packet_add      = math.floor( packet_segment * packet_ratio );

      description = ne.strings.Get( "str_loader_content_downloading", packet_percents.."%" );

    end;    

    frame_next = frame_next + packet_add;

  end;

  ObjSet(
    "anm_loader_content_progressbar",
    {
      tt_desc  = description,
      tt_count = tostring( progress_data.overall.progress.." / "..progress_data.overall.total ),
      frame    = frame_next
    }
  );

end;
--------------------------------------------------------------------------------
function private.Content_FinishDownload ()

  private.Kontagent_LogTechnicalEvent( "complete" );
  
  private.Trace( "Finished downloading content." );

  ObjAnimate(
    "obj_loader_content",
    "color_rgb", 0, 0,
    function () RestartApplication( true ); end,
    {
      0.0, 0, 1.0, 1.0, 1.0,
      0.3, 0, 0.0, 0.0, 0.0
    }
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.Kontagent_LogTechnicalEvent ( event_name )

  local events_table = { 
    name = event_name,
    default1 = "technical",
    default2 = "download_1",
    default3 = "",
    params = { 
      v = 0, 
      l = 1
  } };

  events_table.params = private.Kontagent_AdditionalDetails( events_table or {} );
  
  wiz.bfg.kontagent.LogEvent( events_table );

end;
--------------------------------------------------------------------------------
function private.Kontagent_AdditionalDetails ( events_table )

  local additional_data = events_table.params;
  
  local timeplayed = function ()
    local token = "wtc_ingame_time";
    if ( TimeIsWatchExist( token ) ) then
      local time = TimeGetWatchTime( token );
      return tostring( time );
    end;
    return tostring( 0 );
  end;

  local ispaying = function ()
    if ( private.Kontagent_GetPlayerInfo( "ispaying" ) ) then
      return 1;
    end;
    return 0;
  end;

  local isfb = function ()
    if ( SocialIsLogged().result ) then
      return 1;
    end;
    return 0;
  end;

  local is_googleplay_connected = function ()
    if ( wiz.Android.GooglePlayIsConnected() ) then
      return 1;
    end;
    return 0;
  end;

  additional_data.appVersion = string.match( 
                        VersionGetComponentVersion( "game" ), "(%d+.%d+.%d+)" );
  additional_data.cntVersion = string.match( 
                        parts.version.GetCurrent( "content" ), "%d+.%d+.%d+.(%d+)" );
  additional_data.timeplayed = timeplayed();  
  additional_data.host       = private.Info_GetServer( "server.host" ) or "UNKNOWN";
  additional_data.isgc       = is_googleplay_connected();
  additional_data.isfb       = isfb();
  additional_data.ispaying   = ispaying();
  
  if ( private.is_PC ) then
    
    additional_data.friends   = "0";
    
    local bfginfo = wiz.bfg.lot.GetBigFishInfo();
    
    additional_data.wrappingID = bfginfo.wrappingID;
    additional_data.gmguid     = bfginfo.gmguid;
    additional_data.bfgudID    = bfginfo.bfgudID;
    additional_data.userID     = bfginfo.userID;
    
  else
  
    additional_data.friends   = nil;
      
  end;
  
  return additional_data;

end;
--------------------------------------------------------------------------------
function private.Kontagent_GetPlayerInfo ( target )

  local query =
  {
    query      = "find",
    collection = "player/info",
    result     = target
  };
  
  return DbExecuteQuery( query ).result;

end;  
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.SystemDialog_Show ( dialog_type, func_btn1, func_btn2, text_params_btn1, text_params_btn2 )

  local is_ingame = ( private.sysmsg_type == "ingame" ) or 
                    ( dialog_type == "conflictstorage" );
  
  local func_hide = function()

    private.is_on_screen_system_msg = false;

    if ( is_ingame ) then

      private.InGameMessage_Hide();

    end;

    if ( private.after_close_handler ) then

      private.after_close_handler();

    end;

  end;

  local buttons = {};
  local postfix = "";
  if ( is_ingame ) then
    postfix = "_pc"; -- FIXME: _ingame
  end;

  if ( func_btn1 ) then

    text_params_btn1 = text_params_btn1 or {};
    
    table.insert(
      buttons,
      {
        title  = ne.strings.Get(
          "str_systemmessage_"..dialog_type.."_button1"..postfix,
          text_params_btn1[ 1 ],
          text_params_btn1[ 2 ]
        ),
        action = function()

          func_hide();
          func_btn1();

        end,
        params = text_params_btn1
      }
    );

  end;

  if ( func_btn2 ) then
    
    text_params_btn2 = text_params_btn2 or {};

    table.insert(
      buttons,
      {
        title  = ne.strings.Get(
          "str_systemmessage_"..dialog_type.."_button2"..postfix,
          text_params_btn2[ 1 ],
          text_params_btn2[ 2 ]
        ),
        action = function()

          func_hide();
          func_btn2();

        end,
        params = text_params_btn2
      }
    );

  end;
  
  private.is_on_screen_system_msg = true;
  
  local message_param = "";
  local user_id = public.GetUserId();
  if ( user_id ~= "0" ) then
    message_param = ne.strings.Get( "str_loader_userid", user_id );
  end;
  
  if ( dialog_type == "nofreespace" ) then
    message_param = private.free_space_needed;
  end;
  
  if ( is_ingame ) then
    
    private.InGameMessage_Show( {
      title   = ne.strings.Get( "str_systemmessage_"..dialog_type.."_title" ),
      body    = ne.strings.Get( "str_systemmessage_"..dialog_type.."_body", message_param ),
      buttons = buttons
    } );
    
    return;
    
  end;

  wiz.ShowSystemMessage( {
    title   = ne.strings.Get( "str_systemmessage_"..dialog_type.."_title" ),
    body    = ne.strings.Get( "str_systemmessage_"..dialog_type.."_body", message_param ),
    buttons = buttons
  } );

end;
--------------------------------------------------------------------------------
function private.InGameMessage_Show ( message )
  
  local buttons_max = 2;
  local multiset    = {};

  table.insert( multiset, {
    "obj_loader_message", { active = true, input = true, visible = true }
  } );
  
  table.insert( multiset, {
    "txt_loader_message_title", { text = message.title }
  } );

  table.insert( multiset, {
    "txt_loader_message_text", { text = message.body }
  } );
  
  for b = 1, #message.buttons do
    
    local button = message.buttons[ b ];
 
    table.insert( multiset, {
      "btn_loader_message_"..b,
      { 
        active    = true, 
        input     = true, 
        visible   = true,
        pos_x     = 320 * ( b - 1 ),
        event_mup = button.action }
    } );
    
    table.insert( multiset, {
      "anm_loader_message_"..b, { title = button.title }
    } );
    
    table.insert( multiset, {
      "anm_loader_message_"..b.."_timer", {
        active  = false,
        visible = false }
    } );

    table.insert( multiset, {
      "anm_loader_message_"..b.."_saveinfo", {
        active  = false,
        visible = false }
    } );
    
    if ( button.params[ 1 ] ) then

      table.insert( multiset, {
        "anm_loader_message_"..b.."_timer", {
          active  = true,
          visible = true,
          tt_time = button.params[ 1 ] or "" }
      } );

    end;
    
    if ( button.params[ 2 ] ) then
      
      local section = "str_systemmessage_"..( button.params[ 2 ].section 
                      or "section_1" );
      local gold  = button.params[ 2 ].gold or "0";
      
      local stage_number = button.params[ 2 ].stage_last or "stage_1";
      local stage_animfunc = "stage";
      if ( stage_number == "bonus" ) then
        stage_animfunc = "stage_bonus";
      end;
      
      ObjSet( "anm_loader_message_"..b.."_saveinfo_stage", { tt_stage = stage_number} );
      AnimPlay( "anm_loader_message_"..b.."_saveinfo_stage", stage_animfunc, "" );
      
      table.insert( multiset, {
        "anm_loader_message_"..b.."_saveinfo", {
          active  = true,
          visible = true,
          tt_section = section,
          tt_gold = gold }
      } );
      
    end;
    
  end;
  
  table.insert( multiset, {
    "obj_loader_message_buttons", { pos_x = - 160 * ( #message.buttons - 1 ) }
  } ); 
  
  if ( #message.buttons < buttons_max ) then
    
    for b = #message.buttons + 1, buttons_max do
    
      local button = message.buttons[ b ];
   
      table.insert( multiset, {
        "btn_loader_message_"..b, 
        { 
          active  = false,
          input   = false,
          visible = false }
      } );

    end;
    
  end;
  
  ObjMultiSet( multiset );
  
  local bg_params = ObjGet( "spr_loader_logo_game" );
  
  ObjAnimate( "spr_loader_logo_game", "color_rgb", 0, 0, "",
  {
    0.0, 0, bg_params.color_r, bg_params.color_g, bg_params.color_b,
    0.3, 0, 0.2, 0.2, 0.2
  } );
    
  ObjAnimate( "obj_loader_splash", "color_rgb", 0, 0, "",
  {
    0.0, 0, bg_params.color_r, bg_params.color_g, bg_params.color_b,
    0.3, 0, 0.2, 0.2, 0.2
  } );
  
end;
--------------------------------------------------------------------------------
function private.InGameMessage_Hide ( func_end )
  
  local multiset_table = {};
  
  table.insert( multiset_table, {
    "obj_loader_message", { active = false, input = false, visible = false }
  } );

  ObjMultiSet( multiset_table );
  
  local bg_params = ObjGet( "spr_loader_logo_game" );
  
  ObjAnimate( "spr_loader_logo_game", "color_rgb", 0, 0, "", 
  {
    0.0, 0, bg_params.color_r, bg_params.color_g, bg_params.color_b,
    0.3, 0, 1.0, 1.0, 1.0  
  } );
  
  ObjAnimate( "obj_loader_splash", "color_rgb", 0, 0, func_end, 
  {
    0.0, 0, bg_params.color_r, bg_params.color_g, bg_params.color_b,
    0.3, 0, 1.0, 1.0, 1.0  
  } );
  
end;
--------------------------------------------------------------------------------
function private.OnAndroidBackButtonPressed ()

  if ( private.exitconfirm ) then
  
    private.exitconfirm = false;
    private.InGameMessage_Hide();
    return;
  
  end;

  private.BtnClose_Tap();

end;
--------------------------------------------------------------------------------
function private.AndroidMinimize ()

  ne.msg.Unsubscribe(
    "Event_Android_BackPressed",
    private.OnAndroidBackButtonPressed
  );
  wiz.app.Minimize();
  ne.msg.Subscribe(
    "Event_Application_GainFocus",
    private.OnFocusGainedAfterMinimize
  );

end;
--------------------------------------------------------------------------------
function private.OnFocusGainedAfterMinimize ( message, params )

  ne.msg.Unsubscribe( message, private.OnFocusGainedAfterMinimize );
  ne.msg.Subscribe(
    "Event_Android_BackPressed",
    private.OnAndroidBackButtonPressed
  );

end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.OnPause ()
  
  int_dialog_game_pause.Show();
  
end;
--------------------------------------------------------------------------------
function private.Pause ()
  
  private.SetPaused( true );
  
end;
--------------------------------------------------------------------------------
function private.Resume ()
  
  private.SetPaused( false );
  
end;
--------------------------------------------------------------------------------
function private.SetPaused ( is_paused )
  
  ne.snd.PauseAll( is_paused );
  ObjSet( "ng_application", { active = not is_paused } );
  
end;
--------------------------------------------------------------------------------
-- function ... () end;
--------------------------------------------------------------------------------
function private.SetupSimulation ( platform )

  DbgTrace( "[ SIMULATION ] Setup for "..platform );
    
  private.simulation = {
    platform             = platform,
    googleplay_connected = false
  };
  
  GetPlatform = function ()

    return private.simulation.platform;

  end;
  
  if ( platform == "android" ) then

    wiz.Android.GetPlatform = function ()
   
      return "google";
   
    end;
   
    wiz.Android.GooglePlayIsConnected = function ()
   
      return private.simulation.googleplay_connected;
   
    end;
      
    wiz.Android.GooglePlaySignIn = function ()
      
      private.simulation.googleplay_connected = true;
      
      ne.msg.Send(
        "Event_GooglePlay_LoggedStateChanged",
        { logged = true }
      );
      
    end;
      
    wiz.Android.GooglePlaySignOut = function ()
   
      private.simulation.googleplay_connected = false;
      
      ne.msg.Send(
        "Event_GooglePlay_LoggedStateChanged",
        { logged = false }
      );
   
    end;
    
    wiz.Android.GooglePlayGetPlayerName = function ()
   
      return "Android Simulator";
   
    end;

  end;
  
end;
--------------------------------------------------------------------------------